(herald "DHCR: unified model (UM) original" (bound 20) (limit 12000)
  (algebra diffie-hellman))

(comment "CPSA 4.3.1")
(comment
  "All input read from /Users/guttman/scm/tooldev/manual/cpsa4manual/examples/dhcr_um_expt_assume.scm")
(comment "Step count limited to 12000")
(comment "Strand count bounded at 20")

(defprotocol dhcr-um diffie-hellman
  (defrole init
    (vars (la x rndx) (beta upsilon expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv a la))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb))
    (uniq-orig na)
    (uniq-gen x)
    (absent (x la) (x beta))
    (assume (fact neq (exp (gen) upsilon) (gen)))
    (gen-st (pv a la)))
  (defrole resp
    (vars (lb y rndx) (alpha zeta expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv b lb))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    (uniq-orig nb)
    (uniq-gen y)
    (absent (y lb) (y alpha) (y zeta))
    (assume (fact neq (exp (gen) zeta) (gen)))
    (gen-st (pv b lb)))
  (defrole ltx-gen
    (vars (self name) (l rndx) (priv-stor locn) (ignore mesg))
    (trace (load priv-stor ignore) (stor priv-stor (pv self l))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    (uniq-orig l))
  (defrole ltx-disclose
    (vars (self name) (l rndx) (priv-stor locn))
    (trace (load priv-stor (pv self l)) (stor priv-stor "nil") (send l))
    (gen-st (pv self l)))
  (defrule undisclosed-not-disclosed
    (forall ((z strd) (l rndx))
      (implies
        (and (fact undisclosed l) (p "ltx-disclose" z 2)
          (p "ltx-disclose" "l" z l))
        (false))))
  (defgenrule neqRl_indx
    (forall ((x indx)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_strd
    (forall ((x strd)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_mesg
    (forall ((x mesg)) (implies (fact neq x x) (false))))
  (defgenrule scissorsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (leads-to z0 i0 z2 i2))
        (and (= z1 z2) (= i1 i2)))))
  (defgenrule cakeRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (leads-to z0 i0 z1 i1)
          (leads-to z0 i0 z2 i2) (prec z1 i1 z2 i2)) (false))))
  (defgenrule no-interruption
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (leads-to z0 i0 z2 i2) (trans z1 i1)
          (same-locn z0 i0 z1 i1) (prec z0 i0 z1 i1) (prec z1 i1 z2 i2))
        (false))))
  (defgenrule shearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (same-locn z0 i0 z2 i2)
          (prec z0 i0 z2 i2))
        (or (and (= z1 z2) (= i1 i2)) (prec z1 i1 z2 i2)))))
  (defgenrule invShearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (same-locn z0 i0 z1 i1)
          (leads-to z1 i1 z2 i2) (prec z0 i0 z2 i2))
        (or (and (= z0 z1) (= i0 i1)) (prec z0 i0 z1 i1)))))
  (defgenrule assume-init-0
    (forall ((z strd) (upsilon expt))
      (implies (and (p "init" z 4) (p "init" "upsilon" z upsilon))
        (fact neq (exp (gen) upsilon) (gen)))))
  (defgenrule assume-resp-0
    (forall ((z strd) (zeta expt))
      (implies (and (p "resp" z 3) (p "resp" "zeta" z zeta))
        (fact neq (exp (gen) zeta) (gen)))))
  (defgenrule trRl_ltx-gen-at-1
    (forall ((z strd)) (implies (p "ltx-gen" z 2) (trans z 1))))
  (defgenrule trRl_ltx-gen-at-0
    (forall ((z strd)) (implies (p "ltx-gen" z 2) (trans z 0))))
  (defgenrule trRl_ltx-disclose-at-1
    (forall ((z strd)) (implies (p "ltx-disclose" z 2) (trans z 1))))
  (defgenrule trRl_ltx-disclose-at-0
    (forall ((z strd)) (implies (p "ltx-disclose" z 2) (trans z 0))))
  (defgenrule gen-st-init-0
    (forall ((z strd) (la rndx) (a name))
      (implies
        (and (p "init" z 1) (p "init" "la" z la) (p "init" "a" z a))
        (gen-st (pv a la)))))
  (defgenrule gen-st-resp-0
    (forall ((z strd) (lb rndx) (b name))
      (implies
        (and (p "resp" z 1) (p "resp" "lb" z lb) (p "resp" "b" z b))
        (gen-st (pv b lb)))))
  (defgenrule gen-st-ltx-disclose-0
    (forall ((z strd) (l rndx) (self name))
      (implies
        (and (p "ltx-disclose" z 1) (p "ltx-disclose" "l" z l)
          (p "ltx-disclose" "self" z self)) (gen-st (pv self l)))))
  (lang (sig sign) (body (tuple 3)) (pv (tuple 2))))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (la rndx)
    (beta expt) (x rndx) (upsilon expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta beta) (upsilon upsilon))
  (non-orig (privk "sig" b))
  (uniq-orig na)
  (uniq-gen x)
  (absent (x la) (x beta))
  (facts (neq a b) (undisclosed la) (undisclosed beta))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb)))
  (label 0)
  (unrealized (0 1))
  (origs (na (0 2)))
  (comment "Not closed under rules"))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (la rndx)
    (beta expt) (x rndx) (upsilon expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta beta) (upsilon upsilon))
  (non-orig (privk "sig" b))
  (uniq-orig na)
  (uniq-gen x)
  (absent (x la) (x beta))
  (gen-st (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed la)
    (undisclosed beta))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb)))
  (label 1)
  (parent 0)
  (unrealized (0 0) (0 1))
  (origs (na (0 2)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (a b name) (pt pt-0 pval)
    (priv-stor locn) (beta expt) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (precedes ((1 1) (0 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x beta) (x l))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq a b) (undisclosed l) (undisclosed beta))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv a l))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul beta l))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l)))))
  (label 2)
  (parent 1)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((2 2) (0 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))))
  (label 3)
  (parent 2)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx)
    (zeta expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y zeta))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation encryption-test (added-strand resp 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 4)
  (parent 3)
  (unrealized (0 3) (3 0) (3 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation encryption-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send
        (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 5)
  (parent 3)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation channel-test (displaced 4 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 6)
  (parent 4)
  (unrealized (0 3) (3 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1)))))
  (label 7)
  (parent 4)
  (unrealized (0 3) (3 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 8)
  (parent 5)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (neq self b) (undisclosed l-0) (undisclosed l))
  (operation nonce-test (displaced 1 4 ltx-gen 3) (exp (gen) l-0) (3 1))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 9)
  (parent 6)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 10)
  (parent 6)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l-0)) l-1))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1))))
  (label 11)
  (parent 7)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec l-1))) l-0))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0))))
  (label 12)
  (parent 7)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec l-1))) l))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l))))
  (label 13)
  (parent 7)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 14)
  (parent 8)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 15)
  (parent 8)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (neq self b) (undisclosed l)
    (undisclosed lb))
  (operation nonce-test (displaced 4 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 16)
  (parent 9)
  (realized)
  (shape)
  (maps
    ((0)
      ((a self) (b b) (la l) (beta lb) (x x) (upsilon y) (na na) (nb nb)
        (priv-stor priv-stor-0) (pt pt-2))))
  (origs (nb (2 3)) (l (3 1)) (pt-2 (3 1)) (lb (1 1)) (pt-0 (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (neq self b) (undisclosed l-0) (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 17)
  (parent 9)
  (unrealized (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 18)
  (parent 10)
  (unrealized (0 3) (3 1) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (displaced 5 0 init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 19)
  (parent 10)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 20)
  (parent 10)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (neq self b) (undisclosed l-0) (undisclosed l))
  (operation nonce-test (displaced 1 5 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 21)
  (parent 10)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l-0) (undisclosed l))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 22)
  (parent 10)
  (unrealized (0 3) (3 1) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (displaced 5 2 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 23)
  (parent 10)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 24)
  (parent 10)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 25)
  (parent 11)
  (unrealized (0 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (lb l x rndx)
    (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (neq self b) (undisclosed l)
    (undisclosed lb))
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 26)
  (parent 17)
  (seen 9)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b)
    (undisclosed l-0) (undisclosed l))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 27)
  (parent 17)
  (unrealized (0 3) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b)
    (undisclosed l-0) (undisclosed l))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 28)
  (parent 17)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (la l y x rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la la) (x x) (beta l)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha la) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb la l)
  (uniq-gen y x)
  (absent (y (one)) (y la) (y l) (x la) (x l))
  (precur (4 0))
  (gen-st (pv b l) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b)
    (undisclosed la) (undisclosed l))
  (operation nonce-test (displaced 5 0 init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la l)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 29)
  (parent 17)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b)
    (undisclosed l) (undisclosed l-0))
  (operation nonce-test (displaced 5 1 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 30)
  (parent 17)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (neq self b) (undisclosed l-0)
    (undisclosed l))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 31)
  (parent 17)
  (unrealized (0 3) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b)
    (undisclosed l-0) (undisclosed l))
  (operation nonce-test (displaced 5 3 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 32)
  (parent 17)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 3 1) (trans 1 1) (trans 5 0) (trans 3 0)
    (trans 1 0) (neq self b) (undisclosed l-0) (undisclosed l))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 33)
  (parent 17)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 34)
  (parent 18)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta beta expt) (l x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 35)
  (parent 18)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a-0 l-1)))))
  (label 36)
  (parent 18)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (3 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (neq self b) (undisclosed l)
    (undisclosed lb))
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 37)
  (parent 21)
  (seen 16)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (neq self b) (undisclosed l-0) (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 38)
  (parent 21)
  (unrealized (5 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l-0) (undisclosed l))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 39)
  (parent 22)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l) (undisclosed l-0))
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 40)
  (parent 22)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 41)
  (parent 22)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 42)
  (parent 25)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 43)
  (parent 25)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (4 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3))
    ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b)
    (undisclosed l) (undisclosed l-0))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 44)
  (parent 27)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (0 3))
    ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b)
    (undisclosed l-0) (undisclosed l))
  (operation channel-test (displaced 6 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 45)
  (parent 27)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 2) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 3 1) (trans 1 1) (trans 6 0) (trans 3 0)
    (trans 1 0) (neq self b) (undisclosed l-0) (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 46)
  (parent 27)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (4 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3))
    ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (neq self b) (undisclosed l)
    (undisclosed l-0))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 47)
  (parent 31)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (0 3))
    ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (neq self b) (undisclosed l-0)
    (undisclosed l))
  (operation channel-test (displaced 6 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 48)
  (parent 31)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 3) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-1))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 3 1) (trans 1 1) (trans 6 0) (trans 3 0) (trans 1 0)
    (neq self b) (undisclosed l-0) (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 49)
  (parent 31)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (lb l x rndx)
    (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (3 0) (5 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (neq self b) (undisclosed l)
    (undisclosed lb))
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 50)
  (parent 38)
  (seen 26)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b)
    (undisclosed l-0) (undisclosed l))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 51)
  (parent 38)
  (unrealized (0 3) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b)
    (undisclosed l-0) (undisclosed l))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 52)
  (parent 38)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (la l y x rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la la) (x x) (beta l)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha la) (zeta (one)))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb la l)
  (uniq-gen y x)
  (absent (y (one)) (y la) (y l) (x la) (x l))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b)
    (undisclosed la) (undisclosed l))
  (operation nonce-test (displaced 6 0 init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la l)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 53)
  (parent 38)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b)
    (undisclosed l) (undisclosed l-0))
  (operation nonce-test (displaced 6 1 ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 54)
  (parent 38)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 3) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (neq self b) (undisclosed l-0)
    (undisclosed l))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 55)
  (parent 38)
  (unrealized (0 3) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b)
    (undisclosed l-0) (undisclosed l))
  (operation nonce-test (displaced 6 4 ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 56)
  (parent 38)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 4 1) (trans 1 1) (trans 6 0) (trans 4 0)
    (trans 1 0) (neq self b) (undisclosed l-0) (undisclosed l))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 57)
  (parent 38)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b)
    (undisclosed l) (undisclosed l-0))
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 58)
  (parent 51)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b)
    (undisclosed l-0) (undisclosed l))
  (operation channel-test (displaced 7 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 59)
  (parent 51)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 7 1) (trans 4 1) (trans 1 1) (trans 7 0) (trans 4 0)
    (trans 1 0) (neq self b) (undisclosed l-0) (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 60)
  (parent 51)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (0 3)) ((6 3) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (neq self b) (undisclosed l)
    (undisclosed l-0))
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 61)
  (parent 55)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (0 3)) ((6 3) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (neq self b) (undisclosed l-0)
    (undisclosed l))
  (operation channel-test (displaced 7 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 62)
  (parent 55)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 3) (5 0)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-1))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 7 1)
    (trans 4 1) (trans 1 1) (trans 7 0) (trans 4 0) (trans 1 0)
    (neq self b) (undisclosed l-0) (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 63)
  (parent 55)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(comment "Nothing left to do")

(defprotocol dhcr-um diffie-hellman
  (defrole init
    (vars (la x rndx) (beta upsilon expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv a la))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb))
    (uniq-orig na)
    (uniq-gen x)
    (absent (x la) (x beta))
    (assume (fact neq (exp (gen) upsilon) (gen)))
    (gen-st (pv a la)))
  (defrole resp
    (vars (lb y rndx) (alpha zeta expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv b lb))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    (uniq-orig nb)
    (uniq-gen y)
    (absent (y lb) (y alpha) (y zeta))
    (assume (fact neq (exp (gen) zeta) (gen)))
    (gen-st (pv b lb)))
  (defrole ltx-gen
    (vars (self name) (l rndx) (priv-stor locn) (ignore mesg))
    (trace (load priv-stor ignore) (stor priv-stor (pv self l))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    (uniq-orig l))
  (defrole ltx-disclose
    (vars (self name) (l rndx) (priv-stor locn))
    (trace (load priv-stor (pv self l)) (stor priv-stor "nil") (send l))
    (gen-st (pv self l)))
  (defrule undisclosed-not-disclosed
    (forall ((z strd) (l rndx))
      (implies
        (and (fact undisclosed l) (p "ltx-disclose" z 2)
          (p "ltx-disclose" "l" z l))
        (false))))
  (defgenrule neqRl_indx
    (forall ((x indx)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_strd
    (forall ((x strd)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_mesg
    (forall ((x mesg)) (implies (fact neq x x) (false))))
  (defgenrule scissorsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (leads-to z0 i0 z2 i2))
        (and (= z1 z2) (= i1 i2)))))
  (defgenrule cakeRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (leads-to z0 i0 z1 i1)
          (leads-to z0 i0 z2 i2) (prec z1 i1 z2 i2)) (false))))
  (defgenrule no-interruption
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (leads-to z0 i0 z2 i2) (trans z1 i1)
          (same-locn z0 i0 z1 i1) (prec z0 i0 z1 i1) (prec z1 i1 z2 i2))
        (false))))
  (defgenrule shearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (same-locn z0 i0 z2 i2)
          (prec z0 i0 z2 i2))
        (or (and (= z1 z2) (= i1 i2)) (prec z1 i1 z2 i2)))))
  (defgenrule invShearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (same-locn z0 i0 z1 i1)
          (leads-to z1 i1 z2 i2) (prec z0 i0 z2 i2))
        (or (and (= z0 z1) (= i0 i1)) (prec z0 i0 z1 i1)))))
  (defgenrule assume-init-0
    (forall ((z strd) (upsilon expt))
      (implies (and (p "init" z 4) (p "init" "upsilon" z upsilon))
        (fact neq (exp (gen) upsilon) (gen)))))
  (defgenrule assume-resp-0
    (forall ((z strd) (zeta expt))
      (implies (and (p "resp" z 3) (p "resp" "zeta" z zeta))
        (fact neq (exp (gen) zeta) (gen)))))
  (defgenrule trRl_ltx-gen-at-1
    (forall ((z strd)) (implies (p "ltx-gen" z 2) (trans z 1))))
  (defgenrule trRl_ltx-gen-at-0
    (forall ((z strd)) (implies (p "ltx-gen" z 2) (trans z 0))))
  (defgenrule trRl_ltx-disclose-at-1
    (forall ((z strd)) (implies (p "ltx-disclose" z 2) (trans z 1))))
  (defgenrule trRl_ltx-disclose-at-0
    (forall ((z strd)) (implies (p "ltx-disclose" z 2) (trans z 0))))
  (defgenrule gen-st-init-0
    (forall ((z strd) (la rndx) (a name))
      (implies
        (and (p "init" z 1) (p "init" "la" z la) (p "init" "a" z a))
        (gen-st (pv a la)))))
  (defgenrule gen-st-resp-0
    (forall ((z strd) (lb rndx) (b name))
      (implies
        (and (p "resp" z 1) (p "resp" "lb" z lb) (p "resp" "b" z b))
        (gen-st (pv b lb)))))
  (defgenrule gen-st-ltx-disclose-0
    (forall ((z strd) (l rndx) (self name))
      (implies
        (and (p "ltx-disclose" z 1) (p "ltx-disclose" "l" z l)
          (p "ltx-disclose" "self" z self)) (gen-st (pv self l)))))
  (lang (sig sign) (body (tuple 3)) (pv (tuple 2))))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (la rndx)
    (beta expt) (x rndx) (upsilon expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta beta) (upsilon upsilon))
  (non-orig (privk "sig" b))
  (uniq-orig na)
  (uniq-gen x)
  (absent (x la) (x beta))
  (facts (neq a b) (undisclosed beta))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb)))
  (label 64)
  (unrealized (0 1))
  (origs (na (0 2)))
  (comment "Not closed under rules"))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (la rndx)
    (beta expt) (x rndx) (upsilon expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta beta) (upsilon upsilon))
  (non-orig (privk "sig" b))
  (uniq-orig na)
  (uniq-gen x)
  (absent (x la) (x beta))
  (gen-st (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed beta))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb)))
  (label 65)
  (parent 64)
  (unrealized (0 0) (0 1))
  (origs (na (0 2)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (a b name) (pt pt-0 pval)
    (priv-stor locn) (beta expt) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (precedes ((1 1) (0 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x beta) (x l))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq a b) (undisclosed beta))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv a l))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul beta l))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l)))))
  (label 66)
  (parent 65)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((2 2) (0 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l-0))
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))))
  (label 67)
  (parent 66)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx)
    (zeta expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y zeta))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l-0))
  (operation encryption-test (added-strand resp 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 68)
  (parent 67)
  (unrealized (0 3) (3 0) (3 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l-0))
  (operation encryption-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send
        (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 69)
  (parent 67)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 4 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 70)
  (parent 68)
  (unrealized (0 3) (3 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1)))))
  (label 71)
  (parent 68)
  (unrealized (0 3) (3 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l-0))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 72)
  (parent 69)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (neq self b) (undisclosed l))
  (operation nonce-test (displaced 1 4 ltx-gen 3) (exp (gen) l-0) (3 1))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 73)
  (parent 70)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 74)
  (parent 70)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l-0)) l-1))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1))))
  (label 75)
  (parent 71)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec l-1))) l-0))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0))))
  (label 76)
  (parent 71)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec l-1))) l))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l))))
  (label 77)
  (parent 71)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 78)
  (parent 72)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 79)
  (parent 72)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (neq self b) (undisclosed lb))
  (operation nonce-test (displaced 4 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 80)
  (parent 73)
  (realized)
  (shape)
  (maps
    ((0)
      ((a self) (b b) (la l) (beta lb) (x x) (upsilon y) (na na) (nb nb)
        (priv-stor priv-stor-0) (pt pt-2))))
  (origs (nb (2 3)) (l (3 1)) (pt-2 (3 1)) (lb (1 1)) (pt-0 (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (neq self b) (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 81)
  (parent 73)
  (unrealized (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 82)
  (parent 74)
  (unrealized (0 3) (3 1) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (displaced 5 0 init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 83)
  (parent 74)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 84)
  (parent 74)
  (unrealized (0 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (neq self b) (undisclosed l))
  (operation nonce-test (displaced 1 5 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 85)
  (parent 74)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 86)
  (parent 74)
  (unrealized (0 3) (3 1) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (displaced 5 2 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 87)
  (parent 74)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 88)
  (parent 74)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 89)
  (parent 75)
  (unrealized (0 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b) (undisclosed l))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1)))
  (label 90)
  (parent 77)
  (unrealized (0 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 91)
  (parent 79)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (lb l x rndx)
    (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (neq self b) (undisclosed lb))
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 92)
  (parent 81)
  (seen 73)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b)
    (undisclosed l))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 93)
  (parent 81)
  (unrealized (0 3) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b)
    (undisclosed l))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 94)
  (parent 81)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (la l y x rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la la) (x x) (beta l)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha la) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb la l)
  (uniq-gen y x)
  (absent (y (one)) (y la) (y l) (x la) (x l))
  (precur (4 0))
  (gen-st (pv b l) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b)
    (undisclosed l))
  (operation nonce-test (displaced 5 0 init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la l)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 95)
  (parent 81)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b)
    (undisclosed l-0))
  (operation nonce-test (displaced 5 1 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 96)
  (parent 81)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (neq self b) (undisclosed l))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 97)
  (parent 81)
  (unrealized (0 3) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b)
    (undisclosed l))
  (operation nonce-test (displaced 5 3 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 98)
  (parent 81)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 3 1) (trans 1 1) (trans 5 0) (trans 3 0)
    (trans 1 0) (neq self b) (undisclosed l))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 99)
  (parent 81)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 100)
  (parent 82)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta beta expt) (l x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 101)
  (parent 82)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a-0 l-1)))))
  (label 102)
  (parent 82)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (4 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 103)
  (parent 84)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (3 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (neq self b) (undisclosed lb))
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 104)
  (parent 85)
  (seen 80)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (neq self b) (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 105)
  (parent 85)
  (unrealized (5 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 106)
  (parent 86)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 107)
  (parent 86)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 108)
  (parent 86)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 109)
  (parent 89)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 110)
  (parent 89)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (7 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 111)
  (parent 90)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 112)
  (parent 90)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation nonce-test (displaced 7 2 ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 113)
  (parent 91)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 114)
  (parent 91)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (4 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3))
    ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b)
    (undisclosed l-0))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 115)
  (parent 93)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (0 3))
    ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b)
    (undisclosed l))
  (operation channel-test (displaced 6 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 116)
  (parent 93)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 2) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 3 1) (trans 1 1) (trans 6 0) (trans 3 0)
    (trans 1 0) (neq self b) (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 117)
  (parent 93)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (4 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3))
    ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (neq self b) (undisclosed l-0))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 118)
  (parent 97)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (0 3))
    ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (neq self b) (undisclosed l))
  (operation channel-test (displaced 6 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 119)
  (parent 97)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 3) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-1))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 3 1) (trans 1 1) (trans 6 0) (trans 3 0) (trans 1 0)
    (neq self b) (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 120)
  (parent 97)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0)
    (neq a b) (undisclosed lb))
  (operation nonce-test (displaced 6 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 121)
  (parent 103)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq a b) (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 122)
  (parent 103)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (lb l x rndx)
    (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (3 0) (5 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (neq self b) (undisclosed lb))
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 123)
  (parent 105)
  (seen 92)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b)
    (undisclosed l))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 124)
  (parent 105)
  (unrealized (0 3) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b)
    (undisclosed l))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 125)
  (parent 105)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (la l y x rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la la) (x x) (beta l)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha la) (zeta (one)))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb la l)
  (uniq-gen y x)
  (absent (y (one)) (y la) (y l) (x la) (x l))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b)
    (undisclosed l))
  (operation nonce-test (displaced 6 0 init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la l)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 126)
  (parent 105)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b)
    (undisclosed l-0))
  (operation nonce-test (displaced 6 1 ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 127)
  (parent 105)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 3) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (neq self b) (undisclosed l))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 128)
  (parent 105)
  (unrealized (0 3) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b)
    (undisclosed l))
  (operation nonce-test (displaced 6 4 ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 129)
  (parent 105)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 4 1) (trans 1 1) (trans 6 0) (trans 4 0)
    (trans 1 0) (neq self b) (undisclosed l))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 130)
  (parent 105)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 131)
  (parent 110)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 132)
  (parent 111)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq a b)
    (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 133)
  (parent 113)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 la))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 134)
  (parent 114)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (upsilon expt) (la l x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) l) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na la l)
  (uniq-gen x)
  (absent (x la) (x l))
  (precur (7 0))
  (gen-st (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l))
  (operation nonce-test (displaced 8 0 init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) l) la)) (send (cat (exp (gen) l) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-3 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 135)
  (parent 114)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 136)
  (parent 114)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 1) (2 0))
    ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 1) (5 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 7 1) (trans 1 1)
    (trans 7 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq self b)
    (undisclosed l))
  (operation nonce-test (displaced 1 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 137)
  (parent 114)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l lb rndx) (alpha zeta expt) (l-0 y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y lb) (y alpha) (y zeta))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 138)
  (parent 114)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation nonce-test (displaced 8 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 139)
  (parent 114)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 8 1) (trans 2 1)
    (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 140)
  (parent 114)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (3 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 4 1) (trans 4 0) (neq a b) (undisclosed lb))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 141)
  (parent 121)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq a b)
    (undisclosed lb))
  (operation nonce-test (displaced 7 3 resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 142)
  (parent 122)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 143)
  (parent 122)
  (unrealized (0 3) (6 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b) (undisclosed l))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 144)
  (parent 122)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (la l y x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha la) (zeta (one)))
  (deflistener (cat (gen) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb la l)
  (uniq-gen y x)
  (absent (y (one)) (y la) (y l) (x la) (x l))
  (precur (4 0) (6 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b) (undisclosed l))
  (operation nonce-test (displaced 7 0 init 3) (exp (gen) x-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul la l)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((recv (cat (gen) la)) (send (cat (gen) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-3 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 145)
  (parent 122)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (0 0))
    ((6 1) (4 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 1 1) (trans 6 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq self b) (undisclosed l))
  (operation nonce-test (displaced 1 7 ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 146)
  (parent 122)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)) ((7 3) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0)
    (neq a b) (undisclosed l))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 147)
  (parent 122)
  (unrealized (0 3) (6 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (displaced 7 2 ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 148)
  (parent 122)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 7 1) (trans 2 1) (trans 1 1) (trans 7 0) (trans 2 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 149)
  (parent 122)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b)
    (undisclosed l-0))
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 150)
  (parent 124)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b)
    (undisclosed l))
  (operation channel-test (displaced 7 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 151)
  (parent 124)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 7 1) (trans 4 1) (trans 1 1) (trans 7 0) (trans 4 0)
    (trans 1 0) (neq self b) (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 152)
  (parent 124)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (0 3)) ((6 3) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (neq self b) (undisclosed l-0))
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 153)
  (parent 128)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (0 3)) ((6 3) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (neq self b) (undisclosed l))
  (operation channel-test (displaced 7 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 154)
  (parent 128)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 3) (5 0)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-1))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 7 1)
    (trans 4 1) (trans 1 1) (trans 7 0) (trans 4 0) (trans 1 0)
    (neq self b) (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 155)
  (parent 128)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 156)
  (parent 131)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 157)
  (parent 131)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0)))
  (label 158)
  (parent 132)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 159)
  (parent 132)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (3 0))
    ((3 1) (0 3)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b)
    (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 160)
  (parent 133)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 161)
  (parent 134)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 162)
  (parent 134)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 9 1) (trans 2 1)
    (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 163)
  (parent 134)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon alpha zeta expt) (l y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b) (undisclosed l))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 164)
  (parent 138)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (alpha zeta expt) (y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec y) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 165)
  (parent 138)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (alpha zeta expt) (l-0 y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 166)
  (parent 138)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq a b)
    (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 167)
  (parent 139)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 3 1) (trans 3 0) (neq a b) (undisclosed lb))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 168)
  (parent 141)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (3 0) (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b)
    (undisclosed lb))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 169)
  (parent 142)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (0 3)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b) (undisclosed l))
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 170)
  (parent 143)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (0 3)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 8 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 171)
  (parent 143)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 2 1) (trans 1 1) (trans 8 0) (trans 2 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq a b) (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 172)
  (parent 143)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (0 3)) ((7 3) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0)
    (neq a b) (undisclosed l))
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 173)
  (parent 147)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (0 3)) ((7 3) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 8 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 174)
  (parent 147)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)) ((7 3) (6 0)) ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-1))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 2 1) (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0)
    (trans 5 1) (trans 5 0) (neq a b) (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 175)
  (parent 147)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation nonce-test (displaced 9 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 176)
  (parent 156)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 177)
  (parent 156)
  (unrealized (9 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 178)
  (parent 157)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 179)
  (parent 157)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 180)
  (parent 157)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 181)
  (parent 157)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 182)
  (parent 157)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((2 3) (0 3)) ((3 1) (2 0))
    ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 1) (7 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 3 1) (trans 1 1)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b) (undisclosed l-0))
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 183)
  (parent 157)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (5 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a self) (undisclosed l-0))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 184)
  (parent 157)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 10 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-1))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 185)
  (parent 157)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation nonce-test (displaced 9 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 186)
  (parent 158)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 187)
  (parent 158)
  (unrealized (9 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 188)
  (parent 159)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 189)
  (parent 159)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 190)
  (parent 159)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 191)
  (parent 159)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 192)
  (parent 159)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((2 3) (0 3)) ((3 1) (2 0))
    ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 1) (5 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 3 1) (trans 1 1)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b) (undisclosed l-0))
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 193)
  (parent 159)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (7 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a self) (undisclosed l-0))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 194)
  (parent 159)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 10 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-1))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 195)
  (parent 159)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b)
    (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 196)
  (parent 160)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b) (la l) (beta l-0) (x x) (upsilon upsilon) (na na)
        (nb nb) (priv-stor priv-stor) (pt pt))))
  (origs (l-0 (2 1)) (pt-2 (2 1)) (pt-3 (3 1)) (l (1 1)) (pt (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b)
    (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 197)
  (parent 167)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 3 1) (trans 3 0) (neq a b) (undisclosed lb))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 198)
  (parent 168)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b) (la l) (beta lb) (x x) (upsilon y) (na na) (nb nb)
        (priv-stor priv-stor) (pt pt))))
  (origs (pt-3 (3 1)) (lb (2 1)) (pt-2 (2 1)) (l (1 1)) (pt (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 2) (4 0)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b)
    (undisclosed lb))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 199)
  (parent 169)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1)) ((2 2) (6 0))
    ((3 1) (5 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 200)
  (parent 176)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 201)
  (parent 177)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 202)
  (parent 177)
  (unrealized (0 3) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 203)
  (parent 177)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 204)
  (parent 177)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (8 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 1) (0 0))
    ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-0))
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 205)
  (parent 177)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-1))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 206)
  (parent 177)
  (unrealized (0 3) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 207)
  (parent 177)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 3) (8 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 1) (3 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self) (undisclosed l-0))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 208)
  (parent 177)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 10 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 209)
  (parent 177)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 210)
  (parent 178)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 211)
  (parent 178)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 212)
  (parent 178)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 213)
  (parent 178)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 214)
  (parent 179)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 215)
  (parent 179)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 216)
  (parent 179)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 217)
  (parent 179)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 218)
  (parent 182)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 219)
  (parent 182)
  (unrealized (10 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1)) ((2 2) (6 0))
    ((3 1) (7 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 220)
  (parent 186)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 221)
  (parent 187)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 222)
  (parent 187)
  (unrealized (0 3) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 223)
  (parent 187)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 224)
  (parent 187)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (8 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-0))
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 225)
  (parent 187)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-1))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 226)
  (parent 187)
  (unrealized (0 3) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 227)
  (parent 187)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 3) (8 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 1) (3 0))
    ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self) (undisclosed l-0))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 228)
  (parent 187)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 10 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 229)
  (parent 187)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 230)
  (parent 188)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 231)
  (parent 188)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 232)
  (parent 188)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 233)
  (parent 188)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 234)
  (parent 189)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 235)
  (parent 189)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 236)
  (parent 189)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 237)
  (parent 189)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 238)
  (parent 192)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 239)
  (parent 192)
  (unrealized (10 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 2) (0 3)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b)
    (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 240)
  (parent 197)
  (seen 196)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 3)) ((3 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b)
    (undisclosed lb))
  (operation generalization deleted (4 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 241)
  (parent 199)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (4 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 242)
  (parent 200)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (5 0)) ((4 1) (8 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 243)
  (parent 201)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 244)
  (parent 202)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 245)
  (parent 202)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 246)
  (parent 202)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 247)
  (parent 202)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-1))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 248)
  (parent 206)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 249)
  (parent 206)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 250)
  (parent 206)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 251)
  (parent 206)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1)) ((2 2) (8 0))
    ((3 1) (5 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 252)
  (parent 218)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 253)
  (parent 219)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 254)
  (parent 219)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 255)
  (parent 219)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 256)
  (parent 219)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3))
    ((10 1) (0 0)) ((10 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (9 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 3 1) (trans 1 1) (trans 10 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-0))
  (operation nonce-test (displaced 1 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 257)
  (parent 219)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-1))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 258)
  (parent 219)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (displaced 11 2 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 259)
  (parent 219)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3))
    ((10 1) (3 0)) ((10 1) (5 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self) (undisclosed l-0))
  (operation nonce-test (displaced 4 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 260)
  (parent 219)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 261)
  (parent 219)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 262)
  (parent 220)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (7 0)) ((4 1) (8 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 263)
  (parent 221)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 264)
  (parent 222)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 265)
  (parent 222)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 266)
  (parent 222)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 267)
  (parent 222)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-1))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 268)
  (parent 226)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 269)
  (parent 226)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 270)
  (parent 226)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 271)
  (parent 226)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1)) ((2 2) (8 0))
    ((3 1) (7 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 272)
  (parent 238)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 273)
  (parent 239)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 274)
  (parent 239)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 275)
  (parent 239)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 276)
  (parent 239)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3))
    ((10 1) (0 0)) ((10 1) (5 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (9 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 3 1) (trans 1 1) (trans 10 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-0))
  (operation nonce-test (displaced 1 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 277)
  (parent 239)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-1))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 278)
  (parent 239)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (displaced 11 2 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 279)
  (parent 239)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3))
    ((10 1) (3 0)) ((10 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self) (undisclosed l-0))
  (operation nonce-test (displaced 4 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 280)
  (parent 239)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 281)
  (parent 239)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 1) (4 0)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b)
    (undisclosed lb))
  (operation generalization weakened ((3 2) (4 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 282)
  (parent 241)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 1) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0)
    (neq a b) (undisclosed l-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l)))
  (label 283)
  (parent 242)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (4 0)) ((4 2) (7 0)) ((5 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 284)
  (parent 243)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1)) ((2 2) (7 0))
    ((3 1) (4 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 285)
  (parent 252)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 1) (5 0)) ((4 1) (9 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 286)
  (parent 253)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (11 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 287)
  (parent 254)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (11 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 288)
  (parent 254)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 289)
  (parent 254)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 290)
  (parent 254)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (11 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-1))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 291)
  (parent 258)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (11 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 292)
  (parent 258)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 293)
  (parent 258)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 12 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 12 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 294)
  (parent 258)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (5 0))
    ((4 2) (0 3)) ((5 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 4 1)
    (trans 5 0) (trans 4 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 295)
  (parent 262)
  (seen 308)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (6 0)) ((4 2) (7 0)) ((5 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 296)
  (parent 263)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (7 0))
    ((3 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 297)
  (parent 272)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 1) (7 0)) ((4 1) (9 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 298)
  (parent 273)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (11 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 299)
  (parent 274)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (11 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 300)
  (parent 274)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 301)
  (parent 274)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 302)
  (parent 274)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (11 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-1))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 303)
  (parent 278)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (11 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 304)
  (parent 278)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 305)
  (parent 278)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 12 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 12 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 306)
  (parent 278)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b)
    (undisclosed lb))
  (operation generalization weakened ((3 1) (4 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 307)
  (parent 282)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b) (undisclosed l-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l)))
  (label 308)
  (parent 283)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 1) (6 0)) ((4 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 309)
  (parent 284)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1)) ((2 2) (6 0))
    ((3 1) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0)
    (neq a b) (undisclosed l-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 310)
  (parent 285)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (4 0)) ((4 2) (8 0)) ((5 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 311)
  (parent 286)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 2) (6 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 4 1) (trans 5 0) (trans 4 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 312)
  (parent 296)
  (seen 317)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (6 0))
    ((3 1) (5 0)) ((4 2) (0 3)) ((5 2) (0 3)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 4 1)
    (trans 5 0) (trans 4 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 313)
  (parent 297)
  (seen 318)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (6 0)) ((4 2) (8 0)) ((5 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 314)
  (parent 298)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b)
    (undisclosed lb))
  (operation generalization weakened ((0 2) (4 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 315)
  (parent 307)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b) (undisclosed l-0))
  (operation generalization weakened ((3 1) (0 3)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l)))
  (label 316)
  (parent 308)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 2) (5 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 317)
  (parent 309)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 2) (0 3)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b) (undisclosed l-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 318)
  (parent 310)
  (seen 308)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (7 0)) ((4 1) (7 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 319)
  (parent 311)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (5 0)) ((4 2) (7 0)) ((5 2) (7 0))
    ((6 1) (7 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 4 1) (trans 5 0) (trans 4 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 320)
  (parent 314)
  (seen 324)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b)
    (undisclosed lb))
  (operation generalization weakened ((1 1) (4 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 321)
  (parent 315)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b) (undisclosed l-0))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l)))
  (label 322)
  (parent 316)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((3 1) (5 0)) ((4 2) (0 3)) ((4 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (5 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 323)
  (parent 317)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (6 0)) ((4 2) (6 0)) ((5 1) (6 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b) (undisclosed l-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 324)
  (parent 319)
  (seen 317)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (4 0)) ((2 2) (0 1))
    ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b)
    (undisclosed lb))
  (operation generalization weakened ((2 2) (4 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 325)
  (parent 321)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b-0) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b-0)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((4 2) (0 3)))
  (non-orig (privk "sig" b) (privk "sig" b-0))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b-0) (undisclosed l-0))
  (operation generalization separated b-0)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b-0 (exp (gen) l-0) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na a b-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b-0 l-0)))
      (send
        (sig (body b-0 (exp (gen) l-0) (pubk "sig" b-0))
          (privk "sig" b-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l)))
  (label 326)
  (parent 322)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 2) (0 3)) ((4 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b) (undisclosed l-0))
  (operation generalization weakened ((3 1) (0 3)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 327)
  (parent 323)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b)
    (undisclosed lb))
  (operation generalization weakened ((2 1) (4 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 328)
  (parent 325)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b-0) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b-0)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((4 2) (0 3)))
  (non-orig (privk "sig" b-0))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b-0) (undisclosed l-0))
  (operation generalization forgot (privk "sig" b))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b-0 (exp (gen) l-0) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na a b-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b-0 l-0)))
      (send
        (sig (body b-0 (exp (gen) l-0) (pubk "sig" b-0))
          (privk "sig" b-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l)))
  (label 329)
  (parent 326)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b-0) (la l) (beta l-0) (x x) (upsilon y) (na na) (nb nb)
        (priv-stor priv-stor) (pt pt))))
  (origs (l-0 (2 1)) (pt-2 (2 1)) (pt-5 (4 1)) (lb (3 1)) (pt-3 (3 1))
    (l (1 1)) (pt (1 1)) (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 1) (5 0)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b) (undisclosed l-0))
  (operation generalization weakened ((4 2) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 330)
  (parent 327)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b)
    (undisclosed lb))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 331)
  (parent 328)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b) (la l) (beta lb) (x x) (upsilon (mul w y)) (na na)
        (nb nb) (priv-stor priv-stor) (pt pt))))
  (origs (pt-3 (3 1)) (lb (2 1)) (pt-2 (2 1)) (l (1 1)) (pt (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b) (undisclosed l-0))
  (operation generalization weakened ((4 1) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 332)
  (parent 330)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b) (undisclosed l-0))
  (operation generalization weakened ((3 1) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 333)
  (parent 332)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b) (undisclosed l-0))
  (operation generalization weakened ((0 2) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 334)
  (parent 333)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b) (undisclosed l-0))
  (operation generalization weakened ((1 1) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 335)
  (parent 334)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b) (undisclosed l-0))
  (operation generalization weakened ((2 2) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 336)
  (parent 335)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b) (undisclosed l-0))
  (operation generalization weakened ((2 1) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 337)
  (parent 336)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b) (undisclosed l-0))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 338)
  (parent 337)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b-0) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b-0)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((4 2) (0 3)))
  (non-orig (privk "sig" b) (privk "sig" b-0))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b-0) (undisclosed l-0))
  (operation generalization separated b-0)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b-0 (exp (gen) l-0) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na a b-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b-0 l-0)))
      (send
        (sig (body b-0 (exp (gen) l-0) (pubk "sig" b-0))
          (privk "sig" b-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 339)
  (parent 338)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b-0) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b-0)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((4 2) (0 3)))
  (non-orig (privk "sig" b-0))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b-0) (undisclosed l-0))
  (operation generalization forgot (privk "sig" b))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b-0 (exp (gen) l-0) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na a b-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b-0 l-0)))
      (send
        (sig (body b-0 (exp (gen) l-0) (pubk "sig" b-0))
          (privk "sig" b-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 340)
  (parent 339)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b-0) (la l) (beta l-0) (x x) (upsilon (mul w y)) (na na)
        (nb nb) (priv-stor priv-stor) (pt pt))))
  (origs (l-0 (2 1)) (pt-2 (2 1)) (pt-5 (4 1)) (lb (3 1)) (pt-3 (3 1))
    (l (1 1)) (pt (1 1)) (na (0 2))))

(comment "Nothing left to do")

(defprotocol dhcr-um diffie-hellman
  (defrole init
    (vars (la x rndx) (beta upsilon expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv a la))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb))
    (uniq-orig na)
    (uniq-gen x)
    (absent (x la) (x beta))
    (assume (fact neq (exp (gen) upsilon) (gen)))
    (gen-st (pv a la)))
  (defrole resp
    (vars (lb y rndx) (alpha zeta expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv b lb))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    (uniq-orig nb)
    (uniq-gen y)
    (absent (y lb) (y alpha) (y zeta))
    (assume (fact neq (exp (gen) zeta) (gen)))
    (gen-st (pv b lb)))
  (defrole ltx-gen
    (vars (self name) (l rndx) (priv-stor locn) (ignore mesg))
    (trace (load priv-stor ignore) (stor priv-stor (pv self l))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    (uniq-orig l))
  (defrole ltx-disclose
    (vars (self name) (l rndx) (priv-stor locn))
    (trace (load priv-stor (pv self l)) (stor priv-stor "nil") (send l))
    (gen-st (pv self l)))
  (defrule undisclosed-not-disclosed
    (forall ((z strd) (l rndx))
      (implies
        (and (fact undisclosed l) (p "ltx-disclose" z 2)
          (p "ltx-disclose" "l" z l))
        (false))))
  (defgenrule neqRl_indx
    (forall ((x indx)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_strd
    (forall ((x strd)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_mesg
    (forall ((x mesg)) (implies (fact neq x x) (false))))
  (defgenrule scissorsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (leads-to z0 i0 z2 i2))
        (and (= z1 z2) (= i1 i2)))))
  (defgenrule cakeRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (leads-to z0 i0 z1 i1)
          (leads-to z0 i0 z2 i2) (prec z1 i1 z2 i2)) (false))))
  (defgenrule no-interruption
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (leads-to z0 i0 z2 i2) (trans z1 i1)
          (same-locn z0 i0 z1 i1) (prec z0 i0 z1 i1) (prec z1 i1 z2 i2))
        (false))))
  (defgenrule shearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (same-locn z0 i0 z2 i2)
          (prec z0 i0 z2 i2))
        (or (and (= z1 z2) (= i1 i2)) (prec z1 i1 z2 i2)))))
  (defgenrule invShearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (same-locn z0 i0 z1 i1)
          (leads-to z1 i1 z2 i2) (prec z0 i0 z2 i2))
        (or (and (= z0 z1) (= i0 i1)) (prec z0 i0 z1 i1)))))
  (defgenrule assume-init-0
    (forall ((z strd) (upsilon expt))
      (implies (and (p "init" z 4) (p "init" "upsilon" z upsilon))
        (fact neq (exp (gen) upsilon) (gen)))))
  (defgenrule assume-resp-0
    (forall ((z strd) (zeta expt))
      (implies (and (p "resp" z 3) (p "resp" "zeta" z zeta))
        (fact neq (exp (gen) zeta) (gen)))))
  (defgenrule trRl_ltx-gen-at-1
    (forall ((z strd)) (implies (p "ltx-gen" z 2) (trans z 1))))
  (defgenrule trRl_ltx-gen-at-0
    (forall ((z strd)) (implies (p "ltx-gen" z 2) (trans z 0))))
  (defgenrule trRl_ltx-disclose-at-1
    (forall ((z strd)) (implies (p "ltx-disclose" z 2) (trans z 1))))
  (defgenrule trRl_ltx-disclose-at-0
    (forall ((z strd)) (implies (p "ltx-disclose" z 2) (trans z 0))))
  (defgenrule gen-st-init-0
    (forall ((z strd) (la rndx) (a name))
      (implies
        (and (p "init" z 1) (p "init" "la" z la) (p "init" "a" z a))
        (gen-st (pv a la)))))
  (defgenrule gen-st-resp-0
    (forall ((z strd) (lb rndx) (b name))
      (implies
        (and (p "resp" z 1) (p "resp" "lb" z lb) (p "resp" "b" z b))
        (gen-st (pv b lb)))))
  (defgenrule gen-st-ltx-disclose-0
    (forall ((z strd) (l rndx) (self name))
      (implies
        (and (p "ltx-disclose" z 1) (p "ltx-disclose" "l" z l)
          (p "ltx-disclose" "self" z self)) (gen-st (pv self l)))))
  (lang (sig sign) (body (tuple 3)) (pv (tuple 2))))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (la x rndx)
    (beta upsilon expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta beta) (upsilon upsilon))
  (non-orig (privk "sig" b))
  (uniq-orig na)
  (uniq-gen x)
  (absent (x la) (x beta))
  (facts (neq a b) (undisclosed la))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb)))
  (label 341)
  (unrealized (0 1))
  (origs (na (0 2)))
  (comment "Not closed under rules"))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (la x rndx)
    (beta upsilon expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta beta) (upsilon upsilon))
  (non-orig (privk "sig" b))
  (uniq-orig na)
  (uniq-gen x)
  (absent (x la) (x beta))
  (gen-st (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed la))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb)))
  (label 342)
  (parent 341)
  (unrealized (0 0) (0 1))
  (origs (na (0 2)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (a b name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (beta upsilon expt) (l rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (precedes ((1 1) (0 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x beta) (x l))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq a b) (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv a l))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul beta l))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l)))))
  (label 343)
  (parent 342)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((2 2) (0 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l))
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))))
  (label 344)
  (parent 343)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx)
    (zeta expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y zeta))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l))
  (operation encryption-test (added-strand resp 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 345)
  (parent 344)
  (unrealized (0 3) (3 0) (3 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l))
  (operation encryption-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send
        (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 346)
  (parent 344)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l))
  (operation channel-test (displaced 4 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 347)
  (parent 345)
  (unrealized (0 3) (3 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1)))))
  (label 348)
  (parent 345)
  (unrealized (0 3) (3 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 349)
  (parent 346)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (neq self b) (undisclosed l-0))
  (operation nonce-test (displaced 1 4 ltx-gen 3) (exp (gen) l-0) (3 1))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 350)
  (parent 347)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 351)
  (parent 347)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l-0)) l-1))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1))))
  (label 352)
  (parent 348)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec l-1))) l-0))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0))))
  (label 353)
  (parent 348)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec l-1))) l))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l))))
  (label 354)
  (parent 348)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 355)
  (parent 349)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 356)
  (parent 349)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (neq self b) (undisclosed l))
  (operation nonce-test (displaced 4 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 357)
  (parent 350)
  (realized)
  (shape)
  (maps
    ((0)
      ((a self) (b b) (la l) (x x) (beta lb) (upsilon y) (na na) (nb nb)
        (priv-stor priv-stor-0) (pt pt-2))))
  (origs (nb (2 3)) (l (3 1)) (pt-2 (3 1)) (lb (1 1)) (pt-0 (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (neq self b) (undisclosed l-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 358)
  (parent 350)
  (unrealized (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 359)
  (parent 351)
  (unrealized (0 3) (3 1) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l))
  (operation nonce-test (displaced 5 0 init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 360)
  (parent 351)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 361)
  (parent 351)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (neq self b) (undisclosed l-0))
  (operation nonce-test (displaced 1 5 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 362)
  (parent 351)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l-0))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 363)
  (parent 351)
  (unrealized (0 3) (3 1) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l))
  (operation nonce-test (displaced 5 2 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 364)
  (parent 351)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 365)
  (parent 351)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b) (undisclosed l))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 366)
  (parent 352)
  (unrealized (0 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b) (undisclosed l))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1)))
  (label 367)
  (parent 353)
  (unrealized (0 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv b l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0)))
  (label 368)
  (parent 355)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (lb l x rndx)
    (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (neq self b) (undisclosed l))
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 369)
  (parent 358)
  (seen 350)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b)
    (undisclosed l-0))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 370)
  (parent 358)
  (unrealized (0 3) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b)
    (undisclosed l-0))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 371)
  (parent 358)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (la l y x rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la la) (x x) (beta l)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha la) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb la l)
  (uniq-gen y x)
  (absent (y (one)) (y la) (y l) (x la) (x l))
  (precur (4 0))
  (gen-st (pv b l) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b)
    (undisclosed la))
  (operation nonce-test (displaced 5 0 init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la l)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 372)
  (parent 358)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b)
    (undisclosed l))
  (operation nonce-test (displaced 5 1 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 373)
  (parent 358)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (neq self b) (undisclosed l-0))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 374)
  (parent 358)
  (unrealized (0 3) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b)
    (undisclosed l-0))
  (operation nonce-test (displaced 5 3 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 375)
  (parent 358)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 3 1) (trans 1 1) (trans 5 0) (trans 3 0)
    (trans 1 0) (neq self b) (undisclosed l-0))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 376)
  (parent 358)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 377)
  (parent 359)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta beta expt) (l x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l))
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 378)
  (parent 359)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a-0 l-1)))))
  (label 379)
  (parent 359)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (3 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (neq self b) (undisclosed l))
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 380)
  (parent 362)
  (seen 357)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (neq self b) (undisclosed l-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 381)
  (parent 362)
  (unrealized (5 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 382)
  (parent 363)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l))
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 383)
  (parent 363)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 384)
  (parent 363)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 385)
  (parent 366)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 386)
  (parent 366)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (7 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 387)
  (parent 367)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 388)
  (parent 367)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((6 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 6 1) (trans 1 1)
    (trans 6 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq self b)
    (undisclosed l-0))
  (operation nonce-test (displaced 1 7 ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 389)
  (parent 368)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 390)
  (parent 368)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (4 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3))
    ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b)
    (undisclosed l))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 391)
  (parent 370)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (0 3))
    ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b)
    (undisclosed l-0))
  (operation channel-test (displaced 6 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 392)
  (parent 370)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 2) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 3 1) (trans 1 1) (trans 6 0) (trans 3 0)
    (trans 1 0) (neq self b) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 393)
  (parent 370)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (4 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3))
    ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (neq self b) (undisclosed l))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 394)
  (parent 374)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (0 3))
    ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (neq self b) (undisclosed l-0))
  (operation channel-test (displaced 6 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 395)
  (parent 374)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 3) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-1))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 3 1) (trans 1 1) (trans 6 0) (trans 3 0) (trans 1 0)
    (neq self b) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 396)
  (parent 374)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (lb l x rndx)
    (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (3 0) (5 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (neq self b) (undisclosed l))
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 397)
  (parent 381)
  (seen 369)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b)
    (undisclosed l-0))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 398)
  (parent 381)
  (unrealized (0 3) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b)
    (undisclosed l-0))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 399)
  (parent 381)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (la l y x rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la la) (x x) (beta l)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha la) (zeta (one)))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb la l)
  (uniq-gen y x)
  (absent (y (one)) (y la) (y l) (x la) (x l))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b)
    (undisclosed la))
  (operation nonce-test (displaced 6 0 init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la l)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 400)
  (parent 381)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b)
    (undisclosed l))
  (operation nonce-test (displaced 6 1 ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 401)
  (parent 381)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 3) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (neq self b) (undisclosed l-0))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 402)
  (parent 381)
  (unrealized (0 3) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b)
    (undisclosed l-0))
  (operation nonce-test (displaced 6 4 ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 403)
  (parent 381)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 4 1) (trans 1 1) (trans 6 0) (trans 4 0)
    (trans 1 0) (neq self b) (undisclosed l-0))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 404)
  (parent 381)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1)))
  (label 405)
  (parent 385)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 406)
  (parent 387)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((5 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 5 1) (trans 1 1)
    (trans 5 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq self b)
    (undisclosed l-0))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 407)
  (parent 389)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 408)
  (parent 390)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (upsilon expt) (l l-0 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l))
  (operation nonce-test (displaced 8 0 init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul upsilon x)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 409)
  (parent 390)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 410)
  (parent 390)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 7 1) (trans 1 1)
    (trans 7 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq self b)
    (undisclosed l-0))
  (operation nonce-test (displaced 1 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 411)
  (parent 390)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l lb rndx) (alpha zeta expt) (l-0 y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y lb) (y alpha) (y zeta))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 412)
  (parent 390)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l))
  (operation nonce-test (displaced 8 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 413)
  (parent 390)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 8 1) (trans 2 1)
    (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 414)
  (parent 390)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b)
    (undisclosed l))
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 415)
  (parent 398)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b)
    (undisclosed l-0))
  (operation channel-test (displaced 7 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 416)
  (parent 398)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 7 1) (trans 4 1) (trans 1 1) (trans 7 0) (trans 4 0)
    (trans 1 0) (neq self b) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 417)
  (parent 398)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (0 3)) ((6 3) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (neq self b) (undisclosed l))
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 418)
  (parent 402)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (0 3)) ((6 3) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (neq self b) (undisclosed l-0))
  (operation channel-test (displaced 7 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 419)
  (parent 402)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 3) (5 0)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-1))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 7 1)
    (trans 4 1) (trans 1 1) (trans 7 0) (trans 4 0) (trans 1 0)
    (neq self b) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 420)
  (parent 402)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 8 1) (trans 3 1) (trans 1 1)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b) (undisclosed l-1))
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 421)
  (parent 405)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 422)
  (parent 405)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 8 1) (trans 3 1) (trans 1 1)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b) (undisclosed l-1))
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 423)
  (parent 406)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 424)
  (parent 406)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (2 0))
    ((4 1) (0 0)) ((4 2) (2 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq self b)
    (undisclosed l-0))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 425)
  (parent 407)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 426)
  (parent 408)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l))
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 427)
  (parent 408)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 9 1) (trans 2 1)
    (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 428)
  (parent 408)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (0 0))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 6 1) (trans 1 1)
    (trans 6 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq self b)
    (undisclosed l-0))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 429)
  (parent 411)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (alpha zeta expt) (y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec y) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 430)
  (parent 412)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon alpha zeta expt) (l y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b) (undisclosed l))
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 431)
  (parent 412)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (alpha zeta expt) (l-0 y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-1))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 432)
  (parent 412)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b)
    (undisclosed l-0))
  (operation nonce-test (displaced 9 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 433)
  (parent 421)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 8 1) (trans 3 1) (trans 1 1)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b) (undisclosed l-1))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 434)
  (parent 421)
  (unrealized (9 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 435)
  (parent 422)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 436)
  (parent 422)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 437)
  (parent 422)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) l-0 l-1)) (y l) (x l-0) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 438)
  (parent 422)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 439)
  (parent 422)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 3 1) (trans 1 1)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b) (undisclosed l-1))
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 440)
  (parent 422)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (5 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a self) (undisclosed l-0))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 441)
  (parent 422)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 10 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-1))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 442)
  (parent 422)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b)
    (undisclosed l-0))
  (operation nonce-test (displaced 9 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 443)
  (parent 423)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 8 1) (trans 3 1) (trans 1 1)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b) (undisclosed l-1))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 444)
  (parent 423)
  (unrealized (9 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 445)
  (parent 424)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 446)
  (parent 424)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 447)
  (parent 424)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) l-0 l-1)) (y l) (x l-0) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 448)
  (parent 424)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 449)
  (parent 424)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 3 1) (trans 1 1)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b) (undisclosed l-1))
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 450)
  (parent 424)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (7 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a self) (undisclosed l-0))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 451)
  (parent 424)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 10 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-1))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 452)
  (parent 424)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)) ((3 1) (0 0))
    ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self b)
    (undisclosed l-0))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 453)
  (parent 425)
  (realized)
  (shape)
  (maps
    ((0)
      ((a self) (b b) (la l-0) (x x) (beta l) (upsilon upsilon) (na na)
        (nb nb) (priv-stor priv-stor-0) (pt pt-3))))
  (origs (l-0 (3 1)) (pt-3 (3 1)) (pt-1 (2 1)) (l (1 1)) (pt-0 (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (2 0))
    ((4 1) (2 0)) ((5 1) (0 0)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 5 1) (trans 1 1)
    (trans 5 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq self b)
    (undisclosed l-0))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 454)
  (parent 429)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (4 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (0 0))
    ((7 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 7 1) (trans 2 1) (trans 1 1)
    (trans 7 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 455)
  (parent 433)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 8 1) (trans 3 1)
    (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq self b) (undisclosed l-0))
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 456)
  (parent 434)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-1))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 457)
  (parent 434)
  (unrealized (0 3) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-1))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 458)
  (parent 434)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) la) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed la))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) la l-0)) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) la) l-0)) (send (cat (exp (gen) la) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 459)
  (parent 434)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-0))
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 460)
  (parent 434)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b)
    (undisclosed l-1))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 461)
  (parent 434)
  (unrealized (0 3) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (0 0)) ((7 2) (5 0)) ((8 1) (0 3)) ((9 1) (2 0))
    ((9 1) (4 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self-0))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv self l-0) (pv self-0 l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 7 1) (trans 1 1) (trans 9 0) (trans 7 0)
    (trans 1 0) (trans 6 1) (trans 4 1) (trans 6 0) (trans 4 0)
    (neq self self-0) (undisclosed l-0))
  (operation nonce-test (displaced 3 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-0 (cat pt-4 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-2 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 462)
  (parent 434)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-1))
  (operation nonce-test (displaced 10 8 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 463)
  (parent 434)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 8 1) (trans 3 1) (trans 1 1) (trans 10 0)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b) (undisclosed l-1))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 464)
  (parent 434)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 465)
  (parent 435)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 466)
  (parent 435)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 467)
  (parent 435)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 468)
  (parent 435)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 469)
  (parent 436)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 470)
  (parent 436)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 471)
  (parent 436)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 472)
  (parent 436)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b)
    (undisclosed l-0))
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 473)
  (parent 440)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 3 1) (trans 1 1)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b) (undisclosed l-1))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 474)
  (parent 440)
  (unrealized (10 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (0 0))
    ((7 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 7 1) (trans 2 1) (trans 1 1)
    (trans 7 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 475)
  (parent 443)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 8 1) (trans 3 1)
    (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq self b) (undisclosed l-0))
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 476)
  (parent 444)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-1))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 477)
  (parent 444)
  (unrealized (0 3) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-1))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 478)
  (parent 444)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) la) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed la))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) la l-0)) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) la) l)) (send (cat (exp (gen) la) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 479)
  (parent 444)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-0))
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 480)
  (parent 444)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b)
    (undisclosed l-1))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 481)
  (parent 444)
  (unrealized (0 3) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (0 0)) ((7 2) (5 0)) ((8 1) (0 3)) ((9 1) (2 0))
    ((9 1) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self-0))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv self l-0) (pv self-0 l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 7 1) (trans 1 1) (trans 9 0) (trans 7 0)
    (trans 1 0) (trans 6 1) (trans 4 1) (trans 6 0) (trans 4 0)
    (neq self self-0) (undisclosed l-0))
  (operation nonce-test (displaced 3 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-0 (cat pt-4 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-2 "nil")) (send l-1))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 482)
  (parent 444)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-1))
  (operation nonce-test (displaced 10 8 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 483)
  (parent 444)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 8 1) (trans 3 1) (trans 1 1) (trans 10 0)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b) (undisclosed l-1))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 484)
  (parent 444)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 485)
  (parent 445)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 486)
  (parent 445)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 487)
  (parent 445)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 488)
  (parent 445)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 489)
  (parent 446)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 490)
  (parent 446)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 491)
  (parent 446)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 492)
  (parent 446)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b)
    (undisclosed l-0))
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 493)
  (parent 450)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 3 1) (trans 1 1)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b) (undisclosed l-1))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 494)
  (parent 450)
  (unrealized (10 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)) ((3 1) (0 3))
    ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self b)
    (undisclosed l-0))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 495)
  (parent 454)
  (seen 453)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (3 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (0 0)) ((6 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1)
    (trans 5 0) (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 496)
  (parent 455)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 1) (8 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (0 0)) ((7 2) (5 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 7 1) (trans 2 1)
    (trans 1 1) (trans 7 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 497)
  (parent 456)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((3 1) (10 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-0))
  (operation channel-test (displaced 11 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 498)
  (parent 457)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-0))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 499)
  (parent 457)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (10 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-1))
  (operation channel-test (displaced 11 8 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 500)
  (parent 457)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv a l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 8 1) (trans 3 1) (trans 1 1) (trans 11 0)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a l-2)))))
  (label 501)
  (parent 457)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((3 1) (10 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b)
    (undisclosed l-0))
  (operation channel-test (displaced 11 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 502)
  (parent 461)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b)
    (undisclosed l-0))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 503)
  (parent 461)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (10 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b)
    (undisclosed l-1))
  (operation channel-test (displaced 11 8 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 504)
  (parent 461)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 11 0) (trans 8 0)
    (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0)
    (trans 5 0) (neq self b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 505)
  (parent 461)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (4 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 8 1) (trans 2 1) (trans 1 1)
    (trans 8 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 506)
  (parent 473)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 9 1) (trans 3 1)
    (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq self b) (undisclosed l-0))
  (operation nonce-test (displaced 11 2 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 507)
  (parent 474)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-1))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 508)
  (parent 474)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-1))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 509)
  (parent 474)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) la) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed la))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) la l-0)) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) la) l-0)) (send (cat (exp (gen) la) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 510)
  (parent 474)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-0))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 511)
  (parent 474)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b)
    (undisclosed l-1))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 512)
  (parent 474)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)) ((9 1) (0 3))
    ((10 1) (2 0)) ((10 1) (4 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" self-0))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv self l-0) (pv self-0 l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 8 1) (trans 1 1) (trans 10 0) (trans 8 0)
    (trans 1 0) (trans 6 1) (trans 4 1) (trans 6 0) (trans 4 0)
    (neq self self-0) (undisclosed l-0))
  (operation nonce-test (displaced 3 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-0 (cat pt-4 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 513)
  (parent 474)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-1))
  (operation nonce-test (displaced 11 9 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 514)
  (parent 474)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 9 1) (trans 3 1) (trans 1 1) (trans 11 0)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b) (undisclosed l-1))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 515)
  (parent 474)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (0 0)) ((6 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1)
    (trans 5 0) (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 516)
  (parent 475)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 1) (8 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (0 0)) ((7 2) (5 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 7 1) (trans 2 1)
    (trans 1 1) (trans 7 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 517)
  (parent 476)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((3 1) (10 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-0))
  (operation channel-test (displaced 11 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 518)
  (parent 477)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-0))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 519)
  (parent 477)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (10 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-1))
  (operation channel-test (displaced 11 8 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 520)
  (parent 477)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv a l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 8 1) (trans 3 1) (trans 1 1) (trans 11 0)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a l-2)))))
  (label 521)
  (parent 477)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((3 1) (10 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b)
    (undisclosed l-0))
  (operation channel-test (displaced 11 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 522)
  (parent 481)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b)
    (undisclosed l-0))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 523)
  (parent 481)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (10 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b)
    (undisclosed l-1))
  (operation channel-test (displaced 11 8 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 524)
  (parent 481)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 11 0) (trans 8 0)
    (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0)
    (trans 5 0) (neq self b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 525)
  (parent 481)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 8 1) (trans 2 1) (trans 1 1)
    (trans 8 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 526)
  (parent 493)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 9 1) (trans 3 1)
    (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq self b) (undisclosed l-0))
  (operation nonce-test (displaced 11 2 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 527)
  (parent 494)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-1))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 528)
  (parent 494)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-1))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 529)
  (parent 494)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) la) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed la))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) la l-0)) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) la) l)) (send (cat (exp (gen) la) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 530)
  (parent 494)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-0))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 531)
  (parent 494)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b)
    (undisclosed l-1))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 532)
  (parent 494)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)) ((9 1) (0 3))
    ((10 1) (2 0)) ((10 1) (6 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" self-0))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv self l-0) (pv self-0 l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 8 1) (trans 1 1) (trans 10 0) (trans 8 0)
    (trans 1 0) (trans 6 1) (trans 4 1) (trans 6 0) (trans 4 0)
    (neq self self-0) (undisclosed l-0))
  (operation nonce-test (displaced 3 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-0 (cat pt-4 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-2 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 533)
  (parent 494)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-1))
  (operation nonce-test (displaced 11 9 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 534)
  (parent 494)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 9 1) (trans 3 1) (trans 1 1) (trans 11 0)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b) (undisclosed l-1))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 535)
  (parent 494)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (0 0)) ((5 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq self b) (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 536)
  (parent 496)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (3 0))
    ((3 2) (7 0)) ((4 1) (7 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((6 2) (4 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 537)
  (parent 497)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (3 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 7 1) (trans 2 1) (trans 1 1)
    (trans 7 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1)
    (trans 5 0) (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 538)
  (parent 506)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 1) (9 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (9 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 8 1) (trans 2 1)
    (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 539)
  (parent 507)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((3 1) (11 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-0))
  (operation channel-test (displaced 12 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 540)
  (parent 508)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 1) (11 0)) ((1 2) (0 1))
    ((2 3) (10 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-0))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 541)
  (parent 508)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-1))
  (operation channel-test (displaced 12 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 542)
  (parent 508)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv a l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 9 1) (trans 3 1) (trans 1 1) (trans 12 0)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a l-2)))))
  (label 543)
  (parent 508)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((3 1) (11 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b)
    (undisclosed l-0))
  (operation channel-test (displaced 12 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 544)
  (parent 512)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 1) (11 0)) ((1 2) (0 1))
    ((2 3) (10 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b)
    (undisclosed l-0))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 545)
  (parent 512)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b)
    (undisclosed l-1))
  (operation channel-test (displaced 12 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 546)
  (parent 512)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 12 1)
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 12 0) (trans 9 0)
    (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0)
    (trans 5 0) (neq self b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 547)
  (parent 512)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0)) ((3 2) (0 3))
    ((4 2) (0 3)) ((5 1) (0 0)) ((5 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1)
    (trans 4 0) (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 548)
  (parent 516)
  (seen 560)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (7 0)) ((4 1) (7 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((6 2) (4 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 549)
  (parent 517)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 7 1) (trans 2 1) (trans 1 1)
    (trans 7 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1)
    (trans 5 0) (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 550)
  (parent 526)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 1) (9 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (9 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 8 1) (trans 2 1)
    (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 551)
  (parent 527)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((3 1) (11 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-0))
  (operation channel-test (displaced 12 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 552)
  (parent 528)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 1) (11 0)) ((1 2) (0 1))
    ((2 3) (10 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-0))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 553)
  (parent 528)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-1))
  (operation channel-test (displaced 12 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 554)
  (parent 528)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv a l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 9 1) (trans 3 1) (trans 1 1) (trans 12 0)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a l-2)))))
  (label 555)
  (parent 528)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((3 1) (11 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b)
    (undisclosed l-0))
  (operation channel-test (displaced 12 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 556)
  (parent 532)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 1) (11 0)) ((1 2) (0 1))
    ((2 3) (10 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b)
    (undisclosed l-0))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 557)
  (parent 532)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b)
    (undisclosed l-1))
  (operation channel-test (displaced 12 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 558)
  (parent 532)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 12 1)
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 12 0) (trans 9 0)
    (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0)
    (trans 5 0) (neq self b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 559)
  (parent 532)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (0 3))
    ((4 1) (0 0)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0)
    (neq self b) (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 560)
  (parent 536)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 1) (6 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((5 2) (3 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 561)
  (parent 537)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (0 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq self b) (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 562)
  (parent 538)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (3 0))
    ((3 2) (8 0)) ((4 1) (8 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 7 1) (trans 2 1)
    (trans 1 1) (trans 7 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 563)
  (parent 539)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 2) (6 0)) ((4 2) (6 0)) ((5 1) (0 0)) ((5 2) (6 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 3 1) (trans 4 0) (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 564)
  (parent 549)
  (seen 568)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0)) ((3 2) (0 3))
    ((4 2) (0 3)) ((5 1) (0 3)) ((6 1) (0 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1)
    (trans 4 0) (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 565)
  (parent 550)
  (seen 569)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (8 0)) ((4 1) (8 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 7 1) (trans 2 1)
    (trans 1 1) (trans 7 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 566)
  (parent 551)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((3 2) (0 3)) ((4 1) (0 0))
    ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0)
    (neq self b) (undisclosed l-0))
  (operation generalization weakened ((2 1) (0 3)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 567)
  (parent 560)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (5 0)) ((4 1) (0 0)) ((4 2) (5 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 568)
  (parent 561)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 1) (0 0)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (4 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0)
    (neq self b) (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 569)
  (parent 562)
  (seen 560)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (7 0))
    ((3 1) (7 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (7 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 570)
  (parent 563)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 2) (7 0)) ((4 2) (7 0)) ((5 1) (7 0)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (7 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 3 1) (trans 4 0) (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 571)
  (parent 566)
  (seen 574)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((3 2) (0 3)) ((4 1) (0 0))
    ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0)
    (neq self b) (undisclosed l-0))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 572)
  (parent 567)
  (realized)
  (shape)
  (maps
    ((0)
      ((a self) (b b) (la l-0) (x x) (beta l) (upsilon y) (na na)
        (nb nb) (priv-stor priv-stor-1) (pt pt-5))))
  (origs (l-0 (4 1)) (pt-5 (4 1)) (pt-3 (3 1)) (lb (2 1)) (pt-1 (2 1))
    (l (1 1)) (pt-0 (1 1)) (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((2 1) (5 0)) ((3 2) (0 3)) ((3 2) (5 0)) ((4 1) (0 0))
    ((4 2) (0 3)) ((4 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (5 1))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv (cat (exp (gen) y) w))))
  (label 573)
  (parent 568)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 2) (6 0)) ((4 1) (6 0)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (4 0) (6 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 574)
  (parent 570)
  (seen 568)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (0 3)) ((3 2) (5 0)) ((4 1) (0 0)) ((4 2) (0 3))
    ((4 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization weakened ((2 1) (0 3)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv (cat (exp (gen) y) w))))
  (label 575)
  (parent 573)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 1) (5 0)) ((3 2) (0 3)) ((4 1) (0 0)) ((4 2) (0 3))
    ((4 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization weakened ((3 2) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv (cat (exp (gen) y) w))))
  (label 576)
  (parent 575)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (0 3)) ((4 1) (0 0)) ((4 2) (0 3)) ((4 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization weakened ((3 1) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv (cat (exp (gen) y) w))))
  (label 577)
  (parent 576)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (0 3)) ((4 1) (0 0)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization weakened ((4 2) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv (cat (exp (gen) y) w))))
  (label 578)
  (parent 577)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((3 2) (0 3))
    ((4 1) (0 0)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization weakened ((2 1) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv (cat (exp (gen) y) w))))
  (label 579)
  (parent 578)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((1 2) (5 0)) ((3 2) (0 3))
    ((4 1) (0 0)) ((4 1) (5 0)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization weakened ((0 2) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv (cat (exp (gen) y) w))))
  (label 580)
  (parent 579)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((1 2) (5 0)) ((3 2) (0 3))
    ((4 1) (0 0)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization weakened ((4 1) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv (cat (exp (gen) y) w))))
  (label 581)
  (parent 580)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((3 2) (0 3))
    ((4 1) (0 0)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization weakened ((1 2) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv (cat (exp (gen) y) w))))
  (label 582)
  (parent 581)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((3 2) (0 3)) ((4 1) (0 0))
    ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization weakened ((1 1) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv (cat (exp (gen) y) w))))
  (label 583)
  (parent 582)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((3 2) (0 3)) ((4 1) (0 0))
    ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b) (undisclosed l-0))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv (cat (exp (gen) y) w))))
  (label 584)
  (parent 583)
  (realized)
  (shape)
  (maps
    ((0)
      ((a self) (b b) (la l-0) (x x) (beta l) (upsilon (mul w y))
        (na na) (nb nb) (priv-stor priv-stor-1) (pt pt-5))))
  (origs (l-0 (4 1)) (pt-5 (4 1)) (pt-3 (3 1)) (lb (2 1)) (pt-1 (2 1))
    (l (1 1)) (pt-0 (1 1)) (na (0 2))))

(comment "Nothing left to do")

(defprotocol dhcr-um diffie-hellman
  (defrole init
    (vars (la x rndx) (beta upsilon expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv a la))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb))
    (uniq-orig na)
    (uniq-gen x)
    (absent (x la) (x beta))
    (assume (fact neq (exp (gen) upsilon) (gen)))
    (gen-st (pv a la)))
  (defrole resp
    (vars (lb y rndx) (alpha zeta expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv b lb))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    (uniq-orig nb)
    (uniq-gen y)
    (absent (y lb) (y alpha) (y zeta))
    (assume (fact neq (exp (gen) zeta) (gen)))
    (gen-st (pv b lb)))
  (defrole ltx-gen
    (vars (self name) (l rndx) (priv-stor locn) (ignore mesg))
    (trace (load priv-stor ignore) (stor priv-stor (pv self l))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    (uniq-orig l))
  (defrole ltx-disclose
    (vars (self name) (l rndx) (priv-stor locn))
    (trace (load priv-stor (pv self l)) (stor priv-stor "nil") (send l))
    (gen-st (pv self l)))
  (defrule undisclosed-not-disclosed
    (forall ((z strd) (l rndx))
      (implies
        (and (fact undisclosed l) (p "ltx-disclose" z 2)
          (p "ltx-disclose" "l" z l))
        (false))))
  (defgenrule neqRl_indx
    (forall ((x indx)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_strd
    (forall ((x strd)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_mesg
    (forall ((x mesg)) (implies (fact neq x x) (false))))
  (defgenrule scissorsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (leads-to z0 i0 z2 i2))
        (and (= z1 z2) (= i1 i2)))))
  (defgenrule cakeRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (leads-to z0 i0 z1 i1)
          (leads-to z0 i0 z2 i2) (prec z1 i1 z2 i2)) (false))))
  (defgenrule no-interruption
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (leads-to z0 i0 z2 i2) (trans z1 i1)
          (same-locn z0 i0 z1 i1) (prec z0 i0 z1 i1) (prec z1 i1 z2 i2))
        (false))))
  (defgenrule shearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (same-locn z0 i0 z2 i2)
          (prec z0 i0 z2 i2))
        (or (and (= z1 z2) (= i1 i2)) (prec z1 i1 z2 i2)))))
  (defgenrule invShearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (same-locn z0 i0 z1 i1)
          (leads-to z1 i1 z2 i2) (prec z0 i0 z2 i2))
        (or (and (= z0 z1) (= i0 i1)) (prec z0 i0 z1 i1)))))
  (defgenrule assume-init-0
    (forall ((z strd) (upsilon expt))
      (implies (and (p "init" z 4) (p "init" "upsilon" z upsilon))
        (fact neq (exp (gen) upsilon) (gen)))))
  (defgenrule assume-resp-0
    (forall ((z strd) (zeta expt))
      (implies (and (p "resp" z 3) (p "resp" "zeta" z zeta))
        (fact neq (exp (gen) zeta) (gen)))))
  (defgenrule trRl_ltx-gen-at-1
    (forall ((z strd)) (implies (p "ltx-gen" z 2) (trans z 1))))
  (defgenrule trRl_ltx-gen-at-0
    (forall ((z strd)) (implies (p "ltx-gen" z 2) (trans z 0))))
  (defgenrule trRl_ltx-disclose-at-1
    (forall ((z strd)) (implies (p "ltx-disclose" z 2) (trans z 1))))
  (defgenrule trRl_ltx-disclose-at-0
    (forall ((z strd)) (implies (p "ltx-disclose" z 2) (trans z 0))))
  (defgenrule gen-st-init-0
    (forall ((z strd) (la rndx) (a name))
      (implies
        (and (p "init" z 1) (p "init" "la" z la) (p "init" "a" z a))
        (gen-st (pv a la)))))
  (defgenrule gen-st-resp-0
    (forall ((z strd) (lb rndx) (b name))
      (implies
        (and (p "resp" z 1) (p "resp" "lb" z lb) (p "resp" "b" z b))
        (gen-st (pv b lb)))))
  (defgenrule gen-st-ltx-disclose-0
    (forall ((z strd) (l rndx) (self name))
      (implies
        (and (p "ltx-disclose" z 1) (p "ltx-disclose" "l" z l)
          (p "ltx-disclose" "self" z self)) (gen-st (pv self l)))))
  (lang (sig sign) (body (tuple 3)) (pv (tuple 2))))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (la x rndx)
    (beta upsilon expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta beta) (upsilon upsilon))
  (non-orig (privk "sig" b))
  (uniq-orig na)
  (uniq-gen x)
  (absent (x la) (x beta))
  (facts (neq a b))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb)))
  (label 585)
  (unrealized (0 1))
  (origs (na (0 2)))
  (comment "Not closed under rules"))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (la x rndx)
    (beta upsilon expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta beta) (upsilon upsilon))
  (non-orig (privk "sig" b))
  (uniq-orig na)
  (uniq-gen x)
  (absent (x la) (x beta))
  (gen-st (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb)))
  (label 586)
  (parent 585)
  (unrealized (0 0) (0 1))
  (origs (na (0 2)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (a b name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (beta upsilon expt) (l rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (precedes ((1 1) (0 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x beta) (x l))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv a l))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul beta l))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l)))))
  (label 587)
  (parent 586)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((2 2) (0 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))))
  (label 588)
  (parent 587)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx)
    (zeta expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y zeta))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b))
  (operation encryption-test (added-strand resp 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 589)
  (parent 588)
  (unrealized (0 3) (3 0) (3 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation encryption-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send
        (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 590)
  (parent 588)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b))
  (operation channel-test (displaced 4 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 591)
  (parent 589)
  (unrealized (0 3) (3 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1)))))
  (label 592)
  (parent 589)
  (unrealized (0 3) (3 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 593)
  (parent 590)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (neq self b))
  (operation nonce-test (displaced 1 4 ltx-gen 3) (exp (gen) l-0) (3 1))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 594)
  (parent 591)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 595)
  (parent 591)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l-0)) l-1))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1))))
  (label 596)
  (parent 592)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec l-1))) l-0))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0))))
  (label 597)
  (parent 592)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec l-1))) l))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l))))
  (label 598)
  (parent 592)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 599)
  (parent 593)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 600)
  (parent 593)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (neq self b))
  (operation nonce-test (displaced 4 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 601)
  (parent 594)
  (realized)
  (shape)
  (maps
    ((0)
      ((a self) (b b) (la l) (x x) (beta lb) (upsilon y) (na na) (nb nb)
        (priv-stor priv-stor-0) (pt pt-2))))
  (origs (nb (2 3)) (l (3 1)) (pt-2 (3 1)) (lb (1 1)) (pt-0 (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (neq self b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 602)
  (parent 594)
  (unrealized (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 603)
  (parent 595)
  (unrealized (0 3) (3 1) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b))
  (operation nonce-test (displaced 5 0 init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 604)
  (parent 595)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 605)
  (parent 595)
  (unrealized (0 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (neq self b))
  (operation nonce-test (displaced 1 5 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 606)
  (parent 595)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 607)
  (parent 595)
  (unrealized (0 3) (3 1) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b))
  (operation nonce-test (displaced 5 2 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 608)
  (parent 595)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 609)
  (parent 595)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 610)
  (parent 596)
  (unrealized (0 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1)))
  (label 611)
  (parent 597)
  (unrealized (0 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1)))
  (label 612)
  (parent 598)
  (unrealized (0 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv b l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0)))
  (label 613)
  (parent 599)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 614)
  (parent 600)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (lb l x rndx)
    (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (neq self b))
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 615)
  (parent 602)
  (seen 594)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 616)
  (parent 602)
  (unrealized (0 3) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 617)
  (parent 602)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (la l y x rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la la) (x x) (beta l)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha la) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb la l)
  (uniq-gen y x)
  (absent (y (one)) (y la) (y l) (x la) (x l))
  (precur (4 0))
  (gen-st (pv b l) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b))
  (operation nonce-test (displaced 5 0 init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la l)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 618)
  (parent 602)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b))
  (operation nonce-test (displaced 5 1 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 619)
  (parent 602)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (neq self b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 620)
  (parent 602)
  (unrealized (0 3) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b))
  (operation nonce-test (displaced 5 3 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 621)
  (parent 602)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 3 1) (trans 1 1) (trans 5 0) (trans 3 0)
    (trans 1 0) (neq self b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 622)
  (parent 602)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 623)
  (parent 603)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta beta expt) (l x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b))
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 624)
  (parent 603)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a-0 l-1)))))
  (label 625)
  (parent 603)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (4 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 626)
  (parent 605)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (3 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (neq self b))
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 627)
  (parent 606)
  (seen 601)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (neq self b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 628)
  (parent 606)
  (unrealized (5 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 629)
  (parent 607)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b))
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 630)
  (parent 607)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 631)
  (parent 607)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 632)
  (parent 610)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 633)
  (parent 610)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (7 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 634)
  (parent 611)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 635)
  (parent 611)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (7 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 636)
  (parent 612)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 637)
  (parent 612)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((6 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 6 1) (trans 1 1)
    (trans 6 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 1 7 ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 638)
  (parent 613)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 639)
  (parent 613)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 7 2 ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 640)
  (parent 614)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 641)
  (parent 614)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (4 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3))
    ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 642)
  (parent 616)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (0 3))
    ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self b))
  (operation channel-test (displaced 6 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 643)
  (parent 616)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 2) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 3 1) (trans 1 1) (trans 6 0) (trans 3 0)
    (trans 1 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 644)
  (parent 616)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (4 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3))
    ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (neq self b))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 645)
  (parent 620)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (0 3))
    ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (neq self b))
  (operation channel-test (displaced 6 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 646)
  (parent 620)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 3) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-1))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 3 1) (trans 1 1) (trans 6 0) (trans 3 0) (trans 1 0)
    (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 647)
  (parent 620)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0)
    (neq a b))
  (operation nonce-test (displaced 6 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 648)
  (parent 626)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 649)
  (parent 626)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (lb l x rndx)
    (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (3 0) (5 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (neq self b))
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 650)
  (parent 628)
  (seen 615)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 651)
  (parent 628)
  (unrealized (0 3) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 652)
  (parent 628)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (la l y x rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la la) (x x) (beta l)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha la) (zeta (one)))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb la l)
  (uniq-gen y x)
  (absent (y (one)) (y la) (y l) (x la) (x l))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b))
  (operation nonce-test (displaced 6 0 init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la l)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 653)
  (parent 628)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b))
  (operation nonce-test (displaced 6 1 ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 654)
  (parent 628)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 3) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (neq self b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 655)
  (parent 628)
  (unrealized (0 3) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b))
  (operation nonce-test (displaced 6 4 ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 656)
  (parent 628)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 4 1) (trans 1 1) (trans 6 0) (trans 4 0)
    (trans 1 0) (neq self b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 657)
  (parent 628)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1)))
  (label 658)
  (parent 632)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 659)
  (parent 633)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 660)
  (parent 634)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 661)
  (parent 635)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 662)
  (parent 636)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1)))
  (label 663)
  (parent 637)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((5 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 5 1) (trans 1 1)
    (trans 5 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 664)
  (parent 638)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 665)
  (parent 639)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (upsilon expt) (l l-0 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 8 0 init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul upsilon x)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 666)
  (parent 639)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 667)
  (parent 639)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 7 1) (trans 1 1)
    (trans 7 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 1 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 668)
  (parent 639)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l lb rndx) (alpha zeta expt) (l-0 y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y lb) (y alpha) (y zeta))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 669)
  (parent 639)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 8 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 670)
  (parent 639)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 8 1) (trans 2 1)
    (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 671)
  (parent 639)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 672)
  (parent 640)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 la))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 673)
  (parent 641)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (upsilon expt) (la l x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) l) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na la l)
  (uniq-gen x)
  (absent (x la) (x l))
  (precur (7 0))
  (gen-st (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 8 0 init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) l) la)) (send (cat (exp (gen) l) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-3 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 674)
  (parent 641)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 675)
  (parent 641)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 1) (2 0))
    ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 1) (5 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 7 1) (trans 1 1)
    (trans 7 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 1 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 676)
  (parent 641)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l lb rndx) (alpha zeta expt) (l-0 y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y lb) (y alpha) (y zeta))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 677)
  (parent 641)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 8 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 678)
  (parent 641)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 8 1) (trans 2 1)
    (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 679)
  (parent 641)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (3 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 680)
  (parent 648)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq a b))
  (operation nonce-test (displaced 7 3 resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 681)
  (parent 649)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 682)
  (parent 649)
  (unrealized (0 3) (6 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 683)
  (parent 649)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (la l y x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha la) (zeta (one)))
  (deflistener (cat (gen) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb la l)
  (uniq-gen y x)
  (absent (y (one)) (y la) (y l) (x la) (x l))
  (precur (4 0) (6 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b))
  (operation nonce-test (displaced 7 0 init 3) (exp (gen) x-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul la l)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((recv (cat (gen) la)) (send (cat (gen) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-3 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 684)
  (parent 649)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (0 0))
    ((6 1) (4 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 1 1) (trans 6 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq self b))
  (operation nonce-test (displaced 1 7 ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 685)
  (parent 649)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)) ((7 3) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0)
    (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 686)
  (parent 649)
  (unrealized (0 3) (6 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b))
  (operation nonce-test (displaced 7 2 ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 687)
  (parent 649)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 7 1) (trans 2 1) (trans 1 1) (trans 7 0) (trans 2 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 688)
  (parent 649)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b))
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 689)
  (parent 651)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b))
  (operation channel-test (displaced 7 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 690)
  (parent 651)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 7 1) (trans 4 1) (trans 1 1) (trans 7 0) (trans 4 0)
    (trans 1 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 691)
  (parent 651)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (0 3)) ((6 3) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (neq self b))
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 692)
  (parent 655)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (0 3)) ((6 3) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (neq self b))
  (operation channel-test (displaced 7 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 693)
  (parent 655)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 3) (5 0)) ((7 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-1))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 7 1)
    (trans 4 1) (trans 1 1) (trans 7 0) (trans 4 0) (trans 1 0)
    (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 694)
  (parent 655)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 8 1) (trans 3 1) (trans 1 1)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 695)
  (parent 658)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 696)
  (parent 658)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 697)
  (parent 659)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 698)
  (parent 659)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 8 1) (trans 3 1) (trans 1 1)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 699)
  (parent 660)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 700)
  (parent 660)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec l) (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec l) (rec w))) w))
    (exp (gen) (rec l)) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec l) (rec w))) w))
      (send (cat (exp (gen) (mul (rec l) (rec w))) w))))
  (label 701)
  (parent 661)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0)))
  (label 702)
  (parent 662)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 703)
  (parent 662)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec l) (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec l) (rec w))) w))
    (exp (gen) (rec l)) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec l) (rec w))) w))
      (send (cat (exp (gen) (mul (rec l) (rec w))) w))))
  (label 704)
  (parent 663)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (2 0))
    ((4 1) (0 0)) ((4 2) (2 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 705)
  (parent 664)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 706)
  (parent 665)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 707)
  (parent 665)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 9 1) (trans 2 1)
    (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 708)
  (parent 665)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 709)
  (parent 667)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (0 0))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 6 1) (trans 1 1)
    (trans 6 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 710)
  (parent 668)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (alpha zeta expt) (y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec y) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 711)
  (parent 669)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon alpha zeta expt) (l y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 712)
  (parent 669)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (alpha zeta expt) (l-0 y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-1))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 713)
  (parent 669)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (3 0))
    ((3 1) (0 3)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 714)
  (parent 672)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 715)
  (parent 673)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 716)
  (parent 673)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 9 1) (trans 2 1)
    (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 717)
  (parent 673)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv b l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 718)
  (parent 675)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon alpha zeta expt) (l y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 719)
  (parent 677)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (alpha zeta expt) (y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec y) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 720)
  (parent 677)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (alpha zeta expt) (l-0 y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 721)
  (parent 677)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 722)
  (parent 678)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 3 1) (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 723)
  (parent 680)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (3 0) (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 724)
  (parent 681)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (0 3)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b))
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 725)
  (parent 682)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (0 3)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b))
  (operation channel-test (displaced 8 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 726)
  (parent 682)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 2 1) (trans 1 1) (trans 8 0) (trans 2 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 727)
  (parent 682)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((2 1) (3 0)) ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (0 3)) ((7 3) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0)
    (neq a b))
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 728)
  (parent 686)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (0 3)) ((7 3) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0)
    (neq a b))
  (operation channel-test (displaced 8 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 729)
  (parent 686)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)) ((7 3) (6 0)) ((8 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-1))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 2 1) (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0)
    (trans 5 1) (trans 5 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 730)
  (parent 686)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 9 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 731)
  (parent 695)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 8 1) (trans 3 1) (trans 1 1)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 732)
  (parent 695)
  (unrealized (9 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 733)
  (parent 696)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 734)
  (parent 696)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 735)
  (parent 696)
  (unrealized (0 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) l-0 l-1)) (y l) (x l-0) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 736)
  (parent 696)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 737)
  (parent 696)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 3 1) (trans 1 1)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 738)
  (parent 696)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (5 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 739)
  (parent 696)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 10 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 740)
  (parent 696)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 9 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 741)
  (parent 697)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 742)
  (parent 697)
  (unrealized (9 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 743)
  (parent 698)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 744)
  (parent 698)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 745)
  (parent 698)
  (unrealized (0 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 746)
  (parent 698)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 747)
  (parent 698)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((2 3) (0 3)) ((3 1) (2 0))
    ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 1) (7 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 3 1) (trans 1 1)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 748)
  (parent 698)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (5 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 749)
  (parent 698)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 10 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 750)
  (parent 698)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 9 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 751)
  (parent 699)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 8 1) (trans 3 1) (trans 1 1)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 752)
  (parent 699)
  (unrealized (9 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 753)
  (parent 700)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 754)
  (parent 700)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 755)
  (parent 700)
  (unrealized (0 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) l-0 l-1)) (y l) (x l-0) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 756)
  (parent 700)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 757)
  (parent 700)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 3 1) (trans 1 1)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 758)
  (parent 700)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (7 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 759)
  (parent 700)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 10 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 760)
  (parent 700)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 761)
  (parent 701)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 762)
  (parent 701)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w (rec l-1))) (gen)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1)))))
  (label 763)
  (parent 701)
  (unrealized (0 3) (7 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul (rec l) (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) la))
      (send (cat (exp (gen) (rec l)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul (rec l) (rec x))))
      (send (cat (exp (gen) x) (mul (rec l) (rec x))))))
  (label 764)
  (parent 701)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))))
  (label 765)
  (parent 701)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 1) (7 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 3 1) (trans 1 1)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 766)
  (parent 701)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 767)
  (parent 701)
  (unrealized (0 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 10 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 768)
  (parent 701)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 9 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 769)
  (parent 702)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 770)
  (parent 702)
  (unrealized (9 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 771)
  (parent 703)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 772)
  (parent 703)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 773)
  (parent 703)
  (unrealized (0 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 774)
  (parent 703)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 775)
  (parent 703)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((2 3) (0 3)) ((3 1) (2 0))
    ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 1) (5 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 3 1) (trans 1 1)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 776)
  (parent 703)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (7 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 777)
  (parent 703)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 10 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 778)
  (parent 703)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 779)
  (parent 704)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 780)
  (parent 704)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w (rec l-1))) (gen)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1)))))
  (label 781)
  (parent 704)
  (unrealized (0 3) (7 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul (rec l) (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul (rec l) (rec x))))
      (send (cat (exp (gen) x) (mul (rec l) (rec x))))))
  (label 782)
  (parent 704)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))))
  (label 783)
  (parent 704)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 1) (5 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 3 1) (trans 1 1)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 784)
  (parent 704)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 785)
  (parent 704)
  (unrealized (0 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 10 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 786)
  (parent 704)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)) ((3 1) (0 0))
    ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 787)
  (parent 705)
  (realized)
  (shape)
  (maps
    ((0)
      ((a self) (b b) (la l-0) (x x) (beta l) (upsilon upsilon) (na na)
        (nb nb) (priv-stor priv-stor-0) (pt pt-3))))
  (origs (l-0 (3 1)) (pt-3 (3 1)) (pt-1 (2 1)) (l (1 1)) (pt-0 (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0)
    (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 788)
  (parent 709)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (2 0))
    ((4 1) (2 0)) ((5 1) (0 0)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 5 1) (trans 1 1)
    (trans 5 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 789)
  (parent 710)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 790)
  (parent 714)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b) (la l) (x x) (beta l-0) (upsilon upsilon) (na na)
        (nb nb) (priv-stor priv-stor) (pt pt))))
  (origs (l-0 (2 1)) (pt-2 (2 1)) (pt-3 (3 1)) (l (1 1)) (pt (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0)
    (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 791)
  (parent 718)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 792)
  (parent 722)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 3 1) (trans 3 0) (neq a b))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 793)
  (parent 723)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b) (la l) (x x) (beta lb) (upsilon y) (na na) (nb nb)
        (priv-stor priv-stor) (pt pt))))
  (origs (pt-3 (3 1)) (lb (2 1)) (pt-2 (2 1)) (l (1 1)) (pt (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 2) (4 0)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 794)
  (parent 724)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (4 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (0 0))
    ((7 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 7 1) (trans 2 1) (trans 1 1)
    (trans 7 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 795)
  (parent 731)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 8 1) (trans 3 1)
    (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 796)
  (parent 732)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 797)
  (parent 732)
  (unrealized (0 3) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 798)
  (parent 732)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) la) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) la l-0)) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) la) l-0)) (send (cat (exp (gen) la) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 799)
  (parent 732)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 800)
  (parent 732)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 801)
  (parent 732)
  (unrealized (0 3) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (0 0)) ((7 2) (5 0)) ((8 1) (0 3)) ((9 1) (2 0))
    ((9 1) (4 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self-0))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv self l-0) (pv self-0 l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 7 1) (trans 1 1) (trans 9 0) (trans 7 0)
    (trans 1 0) (trans 6 1) (trans 4 1) (trans 6 0) (trans 4 0)
    (neq self self-0))
  (operation nonce-test (displaced 3 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-0 (cat pt-4 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-2 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 802)
  (parent 732)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (displaced 10 8 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 803)
  (parent 732)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 8 1) (trans 3 1) (trans 1 1) (trans 10 0)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 804)
  (parent 732)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 805)
  (parent 733)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 806)
  (parent 733)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 807)
  (parent 733)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 808)
  (parent 733)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 809)
  (parent 734)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 810)
  (parent 734)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 811)
  (parent 734)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 812)
  (parent 734)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1)))
  (label 813)
  (parent 735)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 814)
  (parent 738)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 3 1) (trans 1 1)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 815)
  (parent 738)
  (unrealized (10 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1)) ((2 2) (6 0))
    ((3 1) (5 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 816)
  (parent 741)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 817)
  (parent 742)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 818)
  (parent 742)
  (unrealized (0 3) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 819)
  (parent 742)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 820)
  (parent 742)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (8 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 1) (0 0))
    ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 821)
  (parent 742)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 822)
  (parent 742)
  (unrealized (0 3) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 823)
  (parent 742)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 3) (8 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 1) (3 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 824)
  (parent 742)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 10 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 825)
  (parent 742)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 826)
  (parent 743)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 827)
  (parent 743)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 828)
  (parent 743)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 829)
  (parent 743)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 830)
  (parent 744)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 831)
  (parent 744)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 832)
  (parent 744)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 833)
  (parent 744)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1)))
  (label 834)
  (parent 745)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 835)
  (parent 747)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 836)
  (parent 747)
  (unrealized (10 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (0 0))
    ((7 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 7 1) (trans 2 1) (trans 1 1)
    (trans 7 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 837)
  (parent 751)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 8 1) (trans 3 1)
    (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 838)
  (parent 752)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 839)
  (parent 752)
  (unrealized (0 3) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 840)
  (parent 752)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) la) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) la l-0)) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) la) l)) (send (cat (exp (gen) la) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 841)
  (parent 752)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 842)
  (parent 752)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 843)
  (parent 752)
  (unrealized (0 3) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (0 0)) ((7 2) (5 0)) ((8 1) (0 3)) ((9 1) (2 0))
    ((9 1) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self-0))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv self l-0) (pv self-0 l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 7 1) (trans 1 1) (trans 9 0) (trans 7 0)
    (trans 1 0) (trans 6 1) (trans 4 1) (trans 6 0) (trans 4 0)
    (neq self self-0))
  (operation nonce-test (displaced 3 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-0 (cat pt-4 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-2 "nil")) (send l-1))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 844)
  (parent 752)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (displaced 10 8 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 845)
  (parent 752)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 8 1) (trans 3 1) (trans 1 1) (trans 10 0)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 846)
  (parent 752)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 847)
  (parent 753)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 848)
  (parent 753)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 849)
  (parent 753)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 850)
  (parent 753)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 851)
  (parent 754)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 852)
  (parent 754)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 853)
  (parent 754)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 854)
  (parent 754)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1)))
  (label 855)
  (parent 755)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 856)
  (parent 758)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 3 1) (trans 1 1)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 857)
  (parent 758)
  (unrealized (10 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 858)
  (parent 761)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 1) (6 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 859)
  (parent 761)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
      (send (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 860)
  (parent 761)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 861)
  (parent 761)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 862)
  (parent 762)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 1) (6 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 863)
  (parent 762)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
      (send (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 864)
  (parent 762)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 865)
  (parent 762)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-listener l-1) (rec l-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1)))
  (label 866)
  (parent 763)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (10 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (added-listener l-1) (mul (rec l-1) (rec l-1))
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1)))
  (label 867)
  (parent 767)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1)) ((2 2) (6 0))
    ((3 1) (7 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 868)
  (parent 769)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 869)
  (parent 770)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 870)
  (parent 770)
  (unrealized (0 3) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 871)
  (parent 770)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 872)
  (parent 770)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (8 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 873)
  (parent 770)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 874)
  (parent 770)
  (unrealized (0 3) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 875)
  (parent 770)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 3) (8 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 1) (3 0))
    ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 876)
  (parent 770)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 10 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 877)
  (parent 770)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 878)
  (parent 771)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 879)
  (parent 771)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 880)
  (parent 771)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 881)
  (parent 771)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 882)
  (parent 772)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 883)
  (parent 772)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 884)
  (parent 772)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 885)
  (parent 772)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1)))
  (label 886)
  (parent 773)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 887)
  (parent 775)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 888)
  (parent 775)
  (unrealized (10 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 889)
  (parent 779)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 890)
  (parent 779)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
      (send (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 891)
  (parent 779)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 892)
  (parent 779)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 893)
  (parent 780)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 894)
  (parent 780)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
      (send (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 895)
  (parent 780)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 896)
  (parent 780)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-listener l-1) (rec l-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1)))
  (label 897)
  (parent 781)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (10 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (added-listener l-1) (mul (rec l-1) (rec l-1))
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1)))
  (label 898)
  (parent 785)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1) (trans 6 0)
    (trans 4 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 899)
  (parent 788)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)) ((3 1) (0 3))
    ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 900)
  (parent 789)
  (seen 787)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1) (trans 6 0)
    (trans 4 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 901)
  (parent 791)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 2) (0 3)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 902)
  (parent 792)
  (seen 790)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 3)) ((3 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b))
  (operation generalization deleted (4 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 903)
  (parent 794)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (3 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (0 0)) ((6 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1)
    (trans 5 0) (trans 3 0) (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 904)
  (parent 795)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 1) (8 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (0 0)) ((7 2) (5 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 7 1) (trans 2 1)
    (trans 1 1) (trans 7 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 905)
  (parent 796)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((3 1) (10 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation channel-test (displaced 11 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 906)
  (parent 797)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 907)
  (parent 797)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (10 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation channel-test (displaced 11 8 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 908)
  (parent 797)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv a l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 8 1) (trans 3 1) (trans 1 1) (trans 11 0)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a l-2)))))
  (label 909)
  (parent 797)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((3 1) (10 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation channel-test (displaced 11 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 910)
  (parent 801)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 911)
  (parent 801)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (10 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation channel-test (displaced 11 8 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 912)
  (parent 801)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 11 0) (trans 8 0)
    (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0)
    (trans 5 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 913)
  (parent 801)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 914)
  (parent 813)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 915)
  (parent 813)
  (unrealized (11 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (4 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 8 1) (trans 2 1) (trans 1 1)
    (trans 8 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 916)
  (parent 814)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 9 1) (trans 3 1)
    (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 11 2 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 917)
  (parent 815)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 918)
  (parent 815)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 919)
  (parent 815)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) la) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) la l-0)) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) la) l-0)) (send (cat (exp (gen) la) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 920)
  (parent 815)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 921)
  (parent 815)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 922)
  (parent 815)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)) ((9 1) (0 3))
    ((10 1) (2 0)) ((10 1) (4 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" self-0))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv self l-0) (pv self-0 l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 8 1) (trans 1 1) (trans 10 0) (trans 8 0)
    (trans 1 0) (trans 6 1) (trans 4 1) (trans 6 0) (trans 4 0)
    (neq self self-0))
  (operation nonce-test (displaced 3 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-0 (cat pt-4 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 923)
  (parent 815)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (displaced 11 9 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 924)
  (parent 815)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 9 1) (trans 3 1) (trans 1 1) (trans 11 0)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 925)
  (parent 815)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (4 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 926)
  (parent 816)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (5 0)) ((4 1) (8 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 927)
  (parent 817)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 928)
  (parent 818)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 929)
  (parent 818)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 930)
  (parent 818)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 931)
  (parent 818)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 932)
  (parent 822)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 933)
  (parent 822)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 934)
  (parent 822)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 935)
  (parent 822)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 936)
  (parent 834)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 937)
  (parent 834)
  (unrealized (11 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1)) ((2 2) (8 0))
    ((3 1) (5 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 938)
  (parent 835)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 939)
  (parent 836)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 940)
  (parent 836)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 941)
  (parent 836)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 942)
  (parent 836)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3))
    ((10 1) (0 0)) ((10 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (9 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 3 1) (trans 1 1) (trans 10 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (displaced 1 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 943)
  (parent 836)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 944)
  (parent 836)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 11 2 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 945)
  (parent 836)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3))
    ((10 1) (3 0)) ((10 1) (5 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self))
  (operation nonce-test (displaced 4 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 946)
  (parent 836)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 947)
  (parent 836)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (0 0)) ((6 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1)
    (trans 5 0) (trans 3 0) (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 948)
  (parent 837)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 1) (8 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (0 0)) ((7 2) (5 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 7 1) (trans 2 1)
    (trans 1 1) (trans 7 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 949)
  (parent 838)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((3 1) (10 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation channel-test (displaced 11 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 950)
  (parent 839)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 951)
  (parent 839)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (10 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation channel-test (displaced 11 8 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 952)
  (parent 839)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv a l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 8 1) (trans 3 1) (trans 1 1) (trans 11 0)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a l-2)))))
  (label 953)
  (parent 839)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((3 1) (10 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation channel-test (displaced 11 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 954)
  (parent 843)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 955)
  (parent 843)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (10 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation channel-test (displaced 11 8 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 956)
  (parent 843)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 11 0) (trans 8 0)
    (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0)
    (trans 5 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 957)
  (parent 843)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 958)
  (parent 855)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 959)
  (parent 855)
  (unrealized (11 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 8 1) (trans 2 1) (trans 1 1)
    (trans 8 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 960)
  (parent 856)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 9 1) (trans 3 1)
    (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 11 2 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 961)
  (parent 857)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 962)
  (parent 857)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 963)
  (parent 857)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) la) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) la l-0)) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) la) l)) (send (cat (exp (gen) la) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 964)
  (parent 857)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 965)
  (parent 857)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 966)
  (parent 857)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)) ((9 1) (0 3))
    ((10 1) (2 0)) ((10 1) (6 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" self-0))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv self l-0) (pv self-0 l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 8 1) (trans 1 1) (trans 10 0) (trans 8 0)
    (trans 1 0) (trans 6 1) (trans 4 1) (trans 6 0) (trans 4 0)
    (neq self self-0))
  (operation nonce-test (displaced 3 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-0 (cat pt-4 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-2 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 967)
  (parent 857)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (displaced 11 9 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 968)
  (parent 857)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 9 1) (trans 3 1) (trans 1 1) (trans 11 0)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 969)
  (parent 857)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1)
    (trans 6 1) (trans 11 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 970)
  (parent 866)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 7 1)
    (trans 5 1) (trans 11 0) (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-1 (cat pt-6 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 971)
  (parent 867)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 972)
  (parent 868)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (7 0)) ((4 1) (8 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 973)
  (parent 869)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 974)
  (parent 870)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 975)
  (parent 870)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 976)
  (parent 870)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 977)
  (parent 870)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 978)
  (parent 874)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 979)
  (parent 874)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 980)
  (parent 874)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 981)
  (parent 874)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 982)
  (parent 886)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 983)
  (parent 886)
  (unrealized (11 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1)) ((2 2) (8 0))
    ((3 1) (7 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 984)
  (parent 887)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 985)
  (parent 888)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 986)
  (parent 888)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 987)
  (parent 888)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 988)
  (parent 888)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3))
    ((10 1) (0 0)) ((10 1) (5 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (9 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 3 1) (trans 1 1) (trans 10 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (displaced 1 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 989)
  (parent 888)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 990)
  (parent 888)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 11 2 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 991)
  (parent 888)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3))
    ((10 1) (3 0)) ((10 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self))
  (operation nonce-test (displaced 4 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 992)
  (parent 888)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 993)
  (parent 888)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1)
    (trans 6 1) (trans 11 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 994)
  (parent 897)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 7 1)
    (trans 5 1) (trans 11 0) (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-1 (cat pt-6 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 995)
  (parent 898)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1) (trans 5 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 996)
  (parent 899)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1) (trans 5 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 997)
  (parent 901)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 1) (4 0)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b))
  (operation generalization weakened ((3 2) (4 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 998)
  (parent 903)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (0 0)) ((5 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 999)
  (parent 904)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (3 0))
    ((3 2) (7 0)) ((4 1) (7 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((6 2) (4 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1000)
  (parent 905)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 5 1) (trans 9 0) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 1001)
  (parent 914)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1002)
  (parent 915)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1003)
  (parent 915)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 1004)
  (parent 915)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) la) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a la) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 12 0 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) la) l-0)) (send (cat (exp (gen) la) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) la)) (send (cat (gen) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-7 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 1005)
  (parent 915)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)) ((11 1) (0 0)) ((11 1) (9 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 3 1) (trans 1 1) (trans 11 0) (trans 3 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 5 1) (trans 9 0)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 1 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1006)
  (parent 915)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 1007)
  (parent 915)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 12 2 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 1008)
  (parent 915)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)) ((11 1) (3 0)) ((11 1) (5 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (10 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 5 1) (trans 9 0)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (displaced 4 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1009)
  (parent 915)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1010)
  (parent 915)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (3 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 7 1) (trans 2 1) (trans 1 1)
    (trans 7 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1)
    (trans 5 0) (trans 3 0) (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1011)
  (parent 916)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 1) (9 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (9 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 8 1) (trans 2 1)
    (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1012)
  (parent 917)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((3 1) (11 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation channel-test (displaced 12 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1013)
  (parent 918)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 1) (11 0)) ((1 2) (0 1))
    ((2 3) (10 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1014)
  (parent 918)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation channel-test (displaced 12 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1015)
  (parent 918)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv a l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 9 1) (trans 3 1) (trans 1 1) (trans 12 0)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a l-2)))))
  (label 1016)
  (parent 918)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((3 1) (11 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation channel-test (displaced 12 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1017)
  (parent 922)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 1) (11 0)) ((1 2) (0 1))
    ((2 3) (10 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1018)
  (parent 922)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation channel-test (displaced 12 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1019)
  (parent 922)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 12 1)
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 12 0) (trans 9 0)
    (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0)
    (trans 5 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 1020)
  (parent 922)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 1) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l)))
  (label 1021)
  (parent 926)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (4 0)) ((4 2) (7 0)) ((5 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1022)
  (parent 927)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (9 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 5 1) (trans 9 0) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 1023)
  (parent 936)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1024)
  (parent 937)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1025)
  (parent 937)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 1026)
  (parent 937)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a la) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 12 0 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 1027)
  (parent 937)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (9 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)) ((11 1) (0 0)) ((11 1) (7 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 3 1) (trans 1 1) (trans 11 0) (trans 3 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 5 1) (trans 9 0)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 1 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1028)
  (parent 937)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 1029)
  (parent 937)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 12 2 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 1030)
  (parent 937)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)) ((11 1) (3 0)) ((11 1) (5 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (10 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 5 1) (trans 9 0)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (displaced 4 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1031)
  (parent 937)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1032)
  (parent 937)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1)) ((2 2) (7 0))
    ((3 1) (4 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1033)
  (parent 938)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 1) (5 0)) ((4 1) (9 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1034)
  (parent 939)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (11 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1035)
  (parent 940)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (11 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1036)
  (parent 940)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1037)
  (parent 940)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 1038)
  (parent 940)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (11 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1039)
  (parent 944)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (11 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1040)
  (parent 944)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1041)
  (parent 944)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 12 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 12 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 1042)
  (parent 944)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0)) ((3 2) (0 3))
    ((4 2) (0 3)) ((5 1) (0 0)) ((5 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1)
    (trans 4 0) (trans 3 0) (neq self b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1043)
  (parent 948)
  (seen 1098)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (7 0)) ((4 1) (7 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((6 2) (4 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1044)
  (parent 949)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (7 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 5 1) (trans 9 0) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 1045)
  (parent 958)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1046)
  (parent 959)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1047)
  (parent 959)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 1048)
  (parent 959)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) la) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a la) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 12 0 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) la) l)) (send (cat (exp (gen) la) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) la)) (send (cat (gen) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-7 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 1049)
  (parent 959)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)) ((11 1) (0 0)) ((11 1) (9 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 3 1) (trans 1 1) (trans 11 0) (trans 3 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 5 1) (trans 9 0)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 1 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1050)
  (parent 959)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 1051)
  (parent 959)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 12 2 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 1052)
  (parent 959)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)) ((11 1) (3 0)) ((11 1) (7 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (10 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 5 1) (trans 9 0)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (displaced 4 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1053)
  (parent 959)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1054)
  (parent 959)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 7 1) (trans 2 1) (trans 1 1)
    (trans 7 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1)
    (trans 5 0) (trans 3 0) (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1055)
  (parent 960)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 1) (9 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (9 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 8 1) (trans 2 1)
    (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1056)
  (parent 961)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((3 1) (11 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation channel-test (displaced 12 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1057)
  (parent 962)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 1) (11 0)) ((1 2) (0 1))
    ((2 3) (10 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1058)
  (parent 962)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation channel-test (displaced 12 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1059)
  (parent 962)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv a l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 9 1) (trans 3 1) (trans 1 1) (trans 12 0)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a l-2)))))
  (label 1060)
  (parent 962)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((3 1) (11 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation channel-test (displaced 12 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1061)
  (parent 966)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 1) (11 0)) ((1 2) (0 1))
    ((2 3) (10 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1062)
  (parent 966)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation channel-test (displaced 12 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1063)
  (parent 966)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 12 1)
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 12 0) (trans 9 0)
    (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0)
    (trans 5 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 1064)
  (parent 966)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 1065)
  (parent 970)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1)
    (trans 6 1) (trans 11 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1066)
  (parent 970)
  (unrealized (12 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 9 1)
    (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0) (trans 7 0)
    (trans 5 0) (neq a self))
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 1067)
  (parent 971)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 7 1)
    (trans 5 1) (trans 11 0) (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1068)
  (parent 971)
  (unrealized (12 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (5 0))
    ((4 2) (0 3)) ((5 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 4 1)
    (trans 5 0) (trans 4 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 1069)
  (parent 972)
  (seen 1112)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (6 0)) ((4 2) (7 0)) ((5 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1070)
  (parent 973)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (9 0)) ((2 2) (0 1))
    ((3 1) (7 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 5 1) (trans 9 0) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 1071)
  (parent 982)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1072)
  (parent 983)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1073)
  (parent 983)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 1074)
  (parent 983)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a la) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 12 0 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 1075)
  (parent 983)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (9 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)) ((11 1) (0 0)) ((11 1) (5 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 3 1) (trans 1 1) (trans 11 0) (trans 3 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 5 1) (trans 9 0)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 1 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1076)
  (parent 983)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 1077)
  (parent 983)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 12 2 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 1078)
  (parent 983)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)) ((11 1) (3 0)) ((11 1) (7 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (10 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 5 1) (trans 9 0)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (displaced 4 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1079)
  (parent 983)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1080)
  (parent 983)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (7 0))
    ((3 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1081)
  (parent 984)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 1) (7 0)) ((4 1) (9 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1082)
  (parent 985)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (11 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1083)
  (parent 986)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (11 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1084)
  (parent 986)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1085)
  (parent 986)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 1086)
  (parent 986)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (11 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1087)
  (parent 990)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (11 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1088)
  (parent 990)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1089)
  (parent 990)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 12 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 12 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 1090)
  (parent 990)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 1091)
  (parent 994)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1)
    (trans 6 1) (trans 11 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1092)
  (parent 994)
  (unrealized (12 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-0) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 9 1)
    (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0) (trans 7 0)
    (trans 5 0) (neq a self))
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 1093)
  (parent 995)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 7 1)
    (trans 5 1) (trans 11 0) (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1094)
  (parent 995)
  (unrealized (12 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 1095)
  (parent 996)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (4 0)) ((2 2) (0 1))
    ((3 2) (0 3)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1) (trans 4 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 1096)
  (parent 997)
  (seen 1192)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b))
  (operation generalization weakened ((3 1) (4 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 1097)
  (parent 998)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (0 3))
    ((4 1) (0 0)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0)
    (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1098)
  (parent 999)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 1) (6 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((5 2) (3 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1099)
  (parent 1000)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (4 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 4 1) (trans 8 0) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 1100)
  (parent 1001)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 1) (5 0)) ((4 1) (10 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 7 1) (trans 5 1) (trans 9 0) (trans 7 0) (trans 5 0)
    (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1101)
  (parent 1002)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 1) (12 0)) ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1102)
  (parent 1003)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((1 1) (12 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1103)
  (parent 1003)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (12 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1104)
  (parent 1003)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0))
    ((13 1) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 1105)
  (parent 1003)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 1) (12 0)) ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1106)
  (parent 1007)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((1 1) (12 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1107)
  (parent 1007)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (12 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1108)
  (parent 1007)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0))
    ((13 1) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 13 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 13 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1)
    (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 1109)
  (parent 1007)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (0 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1110)
  (parent 1011)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (3 0))
    ((3 2) (8 0)) ((4 1) (8 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 7 1) (trans 2 1)
    (trans 1 1) (trans 7 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1111)
  (parent 1012)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l)))
  (label 1112)
  (parent 1021)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 1) (6 0)) ((4 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1113)
  (parent 1022)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 1) (4 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 4 1) (trans 8 0) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 1114)
  (parent 1023)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((3 1) (5 0)) ((4 1) (10 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 7 1) (trans 5 1) (trans 9 0) (trans 7 0) (trans 5 0)
    (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1115)
  (parent 1024)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 1) (12 0)) ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1116)
  (parent 1025)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (12 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1117)
  (parent 1025)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (12 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1118)
  (parent 1025)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0))
    ((13 1) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 1119)
  (parent 1025)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 1) (12 0)) ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1120)
  (parent 1029)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (12 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1121)
  (parent 1029)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (12 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1122)
  (parent 1029)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0))
    ((13 1) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 13 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 13 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1)
    (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 1123)
  (parent 1029)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1)) ((2 2) (6 0))
    ((3 1) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1124)
  (parent 1033)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (4 0)) ((4 2) (8 0)) ((5 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1125)
  (parent 1034)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 2) (6 0)) ((4 2) (6 0)) ((5 1) (0 0)) ((5 2) (6 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 3 1) (trans 4 0) (trans 3 0) (neq self b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1126)
  (parent 1044)
  (seen 1195)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (4 0)) ((2 2) (0 1))
    ((3 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 4 1) (trans 8 0) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 1127)
  (parent 1045)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 1) (7 0)) ((4 1) (10 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 7 1) (trans 5 1) (trans 9 0) (trans 7 0) (trans 5 0)
    (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1128)
  (parent 1046)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 1) (12 0)) ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1129)
  (parent 1047)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((1 1) (12 0))
    ((2 1) (6 0)) ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1130)
  (parent 1047)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (12 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1131)
  (parent 1047)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0))
    ((13 1) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 1132)
  (parent 1047)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 1) (12 0)) ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1133)
  (parent 1051)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((1 1) (12 0))
    ((2 1) (6 0)) ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1134)
  (parent 1051)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (12 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1135)
  (parent 1051)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0))
    ((13 1) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 13 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 13 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1)
    (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 1136)
  (parent 1051)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0)) ((3 2) (0 3))
    ((4 2) (0 3)) ((5 1) (0 3)) ((6 1) (0 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1)
    (trans 4 0) (trans 3 0) (neq self b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1137)
  (parent 1055)
  (seen 1198)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (8 0)) ((4 1) (8 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 7 1) (trans 2 1)
    (trans 1 1) (trans 7 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1138)
  (parent 1056)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (10 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 7 1)
    (trans 5 1) (trans 10 0) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 1139)
  (parent 1065)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 11 1)
    (trans 8 1) (trans 6 1) (trans 11 0) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 13 3 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1140)
  (parent 1066)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1141)
  (parent 1066)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 1142)
  (parent 1066)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (gen) (rec l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0) (12 0))
  (gen-st (pv a la) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 13 0 init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) la))
      (send (cat (exp (gen) (rec l)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (gen) (rec l))) (send (cat (gen) (rec l))))
    ((recv l) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 1143)
  (parent 1066)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (11 0))
    ((3 1) (2 0)) ((3 1) (10 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 3)) ((12 1) (0 0)) ((12 1) (7 0))
    ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (11 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 3 1) (trans 1 1) (trans 12 0) (trans 3 0)
    (trans 1 0) (trans 10 1) (trans 7 1) (trans 5 1) (trans 10 0)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 1 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1144)
  (parent 1066)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 3) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 1145)
  (parent 1066)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 13 2 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 1146)
  (parent 1066)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 3)) ((12 1) (3 0)) ((12 1) (10 0))
    ((12 2) (11 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (11 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 2 1) (trans 1 1) (trans 12 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 7 1) (trans 5 1) (trans 10 0)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (displaced 4 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-7 (pv self l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1147)
  (parent 1066)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1)
    (trans 6 1) (trans 11 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1148)
  (parent 1066)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (10 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (trans 8 1) (trans 2 1) (trans 1 1)
    (trans 8 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 6 1)
    (trans 4 1) (trans 10 0) (trans 6 0) (trans 4 0) (neq a self))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 1149)
  (parent 1067)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 9 1) (trans 2 1)
    (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0) (trans 11 1)
    (trans 7 1) (trans 5 1) (trans 11 0) (trans 7 0) (trans 5 0)
    (neq a self))
  (operation nonce-test (displaced 13 3 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1150)
  (parent 1068)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0)))))
  (label 1151)
  (parent 1068)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 1152)
  (parent 1068)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l) (mul (rec l) (rec l))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (8 0) (12 0))
  (gen-st (pv a la) (pv self l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (displaced 13 0 init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) la))
      (send (cat (exp (gen) (rec l)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-4 "nil")) (send la))
    ((recv (cat (exp (gen) l) (mul (rec l) (rec l))))
      (send (cat (exp (gen) l) (mul (rec l) (rec l)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))) ((recv l) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 1153)
  (parent 1068)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 3) (11 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (2 0)) ((8 1) (10 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 1) (0 0))
    ((12 1) (6 0)) ((12 2) (11 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (7 0) (11 0))
  (gen-st (pv self l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 8 1) (trans 1 1) (trans 12 0) (trans 8 0)
    (trans 1 0) (trans 10 1) (trans 6 1) (trans 4 1) (trans 10 0)
    (trans 6 0) (trans 4 0) (neq self-0 self))
  (operation nonce-test (displaced 1 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-4 (pv self l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-2 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-4 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1154)
  (parent 1068)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0) (trans 7 0)
    (trans 5 0) (neq a self))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 1155)
  (parent 1068)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (displaced 13 2 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 1156)
  (parent 1068)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (displaced 13 9 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 1157)
  (parent 1068)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 9 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 7 1)
    (trans 5 1) (trans 11 0) (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1158)
  (parent 1068)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 2) (6 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 4 1) (trans 5 0) (trans 4 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1159)
  (parent 1070)
  (seen 1201)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 4 1) (trans 8 0) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 1160)
  (parent 1071)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((3 1) (7 0)) ((4 1) (10 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 7 1) (trans 5 1) (trans 9 0) (trans 7 0) (trans 5 0)
    (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1161)
  (parent 1072)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 1) (12 0)) ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1162)
  (parent 1073)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (12 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1163)
  (parent 1073)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (12 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1164)
  (parent 1073)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0))
    ((13 1) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 1165)
  (parent 1073)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 1) (12 0)) ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1166)
  (parent 1077)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (12 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1167)
  (parent 1077)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (12 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1168)
  (parent 1077)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0))
    ((13 1) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 13 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 13 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1)
    (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 1169)
  (parent 1077)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (6 0))
    ((3 1) (5 0)) ((4 2) (0 3)) ((5 2) (0 3)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 4 1)
    (trans 5 0) (trans 4 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1170)
  (parent 1081)
  (seen 1204)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (6 0)) ((4 2) (8 0)) ((5 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1171)
  (parent 1082)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 1) (10 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 7 1)
    (trans 5 1) (trans 10 0) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 1172)
  (parent 1091)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 11 1)
    (trans 8 1) (trans 6 1) (trans 11 0) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 13 3 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1173)
  (parent 1092)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1174)
  (parent 1092)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 1175)
  (parent 1092)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0) (12 0))
  (gen-st (pv a la) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 13 0 init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l))) (send (cat (gen) (rec l))))
    ((recv l) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 1176)
  (parent 1092)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (11 0))
    ((3 1) (2 0)) ((3 1) (10 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 3)) ((12 1) (0 0)) ((12 1) (5 0))
    ((12 2) (11 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (11 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 3 1) (trans 1 1) (trans 12 0) (trans 3 0)
    (trans 1 0) (trans 10 1) (trans 7 1) (trans 5 1) (trans 10 0)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 1 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1177)
  (parent 1092)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 3) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 1178)
  (parent 1092)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 13 2 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 1179)
  (parent 1092)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 3)) ((12 1) (3 0)) ((12 1) (10 0))
    ((12 2) (11 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (11 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 2 1) (trans 1 1) (trans 12 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 7 1) (trans 5 1) (trans 10 0)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (displaced 4 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-7 (pv self l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1180)
  (parent 1092)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1)
    (trans 6 1) (trans 11 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1181)
  (parent 1092)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (10 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (trans 8 1) (trans 2 1) (trans 1 1)
    (trans 8 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 6 1)
    (trans 4 1) (trans 10 0) (trans 6 0) (trans 4 0) (neq a self))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 1182)
  (parent 1093)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 9 1) (trans 2 1)
    (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0) (trans 11 1)
    (trans 7 1) (trans 5 1) (trans 11 0) (trans 7 0) (trans 5 0)
    (neq a self))
  (operation nonce-test (displaced 13 3 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1183)
  (parent 1094)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0)))))
  (label 1184)
  (parent 1094)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 1185)
  (parent 1094)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) (rec l))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (8 0) (12 0))
  (gen-st (pv a la) (pv self l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (displaced 13 0 init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-3 "nil")) (send la))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (mul (rec l) (rec l))))
      (send (cat (exp (gen) l) (mul (rec l) (rec l)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))) ((recv l) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 1186)
  (parent 1094)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 3) (11 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (2 0)) ((8 1) (10 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 1) (0 0))
    ((12 1) (4 0)) ((12 2) (11 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (7 0) (11 0))
  (gen-st (pv self l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 8 1) (trans 1 1) (trans 12 0) (trans 8 0)
    (trans 1 0) (trans 10 1) (trans 6 1) (trans 4 1) (trans 10 0)
    (trans 6 0) (trans 4 0) (neq self-0 self))
  (operation nonce-test (displaced 1 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-4 (pv self l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-4 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1187)
  (parent 1094)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0) (trans 7 0)
    (trans 5 0) (neq a self))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 1188)
  (parent 1094)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (displaced 13 2 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv self l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 1189)
  (parent 1094)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (displaced 13 9 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 1190)
  (parent 1094)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 9 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 7 1)
    (trans 5 1) (trans 11 0) (trans 7 0) (trans 5 0) (neq a self))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1191)
  (parent 1094)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 1192)
  (parent 1095)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b) (la l-0) (x x) (beta l) (upsilon upsilon) (na na)
        (nb nb) (priv-stor priv-stor) (pt pt))))
  (origs (pt-3 (3 1)) (l (2 1)) (pt-2 (2 1)) (l-0 (1 1)) (pt (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b))
  (operation generalization weakened ((0 2) (4 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 1193)
  (parent 1097)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((3 2) (0 3)) ((4 1) (0 0))
    ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0)
    (neq self b))
  (operation generalization weakened ((2 1) (0 3)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1194)
  (parent 1098)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (5 0)) ((4 1) (0 0)) ((4 2) (5 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1195)
  (parent 1099)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 1196)
  (parent 1100)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (4 0)) ((4 2) (9 0)) ((5 1) (9 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 4 1) (trans 8 0) (trans 6 0) (trans 4 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1197)
  (parent 1101)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 1) (0 0)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (4 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0)
    (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1198)
  (parent 1110)
  (seen 1098)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (7 0))
    ((3 1) (7 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (7 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1199)
  (parent 1111)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b))
  (operation generalization weakened ((3 1) (0 3)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l)))
  (label 1200)
  (parent 1112)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 2) (5 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1201)
  (parent 1113)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0)))
  (label 1202)
  (parent 1114)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (4 0)) ((4 2) (9 0)) ((5 1) (9 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 4 1) (trans 8 0) (trans 6 0) (trans 4 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1203)
  (parent 1115)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 2) (0 3)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1204)
  (parent 1124)
  (seen 1112)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (7 0)) ((4 1) (7 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1205)
  (parent 1125)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1)) ((3 1) (5 0))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 1206)
  (parent 1127)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (4 0))
    ((2 2) (0 1)) ((3 1) (6 0)) ((4 2) (9 0)) ((5 1) (9 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 4 1) (trans 8 0) (trans 6 0) (trans 4 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1207)
  (parent 1128)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 2) (7 0)) ((4 2) (7 0)) ((5 1) (7 0)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (7 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 3 1) (trans 4 0) (trans 3 0) (neq self b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1208)
  (parent 1138)
  (seen 1257)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((2 2) (0 1))
    ((3 1) (9 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 6 1)
    (trans 4 1) (trans 9 0) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 1209)
  (parent 1139)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 1) (10 0)) ((4 1) (11 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 7 1) (trans 5 1) (trans 10 0) (trans 7 0) (trans 5 0)
    (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1210)
  (parent 1140)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 1) (13 0)) ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1211)
  (parent 1141)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (13 0))
    ((2 1) (6 0)) ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1212)
  (parent 1141)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((4 1) (13 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 14 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1213)
  (parent 1141)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)) ((14 1) (13 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1)
    (trans 6 1) (trans 11 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 1214)
  (parent 1141)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 1) (13 0)) ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1215)
  (parent 1145)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (13 0))
    ((2 1) (6 0)) ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1216)
  (parent 1145)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((4 1) (13 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 14 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1217)
  (parent 1145)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 3) (12 0)) ((14 1) (13 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 14 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 14 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1)
    (trans 11 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 1218)
  (parent 1145)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l-0) (pv self l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (trans 7 1) (trans 2 1) (trans 1 1)
    (trans 7 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 5 1)
    (trans 3 1) (trans 9 0) (trans 5 0) (trans 3 0) (neq a self))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 1219)
  (parent 1149)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0))
    ((2 2) (0 1)) ((3 1) (11 0)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (10 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (11 0))
  (gen-st (pv a l-0) (pv self l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 8 1) (trans 2 1)
    (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 6 1) (trans 4 1) (trans 10 0) (trans 6 0) (trans 4 0)
    (neq a self))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1220)
  (parent 1150)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 1) (13 0)) ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (0 3)) ((13 2) (12 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 1221)
  (parent 1151)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor) (la l-1)
    (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 1) (13 0))
    ((2 1) (5 0)) ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (0 3)) ((13 2) (12 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l-1) (pv self l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 1222)
  (parent 1151)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 1) (13 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (0 3)) ((13 2) (12 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation channel-test (displaced 14 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 1223)
  (parent 1151)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a self a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)) ((14 1) (13 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 9 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 7 1)
    (trans 5 1) (trans 11 0) (trans 7 0) (trans 5 0) (neq a self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 1224)
  (parent 1151)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 1) (13 0)) ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (0 3)) ((13 3) (12 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0) (trans 7 0)
    (trans 5 0) (neq a self))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1225)
  (parent 1155)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 1) (13 0))
    ((2 1) (5 0)) ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (0 3)) ((13 3) (12 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l-1) (pv self l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0) (trans 7 0)
    (trans 5 0) (neq a self))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1226)
  (parent 1155)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 1) (13 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (0 3)) ((13 3) (12 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0) (trans 7 0)
    (trans 5 0) (neq a self))
  (operation channel-test (displaced 14 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1227)
  (parent 1155)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)) ((14 1) (13 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1) (pv b l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 14 1)
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 14 0) (trans 9 0)
    (trans 2 0) (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1)
    (trans 11 0) (trans 7 0) (trans 5 0) (neq a self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b l-2)))))
  (label 1228)
  (parent 1155)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 2) (0 3)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 4 1) (trans 7 0) (trans 5 0) (trans 4 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 1229)
  (parent 1160)
  (seen 1260)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (6 0)) ((4 2) (9 0)) ((5 1) (9 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 4 1) (trans 8 0) (trans 6 0) (trans 4 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1230)
  (parent 1161)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (5 0)) ((4 2) (7 0)) ((5 2) (7 0))
    ((6 1) (7 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 4 1) (trans 5 0) (trans 4 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1231)
  (parent 1171)
  (seen 1262)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (9 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 6 1)
    (trans 4 1) (trans 9 0) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 1232)
  (parent 1172)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 1) (10 0)) ((4 1) (11 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 7 1) (trans 5 1) (trans 10 0) (trans 7 0) (trans 5 0)
    (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1233)
  (parent 1173)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 1) (13 0)) ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1234)
  (parent 1174)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (13 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1235)
  (parent 1174)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((4 1) (13 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 14 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1236)
  (parent 1174)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)) ((14 1) (13 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1)
    (trans 6 1) (trans 11 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 1237)
  (parent 1174)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 1) (13 0)) ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1238)
  (parent 1178)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (13 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1239)
  (parent 1178)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((4 1) (13 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 14 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1240)
  (parent 1178)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 3) (12 0)) ((14 1) (13 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 14 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 14 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1)
    (trans 11 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 1241)
  (parent 1178)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l) (pv self l-0) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (trans 7 1) (trans 2 1) (trans 1 1)
    (trans 7 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 5 1)
    (trans 3 1) (trans 9 0) (trans 5 0) (trans 3 0) (neq a self))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 1242)
  (parent 1182)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (11 0)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (10 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (11 0))
  (gen-st (pv a l) (pv self l-0) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 8 1) (trans 2 1)
    (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 6 1) (trans 4 1) (trans 10 0) (trans 6 0) (trans 4 0)
    (neq a self))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1243)
  (parent 1183)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 1) (13 0)) ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (0 3)) ((13 2) (12 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv self l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 1244)
  (parent 1184)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor) (la l-1)
    (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (13 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (0 3)) ((13 2) (12 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l-1) (pv self l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 1245)
  (parent 1184)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 1) (13 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (0 3)) ((13 2) (12 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self))
  (operation channel-test (displaced 14 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 1246)
  (parent 1184)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a self a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)) ((14 1) (13 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 9 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 7 1)
    (trans 5 1) (trans 11 0) (trans 7 0) (trans 5 0) (neq a self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 1247)
  (parent 1184)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 1) (13 0)) ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (0 3)) ((13 3) (12 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0) (trans 7 0)
    (trans 5 0) (neq a self))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv self l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1248)
  (parent 1188)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (13 0))
    ((2 1) (7 0)) ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (0 3)) ((13 3) (12 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l-1) (pv self l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0) (trans 7 0)
    (trans 5 0) (neq a self))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1249)
  (parent 1188)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 1) (13 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (0 3)) ((13 3) (12 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0) (trans 7 0)
    (trans 5 0) (neq a self))
  (operation channel-test (displaced 14 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1250)
  (parent 1188)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)) ((14 1) (13 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1) (pv b l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 14 1)
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 14 0) (trans 9 0)
    (trans 2 0) (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1)
    (trans 11 0) (trans 7 0) (trans 5 0) (neq a self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b l-2)))))
  (label 1251)
  (parent 1188)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b))
  (operation generalization weakened ((1 1) (4 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 1252)
  (parent 1193)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((3 2) (0 3)) ((4 1) (0 0))
    ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0)
    (neq self b))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1253)
  (parent 1194)
  (realized)
  (shape)
  (maps
    ((0)
      ((a self) (b b) (la l-0) (x x) (beta l) (upsilon y) (na na)
        (nb nb) (priv-stor priv-stor-1) (pt pt-5))))
  (origs (l-0 (4 1)) (pt-5 (4 1)) (pt-3 (3 1)) (lb (2 1)) (pt-1 (2 1))
    (l (1 1)) (pt-0 (1 1)) (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((2 1) (5 0)) ((3 2) (0 3)) ((3 2) (5 0)) ((4 1) (0 0))
    ((4 2) (0 3)) ((4 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b))
  (operation generalization deleted (5 1))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv (cat (exp (gen) y) w))))
  (label 1254)
  (parent 1195)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 1255)
  (parent 1196)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 1) (8 0)) ((4 1) (8 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (8 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1256)
  (parent 1197)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 2) (6 0)) ((4 1) (6 0)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (4 0) (6 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1257)
  (parent 1199)
  (seen 1195)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l)))
  (label 1258)
  (parent 1200)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((3 1) (5 0)) ((4 2) (0 3)) ((4 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization deleted (5 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 1259)
  (parent 1201)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0)))
  (label 1260)
  (parent 1202)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 1) (8 0)) ((4 1) (8 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1261)
  (parent 1203)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (6 0)) ((4 2) (6 0)) ((5 1) (6 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1262)
  (parent 1205)
  (seen 1201)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1)) ((3 1) (4 0))
    ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 1263)
  (parent 1206)
  (seen 1276)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 1) (8 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (8 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1264)
  (parent 1207)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1)) ((3 1) (8 0))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0))
    ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 5 1)
    (trans 8 0) (trans 5 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 1265)
  (parent 1209)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0))
    ((2 2) (0 1)) ((3 1) (9 0)) ((4 2) (10 0)) ((5 1) (10 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 6 1) (trans 4 1) (trans 9 0) (trans 6 0) (trans 4 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1266)
  (parent 1210)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (8 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv a l-0) (pv self l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a self))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-5 (pv self lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 1267)
  (parent 1219)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 2) (10 0)) ((4 1) (10 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (10 0))
  (gen-st (pv a l-0) (pv self l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 7 1) (trans 2 1)
    (trans 1 1) (trans 7 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 5 1) (trans 3 1) (trans 9 0) (trans 5 0) (trans 3 0)
    (neq a self))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1268)
  (parent 1220)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 1) (5 0)) ((4 2) (8 0)) ((5 2) (8 0))
    ((6 1) (8 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 4 1) (trans 7 0) (trans 5 0) (trans 4 0)
    (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1269)
  (parent 1230)
  (seen 1281)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (8 0)) ((4 2) (0 3)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 5 1)
    (trans 4 1) (trans 8 0) (trans 5 0) (trans 4 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 1270)
  (parent 1232)
  (seen 1283)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (9 0)) ((4 2) (10 0)) ((5 1) (10 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 6 1) (trans 4 1) (trans 9 0) (trans 6 0) (trans 4 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1271)
  (parent 1233)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (4 0)) ((2 2) (0 1))
    ((3 2) (0 3)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv a l) (pv self l-0) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 3 1) (trans 8 0) (trans 4 0) (trans 3 0) (neq a self))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 1272)
  (parent 1242)
  (seen 1285)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 2) (10 0)) ((4 1) (10 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (10 0))
  (gen-st (pv a l) (pv self l-0) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 7 1) (trans 2 1)
    (trans 1 1) (trans 7 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 5 1) (trans 3 1) (trans 9 0) (trans 5 0) (trans 3 0)
    (neq a self))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1273)
  (parent 1243)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (4 0)) ((2 2) (0 1))
    ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b))
  (operation generalization weakened ((2 2) (4 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 1274)
  (parent 1252)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (0 3)) ((3 2) (5 0)) ((4 1) (0 0)) ((4 2) (0 3))
    ((4 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b))
  (operation generalization weakened ((2 1) (0 3)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv (cat (exp (gen) y) w))))
  (label 1275)
  (parent 1254)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0)))
  (label 1276)
  (parent 1255)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0))
    ((2 2) (0 1)) ((3 1) (7 0)) ((4 2) (7 0)) ((5 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (7 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1277)
  (parent 1256)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b-0) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b-0)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((4 2) (0 3)))
  (non-orig (privk "sig" b) (privk "sig" b-0))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b-0))
  (operation generalization separated b-0)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b-0 (exp (gen) l-0) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na a b-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b-0 l-0)))
      (send
        (sig (body b-0 (exp (gen) l-0) (pubk "sig" b-0))
          (privk "sig" b-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l)))
  (label 1278)
  (parent 1258)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 2) (0 3)) ((4 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((3 1) (0 3)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 1279)
  (parent 1259)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 2) (0 3)) ((5 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 4 1)
    (trans 5 0) (trans 4 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0)))
  (label 1280)
  (parent 1260)
  (seen 1291)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (7 0)) ((4 2) (7 0)) ((5 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (7 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1281)
  (parent 1261)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (4 0)) ((4 2) (7 0)) ((5 1) (7 0)) ((6 2) (5 0))
    ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (7 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1282)
  (parent 1264)
  (seen 1292)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (7 0))
    ((4 2) (0 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 4 1)
    (trans 7 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 1283)
  (parent 1265)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 1) (8 0)) ((4 1) (9 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 5 1) (trans 8 0) (trans 5 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1284)
  (parent 1266)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 1) (7 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 3 1)
    (trans 7 0) (trans 3 0) (neq a self))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-5 (pv self lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 1285)
  (parent 1267)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (9 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 4 1) (trans 8 0) (trans 4 0) (neq a self))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-5 (pv self lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1286)
  (parent 1268)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 1) (8 0)) ((4 2) (9 0)) ((5 2) (9 0))
    ((6 1) (9 0)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 5 1) (trans 4 1) (trans 8 0) (trans 5 0) (trans 4 0)
    (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1287)
  (parent 1271)
  (seen 1297)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (4 0))
    ((2 2) (0 1)) ((3 2) (9 0)) ((4 2) (9 0)) ((5 1) (9 0))
    ((6 1) (8 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 4 1) (trans 3 1) (trans 8 0) (trans 4 0) (trans 3 0)
    (neq a self))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1288)
  (parent 1273)
  (seen 1299)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b))
  (operation generalization weakened ((2 1) (4 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 1289)
  (parent 1274)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 1) (5 0)) ((3 2) (0 3)) ((4 1) (0 0)) ((4 2) (0 3))
    ((4 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b))
  (operation generalization weakened ((3 2) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv (cat (exp (gen) y) w))))
  (label 1290)
  (parent 1275)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0)))
  (label 1291)
  (parent 1276)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 1) (6 0)) ((4 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1292)
  (parent 1277)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b-0) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b-0)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((4 2) (0 3)))
  (non-orig (privk "sig" b-0))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b-0))
  (operation generalization forgot (privk "sig" b))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b-0 (exp (gen) l-0) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na a b-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b-0 l-0)))
      (send
        (sig (body b-0 (exp (gen) l-0) (pubk "sig" b-0))
          (privk "sig" b-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l)))
  (label 1293)
  (parent 1278)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b-0) (la l) (x x) (beta l-0) (upsilon y) (na na) (nb nb)
        (priv-stor priv-stor) (pt pt))))
  (origs (l-0 (2 1)) (pt-2 (2 1)) (pt-5 (4 1)) (lb (3 1)) (pt-3 (3 1))
    (l (1 1)) (pt (1 1)) (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 1) (5 0)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((4 2) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 1294)
  (parent 1279)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 1) (6 0)) ((4 2) (6 0)) ((5 2) (6 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 4 1) (trans 5 0) (trans 4 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1295)
  (parent 1281)
  (seen 1303)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (6 0))
    ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 1296)
  (parent 1283)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (7 0)) ((4 2) (8 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (8 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 4 1) (trans 7 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1297)
  (parent 1284)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3))
    ((4 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l-0) (pv self l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 3 1)
    (trans 6 0) (trans 3 0) (neq a self))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-5 (pv self lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 1298)
  (parent 1285)
  (seen 1296)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 2) (8 0)) ((4 1) (8 0)) ((5 1) (7 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (4 0) (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 3 1) (trans 7 0) (trans 3 0) (neq a self))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-5 (pv self lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1299)
  (parent 1286)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 1300)
  (parent 1289)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b) (la l) (x x) (beta lb) (upsilon (mul w y)) (na na)
        (nb nb) (priv-stor priv-stor) (pt pt))))
  (origs (pt-3 (3 1)) (lb (2 1)) (pt-2 (2 1)) (l (1 1)) (pt (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (0 3)) ((4 1) (0 0)) ((4 2) (0 3)) ((4 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b))
  (operation generalization weakened ((3 1) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv (cat (exp (gen) y) w))))
  (label 1301)
  (parent 1290)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b))
  (operation generalization weakened ((3 1) (0 3)))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0)))
  (label 1302)
  (parent 1291)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 2) (5 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1303)
  (parent 1292)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((4 1) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 1304)
  (parent 1294)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (5 0))
    ((4 2) (0 3)) ((5 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 4 1)
    (trans 5 0) (trans 4 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 1305)
  (parent 1296)
  (seen 1291)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (6 0)) ((4 2) (7 0)) ((5 1) (7 0)) ((6 2) (5 0))
    ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1306)
  (parent 1297)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 2) (7 0)) ((4 1) (6 0)) ((4 2) (7 0)) ((5 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 3 1) (trans 6 0) (trans 3 0) (neq a self))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-5 (pv self lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1307)
  (parent 1299)
  (seen 1306)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (0 3)) ((4 1) (0 0)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b))
  (operation generalization weakened ((4 2) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv (cat (exp (gen) y) w))))
  (label 1308)
  (parent 1301)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0)))
  (label 1309)
  (parent 1302)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b) (la l-0) (x x) (beta l) (upsilon y) (na na) (nb nb)
        (priv-stor priv-stor) (pt pt))))
  (origs (pt-5 (4 1)) (lb (3 1)) (pt-3 (3 1)) (l (2 1)) (pt-2 (2 1))
    (l-0 (1 1)) (pt (1 1)) (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((3 1) (5 0)) ((4 2) (0 3)) ((4 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization deleted (5 1))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w))))
  (label 1310)
  (parent 1303)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((3 1) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 1311)
  (parent 1304)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 2) (6 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 4 1) (trans 5 0) (trans 4 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1312)
  (parent 1306)
  (seen 1303)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((3 2) (0 3))
    ((4 1) (0 0)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b))
  (operation generalization weakened ((2 1) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv (cat (exp (gen) y) w))))
  (label 1313)
  (parent 1308)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 2) (0 3)) ((4 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((3 1) (0 3)))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w))))
  (label 1314)
  (parent 1310)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((0 2) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 1315)
  (parent 1311)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((1 2) (5 0)) ((3 2) (0 3))
    ((4 1) (0 0)) ((4 1) (5 0)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b))
  (operation generalization weakened ((0 2) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv (cat (exp (gen) y) w))))
  (label 1316)
  (parent 1313)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 1) (5 0)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((4 2) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w))))
  (label 1317)
  (parent 1314)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((1 1) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 1318)
  (parent 1315)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((1 2) (5 0)) ((3 2) (0 3))
    ((4 1) (0 0)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b))
  (operation generalization weakened ((4 1) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv (cat (exp (gen) y) w))))
  (label 1319)
  (parent 1316)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((4 1) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w))))
  (label 1320)
  (parent 1317)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((2 2) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 1321)
  (parent 1318)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((3 2) (0 3))
    ((4 1) (0 0)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b))
  (operation generalization weakened ((1 2) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv (cat (exp (gen) y) w))))
  (label 1322)
  (parent 1319)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((3 1) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w))))
  (label 1323)
  (parent 1320)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((2 1) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 1324)
  (parent 1321)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((3 2) (0 3)) ((4 1) (0 0))
    ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b))
  (operation generalization weakened ((1 1) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv (cat (exp (gen) y) w))))
  (label 1325)
  (parent 1322)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((0 2) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w))))
  (label 1326)
  (parent 1323)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 1327)
  (parent 1324)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((3 2) (0 3)) ((4 1) (0 0))
    ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self b))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))) ((recv (cat (exp (gen) y) w))))
  (label 1328)
  (parent 1325)
  (realized)
  (shape)
  (maps
    ((0)
      ((a self) (b b) (la l-0) (x x) (beta l) (upsilon (mul w y))
        (na na) (nb nb) (priv-stor priv-stor-1) (pt pt-5))))
  (origs (l-0 (4 1)) (pt-5 (4 1)) (pt-3 (3 1)) (lb (2 1)) (pt-1 (2 1))
    (l (1 1)) (pt-0 (1 1)) (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((1 1) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w))))
  (label 1329)
  (parent 1326)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b-0) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b-0)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((4 2) (0 3)))
  (non-orig (privk "sig" b) (privk "sig" b-0))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b-0))
  (operation generalization separated b-0)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b-0 (exp (gen) l-0) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na a b-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b-0 l-0)))
      (send
        (sig (body b-0 (exp (gen) l-0) (pubk "sig" b-0))
          (privk "sig" b-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 1330)
  (parent 1327)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((2 2) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w))))
  (label 1331)
  (parent 1329)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b-0) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b-0)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((4 2) (0 3)))
  (non-orig (privk "sig" b-0))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b-0))
  (operation generalization forgot (privk "sig" b))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b-0 (exp (gen) l-0) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na a b-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b-0 l-0)))
      (send
        (sig (body b-0 (exp (gen) l-0) (pubk "sig" b-0))
          (privk "sig" b-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 1332)
  (parent 1330)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b-0) (la l) (x x) (beta l-0) (upsilon (mul w y)) (na na)
        (nb nb) (priv-stor priv-stor) (pt pt))))
  (origs (l-0 (2 1)) (pt-2 (2 1)) (pt-5 (4 1)) (lb (3 1)) (pt-3 (3 1))
    (l (1 1)) (pt (1 1)) (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((2 1) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w))))
  (label 1333)
  (parent 1331)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w))))
  (label 1334)
  (parent 1333)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b) (la l-0) (x x) (beta l) (upsilon (mul w y)) (na na)
        (nb nb) (priv-stor priv-stor) (pt pt))))
  (origs (pt-5 (4 1)) (lb (3 1)) (pt-3 (3 1)) (l (2 1)) (pt-2 (2 1))
    (l-0 (1 1)) (pt (1 1)) (na (0 2))))

(comment "Nothing left to do")

(defprotocol dhcr-um diffie-hellman
  (defrole init
    (vars (la x rndx) (beta upsilon expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv a la))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb))
    (uniq-orig na)
    (uniq-gen x)
    (absent (x la) (x beta))
    (assume (fact neq (exp (gen) upsilon) (gen)))
    (gen-st (pv a la)))
  (defrole resp
    (vars (lb y rndx) (alpha zeta expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv b lb))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    (uniq-orig nb)
    (uniq-gen y)
    (absent (y lb) (y alpha) (y zeta))
    (assume (fact neq (exp (gen) zeta) (gen)))
    (gen-st (pv b lb)))
  (defrole ltx-gen
    (vars (self name) (l rndx) (priv-stor locn) (ignore mesg))
    (trace (load priv-stor ignore) (stor priv-stor (pv self l))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    (uniq-orig l))
  (defrole ltx-disclose
    (vars (self name) (l rndx) (priv-stor locn))
    (trace (load priv-stor (pv self l)) (stor priv-stor "nil") (send l))
    (gen-st (pv self l)))
  (defrule undisclosed-not-disclosed
    (forall ((z strd) (l rndx))
      (implies
        (and (fact undisclosed l) (p "ltx-disclose" z 2)
          (p "ltx-disclose" "l" z l))
        (false))))
  (defgenrule neqRl_indx
    (forall ((x indx)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_strd
    (forall ((x strd)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_mesg
    (forall ((x mesg)) (implies (fact neq x x) (false))))
  (defgenrule scissorsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (leads-to z0 i0 z2 i2))
        (and (= z1 z2) (= i1 i2)))))
  (defgenrule cakeRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (leads-to z0 i0 z1 i1)
          (leads-to z0 i0 z2 i2) (prec z1 i1 z2 i2)) (false))))
  (defgenrule no-interruption
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (leads-to z0 i0 z2 i2) (trans z1 i1)
          (same-locn z0 i0 z1 i1) (prec z0 i0 z1 i1) (prec z1 i1 z2 i2))
        (false))))
  (defgenrule shearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (same-locn z0 i0 z2 i2)
          (prec z0 i0 z2 i2))
        (or (and (= z1 z2) (= i1 i2)) (prec z1 i1 z2 i2)))))
  (defgenrule invShearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (same-locn z0 i0 z1 i1)
          (leads-to z1 i1 z2 i2) (prec z0 i0 z2 i2))
        (or (and (= z0 z1) (= i0 i1)) (prec z0 i0 z1 i1)))))
  (defgenrule assume-init-0
    (forall ((z strd) (upsilon expt))
      (implies (and (p "init" z 4) (p "init" "upsilon" z upsilon))
        (fact neq (exp (gen) upsilon) (gen)))))
  (defgenrule assume-resp-0
    (forall ((z strd) (zeta expt))
      (implies (and (p "resp" z 3) (p "resp" "zeta" z zeta))
        (fact neq (exp (gen) zeta) (gen)))))
  (defgenrule trRl_ltx-gen-at-1
    (forall ((z strd)) (implies (p "ltx-gen" z 2) (trans z 1))))
  (defgenrule trRl_ltx-gen-at-0
    (forall ((z strd)) (implies (p "ltx-gen" z 2) (trans z 0))))
  (defgenrule trRl_ltx-disclose-at-1
    (forall ((z strd)) (implies (p "ltx-disclose" z 2) (trans z 1))))
  (defgenrule trRl_ltx-disclose-at-0
    (forall ((z strd)) (implies (p "ltx-disclose" z 2) (trans z 0))))
  (defgenrule gen-st-init-0
    (forall ((z strd) (la rndx) (a name))
      (implies
        (and (p "init" z 1) (p "init" "la" z la) (p "init" "a" z a))
        (gen-st (pv a la)))))
  (defgenrule gen-st-resp-0
    (forall ((z strd) (lb rndx) (b name))
      (implies
        (and (p "resp" z 1) (p "resp" "lb" z lb) (p "resp" "b" z b))
        (gen-st (pv b lb)))))
  (defgenrule gen-st-ltx-disclose-0
    (forall ((z strd) (l rndx) (self name))
      (implies
        (and (p "ltx-disclose" z 1) (p "ltx-disclose" "l" z l)
          (p "ltx-disclose" "self" z self)) (gen-st (pv self l)))))
  (lang (sig sign) (body (tuple 3)) (pv (tuple 2))))

(defskeleton dhcr-um
  (vars (na nb na-0 nb-0 data) (a b name) (pt pt-0 pval)
    (priv-stor priv-stor-0 locn) (la lb rndx) (alpha beta expt) (y rndx)
    (zeta expt) (x rndx) (upsilon expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la la) (x x) (beta beta) (upsilon upsilon))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x la) (x beta))
  (facts (neq a b) (undisclosed la) (undisclosed lb))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb-0)))
  (label 1335)
  (unrealized (0 1) (1 1))
  (origs (na-0 (1 2)) (nb (0 3)))
  (comment "Not closed under rules"))

(defskeleton dhcr-um
  (vars (na nb na-0 nb-0 data) (a b name) (pt pt-0 pval)
    (priv-stor priv-stor-0 locn) (la lb rndx) (alpha beta expt) (y rndx)
    (zeta expt) (x rndx) (upsilon expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la la) (x x) (beta beta) (upsilon upsilon))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x la) (x beta))
  (gen-st (pv a la) (pv b lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed la) (undisclosed lb))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb-0)))
  (label 1336)
  (parent 1335)
  (unrealized (0 0) (0 1) (1 0) (1 1))
  (origs (na-0 (1 2)) (nb (0 3)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pval) (priv-stor priv-stor-0 locn) (lb rndx)
    (alpha beta expt) (y rndx) (zeta expt) (x rndx) (upsilon expt)
    (l rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (precedes ((2 1) (1 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x beta) (x l))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 2 0) (neq a b) (undisclosed l) (undisclosed lb))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-0 (pv a l))) (1 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul beta l))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l)))))
  (label 1337)
  (parent 1336)
  (unrealized (0 0) (0 1) (1 1) (1 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((2 1) (1 0)) ((3 2) (1 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 3 1) (trans 2 1) (trans 3 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed lb))
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)) (1 1))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))))
  (label 1338)
  (parent 1337)
  (unrealized (0 0) (0 1) (1 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 lb rndx)
    (zeta expt) (y rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta zeta))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) zeta y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 3) (1 3)) ((1 2) (0 2)) ((2 1) (1 0)) ((3 2) (1 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y zeta))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) zeta y)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 2 1) (trans 3 0)
    (trans 2 0) (neq a b) (undisclosed l) (undisclosed lb))
  (operation encryption-test (displaced 4 0 resp 4)
    (enc na-0 nb-0 a b
      (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y-0 zeta-0))))
    (1 3))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) zeta y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))))
      (send nb))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))))
  (label 1339)
  (parent 1338)
  (unrealized (0 0) (0 1) (0 4) (1 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt) (l l-0 lb-0 x y-0 rndx)
    (zeta-0 expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y-0 zeta-0)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-2) (lb lb-0) (y y-0)
    (alpha (mul l l-0 (rec lb-0))) (zeta zeta-0))
  (precedes ((1 2) (4 2)) ((2 1) (1 0)) ((3 2) (1 1)) ((4 3) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0)
    (y-0 (mul l l-0 (rec lb-0))) (y-0 lb-0) (y-0 zeta-0))
  (gen-st (pv a l) (pv b lb) (pv b lb-0))
  (facts (neq (exp (gen) (mul (rec x) y-0 zeta-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 3 1) (trans 2 1) (trans 3 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed lb))
  (operation encryption-test (added-strand resp 4)
    (enc na-0 nb-0 a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y-0 zeta-0))))
    (1 3))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y-0 zeta-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y-0 zeta-0)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-2 (cat pt-4 (pv b lb-0)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb-0))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na-0 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y-0 zeta-0))))))))
  (label 1340)
  (parent 1338)
  (unrealized (0 0) (0 1) (1 3) (4 0) (4 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((1 2) (4 0)) ((2 1) (1 0)) ((3 2) (1 1)) ((4 1) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 3 1) (trans 2 1) (trans 3 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed lb))
  (operation encryption-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (enc na-0 nb-0 a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))) (1 3))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send
        (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 1341)
  (parent 1338)
  (unrealized (0 0) (0 1) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 3) (1 3)) ((1 2) (0 2)) ((2 1) (1 0)) ((3 2) (1 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 3 1)
    (trans 2 1) (trans 3 0) (trans 2 0) (neq a b) (undisclosed l)
    (undisclosed lb))
  (operation nonce-test (displaced 4 0 resp 4) (exp (gen) y-0) (1 3))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))))
  (label 1342)
  (parent 1339)
  (unrealized (0 0) (0 1) (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 lb rndx)
    (zeta expt) (y rndx) (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta zeta))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) zeta y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) zeta y (rec w))) w))
  (precedes ((0 3) (4 0)) ((1 2) (0 2)) ((2 1) (1 0)) ((3 2) (1 1))
    ((4 1) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y zeta))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) zeta y)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 2 1) (trans 3 0)
    (trans 2 0) (neq a b) (undisclosed l) (undisclosed lb))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) zeta y (rec w))) w))
    (exp (gen) (mul (rec x) zeta y)) (1 3))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) zeta y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))))
      (send nb))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (mul (rec x) zeta y (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) zeta y (rec w))) w))))
  (label 1343)
  (parent 1339)
  (unrealized (0 0) (0 1) (0 4) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (l x y-0 rndx) (zeta-0 expt) (l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y-0 zeta-0)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (lb l-0) (y y-0) (alpha l) (zeta zeta-0))
  (precedes ((1 2) (4 2)) ((2 1) (1 0)) ((3 1) (4 0)) ((3 2) (1 1))
    ((4 3) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 l) (y-0 zeta-0)
    (y-0 l-0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y-0 zeta-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 3 1) (trans 2 1) (trans 3 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed lb))
  (operation channel-test (displaced 5 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-4 (pv b l-1))) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y-0 zeta-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y-0 zeta-0)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y-0 zeta-0))))))))
  (label 1344)
  (parent 1340)
  (unrealized (0 0) (0 1) (1 3) (4 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt) (l l-0 x y-0 rndx) (zeta-0 expt)
    (l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y-0 zeta-0)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-2) (lb l-1) (y y-0)
    (alpha (mul l l-0 (rec l-1))) (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (4 2)) ((2 1) (1 0)) ((3 2) (1 1)) ((4 3) (1 3))
    ((5 1) (4 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0)
    (y-0 (mul l l-0 (rec l-1))) (y-0 zeta-0) (y-0 l-1))
  (gen-st (pv a l) (pv b lb) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y-0 zeta-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 3 1) (trans 2 1) (trans 5 0) (trans 3 0)
    (trans 2 0) (neq a b) (undisclosed l) (undisclosed lb))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-4 (pv b l-1))) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y-0 zeta-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y-0 zeta-0)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-2 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na-0 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y-0 zeta-0)))))))
    ((load priv-stor-2 (cat pt-5 ignore-1))
      (stor priv-stor-2 (cat pt-4 (pv b l-1)))))
  (label 1345)
  (parent 1340)
  (unrealized (0 0) (0 1) (1 3) (4 1))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((3 2) (1 1)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 3 1) (trans 2 1) (trans 3 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed lb))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 1346)
  (parent 1341)
  (unrealized (0 0) (0 1) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta x))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (1 3)) ((1 2) (0 2)) ((2 1) (1 0)) ((3 1) (0 0))
    ((3 2) (1 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y x) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 3 1)
    (trans 2 1) (trans 3 0) (trans 2 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation channel-test (displaced 4 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (0 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (recv nb))
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))))
  (label 1347)
  (parent 1342)
  (unrealized (0 1) (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta x))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (precedes ((0 3) (1 3)) ((1 2) (0 2)) ((2 1) (1 0)) ((3 2) (1 1))
    ((4 1) (0 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y x) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 3 1) (trans 2 1) (trans 4 0) (trans 3 0) (trans 2 0)
    (neq a b) (undisclosed l) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv b l-1))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1)))))
  (label 1348)
  (parent 1342)
  (unrealized (0 1) (0 4))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 3) (4 0)) ((1 2) (0 2)) ((2 1) (1 0)) ((3 2) (1 1))
    ((4 1) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 3 1) (trans 2 1)
    (trans 3 0) (trans 2 0) (neq a b) (undisclosed l) (undisclosed lb))
  (operation nonce-test (displaced 5 0 resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1349)
  (parent 1343)
  (unrealized (0 0) (0 1) (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 lb la rndx) (beta expt) (x y x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (one)))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (4 0)) ((1 2) (0 2)) ((2 1) (1 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec lb))) (y lb)
    (x-0 la) (x-0 beta))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 2 1) (trans 3 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed lb))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1350)
  (parent 1343)
  (unrealized (0 0) (0 1) (0 4) (1 3) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (one)))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 3) (4 0)) ((1 2) (0 2)) ((2 1) (1 0)) ((3 2) (1 1))
    ((4 1) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec lb))) (y lb))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 2 1) (trans 3 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed lb))
  (operation nonce-test
    (contracted (x-0 x) (zeta (one)) (y-0 y) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 1351)
  (parent 1343)
  (unrealized (0 0) (0 1) (0 4) (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb la l y x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha (mul (rec lb) la l)) (zeta (one)))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la la) (x x) (beta l) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 3) (4 0)) ((1 2) (0 2)) ((2 1) (1 0)) ((3 2) (1 1))
    ((4 1) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb la l)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec lb) la l)) (y lb) (x la) (x l))
  (precur (4 0))
  (gen-st (pv a la) (pv b lb))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 2 1) (trans 3 0) (trans 2 0) (neq a b)
    (undisclosed la) (undisclosed lb))
  (operation nonce-test (displaced 5 1 init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul (rec lb) la l)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul la l)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul la l)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a la))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 1352)
  (parent 1343)
  (unrealized (0 0) (0 1) (0 4) (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l lb x y l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha (mul l (rec lb) l-0)) (zeta (one)))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 3) (3 0)) ((1 2) (0 2)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 1) (1 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l (rec lb) l-0)) (y lb))
  (precur (3 0))
  (gen-st (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 4 0) (trans 2 0) (neq self b)
    (undisclosed l-0) (undisclosed lb))
  (operation nonce-test (displaced 2 5 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec lb) l-0)) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0)))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1353)
  (parent 1343)
  (unrealized (0 0) (0 1) (0 4) (1 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 lb lb-0 rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (one)))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (4 0)) ((1 2) (0 2)) ((2 1) (1 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 3) (4 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec lb))) (y lb)
    (y-0 lb-0) (y-0 alpha) (y-0 zeta))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb) (pv b-0 lb-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 2 1) (trans 3 0) (trans 2 0) (neq a b) (undisclosed l)
    (undisclosed lb))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-4 (pv b-0 lb-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb-0 alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 1354)
  (parent 1343)
  (unrealized (0 0) (0 1) (0 4) (1 3) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l lb x y l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha (mul l (rec lb) l-0)) (zeta (one)))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 3) (4 0)) ((1 2) (0 2)) ((2 1) (1 0)) ((3 2) (1 1))
    ((4 1) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l (rec lb) l-0)) (y lb))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 2 1) (trans 3 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed lb))
  (operation nonce-test (displaced 5 3 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l (rec lb) l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 1355)
  (parent 1343)
  (unrealized (0 0) (0 1) (0 4) (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 lb x y l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (one)))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (4 0)) ((1 2) (0 2)) ((2 1) (1 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec lb))) (y lb))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 3 1) (trans 2 1) (trans 5 0) (trans 3 0)
    (trans 2 0) (neq a b) (undisclosed l) (undisclosed lb))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-2 (cat pt-4 ignore-1))
      (stor priv-stor-2 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1356)
  (parent 1343)
  (unrealized (0 0) (0 1) (0 4) (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x y-0 rndx) (zeta-0 expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y-0 zeta-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l) (y y-0) (alpha l-0) (zeta zeta-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (3 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 zeta-0) (y-0 l)
    (y-0 l-0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y-0 zeta-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l-0) (undisclosed lb))
  (operation encryption-test (displaced 2 5 ltx-gen 3)
    (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)) (4 1))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y-0 zeta-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y-0 zeta-0)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y-0 zeta-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))))
  (label 1357)
  (parent 1344)
  (unrealized (0 0) (0 1) (1 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((3 1) (6 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 3 1) (trans 2 1) (trans 3 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed lb))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1358)
  (parent 1346)
  (unrealized (0 0) (0 1) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((2 1) (6 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 3 1) (trans 2 1) (trans 3 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed lb))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1359)
  (parent 1346)
  (unrealized (0 0) (0 1) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta x))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (1 3)) ((1 2) (0 2)) ((2 1) (0 0)) ((2 2) (1 1))
    ((3 1) (1 0)) ((3 2) (0 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y x) (y l) (y l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 3 1)
    (trans 2 1) (trans 3 0) (trans 2 0) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (operation encryption-test (displaced 2 4 ltx-gen 3)
    (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))))
  (label 1360)
  (parent 1347)
  (unrealized (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x rndx)
    (w expt) (y l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta (mul x w)))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 3) (4 0)) ((1 2) (0 2)) ((2 1) (1 0)) ((3 1) (0 0))
    ((3 2) (1 1)) ((4 1) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y (mul x w)) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 3 1) (trans 2 1)
    (trans 3 0) (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation channel-test (displaced 5 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (0 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (recv nb))
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1361)
  (parent 1349)
  (unrealized (0 1) (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x rndx) (w expt)
    (y l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (mul x w)))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (precedes ((0 3) (4 0)) ((1 2) (0 2)) ((2 1) (1 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 1) (0 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y (mul x w)) (y l-1))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 5 1) (trans 3 1)
    (trans 2 1) (trans 5 0) (trans 3 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv b l-1))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1)))))
  (label 1362)
  (parent 1349)
  (unrealized (0 1) (0 4))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l lb rndx) (beta expt)
    (x y x-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha (mul l (rec lb) l-0)) (zeta (one)))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 3) (4 0)) ((1 2) (0 2)) ((2 1) (1 0)) ((2 1) (5 0))
    ((3 2) (1 1)) ((4 1) (1 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l (rec lb) l-0)) (y lb)
    (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 2 1) (trans 3 0) (trans 2 0) (neq a b)
    (undisclosed l-0) (undisclosed lb))
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-4 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l (rec lb) l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l-0))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1363)
  (parent 1350)
  (unrealized (0 0) (0 1) (0 4) (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l lb rndx) (beta expt)
    (x y x-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha (mul l (rec lb) l-0)) (zeta (one)))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 3) (4 0)) ((1 2) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((3 2) (1 1)) ((4 1) (1 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l (rec lb) l-0)) (y lb)
    (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 2 1) (trans 3 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed lb))
  (operation channel-test (displaced 6 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-4 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l (rec lb) l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1364)
  (parent 1350)
  (unrealized (0 0) (0 1) (0 4) (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (l l-0 lb rndx)
    (beta expt) (x y x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (one)))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (4 0)) ((1 2) (0 2)) ((2 1) (1 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec lb))) (y lb)
    (x-0 beta) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 3 1) (trans 2 1) (trans 6 0) (trans 3 0)
    (trans 2 0) (neq a b) (undisclosed l) (undisclosed lb))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-4 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-5 ignore-1))
      (stor priv-stor-2 (cat pt-4 (pv a-0 l-1)))))
  (label 1365)
  (parent 1350)
  (unrealized (0 0) (0 1) (0 4) (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l lb rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha (mul l (rec lb) l-0)) (zeta (one)))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (4 0)) ((1 2) (0 2)) ((2 1) (1 0)) ((2 1) (5 0))
    ((3 2) (1 1)) ((4 1) (1 3)) ((5 3) (4 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l (rec lb) l-0)) (y lb)
    (y-0 alpha) (y-0 zeta) (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 2 1) (trans 3 0) (trans 2 0) (neq a b) (undisclosed l-0)
    (undisclosed lb))
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-4 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l (rec lb) l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l-0))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 1366)
  (parent 1354)
  (unrealized (0 0) (0 1) (0 4) (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l lb rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha (mul l (rec lb) l-0)) (zeta (one)))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (4 0)) ((1 2) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((3 2) (1 1)) ((4 1) (1 3)) ((5 3) (4 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l (rec lb) l-0)) (y lb)
    (y-0 alpha) (y-0 zeta) (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 2 1) (trans 3 0) (trans 2 0) (neq a b) (undisclosed l)
    (undisclosed lb))
  (operation channel-test (displaced 6 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-4 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l (rec lb) l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 1367)
  (parent 1354)
  (unrealized (0 0) (0 1) (0 4) (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (one)))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (4 0)) ((1 2) (0 2)) ((2 1) (1 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 3) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec lb))) (y lb)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 3 1) (trans 2 1) (trans 6 0) (trans 3 0) (trans 2 0)
    (neq a b) (undisclosed l) (undisclosed lb))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-4 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-5 ignore-1))
      (stor priv-stor-2 (cat pt-4 (pv b-0 l-1)))))
  (label 1368)
  (parent 1354)
  (unrealized (0 0) (0 1) (0 4) (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (lb-0 l x y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb-0) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (3 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb-0 l)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x lb-0) (x l) (y-0 lb-0) (y-0 l)
    (y-0 x))
  (gen-st (pv a l) (pv b lb) (pv b lb-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l) (undisclosed lb))
  (operation nonce-test (displaced 5 3 resp 4) (exp (gen) y-1) (1 3))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb-0 l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb-0)))
      (send
        (sig (body b (exp (gen) lb-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb-0 l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))))
  (label 1369)
  (parent 1357)
  (unrealized (0 0) (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x y-0 rndx) (zeta-0 expt) (l l-0 rndx)
    (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y-0 zeta-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l) (y y-0) (alpha l-0) (zeta zeta-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0 zeta-0 (rec w))) w))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (5 0))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 zeta-0) (y-0 l)
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y-0 zeta-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l-0) (undisclosed lb))
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y-0 zeta-0 (rec w))) w))
    (exp (gen) (mul (rec x) y-0 zeta-0)) (1 3))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y-0 zeta-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y-0 zeta-0)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y-0 zeta-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul (rec x) y-0 zeta-0 (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y-0 zeta-0 (rec w))) w))))
  (label 1370)
  (parent 1357)
  (unrealized (0 0) (0 1) (5 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((3 1) (7 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 3 1) (trans 2 1) (trans 3 0) (trans 2 0) (trans 7 1)
    (trans 7 0) (neq a b) (undisclosed l) (undisclosed lb))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (6 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-0))))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-0)))
  (label 1371)
  (parent 1358)
  (unrealized (0 0) (0 1) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y la x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha la) (zeta x))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la la) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l la))
  (precedes ((0 3) (1 3)) ((1 2) (0 2)) ((1 4) (0 4)) ((2 1) (0 0))
    ((2 2) (1 1)) ((3 1) (1 0)) ((3 2) (0 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l la)
  (uniq-gen y x)
  (absent (y l) (y la) (y x) (x l) (x la))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 3 1)
    (trans 2 1) (trans 3 0) (trans 2 0) (neq a b) (undisclosed la)
    (undisclosed l))
  (operation nonce-test (displaced 4 1 init 5) nb (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 y)))))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l la)) (exp (gen) (mul y x))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l la)) (exp (gen) (mul y x))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a la)))
      (send
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a)))))
  (label 1372)
  (parent 1360)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((a a) (b b) (la la) (lb l) (alpha la) (beta l) (y y) (zeta x)
        (na na) (nb nb) (priv-stor priv-stor) (pt pt-0) (x x)
        (upsilon y) (na-0 na) (nb-0 nb) (priv-stor-0 priv-stor-0)
        (pt-0 pt-2))))
  (origs (na (1 2)) (la (3 1)) (pt-2 (3 1)) (l (2 1)) (pt-0 (2 1))
    (nb (0 3))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta x))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (precedes ((0 3) (1 3)) ((0 3) (4 0)) ((1 2) (0 2)) ((2 1) (0 0))
    ((2 2) (1 1)) ((3 1) (1 0)) ((3 2) (0 1)) ((4 1) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y x) (y l) (y l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 3 1)
    (trans 2 1) (trans 3 0) (trans 2 0) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))) nb (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
  (label 1373)
  (parent 1360)
  (unrealized (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (w expt) (y l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (mul x w)))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (3 0)) ((1 2) (0 2)) ((2 1) (0 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (1 0)) ((4 2) (0 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul x w)) (y l) (y l-0))
  (precur (3 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation encryption-test (displaced 2 5 ltx-gen 3)
    (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))))
  (label 1374)
  (parent 1361)
  (unrealized (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (alpha expt)
    (y rndx) (zeta expt) (l x y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y alpha) (y zeta) (y l-0) (x l) (x l-0) (y-0 l) (y-0 x)
    (y-0 l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation channel-test (displaced 5 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (0 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))))
  (label 1375)
  (parent 1369)
  (unrealized (0 1) (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (alpha expt) (y rndx)
    (zeta expt) (lb l x y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0)
  (uniq-gen y x y-0)
  (absent (y alpha) (y zeta) (y l-0) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv b l-0))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0)))))
  (label 1376)
  (parent 1369)
  (unrealized (0 1) (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (precedes ((0 3) (5 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed lb))
  (operation nonce-test (displaced 6 0 resp 4) (exp (gen) y-1) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))))
  (label 1377)
  (parent 1370)
  (unrealized (0 0) (0 1) (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt) (l l-0 la rndx) (beta expt)
    (x y-0 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0))))
  (defstrand init 3 (na na-1) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (5 0))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0)
  (uniq-gen y x y-0 x-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 la) (x-0 beta))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y-0)) (gen)) (neq (gen) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed lb))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0)))))
    ((load priv-stor-2 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-1 a-0 b-0 (exp (gen) x-0)))))
  (label 1378)
  (parent 1370)
  (unrealized (0 0) (0 1) (1 3) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (l l-0 x y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y-0)))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (5 0))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y-0)) (gen)) (neq (gen) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed lb))
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (zeta-0 (one)) (w (mul (rec x) y-0))
      (x-1 x) (y-2 y-0) (w-0 (mul (rec x) y-0))) (gen) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (gen) (mul (rec x) y-0)))
      (send (cat (gen) (mul (rec x) y-0)))))
  (label 1379)
  (parent 1370)
  (unrealized (0 0) (0 1) (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (la l y-0 x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l)
    (upsilon (mul y-0 (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l) (y y-0) (alpha la) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l la))
  (deflistener (cat (exp (gen) x) (mul y-0 (rec x) (rec x))))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (5 0))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 la l)
  (uniq-gen y y-0 x)
  (absent (y lb) (y alpha) (y zeta) (y-0 (one)) (y-0 la) (y-0 l) (x la)
    (x l))
  (precur (5 0))
  (gen-st (pv a la) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul y-0 (rec x))) (gen)) (neq (gen) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed la) (undisclosed lb))
  (operation nonce-test (displaced 6 1 init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x)))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul la l)) (exp (gen) y-0)))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul la l)) (exp (gen) y-0))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a la)))
      (send
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) x) (mul y-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y-0 (rec x) (rec x))))))
  (label 1380)
  (parent 1370)
  (unrealized (0 0) (0 1) (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt) (l l-0 lb-0 rndx)
    (alpha-0 zeta-0 expt) (x y-0 y-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb-0) (y y-1) (alpha alpha-0)
    (zeta zeta-0))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (5 0))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3)) ((6 3) (5 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0)
  (uniq-gen y x y-0 y-1)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0) (y-1 lb-0) (y-1 alpha-0) (y-1 zeta-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l) (pv b-0 lb-0))
  (facts (neq (exp (gen) (mul (rec x) y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (gen) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed lb))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-1) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1)))))
    ((load priv-stor-2 (cat pt-4 (pv b-0 lb-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha-0) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b-0
            (hash (exp (gen) (mul lb-0 alpha-0))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1381)
  (parent 1370)
  (unrealized (0 0) (0 1) (1 3) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (lb-0 l x rndx) (w expt) (y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb-0) (y y-0) (alpha l)
    (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (5 0))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb-0 l)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x lb-0) (x l) (y-0 lb-0) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb) (pv b lb-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed lb))
  (operation nonce-test (displaced 6 3 resp 4) (exp (gen) y-1) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb-0 l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb-0)))
      (send
        (sig (body b (exp (gen) lb-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb-0 l))
              (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 1382)
  (parent 1370)
  (unrealized (0 0) (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (l x y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y-0 (rec l-0))))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (5 0))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y-0)) (gen)) (neq (gen) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l) (undisclosed lb))
  (operation nonce-test (displaced 6 2 ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y-0 (rec l-0))))))
  (label 1383)
  (parent 1370)
  (unrealized (0 0) (0 1) (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (l x y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y-0 (rec l-0))))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (5 0))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y-0)) (gen)) (neq (gen) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed lb))
  (operation nonce-test (displaced 6 4 ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y-0 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y-0 (rec l-0))))))
  (label 1384)
  (parent 1370)
  (unrealized (0 0) (0 1) (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt) (l l-0 x y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (5 0))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y-0)) (gen)) (neq (gen) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 4 1) (trans 2 1)
    (trans 6 0) (trans 4 0) (trans 2 0) (neq a b) (undisclosed l-0)
    (undisclosed lb))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y-0 (rec l-1)))))
    ((load priv-stor-2 (cat pt-4 ignore-1))
      (stor priv-stor-2 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1385)
  (parent 1370)
  (unrealized (0 0) (0 1) (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (1 0))
    ((7 2) (5 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (gen-st (pv b lb) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 7 1) (trans 2 1) (trans 7 0) (trans 2 0) (trans 6 1)
    (trans 6 0) (neq self b) (undisclosed l-0) (undisclosed lb))
  (operation nonce-test (displaced 2 8 ltx-gen 3) (exp (gen) l-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body self (exp (gen) alpha) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1386)
  (parent 1371)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((3 1) (7 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 3 1) (trans 2 1) (trans 3 0) (trans 2 0) (trans 7 1)
    (trans 7 0) (neq a b) (undisclosed l) (undisclosed lb))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (6 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1387)
  (parent 1371)
  (unrealized (0 0) (0 1) (8 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta x))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (precedes ((0 3) (1 3)) ((0 3) (5 0)) ((1 2) (0 2)) ((2 1) (0 0))
    ((2 2) (1 1)) ((3 1) (1 0)) ((3 2) (0 1)) ((4 1) (0 4))
    ((5 1) (4 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y x) (y l) (y l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 3 1)
    (trans 2 1) (trans 3 0) (trans 2 0) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (operation encryption-test
    (added-listener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))) (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
  (label 1388)
  (parent 1373)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l la y x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha la) (zeta (one)))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la la) (x x) (beta l) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) y) (rec x)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l la))
  (precedes ((0 3) (3 0)) ((1 2) (0 2)) ((1 4) (0 4)) ((2 1) (0 0))
    ((2 2) (1 1)) ((3 1) (1 3)) ((4 1) (1 0)) ((4 2) (0 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l la)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y la) (x l) (x la))
  (precur (3 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed la) (undisclosed l))
  (operation nonce-test (displaced 5 1 init 5) nb (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x-0 upsilon)))))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l la)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l la)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a la)))
      (send
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a)))))
  (label 1389)
  (parent 1374)
  (unrealized (1 3) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (w expt) (y l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (mul x w)))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (precedes ((0 3) (3 0)) ((0 3) (5 0)) ((1 2) (0 2)) ((2 1) (0 0))
    ((2 2) (1 1)) ((3 1) (1 3)) ((4 1) (1 0)) ((4 2) (0 1))
    ((5 1) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul x w)) (y l) (y l-0))
  (precur (3 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))) nb (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
  (label 1390)
  (parent 1374)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (x y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y-0) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (0 1)) ((4 2) (3 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0) (y-0 x) (y-0 l)
    (y-0 l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation encryption-test (displaced 5 4 ltx-gen 3)
    (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))))
  (label 1391)
  (parent 1375)
  (unrealized (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 2) (0 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 x)
    (y-0 l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))))
  (label 1392)
  (parent 1375)
  (unrealized (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (lb x y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (0 1)) ((4 2) (3 1)) ((5 1) (0 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-0) (y-0 lb) (y-0 x)
    (y-0 l-0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (operation encryption-test (displaced 6 4 ltx-gen 3)
    (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l)))))
  (label 1393)
  (parent 1376)
  (unrealized (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0)) ((6 2) (0 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 6 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))))
  (label 1394)
  (parent 1376)
  (unrealized (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 data)
    (a b b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (l rndx) (beta expt) (x y-0 x-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0))))
  (defstrand init 3 (na na-1) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 2) (5 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0)
  (uniq-gen y x y-0 x-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 beta) (x-0 l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y-0)) (gen)) (neq (gen) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l) (undisclosed lb))
  (operation channel-test (displaced 7 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-4 (pv a-0 l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 1395)
  (parent 1378)
  (unrealized (0 0) (0 1) (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 data)
    (a b b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (l rndx) (beta expt) (x y-0 x-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-0) (x x-0) (beta beta))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (5 0))
    ((4 1) (1 0)) ((4 1) (6 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 2) (5 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0)
  (uniq-gen y x y-0 x-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 beta) (x-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y-0)) (gen)) (neq (gen) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed lb))
  (operation channel-test (displaced 7 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-4 (pv a-0 l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 1396)
  (parent 1378)
  (unrealized (0 0) (0 1) (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt) (l l-0 rndx) (beta expt)
    (x y-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0))))
  (defstrand init 3 (na na-1) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (5 0))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3)) ((6 2) (5 0))
    ((7 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 beta) (x-0 l-1))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y-0)) (gen)) (neq (gen) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 7 1) (trans 4 1) (trans 2 1)
    (trans 7 0) (trans 4 0) (trans 2 0) (neq a b) (undisclosed l-0)
    (undisclosed lb))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-4 (pv a-0 l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0)))))
    ((load priv-stor-2 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-5 ignore-1))
      (stor priv-stor-2 (cat pt-4 (pv a-0 l-1)))))
  (label 1397)
  (parent 1378)
  (unrealized (0 0) (0 1) (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (l rndx) (alpha-0 zeta-0 expt)
    (x y-0 y-1 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-1) (alpha alpha-0)
    (zeta zeta-0))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 3) (5 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0)
  (uniq-gen y x y-0 y-1)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0) (y-1 alpha-0) (y-1 zeta-0) (y-1 l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (gen) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l) (undisclosed lb))
  (operation channel-test (displaced 7 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-4 (pv b-0 l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha-0) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b
            (hash (exp (gen) (mul alpha-0 l-0))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1398)
  (parent 1381)
  (unrealized (0 0) (0 1) (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (l rndx) (alpha-0 zeta-0 expt)
    (x y-0 y-1 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b a)
    (priv-stor priv-stor-1) (lb l-0) (y y-1) (alpha alpha-0)
    (zeta zeta-0))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (5 0))
    ((4 1) (1 0)) ((4 1) (6 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 3) (5 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0)
  (uniq-gen y x y-0 y-1)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0) (y-1 alpha-0) (y-1 zeta-0) (y-1 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (gen) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed lb))
  (operation channel-test (displaced 7 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-4 (pv b-0 l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha-0) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 a
            (hash (exp (gen) (mul alpha-0 l-0))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1399)
  (parent 1381)
  (unrealized (0 0) (0 1) (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt) (l l-0 rndx) (alpha-0 zeta-0 expt)
    (x y-0 y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-1) (y y-1) (alpha alpha-0)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (5 0))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3)) ((6 3) (5 0))
    ((7 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0) (y-1 alpha-0) (y-1 zeta-0) (y-1 l-1))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (gen) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 7 1) (trans 4 1) (trans 2 1)
    (trans 7 0) (trans 4 0) (trans 2 0) (neq a b) (undisclosed l-0)
    (undisclosed lb))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-4 (pv b-0 l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1)))))
    ((load priv-stor-2 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha-0) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b-0
            (hash (exp (gen) (mul alpha-0 l-1))
              (exp (gen) (mul zeta-0 y-1)))))))
    ((load priv-stor-2 (cat pt-5 ignore-1))
      (stor priv-stor-2 (cat pt-4 (pv b-0 l-1)))))
  (label 1400)
  (parent 1381)
  (unrealized (0 0) (0 1) (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (alpha expt)
    (y rndx) (zeta expt) (l x rndx) (w expt) (y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y alpha) (y zeta) (y l-0) (x l) (x l-0) (y-0 l)
    (y-0 (mul x w)) (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed l-0))
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (0 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 1401)
  (parent 1382)
  (unrealized (0 1) (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (alpha expt) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (5 0))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (0 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0)
  (uniq-gen y x y-0)
  (absent (y alpha) (y zeta) (y l-0) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv b l-0))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0)))))
  (label 1402)
  (parent 1382)
  (unrealized (0 1) (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (alpha expt) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (precedes ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (1 0))
    ((7 2) (5 0)) ((8 1) (0 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y alpha) (y zeta) (y l-1) (x l) (x l-0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 8 1) (trans 7 1) (trans 2 1) (trans 8 0) (trans 7 0)
    (trans 2 0) (trans 6 1) (trans 6 0) (neq self b) (undisclosed l-0)
    (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv b l-1))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) alpha) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1)))))
  (label 1403)
  (parent 1386)
  (unrealized (0 1) (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l l-0 lb rndx) (alpha zeta expt) (y rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (precedes ((0 3) (8 0)) ((1 2) (5 0)) ((2 1) (1 0)) ((3 1) (7 0))
    ((3 2) (1 1)) ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y lb) (y alpha) (y zeta))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 3 1) (trans 2 1) (trans 3 0) (trans 2 0) (trans 7 1)
    (trans 7 0) (neq a b) (undisclosed l-0) (undisclosed lb))
  (operation nonce-test (displaced 9 0 resp 4) (exp (gen) y-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l-0))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y))))))
  (label 1404)
  (parent 1387)
  (unrealized (0 0) (0 1) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt) (x rndx) (upsilon expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-1) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((3 1) (7 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 na-1 l l-0)
  (uniq-gen y x x-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 3 1) (trans 2 1) (trans 3 0) (trans 2 0) (trans 7 1)
    (trans 7 0) (neq a b) (undisclosed l-0) (undisclosed lb))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l-0))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-2 (cat pt-5 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-1 a-0 b-0 (exp (gen) x-0)))))
  (label 1405)
  (parent 1387)
  (unrealized (0 0) (0 1) (6 0) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((2 1) (8 0)) ((3 1) (7 0))
    ((3 2) (1 1)) ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 3 1) (trans 2 1) (trans 3 0) (trans 2 0) (trans 7 1)
    (trans 7 0) (neq a b) (undisclosed l-0) (undisclosed lb))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l-0))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 1406)
  (parent 1387)
  (unrealized (0 0) (0 1) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta upsilon expt) (l l-0 x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((1 2) (8 0)) ((2 1) (1 0)) ((3 1) (7 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 3 1) (trans 2 1) (trans 3 0) (trans 2 0) (trans 7 1)
    (trans 7 0) (neq a b) (undisclosed l) (undisclosed lb))
  (operation nonce-test (displaced 9 1 init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul upsilon x)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 1407)
  (parent 1387)
  (unrealized (0 0) (0 1) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (1 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b lb) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 8 1) (trans 2 1) (trans 8 0) (trans 2 0) (trans 6 1)
    (trans 6 0) (neq self b) (undisclosed l-0) (undisclosed lb))
  (operation nonce-test (displaced 2 9 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body self (exp (gen) alpha) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1408)
  (parent 1387)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt) (x rndx) (upsilon expt)
    (l lb-0 rndx) (alpha-0 zeta-0 expt) (l-0 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb-0) (y y-0) (alpha alpha-0)
    (zeta zeta-0))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((3 1) (7 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-1 l l-0)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 lb-0)
    (y-0 alpha-0) (y-0 zeta-0))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l) (pv b-0 lb-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 2 1) (trans 3 0)
    (trans 2 0) (trans 7 1) (trans 7 0) (neq a b) (undisclosed l-0)
    (undisclosed lb))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l-0))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-2 (cat pt-5 (pv b-0 lb-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha-0) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-0 b-0
            (hash (exp (gen) (mul lb-0 alpha-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1409)
  (parent 1387)
  (unrealized (0 0) (0 1) (6 0) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((3 1) (7 0)) ((3 2) (1 1))
    ((3 2) (8 0)) ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 3 1) (trans 2 1) (trans 3 0) (trans 2 0) (trans 7 1)
    (trans 7 0) (neq a b) (undisclosed l) (undisclosed lb))
  (operation nonce-test (displaced 9 3 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 1410)
  (parent 1387)
  (unrealized (0 0) (0 1) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt) (x rndx) (upsilon expt)
    (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((3 1) (7 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 3 1) (trans 2 1) (trans 9 0) (trans 3 0)
    (trans 2 0) (trans 7 1) (trans 7 0) (neq a b) (undisclosed l-0)
    (undisclosed lb))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l-0))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-2 (cat pt-5 ignore-1))
      (stor priv-stor-2 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1411)
  (parent 1387)
  (unrealized (0 0) (0 1) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta x))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (1 3)) ((0 3) (5 0)) ((1 2) (0 2)) ((2 1) (0 0))
    ((2 2) (1 1)) ((3 1) (1 0)) ((3 1) (6 0)) ((3 2) (0 1))
    ((4 1) (0 4)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y x) (y l) (y l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 3 1)
    (trans 2 1) (trans 3 0) (trans 2 0) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1412)
  (parent 1388)
  (unrealized (5 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta x))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 3) (1 3)) ((0 3) (5 0)) ((1 2) (0 2)) ((2 1) (0 0))
    ((2 1) (6 0)) ((2 2) (1 1)) ((3 1) (1 0)) ((3 2) (0 1))
    ((4 1) (0 4)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y x) (y l) (y l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 3 1)
    (trans 2 1) (trans 3 0) (trans 2 0) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1413)
  (parent 1388)
  (unrealized (5 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l la y x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha la) (zeta (one)))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la la) (x x) (beta l) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) y) (rec x)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l la))
  (deflistener x)
  (precedes ((0 3) (3 0)) ((1 2) (0 2)) ((1 2) (5 0)) ((1 4) (0 4))
    ((2 1) (0 0)) ((2 2) (1 1)) ((3 1) (1 3)) ((4 1) (1 0))
    ((4 2) (0 1)) ((5 1) (3 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l la)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y la) (x l) (x la))
  (precur (3 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed la) (undisclosed l))
  (operation nonce-test (added-listener x) (rec x) (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l la)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l la)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a la)))
      (send
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a))))
    ((recv x) (send x)))
  (label 1414)
  (parent 1389)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (w expt) (y l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (mul x w)))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (precedes ((0 3) (3 0)) ((0 3) (6 0)) ((1 2) (0 2)) ((2 1) (0 0))
    ((2 2) (1 1)) ((3 1) (1 3)) ((4 1) (1 0)) ((4 2) (0 1))
    ((5 1) (0 4)) ((6 1) (5 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul x w)) (y l) (y l-0))
  (precur (3 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
  (label 1415)
  (parent 1390)
  (unrealized (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 la y-0 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta (mul l l-0 (rec la)))
    (upsilon (mul y-0 (rec x-0))))
  (precedes ((0 3) (5 3)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (0 1))
    ((4 2) (3 1)) ((5 2) (0 2)) ((5 4) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-0) (y x) (y l) (y l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 (mul l l-0 (rec la))) (x-0 la))
  (gen-st (pv a l-0) (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 4 1) (trans 2 1) (trans 4 0) (trans 2 0)
    (neq a b) (undisclosed l-0) (undisclosed l))
  (operation nonce-test (added-strand init 5) nb (0 4)
    (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a la)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec la))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb)))
  (label 1416)
  (parent 1391)
  (unrealized (5 0) (5 1) (5 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (x y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y-0) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (5 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (0 1))
    ((4 2) (3 1)) ((5 1) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0) (y-0 x) (y-0 l)
    (y-0 l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) nb (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 1417)
  (parent 1391)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l x y l-0 l-1 la y-0 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta (mul l-0 l-1 (rec la)))
    (upsilon (mul y-0 (rec x-0))))
  (precedes ((0 3) (6 3)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 2) (0 1)) ((6 2) (0 2)) ((6 4) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-0) (y l) (y x) (y l-0) (y-0 (one)) (y-0 l-0)
    (y-0 l-1) (x-0 (mul l-0 l-1 (rec la))) (x-0 la))
  (gen-st (pv a l) (pv a la) (pv b l-0))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 5 1) (trans 4 1) (trans 2 1) (trans 5 0)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-strand init 5) nb (0 4)
    (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-2 (cat pt-5 (pv a la)))
      (recv
        (sig (body b (exp (gen) (mul l-0 l-1 (rec la))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (send nb)))
  (label 1418)
  (parent 1392)
  (unrealized (6 0) (6 1) (6 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 2) (0 1)) ((6 1) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 x)
    (y-0 l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))) nb
    (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
  (label 1419)
  (parent 1392)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (lb x y l l-0 la y-0 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta (mul l l-0 (rec la)))
    (upsilon (mul y-0 (rec x-0))))
  (precedes ((0 3) (6 3)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (0 1)) ((4 2) (3 1))
    ((5 1) (0 0)) ((6 2) (0 2)) ((6 4) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l-0) (y lb) (y x) (y l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 (mul l l-0 (rec la))) (x-0 la))
  (gen-st (pv a l-0) (pv a la) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 5 1) (trans 4 1) (trans 2 1) (trans 5 0)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation nonce-test (added-strand init 5) nb (0 4)
    (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-2 (cat pt-5 (pv a la)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec la))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb)))
  (label 1420)
  (parent 1393)
  (unrealized (6 0) (6 1) (6 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (lb x y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (0 1)) ((4 2) (3 1))
    ((5 1) (0 0)) ((6 1) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-0) (y-0 lb) (y-0 x)
    (y-0 l-0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) nb (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 1421)
  (parent 1393)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (lb l x y l-0 l-1 la y-0 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-3)
    (la la) (x x-0) (beta (mul l-0 l-1 (rec la)))
    (upsilon (mul y-0 (rec x-0))))
  (precedes ((0 3) (7 3)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 2) (0 1)) ((7 2) (0 2)) ((7 4) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l) (y lb) (y l) (y x) (y-0 (one)) (y-0 l-0)
    (y-0 l-1) (x-0 (mul l-0 l-1 (rec la))) (x-0 la))
  (gen-st (pv a l) (pv a la) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 6 1) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 6 0) (trans 5 0) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-strand init 5) nb (0 4)
    (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-3 (cat pt-7 (pv a la)))
      (recv
        (sig (body b (exp (gen) (mul l-0 l-1 (rec la))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (send nb)))
  (label 1422)
  (parent 1394)
  (unrealized (7 0) (7 1) (7 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 2) (0 1)) ((7 1) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 6 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))) nb
    (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
  (label 1423)
  (parent 1394)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (x rndx) (w expt) (y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y-0) (alpha l-0) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (0 1)) ((4 2) (3 1))
    ((5 1) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0) (y-0 (mul x w)) (y-0 l)
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l-0) (undisclosed l))
  (operation encryption-test (displaced 6 4 ltx-gen 3)
    (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 1424)
  (parent 1401)
  (unrealized (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 2) (0 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))))
  (label 1425)
  (parent 1401)
  (unrealized (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (lb x rndx) (w expt) (y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-0)
    (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (5 0))
    ((4 1) (1 0)) ((4 2) (0 1)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-0) (y-0 lb) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation encryption-test (displaced 7 4 ltx-gen 3)
    (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0))
              (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l)))))
  (label 1426)
  (parent 1402)
  (unrealized (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (5 0))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (0 0))
    ((7 2) (0 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 7 1) (trans 6 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 6 0) (trans 4 0) (trans 2 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))))
  (label 1427)
  (parent 1402)
  (unrealized (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (upsilon expt) (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (precedes ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (1 0))
    ((7 2) (0 1)) ((7 2) (5 0)) ((8 1) (0 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 8 1) (trans 7 1) (trans 2 1) (trans 8 0) (trans 7 0)
    (trans 2 0) (trans 6 1) (trans 6 0) (neq self b) (undisclosed l-1)
    (undisclosed l-0))
  (operation encryption-test (displaced 9 7 ltx-gen 3)
    (sig (body self (exp (gen) l-2) (pubk "sig" self))
      (privk "sig" self)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0)))))
  (label 1428)
  (parent 1403)
  (unrealized (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (1 0))
    ((7 2) (5 0)) ((8 1) (0 0)) ((9 2) (0 1)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 8 1) (trans 7 1) (trans 2 1) (trans 9 0)
    (trans 8 0) (trans 7 0) (trans 2 0) (trans 6 1) (trans 6 0)
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body self (exp (gen) l-2) (pubk "sig" self))
      (privk "sig" self)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1429)
  (parent 1403)
  (unrealized (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 data)
    (a b b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l rndx) (beta expt)
    (x-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((2 1) (9 0)) ((3 1) (7 0))
    ((3 2) (1 1)) ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 na-1 l l-0)
  (uniq-gen y x x-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 3 1) (trans 2 1) (trans 3 0) (trans 2 0) (trans 7 1)
    (trans 7 0) (neq a b) (undisclosed l-0) (undisclosed lb))
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-5 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l-0))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 1430)
  (parent 1405)
  (unrealized (0 0) (0 1) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 data)
    (a b b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-1) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-0) (x x-0) (beta beta))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((3 1) (7 0)) ((3 1) (9 0))
    ((3 2) (1 1)) ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 na-1 l l-0)
  (uniq-gen y x x-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 3 1) (trans 2 1) (trans 3 0) (trans 2 0) (trans 7 1)
    (trans 7 0) (neq a b) (undisclosed l) (undisclosed lb))
  (operation channel-test (displaced 10 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-5 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 1431)
  (parent 1405)
  (unrealized (0 0) (0 1) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt) (x rndx) (upsilon expt) (l rndx)
    (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-1) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((3 1) (7 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 na-1 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l) (pv a-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 3 1) (trans 2 1) (trans 10 0) (trans 3 0)
    (trans 2 0) (trans 7 1) (trans 7 0) (neq a b) (undisclosed l-0)
    (undisclosed lb))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-5 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l-0))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-2 (cat pt-5 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-6 ignore-1))
      (stor priv-stor-2 (cat pt-5 (pv a-0 l-1)))))
  (label 1432)
  (parent 1405)
  (unrealized (0 0) (0 1) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (alpha expt) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (precedes ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y alpha) (y zeta) (y l-1) (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 8 1) (trans 2 1) (trans 9 0) (trans 8 0)
    (trans 2 0) (trans 6 1) (trans 6 0) (neq self b) (undisclosed l-0)
    (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv b l-1))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) alpha) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1)))))
  (label 1433)
  (parent 1408)
  (unrealized (0 1) (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l rndx)
    (alpha-0 zeta-0 expt) (y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha-0)
    (zeta zeta-0))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((2 1) (9 0)) ((3 1) (7 0))
    ((3 2) (1 1)) ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-1 l l-0)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 alpha-0)
    (y-0 zeta-0) (y-0 l-0))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 2 1) (trans 3 0)
    (trans 2 0) (trans 7 1) (trans 7 0) (neq a b) (undisclosed l-0)
    (undisclosed lb))
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-5 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l-0))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha-0) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-0 a
            (hash (exp (gen) (mul alpha-0 l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1434)
  (parent 1409)
  (unrealized (0 0) (0 1) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon alpha-0 zeta-0 expt)
    (l y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-0) (y y-0) (alpha alpha-0)
    (zeta zeta-0))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((3 1) (7 0)) ((3 1) (9 0))
    ((3 2) (1 1)) ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-1 l l-0)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 alpha-0)
    (y-0 zeta-0) (y-0 l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 2 1) (trans 3 0)
    (trans 2 0) (trans 7 1) (trans 7 0) (neq a b) (undisclosed l)
    (undisclosed lb))
  (operation channel-test (displaced 10 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-5 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha-0) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-0 b
            (hash (exp (gen) (mul alpha-0 l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1435)
  (parent 1409)
  (unrealized (0 0) (0 1) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt) (x rndx) (upsilon expt) (l rndx)
    (alpha-0 zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-1) (y y-0) (alpha alpha-0)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((3 1) (7 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 3) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-1 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 alpha-0)
    (y-0 zeta-0) (y-0 l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l) (pv b-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 3 1) (trans 2 1)
    (trans 10 0) (trans 3 0) (trans 2 0) (trans 7 1) (trans 7 0)
    (neq a b) (undisclosed l-0) (undisclosed lb))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-5 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l-0))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-2 (cat pt-5 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha-0) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-0 b-0
            (hash (exp (gen) (mul alpha-0 l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-6 ignore-1))
      (stor priv-stor-2 (cat pt-5 (pv b-0 l-1)))))
  (label 1436)
  (parent 1409)
  (unrealized (0 0) (0 1) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (w expt) (y l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (mul x w)))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (3 0)) ((0 3) (6 0)) ((1 2) (0 2)) ((2 1) (0 0))
    ((2 2) (1 1)) ((3 1) (1 3)) ((4 1) (1 0)) ((4 1) (7 0))
    ((4 2) (0 1)) ((5 1) (0 4)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul x w)) (y l) (y l-0))
  (precur (3 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1437)
  (parent 1415)
  (unrealized (6 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (w expt) (y l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (mul x w)))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 3) (3 0)) ((0 3) (6 0)) ((1 2) (0 2)) ((2 1) (0 0))
    ((2 1) (7 0)) ((2 2) (1 1)) ((3 1) (1 3)) ((4 1) (1 0))
    ((4 2) (0 1)) ((5 1) (0 4)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul x w)) (y l) (y l-0))
  (precur (3 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1438)
  (parent 1415)
  (unrealized (6 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y l y-0 x-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta l) (upsilon (mul y-0 (rec x-0))))
  (precedes ((0 3) (5 3)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 1) (5 0))
    ((4 2) (0 1)) ((4 2) (3 1)) ((5 2) (0 2)) ((5 4) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-0) (y x) (y l) (y l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 l) (x-0 l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 4 1) (trans 2 1) (trans 4 0) (trans 2 0)
    (neq a b) (undisclosed l-0) (undisclosed l))
  (operation channel-test (displaced 6 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb)))
  (label 1439)
  (parent 1416)
  (unrealized (5 1) (5 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y-0 (rec x-0))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (5 3)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (0 1))
    ((4 2) (3 1)) ((5 2) (0 2)) ((5 4) (0 4)) ((6 1) (5 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-0) (y x) (y l) (y l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 (mul l l-0 (rec l-1))) (x-0 l-1))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 1440)
  (parent 1416)
  (unrealized (5 1) (5 3))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (x y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y-0) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (0 1))
    ((4 2) (3 1)) ((5 1) (0 4)) ((6 1) (5 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0) (y-0 x) (y-0 l)
    (y-0 l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 1441)
  (parent 1417)
  (unrealized (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn)
    (x y l l-0 y-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-1) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-1) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y-0 (rec x-0))))
  (precedes ((0 3) (6 3)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 1) (6 0))
    ((4 2) (3 1)) ((5 2) (0 1)) ((6 2) (0 2)) ((6 4) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-1) (y x) (y l) (y l-1) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 (mul l l-0 (rec l-1))) (x-0 l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 5 1) (trans 4 1) (trans 2 1) (trans 5 0)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l-1) (undisclosed l))
  (operation channel-test (displaced 7 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-5 (pv a l-2))) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb)))
  (label 1442)
  (parent 1418)
  (unrealized (6 1) (6 3))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x y l-0 y-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta l-0) (upsilon (mul y-0 (rec x-0))))
  (precedes ((0 3) (6 3)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (6 0)) ((5 2) (0 1)) ((6 2) (0 2)) ((6 4) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-0) (y l) (y x) (y l-0) (y-0 (one)) (y-0 l-0)
    (y-0 l-1) (x-0 l-0) (x-0 l-1))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 5 1) (trans 4 1) (trans 2 1) (trans 5 0)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation channel-test (displaced 7 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-5 (pv a l-2))) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (send nb)))
  (label 1443)
  (parent 1418)
  (unrealized (6 1) (6 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l x y l-0 l-1 y-0 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta (mul l-0 l-1 (rec l-2)))
    (upsilon (mul y-0 (rec x-0))))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (6 3)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 2) (0 1)) ((6 2) (0 2)) ((6 4) (0 4)) ((7 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-0) (y l) (y x) (y l-0) (y-0 (one)) (y-0 l-0)
    (y-0 l-1) (x-0 (mul l-0 l-1 (rec l-2))) (x-0 l-2))
  (gen-st (pv a l) (pv a l-2) (pv b l-0))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 7 1) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 7 0) (trans 5 0) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-5 (pv a l-2))) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-2 (cat pt-5 (pv a l-2)))
      (recv
        (sig (body b (exp (gen) (mul l-0 l-1 (rec l-2))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (send nb))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-5 (pv a l-2)))))
  (label 1444)
  (parent 1418)
  (unrealized (6 1) (6 3))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 x)
    (y-0 l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
  (label 1445)
  (parent 1419)
  (unrealized (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn)
    (lb x y l y-0 x-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x-0) (beta l) (upsilon (mul y-0 (rec x-0))))
  (precedes ((0 3) (6 3)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 1) (6 0)) ((4 2) (0 1))
    ((4 2) (3 1)) ((5 1) (0 0)) ((6 2) (0 2)) ((6 4) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l-0) (y lb) (y x) (y l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 l) (x-0 l-0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 5 1) (trans 4 1) (trans 2 1) (trans 5 0)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation channel-test (displaced 7 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-5 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb)))
  (label 1446)
  (parent 1420)
  (unrealized (6 1) (6 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (lb x y l l-0 y-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-2)
    (la l-1) (x x-0) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y-0 (rec x-0))))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (6 3)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (0 1)) ((4 2) (3 1))
    ((5 1) (0 0)) ((6 2) (0 2)) ((6 4) (0 4)) ((7 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l-0) (y lb) (y x) (y l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 (mul l l-0 (rec l-1))) (x-0 l-1))
  (gen-st (pv a l-0) (pv a l-1) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 7 1) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 7 0) (trans 5 0) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l-0) (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-5 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-2 (cat pt-5 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (send nb))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-5 (pv a l-1)))))
  (label 1447)
  (parent 1420)
  (unrealized (6 1) (6 3))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (lb x y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (0 1)) ((4 2) (3 1))
    ((5 1) (0 0)) ((6 1) (0 4)) ((7 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-0) (y-0 lb) (y-0 x)
    (y-0 l-0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 1448)
  (parent 1421)
  (unrealized (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (lb x y l l-0 y-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l-1) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y-0 (rec x-0))))
  (precedes ((0 3) (7 3)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 1) (7 0)) ((4 2) (3 1))
    ((5 1) (0 0)) ((6 2) (0 1)) ((7 2) (0 2)) ((7 4) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l-1) (y lb) (y x) (y l-1) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 (mul l l-0 (rec l-1))) (x-0 l-1))
  (gen-st (pv a l-1) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 6 1) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 6 0) (trans 5 0) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l-1) (undisclosed l))
  (operation channel-test (displaced 8 4 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-7 (pv a l-2))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb)))
  (label 1449)
  (parent 1422)
  (unrealized (7 1) (7 3))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (lb l x y l-0 y-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-2)
    (la l-1) (x x-0) (beta l-0) (upsilon (mul y-0 (rec x-0))))
  (precedes ((0 3) (7 3)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (7 0)) ((6 2) (0 1)) ((7 2) (0 2)) ((7 4) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l) (y lb) (y l) (y x) (y-0 (one)) (y-0 l-0)
    (y-0 l-1) (x-0 l-0) (x-0 l-1))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 6 1) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 6 0) (trans 5 0) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed l-0))
  (operation channel-test (displaced 8 6 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-7 (pv a l-2))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (send nb)))
  (label 1450)
  (parent 1422)
  (unrealized (7 1) (7 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (lb l x y l-0 l-1 y-0 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-3)
    (la l-2) (x x-0) (beta (mul l-0 l-1 (rec l-2)))
    (upsilon (mul y-0 (rec x-0))))
  (defstrand ltx-gen 2 (ignore ignore-3) (self a)
    (priv-stor priv-stor-3) (l l-2))
  (precedes ((0 3) (7 3)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 2) (0 1)) ((7 2) (0 2)) ((7 4) (0 4)) ((8 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0 l-1 l-2)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l) (y lb) (y l) (y x) (y-0 (one)) (y-0 l-0)
    (y-0 l-1) (x-0 (mul l-0 l-1 (rec l-2))) (x-0 l-2))
  (gen-st (pv a l) (pv a l-2) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 8 1) (trans 6 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 8 0) (trans 6 0) (trans 5 0) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-7 (pv a l-2))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-3 (cat pt-7 (pv a l-2)))
      (recv
        (sig (body b (exp (gen) (mul l-0 l-1 (rec l-2))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (send nb))
    ((load priv-stor-3 (cat pt-8 ignore-3))
      (stor priv-stor-3 (cat pt-7 (pv a l-2)))))
  (label 1451)
  (parent 1422)
  (unrealized (7 1) (7 3))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 6 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
  (label 1452)
  (parent 1423)
  (unrealized (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (w expt)
    (y l l-0 la y-0 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta (mul l l-0 (rec la)))
    (upsilon (mul y-0 (rec x-0))))
  (precedes ((0 3) (6 3)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (0 1))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 2) (0 2)) ((6 4) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-0) (y (mul x w)) (y l) (y l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 (mul l l-0 (rec la))) (x-0 la))
  (precur (5 0))
  (gen-st (pv a l-0) (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 4 1) (trans 2 1) (trans 4 0) (trans 2 0)
    (neq a b) (undisclosed l-0) (undisclosed l))
  (operation nonce-test (added-strand init 5) nb (0 4)
    (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-3 (pv a la)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec la))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb)))
  (label 1453)
  (parent 1424)
  (unrealized (6 0) (6 1) (6 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (x rndx) (w expt) (y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y-0) (alpha l-0) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (0 1))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0) (y-0 (mul x w)) (y-0 l)
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l-0) (undisclosed l))
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) nb (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 1454)
  (parent 1424)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (l x rndx)
    (w expt) (y l-0 l-1 la y-0 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta (mul l-0 l-1 (rec la)))
    (upsilon (mul y-0 (rec x-0))))
  (precedes ((0 3) (7 3)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 2) (0 1)) ((7 2) (0 2)) ((7 4) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-0) (y l) (y (mul x w)) (y l-0) (y-0 (one))
    (y-0 l-0) (y-0 l-1) (x-0 (mul l-0 l-1 (rec la))) (x-0 la))
  (precur (5 0))
  (gen-st (pv a l) (pv a la) (pv b l-0))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-strand init 5) nb (0 4)
    (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-2 (cat pt-5 (pv a la)))
      (recv
        (sig (body b (exp (gen) (mul l-0 l-1 (rec la))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (send nb)))
  (label 1455)
  (parent 1425)
  (unrealized (7 0) (7 1) (7 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 2) (0 1)) ((7 1) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))) nb
    (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
  (label 1456)
  (parent 1425)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb x rndx)
    (w expt) (y l l-0 la y-0 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb)
    (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l-0) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta (mul l l-0 (rec la)))
    (upsilon (mul y-0 (rec x-0))))
  (precedes ((0 3) (7 3)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (0 1)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (0 0)) ((7 2) (0 2)) ((7 4) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l-0) (y lb) (y (mul x w)) (y l-0) (y-0 (one))
    (y-0 l) (y-0 l-0) (x-0 (mul l l-0 (rec la))) (x-0 la))
  (precur (5 0))
  (gen-st (pv a l-0) (pv a la) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation nonce-test (added-strand init 5) nb (0 4)
    (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-2 (cat pt-5 (pv a la)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec la))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb)))
  (label 1457)
  (parent 1426)
  (unrealized (7 0) (7 1) (7 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (lb x rndx) (w expt) (y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-0)
    (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (0 1)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (0 0)) ((7 1) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-0) (y-0 lb) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) nb (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0))
              (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 1458)
  (parent 1426)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (lb l x rndx) (w expt) (y l-0 l-1 la y-0 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-3)
    (la la) (x x-0) (beta (mul l-0 l-1 (rec la)))
    (upsilon (mul y-0 (rec x-0))))
  (precedes ((0 3) (8 3)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 2) (0 1)) ((8 2) (0 2)) ((8 4) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)) (y-0 (one)) (y-0 l-0)
    (y-0 l-1) (x-0 (mul l-0 l-1 (rec la))) (x-0 la))
  (precur (5 0))
  (gen-st (pv a l) (pv a la) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 7 1) (trans 6 1) (trans 4 1) (trans 2 1)
    (trans 7 0) (trans 6 0) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-strand init 5) nb (0 4)
    (enc na nb a b (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-3 (cat pt-7 (pv a la)))
      (recv
        (sig (body b (exp (gen) (mul l-0 l-1 (rec la))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (send nb)))
  (label 1459)
  (parent 1427)
  (unrealized (8 0) (8 1) (8 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 2) (0 1)) ((8 1) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 7 1) (trans 6 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 6 0) (trans 4 0) (trans 2 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))) nb
    (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
  (label 1460)
  (parent 1427)
  (unrealized (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x rndx)
    (upsilon expt) (l l-0 l-1 la y x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-2) (la la) (x x-0)
    (beta (mul l-0 l-1 (rec la))) (upsilon (mul y (rec x-0))))
  (precedes ((0 3) (9 3)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (0 1)) ((7 2) (5 0)) ((8 1) (0 0))
    ((9 2) (0 2)) ((9 4) (0 4)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y l-0) (y l-1)
    (x-0 (mul l-0 l-1 (rec la))) (x-0 la))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 7 1) (trans 2 1) (trans 8 0) (trans 7 0) (trans 2 0)
    (trans 6 1) (trans 6 0) (neq self b) (undisclosed l-1)
    (undisclosed l-0))
  (operation nonce-test (added-strand init 5) nb (0 4)
    (enc na nb self b (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-6 (pv self la)))
      (recv
        (sig (body b (exp (gen) (mul l-0 l-1 (rec la))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y)))))
      (send nb)))
  (label 1461)
  (parent 1428)
  (unrealized (9 0) (9 1) (9 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (upsilon expt) (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (9 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (0 1)) ((7 2) (5 0)) ((8 1) (0 0))
    ((9 1) (0 4)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 8 1) (trans 7 1) (trans 2 1) (trans 8 0) (trans 7 0)
    (trans 2 0) (trans 6 1) (trans 6 0) (neq self b) (undisclosed l-1)
    (undisclosed l-0))
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))) nb
    (0 4)
    (enc na nb self b
      (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
  (label 1462)
  (parent 1428)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (x rndx) (upsilon expt) (l l-0 l-1 l-2 la y x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-3) (la la) (x x-0)
    (beta (mul l-1 l-2 (rec la))) (upsilon (mul y (rec x-0))))
  (precedes ((0 3) (10 3)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 2) (0 1))
    ((10 2) (0 2)) ((10 4) (0 4)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l-1) (y l-2)
    (x-0 (mul l-1 l-2 (rec la))) (x-0 la))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 8 1) (trans 7 1) (trans 2 1) (trans 9 0) (trans 8 0)
    (trans 7 0) (trans 2 0) (trans 6 1) (trans 6 0) (neq self b)
    (undisclosed l-0) (undisclosed l-1))
  (operation nonce-test (added-strand init 5) nb (0 4)
    (enc na nb self b (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y))))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-3 (cat pt-8 (pv self la)))
      (recv
        (sig (body b (exp (gen) (mul l-1 l-2 (rec la))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y)))))
      (send nb)))
  (label 1463)
  (parent 1429)
  (unrealized (10 0) (10 1) (10 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (10 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 2) (0 1))
    ((10 1) (0 4)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 8 1) (trans 7 1) (trans 2 1) (trans 9 0)
    (trans 8 0) (trans 7 0) (trans 2 0) (trans 6 1) (trans 6 0)
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))) nb
    (0 4)
    (enc na nb self b
      (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
  (label 1464)
  (parent 1429)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (upsilon expt) (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (precedes ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (1 0)) ((8 2) (0 1)) ((8 2) (7 0)) ((9 1) (0 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 8 1) (trans 2 1) (trans 9 0) (trans 8 0)
    (trans 2 0) (trans 6 1) (trans 6 0) (neq self b) (undisclosed l-1)
    (undisclosed l-0))
  (operation encryption-test (displaced 10 8 ltx-gen 3)
    (sig (body self (exp (gen) l-2) (pubk "sig" self))
      (privk "sig" self)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0)))))
  (label 1465)
  (parent 1433)
  (unrealized (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0)) ((10 2) (0 1)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 9 1) (trans 8 1) (trans 2 1) (trans 10 0)
    (trans 9 0) (trans 8 0) (trans 2 0) (trans 6 1) (trans 6 0)
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body self (exp (gen) l-2) (pubk "sig" self))
      (privk "sig" self)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1466)
  (parent 1433)
  (unrealized (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y y-0 x-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x-0) (beta l-0) (upsilon (mul y-0 (rec x-0))))
  (precedes ((0 3) (5 3)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (5 1)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 1) (5 0)) ((4 2) (0 1)) ((4 2) (3 1)) ((5 2) (0 2))
    ((5 4) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-0) (y x) (y l) (y l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 l) (x-0 l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 4 1) (trans 2 1) (trans 4 0) (trans 2 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation encryption-test (displaced 6 2 ltx-gen 3)
    (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)) (5 1))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb)))
  (label 1467)
  (parent 1439)
  (unrealized (5 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (x y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y-0) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 1) (7 0))
    ((4 2) (0 1)) ((4 2) (3 1)) ((5 1) (0 4)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0) (y-0 x) (y-0 l)
    (y-0 l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1468)
  (parent 1441)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (x y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y-0) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 1) (7 0)) ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (0 1)) ((4 2) (3 1)) ((5 1) (0 4)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0) (y-0 x) (y-0 l)
    (y-0 l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1469)
  (parent 1441)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x y y-0 x-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x-0) (beta l-1) (upsilon (mul y-0 (rec x-0))))
  (precedes ((0 3) (6 3)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (6 1)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (6 0)) ((5 2) (0 1)) ((6 2) (0 2))
    ((6 4) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-1) (y l) (y x) (y l-1) (y-0 (one)) (y-0 l-0)
    (y-0 l-1) (x-0 l-0) (x-0 l-1))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 5 1) (trans 4 1) (trans 2 1) (trans 5 0)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-1))
  (operation encryption-test (displaced 7 2 ltx-gen 3)
    (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)) (6 1))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (send nb)))
  (label 1470)
  (parent 1443)
  (unrealized (6 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (8 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0))
    ((8 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 x)
    (y-0 l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l-1))
    (exp (gen) (mul l-0 l-1)) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1))))
  (label 1471)
  (parent 1445)
  (unrealized (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 1) (8 0)) ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0))
    ((8 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 x)
    (y-0 l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l-1) l-0))
    (exp (gen) (mul l-0 l-1)) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0))))
  (label 1472)
  (parent 1445)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn)
    (lb x y y-0 x-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y-0) (alpha l) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x-0) (beta l-0) (upsilon (mul y-0 (rec x-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 3) (5 3)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 1) (5 0)) ((4 2) (0 1))
    ((4 2) (3 1)) ((5 2) (0 2)) ((5 4) (0 4)) ((6 1) (0 0))
    ((6 2) (5 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l) (y lb) (y x) (y l) (y-0 (one)) (y-0 l) (y-0 l-0)
    (x-0 l) (x-0 l-0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation encryption-test (displaced 5 7 ltx-gen 3)
    (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)) (6 1))
  (traces
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (send nb))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))))
  (label 1473)
  (parent 1446)
  (unrealized (5 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (lb x y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 1) (8 0)) ((4 2) (0 1))
    ((4 2) (3 1)) ((5 1) (0 0)) ((6 1) (0 4)) ((7 1) (6 0))
    ((8 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-0) (y-0 lb) (y-0 x)
    (y-0 l-0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1474)
  (parent 1448)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (lb x y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (0 1)) ((4 2) (3 1))
    ((5 1) (0 0)) ((5 1) (8 0)) ((6 1) (0 4)) ((7 1) (6 0))
    ((8 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-0) (y-0 lb) (y-0 x)
    (y-0 l-0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1475)
  (parent 1448)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (lb l x y y-0 x-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-2)
    (lb l-1) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x-0) (beta l-1) (upsilon (mul y-0 (rec x-0))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self b)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (6 3)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (6 0))
    ((5 2) (0 1)) ((6 2) (0 2)) ((6 4) (0 4)) ((7 1) (0 0))
    ((7 2) (6 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l) (y lb) (y l) (y x) (y-0 (one)) (y-0 l-0)
    (y-0 l-1) (x-0 l-0) (x-0 l-1))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 7 1) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 7 0) (trans 5 0) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed l-1))
  (operation encryption-test (displaced 5 8 ltx-gen 3)
    (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)) (7 1))
  (traces
    ((load priv-stor-2 (cat pt-6 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (send nb))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))))
  (label 1476)
  (parent 1450)
  (unrealized (6 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (9 0)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0))
    ((9 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 6 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l-1))
    (exp (gen) (mul l-0 l-1)) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1))))
  (label 1477)
  (parent 1452)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((5 1) (9 0)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0))
    ((9 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 6 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l-1) l-0))
    (exp (gen) (mul l-0 l-1)) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0))))
  (label 1478)
  (parent 1452)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (w expt) (y l y-0 x-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta l) (upsilon (mul y-0 (rec x-0))))
  (precedes ((0 3) (6 3)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 1) (6 0))
    ((4 2) (0 1)) ((4 2) (3 1)) ((5 1) (1 3)) ((6 2) (0 2))
    ((6 4) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-0) (y (mul x w)) (y l) (y l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 l) (x-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 4 1) (trans 2 1) (trans 4 0) (trans 2 0)
    (neq a b) (undisclosed l-0) (undisclosed l))
  (operation channel-test (displaced 7 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb)))
  (label 1479)
  (parent 1453)
  (unrealized (6 1) (6 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (w expt)
    (y l l-0 y-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y-0 (rec x-0))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (6 3)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (0 1))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 2) (0 2)) ((6 4) (0 4))
    ((7 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-0) (y (mul x w)) (y l) (y l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 (mul l l-0 (rec l-1))) (x-0 l-1))
  (precur (5 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 7 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 1480)
  (parent 1453)
  (unrealized (6 1) (6 3))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (x rndx) (w expt) (y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y-0) (alpha l-0) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (0 1))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (0 4)) ((7 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0) (y-0 (mul x w)) (y-0 l)
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l-0) (undisclosed l))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 1481)
  (parent 1454)
  (unrealized (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (w expt)
    (y l l-0 y-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-1) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-1) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y-0 (rec x-0))))
  (precedes ((0 3) (7 3)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 1) (7 0))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 2) (0 1)) ((7 2) (0 2))
    ((7 4) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-1) (y (mul x w)) (y l) (y l-1) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 (mul l l-0 (rec l-1))) (x-0 l-1))
  (precur (5 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l-1) (undisclosed l))
  (operation channel-test (displaced 8 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-5 (pv a l-2))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb)))
  (label 1482)
  (parent 1455)
  (unrealized (7 1) (7 3))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x rndx) (w expt)
    (y l-0 y-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta l-0) (upsilon (mul y-0 (rec x-0))))
  (precedes ((0 3) (7 3)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (7 0)) ((6 2) (0 1)) ((7 2) (0 2))
    ((7 4) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-0) (y l) (y (mul x w)) (y l-0) (y-0 (one))
    (y-0 l-0) (y-0 l-1) (x-0 l-0) (x-0 l-1))
  (precur (5 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation channel-test (displaced 8 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-5 (pv a l-2))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (send nb)))
  (label 1483)
  (parent 1455)
  (unrealized (7 1) (7 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (l x rndx)
    (w expt) (y l-0 l-1 y-0 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta (mul l-0 l-1 (rec l-2)))
    (upsilon (mul y-0 (rec x-0))))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (7 3)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 2) (0 1)) ((7 2) (0 2)) ((7 4) (0 4))
    ((8 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-0) (y l) (y (mul x w)) (y l-0) (y-0 (one))
    (y-0 l-0) (y-0 l-1) (x-0 (mul l-0 l-1 (rec l-2))) (x-0 l-2))
  (precur (5 0))
  (gen-st (pv a l) (pv a l-2) (pv b l-0))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 8 1) (trans 6 1) (trans 4 1) (trans 2 1)
    (trans 8 0) (trans 6 0) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-5 (pv a l-2))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-2 (cat pt-5 (pv a l-2)))
      (recv
        (sig (body b (exp (gen) (mul l-0 l-1 (rec l-2))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (send nb))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-5 (pv a l-2)))))
  (label 1484)
  (parent 1455)
  (unrealized (7 1) (7 3))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
  (label 1485)
  (parent 1456)
  (unrealized (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb x rndx) (w expt)
    (y l y-0 x-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb)
    (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l-0) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x-0) (beta l) (upsilon (mul y-0 (rec x-0))))
  (precedes ((0 3) (7 3)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 1) (7 0)) ((4 2) (0 1))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (0 0)) ((7 2) (0 2))
    ((7 4) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l-0) (y lb) (y (mul x w)) (y l-0) (y-0 (one))
    (y-0 l) (y-0 l-0) (x-0 l) (x-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation channel-test (displaced 8 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-5 (pv a l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb)))
  (label 1486)
  (parent 1457)
  (unrealized (7 1) (7 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb x rndx)
    (w expt) (y l l-0 y-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb)
    (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l-0) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-2)
    (la l-1) (x x-0) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y-0 (rec x-0))))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (7 3)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (0 1)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (0 0)) ((7 2) (0 2)) ((7 4) (0 4))
    ((8 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l-0) (y lb) (y (mul x w)) (y l-0) (y-0 (one))
    (y-0 l) (y-0 l-0) (x-0 (mul l l-0 (rec l-1))) (x-0 l-1))
  (precur (5 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 8 1) (trans 6 1) (trans 4 1) (trans 2 1)
    (trans 8 0) (trans 6 0) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l-0) (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-5 (pv a l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-2 (cat pt-5 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (send nb))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-5 (pv a l-1)))))
  (label 1487)
  (parent 1457)
  (unrealized (7 1) (7 3))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (lb x rndx) (w expt) (y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-0)
    (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (0 1)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (0 0)) ((7 1) (0 4)) ((8 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-0) (y-0 lb) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0))
              (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 1488)
  (parent 1458)
  (unrealized (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb x rndx)
    (w expt) (y l l-0 y-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta lb)
    (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l-1) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y-0 (rec x-0))))
  (precedes ((0 3) (8 3)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 1) (8 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (0 0)) ((7 2) (0 1)) ((8 2) (0 2))
    ((8 4) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l-1) (y lb) (y (mul x w)) (y l-1) (y-0 (one))
    (y-0 l) (y-0 l-0) (x-0 (mul l l-0 (rec l-1))) (x-0 l-1))
  (precur (5 0))
  (gen-st (pv a l-1) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 7 1) (trans 6 1) (trans 4 1) (trans 2 1)
    (trans 7 0) (trans 6 0) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l-1) (undisclosed l))
  (operation channel-test (displaced 9 4 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-7 (pv a l-2))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb)))
  (label 1489)
  (parent 1459)
  (unrealized (8 1) (8 3))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb l x rndx)
    (w expt) (y l-0 y-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-2)
    (la l-1) (x x-0) (beta l-0) (upsilon (mul y-0 (rec x-0))))
  (precedes ((0 3) (8 3)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (8 0)) ((7 2) (0 1)) ((8 2) (0 2))
    ((8 4) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)) (y-0 (one)) (y-0 l-0)
    (y-0 l-1) (x-0 l-0) (x-0 l-1))
  (precur (5 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 7 1) (trans 6 1) (trans 4 1) (trans 2 1)
    (trans 7 0) (trans 6 0) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed l-0))
  (operation channel-test (displaced 9 7 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-7 (pv a l-2))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (send nb)))
  (label 1490)
  (parent 1459)
  (unrealized (8 1) (8 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (lb l x rndx) (w expt) (y l-0 l-1 y-0 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-3)
    (la l-2) (x x-0) (beta (mul l-0 l-1 (rec l-2)))
    (upsilon (mul y-0 (rec x-0))))
  (defstrand ltx-gen 2 (ignore ignore-3) (self a)
    (priv-stor priv-stor-3) (l l-2))
  (precedes ((0 3) (8 3)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 2) (0 1)) ((8 2) (0 2)) ((8 4) (0 4))
    ((9 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0 l-1 l-2)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)) (y-0 (one)) (y-0 l-0)
    (y-0 l-1) (x-0 (mul l-0 l-1 (rec l-2))) (x-0 l-2))
  (precur (5 0))
  (gen-st (pv a l) (pv a l-2) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 9 1) (trans 7 1) (trans 6 1) (trans 4 1)
    (trans 2 1) (trans 9 0) (trans 7 0) (trans 6 0) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-7 (pv a l-2))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-3 (cat pt-7 (pv a l-2)))
      (recv
        (sig (body b (exp (gen) (mul l-0 l-1 (rec l-2))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (send nb))
    ((load priv-stor-3 (cat pt-8 ignore-3))
      (stor priv-stor-3 (cat pt-7 (pv a l-2)))))
  (label 1491)
  (parent 1459)
  (unrealized (8 1) (8 3))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 2) (0 1)) ((8 1) (0 4)) ((9 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 7 1) (trans 6 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 6 0) (trans 4 0) (trans 2 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
  (label 1492)
  (parent 1460)
  (unrealized (9 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l l-0 y x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x-0) (beta l-0)
    (upsilon (mul y (rec x-0))))
  (precedes ((0 3) (9 3)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 1) (9 0)) ((7 2) (0 1)) ((7 2) (5 0))
    ((8 1) (0 0)) ((9 2) (0 2)) ((9 4) (0 4)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y l-0) (y l-1) (x-0 l-0) (x-0 l-1))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 7 1) (trans 2 1) (trans 8 0) (trans 7 0) (trans 2 0)
    (trans 6 1) (trans 6 0) (neq self b) (undisclosed l-1)
    (undisclosed l-0))
  (operation channel-test (displaced 10 7 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv self l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y)))))
      (send nb)))
  (label 1493)
  (parent 1461)
  (unrealized (9 1) (9 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x rndx)
    (upsilon expt) (l l-0 l-1 y x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-2) (la l-2) (x x-0)
    (beta (mul l-0 l-1 (rec l-2))) (upsilon (mul y (rec x-0))))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (9 3)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (0 1)) ((7 2) (5 0)) ((8 1) (0 0))
    ((9 2) (0 2)) ((9 4) (0 4)) ((10 1) (9 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y l-0) (y l-1)
    (x-0 (mul l-0 l-1 (rec l-2))) (x-0 l-2))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv self l-2))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 8 1) (trans 7 1) (trans 2 1) (trans 10 0) (trans 8 0)
    (trans 7 0) (trans 2 0) (trans 6 1) (trans 6 0) (neq self b)
    (undisclosed l-1) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv self l-2))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-6 (pv self l-2)))
      (recv
        (sig (body b (exp (gen) (mul l-0 l-1 (rec l-2))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv self l-2)))))
  (label 1494)
  (parent 1461)
  (unrealized (9 1) (9 3))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (upsilon expt) (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (10 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (0 1)) ((7 2) (5 0)) ((8 1) (0 0))
    ((9 1) (0 4)) ((10 1) (9 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 8 1) (trans 7 1) (trans 2 1) (trans 8 0) (trans 7 0)
    (trans 2 0) (trans 6 1) (trans 6 0) (neq self b) (undisclosed l-1)
    (undisclosed l-0))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
  (label 1495)
  (parent 1462)
  (unrealized (10 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x rndx)
    (upsilon expt) (l l-0 l-1 y x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-2) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-2) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-2) (x x-0)
    (beta (mul l-0 l-1 (rec l-2))) (upsilon (mul y (rec x-0))))
  (precedes ((0 3) (10 3)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 1) (10 0)) ((7 2) (5 0)) ((8 1) (0 0))
    ((9 2) (0 1)) ((10 2) (0 2)) ((10 4) (0 4)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-2) (y (one)) (y l-0) (y l-1)
    (x-0 (mul l-0 l-1 (rec l-2))) (x-0 l-2))
  (gen-st (pv b l) (pv b l-0) (pv self l-2))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 8 1) (trans 7 1) (trans 2 1) (trans 9 0) (trans 8 0)
    (trans 7 0) (trans 2 0) (trans 6 1) (trans 6 0) (neq self b)
    (undisclosed l-2) (undisclosed l-0))
  (operation channel-test (displaced 11 7 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv self l-3))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-2)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-2) l)) (send (cat (exp (gen) l-2) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 (pv self l-2)))
      (recv
        (sig (body b (exp (gen) (mul l-0 l-1 (rec l-2))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y)))))
      (send nb)))
  (label 1496)
  (parent 1463)
  (unrealized (10 1) (10 3))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x rndx)
    (upsilon expt) (l l-0 l-1 y x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-2) (la l-2) (x x-0) (beta l-1)
    (upsilon (mul y (rec x-0))))
  (precedes ((0 3) (10 3)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (10 0))
    ((9 2) (0 1)) ((10 2) (0 2)) ((10 4) (0 4)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l-1) (y l-2) (x-0 l-1) (x-0 l-2))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 8 1) (trans 7 1) (trans 2 1) (trans 9 0) (trans 8 0)
    (trans 7 0) (trans 2 0) (trans 6 1) (trans 6 0) (neq self b)
    (undisclosed l-0) (undisclosed l-1))
  (operation channel-test (displaced 11 9 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv self l-3))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y)))))
      (send nb)))
  (label 1497)
  (parent 1463)
  (unrealized (10 1) (10 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (x rndx) (upsilon expt) (l l-0 l-1 l-2 y x-0 l-3 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-3) (la l-3) (x x-0)
    (beta (mul l-1 l-2 (rec l-3))) (upsilon (mul y (rec x-0))))
  (defstrand ltx-gen 2 (ignore ignore-3) (self self)
    (priv-stor priv-stor-3) (l l-3))
  (precedes ((0 3) (10 3)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 2) (0 1))
    ((10 2) (0 2)) ((10 4) (0 4)) ((11 1) (10 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1 l-2 l-3)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l-1) (y l-2)
    (x-0 (mul l-1 l-2 (rec l-3))) (x-0 l-3))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-3))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 9 1) (trans 8 1) (trans 7 1) (trans 2 1) (trans 11 0)
    (trans 9 0) (trans 8 0) (trans 7 0) (trans 2 0) (trans 6 1)
    (trans 6 0) (neq self b) (undisclosed l-0) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv self l-3))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-3 (cat pt-8 (pv self l-3)))
      (recv
        (sig (body b (exp (gen) (mul l-1 l-2 (rec l-3))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y))))) (send nb))
    ((load priv-stor-3 (cat pt-9 ignore-3))
      (stor priv-stor-3 (cat pt-8 (pv self l-3)))))
  (label 1498)
  (parent 1463)
  (unrealized (10 1) (10 3))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 2) (0 1))
    ((10 1) (0 4)) ((11 1) (10 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 8 1) (trans 7 1) (trans 2 1) (trans 9 0)
    (trans 8 0) (trans 7 0) (trans 2 0) (trans 6 1) (trans 6 0)
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
  (label 1499)
  (parent 1464)
  (unrealized (11 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x rndx)
    (upsilon expt) (l l-0 l-1 la y x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-2) (la la) (x x-0)
    (beta (mul l-0 l-1 (rec la))) (upsilon (mul y (rec x-0))))
  (precedes ((0 3) (10 3)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (0 1)) ((8 2) (7 0))
    ((9 1) (0 0)) ((10 2) (0 2)) ((10 4) (0 4)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y l-0) (y l-1)
    (x-0 (mul l-0 l-1 (rec la))) (x-0 la))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 8 1) (trans 2 1) (trans 9 0) (trans 8 0) (trans 2 0)
    (trans 6 1) (trans 6 0) (neq self b) (undisclosed l-1)
    (undisclosed l-0))
  (operation nonce-test (added-strand init 5) nb (0 4)
    (enc na nb self b (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-6 (pv self la)))
      (recv
        (sig (body b (exp (gen) (mul l-0 l-1 (rec la))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y)))))
      (send nb)))
  (label 1500)
  (parent 1465)
  (unrealized (10 0) (10 1) (10 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (upsilon expt) (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (10 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (0 1)) ((8 2) (7 0))
    ((9 1) (0 0)) ((10 1) (0 4)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 8 1) (trans 2 1) (trans 9 0) (trans 8 0)
    (trans 2 0) (trans 6 1) (trans 6 0) (neq self b) (undisclosed l-1)
    (undisclosed l-0))
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))) nb
    (0 4)
    (enc na nb self b
      (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
  (label 1501)
  (parent 1465)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (x rndx) (upsilon expt) (l l-0 l-1 l-2 la y x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-3) (la la) (x x-0)
    (beta (mul l-1 l-2 (rec la))) (upsilon (mul y (rec x-0))))
  (precedes ((0 3) (11 3)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 2) (0 1)) ((11 2) (0 2)) ((11 4) (0 4)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l-1) (y l-2)
    (x-0 (mul l-1 l-2 (rec la))) (x-0 la))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 9 1) (trans 8 1) (trans 2 1) (trans 10 0) (trans 9 0)
    (trans 8 0) (trans 2 0) (trans 6 1) (trans 6 0) (neq self b)
    (undisclosed l-0) (undisclosed l-1))
  (operation nonce-test (added-strand init 5) nb (0 4)
    (enc na nb self b (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y))))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-3 (cat pt-8 (pv self la)))
      (recv
        (sig (body b (exp (gen) (mul l-1 l-2 (rec la))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y)))))
      (send nb)))
  (label 1502)
  (parent 1466)
  (unrealized (11 0) (11 1) (11 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 2) (0 1)) ((11 1) (0 4)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 9 1) (trans 8 1) (trans 2 1) (trans 10 0)
    (trans 9 0) (trans 8 0) (trans 2 0) (trans 6 1) (trans 6 0)
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))) nb
    (0 4)
    (enc na nb self b
      (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
  (label 1503)
  (parent 1466)
  (unrealized (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y y-0 x-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x-0) (beta l-0) (upsilon (mul y-0 (rec x-0))))
  (deflistener (cat (exp (gen) y-0) x-0))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (5 1)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 1) (5 0)) ((4 2) (0 1)) ((4 2) (3 1)) ((5 2) (0 2))
    ((5 4) (0 4)) ((6 1) (5 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-0) (y x) (y l) (y l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 l) (x-0 l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 4 1) (trans 2 1) (trans 4 0) (trans 2 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) y-0) x-0))
    (exp (gen) (mul y-0 (rec x-0))) (5 3))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (send nb))
    ((recv (cat (exp (gen) y-0) x-0)) (send (cat (exp (gen) y-0) x-0))))
  (label 1504)
  (parent 1467)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (x y y-0 x-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x-0) (beta l-0) (upsilon (mul y-0 (rec x-0))))
  (deflistener (cat (exp (gen) (rec x-0)) y-0))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (5 1)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 1) (5 0)) ((4 2) (0 1)) ((4 2) (3 1)) ((5 2) (0 2))
    ((5 4) (0 4)) ((6 1) (5 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-0) (y x) (y l) (y l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 l) (x-0 l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 4 1) (trans 2 1) (trans 4 0) (trans 2 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x-0)) y-0))
    (exp (gen) (mul y-0 (rec x-0))) (5 3))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (send nb))
    ((recv (cat (exp (gen) (rec x-0)) y-0))
      (send (cat (exp (gen) (rec x-0)) y-0))))
  (label 1505)
  (parent 1467)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x y y-0 x-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x-0) (beta l-1) (upsilon (mul y-0 (rec x-0))))
  (deflistener (cat (exp (gen) y-0) x-0))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (6 1)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (6 0)) ((5 2) (0 1)) ((6 2) (0 2))
    ((6 4) (0 4)) ((7 1) (6 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-1) (y l) (y x) (y l-1) (y-0 (one)) (y-0 l-0)
    (y-0 l-1) (x-0 l-0) (x-0 l-1))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 5 1) (trans 4 1) (trans 2 1) (trans 5 0)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) y-0) x-0))
    (exp (gen) (mul y-0 (rec x-0))) (6 3))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (send nb))
    ((recv (cat (exp (gen) y-0) x-0)) (send (cat (exp (gen) y-0) x-0))))
  (label 1506)
  (parent 1470)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn)
    (l x y y-0 x-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x-0) (beta l-1) (upsilon (mul y-0 (rec x-0))))
  (deflistener (cat (exp (gen) (rec x-0)) y-0))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (6 1)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (6 0)) ((5 2) (0 1)) ((6 2) (0 2))
    ((6 4) (0 4)) ((7 1) (6 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-1) (y l) (y x) (y l-1) (y-0 (one)) (y-0 l-0)
    (y-0 l-1) (x-0 l-0) (x-0 l-1))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 5 1) (trans 4 1) (trans 2 1) (trans 5 0)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x-0)) y-0))
    (exp (gen) (mul y-0 (rec x-0))) (6 3))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (send nb))
    ((recv (cat (exp (gen) (rec x-0)) y-0))
      (send (cat (exp (gen) (rec x-0)) y-0))))
  (label 1507)
  (parent 1470)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0))
    ((8 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 x)
    (y-0 l-0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 9 1) (trans 9 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (8 0)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 1508)
  (parent 1471)
  (unrealized (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn)
    (lb x y y-0 x-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y-0) (alpha l) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x-0) (beta l-0) (upsilon (mul y-0 (rec x-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) x-0))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 1) (5 0)) ((4 2) (0 1))
    ((4 2) (3 1)) ((5 2) (0 2)) ((5 4) (0 4)) ((6 1) (0 0))
    ((6 2) (5 1)) ((7 1) (5 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l) (y lb) (y x) (y l) (y-0 (one)) (y-0 l) (y-0 l-0)
    (x-0 l) (x-0 l-0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) y-0) x-0))
    (exp (gen) (mul y-0 (rec x-0))) (5 3))
  (traces
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (send nb))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) y-0) x-0)) (send (cat (exp (gen) y-0) x-0))))
  (label 1509)
  (parent 1473)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn)
    (lb x y y-0 x-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y-0) (alpha l) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x-0) (beta l-0) (upsilon (mul y-0 (rec x-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (rec x-0)) y-0))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 1) (5 0)) ((4 2) (0 1))
    ((4 2) (3 1)) ((5 2) (0 2)) ((5 4) (0 4)) ((6 1) (0 0))
    ((6 2) (5 1)) ((7 1) (5 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l) (y lb) (y x) (y l) (y-0 (one)) (y-0 l) (y-0 l-0)
    (x-0 l) (x-0 l-0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x-0)) y-0))
    (exp (gen) (mul y-0 (rec x-0))) (5 3))
  (traces
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (send nb))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (rec x-0)) y-0))
      (send (cat (exp (gen) (rec x-0)) y-0))))
  (label 1510)
  (parent 1473)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (lb l x y y-0 x-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-2)
    (lb l-1) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x-0) (beta l-1) (upsilon (mul y-0 (rec x-0))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self b)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) y-0) x-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (6 0))
    ((5 2) (0 1)) ((6 2) (0 2)) ((6 4) (0 4)) ((7 1) (0 0))
    ((7 2) (6 1)) ((8 1) (6 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l) (y lb) (y l) (y x) (y-0 (one)) (y-0 l-0)
    (y-0 l-1) (x-0 l-0) (x-0 l-1))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 7 1) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 7 0) (trans 5 0) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) y-0) x-0))
    (exp (gen) (mul y-0 (rec x-0))) (6 3))
  (traces
    ((load priv-stor-2 (cat pt-6 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (send nb))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) y-0) x-0)) (send (cat (exp (gen) y-0) x-0))))
  (label 1511)
  (parent 1476)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (lb l x y y-0 x-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-2)
    (lb l-1) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x-0) (beta l-1) (upsilon (mul y-0 (rec x-0))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self b)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) (rec x-0)) y-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (6 0))
    ((5 2) (0 1)) ((6 2) (0 2)) ((6 4) (0 4)) ((7 1) (0 0))
    ((7 2) (6 1)) ((8 1) (6 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l) (y lb) (y l) (y x) (y-0 (one)) (y-0 l-0)
    (y-0 l-1) (x-0 l-0) (x-0 l-1))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (gen) (gen)) (trans 7 1) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 7 0) (trans 5 0) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x-0)) y-0))
    (exp (gen) (mul y-0 (rec x-0))) (6 3))
  (traces
    ((load priv-stor-2 (cat pt-6 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (send nb))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (rec x-0)) y-0))
      (send (cat (exp (gen) (rec x-0)) y-0))))
  (label 1512)
  (parent 1476)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0))
    ((9 1) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 6 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 10 1) (trans 10 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor-2 (cat pt-6 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-1)))
  (label 1513)
  (parent 1477)
  (unrealized (9 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (w expt) (y y-0 x-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x-0) (beta l-0) (upsilon (mul y-0 (rec x-0))))
  (precedes ((0 3) (6 3)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (6 1)) ((3 3) (5 0)) ((4 1) (1 0))
    ((4 1) (6 0)) ((4 2) (0 1)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 2) (0 2)) ((6 4) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-0) (y (mul x w)) (y l) (y l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 l) (x-0 l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 4 1) (trans 2 1) (trans 4 0) (trans 2 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation encryption-test (displaced 7 2 ltx-gen 3)
    (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)) (6 1))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0)))))
      (send nb)))
  (label 1514)
  (parent 1479)
  (unrealized (6 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (x rndx) (w expt) (y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y-0) (alpha l-0) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 1) (8 0))
    ((4 2) (0 1)) ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0) (y-0 (mul x w)) (y-0 l)
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l-0) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1515)
  (parent 1481)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (x rndx) (w expt) (y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y-0) (alpha l-0) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 1) (8 0)) ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0))
    ((4 2) (0 1)) ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0) (y-0 (mul x w)) (y-0 l)
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l-0) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1516)
  (parent 1481)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x rndx) (w expt)
    (y y-0 x-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x-0) (beta l-1) (upsilon (mul y-0 (rec x-0))))
  (precedes ((0 3) (7 3)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (7 1)) ((3 3) (5 0)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (7 0)) ((6 2) (0 1))
    ((7 2) (0 2)) ((7 4) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-1) (y l) (y (mul x w)) (y l-1) (y-0 (one))
    (y-0 l-0) (y-0 l-1) (x-0 l-0) (x-0 l-1))
  (precur (5 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-1))
  (operation encryption-test (displaced 8 2 ltx-gen 3)
    (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)) (7 1))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (send nb)))
  (label 1517)
  (parent 1483)
  (unrealized (7 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (9 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l-1))
    (exp (gen) (mul l-0 l-1)) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1))))
  (label 1518)
  (parent 1485)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 1) (9 0)) ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l-1) l-0))
    (exp (gen) (mul l-0 l-1)) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0))))
  (label 1519)
  (parent 1485)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb x rndx) (w expt)
    (y y-0 x-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y-0) (alpha l) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x-0) (beta l-0) (upsilon (mul y-0 (rec x-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 3) (6 3)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 1) (6 0)) ((4 2) (0 1))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 2) (0 2)) ((6 4) (0 4))
    ((7 1) (0 0)) ((7 2) (6 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l) (y lb) (y (mul x w)) (y l) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 l) (x-0 l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 7 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation encryption-test (displaced 6 8 ltx-gen 3)
    (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)) (7 1))
  (traces
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (send nb))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))))
  (label 1520)
  (parent 1486)
  (unrealized (6 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (lb x rndx) (w expt) (y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-0)
    (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 1) (9 0)) ((4 2) (0 1))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (0 0)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-0) (y-0 lb) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0))
              (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1521)
  (parent 1488)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (lb x rndx) (w expt) (y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-0)
    (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (0 1)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (0 0)) ((6 1) (9 0)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-0) (y-0 lb) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0))
              (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1522)
  (parent 1488)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb l x rndx)
    (w expt) (y y-0 x-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-2)
    (lb l-1) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x-0) (beta l-1) (upsilon (mul y-0 (rec x-0))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self b)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (7 3)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (7 0)) ((6 2) (0 1)) ((7 2) (0 2)) ((7 4) (0 4))
    ((8 1) (0 0)) ((8 2) (7 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)) (y-0 (one)) (y-0 l-0)
    (y-0 l-1) (x-0 l-0) (x-0 l-1))
  (precur (5 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 8 1) (trans 6 1) (trans 4 1) (trans 2 1)
    (trans 8 0) (trans 6 0) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed l-1))
  (operation encryption-test (displaced 6 9 ltx-gen 3)
    (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)) (8 1))
  (traces
    ((load priv-stor-2 (cat pt-6 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (send nb))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))))
  (label 1523)
  (parent 1490)
  (unrealized (7 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (10 0)) ((7 2) (0 1)) ((8 1) (0 4))
    ((9 1) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 7 1) (trans 6 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 6 0) (trans 4 0) (trans 2 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l-1))
    (exp (gen) (mul l-0 l-1)) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1))))
  (label 1524)
  (parent 1492)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((6 1) (10 0)) ((7 2) (0 1)) ((8 1) (0 4))
    ((9 1) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 7 1) (trans 6 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 6 0) (trans 4 0) (trans 2 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l-1) l-0))
    (exp (gen) (mul l-0 l-1)) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0))))
  (label 1525)
  (parent 1492)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l y x-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x-0) (beta l-1)
    (upsilon (mul y (rec x-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (8 3)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 1) (8 0)) ((7 2) (0 1)) ((7 2) (5 0))
    ((8 2) (0 2)) ((8 4) (0 4)) ((9 1) (0 0)) ((9 2) (8 1)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l-0) (y l-1) (x-0 l-0) (x-0 l-1))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 7 1) (trans 2 1) (trans 9 0) (trans 7 0) (trans 2 0)
    (trans 6 1) (trans 6 0) (neq self b) (undisclosed l-0)
    (undisclosed l-1))
  (operation encryption-test (displaced 8 10 ltx-gen 3)
    (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)) (9 1))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))))
  (label 1526)
  (parent 1493)
  (unrealized (8 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (upsilon expt) (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (precedes ((0 3) (10 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 1) (11 0)) ((7 2) (0 1)) ((7 2) (5 0))
    ((8 1) (0 0)) ((9 1) (0 4)) ((10 1) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 8 1) (trans 7 1) (trans 2 1) (trans 8 0) (trans 7 0)
    (trans 2 0) (trans 6 1) (trans 6 0) (neq self b) (undisclosed l-1)
    (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l-1))
    (exp (gen) (mul l-0 l-1)) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1))))
  (label 1527)
  (parent 1495)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (upsilon expt) (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (precedes ((0 3) (10 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (0 1)) ((7 2) (5 0)) ((8 1) (0 0))
    ((8 1) (11 0)) ((9 1) (0 4)) ((10 1) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 8 1) (trans 7 1) (trans 2 1) (trans 8 0) (trans 7 0)
    (trans 2 0) (trans 6 1) (trans 6 0) (neq self b) (undisclosed l-1)
    (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l-1) l-0))
    (exp (gen) (mul l-0 l-1)) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0))))
  (label 1528)
  (parent 1495)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x rndx)
    (upsilon expt) (l l-0 y x-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x-0) (beta l-2)
    (upsilon (mul y (rec x-0))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self b)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (9 3)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (9 0)) ((8 2) (0 1))
    ((9 2) (0 2)) ((9 4) (0 4)) ((10 1) (0 0)) ((10 2) (9 1)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l-1) (y l-2) (x-0 l-1) (x-0 l-2))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 8 1) (trans 7 1) (trans 2 1) (trans 10 0) (trans 8 0)
    (trans 7 0) (trans 2 0) (trans 6 1) (trans 6 0) (neq self b)
    (undisclosed l-0) (undisclosed l-2))
  (operation encryption-test (displaced 8 11 ltx-gen 3)
    (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))
    (10 1))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y))))) (send nb))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))))
  (label 1529)
  (parent 1497)
  (unrealized (9 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (12 0))
    ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 8 1) (trans 7 1) (trans 2 1) (trans 9 0)
    (trans 8 0) (trans 7 0) (trans 2 0) (trans 6 1) (trans 6 0)
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) l-1) l-2))
    (exp (gen) (mul l-1 l-2)) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2))))
  (label 1530)
  (parent 1499)
  (unrealized (12 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((8 1) (12 0))
    ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 8 1) (trans 7 1) (trans 2 1) (trans 9 0)
    (trans 8 0) (trans 7 0) (trans 2 0) (trans 6 1) (trans 6 0)
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) l-2) l-1))
    (exp (gen) (mul l-1 l-2)) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1))))
  (label 1531)
  (parent 1499)
  (unrealized (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l l-0 y x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x-0) (beta l-0)
    (upsilon (mul y (rec x-0))))
  (precedes ((0 3) (10 3)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 1) (10 0)) ((8 2) (0 1))
    ((8 2) (7 0)) ((9 1) (0 0)) ((10 2) (0 2)) ((10 4) (0 4)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y l-0) (y l-1) (x-0 l-0) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 8 1) (trans 2 1) (trans 9 0) (trans 8 0) (trans 2 0)
    (trans 6 1) (trans 6 0) (neq self b) (undisclosed l-1)
    (undisclosed l-0))
  (operation channel-test (displaced 11 8 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv self l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y)))))
      (send nb)))
  (label 1532)
  (parent 1500)
  (unrealized (10 1) (10 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x rndx)
    (upsilon expt) (l l-0 l-1 y x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-2) (la l-2) (x x-0)
    (beta (mul l-0 l-1 (rec l-2))) (upsilon (mul y (rec x-0))))
  (defstrand ltx-gen 2 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (10 3)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (0 1)) ((8 2) (7 0))
    ((9 1) (0 0)) ((10 2) (0 2)) ((10 4) (0 4)) ((11 1) (10 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y l-0) (y l-1)
    (x-0 (mul l-0 l-1 (rec l-2))) (x-0 l-2))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1) (pv self l-2))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 9 1) (trans 8 1) (trans 2 1) (trans 11 0) (trans 9 0)
    (trans 8 0) (trans 2 0) (trans 6 1) (trans 6 0) (neq self b)
    (undisclosed l-1) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv self l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-6 (pv self l-2)))
      (recv
        (sig (body b (exp (gen) (mul l-0 l-1 (rec l-2))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv self l-2)))))
  (label 1533)
  (parent 1500)
  (unrealized (10 1) (10 3))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (upsilon expt) (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (0 1)) ((8 2) (7 0))
    ((9 1) (0 0)) ((10 1) (0 4)) ((11 1) (10 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 8 1) (trans 2 1) (trans 9 0) (trans 8 0)
    (trans 2 0) (trans 6 1) (trans 6 0) (neq self b) (undisclosed l-1)
    (undisclosed l-0))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
  (label 1534)
  (parent 1501)
  (unrealized (11 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x rndx)
    (upsilon expt) (l l-0 l-1 y x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-2) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-2) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-2) (x x-0)
    (beta (mul l-0 l-1 (rec l-2))) (upsilon (mul y (rec x-0))))
  (precedes ((0 3) (11 3)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 1) (11 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((10 2) (0 1)) ((11 2) (0 2)) ((11 4) (0 4)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-2) (y (one)) (y l-0) (y l-1)
    (x-0 (mul l-0 l-1 (rec l-2))) (x-0 l-2))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-2))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 9 1) (trans 8 1) (trans 2 1) (trans 10 0) (trans 9 0)
    (trans 8 0) (trans 2 0) (trans 6 1) (trans 6 0) (neq self b)
    (undisclosed l-2) (undisclosed l-0))
  (operation channel-test (displaced 12 8 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv self l-3))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-2)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-2) l)) (send (cat (exp (gen) l-2) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 (pv self l-2)))
      (recv
        (sig (body b (exp (gen) (mul l-0 l-1 (rec l-2))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y)))))
      (send nb)))
  (label 1535)
  (parent 1502)
  (unrealized (11 1) (11 3))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x rndx)
    (upsilon expt) (l l-0 l-1 y x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-2) (la l-2) (x x-0) (beta l-1)
    (upsilon (mul y (rec x-0))))
  (precedes ((0 3) (11 3)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (11 0)) ((10 2) (0 1)) ((11 2) (0 2)) ((11 4) (0 4)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l-1) (y l-2) (x-0 l-1) (x-0 l-2))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 9 1) (trans 8 1) (trans 2 1) (trans 10 0) (trans 9 0)
    (trans 8 0) (trans 2 0) (trans 6 1) (trans 6 0) (neq self b)
    (undisclosed l-0) (undisclosed l-1))
  (operation channel-test (displaced 12 10 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv self l-3))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y)))))
      (send nb)))
  (label 1536)
  (parent 1502)
  (unrealized (11 1) (11 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (x rndx) (upsilon expt) (l l-0 l-1 l-2 y x-0 l-3 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-3) (la l-3) (x x-0)
    (beta (mul l-1 l-2 (rec l-3))) (upsilon (mul y (rec x-0))))
  (defstrand ltx-gen 2 (ignore ignore-3) (self self)
    (priv-stor priv-stor-3) (l l-3))
  (precedes ((0 3) (11 3)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 2) (0 1)) ((11 2) (0 2)) ((11 4) (0 4)) ((12 1) (11 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1 l-2 l-3)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l-1) (y l-2)
    (x-0 (mul l-1 l-2 (rec l-3))) (x-0 l-3))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-3))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 12 1)
    (trans 10 1) (trans 9 1) (trans 8 1) (trans 2 1) (trans 12 0)
    (trans 10 0) (trans 9 0) (trans 8 0) (trans 2 0) (trans 6 1)
    (trans 6 0) (neq self b) (undisclosed l-0) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv self l-3))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-3 (cat pt-8 (pv self l-3)))
      (recv
        (sig (body b (exp (gen) (mul l-1 l-2 (rec l-3))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y))))) (send nb))
    ((load priv-stor-3 (cat pt-9 ignore-3))
      (stor priv-stor-3 (cat pt-8 (pv self l-3)))))
  (label 1537)
  (parent 1502)
  (unrealized (11 1) (11 3))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 2) (0 1)) ((11 1) (0 4)) ((12 1) (11 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 9 1) (trans 8 1) (trans 2 1) (trans 10 0)
    (trans 9 0) (trans 8 0) (trans 2 0) (trans 6 1) (trans 6 0)
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
  (label 1538)
  (parent 1503)
  (unrealized (12 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (8 0)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 9 1) (trans 9 0)
    (neq a b) (undisclosed l) (undisclosed l-1))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-1)
    (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0)))
  (label 1539)
  (parent 1508)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx) (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec w))) w))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 x)
    (y-0 l-0))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 9 1) (trans 9 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec w))) w))
    (exp (gen) l-0) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l-0 (rec w))) w))
      (send (cat (exp (gen) (mul l-0 (rec w))) w))))
  (label 1540)
  (parent 1508)
  (unrealized (10 0))
  (comment "10 in cohort - 10 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (9 0))
    ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 0)) ((10 2) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 10 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 9 1) (trans 9 0) (neq a self) (undisclosed l)
    (undisclosed l-1))
  (operation nonce-test (displaced 5 11 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1541)
  (parent 1513)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx) (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec w))) w))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0))
    ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 6 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 10 1) (trans 10 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec w))) w))
    (exp (gen) l-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l-0 (rec w))) w))
      (send (cat (exp (gen) (mul l-0 (rec w))) w))))
  (label 1542)
  (parent 1513)
  (unrealized (11 0))
  (comment "11 in cohort - 11 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (w expt) (y y-0 x-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x-0) (beta l-0) (upsilon (mul y-0 (rec x-0))))
  (deflistener (cat (exp (gen) y-0) x-0))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (6 1)) ((3 3) (5 0)) ((4 1) (1 0))
    ((4 1) (6 0)) ((4 2) (0 1)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 2) (0 2)) ((6 4) (0 4)) ((7 1) (6 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-0) (y (mul x w)) (y l) (y l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 l) (x-0 l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 4 1) (trans 2 1) (trans 4 0) (trans 2 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) y-0) x-0))
    (exp (gen) (mul y-0 (rec x-0))) (6 3))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (send nb))
    ((recv (cat (exp (gen) y-0) x-0)) (send (cat (exp (gen) y-0) x-0))))
  (label 1543)
  (parent 1514)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (w expt) (y y-0 x-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x-0) (beta l-0) (upsilon (mul y-0 (rec x-0))))
  (deflistener (cat (exp (gen) (rec x-0)) y-0))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (6 1)) ((3 3) (5 0)) ((4 1) (1 0))
    ((4 1) (6 0)) ((4 2) (0 1)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 2) (0 2)) ((6 4) (0 4)) ((7 1) (6 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-0) (y (mul x w)) (y l) (y l-0) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 l) (x-0 l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 4 1) (trans 2 1) (trans 4 0) (trans 2 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x-0)) y-0))
    (exp (gen) (mul y-0 (rec x-0))) (6 3))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (send nb))
    ((recv (cat (exp (gen) (rec x-0)) y-0))
      (send (cat (exp (gen) (rec x-0)) y-0))))
  (label 1544)
  (parent 1514)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x rndx) (w expt)
    (y y-0 x-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x-0) (beta l-1) (upsilon (mul y-0 (rec x-0))))
  (deflistener (cat (exp (gen) y-0) x-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (7 1)) ((3 3) (5 0)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (7 0)) ((6 2) (0 1))
    ((7 2) (0 2)) ((7 4) (0 4)) ((8 1) (7 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-1) (y l) (y (mul x w)) (y l-1) (y-0 (one))
    (y-0 l-0) (y-0 l-1) (x-0 l-0) (x-0 l-1))
  (precur (5 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) y-0) x-0))
    (exp (gen) (mul y-0 (rec x-0))) (7 3))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (send nb))
    ((recv (cat (exp (gen) y-0) x-0)) (send (cat (exp (gen) y-0) x-0))))
  (label 1545)
  (parent 1517)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x rndx) (w expt)
    (y y-0 x-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x-0) (beta l-1) (upsilon (mul y-0 (rec x-0))))
  (deflistener (cat (exp (gen) (rec x-0)) y-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (7 1)) ((3 3) (5 0)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (7 0)) ((6 2) (0 1))
    ((7 2) (0 2)) ((7 4) (0 4)) ((8 1) (7 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x l) (x l-1) (y l) (y (mul x w)) (y l-1) (y-0 (one))
    (y-0 l-0) (y-0 l-1) (x-0 l-0) (x-0 l-1))
  (precur (5 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x-0)) y-0))
    (exp (gen) (mul y-0 (rec x-0))) (7 3))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (send nb))
    ((recv (cat (exp (gen) (rec x-0)) y-0))
      (send (cat (exp (gen) (rec x-0)) y-0))))
  (label 1546)
  (parent 1517)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (trans 10 1) (trans 10 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 1547)
  (parent 1518)
  (unrealized (9 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb x rndx) (w expt)
    (y y-0 x-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y-0) (alpha l) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x-0) (beta l-0) (upsilon (mul y-0 (rec x-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) x-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 1) (6 0)) ((4 2) (0 1))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 2) (0 2)) ((6 4) (0 4))
    ((7 1) (0 0)) ((7 2) (6 1)) ((8 1) (6 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l) (y lb) (y (mul x w)) (y l) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 l) (x-0 l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 7 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) y-0) x-0))
    (exp (gen) (mul y-0 (rec x-0))) (6 3))
  (traces
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (send nb))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) y-0) x-0)) (send (cat (exp (gen) y-0) x-0))))
  (label 1548)
  (parent 1520)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb x rndx) (w expt)
    (y y-0 x-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y-0) (alpha l) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x-0) (beta l-0) (upsilon (mul y-0 (rec x-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (rec x-0)) y-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 1) (6 0)) ((4 2) (0 1))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 2) (0 2)) ((6 4) (0 4))
    ((7 1) (0 0)) ((7 2) (6 1)) ((8 1) (6 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l) (y lb) (y (mul x w)) (y l) (y-0 (one)) (y-0 l)
    (y-0 l-0) (x-0 l) (x-0 l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 7 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 4 0) (trans 2 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x-0)) y-0))
    (exp (gen) (mul y-0 (rec x-0))) (6 3))
  (traces
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y-0))))) (send nb))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (rec x-0)) y-0))
      (send (cat (exp (gen) (rec x-0)) y-0))))
  (label 1549)
  (parent 1520)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb l x rndx)
    (w expt) (y y-0 x-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-2)
    (lb l-1) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x-0) (beta l-1) (upsilon (mul y-0 (rec x-0))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self b)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) y-0) x-0))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (7 0)) ((6 2) (0 1)) ((7 2) (0 2)) ((7 4) (0 4))
    ((8 1) (0 0)) ((8 2) (7 1)) ((9 1) (7 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)) (y-0 (one)) (y-0 l-0)
    (y-0 l-1) (x-0 l-0) (x-0 l-1))
  (precur (5 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 8 1) (trans 6 1) (trans 4 1) (trans 2 1)
    (trans 8 0) (trans 6 0) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) y-0) x-0))
    (exp (gen) (mul y-0 (rec x-0))) (7 3))
  (traces
    ((load priv-stor-2 (cat pt-6 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (send nb))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) y-0) x-0)) (send (cat (exp (gen) y-0) x-0))))
  (label 1550)
  (parent 1523)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb l x rndx)
    (w expt) (y y-0 x-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-2)
    (lb l-1) (y y-0) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x-0) (beta l-1) (upsilon (mul y-0 (rec x-0))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self b)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) (rec x-0)) y-0))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (7 0)) ((6 2) (0 1)) ((7 2) (0 2)) ((7 4) (0 4))
    ((8 1) (0 0)) ((8 2) (7 1)) ((9 1) (7 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen x y y-0 x-0)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)) (y-0 (one)) (y-0 l-0)
    (y-0 l-1) (x-0 l-0) (x-0 l-1))
  (precur (5 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1))
  (facts (neq (exp (gen) (mul y-0 (rec x-0))) (gen))
    (neq (exp (gen) (mul w y)) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (gen) (gen)) (trans 8 1) (trans 6 1) (trans 4 1) (trans 2 1)
    (trans 8 0) (trans 6 0) (trans 4 0) (trans 2 0) (neq a b)
    (undisclosed l) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x-0)) y-0))
    (exp (gen) (mul y-0 (rec x-0))) (7 3))
  (traces
    ((load priv-stor-2 (cat pt-6 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y-0 (rec x-0)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y-0)))))
      (send nb))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (rec x-0)) y-0))
      (send (cat (exp (gen) (rec x-0)) y-0))))
  (label 1551)
  (parent 1523)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1)) ((8 1) (0 4))
    ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 7 1) (trans 6 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 6 0) (trans 4 0) (trans 2 0) (trans 11 1) (trans 11 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-2 (cat pt-6 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-1)))
  (label 1552)
  (parent 1524)
  (unrealized (10 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l y x-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x-0) (beta l-1)
    (upsilon (mul y (rec x-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) x-0))
  (precedes ((0 3) (10 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 1) (8 0)) ((7 2) (0 1)) ((7 2) (5 0))
    ((8 2) (0 2)) ((8 4) (0 4)) ((9 1) (0 0)) ((9 2) (8 1))
    ((10 1) (8 3)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l-0) (y l-1) (x-0 l-0) (x-0 l-1))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 7 1) (trans 2 1) (trans 9 0) (trans 7 0) (trans 2 0)
    (trans 6 1) (trans 6 0) (neq self b) (undisclosed l-0)
    (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) y) x-0))
    (exp (gen) (mul y (rec x-0))) (8 3))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) y) x-0)) (send (cat (exp (gen) y) x-0))))
  (label 1553)
  (parent 1526)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l y x-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x-0) (beta l-1)
    (upsilon (mul y (rec x-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x-0)) y))
  (precedes ((0 3) (10 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 1) (8 0)) ((7 2) (0 1)) ((7 2) (5 0))
    ((8 2) (0 2)) ((8 4) (0 4)) ((9 1) (0 0)) ((9 2) (8 1))
    ((10 1) (8 3)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l-0) (y l-1) (x-0 l-0) (x-0 l-1))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 7 1) (trans 2 1) (trans 9 0) (trans 7 0) (trans 2 0)
    (trans 6 1) (trans 6 0) (neq self b) (undisclosed l-0)
    (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x-0)) y))
    (exp (gen) (mul y (rec x-0))) (8 3))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (rec x-0)) y))
      (send (cat (exp (gen) (rec x-0)) y))))
  (label 1554)
  (parent 1526)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x rndx)
    (upsilon expt) (l l-0 y x-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x-0) (beta l-2)
    (upsilon (mul y (rec x-0))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self b)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) y) x-0))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (9 0)) ((8 2) (0 1))
    ((9 2) (0 2)) ((9 4) (0 4)) ((10 1) (0 0)) ((10 2) (9 1))
    ((11 1) (9 3)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l-1) (y l-2) (x-0 l-1) (x-0 l-2))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 8 1) (trans 7 1) (trans 2 1) (trans 10 0) (trans 8 0)
    (trans 7 0) (trans 2 0) (trans 6 1) (trans 6 0) (neq self b)
    (undisclosed l-0) (undisclosed l-2))
  (operation nonce-test (added-listener (cat (exp (gen) y) x-0))
    (exp (gen) (mul y (rec x-0))) (9 3))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y))))) (send nb))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) y) x-0)) (send (cat (exp (gen) y) x-0))))
  (label 1555)
  (parent 1529)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x rndx)
    (upsilon expt) (l l-0 y x-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x-0) (beta l-2)
    (upsilon (mul y (rec x-0))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self b)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (rec x-0)) y))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (9 0)) ((8 2) (0 1))
    ((9 2) (0 2)) ((9 4) (0 4)) ((10 1) (0 0)) ((10 2) (9 1))
    ((11 1) (9 3)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l-1) (y l-2) (x-0 l-1) (x-0 l-2))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 8 1) (trans 7 1) (trans 2 1) (trans 10 0) (trans 8 0)
    (trans 7 0) (trans 2 0) (trans 6 1) (trans 6 0) (neq self b)
    (undisclosed l-0) (undisclosed l-2))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x-0)) y))
    (exp (gen) (mul y (rec x-0))) (9 3))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y))))) (send nb))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (rec x-0)) y))
      (send (cat (exp (gen) (rec x-0)) y))))
  (label 1556)
  (parent 1529)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0)) ((12 1) (11 0))
    ((13 2) (12 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 8 1) (trans 7 1) (trans 2 1) (trans 9 0)
    (trans 8 0) (trans 7 0) (trans 2 0) (trans 13 1) (trans 6 1)
    (trans 13 0) (trans 6 0) (neq self b) (undisclosed l-0)
    (undisclosed l-1))
  (operation nonce-test (added-strand ltx-disclose 3) l-2 (12 0)
    (ch-msg priv-stor-2 (cat pt-7 (pv self l-2))))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-2)))
  (label 1557)
  (parent 1530)
  (unrealized (12 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l y x-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x-0) (beta l-1)
    (upsilon (mul y (rec x-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (9 3)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 1) (9 0)) ((8 2) (0 1))
    ((8 2) (7 0)) ((9 2) (0 2)) ((9 4) (0 4)) ((10 1) (0 0))
    ((10 2) (9 1)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l-0) (y l-1) (x-0 l-0) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 8 1) (trans 2 1) (trans 10 0) (trans 8 0) (trans 2 0)
    (trans 6 1) (trans 6 0) (neq self b) (undisclosed l-0)
    (undisclosed l-1))
  (operation encryption-test (displaced 9 11 ltx-gen 3)
    (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))
    (10 1))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))))
  (label 1558)
  (parent 1532)
  (unrealized (9 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (upsilon expt) (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 1) (12 0)) ((8 2) (0 1))
    ((8 2) (7 0)) ((9 1) (0 0)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 8 1) (trans 2 1) (trans 9 0) (trans 8 0)
    (trans 2 0) (trans 6 1) (trans 6 0) (neq self b) (undisclosed l-1)
    (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l-1))
    (exp (gen) (mul l-0 l-1)) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1))))
  (label 1559)
  (parent 1534)
  (unrealized (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (upsilon expt) (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (0 1)) ((8 2) (7 0))
    ((9 1) (0 0)) ((9 1) (12 0)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 8 1) (trans 2 1) (trans 9 0) (trans 8 0)
    (trans 2 0) (trans 6 1) (trans 6 0) (neq self b) (undisclosed l-1)
    (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l-1) l-0))
    (exp (gen) (mul l-0 l-1)) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0))))
  (label 1560)
  (parent 1534)
  (unrealized (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x rndx)
    (upsilon expt) (l l-0 y x-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x-0) (beta l-2)
    (upsilon (mul y (rec x-0))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self b)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (10 3)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (10 0))
    ((9 2) (0 1)) ((10 2) (0 2)) ((10 4) (0 4)) ((11 1) (0 0))
    ((11 2) (10 1)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l-1) (y l-2) (x-0 l-1) (x-0 l-2))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 9 1) (trans 8 1) (trans 2 1) (trans 11 0) (trans 9 0)
    (trans 8 0) (trans 2 0) (trans 6 1) (trans 6 0) (neq self b)
    (undisclosed l-0) (undisclosed l-2))
  (operation encryption-test (displaced 9 12 ltx-gen 3)
    (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))
    (11 1))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y))))) (send nb))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))))
  (label 1561)
  (parent 1536)
  (unrealized (10 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (13 0)) ((10 2) (0 1)) ((11 1) (0 4)) ((12 1) (11 0))
    ((13 1) (12 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 9 1) (trans 8 1) (trans 2 1) (trans 10 0)
    (trans 9 0) (trans 8 0) (trans 2 0) (trans 6 1) (trans 6 0)
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) l-1) l-2))
    (exp (gen) (mul l-1 l-2)) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2))))
  (label 1562)
  (parent 1538)
  (unrealized (13 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 1) (13 0)) ((10 2) (0 1)) ((11 1) (0 4)) ((12 1) (11 0))
    ((13 1) (12 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 9 1) (trans 8 1) (trans 2 1) (trans 10 0)
    (trans 9 0) (trans 8 0) (trans 2 0) (trans 6 1) (trans 6 0)
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) l-2) l-1))
    (exp (gen) (mul l-1 l-2)) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1))))
  (label 1563)
  (parent 1538)
  (unrealized (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (7 0)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (8 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 8 1) (trans 8 0)
    (neq a b) (undisclosed l) (undisclosed l-1))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0)))
  (label 1564)
  (parent 1539)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 x y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-1) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l-1) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((3 3) (10 0)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l-0) (x l-1) (y-0 l-0) (y-0 l-1)
    (y-0 x))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 9 1) (trans 9 0)
    (neq a b) (undisclosed l-1) (undisclosed l-0))
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-1) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0))))))
  (label 1565)
  (parent 1540)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y l-0 l-1 rndx)
    (zeta expt) (y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
  (deflistener
    (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (precedes ((0 3) (10 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y l) (y x) (y l-0) (y-0 l-0) (y-0 l-1)
    (y-0 zeta))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 9 1) (trans 9 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (displaced 11 0 resp 4) (exp (gen) y-1) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul zeta y-0)))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
      (send
        (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0))))))
  (label 1566)
  (parent 1540)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x y-0 l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1) (x-0 la) (x-0 beta))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv a-0 la))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 9 1) (trans 9 0)
    (neq a b) (undisclosed l) (undisclosed l-1))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-6 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-1 a-0 b-0 (exp (gen) x-0)))))
  (label 1567)
  (parent 1540)
  (unrealized (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 1) (10 0)) ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 9 1) (trans 9 0)
    (neq a b) (undisclosed l) (undisclosed l-1))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 1568)
  (parent 1540)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (y-0 l la l-0 x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la la) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha la) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l la))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l la l-0)
  (uniq-gen y y-0 x)
  (absent (y zeta) (y l) (y l-0) (y-0 la) (y-0 l-0) (y-0 x) (x la)
    (x l-0))
  (precur (10 0))
  (gen-st (pv a l) (pv a la) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 9 1) (trans 9 0)
    (neq a b) (undisclosed la) (undisclosed l-0))
  (operation nonce-test (displaced 11 1 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y-0 x))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y-0 x)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a la)))
      (send
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 1569)
  (parent 1540)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (10 0)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 9 1) (trans 9 0)
    (neq a b) (undisclosed l) (undisclosed l-1))
  (operation nonce-test (displaced 11 2 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 1570)
  (parent 1540)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x y-0 l-0 lb rndx) (alpha zeta-0 expt)
    (l-1 y-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-1) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1) (y-1 lb) (y-1 alpha) (y-1 zeta-0))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b-0 lb))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) x) (gen)) (neq (exp (gen) zeta) (gen)) (trans 5 1)
    (trans 4 1) (trans 2 1) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 9 1) (trans 9 0) (neq a b) (undisclosed l) (undisclosed l-1))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-1) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-1 (rec y-1)))))
    ((load priv-stor-2 (cat pt-6 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1571)
  (parent 1540)
  (unrealized (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x y-0 l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-1) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l-1) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((4 2) (10 0)) ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l-0) (x l-1) (y-0 x) (y-0 l-0)
    (y-0 l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 9 1) (trans 9 0)
    (neq a b) (undisclosed l-1) (undisclosed l-0))
  (operation nonce-test (displaced 11 4 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 1572)
  (parent 1540)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (9 0)) ((5 2) (0 1)) ((5 2) (10 0)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 x)
    (y-0 l-0))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 9 1) (trans 9 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (displaced 11 5 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 1573)
  (parent 1540)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x y-0 l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 11 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 9 1) (trans 9 0) (neq a b) (undisclosed l) (undisclosed l-1))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1574)
  (parent 1540)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (8 0))
    ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((9 2) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 9 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 8 1) (trans 8 0) (neq a self) (undisclosed l)
    (undisclosed l-1))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1575)
  (parent 1541)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l l-0 lb l-1 x y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-1) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((3 3) (11 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (0 0)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 lb l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-1) (y-0 lb) (y-0 l-1)
    (y-0 x))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 6 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 10 1) (trans 10 0) (neq a b) (undisclosed l-1)
    (undisclosed l-0))
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-1) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-2 (cat pt-6 (pv a l)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0))))))
  (label 1576)
  (parent 1542)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (lb l x y l-0 l-1 rndx) (zeta expt) (y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
  (deflistener
    (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (precedes ((0 3) (11 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0))
    ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x lb) (x l) (y lb) (y l) (y x) (y-0 l-0) (y-0 l-1)
    (y-0 zeta))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 6 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 10 1) (trans 10 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation nonce-test (displaced 12 0 resp 4) (exp (gen) y-1) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul zeta y-0)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
      (send
        (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0))))))
  (label 1577)
  (parent 1542)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (lb l x y-0 l-0 la rndx) (beta expt)
    (l-1 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a a-0) (b b-0) (priv-stor priv-stor-3)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0))
    ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 lb l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l) (y-0 x)
    (x-0 la) (x-0 beta))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1) (pv a-0 la))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 6 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 10 1) (trans 10 0) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-3 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-1 a-0 b-0 (exp (gen) x-0)))))
  (label 1578)
  (parent 1542)
  (unrealized (9 0) (11 0) (12 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((5 1) (11 0)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 6 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 10 1) (trans 10 0) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 1579)
  (parent 1542)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (y-0 l l-0 la lb x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha la) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l la))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((1 2) (11 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (0 0)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 la lb)
  (uniq-gen y y-0 x)
  (absent (y zeta) (y l) (y l-0) (y-0 la) (y-0 lb) (y-0 x) (x la)
    (x lb))
  (precur (11 0))
  (gen-st (pv a l) (pv a la) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 6 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 10 1) (trans 10 0) (neq a b) (undisclosed la)
    (undisclosed l-0))
  (operation nonce-test (displaced 12 1 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a la)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul la lb)) (exp (gen) (mul y-0 x))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul la lb)) (exp (gen) (mul y-0 x)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a la)))
      (send
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-2 (cat pt-6 (pv a l)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 1580)
  (parent 1542)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb x y-0 l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-1) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((4 2) (11 0))
    ((5 1) (0 0)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-1) (y-0 lb) (y-0 x)
    (y-0 l-1))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 6 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 10 1) (trans 10 0) (neq a b) (undisclosed l-1)
    (undisclosed l-0))
  (operation nonce-test (displaced 12 4 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-2 (cat pt-6 (pv a l)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 1581)
  (parent 1542)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x y-0 l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-2) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-2) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((2 2) (11 0)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (0 0)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (y-0 l) (y-0 x)
    (y-0 l-2))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b l-2))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 6 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 10 1) (trans 10 0) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (operation nonce-test (displaced 12 2 ltx-gen 3) (exp (gen) l-3)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2))))))
  (label 1582)
  (parent 1542)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (lb l x y-0 l-0 lb-0 rndx) (alpha zeta-0 expt)
    (l-1 y-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b-0)
    (priv-stor priv-stor-3) (lb lb-0) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0))
    ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 3) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 lb l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l) (y-0 x)
    (y-1 lb-0) (y-1 alpha) (y-1 zeta-0))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1) (pv b-0 lb-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) x) (gen)) (neq (exp (gen) zeta) (gen)) (trans 6 1)
    (trans 5 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 5 0)
    (trans 4 0) (trans 2 0) (trans 10 1) (trans 10 0) (neq a b)
    (undisclosed l) (undisclosed l-1))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-1) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-1 (rec y-1)))))
    ((load priv-stor-3 (cat pt-8 (pv b-0 lb-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b-0
            (hash (exp (gen) (mul lb-0 alpha))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1583)
  (parent 1542)
  (unrealized (9 0) (11 0) (12 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (9 0))
    ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 11 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 9 1) (trans 9 0) (neq a self) (undisclosed l)
    (undisclosed l-1))
  (operation nonce-test (displaced 5 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1584)
  (parent 1542)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (10 0)) ((6 2) (0 1)) ((6 2) (11 0)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 6 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 10 1) (trans 10 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation nonce-test (displaced 12 6 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 1585)
  (parent 1542)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (lb l x y-0 l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-3) (self self)
    (priv-stor priv-stor-3) (l l-2))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0))
    ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 12 1) (trans 6 1) (trans 5 1)
    (trans 4 1) (trans 2 1) (trans 12 0) (trans 6 0) (trans 5 0)
    (trans 4 0) (trans 2 0) (trans 10 1) (trans 10 0) (neq a b)
    (undisclosed l) (undisclosed l-1))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-3 (cat pt-8 ignore-3))
      (stor priv-stor-3 (cat pt-9 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1586)
  (parent 1542)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (9 0)) ((3 3) (5 0)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1))
  (precur (5 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (trans 10 1) (trans 10 0) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (operation nonce-test (displaced 11 2 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0)))
  (label 1587)
  (parent 1547)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx) (w-0 expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec w-0))) w-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (trans 10 1) (trans 10 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec w-0))) w-0))
    (exp (gen) l-0) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul l-0 (rec w-0))) w-0))))
  (label 1588)
  (parent 1547)
  (unrealized (11 0))
  (comment "10 in cohort - 10 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0))
    ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (0 0)) ((11 2) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 11 1) (trans 6 1) (trans 4 1) (trans 2 1) (trans 11 0)
    (trans 6 0) (trans 4 0) (trans 2 0) (trans 10 1) (trans 10 0)
    (neq a self) (undisclosed l) (undisclosed l-1))
  (operation nonce-test (displaced 6 12 ltx-gen 3) (exp (gen) l-1)
    (10 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1589)
  (parent 1552)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx) (w-0 expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec w-0))) w-0))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1)) ((8 1) (0 4))
    ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 7 1) (trans 6 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 6 0) (trans 4 0) (trans 2 0) (trans 11 1) (trans 11 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec w-0))) w-0))
    (exp (gen) l-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul l-0 (rec w-0))) w-0))))
  (label 1590)
  (parent 1552)
  (unrealized (12 0))
  (comment "11 in cohort - 11 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (10 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (12 0)) ((8 2) (0 1))
    ((9 1) (0 4)) ((10 1) (9 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 0)) ((13 2) (11 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 13 1) (trans 8 1) (trans 7 1) (trans 2 1) (trans 13 0)
    (trans 8 0) (trans 7 0) (trans 2 0) (trans 12 1) (trans 6 1)
    (trans 12 0) (trans 6 0) (neq self self-0) (undisclosed l-0)
    (undisclosed l-2))
  (operation nonce-test (displaced 8 14 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1591)
  (parent 1557)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx) (w expt))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul l-1 (rec w))) w))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0)) ((12 1) (11 0))
    ((13 2) (12 0)) ((14 1) (12 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 8 1) (trans 7 1) (trans 2 1) (trans 9 0)
    (trans 8 0) (trans 7 0) (trans 2 0) (trans 13 1) (trans 6 1)
    (trans 13 0) (trans 6 0) (neq self b) (undisclosed l-0)
    (undisclosed l-1))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-1 (rec w))) w))
    (exp (gen) l-1) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-2))
    ((recv (cat (exp (gen) (mul l-1 (rec w))) w))
      (send (cat (exp (gen) (mul l-1 (rec w))) w))))
  (label 1592)
  (parent 1557)
  (unrealized (14 0))
  (comment "10 in cohort - 10 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l y x-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x-0) (beta l-1)
    (upsilon (mul y (rec x-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) x-0))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 1) (9 0)) ((8 2) (0 1))
    ((8 2) (7 0)) ((9 2) (0 2)) ((9 4) (0 4)) ((10 1) (0 0))
    ((10 2) (9 1)) ((11 1) (9 3)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l-0) (y l-1) (x-0 l-0) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 8 1) (trans 2 1) (trans 10 0) (trans 8 0) (trans 2 0)
    (trans 6 1) (trans 6 0) (neq self b) (undisclosed l-0)
    (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) y) x-0))
    (exp (gen) (mul y (rec x-0))) (9 3))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) y) x-0)) (send (cat (exp (gen) y) x-0))))
  (label 1593)
  (parent 1558)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l y x-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x-0) (beta l-1)
    (upsilon (mul y (rec x-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x-0)) y))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 1) (9 0)) ((8 2) (0 1))
    ((8 2) (7 0)) ((9 2) (0 2)) ((9 4) (0 4)) ((10 1) (0 0))
    ((10 2) (9 1)) ((11 1) (9 3)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l-0) (y l-1) (x-0 l-0) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 8 1) (trans 2 1) (trans 10 0) (trans 8 0) (trans 2 0)
    (trans 6 1) (trans 6 0) (neq self b) (undisclosed l-0)
    (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x-0)) y))
    (exp (gen) (mul y (rec x-0))) (9 3))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (rec x-0)) y))
      (send (cat (exp (gen) (rec x-0)) y))))
  (label 1594)
  (parent 1558)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x rndx)
    (upsilon expt) (l l-0 y x-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x-0) (beta l-2)
    (upsilon (mul y (rec x-0))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self b)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) y) x-0))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (10 0))
    ((9 2) (0 1)) ((10 2) (0 2)) ((10 4) (0 4)) ((11 1) (0 0))
    ((11 2) (10 1)) ((12 1) (10 3)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l-1) (y l-2) (x-0 l-1) (x-0 l-2))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 9 1) (trans 8 1) (trans 2 1) (trans 11 0) (trans 9 0)
    (trans 8 0) (trans 2 0) (trans 6 1) (trans 6 0) (neq self b)
    (undisclosed l-0) (undisclosed l-2))
  (operation nonce-test (added-listener (cat (exp (gen) y) x-0))
    (exp (gen) (mul y (rec x-0))) (10 3))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y))))) (send nb))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) y) x-0)) (send (cat (exp (gen) y) x-0))))
  (label 1595)
  (parent 1561)
  (unrealized (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x rndx)
    (upsilon expt) (l l-0 y x-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta (one)))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x-0) (beta l-2)
    (upsilon (mul y (rec x-0))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self b)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (rec x-0)) y))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (10 0))
    ((9 2) (0 1)) ((10 2) (0 2)) ((10 4) (0 4)) ((11 1) (0 0))
    ((11 2) (10 1)) ((12 1) (10 3)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l-1) (y l-2) (x-0 l-1) (x-0 l-2))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x-0))) (gen))
    (neq (exp (gen) upsilon) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 9 1) (trans 8 1) (trans 2 1) (trans 11 0) (trans 9 0)
    (trans 8 0) (trans 2 0) (trans 6 1) (trans 6 0) (neq self b)
    (undisclosed l-0) (undisclosed l-2))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x-0)) y))
    (exp (gen) (mul y (rec x-0))) (10 3))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y))))) (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x-0)))
      (recv
        (cat (exp (gen) (mul y (rec x-0)))
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) y))))) (send nb))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (rec x-0)) y))
      (send (cat (exp (gen) (rec x-0)) y))))
  (label 1596)
  (parent 1561)
  (unrealized (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4)) ((12 1) (11 0))
    ((13 1) (12 0)) ((14 2) (13 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 9 1) (trans 8 1) (trans 2 1) (trans 10 0)
    (trans 9 0) (trans 8 0) (trans 2 0) (trans 14 1) (trans 6 1)
    (trans 14 0) (trans 6 0) (neq self b) (undisclosed l-0)
    (undisclosed l-1))
  (operation nonce-test (added-strand ltx-disclose 3) l-2 (13 0)
    (ch-msg priv-stor-2 (cat pt-7 (pv self l-2))))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-2)))
  (label 1597)
  (parent 1562)
  (unrealized (13 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (1 1))
    ((2 2) (6 0)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (7 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 2) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 7 1) (trans 7 0)
    (neq a b) (undisclosed l) (undisclosed l-1))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0)))
  (label 1598)
  (parent 1564)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x y-0 l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-1) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l-1) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 1) (11 0))
    ((4 2) (3 1)) ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l) (y l-0) (x l-0) (x l-1) (y-0 x) (y-0 l-0)
    (y-0 l-1) (x-0 beta) (x-0 l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 9 1) (trans 9 0)
    (neq a b) (undisclosed l-1) (undisclosed l-0))
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 1599)
  (parent 1567)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-1) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 1) (11 0)) ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1) (x-0 beta) (x-0 l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 9 1) (trans 9 0)
    (neq a b) (undisclosed l) (undisclosed l-1))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 1600)
  (parent 1567)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (9 0)) ((5 1) (11 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 x)
    (y-0 l-0) (x-0 beta) (x-0 l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 9 1) (trans 9 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 1601)
  (parent 1567)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x y-0 l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1) (x-0 beta) (x-0 l-2))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 12 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 12 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 9 1) (trans 9 0) (neq a b) (undisclosed l) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-6 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a-0 l-2)))))
  (label 1602)
  (parent 1567)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (9 0)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (8 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 8 1) (trans 8 0)
    (neq a b) (undisclosed l) (undisclosed l-1))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 1603)
  (parent 1570)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x y-0 l rndx) (alpha zeta-0 expt) (l-0 y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-1) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l-1) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-1) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 1) (11 0))
    ((4 2) (3 1)) ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 3) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l) (y l-0) (x l-0) (x l-1) (y-0 x) (y-0 l-0)
    (y-0 l-1) (y-1 alpha) (y-1 zeta-0) (y-1 l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) x) (gen)) (neq (exp (gen) zeta) (gen)) (trans 5 1)
    (trans 4 1) (trans 2 1) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 9 1) (trans 9 0) (neq a b) (undisclosed l-1)
    (undisclosed l-0))
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-0 (rec y-1)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1604)
  (parent 1571)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 rndx) (alpha zeta-0 expt) (y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul (rec y-1) l-1)))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-1) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 1) (11 0)) ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 3) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1) (y-1 alpha) (y-1 zeta-0) (y-1 l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) x) (gen)) (neq (exp (gen) zeta) (gen)) (trans 5 1)
    (trans 4 1) (trans 2 1) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 9 1) (trans 9 0) (neq a b) (undisclosed l) (undisclosed l-1))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul (rec y-1) l-1)))
      (send (cat (exp (gen) y-1) (mul (rec y-1) l-1))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1605)
  (parent 1571)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 rndx) (alpha zeta-0 expt) (l-0 y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b a)
    (priv-stor priv-stor-1) (lb l-1) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (9 0)) ((5 1) (11 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 3) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 x)
    (y-0 l-0) (y-1 alpha) (y-1 zeta-0) (y-1 l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) x) (gen)) (neq (exp (gen) zeta) (gen)) (trans 5 1)
    (trans 4 1) (trans 2 1) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 9 1) (trans 9 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-0 (rec y-1)))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1606)
  (parent 1571)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x y-0 l-0 rndx) (alpha zeta-0 expt)
    (l-1 y-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0))
    ((12 1) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1) (y-1 alpha) (y-1 zeta-0) (y-1 l-2))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) x) (gen)) (neq (exp (gen) zeta) (gen)) (trans 12 1)
    (trans 5 1) (trans 4 1) (trans 2 1) (trans 12 0) (trans 5 0)
    (trans 4 0) (trans 2 0) (trans 9 1) (trans 9 0) (neq a b)
    (undisclosed l) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-1 (rec y-1)))))
    ((load priv-stor-2 (cat pt-6 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-1)))))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv b-0 l-2)))))
  (label 1607)
  (parent 1571)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (7 0)) ((5 2) (0 1))
    ((6 1) (0 4)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 8 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 8 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 7 1) (trans 7 0) (neq a self) (undisclosed l)
    (undisclosed l-1))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1608)
  (parent 1575)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb x y-0 l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-1) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 1) (12 0)) ((4 2) (3 1))
    ((5 1) (0 0)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 lb l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-1) (y-0 lb) (y-0 x)
    (y-0 l-1) (x-0 beta) (x-0 l-1))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 6 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 10 1) (trans 10 0) (neq a b) (undisclosed l-1)
    (undisclosed l-0))
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-2 (cat pt-6 (pv a l)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 1609)
  (parent 1578)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x y-0 l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-2) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-2) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-2) (x x-0) (beta beta))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 1) (12 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (0 0)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (y-0 l) (y-0 x)
    (y-0 l-2) (x-0 beta) (x-0 l-2))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b l-2))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 6 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 10 1) (trans 10 0) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv a-0 l-3))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 1610)
  (parent 1578)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-1) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((5 1) (12 0)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 lb l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l) (y-0 x)
    (x-0 beta) (x-0 l-1))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 6 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 10 1) (trans 10 0) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (operation channel-test (displaced 13 5 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 1611)
  (parent 1578)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-2)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (10 0)) ((6 1) (12 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 lb l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l) (y-0 x)
    (x-0 beta) (x-0 l-1))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 6 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 10 1) (trans 10 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation channel-test (displaced 13 6 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 1612)
  (parent 1578)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 na-1 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (lb l x y-0 l-0 rndx) (beta expt)
    (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a a-0) (b b-0) (priv-stor priv-stor-3)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-3) (self a-0)
    (priv-stor priv-stor-3) (l l-2))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0))
    ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0))
    ((13 1) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 lb l l-0 l-1 l-2)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l) (y-0 x)
    (x-0 beta) (x-0 l-2))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 13 1) (trans 6 1) (trans 5 1)
    (trans 4 1) (trans 2 1) (trans 13 0) (trans 6 0) (trans 5 0)
    (trans 4 0) (trans 2 0) (trans 10 1) (trans 10 0) (neq a b)
    (undisclosed l) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-3 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-3 (cat pt-9 ignore-3))
      (stor priv-stor-3 (cat pt-8 (pv a-0 l-2)))))
  (label 1613)
  (parent 1578)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb x y-0 l rndx) (alpha zeta-0 expt)
    (l-0 y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-1) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l))
  (deflistener (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b a)
    (priv-stor priv-stor-1) (lb l-1) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 1) (12 0)) ((4 2) (3 1))
    ((5 1) (0 0)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 3) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 lb l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-1) (y-0 lb) (y-0 x)
    (y-0 l-1) (y-1 alpha) (y-1 zeta-0) (y-1 l-1))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) x) (gen)) (neq (exp (gen) zeta) (gen)) (trans 6 1)
    (trans 5 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 5 0)
    (trans 4 0) (trans 2 0) (trans 10 1) (trans 10 0) (neq a b)
    (undisclosed l-1) (undisclosed l-0))
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-2 (cat pt-6 (pv a l)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-0 (rec y-1)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1614)
  (parent 1583)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x y-0 l-0 rndx) (alpha zeta-0 expt)
    (l-1 y-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-2) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-2) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-2) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 1) (12 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (0 0)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 3) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (y-0 l) (y-0 x)
    (y-0 l-2) (y-1 alpha) (y-1 zeta-0) (y-1 l-2))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b l-2))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) x) (gen)) (neq (exp (gen) zeta) (gen)) (trans 6 1)
    (trans 5 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 5 0)
    (trans 4 0) (trans 2 0) (trans 10 1) (trans 10 0) (neq a b)
    (undisclosed l) (undisclosed l-1))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv b-0 l-3))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-1 (rec y-1)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1615)
  (parent 1583)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 rndx) (alpha zeta-0 expt)
    (y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul (rec y-1) l-1)))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-1) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((5 1) (12 0)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 3) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 lb l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l) (y-0 x)
    (y-1 alpha) (y-1 zeta-0) (y-1 l-1))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) x) (gen)) (neq (exp (gen) zeta) (gen)) (trans 6 1)
    (trans 5 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 5 0)
    (trans 4 0) (trans 2 0) (trans 10 1) (trans 10 0) (neq a b)
    (undisclosed l) (undisclosed l-1))
  (operation channel-test (displaced 13 5 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul (rec y-1) l-1)))
      (send (cat (exp (gen) y-1) (mul (rec y-1) l-1))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1616)
  (parent 1583)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 rndx) (alpha zeta-0 expt)
    (l-0 y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b a)
    (priv-stor priv-stor-2) (lb l-1) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (10 0)) ((6 1) (12 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 3) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 lb l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l) (y-0 x)
    (y-1 alpha) (y-1 zeta-0) (y-1 l-1))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) x) (gen)) (neq (exp (gen) zeta) (gen)) (trans 6 1)
    (trans 5 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 5 0)
    (trans 4 0) (trans 2 0) (trans 10 1) (trans 10 0) (neq a b)
    (undisclosed l) (undisclosed l-0))
  (operation channel-test (displaced 13 6 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-0 (rec y-1)))))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1617)
  (parent 1583)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (lb l x y-0 l-0 rndx) (alpha zeta-0 expt)
    (l-1 y-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b-0)
    (priv-stor priv-stor-3) (lb l-2) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-3) (self b-0)
    (priv-stor priv-stor-3) (l l-2))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0))
    ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 3) (11 0))
    ((13 1) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 lb l l-0 l-1 l-2)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l) (y-0 x)
    (y-1 alpha) (y-1 zeta-0) (y-1 l-2))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) x) (gen)) (neq (exp (gen) zeta) (gen)) (trans 13 1)
    (trans 6 1) (trans 5 1) (trans 4 1) (trans 2 1) (trans 13 0)
    (trans 6 0) (trans 5 0) (trans 4 0) (trans 2 0) (trans 10 1)
    (trans 10 0) (neq a b) (undisclosed l) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-1 (rec y-1)))))
    ((load priv-stor-3 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-1)))))))
    ((load priv-stor-3 (cat pt-9 ignore-3))
      (stor priv-stor-3 (cat pt-8 (pv b-0 l-2)))))
  (label 1618)
  (parent 1583)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (8 0))
    ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 10 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 8 1) (trans 8 0) (neq a self) (undisclosed l)
    (undisclosed l-1))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1619)
  (parent 1584)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (8 0)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 4 1) (trans 2 1) (trans 5 0) (trans 4 0)
    (trans 2 0) (trans 9 1) (trans 9 0) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0)))
  (label 1620)
  (parent 1587)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 x rndx) (zeta-0 expt) (y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) zeta-0 y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l-1) (zeta zeta-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) zeta-0)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((3 3) (11 0)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0))
    ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l-0) (x l-1) (y-0 l-0) (y-0 l-1)
    (y-0 zeta-0))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) zeta-0 y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (trans 10 1) (trans 10 0) (neq a b) (undisclosed l-1)
    (undisclosed l-0))
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-1) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) zeta-0 y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul zeta-0 y-0)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) zeta-0)))
      (send (cat (exp (gen) y-0) (mul (rec x) zeta-0))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0))))))
  (label 1621)
  (parent 1588)
  (unrealized (1 3) (5 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x rndx) (w expt)
    (y l-0 l-1 rndx) (zeta expt) (y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
  (deflistener
    (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (precedes ((0 3) (11 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y l) (y (mul x w)) (y l-0) (y-0 l-0) (y-0 l-1)
    (y-0 zeta))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (trans 10 1) (trans 10 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation nonce-test (displaced 12 0 resp 4) (exp (gen) y-1) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul zeta y-0)))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
      (send
        (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0))))))
  (label 1622)
  (parent 1588)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x rndx) (w expt) (y-0 l-0 la rndx) (beta expt)
    (l-1 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1) (x-0 la) (x-0 beta))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (trans 10 1) (trans 10 0) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-6 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-1 a-0 b-0 (exp (gen) x-0)))))
  (label 1623)
  (parent 1588)
  (unrealized (9 0) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 1) (11 0)) ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0))
    ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (trans 10 1) (trans 10 0) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (operation nonce-test (contracted (l-2 l-1) (w-0 l-1)) (gen) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 1624)
  (parent 1588)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta w expt)
    (y-0 l la l-0 x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la la) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha la) (zeta (mul w x)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l la))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0))
    ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l la l-0)
  (uniq-gen y y-0 x)
  (absent (y zeta) (y l) (y l-0) (y-0 (mul w x)) (y-0 la) (y-0 l-0)
    (x la) (x l-0))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a la) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul w x)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (trans 10 1) (trans 10 0) (neq a b) (undisclosed la)
    (undisclosed l-0))
  (operation nonce-test (displaced 12 1 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul w y-0 x))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul w x))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul la l-0))
              (exp (gen) (mul w y-0 x)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a la)))
      (send
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 1625)
  (parent 1588)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (11 0)) ((3 3) (5 0)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0))
    ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (trans 10 1) (trans 10 0) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (operation nonce-test (displaced 12 2 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 1626)
  (parent 1588)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x rndx) (w expt) (y-0 l-0 lb rndx)
    (alpha zeta-0 expt) (l-1 y-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-1) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 3) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1) (y-1 lb) (y-1 alpha) (y-1 zeta-0))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 4 1) (trans 2 1)
    (trans 6 0) (trans 4 0) (trans 2 0) (trans 10 1) (trans 10 0)
    (neq a b) (undisclosed l) (undisclosed l-1))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-1) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-1 (rec y-1)))))
    ((load priv-stor-2 (cat pt-6 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1627)
  (parent 1588)
  (unrealized (9 0) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (w expt) (y-0 l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-1) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l-1) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((4 2) (11 0)) ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0))
    ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l-0) (x l-1) (y-0 (mul x w))
    (y-0 l-0) (y-0 l-1))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (trans 10 1) (trans 10 0) (neq a b) (undisclosed l-1)
    (undisclosed l-0))
  (operation nonce-test (displaced 12 4 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul x w y-0)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 1628)
  (parent 1588)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1)) ((6 2) (11 0))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0))
    ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (trans 10 1) (trans 10 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation nonce-test (displaced 12 6 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 1629)
  (parent 1588)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x rndx) (w expt) (y-0 l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 12 1) (trans 6 1) (trans 4 1) (trans 2 1) (trans 12 0)
    (trans 6 0) (trans 4 0) (trans 2 0) (trans 10 1) (trans 10 0)
    (neq a b) (undisclosed l) (undisclosed l-1))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1630)
  (parent 1588)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (9 0))
    ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 0)) ((10 2) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 5 1) (trans 4 1) (trans 2 1) (trans 10 0)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 9 1) (trans 9 0)
    (neq a self) (undisclosed l) (undisclosed l-1))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1631)
  (parent 1589)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l l-0 lb l-1 x rndx) (zeta-0 expt) (y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta lb)
    (upsilon (mul (rec x) zeta-0 y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-1) (zeta zeta-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) zeta-0)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((3 3) (12 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 lb l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-1) (y-0 lb) (y-0 l-1)
    (y-0 zeta-0))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) zeta-0 y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 7 1) (trans 6 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 6 0) (trans 4 0) (trans 2 0) (trans 11 1) (trans 11 0)
    (neq a b) (undisclosed l-1) (undisclosed l-0))
  (operation nonce-test (displaced 13 3 resp 4) (exp (gen) y-1) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) zeta-0 y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1))
              (exp (gen) (mul zeta-0 y-0)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) zeta-0)))
      (send (cat (exp (gen) y-0) (mul (rec x) zeta-0))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-2 (cat pt-6 (pv a l)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0))))))
  (label 1632)
  (parent 1590)
  (unrealized (1 3) (5 0) (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb l x rndx)
    (w expt) (y l-0 l-1 rndx) (zeta expt) (y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
  (deflistener
    (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (precedes ((0 3) (12 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1)) ((8 1) (0 4))
    ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)) (y-0 l-0) (y-0 l-1)
    (y-0 zeta))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 7 1) (trans 6 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 6 0) (trans 4 0) (trans 2 0) (trans 11 1) (trans 11 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (displaced 13 0 resp 4) (exp (gen) y-1) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul zeta y-0)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
      (send
        (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0))))))
  (label 1633)
  (parent 1590)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 la rndx)
    (beta expt) (l-1 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a a-0) (b b-0) (priv-stor priv-stor-3)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1)) ((8 1) (0 4))
    ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (10 0))
    ((13 2) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 lb l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)) (x-0 la) (x-0 beta))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 7 1) (trans 6 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 6 0) (trans 4 0) (trans 2 0) (trans 11 1) (trans 11 0)
    (neq a b) (undisclosed l) (undisclosed l-1))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-3 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-1 a-0 b-0 (exp (gen) x-0)))))
  (label 1634)
  (parent 1590)
  (unrealized (10 0) (12 0) (13 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((6 1) (12 0)) ((7 1) (11 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 7 1) (trans 6 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 6 0) (trans 4 0) (trans 2 0) (trans 11 1) (trans 11 0)
    (neq a b) (undisclosed l) (undisclosed l-1))
  (operation nonce-test (contracted (l-2 l-1) (w-0 l-1)) (gen) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 1635)
  (parent 1590)
  (unrealized (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta w expt) (y-0 l l-0 la lb x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha la) (zeta (mul w x)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l la))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((1 2) (12 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 la lb)
  (uniq-gen y y-0 x)
  (absent (y zeta) (y l) (y l-0) (y-0 (mul w x)) (y-0 la) (y-0 lb)
    (x la) (x lb))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a la) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul w x)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 7 1) (trans 6 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 6 0) (trans 4 0) (trans 2 0) (trans 11 1) (trans 11 0)
    (neq a b) (undisclosed la) (undisclosed l-0))
  (operation nonce-test (displaced 13 1 init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a la)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul la lb)) (exp (gen) (mul w y-0 x))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul w x))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul la lb)) (exp (gen) (mul w y-0 x)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a la)))
      (send
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-2 (cat pt-6 (pv a l)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 1636)
  (parent 1590)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb x rndx) (w expt) (y-0 l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-1)
    (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((4 2) (12 0))
    ((5 1) (1 3)) ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-1) (y-0 lb) (y-0 (mul x w))
    (y-0 l-1))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 7 1) (trans 6 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 6 0) (trans 4 0) (trans 2 0) (trans 11 1) (trans 11 0)
    (neq a b) (undisclosed l-1) (undisclosed l-0))
  (operation nonce-test (displaced 13 4 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1))
              (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-2 (cat pt-6 (pv a l)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 1637)
  (parent 1590)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x rndx) (w expt) (y-0 l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-2)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-2) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((2 2) (12 0)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (y-0 l) (y-0 (mul x w))
    (y-0 l-2))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b l-2))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 7 1) (trans 6 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 6 0) (trans 4 0) (trans 2 0) (trans 11 1) (trans 11 0)
    (neq a b) (undisclosed l) (undisclosed l-1))
  (operation nonce-test (displaced 13 2 ltx-gen 3) (exp (gen) l-3)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2))))))
  (label 1638)
  (parent 1590)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 lb-0 rndx)
    (alpha zeta-0 expt) (l-1 y-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b-0)
    (priv-stor priv-stor-3) (lb lb-0) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1)) ((8 1) (0 4))
    ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (10 0))
    ((13 3) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 lb l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)) (y-1 lb-0) (y-1 alpha) (y-1 zeta-0))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1) (pv b-0 lb-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 7 1) (trans 6 1) (trans 4 1)
    (trans 2 1) (trans 7 0) (trans 6 0) (trans 4 0) (trans 2 0)
    (trans 11 1) (trans 11 0) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-1) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-1 (rec y-1)))))
    ((load priv-stor-3 (cat pt-8 (pv b-0 lb-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b-0
            (hash (exp (gen) (mul lb-0 alpha))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1639)
  (parent 1590)
  (unrealized (10 0) (12 0) (13 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0))
    ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 0))
    ((12 2) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 12 1) (trans 6 1) (trans 4 1) (trans 2 1) (trans 12 0)
    (trans 6 0) (trans 4 0) (trans 2 0) (trans 10 1) (trans 10 0)
    (neq a self) (undisclosed l) (undisclosed l-1))
  (operation nonce-test (displaced 6 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1640)
  (parent 1590)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1)) ((7 2) (12 0))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 7 1) (trans 6 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 6 0) (trans 4 0) (trans 2 0) (trans 11 1) (trans 11 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (displaced 13 7 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 1641)
  (parent 1590)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-3) (self self)
    (priv-stor priv-stor-3) (l l-2))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1)) ((8 1) (0 4))
    ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (10 0))
    ((13 2) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 13 1) (trans 7 1) (trans 6 1) (trans 4 1) (trans 2 1)
    (trans 13 0) (trans 7 0) (trans 6 0) (trans 4 0) (trans 2 0)
    (trans 11 1) (trans 11 0) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-3 (cat pt-8 ignore-3))
      (stor priv-stor-3 (cat pt-9 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1642)
  (parent 1590)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (9 0)) ((1 2) (3 0)) ((2 1) (5 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (1 0))
    ((6 2) (4 0)) ((7 1) (11 0)) ((7 2) (0 1)) ((8 1) (0 4))
    ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 0))
    ((12 2) (10 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 12 1) (trans 7 1) (trans 6 1) (trans 2 1) (trans 12 0)
    (trans 7 0) (trans 6 0) (trans 2 0) (trans 11 1) (trans 5 1)
    (trans 11 0) (trans 5 0) (neq self self-0) (undisclosed l-0)
    (undisclosed l-2))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1643)
  (parent 1591)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 lb rndx)
    (alpha zeta-0 expt) (l-2 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-2 (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a) (b b-0)
    (priv-stor priv-stor-3) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0)) ((12 1) (11 0))
    ((13 2) (12 0)) ((14 1) (12 0)) ((15 3) (14 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta-0))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 8 1) (trans 7 1)
    (trans 2 1) (trans 9 0) (trans 8 0) (trans 7 0) (trans 2 0)
    (trans 13 1) (trans 6 1) (trans 13 0) (trans 6 0) (neq self b)
    (undisclosed l-0) (undisclosed l-2))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (14 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-2 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-2 (rec y-0)))))
    ((load priv-stor-3 (cat pt-9 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-1 a b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1644)
  (parent 1592)
  (unrealized (12 0) (14 0) (15 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x rndx)
    (upsilon expt) (l l-0 l-1 l-2 rndx) (zeta expt) (y rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) y) (mul l-1 (rec y))))
  (precedes ((0 3) (14 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0)) ((12 1) (11 0))
    ((13 2) (12 0)) ((14 1) (12 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l-1) (y l-2) (y zeta))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 8 1) (trans 7 1) (trans 2 1) (trans 9 0)
    (trans 8 0) (trans 7 0) (trans 2 0) (trans 13 1) (trans 6 1)
    (trans 13 0) (trans 6 0) (neq self b) (undisclosed l-0)
    (undisclosed l-1))
  (operation nonce-test (displaced 15 0 resp 4) (exp (gen) y-0) (14 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-2))
    ((recv (cat (exp (gen) y) (mul l-1 (rec y))))
      (send (cat (exp (gen) y) (mul l-1 (rec y))))))
  (label 1645)
  (parent 1592)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 la rndx)
    (beta expt) (l-2 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-2 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-3)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0)) ((12 1) (11 0))
    ((13 2) (12 0)) ((14 1) (12 0)) ((15 2) (14 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1) (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 8 1) (trans 7 1) (trans 2 1) (trans 9 0)
    (trans 8 0) (trans 7 0) (trans 2 0) (trans 13 1) (trans 6 1)
    (trans 13 0) (trans 6 0) (neq self b) (undisclosed l-0)
    (undisclosed l-2))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (14 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-2 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-2 (rec x-0)))))
    ((load priv-stor-3 (cat pt-9 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 1646)
  (parent 1592)
  (unrealized (12 0) (14 0) (15 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (gen) l-2))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((8 1) (14 0))
    ((9 1) (13 0)) ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 8 1) (trans 7 1) (trans 2 1) (trans 9 0)
    (trans 8 0) (trans 7 0) (trans 2 0) (trans 13 1) (trans 6 1)
    (trans 13 0) (trans 6 0) (neq self b) (undisclosed l-0)
    (undisclosed l-2))
  (operation nonce-test (contracted (l-3 l-2) (w l-2)) (gen) (14 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2))))
  (label 1647)
  (parent 1592)
  (unrealized (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta upsilon expt) (l l-0 la l-1 x rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l-1) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-1))
  (deflistener
    (hash (exp (gen) (mul la l-1)) (exp (gen) (mul upsilon x))))
  (deflistener
    (cat (exp (gen) (mul la l-1)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) la) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l la))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((1 2) (14 0)) ((2 1) (6 0))
    ((2 2) (1 1)) ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0))
    ((9 1) (13 0)) ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 la l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x la) (x l-1))
  (precur (14 0))
  (gen-st (pv b l-0) (pv b l-1) (pv self l) (pv self la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 8 1) (trans 7 1) (trans 2 1) (trans 9 0)
    (trans 8 0) (trans 7 0) (trans 2 0) (trans 13 1) (trans 6 1)
    (trans 13 0) (trans 6 0) (neq self b) (undisclosed la)
    (undisclosed l-0))
  (operation nonce-test (displaced 15 1 init 3) (exp (gen) x-0) (14 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self la)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul la l-1))
              (exp (gen) (mul upsilon x)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul la l-1)) (exp (gen) (mul upsilon x))))
      (send
        (hash (exp (gen) (mul la l-1)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul la l-1)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul la l-1)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) la) l-1)) (send (cat (exp (gen) la) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-2 (cat pt-7 (pv self l)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 1648)
  (parent 1592)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-2) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-2) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((7 2) (14 0)) ((8 1) (0 0))
    ((9 1) (13 0)) ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 8 1) (trans 7 1) (trans 2 1) (trans 9 0)
    (trans 8 0) (trans 7 0) (trans 2 0) (trans 13 1) (trans 6 1)
    (trans 13 0) (trans 6 0) (neq self b) (undisclosed l-2)
    (undisclosed l-1))
  (operation nonce-test (displaced 15 7 ltx-gen 3) (exp (gen) l-3)
    (14 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-2)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-2) l)) (send (cat (exp (gen) l-2) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-7 (pv self l-0)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2))))))
  (label 1649)
  (parent 1592)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-2) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-2))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((2 2) (14 0)) ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0))
    ((9 1) (13 0)) ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2))
  (precur (14 0))
  (gen-st (pv b l-1) (pv b l-2) (pv self l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 8 1) (trans 7 1) (trans 2 1) (trans 9 0)
    (trans 8 0) (trans 7 0) (trans 2 0) (trans 13 1) (trans 6 1)
    (trans 13 0) (trans 6 0) (neq self b) (undisclosed l)
    (undisclosed l-1))
  (operation nonce-test (displaced 15 2 ltx-gen 3) (exp (gen) l-3)
    (14 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l-2))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-7 (pv self l-0)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2))))))
  (label 1650)
  (parent 1592)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (10 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (12 0)) ((8 2) (0 1))
    ((9 1) (0 4)) ((10 1) (9 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (11 0)) ((14 1) (0 0)) ((14 2) (13 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (13 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 14 1) (trans 8 1) (trans 7 1) (trans 2 1) (trans 14 0)
    (trans 8 0) (trans 7 0) (trans 2 0) (trans 12 1) (trans 6 1)
    (trans 12 0) (trans 6 0) (neq self self-0) (undisclosed l-0)
    (undisclosed l-2))
  (operation nonce-test (displaced 8 15 ltx-gen 3) (exp (gen) l-3)
    (14 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1651)
  (parent 1592)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((9 2) (0 1)) ((9 2) (14 0)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 8 1) (trans 7 1) (trans 2 1) (trans 9 0)
    (trans 8 0) (trans 7 0) (trans 2 0) (trans 13 1) (trans 6 1)
    (trans 13 0) (trans 6 0) (neq self b) (undisclosed l-0)
    (undisclosed l-1))
  (operation nonce-test (displaced 15 9 ltx-gen 3) (exp (gen) l-3)
    (14 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-2))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2))))))
  (label 1652)
  (parent 1592)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pt-10 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (x rndx) (upsilon expt)
    (l l-0 l-1 l-2 l-3 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul l-2 (rec l-3))))
  (defstrand ltx-gen 3 (ignore ignore-3) (self self-0)
    (priv-stor priv-stor-3) (l l-3))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0)) ((12 1) (11 0))
    ((13 2) (12 0)) ((14 1) (12 0)) ((15 2) (14 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2 l-3)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 15 1) (trans 9 1) (trans 8 1) (trans 7 1) (trans 2 1)
    (trans 15 0) (trans 9 0) (trans 8 0) (trans 7 0) (trans 2 0)
    (trans 13 1) (trans 6 1) (trans 13 0) (trans 6 0) (neq self b)
    (undisclosed l-0) (undisclosed l-2))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-3) (14 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-3) (mul l-2 (rec l-3))))
      (send (cat (exp (gen) l-3) (mul l-2 (rec l-3)))))
    ((load priv-stor-3 (cat pt-9 ignore-3))
      (stor priv-stor-3 (cat pt-10 (pv self-0 l-3)))
      (send
        (sig (body self-0 (exp (gen) l-3) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1653)
  (parent 1592)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (13 0))
    ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0)) ((12 1) (11 0))
    ((13 2) (12 0)) ((14 1) (0 0)) ((14 2) (12 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 14 1) (trans 9 1) (trans 8 1) (trans 2 1) (trans 14 0)
    (trans 9 0) (trans 8 0) (trans 2 0) (trans 13 1) (trans 6 1)
    (trans 13 0) (trans 6 0) (neq self self-0) (undisclosed l-0)
    (undisclosed l-2))
  (operation nonce-test (displaced 9 15 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1654)
  (parent 1597)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx) (w expt))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul l-1 (rec w))) w))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4)) ((12 1) (11 0))
    ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 9 1) (trans 8 1) (trans 2 1) (trans 10 0)
    (trans 9 0) (trans 8 0) (trans 2 0) (trans 14 1) (trans 6 1)
    (trans 14 0) (trans 6 0) (neq self b) (undisclosed l-0)
    (undisclosed l-1))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-1 (rec w))) w))
    (exp (gen) l-1) (13 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-2))
    ((recv (cat (exp (gen) (mul l-1 (rec w))) w))
      (send (cat (exp (gen) (mul l-1 (rec w))) w))))
  (label 1655)
  (parent 1597)
  (unrealized (15 0))
  (comment "10 in cohort - 10 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 4))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (6 0)) ((5 2) (0 1)) ((6 2) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 6 1) (trans 6 0)
    (neq a b) (undisclosed l) (undisclosed l-1))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0)))
  (label 1656)
  (parent 1598)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((a a) (b b) (la l) (lb l-1) (alpha l-0) (beta l-1) (y y)
        (zeta zeta) (na na) (nb nb) (priv-stor priv-stor) (pt pt-0)
        (x x) (upsilon y-0) (na-0 na-0) (nb-0 nb-0)
        (priv-stor-0 priv-stor-0) (pt-0 pt-2))))
  (origs (l-1 (2 1)) (pt-0 (2 1)) (pt-5 (6 1)) (l-0 (5 1)) (pt-4 (5 1))
    (nb-0 (3 3)) (l (4 1)) (pt-2 (4 1)) (na-0 (1 2)) (nb (0 3))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (1 1))
    ((2 2) (8 0)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (7 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 7 1) (trans 7 0)
    (neq a b) (undisclosed l) (undisclosed l-1))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 1657)
  (parent 1603)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (6 0)) ((5 2) (0 1))
    ((6 2) (0 4)) ((7 1) (0 0)) ((7 2) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 7 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 7 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 6 1) (trans 6 0) (neq a self) (undisclosed l)
    (undisclosed l-1))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1658)
  (parent 1608)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((a a) (b self) (la l) (lb l-1) (alpha l-0) (beta lb) (y y)
        (zeta zeta) (na na) (nb nb) (priv-stor priv-stor-2) (pt pt-7)
        (x x) (upsilon y-0) (na-0 na-0) (nb-0 nb-0)
        (priv-stor-0 priv-stor-0) (pt-0 pt-2))))
  (origs (l-1 (7 1)) (pt-7 (7 1)) (pt-5 (6 1)) (l-0 (5 1)) (pt-4 (5 1))
    (nb-0 (3 3)) (l (4 1)) (pt-2 (4 1)) (lb (2 1)) (pt-0 (2 1))
    (na-0 (1 2)) (nb (0 3))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (7 0)) ((5 2) (0 1))
    ((6 1) (0 4)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (precur (8 0))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 9 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 7 1) (trans 7 0) (neq a self) (undisclosed l)
    (undisclosed l-1))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1659)
  (parent 1619)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (7 0)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (8 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 4 1) (trans 2 1) (trans 5 0) (trans 4 0)
    (trans 2 0) (trans 8 1) (trans 8 0) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0)))
  (label 1660)
  (parent 1620)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (w expt) (y-0 l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-1) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l-1) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 1) (12 0))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l) (y l-0) (x l-0) (x l-1) (y-0 (mul x w))
    (y-0 l-0) (y-0 l-1) (x-0 beta) (x-0 l-1))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (trans 10 1) (trans 10 0) (neq a b) (undisclosed l-1)
    (undisclosed l-0))
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul x w y-0)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 1661)
  (parent 1623)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-1) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 1) (12 0)) ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1) (x-0 beta) (x-0 l-1))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (trans 10 1) (trans 10 0) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 1662)
  (parent 1623)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (10 0)) ((6 1) (12 0)) ((6 2) (0 1))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 (mul x w))
    (y-0 l-0) (x-0 beta) (x-0 l-1))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 4 1) (trans 2 1) (trans 6 0) (trans 4 0)
    (trans 2 0) (trans 10 1) (trans 10 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation channel-test (displaced 13 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 1663)
  (parent 1623)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x rndx) (w expt) (y-0 l-0 rndx) (beta expt)
    (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1) (x-0 beta) (x-0 l-2))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 13 1) (trans 6 1) (trans 4 1) (trans 2 1) (trans 13 0)
    (trans 6 0) (trans 4 0) (trans 2 0) (trans 10 1) (trans 10 0)
    (neq a b) (undisclosed l) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-6 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a-0 l-2)))))
  (label 1664)
  (parent 1623)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (10 0)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 4 1) (trans 2 1) (trans 5 0) (trans 4 0)
    (trans 2 0) (trans 9 1) (trans 9 0) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 1665)
  (parent 1626)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (w expt) (y-0 l rndx) (alpha zeta-0 expt)
    (l-0 y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-1) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l-1) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-1) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 1) (12 0))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 3) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l) (y l-0) (x l-0) (x l-1) (y-0 (mul x w))
    (y-0 l-0) (y-0 l-1) (y-1 alpha) (y-1 zeta-0) (y-1 l-1))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 4 1) (trans 2 1)
    (trans 6 0) (trans 4 0) (trans 2 0) (trans 10 1) (trans 10 0)
    (neq a b) (undisclosed l-1) (undisclosed l-0))
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul x w y-0)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-0 (rec y-1)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1666)
  (parent 1627)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 rndx) (alpha zeta-0 expt)
    (y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul (rec y-1) l-1)))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-1) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 1) (12 0)) ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 3) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1) (y-1 alpha) (y-1 zeta-0) (y-1 l-1))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 4 1) (trans 2 1)
    (trans 6 0) (trans 4 0) (trans 2 0) (trans 10 1) (trans 10 0)
    (neq a b) (undisclosed l) (undisclosed l-1))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul (rec y-1) l-1)))
      (send (cat (exp (gen) y-1) (mul (rec y-1) l-1))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1667)
  (parent 1627)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 rndx) (alpha zeta-0 expt)
    (l-0 y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b a)
    (priv-stor priv-stor-1) (lb l-1) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (10 0)) ((6 1) (12 0)) ((6 2) (0 1))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 3) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 (mul x w))
    (y-0 l-0) (y-1 alpha) (y-1 zeta-0) (y-1 l-1))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 4 1) (trans 2 1)
    (trans 6 0) (trans 4 0) (trans 2 0) (trans 10 1) (trans 10 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation channel-test (displaced 13 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-0 (rec y-1)))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1668)
  (parent 1627)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x rndx) (w expt) (y-0 l-0 rndx) (alpha zeta-0 expt)
    (l-1 y-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 3) (11 0)) ((13 1) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1) (y-1 alpha) (y-1 zeta-0) (y-1 l-2))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 13 1) (trans 6 1) (trans 4 1)
    (trans 2 1) (trans 13 0) (trans 6 0) (trans 4 0) (trans 2 0)
    (trans 10 1) (trans 10 0) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-1 (rec y-1)))))
    ((load priv-stor-2 (cat pt-6 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-1)))))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv b-0 l-2)))))
  (label 1669)
  (parent 1627)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (8 0))
    ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((9 2) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 5 1) (trans 4 1) (trans 2 1) (trans 9 0)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 8 1) (trans 8 0)
    (neq a self) (undisclosed l) (undisclosed l-1))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1670)
  (parent 1631)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb x rndx) (w expt) (y-0 l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-1)
    (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 1) (13 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)) ((13 2) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 lb l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-1) (y-0 lb) (y-0 (mul x w))
    (y-0 l-1) (x-0 beta) (x-0 l-1))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 7 1) (trans 6 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 6 0) (trans 4 0) (trans 2 0) (trans 11 1) (trans 11 0)
    (neq a b) (undisclosed l-1) (undisclosed l-0))
  (operation channel-test (displaced 14 4 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1))
              (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-2 (cat pt-6 (pv a l)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 1671)
  (parent 1634)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x rndx) (w expt) (y-0 l-0 rndx) (beta expt)
    (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-2)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-2) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-2) (x x-0) (beta beta))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 1) (13 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)) ((13 2) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (y-0 l) (y-0 (mul x w))
    (y-0 l-2) (x-0 beta) (x-0 l-2))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b l-2))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 7 1) (trans 6 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 6 0) (trans 4 0) (trans 2 0) (trans 11 1) (trans 11 0)
    (neq a b) (undisclosed l) (undisclosed l-1))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv a-0 l-3))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 1672)
  (parent 1634)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 rndx) (beta expt)
    (x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-1) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((6 1) (13 0)) ((7 1) (11 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)) ((13 2) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 lb l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)) (x-0 beta) (x-0 l-1))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 7 1) (trans 6 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 6 0) (trans 4 0) (trans 2 0) (trans 11 1) (trans 11 0)
    (neq a b) (undisclosed l) (undisclosed l-1))
  (operation channel-test (displaced 14 6 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 1673)
  (parent 1634)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-2)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (11 0)) ((7 1) (13 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)) ((13 2) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 lb l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)) (x-0 beta) (x-0 l-1))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 7 1) (trans 6 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 6 0) (trans 4 0) (trans 2 0) (trans 11 1) (trans 11 0)
    (neq a b) (undisclosed l) (undisclosed l-0))
  (operation channel-test (displaced 14 7 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 1674)
  (parent 1634)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 na-1 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 rndx)
    (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a a-0) (b b-0) (priv-stor priv-stor-3)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-3) (self a-0)
    (priv-stor priv-stor-3) (l l-2))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1)) ((8 1) (0 4))
    ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (10 0))
    ((13 2) (12 0)) ((14 1) (13 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 lb l l-0 l-1 l-2)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)) (x-0 beta) (x-0 l-2))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 14 1) (trans 7 1) (trans 6 1) (trans 4 1) (trans 2 1)
    (trans 14 0) (trans 7 0) (trans 6 0) (trans 4 0) (trans 2 0)
    (trans 11 1) (trans 11 0) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-3 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-3 (cat pt-9 ignore-3))
      (stor priv-stor-3 (cat pt-8 (pv a-0 l-2)))))
  (label 1675)
  (parent 1634)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb x rndx) (w expt) (y-0 l rndx) (alpha zeta-0 expt)
    (l-0 y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-1)
    (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l))
  (deflistener (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b a)
    (priv-stor priv-stor-1) (lb l-1) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 1) (13 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)) ((13 3) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 lb l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-1) (y-0 lb) (y-0 (mul x w))
    (y-0 l-1) (y-1 alpha) (y-1 zeta-0) (y-1 l-1))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 7 1) (trans 6 1) (trans 4 1)
    (trans 2 1) (trans 7 0) (trans 6 0) (trans 4 0) (trans 2 0)
    (trans 11 1) (trans 11 0) (neq a b) (undisclosed l-1)
    (undisclosed l-0))
  (operation channel-test (displaced 14 4 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1))
              (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-2 (cat pt-6 (pv a l)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-0 (rec y-1)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1676)
  (parent 1639)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x rndx) (w expt) (y-0 l-0 rndx) (alpha zeta-0 expt)
    (l-1 y-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-2)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-2) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-2) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 1) (13 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)) ((13 3) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (y-0 l) (y-0 (mul x w))
    (y-0 l-2) (y-1 alpha) (y-1 zeta-0) (y-1 l-2))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b l-2))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 7 1) (trans 6 1) (trans 4 1)
    (trans 2 1) (trans 7 0) (trans 6 0) (trans 4 0) (trans 2 0)
    (trans 11 1) (trans 11 0) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv b-0 l-3))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-1 (rec y-1)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1677)
  (parent 1639)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 rndx)
    (alpha zeta-0 expt) (y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul (rec y-1) l-1)))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-1) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((6 1) (13 0)) ((7 1) (11 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)) ((13 3) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 lb l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)) (y-1 alpha) (y-1 zeta-0) (y-1 l-1))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 7 1) (trans 6 1) (trans 4 1)
    (trans 2 1) (trans 7 0) (trans 6 0) (trans 4 0) (trans 2 0)
    (trans 11 1) (trans 11 0) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (operation channel-test (displaced 14 6 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul (rec y-1) l-1)))
      (send (cat (exp (gen) y-1) (mul (rec y-1) l-1))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1678)
  (parent 1639)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 rndx) (alpha zeta-0 expt)
    (l-0 y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b a)
    (priv-stor priv-stor-2) (lb l-1) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (11 0)) ((7 1) (13 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)) ((13 3) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 lb l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)) (y-1 alpha) (y-1 zeta-0) (y-1 l-1))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 7 1) (trans 6 1) (trans 4 1)
    (trans 2 1) (trans 7 0) (trans 6 0) (trans 4 0) (trans 2 0)
    (trans 11 1) (trans 11 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation channel-test (displaced 14 7 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-0 (rec y-1)))))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1679)
  (parent 1639)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 rndx)
    (alpha zeta-0 expt) (l-1 y-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b-0)
    (priv-stor priv-stor-3) (lb l-2) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-3) (self b-0)
    (priv-stor priv-stor-3) (l l-2))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1)) ((8 1) (0 4))
    ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (10 0))
    ((13 3) (12 0)) ((14 1) (13 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 lb l l-0 l-1 l-2)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)) (y-1 alpha) (y-1 zeta-0) (y-1 l-2))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 14 1) (trans 7 1) (trans 6 1)
    (trans 4 1) (trans 2 1) (trans 14 0) (trans 7 0) (trans 6 0)
    (trans 4 0) (trans 2 0) (trans 11 1) (trans 11 0) (neq a b)
    (undisclosed l) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-1 (rec y-1)))))
    ((load priv-stor-3 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-1)))))))
    ((load priv-stor-3 (cat pt-9 ignore-3))
      (stor priv-stor-3 (cat pt-8 (pv b-0 l-2)))))
  (label 1680)
  (parent 1639)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (9 0))
    ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 11 1) (trans 5 1) (trans 4 1) (trans 2 1) (trans 11 0)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 9 1) (trans 9 0)
    (neq a self) (undisclosed l) (undisclosed l-1))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1681)
  (parent 1640)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (8 0)) ((2 1) (4 0)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 2) (3 0)) ((5 1) (1 0)) ((5 2) (3 0)) ((6 1) (10 0))
    ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 0)) ((11 2) (9 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 11 1) (trans 6 1) (trans 5 1) (trans 2 1) (trans 11 0)
    (trans 6 0) (trans 5 0) (trans 2 0) (trans 10 1) (trans 4 1)
    (trans 10 0) (trans 4 0) (neq self self-0) (undisclosed l-0)
    (undisclosed l-2))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1682)
  (parent 1643)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (alpha zeta-0 expt)
    (l-1 y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-2) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-2) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 1) (15 0)) ((7 2) (5 0)) ((8 1) (0 0))
    ((9 1) (13 0)) ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)) ((15 3) (14 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-2))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 8 1) (trans 7 1)
    (trans 2 1) (trans 9 0) (trans 8 0) (trans 7 0) (trans 2 0)
    (trans 13 1) (trans 6 1) (trans 13 0) (trans 6 0) (neq self b)
    (undisclosed l-2) (undisclosed l-1))
  (operation channel-test (displaced 16 7 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv b-0 l-3))) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-2)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-2) l)) (send (cat (exp (gen) l-2) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-7 (pv self l-0)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv self l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-1 a self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a self
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1683)
  (parent 1644)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (alpha zeta-0 expt)
    (l-1 y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-2) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-2))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 1) (15 0))
    ((2 2) (1 1)) ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0))
    ((9 1) (13 0)) ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)) ((15 3) (14 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-2))
  (precur (14 0))
  (gen-st (pv b l-1) (pv b l-2) (pv self l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 8 1) (trans 7 1)
    (trans 2 1) (trans 9 0) (trans 8 0) (trans 7 0) (trans 2 0)
    (trans 13 1) (trans 6 1) (trans 13 0) (trans 6 0) (neq self b)
    (undisclosed l) (undisclosed l-1))
  (operation channel-test (displaced 16 2 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv b-0 l-3))) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l-2))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-7 (pv self l-0)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-1 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a b
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1684)
  (parent 1644)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 rndx)
    (alpha zeta-0 expt) (y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-2)))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a) (b b)
    (priv-stor priv-stor) (lb l-2) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((8 1) (15 0))
    ((9 1) (13 0)) ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)) ((15 3) (14 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-2))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 8 1) (trans 7 1)
    (trans 2 1) (trans 9 0) (trans 8 0) (trans 7 0) (trans 2 0)
    (trans 13 1) (trans 6 1) (trans 13 0) (trans 6 0) (neq self b)
    (undisclosed l-0) (undisclosed l-2))
  (operation channel-test (displaced 16 8 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv b-0 l-3))) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-2)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-2))))
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-1 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a b
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1685)
  (parent 1644)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (alpha zeta-0 expt)
    (l-1 y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((9 1) (15 0)) ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)) ((15 3) (14 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-2))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 8 1) (trans 7 1)
    (trans 2 1) (trans 9 0) (trans 8 0) (trans 7 0) (trans 2 0)
    (trans 13 1) (trans 6 1) (trans 13 0) (trans 6 0) (neq self b)
    (undisclosed l-0) (undisclosed l-1))
  (operation channel-test (displaced 16 9 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv b-0 l-3))) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-2))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-1 a self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a self
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1686)
  (parent 1644)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pt-10 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 rndx)
    (alpha zeta-0 expt) (l-2 y-0 l-3 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-2 (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a) (b b-0)
    (priv-stor priv-stor-3) (lb l-3) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-3) (self b-0)
    (priv-stor priv-stor-3) (l l-3))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0)) ((12 1) (11 0))
    ((13 2) (12 0)) ((14 1) (12 0)) ((15 3) (14 0)) ((16 1) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 nb-1 l l-0 l-1 l-2 l-3)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-3))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1) (pv b-0 l-3))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 16 1) (trans 9 1) (trans 8 1)
    (trans 7 1) (trans 2 1) (trans 16 0) (trans 9 0) (trans 8 0)
    (trans 7 0) (trans 2 0) (trans 13 1) (trans 6 1) (trans 13 0)
    (trans 6 0) (neq self b) (undisclosed l-0) (undisclosed l-2))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv b-0 l-3))) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-2 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-2 (rec y-0)))))
    ((load priv-stor-3 (cat pt-9 (pv b-0 l-3)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-1 a b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a b-0
            (hash (exp (gen) (mul alpha l-3))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-3 (cat pt-10 ignore-3))
      (stor priv-stor-3 (cat pt-9 (pv b-0 l-3)))))
  (label 1687)
  (parent 1644)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (beta expt)
    (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-2) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-2) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a self) (b b-0) (priv-stor priv-stor-1)
    (la l-2) (x x-0) (beta beta))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 1) (15 0)) ((7 2) (5 0)) ((8 1) (0 0))
    ((9 1) (13 0)) ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)) ((15 2) (14 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (x-0 beta) (x-0 l-2))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 8 1) (trans 7 1) (trans 2 1) (trans 9 0)
    (trans 8 0) (trans 7 0) (trans 2 0) (trans 13 1) (trans 6 1)
    (trans 13 0) (trans 6 0) (neq self b) (undisclosed l-2)
    (undisclosed l-1))
  (operation channel-test (displaced 16 7 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv a l-3))) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-2)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-2) l)) (send (cat (exp (gen) l-2) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-7 (pv self l-0)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv self l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-1 self b-0 (exp (gen) x-0)))))
  (label 1688)
  (parent 1646)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (beta expt)
    (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-2) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-2))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-2) (x x-0) (beta beta))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 1) (15 0))
    ((2 2) (1 1)) ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0))
    ((9 1) (13 0)) ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)) ((15 2) (14 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (x-0 beta) (x-0 l-2))
  (precur (14 0))
  (gen-st (pv b l-1) (pv b l-2) (pv self l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 8 1) (trans 7 1) (trans 2 1) (trans 9 0)
    (trans 8 0) (trans 7 0) (trans 2 0) (trans 13 1) (trans 6 1)
    (trans 13 0) (trans 6 0) (neq self b) (undisclosed l)
    (undisclosed l-1))
  (operation channel-test (displaced 16 2 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv a l-3))) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l-2))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-7 (pv self l-0)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 1689)
  (parent 1646)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 rndx) (beta expt)
    (x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-2)))
  (defstrand init 3 (na na-1) (a b) (b b-0) (priv-stor priv-stor)
    (la l-2) (x x-0) (beta beta))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((8 1) (15 0))
    ((9 1) (13 0)) ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)) ((15 2) (14 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (x-0 beta) (x-0 l-2))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 8 1) (trans 7 1) (trans 2 1) (trans 9 0)
    (trans 8 0) (trans 7 0) (trans 2 0) (trans 13 1) (trans 6 1)
    (trans 13 0) (trans 6 0) (neq self b) (undisclosed l-0)
    (undisclosed l-2))
  (operation channel-test (displaced 16 8 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv a l-3))) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-2)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-2))))
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 1690)
  (parent 1646)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (beta expt)
    (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a self) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((9 1) (15 0)) ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)) ((15 2) (14 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (x-0 beta) (x-0 l-2))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 8 1) (trans 7 1) (trans 2 1) (trans 9 0)
    (trans 8 0) (trans 7 0) (trans 2 0) (trans 13 1) (trans 6 1)
    (trans 13 0) (trans 6 0) (neq self b) (undisclosed l-0)
    (undisclosed l-1))
  (operation channel-test (displaced 16 9 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv a l-3))) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-2))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-1 self b-0 (exp (gen) x-0)))))
  (label 1691)
  (parent 1646)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 na-1 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pt-10 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 rndx)
    (beta expt) (l-2 x-0 l-3 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-2 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-3)
    (la l-3) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-3) (self a)
    (priv-stor priv-stor-3) (l l-3))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0)) ((12 1) (11 0))
    ((13 2) (12 0)) ((14 1) (12 0)) ((15 2) (14 0)) ((16 1) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 na-1 l l-0 l-1 l-2 l-3)
  (uniq-gen y x x-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (x-0 beta) (x-0 l-3))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1) (pv a l-3))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 16 1) (trans 9 1) (trans 8 1) (trans 7 1) (trans 2 1)
    (trans 16 0) (trans 9 0) (trans 8 0) (trans 7 0) (trans 2 0)
    (trans 13 1) (trans 6 1) (trans 13 0) (trans 6 0) (neq self b)
    (undisclosed l-0) (undisclosed l-2))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv a l-3))) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-2 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-2 (rec x-0)))))
    ((load priv-stor-3 (cat pt-9 (pv a l-3)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0))))
    ((load priv-stor-3 (cat pt-10 ignore-3))
      (stor priv-stor-3 (cat pt-9 (pv a l-3)))))
  (label 1692)
  (parent 1646)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (9 0)) ((1 2) (3 0)) ((2 1) (5 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (1 0))
    ((6 2) (4 0)) ((7 1) (11 0)) ((7 2) (0 1)) ((8 1) (0 4))
    ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (10 0))
    ((13 1) (0 0)) ((13 2) (12 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (12 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 13 1) (trans 7 1) (trans 6 1) (trans 2 1) (trans 13 0)
    (trans 7 0) (trans 6 0) (trans 2 0) (trans 11 1) (trans 5 1)
    (trans 11 0) (trans 5 0) (neq self self-0) (undisclosed l-0)
    (undisclosed l-2))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1693)
  (parent 1651)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (10 0)) ((1 2) (3 0)) ((2 1) (5 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (1 0)) ((7 2) (6 0)) ((8 1) (12 0)) ((8 2) (0 1))
    ((9 1) (0 4)) ((10 1) (9 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 0)) ((13 2) (11 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 13 1) (trans 8 1) (trans 7 1) (trans 2 1) (trans 13 0)
    (trans 8 0) (trans 7 0) (trans 2 0) (trans 12 1) (trans 5 1)
    (trans 12 0) (trans 5 0) (neq self self-0) (undisclosed l-0)
    (undisclosed l-2))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1694)
  (parent 1654)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 lb rndx)
    (alpha zeta-0 expt) (l-2 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-2 (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a) (b b-0)
    (priv-stor priv-stor-3) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4)) ((12 1) (11 0))
    ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0)) ((16 3) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta-0))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 9 1) (trans 8 1)
    (trans 2 1) (trans 10 0) (trans 9 0) (trans 8 0) (trans 2 0)
    (trans 14 1) (trans 6 1) (trans 14 0) (trans 6 0) (neq self b)
    (undisclosed l-0) (undisclosed l-2))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-2 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-2 (rec y-0)))))
    ((load priv-stor-3 (cat pt-9 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-1 a b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1695)
  (parent 1655)
  (unrealized (13 0) (15 0) (16 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x rndx)
    (upsilon expt) (l l-0 l-1 l-2 rndx) (zeta expt) (y rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) y) (mul l-1 (rec y))))
  (precedes ((0 3) (15 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4)) ((12 1) (11 0))
    ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l-1) (y l-2) (y zeta))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 9 1) (trans 8 1) (trans 2 1) (trans 10 0)
    (trans 9 0) (trans 8 0) (trans 2 0) (trans 14 1) (trans 6 1)
    (trans 14 0) (trans 6 0) (neq self b) (undisclosed l-0)
    (undisclosed l-1))
  (operation nonce-test (displaced 16 0 resp 4) (exp (gen) y-0) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-2))
    ((recv (cat (exp (gen) y) (mul l-1 (rec y))))
      (send (cat (exp (gen) y) (mul l-1 (rec y))))))
  (label 1696)
  (parent 1655)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 la rndx)
    (beta expt) (l-2 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-2 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-3)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4)) ((12 1) (11 0))
    ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0)) ((16 2) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1) (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 9 1) (trans 8 1) (trans 2 1) (trans 10 0)
    (trans 9 0) (trans 8 0) (trans 2 0) (trans 14 1) (trans 6 1)
    (trans 14 0) (trans 6 0) (neq self b) (undisclosed l-0)
    (undisclosed l-2))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-2 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-2 (rec x-0)))))
    ((load priv-stor-3 (cat pt-9 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 1697)
  (parent 1655)
  (unrealized (13 0) (15 0) (16 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (gen) l-2))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 1) (15 0)) ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 9 1) (trans 8 1) (trans 2 1) (trans 10 0)
    (trans 9 0) (trans 8 0) (trans 2 0) (trans 14 1) (trans 6 1)
    (trans 14 0) (trans 6 0) (neq self b) (undisclosed l-0)
    (undisclosed l-2))
  (operation nonce-test (contracted (l-3 l-2) (w l-2)) (gen) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2))))
  (label 1698)
  (parent 1655)
  (unrealized (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta upsilon expt) (l l-0 la l-1 x rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l-1) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-1))
  (deflistener
    (hash (exp (gen) (mul la l-1)) (exp (gen) (mul upsilon x))))
  (deflistener
    (cat (exp (gen) (mul la l-1)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) la) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l la))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((1 2) (15 0)) ((2 1) (6 0))
    ((2 2) (1 1)) ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 la l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x la) (x l-1))
  (precur (7 0) (15 0))
  (gen-st (pv b l-0) (pv b l-1) (pv self l) (pv self la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 9 1) (trans 8 1) (trans 2 1) (trans 10 0)
    (trans 9 0) (trans 8 0) (trans 2 0) (trans 14 1) (trans 6 1)
    (trans 14 0) (trans 6 0) (neq self b) (undisclosed la)
    (undisclosed l-0))
  (operation nonce-test (displaced 16 1 init 3) (exp (gen) x-0) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self la)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul la l-1))
              (exp (gen) (mul upsilon x)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul la l-1)) (exp (gen) (mul upsilon x))))
      (send
        (hash (exp (gen) (mul la l-1)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul la l-1)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul la l-1)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) la) l-1)) (send (cat (exp (gen) la) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l-1))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-2 (cat pt-7 (pv self l)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 1699)
  (parent 1655)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-2) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-2) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((8 2) (15 0))
    ((9 1) (0 0)) ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 9 1) (trans 8 1) (trans 2 1) (trans 10 0)
    (trans 9 0) (trans 8 0) (trans 2 0) (trans 14 1) (trans 6 1)
    (trans 14 0) (trans 6 0) (neq self b) (undisclosed l-2)
    (undisclosed l-1))
  (operation nonce-test (displaced 16 8 ltx-gen 3) (exp (gen) l-3)
    (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-2)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-2) l)) (send (cat (exp (gen) l-2) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-7 (pv self l-0)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2))))))
  (label 1700)
  (parent 1655)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-2) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-2))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-2))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((2 2) (15 0)) ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2))
  (precur (7 0) (15 0))
  (gen-st (pv b l-1) (pv b l-2) (pv self l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 9 1) (trans 8 1) (trans 2 1) (trans 10 0)
    (trans 9 0) (trans 8 0) (trans 2 0) (trans 14 1) (trans 6 1)
    (trans 14 0) (trans 6 0) (neq self b) (undisclosed l)
    (undisclosed l-1))
  (operation nonce-test (displaced 16 2 ltx-gen 3) (exp (gen) l-3)
    (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l-2))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-7 (pv self l-0)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2))))))
  (label 1701)
  (parent 1655)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (13 0))
    ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0)) ((12 1) (11 0))
    ((13 2) (12 0)) ((14 1) (12 0)) ((15 1) (0 0)) ((15 2) (14 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0) (14 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 15 1) (trans 9 1) (trans 8 1) (trans 2 1) (trans 15 0)
    (trans 9 0) (trans 8 0) (trans 2 0) (trans 13 1) (trans 6 1)
    (trans 13 0) (trans 6 0) (neq self self-0) (undisclosed l-0)
    (undisclosed l-2))
  (operation nonce-test (displaced 9 16 ltx-gen 3) (exp (gen) l-3)
    (15 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1702)
  (parent 1655)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (14 0)) ((10 2) (0 1)) ((10 2) (15 0)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 9 1) (trans 8 1) (trans 2 1) (trans 10 0)
    (trans 9 0) (trans 8 0) (trans 2 0) (trans 14 1) (trans 6 1)
    (trans 14 0) (trans 6 0) (neq self b) (undisclosed l-0)
    (undisclosed l-1))
  (operation nonce-test (displaced 16 10 ltx-gen 3) (exp (gen) l-3)
    (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-2))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2))))))
  (label 1703)
  (parent 1655)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pt-10 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (x rndx) (upsilon expt)
    (l l-0 l-1 l-2 l-3 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul l-2 (rec l-3))))
  (defstrand ltx-gen 3 (ignore ignore-3) (self self-0)
    (priv-stor priv-stor-3) (l l-3))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4)) ((12 1) (11 0))
    ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0)) ((16 2) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2 l-3)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 16 1) (trans 10 1) (trans 9 1) (trans 8 1) (trans 2 1)
    (trans 16 0) (trans 10 0) (trans 9 0) (trans 8 0) (trans 2 0)
    (trans 14 1) (trans 6 1) (trans 14 0) (trans 6 0) (neq self b)
    (undisclosed l-0) (undisclosed l-2))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-3) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-3) (mul l-2 (rec l-3))))
      (send (cat (exp (gen) l-3) (mul l-2 (rec l-3)))))
    ((load priv-stor-3 (cat pt-9 ignore-3))
      (stor priv-stor-3 (cat pt-10 (pv self-0 l-3)))
      (send
        (sig (body self-0 (exp (gen) l-3) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1704)
  (parent 1655)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (1 1))
    ((2 2) (7 0)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (6 0)) ((5 2) (0 1)) ((6 2) (0 4)) ((7 1) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 4 1) (trans 2 1)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 6 1) (trans 6 0)
    (neq a b) (undisclosed l) (undisclosed l-1))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 1705)
  (parent 1657)
  (seen 1656)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (6 0)) ((5 2) (0 1))
    ((6 2) (0 4)) ((7 1) (0 4)) ((8 1) (0 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (precur (7 0))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 8 1) (trans 5 1) (trans 4 1)
    (trans 2 1) (trans 8 0) (trans 5 0) (trans 4 0) (trans 2 0)
    (trans 6 1) (trans 6 0) (neq a self) (undisclosed l)
    (undisclosed l-1))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1706)
  (parent 1659)
  (seen 1658)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (1 1))
    ((2 2) (6 0)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (7 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 2) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 4 1) (trans 2 1) (trans 5 0) (trans 4 0)
    (trans 2 0) (trans 7 1) (trans 7 0) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0)))
  (label 1707)
  (parent 1660)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (9 0)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (8 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 4 1) (trans 2 1) (trans 5 0) (trans 4 0)
    (trans 2 0) (trans 8 1) (trans 8 0) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 1708)
  (parent 1665)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (7 0)) ((5 2) (0 1))
    ((6 1) (0 4)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 8 1) (trans 5 1) (trans 4 1) (trans 2 1) (trans 8 0)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 7 1) (trans 7 0)
    (neq a self) (undisclosed l) (undisclosed l-1))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1709)
  (parent 1670)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (8 0))
    ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 5 1) (trans 4 1) (trans 2 1) (trans 10 0)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 8 1) (trans 8 0)
    (neq a self) (undisclosed l) (undisclosed l-1))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1710)
  (parent 1681)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (7 0)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 2) (1 3))
    ((4 1) (1 0)) ((4 2) (1 3)) ((5 1) (9 0)) ((5 2) (0 1))
    ((6 1) (0 4)) ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (0 0)) ((10 2) (8 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 5 1) (trans 4 1) (trans 2 1) (trans 10 0)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 9 1) (trans 3 1)
    (trans 9 0) (trans 3 0) (neq self self-0) (undisclosed l-0)
    (undisclosed l-2))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1711)
  (parent 1682)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (8 0)) ((2 1) (4 0)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 2) (3 0)) ((5 1) (1 0)) ((5 2) (3 0)) ((6 1) (10 0))
    ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 0)) ((12 2) (11 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (11 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 12 1) (trans 6 1) (trans 5 1) (trans 2 1) (trans 12 0)
    (trans 6 0) (trans 5 0) (trans 2 0) (trans 10 1) (trans 4 1)
    (trans 10 0) (trans 4 0) (neq self self-0) (undisclosed l-0)
    (undisclosed l-2))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1712)
  (parent 1693)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (9 0)) ((2 1) (4 0)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 1) (11 0)) ((7 2) (0 1)) ((8 1) (0 4)) ((9 1) (8 0))
    ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 0)) ((12 2) (10 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 12 1) (trans 7 1) (trans 6 1) (trans 2 1) (trans 12 0)
    (trans 7 0) (trans 6 0) (trans 2 0) (trans 11 1) (trans 4 1)
    (trans 11 0) (trans 4 0) (neq self self-0) (undisclosed l-0)
    (undisclosed l-2))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1713)
  (parent 1694)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (alpha zeta-0 expt)
    (l-1 y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-2) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-2) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 1) (16 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0))
    ((16 3) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-2))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 9 1) (trans 8 1)
    (trans 2 1) (trans 10 0) (trans 9 0) (trans 8 0) (trans 2 0)
    (trans 14 1) (trans 6 1) (trans 14 0) (trans 6 0) (neq self b)
    (undisclosed l-2) (undisclosed l-1))
  (operation channel-test (displaced 17 8 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv b-0 l-3))) (16 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-2)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-2) l)) (send (cat (exp (gen) l-2) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-7 (pv self l-0)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv self l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-1 a self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a self
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1714)
  (parent 1695)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (alpha zeta-0 expt)
    (l-1 y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-2) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-2))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-2))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 1) (16 0))
    ((2 2) (1 1)) ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0))
    ((16 3) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-2))
  (precur (7 0) (15 0))
  (gen-st (pv b l-1) (pv b l-2) (pv self l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 9 1) (trans 8 1)
    (trans 2 1) (trans 10 0) (trans 9 0) (trans 8 0) (trans 2 0)
    (trans 14 1) (trans 6 1) (trans 14 0) (trans 6 0) (neq self b)
    (undisclosed l) (undisclosed l-1))
  (operation channel-test (displaced 17 2 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv b-0 l-3))) (16 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l-2))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-7 (pv self l-0)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-1 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a b
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1715)
  (parent 1695)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 rndx)
    (alpha zeta-0 expt) (y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-2)))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a) (b b)
    (priv-stor priv-stor) (lb l-2) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 1) (16 0)) ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0))
    ((16 3) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-2))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 9 1) (trans 8 1)
    (trans 2 1) (trans 10 0) (trans 9 0) (trans 8 0) (trans 2 0)
    (trans 14 1) (trans 6 1) (trans 14 0) (trans 6 0) (neq self b)
    (undisclosed l-0) (undisclosed l-2))
  (operation channel-test (displaced 17 9 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv b-0 l-3))) (16 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-2)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-2))))
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-1 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a b
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1716)
  (parent 1695)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (alpha zeta-0 expt)
    (l-1 y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (14 0)) ((10 1) (16 0)) ((10 2) (0 1)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0))
    ((16 3) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-2))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 9 1) (trans 8 1)
    (trans 2 1) (trans 10 0) (trans 9 0) (trans 8 0) (trans 2 0)
    (trans 14 1) (trans 6 1) (trans 14 0) (trans 6 0) (neq self b)
    (undisclosed l-0) (undisclosed l-1))
  (operation channel-test (displaced 17 10 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv b-0 l-3))) (16 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-2))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-1 a self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a self
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1717)
  (parent 1695)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pt-10 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 rndx)
    (alpha zeta-0 expt) (l-2 y-0 l-3 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-2 (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a) (b b-0)
    (priv-stor priv-stor-3) (lb l-3) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-3) (self b-0)
    (priv-stor priv-stor-3) (l l-3))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4)) ((12 1) (11 0))
    ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0)) ((16 3) (15 0))
    ((17 1) (16 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 nb-1 l l-0 l-1 l-2 l-3)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-3))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1) (pv b-0 l-3))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 17 1) (trans 10 1) (trans 9 1)
    (trans 8 1) (trans 2 1) (trans 17 0) (trans 10 0) (trans 9 0)
    (trans 8 0) (trans 2 0) (trans 14 1) (trans 6 1) (trans 14 0)
    (trans 6 0) (neq self b) (undisclosed l-0) (undisclosed l-2))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv b-0 l-3))) (16 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-2 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-2 (rec y-0)))))
    ((load priv-stor-3 (cat pt-9 (pv b-0 l-3)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-1 a b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a b-0
            (hash (exp (gen) (mul alpha l-3))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-3 (cat pt-10 ignore-3))
      (stor priv-stor-3 (cat pt-9 (pv b-0 l-3)))))
  (label 1718)
  (parent 1695)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (beta expt)
    (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-2) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-2) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a self) (b b-0) (priv-stor priv-stor-1)
    (la l-2) (x x-0) (beta beta))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 1) (16 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0))
    ((16 2) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (x-0 beta) (x-0 l-2))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 9 1) (trans 8 1) (trans 2 1) (trans 10 0)
    (trans 9 0) (trans 8 0) (trans 2 0) (trans 14 1) (trans 6 1)
    (trans 14 0) (trans 6 0) (neq self b) (undisclosed l-2)
    (undisclosed l-1))
  (operation channel-test (displaced 17 8 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv a l-3))) (16 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-2)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-2) l)) (send (cat (exp (gen) l-2) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-7 (pv self l-0)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv self l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-1 self b-0 (exp (gen) x-0)))))
  (label 1719)
  (parent 1697)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (beta expt)
    (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-2) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-2))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-2))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-2) (x x-0) (beta beta))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 1) (16 0))
    ((2 2) (1 1)) ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0))
    ((16 2) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (x-0 beta) (x-0 l-2))
  (precur (7 0) (15 0))
  (gen-st (pv b l-1) (pv b l-2) (pv self l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 9 1) (trans 8 1) (trans 2 1) (trans 10 0)
    (trans 9 0) (trans 8 0) (trans 2 0) (trans 14 1) (trans 6 1)
    (trans 14 0) (trans 6 0) (neq self b) (undisclosed l)
    (undisclosed l-1))
  (operation channel-test (displaced 17 2 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv a l-3))) (16 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l-2))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-7 (pv self l-0)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 1720)
  (parent 1697)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 rndx) (beta expt)
    (x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-2)))
  (defstrand init 3 (na na-1) (a b) (b b-0) (priv-stor priv-stor)
    (la l-2) (x x-0) (beta beta))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 1) (16 0)) ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0))
    ((16 2) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (x-0 beta) (x-0 l-2))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 9 1) (trans 8 1) (trans 2 1) (trans 10 0)
    (trans 9 0) (trans 8 0) (trans 2 0) (trans 14 1) (trans 6 1)
    (trans 14 0) (trans 6 0) (neq self b) (undisclosed l-0)
    (undisclosed l-2))
  (operation channel-test (displaced 17 9 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv a l-3))) (16 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-2)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-2))))
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 1721)
  (parent 1697)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (beta expt)
    (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a self) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (14 0)) ((10 1) (16 0)) ((10 2) (0 1)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0))
    ((16 2) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (x-0 beta) (x-0 l-2))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 9 1) (trans 8 1) (trans 2 1) (trans 10 0)
    (trans 9 0) (trans 8 0) (trans 2 0) (trans 14 1) (trans 6 1)
    (trans 14 0) (trans 6 0) (neq self b) (undisclosed l-0)
    (undisclosed l-1))
  (operation channel-test (displaced 17 10 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv a l-3))) (16 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-2))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-1 self b-0 (exp (gen) x-0)))))
  (label 1722)
  (parent 1697)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 na-1 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pt-10 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 rndx)
    (beta expt) (l-2 x-0 l-3 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-2 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-3)
    (la l-3) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-3) (self a)
    (priv-stor priv-stor-3) (l l-3))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4)) ((12 1) (11 0))
    ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0)) ((16 2) (15 0))
    ((17 1) (16 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 na-1 l l-0 l-1 l-2 l-3)
  (uniq-gen y x x-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (x-0 beta) (x-0 l-3))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1) (pv a l-3))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 17 1) (trans 10 1) (trans 9 1) (trans 8 1) (trans 2 1)
    (trans 17 0) (trans 10 0) (trans 9 0) (trans 8 0) (trans 2 0)
    (trans 14 1) (trans 6 1) (trans 14 0) (trans 6 0) (neq self b)
    (undisclosed l-0) (undisclosed l-2))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv a l-3))) (16 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-2 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-2 (rec x-0)))))
    ((load priv-stor-3 (cat pt-9 (pv a l-3)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0))))
    ((load priv-stor-3 (cat pt-10 ignore-3))
      (stor priv-stor-3 (cat pt-9 (pv a l-3)))))
  (label 1723)
  (parent 1697)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (10 0)) ((1 2) (3 0)) ((2 1) (5 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (1 0)) ((7 2) (6 0)) ((8 1) (12 0)) ((8 2) (0 1))
    ((9 1) (0 4)) ((10 1) (9 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (11 0)) ((14 1) (0 0)) ((14 2) (13 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (6 0) (13 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 14 1) (trans 8 1) (trans 7 1) (trans 2 1) (trans 14 0)
    (trans 8 0) (trans 7 0) (trans 2 0) (trans 12 1) (trans 5 1)
    (trans 12 0) (trans 5 0) (neq self self-0) (undisclosed l-0)
    (undisclosed l-2))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1724)
  (parent 1702)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 4))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (6 0)) ((5 2) (0 1)) ((6 2) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 4 1) (trans 2 1) (trans 5 0) (trans 4 0)
    (trans 2 0) (trans 6 1) (trans 6 0) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0)))
  (label 1725)
  (parent 1707)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((a a) (b b) (la l) (lb l-1) (alpha l-0) (beta l-1) (y y)
        (zeta zeta) (na na) (nb nb) (priv-stor priv-stor) (pt pt-0)
        (x x) (upsilon (mul w y-0)) (na-0 na-0) (nb-0 nb-0)
        (priv-stor-0 priv-stor-0) (pt-0 pt-2))))
  (origs (l-1 (2 1)) (pt-0 (2 1)) (pt-5 (6 1)) (l-0 (5 1)) (pt-4 (5 1))
    (nb-0 (3 3)) (l (4 1)) (pt-2 (4 1)) (na-0 (1 2)) (nb (0 3))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (1 1))
    ((2 2) (8 0)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (7 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 4 1) (trans 2 1) (trans 5 0) (trans 4 0)
    (trans 2 0) (trans 7 1) (trans 7 0) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 1726)
  (parent 1708)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (6 0)) ((5 2) (0 1))
    ((6 2) (0 4)) ((7 1) (0 0)) ((7 2) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 7 1) (trans 5 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 6 1) (trans 6 0)
    (neq a self) (undisclosed l) (undisclosed l-1))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1727)
  (parent 1709)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((a a) (b self) (la l) (lb l-1) (alpha l-0) (beta lb) (y y)
        (zeta zeta) (na na) (nb nb) (priv-stor priv-stor-2) (pt pt-7)
        (x x) (upsilon (mul w y-0)) (na-0 na-0) (nb-0 nb-0)
        (priv-stor-0 priv-stor-0) (pt-0 pt-2))))
  (origs (l-1 (7 1)) (pt-7 (7 1)) (pt-5 (6 1)) (l-0 (5 1)) (pt-4 (5 1))
    (nb-0 (3 3)) (l (4 1)) (pt-2 (4 1)) (lb (2 1)) (pt-0 (2 1))
    (na-0 (1 2)) (nb (0 3))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (7 0)) ((5 2) (0 1))
    ((6 1) (0 4)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (8 0))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 5 1) (trans 4 1) (trans 2 1) (trans 9 0)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 7 1) (trans 7 0)
    (neq a self) (undisclosed l) (undisclosed l-1))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1728)
  (parent 1710)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (6 0)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 2) (1 3))
    ((4 1) (1 0)) ((4 2) (1 3)) ((5 1) (8 0)) ((5 2) (0 1))
    ((6 1) (0 4)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 5 1) (trans 4 1) (trans 2 1) (trans 9 0)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 8 1) (trans 3 1)
    (trans 8 0) (trans 3 0) (neq self self-0) (undisclosed l-0)
    (undisclosed l-2))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1729)
  (parent 1711)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (7 0)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 2) (1 3))
    ((4 1) (1 0)) ((4 2) (1 3)) ((5 1) (9 0)) ((5 2) (0 1))
    ((6 1) (0 4)) ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 1) (0 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (10 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 11 1) (trans 5 1) (trans 4 1) (trans 2 1) (trans 11 0)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 9 1) (trans 3 1)
    (trans 9 0) (trans 3 0) (neq self self-0) (undisclosed l-0)
    (undisclosed l-2))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1730)
  (parent 1712)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (8 0)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 2) (1 3))
    ((4 1) (1 3)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 1) (10 0))
    ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 0)) ((11 2) (9 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 11 1) (trans 6 1) (trans 5 1) (trans 2 1) (trans 11 0)
    (trans 6 0) (trans 5 0) (trans 2 0) (trans 10 1) (trans 3 1)
    (trans 10 0) (trans 3 0) (neq self self-0) (undisclosed l-0)
    (undisclosed l-2))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1731)
  (parent 1713)
  (seen 1711)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (9 0)) ((2 1) (4 0)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 1) (11 0)) ((7 2) (0 1)) ((8 1) (0 4)) ((9 1) (8 0))
    ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (10 0)) ((13 1) (0 0))
    ((13 2) (12 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (5 0) (12 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 13 1) (trans 7 1) (trans 6 1) (trans 2 1) (trans 13 0)
    (trans 7 0) (trans 6 0) (trans 2 0) (trans 11 1) (trans 4 1)
    (trans 11 0) (trans 4 0) (neq self self-0) (undisclosed l-0)
    (undisclosed l-2))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1732)
  (parent 1724)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (1 1))
    ((2 2) (7 0)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (6 0)) ((5 2) (0 1)) ((6 2) (0 4)) ((7 1) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 4 1) (trans 2 1) (trans 5 0) (trans 4 0)
    (trans 2 0) (trans 6 1) (trans 6 0) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 1733)
  (parent 1726)
  (seen 1725)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (6 0)) ((5 2) (0 1))
    ((6 2) (0 4)) ((7 1) (0 4)) ((8 1) (0 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (7 0))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 8 1) (trans 5 1) (trans 4 1) (trans 2 1) (trans 8 0)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 6 1) (trans 6 0)
    (neq a self) (undisclosed l) (undisclosed l-1))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1734)
  (parent 1728)
  (seen 1727)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((2 1) (3 0)) ((2 2) (1 1)) ((3 2) (1 3)) ((4 1) (1 0))
    ((4 2) (1 3)) ((5 1) (7 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 8 1) (trans 5 1) (trans 4 1) (trans 2 1) (trans 8 0)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 7 1) (trans 3 1)
    (trans 7 0) (trans 3 0) (neq self self-0) (undisclosed l-0)
    (undisclosed l-2))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1735)
  (parent 1729)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (6 0)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 2) (1 3))
    ((4 1) (1 0)) ((4 2) (1 3)) ((5 1) (8 0)) ((5 2) (0 1))
    ((6 1) (0 4)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (9 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 5 1) (trans 4 1) (trans 2 1) (trans 10 0)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 8 1) (trans 3 1)
    (trans 8 0) (trans 3 0) (neq self self-0) (undisclosed l-0)
    (undisclosed l-2))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1736)
  (parent 1730)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (8 0)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 2) (1 3))
    ((4 1) (1 3)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 1) (10 0))
    ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 0)) ((12 2) (11 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (4 0) (11 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 12 1) (trans 6 1) (trans 5 1) (trans 2 1) (trans 12 0)
    (trans 6 0) (trans 5 0) (trans 2 0) (trans 10 1) (trans 3 1)
    (trans 10 0) (trans 3 0) (neq self self-0) (undisclosed l-0)
    (undisclosed l-2))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1737)
  (parent 1732)
  (seen 1730)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((2 1) (3 0)) ((2 2) (1 1)) ((3 2) (1 3)) ((4 1) (1 0))
    ((4 2) (1 3)) ((5 1) (6 0)) ((5 2) (0 1)) ((6 2) (0 4))
    ((7 1) (0 0)) ((7 2) (0 4)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 7 1) (trans 5 1) (trans 4 1) (trans 2 1) (trans 7 0)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 6 1) (trans 3 1)
    (trans 6 0) (trans 3 0) (neq self self-0) (undisclosed l-0)
    (undisclosed l-2))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1738)
  (parent 1735)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((a self) (b self-0) (la l-0) (lb l-2) (alpha l-1) (beta l) (y y)
        (zeta zeta) (na na) (nb nb) (priv-stor priv-stor-2) (pt pt-8)
        (x x) (upsilon upsilon) (na-0 na-0) (nb-0 nb-0)
        (priv-stor-0 priv-stor-0) (pt-0 pt-3))))
  (origs (l-2 (7 1)) (pt-8 (7 1)) (pt-6 (6 1)) (l-1 (5 1)) (pt-5 (5 1))
    (l-0 (4 1)) (pt-3 (4 1)) (pt-1 (3 1)) (l (2 1)) (pt-0 (2 1))
    (na-0 (1 2)) (nb (0 3))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((2 1) (3 0)) ((2 2) (1 1)) ((3 2) (1 3)) ((4 1) (1 0))
    ((4 2) (1 3)) ((5 1) (7 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 5 1) (trans 4 1) (trans 2 1) (trans 9 0)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 7 1) (trans 3 1)
    (trans 7 0) (trans 3 0) (neq self self-0) (undisclosed l-0)
    (undisclosed l-2))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1739)
  (parent 1736)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((2 1) (3 0)) ((2 2) (1 1)) ((3 2) (1 3)) ((4 1) (1 0))
    ((4 2) (1 3)) ((5 1) (6 0)) ((5 2) (0 1)) ((6 2) (0 4))
    ((7 1) (0 4)) ((8 1) (0 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 8 1) (trans 5 1) (trans 4 1) (trans 2 1) (trans 8 0)
    (trans 5 0) (trans 4 0) (trans 2 0) (trans 6 1) (trans 3 1)
    (trans 6 0) (trans 3 0) (neq self self-0) (undisclosed l-0)
    (undisclosed l-2))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1740)
  (parent 1739)
  (seen 1738)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(comment "Nothing left to do")

(defprotocol dhcr-um diffie-hellman
  (defrole init
    (vars (la x rndx) (beta upsilon expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv a la))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb))
    (uniq-orig na)
    (uniq-gen x)
    (absent (x la) (x beta))
    (assume (fact neq (exp (gen) upsilon) (gen)))
    (gen-st (pv a la)))
  (defrole resp
    (vars (lb y rndx) (alpha zeta expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv b lb))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    (uniq-orig nb)
    (uniq-gen y)
    (absent (y lb) (y alpha) (y zeta))
    (assume (fact neq (exp (gen) zeta) (gen)))
    (gen-st (pv b lb)))
  (defrole ltx-gen
    (vars (self name) (l rndx) (priv-stor locn) (ignore mesg))
    (trace (load priv-stor ignore) (stor priv-stor (pv self l))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    (uniq-orig l))
  (defrole ltx-disclose
    (vars (self name) (l rndx) (priv-stor locn))
    (trace (load priv-stor (pv self l)) (stor priv-stor "nil") (send l))
    (gen-st (pv self l)))
  (defrule undisclosed-not-disclosed
    (forall ((z strd) (l rndx))
      (implies
        (and (fact undisclosed l) (p "ltx-disclose" z 2)
          (p "ltx-disclose" "l" z l))
        (false))))
  (defgenrule neqRl_indx
    (forall ((x indx)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_strd
    (forall ((x strd)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_mesg
    (forall ((x mesg)) (implies (fact neq x x) (false))))
  (defgenrule scissorsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (leads-to z0 i0 z2 i2))
        (and (= z1 z2) (= i1 i2)))))
  (defgenrule cakeRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (leads-to z0 i0 z1 i1)
          (leads-to z0 i0 z2 i2) (prec z1 i1 z2 i2)) (false))))
  (defgenrule no-interruption
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (leads-to z0 i0 z2 i2) (trans z1 i1)
          (same-locn z0 i0 z1 i1) (prec z0 i0 z1 i1) (prec z1 i1 z2 i2))
        (false))))
  (defgenrule shearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (same-locn z0 i0 z2 i2)
          (prec z0 i0 z2 i2))
        (or (and (= z1 z2) (= i1 i2)) (prec z1 i1 z2 i2)))))
  (defgenrule invShearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (same-locn z0 i0 z1 i1)
          (leads-to z1 i1 z2 i2) (prec z0 i0 z2 i2))
        (or (and (= z0 z1) (= i0 i1)) (prec z0 i0 z1 i1)))))
  (defgenrule assume-init-0
    (forall ((z strd) (upsilon expt))
      (implies (and (p "init" z 4) (p "init" "upsilon" z upsilon))
        (fact neq (exp (gen) upsilon) (gen)))))
  (defgenrule assume-resp-0
    (forall ((z strd) (zeta expt))
      (implies (and (p "resp" z 3) (p "resp" "zeta" z zeta))
        (fact neq (exp (gen) zeta) (gen)))))
  (defgenrule trRl_ltx-gen-at-1
    (forall ((z strd)) (implies (p "ltx-gen" z 2) (trans z 1))))
  (defgenrule trRl_ltx-gen-at-0
    (forall ((z strd)) (implies (p "ltx-gen" z 2) (trans z 0))))
  (defgenrule trRl_ltx-disclose-at-1
    (forall ((z strd)) (implies (p "ltx-disclose" z 2) (trans z 1))))
  (defgenrule trRl_ltx-disclose-at-0
    (forall ((z strd)) (implies (p "ltx-disclose" z 2) (trans z 0))))
  (defgenrule gen-st-init-0
    (forall ((z strd) (la rndx) (a name))
      (implies
        (and (p "init" z 1) (p "init" "la" z la) (p "init" "a" z a))
        (gen-st (pv a la)))))
  (defgenrule gen-st-resp-0
    (forall ((z strd) (lb rndx) (b name))
      (implies
        (and (p "resp" z 1) (p "resp" "lb" z lb) (p "resp" "b" z b))
        (gen-st (pv b lb)))))
  (defgenrule gen-st-ltx-disclose-0
    (forall ((z strd) (l rndx) (self name))
      (implies
        (and (p "ltx-disclose" z 1) (p "ltx-disclose" "l" z l)
          (p "ltx-disclose" "self" z self)) (gen-st (pv self l)))))
  (lang (sig sign) (body (tuple 3)) (pv (tuple 2))))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (non-orig (privk "sig" a))
  (uniq-orig nb)
  (uniq-gen y)
  (absent (y lb) (y alpha) (y zeta))
  (facts (neq a b) (undisclosed lb) (undisclosed alpha))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb)))
  (label 1741)
  (unrealized (0 1))
  (origs (nb (0 3)))
  (comment "Not closed under rules"))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (non-orig (privk "sig" a))
  (uniq-orig nb)
  (uniq-gen y)
  (absent (y lb) (y alpha) (y zeta))
  (gen-st (pv b lb))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed lb)
    (undisclosed alpha))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb)))
  (label 1742)
  (parent 1741)
  (unrealized (0 0) (0 1))
  (origs (nb (0 3)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (a b name) (pt pt-0 pval)
    (priv-stor locn) (alpha expt) (y rndx) (zeta expt) (l rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (precedes ((1 1) (0 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l)
  (uniq-gen y)
  (absent (y alpha) (y zeta) (y l))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 1 1) (trans 1 0) (neq a b)
    (undisclosed l) (undisclosed alpha))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv b l))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul alpha l)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l)))))
  (label 1743)
  (parent 1742)
  (unrealized (0 1) (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((2 2) (0 1)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))))
  (label 1744)
  (parent 1743)
  (unrealized (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la y x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la la) (x x) (beta (mul l l-0 (rec la))) (upsilon (mul y (rec x))))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 2) (0 2))
    ((3 4) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec la))) (x la))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-strand init 5) nb (0 4)
    (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a la)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec la))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb)))
  (label 1745)
  (parent 1744)
  (unrealized (3 0) (3 1) (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (3 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) nb (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 1746)
  (parent 1744)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul y (rec x))))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l) (undisclosed l-0))
  (operation channel-test (displaced 4 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb)))
  (label 1747)
  (parent 1745)
  (unrealized (3 1) (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 2) (0 2))
    ((3 4) (0 4)) ((4 1) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 1748)
  (parent 1745)
  (unrealized (3 1) (3 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 4))
    ((4 1) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 1749)
  (parent 1746)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (2 3)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq a self)
    (undisclosed l-0) (undisclosed l))
  (operation nonce-test (displaced 1 4 ltx-gen 3) (exp (gen) l-0) (3 1))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1750)
  (parent 1747)
  (unrealized (2 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx) (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l) (undisclosed l-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (3 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1751)
  (parent 1747)
  (unrealized (3 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 2) (0 2))
    ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l-0)) l-1))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1))))
  (label 1752)
  (parent 1748)
  (unrealized (3 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec l-1))) l-0))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0))))
  (label 1753)
  (parent 1748)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec l-1))) l))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l))))
  (label 1754)
  (parent 1748)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1755)
  (parent 1749)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1756)
  (parent 1749)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq a self)
    (undisclosed l-0) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1757)
  (parent 1750)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq a self)
    (undisclosed l-0) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1758)
  (parent 1750)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l lb rndx)
    (alpha zeta expt) (l-0 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 1759)
  (parent 1751)
  (unrealized (3 1) (3 3) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l la rndx) (beta expt)
    (l-0 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l-0) (undisclosed l))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1760)
  (parent 1751)
  (unrealized (3 1) (3 3) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (gen) l-0))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l-0) (undisclosed l))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 1761)
  (parent 1751)
  (unrealized (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (2 3)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq a self)
    (undisclosed l-0) (undisclosed l))
  (operation nonce-test (displaced 1 5 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1762)
  (parent 1751)
  (unrealized (2 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l) (undisclosed l-0))
  (operation nonce-test (displaced 5 2 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 1763)
  (parent 1751)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1764)
  (parent 1751)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 2) (0 2))
    ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 1765)
  (parent 1752)
  (unrealized (3 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y x l rndx)
    (alpha zeta expt) (y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 3) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 1766)
  (parent 1759)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y x rndx)
    (alpha zeta expt) (l y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 3) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 1767)
  (parent 1759)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx)
    (alpha zeta expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 3) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-1))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (neq a b) (undisclosed l-0) (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 1768)
  (parent 1759)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y x l rndx)
    (beta expt) (x-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l-0) (undisclosed l))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1769)
  (parent 1760)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y x rndx)
    (beta expt) (l x-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l) (undisclosed l-0))
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1770)
  (parent 1760)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l-0) (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a-0 l-1)))))
  (label 1771)
  (parent 1760)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq a self)
    (undisclosed l-0) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1772)
  (parent 1762)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq a self)
    (undisclosed l-0) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1773)
  (parent 1762)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1774)
  (parent 1765)
  (unrealized (3 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1775)
  (parent 1765)
  (unrealized (3 3) (7 0))
  (dead)
  (comment "empty cohort"))

(comment "Nothing left to do")

(defprotocol dhcr-um diffie-hellman
  (defrole init
    (vars (la x rndx) (beta upsilon expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv a la))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb))
    (uniq-orig na)
    (uniq-gen x)
    (absent (x la) (x beta))
    (assume (fact neq (exp (gen) upsilon) (gen)))
    (gen-st (pv a la)))
  (defrole resp
    (vars (lb y rndx) (alpha zeta expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv b lb))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    (uniq-orig nb)
    (uniq-gen y)
    (absent (y lb) (y alpha) (y zeta))
    (assume (fact neq (exp (gen) zeta) (gen)))
    (gen-st (pv b lb)))
  (defrole ltx-gen
    (vars (self name) (l rndx) (priv-stor locn) (ignore mesg))
    (trace (load priv-stor ignore) (stor priv-stor (pv self l))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    (uniq-orig l))
  (defrole ltx-disclose
    (vars (self name) (l rndx) (priv-stor locn))
    (trace (load priv-stor (pv self l)) (stor priv-stor "nil") (send l))
    (gen-st (pv self l)))
  (defrule undisclosed-not-disclosed
    (forall ((z strd) (l rndx))
      (implies
        (and (fact undisclosed l) (p "ltx-disclose" z 2)
          (p "ltx-disclose" "l" z l))
        (false))))
  (defgenrule neqRl_indx
    (forall ((x indx)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_strd
    (forall ((x strd)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_mesg
    (forall ((x mesg)) (implies (fact neq x x) (false))))
  (defgenrule scissorsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (leads-to z0 i0 z2 i2))
        (and (= z1 z2) (= i1 i2)))))
  (defgenrule cakeRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (leads-to z0 i0 z1 i1)
          (leads-to z0 i0 z2 i2) (prec z1 i1 z2 i2)) (false))))
  (defgenrule no-interruption
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (leads-to z0 i0 z2 i2) (trans z1 i1)
          (same-locn z0 i0 z1 i1) (prec z0 i0 z1 i1) (prec z1 i1 z2 i2))
        (false))))
  (defgenrule shearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (same-locn z0 i0 z2 i2)
          (prec z0 i0 z2 i2))
        (or (and (= z1 z2) (= i1 i2)) (prec z1 i1 z2 i2)))))
  (defgenrule invShearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (same-locn z0 i0 z1 i1)
          (leads-to z1 i1 z2 i2) (prec z0 i0 z2 i2))
        (or (and (= z0 z1) (= i0 i1)) (prec z0 i0 z1 i1)))))
  (defgenrule assume-init-0
    (forall ((z strd) (upsilon expt))
      (implies (and (p "init" z 4) (p "init" "upsilon" z upsilon))
        (fact neq (exp (gen) upsilon) (gen)))))
  (defgenrule assume-resp-0
    (forall ((z strd) (zeta expt))
      (implies (and (p "resp" z 3) (p "resp" "zeta" z zeta))
        (fact neq (exp (gen) zeta) (gen)))))
  (defgenrule trRl_ltx-gen-at-1
    (forall ((z strd)) (implies (p "ltx-gen" z 2) (trans z 1))))
  (defgenrule trRl_ltx-gen-at-0
    (forall ((z strd)) (implies (p "ltx-gen" z 2) (trans z 0))))
  (defgenrule trRl_ltx-disclose-at-1
    (forall ((z strd)) (implies (p "ltx-disclose" z 2) (trans z 1))))
  (defgenrule trRl_ltx-disclose-at-0
    (forall ((z strd)) (implies (p "ltx-disclose" z 2) (trans z 0))))
  (defgenrule gen-st-init-0
    (forall ((z strd) (la rndx) (a name))
      (implies
        (and (p "init" z 1) (p "init" "la" z la) (p "init" "a" z a))
        (gen-st (pv a la)))))
  (defgenrule gen-st-resp-0
    (forall ((z strd) (lb rndx) (b name))
      (implies
        (and (p "resp" z 1) (p "resp" "lb" z lb) (p "resp" "b" z b))
        (gen-st (pv b lb)))))
  (defgenrule gen-st-ltx-disclose-0
    (forall ((z strd) (l rndx) (self name))
      (implies
        (and (p "ltx-disclose" z 1) (p "ltx-disclose" "l" z l)
          (p "ltx-disclose" "self" z self)) (gen-st (pv self l)))))
  (lang (sig sign) (body (tuple 3)) (pv (tuple 2))))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (non-orig (privk "sig" a))
  (uniq-orig nb)
  (uniq-gen y)
  (absent (y lb) (y alpha) (y zeta))
  (facts (neq a b) (undisclosed alpha))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb)))
  (label 1776)
  (unrealized (0 1))
  (origs (nb (0 3)))
  (comment "Not closed under rules"))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (non-orig (privk "sig" a))
  (uniq-orig nb)
  (uniq-gen y)
  (absent (y lb) (y alpha) (y zeta))
  (gen-st (pv b lb))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed alpha))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb)))
  (label 1777)
  (parent 1776)
  (unrealized (0 0) (0 1))
  (origs (nb (0 3)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (a b name) (pt pt-0 pval)
    (priv-stor locn) (alpha expt) (y rndx) (zeta expt) (l rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (precedes ((1 1) (0 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l)
  (uniq-gen y)
  (absent (y alpha) (y zeta) (y l))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 1 1) (trans 1 0) (neq a b)
    (undisclosed alpha))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv b l))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul alpha l)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l)))))
  (label 1778)
  (parent 1777)
  (unrealized (0 1) (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((2 2) (0 1)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l-0))
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))))
  (label 1779)
  (parent 1778)
  (unrealized (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la y x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la la) (x x) (beta (mul l l-0 (rec la))) (upsilon (mul y (rec x))))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 2) (0 2))
    ((3 4) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec la))) (x la))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l-0))
  (operation nonce-test (added-strand init 5) nb (0 4)
    (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a la)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec la))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb)))
  (label 1780)
  (parent 1779)
  (unrealized (3 0) (3 1) (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (3 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l-0))
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) nb (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 1781)
  (parent 1779)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul y (rec x))))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 4 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb)))
  (label 1782)
  (parent 1780)
  (unrealized (3 1) (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 2) (0 2))
    ((3 4) (0 4)) ((4 1) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 1783)
  (parent 1780)
  (unrealized (3 1) (3 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 4))
    ((4 1) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l-0))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 1784)
  (parent 1781)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (2 3)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq a self)
    (undisclosed l))
  (operation nonce-test (displaced 1 4 ltx-gen 3) (exp (gen) l-0) (3 1))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1785)
  (parent 1782)
  (unrealized (2 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx) (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (3 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1786)
  (parent 1782)
  (unrealized (3 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 2) (0 2))
    ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l-0)) l-1))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1))))
  (label 1787)
  (parent 1783)
  (unrealized (3 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec l-1))) l-0))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0))))
  (label 1788)
  (parent 1783)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec l-1))) l))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l))))
  (label 1789)
  (parent 1783)
  (unrealized (3 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1790)
  (parent 1784)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1791)
  (parent 1784)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq a self)
    (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1792)
  (parent 1785)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq a self)
    (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1793)
  (parent 1785)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l lb rndx)
    (alpha zeta expt) (l-0 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 1794)
  (parent 1786)
  (unrealized (3 1) (3 3) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l la rndx) (beta expt)
    (l-0 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1795)
  (parent 1786)
  (unrealized (3 1) (3 3) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (gen) l-0))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 1796)
  (parent 1786)
  (unrealized (3 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (2 3)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq a self)
    (undisclosed l))
  (operation nonce-test (displaced 1 5 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1797)
  (parent 1786)
  (unrealized (2 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l-0))
  (operation nonce-test (displaced 5 2 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 1798)
  (parent 1786)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1799)
  (parent 1786)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 2) (0 2))
    ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 1800)
  (parent 1787)
  (unrealized (3 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor (cat pt (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1)))
  (label 1801)
  (parent 1789)
  (unrealized (3 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor (cat pt (pv b l-0))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 1802)
  (parent 1791)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y x l rndx)
    (alpha zeta expt) (y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 3) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 1803)
  (parent 1794)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y x rndx)
    (alpha zeta expt) (l y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 3) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 1804)
  (parent 1794)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx)
    (alpha zeta expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 3) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-1))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (neq a b) (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 1805)
  (parent 1794)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y x l rndx)
    (beta expt) (x-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1806)
  (parent 1795)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y x rndx)
    (beta expt) (l x-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1807)
  (parent 1795)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a-0 l-1)))))
  (label 1808)
  (parent 1795)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (4 0)
    (ch-msg priv-stor (cat pt (pv b l-0))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 1809)
  (parent 1796)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq a self)
    (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1810)
  (parent 1797)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq a self)
    (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1811)
  (parent 1797)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1812)
  (parent 1800)
  (unrealized (3 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1813)
  (parent 1800)
  (unrealized (3 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (7 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1814)
  (parent 1801)
  (unrealized (3 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 1815)
  (parent 1801)
  (unrealized (3 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation nonce-test (displaced 7 2 ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 1816)
  (parent 1802)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1817)
  (parent 1802)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1818)
  (parent 1809)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1819)
  (parent 1809)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor (cat pt (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 1820)
  (parent 1813)
  (unrealized (3 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 1821)
  (parent 1814)
  (unrealized (3 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (precedes ((0 3) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq a b)
    (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 1822)
  (parent 1816)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l rndx) (zeta expt) (y rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul lb l)) (exp (gen) (mul zeta y))))
  (deflistener (cat (exp (gen) (mul lb l)) (exp (gen) (mul zeta y))))
  (deflistener (cat (exp (gen) l) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l lb))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (precedes ((0 3) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb lb l)
  (uniq-gen y)
  (absent (y lb) (y l) (y zeta))
  (precur (7 0))
  (gen-st (pv b lb))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l))
  (operation nonce-test (displaced 8 0 resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul zeta y))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b lb))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul lb l)) (exp (gen) (mul zeta y))))
      (send (hash (exp (gen) (mul lb l)) (exp (gen) (mul zeta y)))))
    ((recv (cat (exp (gen) (mul lb l)) (exp (gen) (mul zeta y))))
      (send (cat (exp (gen) (mul lb l)) (exp (gen) (mul zeta y)))))
    ((recv (cat (exp (gen) l) lb)) (send (cat (exp (gen) l) lb)))
    ((load priv-stor (cat pt (pv b lb)))
      (stor priv-stor (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y))))))
  (label 1823)
  (parent 1817)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x) (beta beta))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x la) (x beta))
  (precur (7 0))
  (gen-st (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation nonce-test (added-strand init 3) (exp (gen) x) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x)))))
  (label 1824)
  (parent 1817)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 1825)
  (parent 1817)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (3 0)) ((1 2) (0 1)) ((2 1) (0 4)) ((3 1) (2 0))
    ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 1) (5 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (6 0))
  (gen-st (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 7 1) (trans 1 1)
    (trans 7 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq a self)
    (undisclosed l))
  (operation nonce-test (displaced 1 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1826)
  (parent 1817)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1827)
  (parent 1817)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation nonce-test (displaced 8 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1828)
  (parent 1817)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0 l-1)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 8 1) (trans 2 1)
    (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1829)
  (parent 1817)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 1830)
  (parent 1820)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx)
    (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1831)
  (parent 1820)
  (unrealized (3 3) (9 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0)))
  (label 1832)
  (parent 1821)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx)
    (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1833)
  (parent 1821)
  (unrealized (3 3) (9 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (3 0))
    ((3 1) (0 4)) ((4 2) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b)
    (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 1834)
  (parent 1822)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta beta expt) (l x l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x) (beta beta))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x beta) (x l-0))
  (precur (7 0))
  (gen-st (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x)))))
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x)))))
  (label 1835)
  (parent 1824)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l rndx) (beta expt) (x l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x) (mul (rec x) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta beta))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x beta) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul (rec x) l-0)))
      (send (cat (exp (gen) x) (mul (rec x) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x)))))
  (label 1836)
  (parent 1824)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x beta) (x l-1))
  (precur (7 0))
  (gen-st (pv b l) (pv a-0 l-1))
  (facts (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1)
    (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1837)
  (parent 1824)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0) (y-0 l-0))
  (precur (7 0))
  (gen-st (pv b l-0))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b) (undisclosed l))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1838)
  (parent 1827)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0) (y-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1839)
  (parent 1827)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0 l-1)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (7 0))
  (gen-st (pv b l) (pv b-0 l-1))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1840)
  (parent 1827)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 3) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (6 0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq a b)
    (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1841)
  (parent 1828)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1842)
  (parent 1830)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1843)
  (parent 1830)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 lb rndx) (alpha zeta expt) (l-1 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-1))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 1844)
  (parent 1831)
  (unrealized (3 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1845)
  (parent 1831)
  (unrealized (3 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 1846)
  (parent 1831)
  (unrealized (3 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 1847)
  (parent 1831)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta (mul (rec l) l-0 l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (2 3)) ((1 2) (0 1)) ((2 2) (0 2)) ((2 4) (0 4))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self) (undisclosed l-0))
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1848)
  (parent 1831)
  (unrealized (2 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-0))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1849)
  (parent 1831)
  (unrealized (3 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 10 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1850)
  (parent 1831)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1851)
  (parent 1832)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1852)
  (parent 1832)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 lb rndx) (alpha zeta expt) (l-1 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-1))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 1853)
  (parent 1833)
  (unrealized (3 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1854)
  (parent 1833)
  (unrealized (3 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 1855)
  (parent 1833)
  (unrealized (3 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 1856)
  (parent 1833)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta (mul (rec l) l-0 l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (2 3)) ((1 2) (0 1)) ((2 2) (0 2)) ((2 4) (0 4))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self) (undisclosed l-0))
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1857)
  (parent 1833)
  (unrealized (2 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-0))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1858)
  (parent 1833)
  (unrealized (3 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 10 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1859)
  (parent 1833)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b)
    (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 1860)
  (parent 1834)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b) (lb l) (alpha l-0) (y y) (zeta zeta) (na na) (nb nb)
        (priv-stor priv-stor) (pt pt))))
  (origs (l-0 (2 1)) (pt-2 (2 1)) (pt-3 (3 1)) (l (1 1)) (pt (1 1))
    (nb (0 3))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 4)) ((4 2) (3 0)) ((5 1) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (5 0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b)
    (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1861)
  (parent 1841)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 rndx)
    (alpha zeta expt) (y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-1))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1862)
  (parent 1844)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx)
    (alpha zeta expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1863)
  (parent 1844)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx)
    (alpha zeta expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1864)
  (parent 1844)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 rndx) (alpha zeta expt) (l-1 y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 1865)
  (parent 1844)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 rndx)
    (beta expt) (x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1866)
  (parent 1845)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1867)
  (parent 1845)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1868)
  (parent 1845)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 1869)
  (parent 1845)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1870)
  (parent 1847)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1871)
  (parent 1847)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 rndx)
    (alpha zeta expt) (y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-1))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1872)
  (parent 1853)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx)
    (alpha zeta expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1873)
  (parent 1853)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx)
    (alpha zeta expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1874)
  (parent 1853)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 rndx) (alpha zeta expt) (l-1 y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 1875)
  (parent 1853)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 rndx)
    (beta expt) (x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1876)
  (parent 1854)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1877)
  (parent 1854)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1878)
  (parent 1854)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 1879)
  (parent 1854)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1880)
  (parent 1856)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1881)
  (parent 1856)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 2) (0 4)) ((4 1) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b)
    (undisclosed l-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1882)
  (parent 1861)
  (seen 1860)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(comment "Nothing left to do")

(defprotocol dhcr-um diffie-hellman
  (defrole init
    (vars (la x rndx) (beta upsilon expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv a la))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb))
    (uniq-orig na)
    (uniq-gen x)
    (absent (x la) (x beta))
    (assume (fact neq (exp (gen) upsilon) (gen)))
    (gen-st (pv a la)))
  (defrole resp
    (vars (lb y rndx) (alpha zeta expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv b lb))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    (uniq-orig nb)
    (uniq-gen y)
    (absent (y lb) (y alpha) (y zeta))
    (assume (fact neq (exp (gen) zeta) (gen)))
    (gen-st (pv b lb)))
  (defrole ltx-gen
    (vars (self name) (l rndx) (priv-stor locn) (ignore mesg))
    (trace (load priv-stor ignore) (stor priv-stor (pv self l))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    (uniq-orig l))
  (defrole ltx-disclose
    (vars (self name) (l rndx) (priv-stor locn))
    (trace (load priv-stor (pv self l)) (stor priv-stor "nil") (send l))
    (gen-st (pv self l)))
  (defrule undisclosed-not-disclosed
    (forall ((z strd) (l rndx))
      (implies
        (and (fact undisclosed l) (p "ltx-disclose" z 2)
          (p "ltx-disclose" "l" z l))
        (false))))
  (defgenrule neqRl_indx
    (forall ((x indx)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_strd
    (forall ((x strd)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_mesg
    (forall ((x mesg)) (implies (fact neq x x) (false))))
  (defgenrule scissorsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (leads-to z0 i0 z2 i2))
        (and (= z1 z2) (= i1 i2)))))
  (defgenrule cakeRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (leads-to z0 i0 z1 i1)
          (leads-to z0 i0 z2 i2) (prec z1 i1 z2 i2)) (false))))
  (defgenrule no-interruption
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (leads-to z0 i0 z2 i2) (trans z1 i1)
          (same-locn z0 i0 z1 i1) (prec z0 i0 z1 i1) (prec z1 i1 z2 i2))
        (false))))
  (defgenrule shearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (same-locn z0 i0 z2 i2)
          (prec z0 i0 z2 i2))
        (or (and (= z1 z2) (= i1 i2)) (prec z1 i1 z2 i2)))))
  (defgenrule invShearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (same-locn z0 i0 z1 i1)
          (leads-to z1 i1 z2 i2) (prec z0 i0 z2 i2))
        (or (and (= z0 z1) (= i0 i1)) (prec z0 i0 z1 i1)))))
  (defgenrule assume-init-0
    (forall ((z strd) (upsilon expt))
      (implies (and (p "init" z 4) (p "init" "upsilon" z upsilon))
        (fact neq (exp (gen) upsilon) (gen)))))
  (defgenrule assume-resp-0
    (forall ((z strd) (zeta expt))
      (implies (and (p "resp" z 3) (p "resp" "zeta" z zeta))
        (fact neq (exp (gen) zeta) (gen)))))
  (defgenrule trRl_ltx-gen-at-1
    (forall ((z strd)) (implies (p "ltx-gen" z 2) (trans z 1))))
  (defgenrule trRl_ltx-gen-at-0
    (forall ((z strd)) (implies (p "ltx-gen" z 2) (trans z 0))))
  (defgenrule trRl_ltx-disclose-at-1
    (forall ((z strd)) (implies (p "ltx-disclose" z 2) (trans z 1))))
  (defgenrule trRl_ltx-disclose-at-0
    (forall ((z strd)) (implies (p "ltx-disclose" z 2) (trans z 0))))
  (defgenrule gen-st-init-0
    (forall ((z strd) (la rndx) (a name))
      (implies
        (and (p "init" z 1) (p "init" "la" z la) (p "init" "a" z a))
        (gen-st (pv a la)))))
  (defgenrule gen-st-resp-0
    (forall ((z strd) (lb rndx) (b name))
      (implies
        (and (p "resp" z 1) (p "resp" "lb" z lb) (p "resp" "b" z b))
        (gen-st (pv b lb)))))
  (defgenrule gen-st-ltx-disclose-0
    (forall ((z strd) (l rndx) (self name))
      (implies
        (and (p "ltx-disclose" z 1) (p "ltx-disclose" "l" z l)
          (p "ltx-disclose" "self" z self)) (gen-st (pv self l)))))
  (lang (sig sign) (body (tuple 3)) (pv (tuple 2))))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (lb y rndx)
    (alpha zeta expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (non-orig (privk "sig" a))
  (uniq-orig nb)
  (uniq-gen y)
  (absent (y lb) (y alpha) (y zeta))
  (facts (neq a b) (undisclosed lb))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb)))
  (label 1883)
  (unrealized (0 1))
  (origs (nb (0 3)))
  (comment "Not closed under rules"))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (lb y rndx)
    (alpha zeta expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (non-orig (privk "sig" a))
  (uniq-orig nb)
  (uniq-gen y)
  (absent (y lb) (y alpha) (y zeta))
  (gen-st (pv b lb))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed lb))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb)))
  (label 1884)
  (parent 1883)
  (unrealized (0 0) (0 1))
  (origs (nb (0 3)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (a b name) (pt pt-0 pval)
    (priv-stor locn) (y rndx) (alpha zeta expt) (l rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (precedes ((1 1) (0 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l)
  (uniq-gen y)
  (absent (y alpha) (y zeta) (y l))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 1 1) (trans 1 0) (neq a b)
    (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv b l))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul alpha l)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l)))))
  (label 1885)
  (parent 1884)
  (unrealized (0 1) (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((2 2) (0 1)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l))
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))))
  (label 1886)
  (parent 1885)
  (unrealized (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la y x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la la) (x x) (beta (mul l l-0 (rec la))) (upsilon (mul y (rec x))))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 2) (0 2))
    ((3 4) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec la))) (x la))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l))
  (operation nonce-test (added-strand init 5) nb (0 4)
    (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a la)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec la))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb)))
  (label 1887)
  (parent 1886)
  (unrealized (3 0) (3 1) (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (3 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l))
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) nb (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 1888)
  (parent 1886)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul y (rec x))))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l))
  (operation channel-test (displaced 4 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb)))
  (label 1889)
  (parent 1887)
  (unrealized (3 1) (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 2) (0 2))
    ((3 4) (0 4)) ((4 1) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 1890)
  (parent 1887)
  (unrealized (3 1) (3 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 4))
    ((4 1) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 1891)
  (parent 1888)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (2 3)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq a self)
    (undisclosed l-0))
  (operation nonce-test (displaced 1 4 ltx-gen 3) (exp (gen) l-0) (3 1))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1892)
  (parent 1889)
  (unrealized (2 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx) (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (3 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1893)
  (parent 1889)
  (unrealized (3 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 2) (0 2))
    ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l-0)) l-1))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1))))
  (label 1894)
  (parent 1890)
  (unrealized (3 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec l-1))) l-0))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0))))
  (label 1895)
  (parent 1890)
  (unrealized (3 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec l-1))) l))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l))))
  (label 1896)
  (parent 1890)
  (unrealized (3 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1897)
  (parent 1891)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1898)
  (parent 1891)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq a self)
    (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1899)
  (parent 1892)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq a self)
    (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1900)
  (parent 1892)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l lb rndx)
    (alpha zeta expt) (l-0 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 1901)
  (parent 1893)
  (unrealized (3 1) (3 3) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l la rndx) (beta expt)
    (l-0 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l-0))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1902)
  (parent 1893)
  (unrealized (3 1) (3 3) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (gen) l-0))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l-0))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 1903)
  (parent 1893)
  (unrealized (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (2 3)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq a self)
    (undisclosed l-0))
  (operation nonce-test (displaced 1 5 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1904)
  (parent 1893)
  (unrealized (2 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l))
  (operation nonce-test (displaced 5 2 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 1905)
  (parent 1893)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1906)
  (parent 1893)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 2) (0 2))
    ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 1907)
  (parent 1894)
  (unrealized (3 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1)))
  (label 1908)
  (parent 1895)
  (unrealized (3 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0)))
  (label 1909)
  (parent 1897)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y x l rndx)
    (alpha zeta expt) (y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 3) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 1910)
  (parent 1901)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y x rndx)
    (alpha zeta expt) (l y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 3) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (neq a b) (undisclosed l))
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 1911)
  (parent 1901)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx)
    (alpha zeta expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 3) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-1))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 1912)
  (parent 1901)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y x l rndx)
    (beta expt) (x-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1913)
  (parent 1902)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y x rndx)
    (beta expt) (l x-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b)
    (undisclosed l))
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1914)
  (parent 1902)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (neq a b) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a-0 l-1)))))
  (label 1915)
  (parent 1902)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq a self)
    (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1916)
  (parent 1904)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq a self)
    (undisclosed l-0))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1917)
  (parent 1904)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1918)
  (parent 1907)
  (unrealized (3 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1919)
  (parent 1907)
  (unrealized (3 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (7 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1920)
  (parent 1908)
  (unrealized (3 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b) (undisclosed l))
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 1921)
  (parent 1908)
  (unrealized (3 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 4))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((6 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 1 1)
    (trans 6 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq a self)
    (undisclosed l-0))
  (operation nonce-test (displaced 1 7 ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1922)
  (parent 1909)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1923)
  (parent 1909)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1)))
  (label 1924)
  (parent 1918)
  (unrealized (3 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 1925)
  (parent 1920)
  (unrealized (3 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (2 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 4))
    ((3 1) (2 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((5 2) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 1 1)
    (trans 5 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a self)
    (undisclosed l-0))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1926)
  (parent 1922)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 rndx) (zeta expt) (y rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (precedes ((0 3) (7 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y l) (y l-0) (y zeta))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l))
  (operation nonce-test (displaced 8 0 resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y))))))
  (label 1927)
  (parent 1923)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x) (beta beta))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x la) (x beta))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation nonce-test (added-strand init 3) (exp (gen) x) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x)))))
  (label 1928)
  (parent 1923)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 1929)
  (parent 1923)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 4))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 7 1) (trans 1 1)
    (trans 7 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq a self)
    (undisclosed l-0))
  (operation nonce-test (displaced 1 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1930)
  (parent 1923)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1931)
  (parent 1923)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l))
  (operation nonce-test (displaced 8 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 1932)
  (parent 1923)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0 l-1)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 8 1) (trans 2 1)
    (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b) (undisclosed l-0))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1933)
  (parent 1923)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta (mul (rec l) l-0 l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (2 3)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self) (undisclosed l-1))
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1934)
  (parent 1924)
  (unrealized (2 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx)
    (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1935)
  (parent 1924)
  (unrealized (3 3) (9 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (2 3)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self) (undisclosed l-1))
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1936)
  (parent 1925)
  (unrealized (2 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx)
    (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1937)
  (parent 1925)
  (unrealized (3 3) (9 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 4)) ((3 2) (2 0))
    ((4 1) (0 0)) ((4 2) (2 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a self)
    (undisclosed l-0))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1938)
  (parent 1926)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l rndx) (beta expt) (x l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x) (mul (rec x) l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x) (beta beta))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x beta) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul (rec x) l-0)))
      (send (cat (exp (gen) x) (mul (rec x) l-0))))
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x)))))
  (label 1939)
  (parent 1928)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta beta expt) (l x l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta beta))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x beta) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b)
    (undisclosed l))
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x)))))
  (label 1940)
  (parent 1928)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x beta) (x l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-1))
  (facts (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1)
    (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1941)
  (parent 1928)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (2 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 4))
    ((3 1) (2 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (0 0))
    ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 1 1)
    (trans 6 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a self)
    (undisclosed l-0))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1942)
  (parent 1930)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0) (y-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1943)
  (parent 1931)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0) (y-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b) (undisclosed l))
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1944)
  (parent 1931)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0 l-1)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b) (undisclosed l-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1945)
  (parent 1931)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta (mul (rec l) l-0 l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (9 0)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1946)
  (parent 1934)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta (mul (rec l) l-0 l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (9 0)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1947)
  (parent 1934)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 lb rndx) (alpha zeta expt) (l-1 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-1))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 1948)
  (parent 1935)
  (unrealized (3 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1949)
  (parent 1935)
  (unrealized (3 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 1950)
  (parent 1935)
  (unrealized (3 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 1951)
  (parent 1935)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta (mul (rec l) l-0 l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (2 3)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (8 0))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self) (undisclosed l-1))
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1952)
  (parent 1935)
  (unrealized (2 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (8 0))
  (gen-st (pv b l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-0))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1953)
  (parent 1935)
  (unrealized (3 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 10 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1954)
  (parent 1935)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (9 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1955)
  (parent 1936)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (9 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1956)
  (parent 1936)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 lb rndx) (alpha zeta expt) (l-1 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-1))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 1957)
  (parent 1937)
  (unrealized (3 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1958)
  (parent 1937)
  (unrealized (3 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 1959)
  (parent 1937)
  (unrealized (3 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 1960)
  (parent 1937)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (2 3)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self) (undisclosed l-1))
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1961)
  (parent 1937)
  (unrealized (2 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (8 0))
  (gen-st (pv b l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b) (undisclosed l-0))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1962)
  (parent 1937)
  (unrealized (3 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 10 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1963)
  (parent 1937)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 4)) ((3 1) (0 0))
    ((3 2) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq a self)
    (undisclosed l-0))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1964)
  (parent 1938)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b self) (lb l-0) (y y) (alpha l) (zeta zeta) (na na)
        (nb nb) (priv-stor priv-stor-0) (pt pt-3))))
  (origs (l-0 (3 1)) (pt-3 (3 1)) (pt-1 (2 1)) (l (1 1)) (pt-0 (1 1))
    (nb (0 3))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 4)) ((3 2) (2 0))
    ((4 1) (2 0)) ((5 1) (0 0)) ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 1 1)
    (trans 5 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a self)
    (undisclosed l-0))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1965)
  (parent 1942)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx)
    (alpha zeta expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1966)
  (parent 1948)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 rndx)
    (alpha zeta expt) (y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-1))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1967)
  (parent 1948)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx)
    (alpha zeta expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1968)
  (parent 1948)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 rndx) (alpha zeta expt) (l-1 y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 1969)
  (parent 1948)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1970)
  (parent 1949)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 rndx)
    (beta expt) (x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1971)
  (parent 1949)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1972)
  (parent 1949)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 1973)
  (parent 1949)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta (mul (rec l) l-0 l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (10 0)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (8 0))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1974)
  (parent 1952)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta (mul (rec l) l-0 l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (10 0)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (8 0))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1975)
  (parent 1952)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx)
    (alpha zeta expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1976)
  (parent 1957)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 rndx)
    (alpha zeta expt) (y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-1))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1977)
  (parent 1957)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx)
    (alpha zeta expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b)
    (undisclosed l-0))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 1978)
  (parent 1957)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 rndx) (alpha zeta expt) (l-1 y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 1979)
  (parent 1957)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1980)
  (parent 1958)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 rndx)
    (beta expt) (x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-1))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1981)
  (parent 1958)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b) (undisclosed l-0))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1982)
  (parent 1958)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b) (undisclosed l-1))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 1983)
  (parent 1958)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (10 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 1984)
  (parent 1961)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (10 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self) (undisclosed l-1))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 1985)
  (parent 1961)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 4)) ((3 1) (0 4))
    ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq a self)
    (undisclosed l-0))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1986)
  (parent 1965)
  (seen 1964)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(comment "Nothing left to do")

(defprotocol dhcr-um diffie-hellman
  (defrole init
    (vars (la x rndx) (beta upsilon expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv a la))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb))
    (uniq-orig na)
    (uniq-gen x)
    (absent (x la) (x beta))
    (assume (fact neq (exp (gen) upsilon) (gen)))
    (gen-st (pv a la)))
  (defrole resp
    (vars (lb y rndx) (alpha zeta expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv b lb))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    (uniq-orig nb)
    (uniq-gen y)
    (absent (y lb) (y alpha) (y zeta))
    (assume (fact neq (exp (gen) zeta) (gen)))
    (gen-st (pv b lb)))
  (defrole ltx-gen
    (vars (self name) (l rndx) (priv-stor locn) (ignore mesg))
    (trace (load priv-stor ignore) (stor priv-stor (pv self l))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    (uniq-orig l))
  (defrole ltx-disclose
    (vars (self name) (l rndx) (priv-stor locn))
    (trace (load priv-stor (pv self l)) (stor priv-stor "nil") (send l))
    (gen-st (pv self l)))
  (defrule undisclosed-not-disclosed
    (forall ((z strd) (l rndx))
      (implies
        (and (fact undisclosed l) (p "ltx-disclose" z 2)
          (p "ltx-disclose" "l" z l))
        (false))))
  (defgenrule neqRl_indx
    (forall ((x indx)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_strd
    (forall ((x strd)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_mesg
    (forall ((x mesg)) (implies (fact neq x x) (false))))
  (defgenrule scissorsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (leads-to z0 i0 z2 i2))
        (and (= z1 z2) (= i1 i2)))))
  (defgenrule cakeRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (leads-to z0 i0 z1 i1)
          (leads-to z0 i0 z2 i2) (prec z1 i1 z2 i2)) (false))))
  (defgenrule no-interruption
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (leads-to z0 i0 z2 i2) (trans z1 i1)
          (same-locn z0 i0 z1 i1) (prec z0 i0 z1 i1) (prec z1 i1 z2 i2))
        (false))))
  (defgenrule shearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (same-locn z0 i0 z2 i2)
          (prec z0 i0 z2 i2))
        (or (and (= z1 z2) (= i1 i2)) (prec z1 i1 z2 i2)))))
  (defgenrule invShearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (same-locn z0 i0 z1 i1)
          (leads-to z1 i1 z2 i2) (prec z0 i0 z2 i2))
        (or (and (= z0 z1) (= i0 i1)) (prec z0 i0 z1 i1)))))
  (defgenrule assume-init-0
    (forall ((z strd) (upsilon expt))
      (implies (and (p "init" z 4) (p "init" "upsilon" z upsilon))
        (fact neq (exp (gen) upsilon) (gen)))))
  (defgenrule assume-resp-0
    (forall ((z strd) (zeta expt))
      (implies (and (p "resp" z 3) (p "resp" "zeta" z zeta))
        (fact neq (exp (gen) zeta) (gen)))))
  (defgenrule trRl_ltx-gen-at-1
    (forall ((z strd)) (implies (p "ltx-gen" z 2) (trans z 1))))
  (defgenrule trRl_ltx-gen-at-0
    (forall ((z strd)) (implies (p "ltx-gen" z 2) (trans z 0))))
  (defgenrule trRl_ltx-disclose-at-1
    (forall ((z strd)) (implies (p "ltx-disclose" z 2) (trans z 1))))
  (defgenrule trRl_ltx-disclose-at-0
    (forall ((z strd)) (implies (p "ltx-disclose" z 2) (trans z 0))))
  (defgenrule gen-st-init-0
    (forall ((z strd) (la rndx) (a name))
      (implies
        (and (p "init" z 1) (p "init" "la" z la) (p "init" "a" z a))
        (gen-st (pv a la)))))
  (defgenrule gen-st-resp-0
    (forall ((z strd) (lb rndx) (b name))
      (implies
        (and (p "resp" z 1) (p "resp" "lb" z lb) (p "resp" "b" z b))
        (gen-st (pv b lb)))))
  (defgenrule gen-st-ltx-disclose-0
    (forall ((z strd) (l rndx) (self name))
      (implies
        (and (p "ltx-disclose" z 1) (p "ltx-disclose" "l" z l)
          (p "ltx-disclose" "self" z self)) (gen-st (pv self l)))))
  (lang (sig sign) (body (tuple 3)) (pv (tuple 2))))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (lb y rndx)
    (alpha zeta expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (non-orig (privk "sig" a))
  (uniq-orig nb)
  (uniq-gen y)
  (absent (y lb) (y alpha) (y zeta))
  (facts (neq a b))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb)))
  (label 1987)
  (unrealized (0 1))
  (origs (nb (0 3)))
  (comment "Not closed under rules"))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (lb y rndx)
    (alpha zeta expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (non-orig (privk "sig" a))
  (uniq-orig nb)
  (uniq-gen y)
  (absent (y lb) (y alpha) (y zeta))
  (gen-st (pv b lb))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb)))
  (label 1988)
  (parent 1987)
  (unrealized (0 0) (0 1))
  (origs (nb (0 3)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (a b name) (pt pt-0 pval)
    (priv-stor locn) (y rndx) (alpha zeta expt) (l rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (precedes ((1 1) (0 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l)
  (uniq-gen y)
  (absent (y alpha) (y zeta) (y l))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 1 1) (trans 1 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv b l))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul alpha l)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l)))))
  (label 1989)
  (parent 1988)
  (unrealized (0 1) (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((2 2) (0 1)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))))
  (label 1990)
  (parent 1989)
  (unrealized (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la y x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la la) (x x) (beta (mul l l-0 (rec la))) (upsilon (mul y (rec x))))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 2) (0 2))
    ((3 4) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec la))) (x la))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test (added-strand init 5) nb (0 4)
    (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a la)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec la))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb)))
  (label 1991)
  (parent 1990)
  (unrealized (3 0) (3 1) (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (3 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) nb (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 1992)
  (parent 1990)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul y (rec x))))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b))
  (operation channel-test (displaced 4 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb)))
  (label 1993)
  (parent 1991)
  (unrealized (3 1) (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 2) (0 2))
    ((3 4) (0 4)) ((4 1) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 1994)
  (parent 1991)
  (unrealized (3 1) (3 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 4))
    ((4 1) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 1995)
  (parent 1992)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (2 3)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq a self))
  (operation nonce-test (displaced 1 4 ltx-gen 3) (exp (gen) l-0) (3 1))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1996)
  (parent 1993)
  (unrealized (2 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l y x l-0 rndx) (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (3 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1997)
  (parent 1993)
  (unrealized (3 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 2) (0 2))
    ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l-0)) l-1))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1))))
  (label 1998)
  (parent 1994)
  (unrealized (3 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec l-1))) l-0))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0))))
  (label 1999)
  (parent 1994)
  (unrealized (3 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec l-1))) l))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l))))
  (label 2000)
  (parent 1994)
  (unrealized (3 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 2001)
  (parent 1995)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 2002)
  (parent 1995)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq a self))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 2003)
  (parent 1996)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq a self))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 2004)
  (parent 1996)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l lb rndx)
    (alpha zeta expt) (l-0 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 2005)
  (parent 1997)
  (unrealized (3 1) (3 3) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l la rndx) (beta expt)
    (l-0 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 2006)
  (parent 1997)
  (unrealized (3 1) (3 3) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (gen) l-0))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2007)
  (parent 1997)
  (unrealized (3 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (2 3)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq a self))
  (operation nonce-test (displaced 1 5 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2008)
  (parent 1997)
  (unrealized (2 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test (displaced 5 2 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 2009)
  (parent 1997)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2010)
  (parent 1997)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 2) (0 2))
    ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 2011)
  (parent 1998)
  (unrealized (3 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1)))
  (label 2012)
  (parent 1999)
  (unrealized (3 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor (cat pt (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1)))
  (label 2013)
  (parent 2000)
  (unrealized (3 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0)))
  (label 2014)
  (parent 2001)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor (cat pt (pv b l-0))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 2015)
  (parent 2002)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y x l rndx)
    (alpha zeta expt) (y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 3) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (neq a b))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 2016)
  (parent 2005)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y x rndx)
    (alpha zeta expt) (l y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 3) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (neq a b))
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 2017)
  (parent 2005)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx)
    (alpha zeta expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 3) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-1))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 2018)
  (parent 2005)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y x l rndx)
    (beta expt) (x-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2019)
  (parent 2006)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y x rndx)
    (beta expt) (l x-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta l) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b))
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2020)
  (parent 2006)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a-0 l-1)))))
  (label 2021)
  (parent 2006)
  (unrealized (3 1) (3 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (4 0)
    (ch-msg priv-stor (cat pt (pv b l-0))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 2022)
  (parent 2007)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq a self))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 2023)
  (parent 2008)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq a self))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 2024)
  (parent 2008)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 2025)
  (parent 2011)
  (unrealized (3 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 y x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 2026)
  (parent 2011)
  (unrealized (3 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (7 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 2027)
  (parent 2012)
  (unrealized (3 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 2028)
  (parent 2012)
  (unrealized (3 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (7 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 2029)
  (parent 2013)
  (unrealized (3 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 2030)
  (parent 2013)
  (unrealized (3 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 4))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((6 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 1 1)
    (trans 6 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq a self))
  (operation nonce-test (displaced 1 7 ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2031)
  (parent 2014)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2032)
  (parent 2014)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 7 2 ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 2033)
  (parent 2015)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2034)
  (parent 2015)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 2035)
  (parent 2022)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (y x l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb)) ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 2036)
  (parent 2022)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1)))
  (label 2037)
  (parent 2025)
  (unrealized (3 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor (cat pt (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 2038)
  (parent 2026)
  (unrealized (3 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 2039)
  (parent 2027)
  (unrealized (3 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor (cat pt (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 2040)
  (parent 2028)
  (unrealized (3 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 2041)
  (parent 2029)
  (unrealized (3 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1)))
  (label 2042)
  (parent 2030)
  (unrealized (3 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (2 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 4))
    ((3 1) (2 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((5 2) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 1 1)
    (trans 5 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a self))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2043)
  (parent 2031)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 rndx) (zeta expt) (y rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (precedes ((0 3) (7 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y l) (y l-0) (y zeta))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 8 0 resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y))))))
  (label 2044)
  (parent 2032)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x) (beta beta))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x la) (x beta))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x)))))
  (label 2045)
  (parent 2032)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2046)
  (parent 2032)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 4))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 7 1) (trans 1 1)
    (trans 7 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq a self))
  (operation nonce-test (displaced 1 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2047)
  (parent 2032)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2048)
  (parent 2032)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 8 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 2049)
  (parent 2032)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0 l-1)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 8 1) (trans 2 1)
    (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2050)
  (parent 2032)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (precedes ((0 3) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 2051)
  (parent 2033)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l rndx) (zeta expt) (y rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul lb l)) (exp (gen) (mul zeta y))))
  (deflistener (cat (exp (gen) (mul lb l)) (exp (gen) (mul zeta y))))
  (deflistener (cat (exp (gen) l) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l lb))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (precedes ((0 3) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb lb l)
  (uniq-gen y)
  (absent (y lb) (y l) (y zeta))
  (precur (7 0))
  (gen-st (pv b lb))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 8 0 resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul zeta y))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b lb))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul lb l)) (exp (gen) (mul zeta y))))
      (send (hash (exp (gen) (mul lb l)) (exp (gen) (mul zeta y)))))
    ((recv (cat (exp (gen) (mul lb l)) (exp (gen) (mul zeta y))))
      (send (cat (exp (gen) (mul lb l)) (exp (gen) (mul zeta y)))))
    ((recv (cat (exp (gen) l) lb)) (send (cat (exp (gen) l) lb)))
    ((load priv-stor (cat pt (pv b lb)))
      (stor priv-stor (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y))))))
  (label 2052)
  (parent 2034)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x) (beta beta))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x la) (x beta))
  (precur (7 0))
  (gen-st (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x)))))
  (label 2053)
  (parent 2034)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2054)
  (parent 2034)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (3 0)) ((1 2) (0 1)) ((2 1) (0 4)) ((3 1) (2 0))
    ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 1) (5 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (6 0))
  (gen-st (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 7 1) (trans 1 1)
    (trans 7 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq a self))
  (operation nonce-test (displaced 1 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2055)
  (parent 2034)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2056)
  (parent 2034)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 8 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2057)
  (parent 2034)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0 l-1)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 8 1) (trans 2 1)
    (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2058)
  (parent 2034)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta (mul (rec l) l-0 l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (2 3)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self))
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2059)
  (parent 2037)
  (unrealized (2 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx)
    (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2060)
  (parent 2037)
  (unrealized (3 3) (9 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 2061)
  (parent 2038)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx)
    (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2062)
  (parent 2038)
  (unrealized (3 3) (9 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (2 3)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self))
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2063)
  (parent 2039)
  (unrealized (2 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx)
    (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2064)
  (parent 2039)
  (unrealized (3 3) (9 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx)
    (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec l) (rec w))) w))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec l) (rec w))) w))
    (exp (gen) (rec l)) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec l) (rec w))) w))
      (send (cat (exp (gen) (mul (rec l) (rec w))) w))))
  (label 2065)
  (parent 2040)
  (unrealized (3 3) (9 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0)))
  (label 2066)
  (parent 2041)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l y x l-0 l-1 rndx)
    (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2067)
  (parent 2041)
  (unrealized (3 3) (9 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx)
    (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec l) (rec w))) w))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec l) (rec w))) w))
    (exp (gen) (rec l)) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec l) (rec w))) w))
      (send (cat (exp (gen) (mul (rec l) (rec w))) w))))
  (label 2068)
  (parent 2042)
  (unrealized (3 3) (9 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 4)) ((3 2) (2 0))
    ((4 1) (0 0)) ((4 2) (2 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a self))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2069)
  (parent 2043)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l rndx) (beta expt) (x l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x) (mul (rec x) l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x) (beta beta))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x beta) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul (rec x) l-0)))
      (send (cat (exp (gen) x) (mul (rec x) l-0))))
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x)))))
  (label 2070)
  (parent 2045)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta beta expt) (l x l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta beta))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x beta) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x)))))
  (label 2071)
  (parent 2045)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x beta) (x l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-1))
  (facts (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1)
    (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 2072)
  (parent 2045)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (y rndx) (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor (cat pt (pv b l-0))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 2073)
  (parent 2046)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (2 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 4))
    ((3 1) (2 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (0 0))
    ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 1 1)
    (trans 6 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a self))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2074)
  (parent 2047)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0) (y-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2075)
  (parent 2048)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0) (y-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2076)
  (parent 2048)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0 l-1)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 2077)
  (parent 2048)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (3 0))
    ((3 1) (0 4)) ((4 2) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 2078)
  (parent 2051)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta beta expt) (l x l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x) (beta beta))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x beta) (x l-0))
  (precur (7 0))
  (gen-st (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x)))))
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x)))))
  (label 2079)
  (parent 2053)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l rndx) (beta expt) (x l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x) (mul (rec x) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta beta))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x beta) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul (rec x) l-0)))
      (send (cat (exp (gen) x) (mul (rec x) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x)))))
  (label 2080)
  (parent 2053)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x beta) (x l-1))
  (precur (7 0))
  (gen-st (pv b l) (pv a-0 l-1))
  (facts (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1)
    (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 2081)
  (parent 2053)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (y rndx) (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 2082)
  (parent 2054)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0) (y-0 l-0))
  (precur (7 0))
  (gen-st (pv b l-0))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2083)
  (parent 2056)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0) (y-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2084)
  (parent 2056)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0 l-1)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (7 0))
  (gen-st (pv b l) (pv b-0 l-1))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 2085)
  (parent 2056)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 3) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (6 0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2086)
  (parent 2057)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta (mul (rec l) l-0 l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (9 0)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 2087)
  (parent 2059)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta (mul (rec l) l-0 l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (9 0)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 2088)
  (parent 2059)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 lb rndx) (alpha zeta expt) (l-1 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 2089)
  (parent 2060)
  (unrealized (3 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 2090)
  (parent 2060)
  (unrealized (3 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 2091)
  (parent 2060)
  (unrealized (3 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 2092)
  (parent 2060)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta (mul (rec l) l-0 l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (2 3)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (8 0))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self))
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2093)
  (parent 2060)
  (unrealized (2 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (8 0))
  (gen-st (pv b l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2094)
  (parent 2060)
  (unrealized (3 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 10 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2095)
  (parent 2060)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 2096)
  (parent 2061)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 2097)
  (parent 2061)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 lb rndx) (alpha zeta expt) (l-1 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 2098)
  (parent 2062)
  (unrealized (3 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 2099)
  (parent 2062)
  (unrealized (3 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 2100)
  (parent 2062)
  (unrealized (3 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 2101)
  (parent 2062)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta (mul (rec l) l-0 l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (2 3)) ((1 2) (0 1)) ((2 2) (0 2)) ((2 4) (0 4))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self))
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2102)
  (parent 2062)
  (unrealized (2 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2103)
  (parent 2062)
  (unrealized (3 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 10 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2104)
  (parent 2062)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (9 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 2105)
  (parent 2063)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (9 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 2106)
  (parent 2063)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 lb rndx) (alpha zeta expt) (l-1 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 2107)
  (parent 2064)
  (unrealized (3 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 2108)
  (parent 2064)
  (unrealized (3 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 2109)
  (parent 2064)
  (unrealized (3 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 2110)
  (parent 2064)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (2 3)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self))
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2111)
  (parent 2064)
  (unrealized (2 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (8 0))
  (gen-st (pv b l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2112)
  (parent 2064)
  (unrealized (3 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 10 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2113)
  (parent 2064)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 lb rndx) (alpha zeta expt) (l-1 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 2114)
  (parent 2065)
  (unrealized (3 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 2115)
  (parent 2065)
  (unrealized (3 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w (rec l-1))) (gen)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1)))))
  (label 2116)
  (parent 2065)
  (unrealized (3 3) (7 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 2) (0 2)) ((3 4) (0 4))
    ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))))
  (label 2117)
  (parent 2065)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (2 3)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self))
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2118)
  (parent 2065)
  (unrealized (2 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (8 0))
  (gen-st (pv b l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2119)
  (parent 2065)
  (unrealized (3 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 10 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2120)
  (parent 2065)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 2121)
  (parent 2066)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 2122)
  (parent 2066)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 lb rndx) (alpha zeta expt) (l-1 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 2123)
  (parent 2067)
  (unrealized (3 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 2124)
  (parent 2067)
  (unrealized (3 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 2125)
  (parent 2067)
  (unrealized (3 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 2126)
  (parent 2067)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta (mul (rec l) l-0 l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (2 3)) ((1 2) (0 1)) ((2 2) (0 2)) ((2 4) (0 4))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self))
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2127)
  (parent 2067)
  (unrealized (2 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2128)
  (parent 2067)
  (unrealized (3 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 10 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2129)
  (parent 2067)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 lb rndx) (alpha zeta expt) (l-1 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 2130)
  (parent 2068)
  (unrealized (3 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 2131)
  (parent 2068)
  (unrealized (3 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w (rec l-1))) (gen)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1)))))
  (label 2132)
  (parent 2068)
  (unrealized (3 3) (7 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 2) (0 2)) ((3 4) (0 4))
    ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))))
  (label 2133)
  (parent 2068)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (2 3)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self))
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2134)
  (parent 2068)
  (unrealized (2 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2135)
  (parent 2068)
  (unrealized (3 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 10 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2136)
  (parent 2068)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 4)) ((3 1) (0 0))
    ((3 2) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq a self))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2137)
  (parent 2069)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b self) (lb l-0) (y y) (alpha l) (zeta zeta) (na na)
        (nb nb) (priv-stor priv-stor-0) (pt pt-3))))
  (origs (l-0 (3 1)) (pt-3 (3 1)) (pt-1 (2 1)) (l (1 1)) (pt-0 (1 1))
    (nb (0 3))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (y rndx) (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (precedes ((0 3) (3 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0)
    (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 2138)
  (parent 2073)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 4)) ((3 2) (2 0))
    ((4 1) (2 0)) ((5 1) (0 0)) ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 1 1)
    (trans 5 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a self))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2139)
  (parent 2074)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 2140)
  (parent 2078)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b) (lb l) (y y) (alpha l-0) (zeta zeta) (na na) (nb nb)
        (priv-stor priv-stor) (pt pt))))
  (origs (l-0 (2 1)) (pt-2 (2 1)) (pt-3 (3 1)) (l (1 1)) (pt (1 1))
    (nb (0 3))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (y rndx) (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0)
    (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 2141)
  (parent 2082)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 4)) ((4 2) (3 0)) ((5 1) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (5 0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2142)
  (parent 2086)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx)
    (alpha zeta expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 2143)
  (parent 2089)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 rndx)
    (alpha zeta expt) (y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 2144)
  (parent 2089)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx)
    (alpha zeta expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 2145)
  (parent 2089)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 rndx) (alpha zeta expt) (l-1 y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 2146)
  (parent 2089)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2147)
  (parent 2090)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 rndx)
    (beta expt) (x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2148)
  (parent 2090)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2149)
  (parent 2090)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 2150)
  (parent 2090)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor (cat pt (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1)))
  (label 2151)
  (parent 2091)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta (mul (rec l) l-0 l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (10 0)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (8 0))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 2152)
  (parent 2093)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta (mul (rec l) l-0 l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (10 0)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (8 0))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 2153)
  (parent 2093)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 rndx)
    (alpha zeta expt) (y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 2154)
  (parent 2098)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx)
    (alpha zeta expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 2155)
  (parent 2098)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx)
    (alpha zeta expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 2156)
  (parent 2098)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 rndx) (alpha zeta expt) (l-1 y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 2157)
  (parent 2098)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 rndx)
    (beta expt) (x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2158)
  (parent 2099)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2159)
  (parent 2099)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2160)
  (parent 2099)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 2161)
  (parent 2099)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1)))
  (label 2162)
  (parent 2100)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 2163)
  (parent 2101)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (10 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 2164)
  (parent 2101)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx)
    (alpha zeta expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 2165)
  (parent 2107)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 rndx)
    (alpha zeta expt) (y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 2166)
  (parent 2107)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx)
    (alpha zeta expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 2167)
  (parent 2107)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 rndx) (alpha zeta expt) (l-1 y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 2168)
  (parent 2107)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2169)
  (parent 2108)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 rndx)
    (beta expt) (x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2170)
  (parent 2108)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2171)
  (parent 2108)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 2172)
  (parent 2108)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor (cat pt (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1)))
  (label 2173)
  (parent 2109)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (10 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 2174)
  (parent 2111)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (10 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 2) (0 2))
    ((2 4) (0 4)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (2 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv a l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (2 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 2175)
  (parent 2111)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx)
    (alpha zeta expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4))
    ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 2176)
  (parent 2114)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx)
    (alpha zeta expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 1) (6 0)) ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4))
    ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 2177)
  (parent 2114)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 rndx)
    (alpha zeta expt) (y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
      (send (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 2178)
  (parent 2114)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 rndx) (alpha zeta expt) (l-1 y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 2179)
  (parent 2114)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4))
    ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2180)
  (parent 2115)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 1) (6 0)) ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4))
    ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2181)
  (parent 2115)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 rndx)
    (beta expt) (x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
      (send (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2182)
  (parent 2115)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 2183)
  (parent 2115)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-listener l-1) (rec l-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1)))
  (label 2184)
  (parent 2116)
  (unrealized (3 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 1) (10 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (8 0))
  (gen-st (pv b l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (added-listener l-1) (mul (rec l-1) (rec l-1))
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1)))
  (label 2185)
  (parent 2119)
  (unrealized (3 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 rndx)
    (alpha zeta expt) (y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 2186)
  (parent 2123)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx)
    (alpha zeta expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 2187)
  (parent 2123)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx)
    (alpha zeta expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 2188)
  (parent 2123)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 rndx) (alpha zeta expt) (l-1 y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 2189)
  (parent 2123)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 rndx)
    (beta expt) (x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2190)
  (parent 2124)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2191)
  (parent 2124)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2192)
  (parent 2124)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 2193)
  (parent 2124)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1)))
  (label 2194)
  (parent 2125)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 2195)
  (parent 2126)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 2196)
  (parent 2126)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx)
    (alpha zeta expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4))
    ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 2197)
  (parent 2130)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx)
    (alpha zeta expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4))
    ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 2198)
  (parent 2130)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 rndx)
    (alpha zeta expt) (y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
      (send (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 2199)
  (parent 2130)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 rndx) (alpha zeta expt) (l-1 y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul y (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 2200)
  (parent 2130)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4))
    ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2201)
  (parent 2131)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4))
    ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2202)
  (parent 2131)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 rndx)
    (beta expt) (x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
      (send (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2203)
  (parent 2131)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (y x l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 2204)
  (parent 2131)
  (unrealized (3 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-listener l-1) (rec l-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1)))
  (label 2205)
  (parent 2132)
  (unrealized (3 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 1) (10 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (added-listener l-1) (mul (rec l-1) (rec l-1))
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1)))
  (label 2206)
  (parent 2135)
  (unrealized (3 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (y rndx) (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1) (trans 6 0)
    (trans 4 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 2207)
  (parent 2138)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 4)) ((3 1) (0 4))
    ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq a self))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2208)
  (parent 2139)
  (seen 2137)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (y rndx) (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1) (trans 6 0)
    (trans 4 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 2209)
  (parent 2141)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 2) (0 4)) ((4 1) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2210)
  (parent 2142)
  (seen 2140)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (11 0)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 2211)
  (parent 2151)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (11 0)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 2212)
  (parent 2151)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (11 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 2213)
  (parent 2162)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l) (x x) (beta (mul (rec l) l-0 l-1)) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (11 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l-0) (y l-1) (x (mul (rec l) l-0 l-1)) (x l))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))) (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 2214)
  (parent 2162)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (11 0)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 2215)
  (parent 2173)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (11 0)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv b l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 2216)
  (parent 2173)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 2217)
  (parent 2184)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (8 0))
  (gen-st (pv b l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-1 (cat pt-6 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 2218)
  (parent 2185)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (11 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 2219)
  (parent 2194)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-1) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-0) (x x) (beta (mul l (rec l-0) l-1))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (11 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-1) (x (mul l (rec l-0) l-1)) (x l-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-1)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 2220)
  (parent 2194)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 2221)
  (parent 2205)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (3 3)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-1 (cat pt-6 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 2222)
  (parent 2206)
  (unrealized (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (y rndx) (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 4)) ((4 1) (0 4)) ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1) (trans 5 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 2223)
  (parent 2207)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (y rndx) (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 2) (0 4)) ((4 1) (0 4)) ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1) (trans 5 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 2224)
  (parent 2209)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (12 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 2225)
  (parent 2217)
  (unrealized (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (12 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 2226)
  (parent 2217)
  (unrealized (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (12 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (3 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (8 0))
  (gen-st (pv b l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 2227)
  (parent 2218)
  (unrealized (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (12 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (3 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (8 0))
  (gen-st (pv b l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 2228)
  (parent 2218)
  (unrealized (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (12 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 2229)
  (parent 2221)
  (unrealized (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (12 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (3 3)))
  (non-orig (privk "sig" a))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b (hash (exp (gen) (mul l l-0)) (exp (gen) y)))))
      (send nb))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 2230)
  (parent 2221)
  (unrealized (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 3) (12 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (3 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (added-listener (cat (exp (gen) y) x))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 2231)
  (parent 2222)
  (unrealized (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y x l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta (mul l l-0 (rec l-1)))
    (upsilon (mul y (rec x))))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec x)) y))
  (precedes ((0 3) (12 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 2) (0 2)) ((3 4) (0 4)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (3 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y (one)) (y l) (y l-0) (x (mul l l-0 (rec l-1))) (x l-1))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec x)) y))
    (exp (gen) (mul y (rec x))) (3 3))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" b))
          (privk "sig" b))) (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))) (send nb))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec x)) y))
      (send (cat (exp (gen) (rec x)) y))))
  (label 2232)
  (parent 2222)
  (unrealized (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (0 4))
    ((4 2) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 2233)
  (parent 2223)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (y rndx) (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (4 0)) ((2 2) (0 1))
    ((3 2) (0 4)) ((4 2) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1) (trans 4 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 2234)
  (parent 2224)
  (seen 2235)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 2235)
  (parent 2233)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b) (lb l-0) (y y) (alpha l) (zeta zeta) (na na) (nb nb)
        (priv-stor priv-stor) (pt pt))))
  (origs (pt-3 (3 1)) (l (2 1)) (pt-2 (2 1)) (l-0 (1 1)) (pt (1 1))
    (nb (0 3))))

(comment "Nothing left to do")

(defprotocol dhcr-um diffie-hellman
  (defrole init
    (vars (la x rndx) (beta upsilon expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv a la))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb))
    (uniq-orig na)
    (uniq-gen x)
    (absent (x la) (x beta))
    (assume (fact neq (exp (gen) upsilon) (gen)))
    (gen-st (pv a la)))
  (defrole resp
    (vars (lb y rndx) (alpha zeta expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv b lb))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    (uniq-orig nb)
    (uniq-gen y)
    (absent (y lb) (y alpha) (y zeta))
    (assume (fact neq (exp (gen) zeta) (gen)))
    (gen-st (pv b lb)))
  (defrole ltx-gen
    (vars (self name) (l rndx) (priv-stor locn) (ignore mesg))
    (trace (load priv-stor ignore) (stor priv-stor (pv self l))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    (uniq-orig l))
  (defrole ltx-disclose
    (vars (self name) (l rndx) (priv-stor locn))
    (trace (load priv-stor (pv self l)) (stor priv-stor "nil") (send l))
    (gen-st (pv self l)))
  (defrule undisclosed-not-disclosed
    (forall ((z strd) (l rndx))
      (implies
        (and (fact undisclosed l) (p "ltx-disclose" z 2)
          (p "ltx-disclose" "l" z l))
        (false))))
  (defgenrule neqRl_indx
    (forall ((x indx)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_strd
    (forall ((x strd)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_mesg
    (forall ((x mesg)) (implies (fact neq x x) (false))))
  (defgenrule scissorsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (leads-to z0 i0 z2 i2))
        (and (= z1 z2) (= i1 i2)))))
  (defgenrule cakeRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (leads-to z0 i0 z1 i1)
          (leads-to z0 i0 z2 i2) (prec z1 i1 z2 i2)) (false))))
  (defgenrule no-interruption
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (leads-to z0 i0 z2 i2) (trans z1 i1)
          (same-locn z0 i0 z1 i1) (prec z0 i0 z1 i1) (prec z1 i1 z2 i2))
        (false))))
  (defgenrule shearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (same-locn z0 i0 z2 i2)
          (prec z0 i0 z2 i2))
        (or (and (= z1 z2) (= i1 i2)) (prec z1 i1 z2 i2)))))
  (defgenrule invShearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (same-locn z0 i0 z1 i1)
          (leads-to z1 i1 z2 i2) (prec z0 i0 z2 i2))
        (or (and (= z0 z1) (= i0 i1)) (prec z0 i0 z1 i1)))))
  (defgenrule assume-init-0
    (forall ((z strd) (upsilon expt))
      (implies (and (p "init" z 4) (p "init" "upsilon" z upsilon))
        (fact neq (exp (gen) upsilon) (gen)))))
  (defgenrule assume-resp-0
    (forall ((z strd) (zeta expt))
      (implies (and (p "resp" z 3) (p "resp" "zeta" z zeta))
        (fact neq (exp (gen) zeta) (gen)))))
  (defgenrule trRl_ltx-gen-at-1
    (forall ((z strd)) (implies (p "ltx-gen" z 2) (trans z 1))))
  (defgenrule trRl_ltx-gen-at-0
    (forall ((z strd)) (implies (p "ltx-gen" z 2) (trans z 0))))
  (defgenrule trRl_ltx-disclose-at-1
    (forall ((z strd)) (implies (p "ltx-disclose" z 2) (trans z 1))))
  (defgenrule trRl_ltx-disclose-at-0
    (forall ((z strd)) (implies (p "ltx-disclose" z 2) (trans z 0))))
  (defgenrule gen-st-init-0
    (forall ((z strd) (la rndx) (a name))
      (implies
        (and (p "init" z 1) (p "init" "la" z la) (p "init" "a" z a))
        (gen-st (pv a la)))))
  (defgenrule gen-st-resp-0
    (forall ((z strd) (lb rndx) (b name))
      (implies
        (and (p "resp" z 1) (p "resp" "lb" z lb) (p "resp" "b" z b))
        (gen-st (pv b lb)))))
  (defgenrule gen-st-ltx-disclose-0
    (forall ((z strd) (l rndx) (self name))
      (implies
        (and (p "ltx-disclose" z 1) (p "ltx-disclose" "l" z l)
          (p "ltx-disclose" "self" z self)) (gen-st (pv self l)))))
  (lang (sig sign) (body (tuple 3)) (pv (tuple 2))))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (a b self name) (pt pt-0 pt-1 pval)
    (priv-stor priv-stor-0 locn) (la beta x rndx) (upsilon expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l beta))
  (uniq-orig na beta)
  (uniq-gen x)
  (absent (x la) (x beta))
  (facts (neq a b))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self beta)))
      (send
        (sig (body self (exp (gen) beta) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2236)
  (unrealized (0 1) (0 3))
  (origs (pt-1 (1 1)) (na (0 2)) (beta (1 1)))
  (comment "Not closed under rules"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (a b self name) (pt pt-0 pt-1 pval)
    (priv-stor priv-stor-0 locn) (la beta x rndx) (upsilon expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l beta))
  (uniq-orig na beta)
  (uniq-gen x)
  (absent (x la) (x beta))
  (gen-st (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq a b))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self beta)))
      (send
        (sig (body self (exp (gen) beta) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2237)
  (parent 2236)
  (unrealized (0 0) (0 1) (0 3))
  (origs (pt-1 (1 1)) (na (0 2)) (beta (1 1)))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (precedes ((1 1) (0 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation channel-test (displaced 2 1 ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv a l-0))) (0 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2238)
  (parent 2237)
  (unrealized (0 1) (0 3))
  (origs (l (1 1)) (pt-0 (1 1)) (na (0 2)))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (beta x rndx)
    (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l beta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (precedes ((2 1) (0 0)))
  (uniq-orig na beta l)
  (uniq-gen x)
  (absent (x beta) (x l))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv a l))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul beta l))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self beta)))
      (send
        (sig (body self (exp (gen) beta) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l)))))
  (label 2239)
  (parent 2237)
  (unrealized (0 1) (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (precedes ((1 1) (0 0)) ((1 2) (0 1)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation nonce-test (displaced 2 1 ltx-gen 3) (exp (gen) l) (0 1))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 2240)
  (parent 2238)
  (unrealized (0 3))
  (origs (l (1 1)) (pt-0 (1 1)) (na (0 2)))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((1 1) (0 0)) ((2 1) (0 1)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (0 1))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2241)
  (parent 2238)
  (unrealized (0 3) (2 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test (displaced 3 1 ltx-gen 3) (exp (gen) l-0) (0 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l)))))
  (label 2242)
  (parent 2239)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (beta x rndx)
    (upsilon expt) (l rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l beta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) (mul beta (rec w))) w))
  (precedes ((2 1) (0 0)) ((3 1) (0 1)))
  (uniq-orig na beta l)
  (uniq-gen x)
  (absent (x beta) (x l))
  (precur (3 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul beta (rec w))) w))
    (exp (gen) beta) (0 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul beta l))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self beta)))
      (send
        (sig (body self (exp (gen) beta) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) (mul beta (rec w))) w))
      (send (cat (exp (gen) (mul beta (rec w))) w))))
  (label 2243)
  (parent 2239)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor priv-stor-0 locn) (l lb x y rndx) (zeta expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3)))
  (uniq-orig na nb l)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y zeta))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 1 1) (trans 1 0) (neq self b))
  (operation encryption-test (added-strand resp 4)
    (enc na nb self b
      (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta))))))))
  (label 2244)
  (parent 2240)
  (unrealized (0 3) (2 0) (2 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (2 0)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 1) (0 3)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation encryption-test
    (added-listener
      (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
    (enc na nb self b
      (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))))
  (label 2245)
  (parent 2240)
  (unrealized (2 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (la rndx) (beta expt) (l x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la la) (x x-0) (beta beta))
  (precedes ((1 1) (0 0)) ((2 1) (0 1)) ((3 2) (2 0)))
  (uniq-orig na na-0 l)
  (uniq-gen x x-0)
  (absent (x l) (x-0 la) (x-0 beta))
  (precur (2 0))
  (gen-st (pv self l) (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2246)
  (parent 2241)
  (unrealized (0 1) (0 3) (2 0) (3 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (precedes ((1 1) (0 0)) ((1 1) (2 0)) ((2 1) (0 1)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation nonce-test (contracted (l-0 l) (w l)) (gen) (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l))))
  (label 2247)
  (parent 2241)
  (unrealized (2 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 nb-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (lb rndx) (alpha zeta expt) (l y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((1 1) (0 0)) ((2 1) (0 1)) ((3 3) (2 0)))
  (uniq-orig na nb-0 l)
  (uniq-gen x y)
  (absent (x l) (y lb) (y alpha) (y zeta))
  (precur (2 0))
  (gen-st (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 1 1) (trans 1 0) (neq self b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor-0 (cat pt-1 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 2248)
  (parent 2241)
  (unrealized (0 1) (0 3) (2 0) (3 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (precedes ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation nonce-test (displaced 3 1 ltx-gen 3) (exp (gen) l-0) (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one)))))
  (label 2249)
  (parent 2241)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((2 1) (0 1)) ((3 2) (2 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (neq self b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-0) (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0)))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2250)
  (parent 2241)
  (unrealized (0 1) (0 3) (2 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx)
    (zeta expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((3 3) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y zeta))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b))
  (operation encryption-test (added-strand resp 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 2251)
  (parent 2242)
  (unrealized (0 3) (3 0) (3 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (3 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((3 1) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation encryption-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send
        (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 2252)
  (parent 2242)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l lb rndx) (alpha zeta expt) (beta y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l beta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) y) (mul beta (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((2 1) (0 0)) ((3 1) (0 1)) ((4 3) (3 0)))
  (uniq-orig na nb-0 l beta)
  (uniq-gen x y)
  (absent (x l) (x beta) (y lb) (y alpha) (y zeta))
  (precur (3 0))
  (gen-st (pv a l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l beta))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self beta)))
      (send
        (sig (body self (exp (gen) beta) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) y) (mul beta (rec y))))
      (send (cat (exp (gen) y) (mul beta (rec y)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 2253)
  (parent 2243)
  (unrealized (0 1) (0 3) (3 0) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l la rndx) (beta expt) (beta-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l beta-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) x-0) (mul beta-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((2 1) (0 0)) ((3 1) (0 1)) ((4 2) (3 0)))
  (uniq-orig na na-0 l beta-0)
  (uniq-gen x x-0)
  (absent (x l) (x beta-0) (x-0 la) (x-0 beta))
  (precur (3 0))
  (gen-st (pv a l) (pv a-0 la))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta-0) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l beta-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self beta-0)))
      (send
        (sig (body self (exp (gen) beta-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) x-0) (mul beta-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul beta-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 2254)
  (parent 2243)
  (unrealized (0 1) (0 3) (3 0) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l beta rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l beta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) beta))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((3 1) (0 1)))
  (uniq-orig na l beta)
  (uniq-gen x)
  (absent (x l) (x beta))
  (precur (3 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test (contracted (beta-0 beta) (w beta)) (gen) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l beta))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self beta)))
      (send
        (sig (body self (exp (gen) beta) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) beta)) (send (cat (gen) beta))))
  (label 2255)
  (parent 2243)
  (unrealized (0 3) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 2) (3 0)) ((2 1) (0 0)) ((3 1) (0 1)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test (displaced 4 1 ltx-gen 3) (exp (gen) l-1) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2256)
  (parent 2243)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (beta l rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l beta))
  (deflistener (cat (exp (gen) l) (mul beta (rec l))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((2 1) (0 1)) ((3 1) (0 0)) ((3 2) (2 0)))
  (uniq-orig na beta l)
  (uniq-gen x)
  (absent (x beta) (x l))
  (precur (2 0))
  (gen-st (pv self-0 l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (neq self-0 b))
  (operation nonce-test (displaced 2 4 ltx-gen 3) (exp (gen) l-0) (3 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul beta l))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self beta)))
      (send
        (sig (body self (exp (gen) beta) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (mul beta (rec l))))
      (send (cat (exp (gen) l) (mul beta (rec l)))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2257)
  (parent 2243)
  (unrealized (0 1) (0 3) (2 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l beta l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l beta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (mul beta (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((2 1) (0 0)) ((3 1) (0 1)) ((4 2) (3 0)))
  (uniq-orig na l beta l-0)
  (uniq-gen x)
  (absent (x l) (x beta))
  (precur (3 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-0) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l beta))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self beta)))
      (send
        (sig (body self (exp (gen) beta) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (mul beta (rec l-0))))
      (send (cat (exp (gen) l-0) (mul beta (rec l-0)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2258)
  (parent 2243)
  (unrealized (0 1) (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec l-0))) (y zeta) (y l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv b l-0))) (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0)))))
  (label 2259)
  (parent 2244)
  (unrealized (0 3) (2 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
    (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))) (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))))
  (label 2260)
  (parent 2245)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 data) (b self b-0 name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon beta expt) (x-0 l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l) (x x-0) (beta beta))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0)))
  (uniq-orig na na-0 l)
  (uniq-gen x x-0)
  (absent (x l) (x-0 beta) (x-0 l))
  (precur (2 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation channel-test (displaced 4 1 ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv a l-0))) (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2261)
  (parent 2246)
  (unrealized (0 1) (0 3) (2 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((2 1) (0 1)) ((3 2) (2 0)) ((4 1) (3 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 beta) (x-0 l-0))
  (precur (2 0))
  (gen-st (pv self l) (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv a l-0))) (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l-0)))))
  (label 2262)
  (parent 2246)
  (unrealized (0 1) (0 3) (2 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (trans 3 1) (trans 3 0) (neq self b))
  (operation nonce-test (added-strand ltx-disclose 3) l (2 0)
    (ch-msg priv-stor (cat pt-0 (pv self l))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 2263)
  (parent 2247)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pval) (priv-stor locn) (x rndx) (upsilon alpha zeta expt)
    (y l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha alpha) (zeta zeta))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 3) (2 0)))
  (uniq-orig na nb-0 l)
  (uniq-gen x y)
  (absent (x l) (y alpha) (y zeta) (y l))
  (precur (2 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 1 1) (trans 1 0) (neq self b))
  (operation channel-test (displaced 4 1 ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv b-0 l-0))) (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec y) l)))
      (send (cat (exp (gen) y) (mul (rec y) l))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l))
              (exp (gen) (mul zeta y))))))))
  (label 2264)
  (parent 2248)
  (unrealized (0 1) (0 3) (2 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon alpha zeta expt)
    (l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((2 1) (0 1)) ((3 3) (2 0)) ((4 1) (3 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y alpha) (y zeta) (y l-0))
  (precur (2 0))
  (gen-st (pv self l) (pv b-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv b-0 l-0))) (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor-0 (cat pt-1 (pv b-0 l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b-0 l-0)))))
  (label 2265)
  (parent 2248)
  (unrealized (0 1) (0 3) (2 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor priv-stor-0 locn) (l lb x y rndx) (zeta expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)))
  (uniq-orig na nb l)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y zeta))
  (precur (2 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 1 1) (trans 1 0) (neq self b))
  (operation encryption-test (added-strand resp 4)
    (enc na nb self b
      (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta))))))))
  (label 2266)
  (parent 2249)
  (unrealized (0 3) (3 0) (3 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 1) (0 3)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation encryption-test
    (added-listener
      (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
    (enc na nb self b
      (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))))
  (label 2267)
  (parent 2249)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a self))
  (operation channel-test (displaced 4 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 2268)
  (parent 2251)
  (unrealized (0 3) (3 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((3 3) (0 3))
    ((4 1) (3 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1)))))
  (label 2269)
  (parent 2251)
  (unrealized (0 3) (3 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((3 1) (0 3))
    ((4 1) (3 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 2270)
  (parent 2252)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon expt) (l rndx)
    (alpha zeta expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1)) ((4 3) (3 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b))
  (operation channel-test (displaced 5 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) y) (mul (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec y) l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 2271)
  (parent 2253)
  (unrealized (0 1) (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon alpha zeta expt)
    (beta y l rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l beta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) y) (mul beta (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l) (y y) (alpha alpha) (zeta zeta))
  (precedes ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 3) (3 0)))
  (uniq-orig na nb-0 beta l)
  (uniq-gen x y)
  (absent (x beta) (x l) (y alpha) (y zeta) (y l))
  (precur (3 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a b))
  (operation channel-test (displaced 5 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-0))) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul beta l))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self beta)))
      (send
        (sig (body self (exp (gen) beta) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) y) (mul beta (rec y))))
      (send (cat (exp (gen) y) (mul beta (rec y)))))
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l))
              (exp (gen) (mul zeta y))))))))
  (label 2272)
  (parent 2253)
  (unrealized (0 1) (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (alpha zeta expt) (beta y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l beta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) y) (mul beta (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((2 1) (0 0)) ((3 1) (0 1)) ((4 3) (3 0)) ((5 1) (4 0)))
  (uniq-orig na nb-0 l beta l-0)
  (uniq-gen x y)
  (absent (x l) (x beta) (y alpha) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv b-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-0))) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l beta))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self beta)))
      (send
        (sig (body self (exp (gen) beta) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) y) (mul beta (rec y))))
      (send (cat (exp (gen) y) (mul beta (rec y)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-0)))))
  (label 2273)
  (parent 2253)
  (unrealized (0 1) (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1)) ((4 2) (3 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation channel-test (displaced 5 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2274)
  (parent 2254)
  (unrealized (0 1) (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (beta-0 x-0 l rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l beta-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) x-0) (mul beta-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor) (la l)
    (x x-0) (beta beta))
  (precedes ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 2) (3 0)))
  (uniq-orig na na-0 beta-0 l)
  (uniq-gen x x-0)
  (absent (x beta-0) (x l) (x-0 beta) (x-0 l))
  (precur (3 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation channel-test (displaced 5 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-0))) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta-0) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul beta-0 l))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self beta-0)))
      (send
        (sig (body self (exp (gen) beta-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) x-0) (mul beta-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul beta-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2275)
  (parent 2254)
  (unrealized (0 1) (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (beta expt) (beta-0 x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l beta-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) x-0) (mul beta-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-0) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((2 1) (0 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (4 0)))
  (uniq-orig na na-0 l beta-0 l-0)
  (uniq-gen x x-0)
  (absent (x l) (x beta-0) (x-0 beta) (x-0 l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv a-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-0))) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta-0) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l beta-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self beta-0)))
      (send
        (sig (body self (exp (gen) beta-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) x-0) (mul beta-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul beta-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a-0 l-0)))))
  (label 2276)
  (parent 2254)
  (unrealized (0 1) (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1)) ((4 2) (3 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (3 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0)))
  (label 2277)
  (parent 2255)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx)
    (zeta expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 3) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y zeta))
  (precur (3 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a b))
  (operation encryption-test (added-strand resp 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 2278)
  (parent 2256)
  (unrealized (0 3) (4 0) (4 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (4 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 1) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation encryption-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send
        (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 2279)
  (parent 2256)
  (unrealized (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (2 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec l-0))) (y zeta) (y l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (neq self b))
  (operation nonce-test (added-listener (cat (exp (gen) (mul l l)) l-0))
    (exp (gen) (mul l l (rec l-0))) (2 1))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0))))
  (label 2280)
  (parent 2259)
  (unrealized (0 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec l-0))) (y zeta) (y l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (neq self b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l l (rec l-0))) (2 1))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 2281)
  (parent 2259)
  (unrealized (0 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l (rec w))) w))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l (rec w))) w))
    (exp (gen) (mul l l)) (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l (rec w))) w))
      (send (cat (exp (gen) (mul l l (rec w))) w))))
  (label 2282)
  (parent 2260)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (2 0)) ((2 2) (0 1)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (trans 2 1) (trans 2 0) (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 2283)
  (parent 2263)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta l) (a self) (b b) (x x) (upsilon upsilon) (na na)
        (nb nb) (priv-stor priv-stor) (pt pt-0) (self self)
        (priv-stor-0 priv-stor) (ignore ignore) (pt-0 pt) (pt-1 pt-0))))
  (origs (pt-1 (2 1)) (l (1 1)) (pt-0 (1 1)) (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec l-0))) (y zeta) (y l-0))
  (precur (2 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv b l-0))) (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0)))))
  (label 2284)
  (parent 2266)
  (unrealized (0 3) (3 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
    (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))) (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))))
  (label 2285)
  (parent 2267)
  (unrealized (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (neq self-0 self))
  (operation nonce-test (displaced 2 4 ltx-gen 3) (exp (gen) l-0) (3 1))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2286)
  (parent 2268)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a self))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2287)
  (parent 2268)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (3 1)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l-0)) l-1))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1))))
  (label 2288)
  (parent 2269)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec l-1))) l-0))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0))))
  (label 2289)
  (parent 2269)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec l-1))) l))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l))))
  (label 2290)
  (parent 2269)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 2291)
  (parent 2270)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 2292)
  (parent 2270)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx)
    (zeta expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((5 3) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y zeta))
  (precur (3 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation encryption-test (added-strand resp 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 2293)
  (parent 2277)
  (unrealized (0 3) (5 0) (5 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (5 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((5 1) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation encryption-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send
        (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 2294)
  (parent 2277)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a self))
  (operation channel-test (displaced 5 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 2295)
  (parent 2278)
  (unrealized (0 3) (4 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 3) (0 3)) ((5 1) (4 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1)))))
  (label 2296)
  (parent 2278)
  (unrealized (0 3) (4 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 1) (0 3)) ((5 1) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 2297)
  (parent 2279)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x y rndx) (zeta expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec l-0))) (y zeta) (y l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (4 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv b l-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0)))
  (label 2298)
  (parent 2280)
  (unrealized (0 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (4 0)
    (ch-msg priv-stor (cat pt-0 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 2299)
  (parent 2281)
  (unrealized (0 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (la rndx) (beta expt) (l x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) x-0) (mul l l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)))
  (uniq-orig na na-0 l)
  (uniq-gen x x-0)
  (absent (x l) (x-0 la) (x-0 beta))
  (precur (4 0))
  (gen-st (pv self l) (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) x-0) (mul l l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2300)
  (parent 2282)
  (unrealized (3 0) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (upsilon expt) (l x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) x) (mul l l (rec x))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation nonce-test (displaced 5 0 init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul upsilon x)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) x) (mul l l (rec x))))
      (send (cat (exp (gen) x) (mul l l (rec x))))))
  (label 2301)
  (parent 2282)
  (unrealized (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (gen) (mul l l)))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation nonce-test (contracted (l-0 l) (w (mul l l))) (gen) (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l)))))
  (label 2302)
  (parent 2282)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 nb-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (lb rndx) (alpha zeta expt) (l y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) y) (mul l l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 3) (4 0)))
  (uniq-orig na nb-0 l)
  (uniq-gen x y)
  (absent (x l) (y lb) (y alpha) (y zeta))
  (precur (4 0))
  (gen-st (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 1 1) (trans 1 0) (neq self b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) y) (mul l l (rec y))))
      (send (cat (exp (gen) y) (mul l l (rec y)))))
    ((load priv-stor-0 (cat pt-1 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 2303)
  (parent 2282)
  (unrealized (3 0) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation nonce-test (displaced 5 1 ltx-gen 3) (exp (gen) l-0) (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l))))
  (label 2304)
  (parent 2282)
  (unrealized (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) (mul l l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 5 1) (trans 1 1)
    (trans 5 0) (trans 1 0) (neq self b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-0) (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) (mul l l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l l (rec l-0)))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2305)
  (parent 2282)
  (unrealized (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (3 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec l-0))) (y zeta) (y l-0))
  (precur (2 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (neq self b))
  (operation nonce-test (added-listener (cat (exp (gen) (mul l l)) l-0))
    (exp (gen) (mul l l (rec l-0))) (3 1))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0))))
  (label 2306)
  (parent 2284)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec l-0))) (y zeta) (y l-0))
  (precur (2 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (neq self b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l l (rec l-0))) (3 1))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 2307)
  (parent 2284)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0) (5 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l (rec w))) w))
    (exp (gen) (mul l l)) (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l (rec w))) w))
      (send (cat (exp (gen) (mul l l (rec w))) w))))
  (label 2308)
  (parent 2285)
  (unrealized (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (displaced 4 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2309)
  (parent 2286)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta lb) (a self-0) (b self) (x x) (upsilon y) (na na)
        (nb nb) (priv-stor priv-stor-0) (pt pt-2) (self self)
        (priv-stor-0 priv-stor) (ignore ignore) (pt-0 pt) (pt-1 pt-0))))
  (origs (nb (2 3)) (l (3 1)) (pt-2 (3 1)) (lb (1 1)) (pt-0 (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (neq self-0 self))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2310)
  (parent 2286)
  (unrealized (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a self))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0)))))
  (label 2311)
  (parent 2287)
  (unrealized (0 3) (3 1) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (4 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a self))
  (operation nonce-test (displaced 5 0 init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 2312)
  (parent 2287)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 3) (0 3)) ((4 1) (3 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a self))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2313)
  (parent 2287)
  (unrealized (0 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 0)) ((3 3) (0 3)) ((4 1) (3 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a self))
  (operation nonce-test (displaced 5 1 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 2314)
  (parent 2287)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a self))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2315)
  (parent 2287)
  (unrealized (0 3) (3 1) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (neq self-0 self))
  (operation nonce-test (displaced 2 5 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2316)
  (parent 2287)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (neq a self))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2317)
  (parent 2287)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 2318)
  (parent 2288)
  (unrealized (0 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1)))
  (label 2319)
  (parent 2289)
  (unrealized (0 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1)))
  (label 2320)
  (parent 2290)
  (unrealized (0 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0)))
  (label 2321)
  (parent 2291)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 2322)
  (parent 2292)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x y rndx) (zeta expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 4 1) (trans 4 0) (neq a self))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 2323)
  (parent 2293)
  (unrealized (0 3) (5 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1)))))
  (label 2324)
  (parent 2293)
  (unrealized (0 3) (5 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 2325)
  (parent 2294)
  (unrealized (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (0 0)) ((4 2) (3 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (2 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (neq self-0 self))
  (operation nonce-test (displaced 2 5 ltx-gen 3) (exp (gen) l-0) (4 1))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2326)
  (parent 2295)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a self))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (4 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2327)
  (parent 2295)
  (unrealized (0 3) (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (4 1)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l-0)) l-1))
    (exp (gen) (mul l l-0 (rec l-1))) (4 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1))))
  (label 2328)
  (parent 2296)
  (unrealized (0 3) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((6 1) (4 1)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec l-1))) l-0))
    (exp (gen) (mul l l-0 (rec l-1))) (4 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0))))
  (label 2329)
  (parent 2296)
  (unrealized (0 3) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((6 1) (4 1)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec l-1))) l))
    (exp (gen) (mul l l-0 (rec l-1))) (4 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l))))
  (label 2330)
  (parent 2296)
  (unrealized (0 3) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (5 0)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 2331)
  (parent 2297)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 2332)
  (parent 2297)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x y rndx) (zeta expt) (l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec l-0))) (y zeta) (y l-0))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l (rec w))) w))
    (exp (gen) (mul l l)) (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l l (rec w))) w))
      (send (cat (exp (gen) (mul l l (rec w))) w))))
  (label 2333)
  (parent 2298)
  (unrealized (0 3) (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l) (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec l) (rec w))) w))
    (exp (gen) (rec l)) (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec l) (rec w))) w))
      (send (cat (exp (gen) (mul (rec l) (rec w))) w))))
  (label 2334)
  (parent 2299)
  (unrealized (0 3) (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 data) (b self b-0 name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon beta expt) (x-0 l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l l)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l) (x x-0) (beta beta))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)))
  (uniq-orig na na-0 l)
  (uniq-gen x x-0)
  (absent (x l) (x-0 beta) (x-0 l))
  (precur (4 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv a l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l l)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l l))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2335)
  (parent 2300)
  (unrealized (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) x-0) (mul l l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (5 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv self l) (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 6 1) (trans 1 1)
    (trans 6 0) (trans 1 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv a l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) x-0) (mul l l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l-0)))))
  (label 2336)
  (parent 2300)
  (unrealized (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 1) (4 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation nonce-test (added-listener l) (mul l l) (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l)))
  (label 2337)
  (parent 2302)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pval) (priv-stor locn) (x rndx) (upsilon alpha zeta expt)
    (y l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) y) (mul (rec y) l l)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 3) (4 0)))
  (uniq-orig na nb-0 l)
  (uniq-gen x y)
  (absent (x l) (y alpha) (y zeta) (y l))
  (precur (4 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 1 1) (trans 1 0) (neq self b))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv b-0 l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) y) (mul (rec y) l l)))
      (send (cat (exp (gen) y) (mul (rec y) l l))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l))
              (exp (gen) (mul zeta y))))))))
  (label 2338)
  (parent 2303)
  (unrealized (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon alpha zeta expt)
    (l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) y) (mul l l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 3) (4 0)) ((6 1) (5 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y alpha) (y zeta) (y l-0))
  (precur (4 0))
  (gen-st (pv self l) (pv b-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 1 1) (trans 6 0) (trans 1 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv b-0 l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) y) (mul l l (rec y))))
      (send (cat (exp (gen) y) (mul l l (rec y)))))
    ((load priv-stor-0 (cat pt-1 (pv b-0 l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b-0 l-0)))))
  (label 2339)
  (parent 2303)
  (unrealized (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (trans 5 1) (trans 5 0) (neq self b))
  (operation nonce-test (added-strand ltx-disclose 3) l (4 0)
    (ch-msg priv-stor (cat pt-0 (pv self l))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 2340)
  (parent 2304)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x y rndx) (zeta expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec l-0))) (y zeta) (y l-0))
  (precur (2 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq self b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv b l-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0)))
  (label 2341)
  (parent 2306)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq self b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor (cat pt-0 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 2342)
  (parent 2307)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (la rndx) (beta expt) (l x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) x-0) (mul l l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (uniq-orig na na-0 l)
  (uniq-gen x x-0)
  (absent (x l) (x-0 la) (x-0 beta))
  (precur (2 0) (5 0))
  (gen-st (pv self l) (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) x-0) (mul l l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2343)
  (parent 2308)
  (unrealized (4 0) (5 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (upsilon expt) (l x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) x) (mul l l (rec x))))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0) (5 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation nonce-test (displaced 6 0 init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul upsilon x)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) x) (mul l l (rec x))))
      (send (cat (exp (gen) x) (mul l l (rec x))))))
  (label 2344)
  (parent 2308)
  (unrealized (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (gen) (mul l l)))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0) (5 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation nonce-test (contracted (l-0 l) (w (mul l l))) (gen) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l)))))
  (label 2345)
  (parent 2308)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 nb-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (lb rndx) (alpha zeta expt) (l y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) y) (mul l l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 3) (5 0)))
  (uniq-orig na nb-0 l)
  (uniq-gen x y)
  (absent (x l) (y lb) (y alpha) (y zeta))
  (precur (2 0) (5 0))
  (gen-st (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 1 1) (trans 1 0) (neq self b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) y) (mul l l (rec y))))
      (send (cat (exp (gen) y) (mul l l (rec y)))))
    ((load priv-stor-0 (cat pt-1 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 2346)
  (parent 2308)
  (unrealized (4 0) (5 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 2) (2 0)) ((1 2) (5 0))
    ((2 1) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0) (5 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation nonce-test (displaced 6 1 ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l))))
  (label 2347)
  (parent 2308)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) (mul l l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0) (5 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 6 1) (trans 1 1)
    (trans 6 0) (trans 1 0) (neq self b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) (mul l l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l l (rec l-0)))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2348)
  (parent 2308)
  (unrealized (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (lb l x rndx)
    (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2349)
  (parent 2310)
  (seen 2286)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (self self-0 a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (4 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 2350)
  (parent 2310)
  (unrealized (0 3) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 2351)
  (parent 2310)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (la l y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la la) (x x) (beta l)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha la) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (uniq-orig na nb la l)
  (uniq-gen y x)
  (absent (y (one)) (y la) (y l) (x la) (x l))
  (precur (4 0))
  (gen-st (pv self l) (pv self-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (displaced 5 0 init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 la)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self-0 self
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 la)))
      (send
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 2352)
  (parent 2310)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (displaced 5 1 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 2353)
  (parent 2310)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (self self-0 a b name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 3) (4 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta))
  (precur (4 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 2354)
  (parent 2310)
  (unrealized (0 3) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (displaced 5 3 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 2355)
  (parent 2310)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 3 1) (trans 1 1) (trans 5 0) (trans 3 0)
    (trans 1 0) (neq self-0 self))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 2356)
  (parent 2310)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a self))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 2357)
  (parent 2311)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor) (la l-0)
    (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a self))
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 2358)
  (parent 2311)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (5 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (neq a self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a-0 l-1)))))
  (label 2359)
  (parent 2311)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq a self))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (4 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 2360)
  (parent 2313)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a self a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a self))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2361)
  (parent 2315)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a self a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a self))
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2362)
  (parent 2315)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)) ((6 1) (5 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (neq a self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1)))))
  (label 2363)
  (parent 2315)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (3 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2364)
  (parent 2316)
  (seen 2309)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (neq self-0 self))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2365)
  (parent 2316)
  (unrealized (5 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 2366)
  (parent 2318)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 2367)
  (parent 2318)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (7 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 2368)
  (parent 2319)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 2369)
  (parent 2319)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (7 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 2370)
  (parent 2320)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 2371)
  (parent 2320)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((6 2) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 6 1) (trans 1 1)
    (trans 6 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 2 7 ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2372)
  (parent 2321)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2373)
  (parent 2321)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((1 2) (5 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 7 1 ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 2374)
  (parent 2322)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2375)
  (parent 2322)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (0 3)) ((5 1) (0 0)) ((5 2) (4 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (2 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 1 1) (trans 5 0)
    (trans 1 0) (trans 3 1) (trans 3 0) (neq self-0 self))
  (operation nonce-test (displaced 2 6 ltx-gen 3) (exp (gen) l-0) (5 1))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2376)
  (parent 2323)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x y rndx) (zeta expt) (l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (precur (3 0) (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 4 1) (trans 4 0) (neq a self))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (5 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2377)
  (parent 2323)
  (unrealized (0 3) (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0)) ((6 1) (7 0))
    ((7 1) (5 1)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l-0)) l-1))
    (exp (gen) (mul l l-0 (rec l-1))) (5 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1))))
  (label 2378)
  (parent 2324)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (7 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((7 1) (5 1)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec l-1))) l-0))
    (exp (gen) (mul l l-0 (rec l-1))) (5 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0))))
  (label 2379)
  (parent 2324)
  (seen 2469)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((7 1) (5 1)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec l-1))) l))
    (exp (gen) (mul l l-0 (rec l-1))) (5 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l))))
  (label 2380)
  (parent 2324)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 1) (7 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 2381)
  (parent 2325)
  (seen 2471)
  (unrealized (7 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 2382)
  (parent 2325)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (deflistener (cat (exp (gen) lb) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (0 0)) ((4 2) (3 1)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (2 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (displaced 5 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) lb) (one)))
      (send (cat (exp (gen) lb) (one))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2383)
  (parent 2326)
  (seen 2309)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (5 0)) ((4 1) (0 0)) ((4 2) (3 1)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (2 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (neq self-0 self))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2384)
  (parent 2326)
  (unrealized (5 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)) ((6 2) (5 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (3 0) (5 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a self))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0)))))
  (label 2385)
  (parent 2327)
  (unrealized (0 3) (4 1) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (5 0)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0))
  (precur (3 0) (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a self))
  (operation nonce-test (displaced 6 0 init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 2386)
  (parent 2327)
  (unrealized (0 3) (4 1) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a self))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2387)
  (parent 2327)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a self))
  (operation nonce-test (displaced 6 1 ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 2388)
  (parent 2327)
  (unrealized (0 3) (4 1) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)) ((6 3) (5 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta-0))
  (precur (3 0) (5 0))
  (gen-st (pv a l-0) (pv self l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a self))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2389)
  (parent 2327)
  (unrealized (0 3) (4 1) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 1) (0 0)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (2 0) (4 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 1 1) (trans 5 0)
    (trans 1 0) (neq self-0 self))
  (operation nonce-test (displaced 2 6 ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2390)
  (parent 2327)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (neq a self))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2391)
  (parent 2327)
  (unrealized (0 3) (4 1) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0)) ((6 1) (4 1))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0)
    (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (6 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 2392)
  (parent 2328)
  (unrealized (0 3) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((6 1) (4 1))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0)
    (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (6 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1)))
  (label 2393)
  (parent 2329)
  (unrealized (0 3) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((6 1) (4 1))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0)
    (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (6 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1)))
  (label 2394)
  (parent 2330)
  (unrealized (0 3) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (6 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0)))
  (label 2395)
  (parent 2331)
  (unrealized (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (6 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 2396)
  (parent 2332)
  (unrealized (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (x-0 la)
    (x-0 beta))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2397)
  (parent 2333)
  (unrealized (0 3) (4 0) (6 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 l-0 (rec x))))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (x l-0))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 7 0 init 3) (exp (gen) x-0) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 l-0 (rec x))))))
  (label 2398)
  (parent 2333)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation nonce-test (contracted (l-1 l-0) (w (mul l-0 l-0))) (gen)
    (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0)))))
  (label 2399)
  (parent 2333)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 7 1 ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0))))
  (label 2400)
  (parent 2333)
  (unrealized (0 3) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (y-0 lb)
    (y-0 alpha) (y-0 zeta-0))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq self b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2401)
  (parent 2333)
  (unrealized (0 3) (4 0) (6 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (2 0))
    ((6 1) (4 0)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 1 1) (trans 6 0)
    (trans 1 0) (trans 4 1) (trans 4 0) (neq self self-0))
  (operation nonce-test (displaced 3 7 ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l l (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2402)
  (parent 2333)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 7 1) (trans 3 1) (trans 1 1)
    (trans 7 0) (trans 3 0) (trans 1 0) (trans 5 1) (trans 5 0)
    (neq self b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2403)
  (parent 2333)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (x-0 la)
    (x-0 beta))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2404)
  (parent 2334)
  (unrealized (0 3) (4 0) (6 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l la x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta la) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la la))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) la))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul (rec l) (rec x))))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (uniq-orig na nb l la)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la la)) (y l) (x la))
  (precur (6 0))
  (gen-st (pv b l) (pv self la))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 7 0 init 3) (exp (gen) x-0) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self la)))
      (recv
        (sig (body b (exp (gen) la) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la la)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) la la)) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la la)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) la))
      (send (cat (exp (gen) (rec l)) la)))
    ((load priv-stor (cat pt-0 (pv self la)))
      (stor priv-stor (cat pt-3 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul (rec l) (rec x))))
      (send (cat (exp (gen) x) (mul (rec l) (rec x))))))
  (label 2405)
  (parent 2334)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation nonce-test (contracted (l-1 l-0) (w (rec l-0))) (gen)
    (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0)))))
  (label 2406)
  (parent 2334)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec l) (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((1 2) (6 0)) ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 7 1 ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec l) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l) (rec l-0))))))
  (label 2407)
  (parent 2334)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (y-0 lb)
    (y-0 alpha) (y-0 zeta-0))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq self b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2408)
  (parent 2334)
  (unrealized (0 3) (4 0) (6 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (2 0)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 1 1) (trans 6 0)
    (trans 1 0) (trans 4 1) (trans 4 0) (neq self self-0))
  (operation nonce-test (displaced 3 7 ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2409)
  (parent 2334)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 7 1) (trans 3 1) (trans 1 1)
    (trans 7 0) (trans 3 0) (trans 1 0) (trans 5 1) (trans 5 0)
    (neq self b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2410)
  (parent 2334)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (trans 6 1) (trans 6 0) (neq self b))
  (operation nonce-test (added-strand ltx-disclose 3) l (5 0)
    (ch-msg priv-stor (cat pt-0 (pv self l))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 2411)
  (parent 2337)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (2 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((1 2) (3 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 2) (3 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (3 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (trans 4 1) (trans 4 0) (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 2412)
  (parent 2340)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x y rndx) (zeta expt) (l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec l-0))) (y zeta) (y l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq self b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l (rec w))) w))
    (exp (gen) (mul l l)) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l l (rec w))) w))
      (send (cat (exp (gen) (mul l l (rec w))) w))))
  (label 2413)
  (parent 2341)
  (unrealized (0 3) (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l) (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq self b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec l) (rec w))) w))
    (exp (gen) (rec l)) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec l) (rec w))) w))
      (send (cat (exp (gen) (mul (rec l) (rec w))) w))))
  (label 2414)
  (parent 2342)
  (unrealized (0 3) (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 data) (b self b-0 name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon beta expt) (x-0 l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l l)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)))
  (uniq-orig na na-0 l)
  (uniq-gen x x-0)
  (absent (x l) (x-0 beta) (x-0 l))
  (precur (2 0) (5 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv a l-0))) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l l)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l l))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2415)
  (parent 2343)
  (unrealized (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) x-0) (mul l l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (6 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 beta) (x-0 l-0))
  (precur (2 0) (5 0))
  (gen-st (pv self l) (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 7 1) (trans 1 1)
    (trans 7 0) (trans 1 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv a l-0))) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) x-0) (mul l l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l-0)))))
  (label 2416)
  (parent 2343)
  (unrealized (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0) (5 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (neq self b))
  (operation nonce-test (added-listener l) (mul l l) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l)))
  (label 2417)
  (parent 2345)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pval) (priv-stor locn) (x rndx) (upsilon alpha zeta expt)
    (y l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) y) (mul (rec y) l l)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 3) (5 0)))
  (uniq-orig na nb-0 l)
  (uniq-gen x y)
  (absent (x l) (y alpha) (y zeta) (y l))
  (precur (2 0) (5 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 1 1) (trans 1 0) (neq self b))
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv b-0 l-0))) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) y) (mul (rec y) l l)))
      (send (cat (exp (gen) y) (mul (rec y) l l))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l))
              (exp (gen) (mul zeta y))))))))
  (label 2418)
  (parent 2346)
  (unrealized (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon alpha zeta expt)
    (l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) y) (mul l l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 3) (5 0))
    ((7 1) (6 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y alpha) (y zeta) (y l-0))
  (precur (2 0) (5 0))
  (gen-st (pv self l) (pv b-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 7 1) (trans 1 1) (trans 7 0) (trans 1 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv b-0 l-0))) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) y) (mul l l (rec y))))
      (send (cat (exp (gen) y) (mul l l (rec y)))))
    ((load priv-stor-0 (cat pt-1 (pv b-0 l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b-0 l-0)))))
  (label 2419)
  (parent 2346)
  (unrealized (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((1 2) (5 0)) ((2 1) (0 1)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0) (5 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (trans 6 1) (trans 6 0) (neq self b))
  (operation nonce-test (added-strand ltx-disclose 3) l (5 0)
    (ch-msg priv-stor (cat pt-0 (pv self l))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 2420)
  (parent 2347)
  (seen 2340)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (self self-0 b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (4 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3))
    ((5 2) (4 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self-0 self))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 2421)
  (parent 2350)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (self self-0 b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (0 3))
    ((5 2) (4 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (neq self-0 self))
  (operation channel-test (displaced 6 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self-0 b (exp (gen) x-0)))))
  (label 2422)
  (parent 2350)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (self self-0 a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 2) (4 0))
    ((6 1) (5 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 3 1) (trans 1 1) (trans 6 0) (trans 3 0)
    (trans 1 0) (neq self-0 self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 2423)
  (parent 2350)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (self self-0 a name) (pt pt-0 pt-1 pt-2 pval)
    (priv-stor priv-stor-0 locn) (l rndx) (alpha zeta expt)
    (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (4 0)) ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3))
    ((5 3) (4 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (neq self-0 self))
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 2424)
  (parent 2354)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (self self-0 a name) (pt pt-0 pt-1 pt-2 pval)
    (priv-stor priv-stor-0 locn) (l rndx) (alpha zeta expt)
    (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 1) (5 0)) ((3 2) (2 1)) ((4 1) (0 3))
    ((5 3) (4 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (neq self-0 self))
  (operation channel-test (displaced 6 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self-0
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 2425)
  (parent 2354)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (self self-0 a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)) ((5 3) (4 0))
    ((6 1) (5 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-1))
  (precur (4 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 3 1) (trans 1 1) (trans 6 0) (trans 3 0) (trans 1 0)
    (neq self-0 self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1)))))
  (label 2426)
  (parent 2354)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (4 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0)
    (neq a self))
  (operation nonce-test (displaced 6 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 2427)
  (parent 2360)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq a self))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2428)
  (parent 2360)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (lb l x rndx)
    (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (3 0) (5 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2429)
  (parent 2365)
  (seen 2349)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (self self-0 a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (3 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 2430)
  (parent 2365)
  (unrealized (0 3) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 2431)
  (parent 2365)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (la l y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la la) (x x) (beta l)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha la) (zeta (one)))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (uniq-orig na nb la l)
  (uniq-gen y x)
  (absent (y (one)) (y la) (y l) (x la) (x l))
  (precur (3 0) (5 0))
  (gen-st (pv self l) (pv self-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (displaced 6 0 init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 la)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self-0 self
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 la)))
      (send
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 2432)
  (parent 2365)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (displaced 6 1 ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 2433)
  (parent 2365)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (self self-0 a b name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 3) (5 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta))
  (precur (3 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 2434)
  (parent 2365)
  (unrealized (0 3) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (displaced 6 4 ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 2435)
  (parent 2365)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 4 1) (trans 1 1) (trans 6 0) (trans 4 0)
    (trans 1 0) (neq self-0 self))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 2436)
  (parent 2365)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1)))
  (label 2437)
  (parent 2366)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 2438)
  (parent 2367)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 2439)
  (parent 2368)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 2440)
  (parent 2369)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 2441)
  (parent 2370)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1)))
  (label 2442)
  (parent 2371)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((5 2) (3 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 5 1) (trans 1 1)
    (trans 5 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq self-0 b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2443)
  (parent 2372)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 2444)
  (parent 2373)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (upsilon expt) (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (7 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 8 0 init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul upsilon x)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 2445)
  (parent 2373)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2446)
  (parent 2373)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((1 2) (7 0))
    ((2 1) (0 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 8 1 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 2447)
  (parent 2373)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l lb rndx) (alpha zeta expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y lb) (y alpha) (y zeta))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 2448)
  (parent 2373)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 2) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 7 1) (trans 1 1)
    (trans 7 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 2 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2449)
  (parent 2373)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 8 1) (trans 2 1)
    (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2450)
  (parent 2373)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 0)) ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 2) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 2451)
  (parent 2374)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 la))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 2452)
  (parent 2375)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (upsilon expt) (la l x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener
    (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) l) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (uniq-orig na la l)
  (uniq-gen x)
  (absent (x la) (x l))
  (precur (7 0))
  (gen-st (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 8 0 init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la l))
              (exp (gen) (mul upsilon x)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) l) la)) (send (cat (exp (gen) l) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-3 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 2453)
  (parent 2375)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2454)
  (parent 2375)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 8 1 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2455)
  (parent 2375)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l lb rndx) (alpha zeta expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y lb) (y alpha) (y zeta))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 2456)
  (parent 2375)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 1) (2 0))
    ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 1) (5 0)) ((7 2) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 7 1) (trans 1 1)
    (trans 7 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 2 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2457)
  (parent 2375)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 8 1) (trans 2 1)
    (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2458)
  (parent 2375)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (deflistener (cat (gen) lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (0 3)) ((5 1) (0 0)) ((5 2) (4 1)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (2 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 5 1)
    (trans 1 1) (trans 5 0) (trans 1 0) (trans 3 1) (trans 3 0)
    (neq self-0 self))
  (operation nonce-test (displaced 6 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) lb)) (send (cat (gen) lb)))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2459)
  (parent 2376)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (6 0)) ((5 1) (0 0)) ((5 2) (4 1))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (2 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 1 1) (trans 5 0)
    (trans 1 0) (trans 3 1) (trans 3 0) (neq self-0 self))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2460)
  (parent 2376)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 1))
    ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (3 0) (6 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 4 1) (trans 4 0) (neq a self))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0)))))
  (label 2461)
  (parent 2377)
  (unrealized (0 3) (5 1) (6 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0))
  (precur (3 0) (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 4 1) (trans 4 0) (neq a self))
  (operation nonce-test (displaced 7 0 init 3) (exp (gen) x-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 2462)
  (parent 2377)
  (unrealized (0 3) (5 1) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (6 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 4 1) (trans 4 0) (neq a self))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2463)
  (parent 2377)
  (unrealized (0 3) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((1 2) (6 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 4 1) (trans 4 0) (neq a self))
  (operation nonce-test (displaced 7 1 ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 2464)
  (parent 2377)
  (unrealized (0 3) (5 1) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 1))
    ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta-0))
  (precur (3 0) (6 0))
  (gen-st (pv a l-0) (pv self l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a self))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2465)
  (parent 2377)
  (unrealized (0 3) (5 1) (6 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (0 3)) ((5 1) (4 1)) ((6 1) (0 0))
    ((6 2) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (2 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 1 1) (trans 6 0)
    (trans 1 0) (trans 3 1) (trans 3 0) (neq self-0 self))
  (operation nonce-test (displaced 2 7 ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2466)
  (parent 2377)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 1))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (6 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 7 1) (trans 2 1) (trans 1 1)
    (trans 7 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a self))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2467)
  (parent 2377)
  (unrealized (0 3) (5 1) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0)) ((6 1) (8 0))
    ((7 1) (5 1)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 2468)
  (parent 2378)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((7 1) (5 1)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b))
  (operation nonce-test (displaced 8 4 ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1))))
  (label 2469)
  (parent 2379)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((7 1) (5 1)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 2470)
  (parent 2380)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 8 4 ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 2471)
  (parent 2381)
  (unrealized (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 2) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 2472)
  (parent 2382)
  (unrealized (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (lb l x rndx)
    (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (deflistener (cat (exp (gen) lb) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (5 0)) ((4 1) (0 0)) ((4 2) (3 1)) ((5 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (2 0) (5 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (displaced 6 3 resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) lb) (one)))
      (send (cat (exp (gen) lb) (one))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2473)
  (parent 2384)
  (seen 2349)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (self self-0 a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (5 0)) ((4 1) (0 0)) ((4 2) (3 1)) ((5 1) (0 3))
    ((6 2) (5 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (2 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 2474)
  (parent 2384)
  (unrealized (0 3) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (5 0)) ((4 1) (0 0)) ((4 2) (3 1)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (2 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 2475)
  (parent 2384)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (la l y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la la) (x x) (beta l)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha la) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (5 0)) ((4 1) (0 0)) ((4 2) (3 1)) ((5 1) (0 3)))
  (uniq-orig na nb la l)
  (uniq-gen y x)
  (absent (y (one)) (y la) (y l) (x la) (x l))
  (precur (2 0) (5 0))
  (gen-st (pv self l) (pv self-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (displaced 6 0 init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 la)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self-0 self
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 la)))
      (send
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 2476)
  (parent 2384)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (5 0)) ((4 1) (0 0)) ((4 2) (3 1)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (2 0) (5 0))
  (gen-st (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (displaced 6 1 ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 2477)
  (parent 2384)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (self self-0 a b name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (5 0)) ((4 1) (0 0)) ((4 2) (3 1)) ((5 1) (0 3))
    ((6 3) (5 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta))
  (precur (2 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 2478)
  (parent 2384)
  (unrealized (0 3) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (5 0)) ((4 1) (0 0)) ((4 2) (3 1)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (2 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (displaced 6 4 ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 2479)
  (parent 2384)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (5 0)) ((4 1) (0 0)) ((4 2) (3 1)) ((5 1) (0 3))
    ((6 2) (5 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (2 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 4 1) (trans 1 1) (trans 6 0) (trans 4 0)
    (trans 1 0) (neq self-0 self))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 2480)
  (parent 2384)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 1) (6 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a self))
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 2481)
  (parent 2385)
  (unrealized (0 3) (4 1) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor) (la l-0)
    (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (neq a self))
  (operation channel-test (displaced 7 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 2482)
  (parent 2385)
  (unrealized (0 3) (4 1) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)) ((6 2) (5 0))
    ((7 1) (6 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (3 0) (5 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 7 1) (trans 2 1) (trans 1 1)
    (trans 7 0) (trans 2 0) (trans 1 0) (neq a self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a-0 l-1)))))
  (label 2483)
  (parent 2385)
  (unrealized (0 3) (4 1) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a self))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 2484)
  (parent 2387)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a self a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 1) (6 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 3) (5 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a self))
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2485)
  (parent 2389)
  (unrealized (0 3) (4 1) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a self a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 3) (5 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (neq a self))
  (operation channel-test (displaced 7 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2486)
  (parent 2389)
  (unrealized (0 3) (4 1) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)) ((6 3) (5 0))
    ((7 1) (6 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-1))
  (precur (3 0) (5 0))
  (gen-st (pv a l-0) (pv self l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 7 1) (trans 2 1) (trans 1 1) (trans 7 0) (trans 2 0)
    (trans 1 0) (neq a self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1)))))
  (label 2487)
  (parent 2389)
  (unrealized (0 3) (4 1) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (deflistener (cat (exp (gen) lb) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 1) (0 0)) ((5 2) (4 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (2 0) (4 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 5 1)
    (trans 1 1) (trans 5 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (displaced 6 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) lb) (one)))
      (send (cat (exp (gen) lb) (one))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2488)
  (parent 2390)
  (seen 2364)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (6 0)) ((4 1) (3 1)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (2 0) (4 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 1 1) (trans 5 0)
    (trans 1 0) (neq self-0 self))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2489)
  (parent 2390)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 2490)
  (parent 2392)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 2491)
  (parent 2392)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 2492)
  (parent 2393)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 2493)
  (parent 2393)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 2494)
  (parent 2394)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 2495)
  (parent 2394)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (0 0)) ((7 2) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (2 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 7 1) (trans 1 1)
    (trans 7 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq self-0 b))
  (operation nonce-test (displaced 2 8 ltx-gen 3) (exp (gen) l-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2496)
  (parent 2395)
  (seen 2372)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2497)
  (parent 2395)
  (unrealized (8 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 8 1 ltx-gen 3) (exp (gen) l-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 2498)
  (parent 2396)
  (seen 2374)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2499)
  (parent 2396)
  (unrealized (8 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0 l-0)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (x-0 beta)
    (x-0 l-0))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0 l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0 l-0))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2500)
  (parent 2397)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l l (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (x-0 beta)
    (x-0 l-0))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation channel-test (displaced 8 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2501)
  (parent 2397)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (7 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (x-0 beta)
    (x-0 l-1))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 8 1) (trans 3 1) (trans 1 1)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 5 1) (trans 5 0)
    (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))))
  (label 2502)
  (parent 2397)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation nonce-test (added-listener l-0) (mul l-0 l-0) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0)))
  (label 2503)
  (parent 2399)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (6 0)) ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (5 0))
    ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (6 0)
    (ch-msg priv-stor (cat pt-0 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 2504)
  (parent 2400)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0 l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-0))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq self b))
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0 l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0 l-0))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2505)
  (parent 2401)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-0))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq self b))
  (operation channel-test (displaced 8 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l l (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2506)
  (parent 2401)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 3) (6 0)) ((8 1) (7 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-1))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 2507)
  (parent 2401)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l) (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((1 2) (0 1)) ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (x-0 beta)
    (x-0 l-0))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l) (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2508)
  (parent 2404)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) (rec l-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (x-0 beta)
    (x-0 l-0))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation channel-test (displaced 8 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) (rec l-0))))
      (send (cat (exp (gen) x-0) (mul (rec x-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2509)
  (parent 2404)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (7 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (x-0 beta)
    (x-0 l-1))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 8 1) (trans 3 1) (trans 1 1)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 5 1) (trans 5 0)
    (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))))
  (label 2510)
  (parent 2404)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation nonce-test (added-listener l-0) (rec l-0) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0)))
  (label 2511)
  (parent 2406)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((1 2) (0 1)) ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-0))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq self b))
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l) (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2512)
  (parent 2408)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) (rec l-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-0))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq self b))
  (operation channel-test (displaced 8 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) (rec l-0))))
      (send (cat (exp (gen) y-0) (mul (rec y-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2513)
  (parent 2408)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 3) (6 0)) ((8 1) (7 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-1))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 2514)
  (parent 2408)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (2 0)) ((6 1) (7 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 1 1) (trans 6 0)
    (trans 1 0) (trans 4 1) (trans 4 0) (neq self self-0))
  (operation nonce-test (added-listener l-0) (mul (rec l-0) (rec l-0))
    (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0)))
  (label 2515)
  (parent 2409)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (2 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (3 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (trans 5 1) (trans 5 0) (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 2516)
  (parent 2411)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((1 2) (2 0))
    ((2 1) (0 3)) ((3 2) (2 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (trans 3 1) (trans 3 0) (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 2517)
  (parent 2412)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (x-0 la)
    (x-0 beta))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq self b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2518)
  (parent 2413)
  (unrealized (0 3) (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 l-0 (rec x))))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (x l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq self b))
  (operation nonce-test (displaced 8 0 init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 l-0 (rec x))))))
  (label 2519)
  (parent 2413)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq self b))
  (operation nonce-test (contracted (l-1 l-0) (w (mul l-0 l-0))) (gen)
    (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0)))))
  (label 2520)
  (parent 2413)
  (unrealized (0 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((1 2) (7 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq self b))
  (operation nonce-test (displaced 8 1 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0))))
  (label 2521)
  (parent 2413)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (y-0 lb)
    (y-0 alpha) (y-0 zeta-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq self b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2522)
  (parent 2413)
  (unrealized (0 3) (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (3 0)) ((7 1) (5 0)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 7 1) (trans 1 1) (trans 7 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq self self-0))
  (operation nonce-test (displaced 4 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l l (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2523)
  (parent 2413)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 8 1) (trans 4 1) (trans 1 1)
    (trans 8 0) (trans 4 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq self b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2524)
  (parent 2413)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (x-0 la)
    (x-0 beta))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq self b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2525)
  (parent 2414)
  (unrealized (0 3) (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l la x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta la) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la la))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) la))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul (rec l) (rec x))))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l la)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la la)) (y l) (x la))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self la))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq self b))
  (operation nonce-test (displaced 8 0 init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self la)))
      (recv
        (sig (body b (exp (gen) la) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la la)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) la la)) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la la)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) la))
      (send (cat (exp (gen) (rec l)) la)))
    ((load priv-stor (cat pt-0 (pv self la)))
      (stor priv-stor (cat pt-3 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul (rec l) (rec x))))
      (send (cat (exp (gen) x) (mul (rec l) (rec x))))))
  (label 2526)
  (parent 2414)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq self b))
  (operation nonce-test (contracted (l-1 l-0) (w (rec l-0))) (gen)
    (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0)))))
  (label 2527)
  (parent 2414)
  (unrealized (0 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec l) (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((1 2) (7 0)) ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq self b))
  (operation nonce-test (displaced 8 1 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec l) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l) (rec l-0))))))
  (label 2528)
  (parent 2414)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (y-0 lb)
    (y-0 alpha) (y-0 zeta-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq self b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2529)
  (parent 2414)
  (unrealized (0 3) (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 7 1) (trans 1 1) (trans 7 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq self self-0))
  (operation nonce-test (displaced 4 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2530)
  (parent 2414)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 8 1) (trans 4 1) (trans 1 1)
    (trans 8 0) (trans 4 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq self b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2531)
  (parent 2414)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0) (5 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (trans 7 1) (trans 7 0) (neq self b))
  (operation nonce-test (added-strand ltx-disclose 3) l (6 0)
    (ch-msg priv-stor (cat pt-0 (pv self l))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 2532)
  (parent 2417)
  (seen 2411)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (0 3))
    ((4 2) (3 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (3 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 4 1) (trans 4 0) (neq a self))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 2533)
  (parent 2427)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq a self))
  (operation nonce-test (displaced 7 3 resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2534)
  (parent 2428)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a self))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0)))))
  (label 2535)
  (parent 2428)
  (unrealized (0 3) (6 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a self))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 2536)
  (parent 2428)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (la l y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la la) (x x) (beta l) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha la) (zeta (one)))
  (deflistener (cat (gen) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb la l)
  (uniq-gen y x)
  (absent (y (one)) (y la) (y l) (x la) (x l))
  (precur (4 0) (6 0))
  (gen-st (pv a la) (pv self l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a self))
  (operation nonce-test (displaced 7 0 init 3) (exp (gen) x-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((recv (cat (gen) la)) (send (cat (gen) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-3 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 2537)
  (parent 2428)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a self))
  (operation nonce-test (displaced 7 1 ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 2538)
  (parent 2428)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv self l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0)
    (neq a self))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 2539)
  (parent 2428)
  (unrealized (0 3) (6 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (0 0))
    ((6 1) (4 0)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 1 1) (trans 6 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq self-0 self))
  (operation nonce-test (displaced 2 7 ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2540)
  (parent 2428)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 7 1) (trans 2 1) (trans 1 1) (trans 7 0) (trans 2 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq a self))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2541)
  (parent 2428)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (self self-0 b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (0 3)) ((6 2) (5 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self-0 self))
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 2542)
  (parent 2430)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (self self-0 b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (0 3)) ((6 2) (5 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self-0 self))
  (operation channel-test (displaced 7 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self-0 b (exp (gen) x-0)))))
  (label 2543)
  (parent 2430)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (self self-0 a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 2) (5 0)) ((7 1) (6 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (3 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 7 1) (trans 4 1) (trans 1 1) (trans 7 0) (trans 4 0)
    (trans 1 0) (neq self-0 self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 2544)
  (parent 2430)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (self self-0 a name) (pt pt-0 pt-1 pt-2 pval)
    (priv-stor priv-stor-0 locn) (l rndx) (alpha zeta expt)
    (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 3) (5 0)) ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0))
    ((5 1) (0 3)) ((6 3) (5 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (neq self-0 self))
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 2545)
  (parent 2434)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (self self-0 a name) (pt pt-0 pt-1 pt-2 pval)
    (priv-stor priv-stor-0 locn) (l rndx) (alpha zeta expt)
    (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (0 3)) ((6 3) (5 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (neq self-0 self))
  (operation channel-test (displaced 7 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self-0
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 2546)
  (parent 2434)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (self self-0 a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 3) (5 0)) ((7 1) (6 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-1))
  (precur (3 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 7 1)
    (trans 4 1) (trans 1 1) (trans 7 0) (trans 4 0) (trans 1 0)
    (neq self-0 self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1)))))
  (label 2547)
  (parent 2434)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 8 1) (trans 3 1) (trans 1 1)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 2 9 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2548)
  (parent 2437)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2549)
  (parent 2437)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 9 1 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 2550)
  (parent 2438)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2551)
  (parent 2438)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 8 1) (trans 3 1) (trans 1 1)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 2 9 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2552)
  (parent 2439)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2553)
  (parent 2439)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec l) (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec l) (rec w))) w))
    (exp (gen) (rec l)) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec l) (rec w))) w))
      (send (cat (exp (gen) (mul (rec l) (rec w))) w))))
  (label 2554)
  (parent 2440)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 9 1 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0)))
  (label 2555)
  (parent 2441)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2556)
  (parent 2441)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec l) (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec l) (rec w))) w))
    (exp (gen) (rec l)) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec l) (rec w))) w))
      (send (cat (exp (gen) (mul (rec l) (rec w))) w))))
  (label 2557)
  (parent 2442)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (2 0))
    ((4 1) (0 0)) ((4 2) (2 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq self-0 b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2558)
  (parent 2443)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2559)
  (parent 2444)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2560)
  (parent 2444)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 9 1) (trans 2 1)
    (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 2561)
  (parent 2444)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 2562)
  (parent 2446)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon alpha zeta expt)
    (l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 2563)
  (parent 2448)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon expt) (l rndx)
    (alpha zeta expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec y) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 2564)
  (parent 2448)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (alpha zeta expt) (l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-1))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l) (pv b-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 2565)
  (parent 2448)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (0 0))
    ((6 2) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 6 1) (trans 1 1)
    (trans 6 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq self-0 b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2566)
  (parent 2449)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (0 3)) ((4 2) (3 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 2567)
  (parent 2451)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2568)
  (parent 2452)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((2 1) (8 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2569)
  (parent 2452)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 9 1) (trans 2 1)
    (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 2570)
  (parent 2452)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 2571)
  (parent 2454)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (3 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2572)
  (parent 2455)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon expt) (l rndx)
    (alpha zeta expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec y) l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 2573)
  (parent 2456)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon alpha zeta expt)
    (l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((2 1) (8 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 2574)
  (parent 2456)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (alpha zeta expt) (l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 2575)
  (parent 2456)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (3 2)) ((1 1) (2 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (0 0)) ((4 2) (3 1)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (trans 2 1) (trans 2 0)
    (neq self-0 self))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2576)
  (parent 2459)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (deflistener (cat (gen) lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (6 0)) ((5 1) (0 0)) ((5 2) (4 1))
    ((6 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (2 0) (6 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 5 1) (trans 1 1)
    (trans 5 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq self-0 self))
  (operation nonce-test (displaced 7 4 resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) lb)) (send (cat (gen) lb)))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2577)
  (parent 2460)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (self self-0 a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (6 0)) ((5 1) (0 0)) ((5 2) (4 1))
    ((6 1) (0 3)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (2 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 1 1) (trans 5 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 self))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 2578)
  (parent 2460)
  (unrealized (0 3) (6 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (6 0)) ((5 1) (0 0)) ((5 2) (4 1))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (2 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 1 1) (trans 5 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 self))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 2579)
  (parent 2460)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (la l y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la la) (x x) (beta l)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha la) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (6 0)) ((5 1) (0 0)) ((5 2) (4 1))
    ((6 1) (0 3)))
  (uniq-orig na nb la l)
  (uniq-gen y x)
  (absent (y (one)) (y la) (y l) (x la) (x l))
  (precur (2 0) (6 0))
  (gen-st (pv self l) (pv self-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 1 1) (trans 5 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 self))
  (operation nonce-test (displaced 7 0 init 3) (exp (gen) x-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 la)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self-0 self
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 la)))
      (send
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 2580)
  (parent 2460)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((1 2) (6 0))
    ((2 1) (0 1)) ((3 2) (2 0)) ((4 3) (6 0)) ((5 1) (0 0))
    ((5 2) (4 1)) ((6 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (2 0) (6 0))
  (gen-st (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 1 1) (trans 5 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 self))
  (operation nonce-test (displaced 7 1 ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 2581)
  (parent 2460)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (self self-0 a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (6 0)) ((5 1) (0 0)) ((5 2) (4 1))
    ((6 1) (0 3)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta))
  (precur (2 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 1 1) (trans 5 0) (trans 1 0) (trans 3 1) (trans 3 0)
    (neq self-0 self))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 2582)
  (parent 2460)
  (unrealized (0 3) (6 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (6 0)) ((5 1) (0 0)) ((5 2) (4 1))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (2 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 1 1) (trans 5 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 self))
  (operation nonce-test (displaced 7 5 ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 2583)
  (parent 2460)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (6 0)) ((5 1) (0 0)) ((5 2) (4 1))
    ((6 1) (0 3)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (2 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 7 1) (trans 5 1) (trans 1 1) (trans 7 0) (trans 5 0)
    (trans 1 0) (trans 3 1) (trans 3 0) (neq self-0 self))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 2584)
  (parent 2460)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 4 1) (trans 4 0) (neq a self))
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 2585)
  (parent 2461)
  (unrealized (0 3) (5 1) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor) (la l-0)
    (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (6 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 4 1) (trans 4 0) (neq a self))
  (operation channel-test (displaced 8 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 2586)
  (parent 2461)
  (unrealized (0 3) (5 1) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 1))
    ((7 2) (6 0)) ((8 1) (7 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (3 0) (6 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 8 1) (trans 2 1) (trans 1 1)
    (trans 8 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 2587)
  (parent 2461)
  (unrealized (0 3) (5 1) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (6 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 4 1) (trans 7 0) (trans 4 0)
    (neq a self))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (6 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 2588)
  (parent 2463)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (3 0) (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a self))
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2589)
  (parent 2465)
  (unrealized (0 3) (5 1) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (3 0) (6 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a self))
  (operation channel-test (displaced 8 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2590)
  (parent 2465)
  (unrealized (0 3) (5 1) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 1))
    ((7 3) (6 0)) ((8 1) (7 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-1))
  (precur (3 0) (6 0))
  (gen-st (pv a l-0) (pv self l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 8 1) (trans 2 1) (trans 1 1) (trans 8 0) (trans 2 0)
    (trans 1 0) (trans 4 1) (trans 4 0) (neq a self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1)))))
  (label 2591)
  (parent 2465)
  (unrealized (0 3) (5 1) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (deflistener (cat (gen) lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (0 3)) ((5 1) (4 1)) ((6 1) (0 0))
    ((6 2) (5 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (2 0) (5 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 6 1)
    (trans 1 1) (trans 6 0) (trans 1 0) (trans 3 1) (trans 3 0)
    (neq self-0 self))
  (operation nonce-test (displaced 7 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) lb)) (send (cat (gen) lb)))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2592)
  (parent 2466)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (7 0)) ((5 1) (4 1)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (2 0) (5 0) (7 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 1 1) (trans 6 0)
    (trans 1 0) (trans 3 1) (trans 3 0) (neq self-0 self))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2593)
  (parent 2466)
  (unrealized (7 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (9 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 2594)
  (parent 2468)
  (seen 2752)
  (unrealized (0 3) (9 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 2595)
  (parent 2468)
  (unrealized (0 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 2596)
  (parent 2469)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 2597)
  (parent 2469)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (9 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 2598)
  (parent 2470)
  (unrealized (0 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 2599)
  (parent 2470)
  (seen 2757)
  (unrealized (0 3) (9 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 1) (0 0)) ((7 2) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (2 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 7 1) (trans 1 1)
    (trans 7 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 2 8 ltx-gen 3) (exp (gen) l-0) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2600)
  (parent 2471)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 1) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2601)
  (parent 2471)
  (unrealized (8 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 2) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (displaced 9 1 ltx-gen 3) (exp (gen) l-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 2602)
  (parent 2472)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2603)
  (parent 2472)
  (unrealized (9 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (self self-0 b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (5 0)) ((4 1) (0 0)) ((4 2) (3 1))
    ((5 1) (0 3)) ((6 2) (5 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (2 0) (5 0))
  (gen-st (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self-0 self))
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 2604)
  (parent 2474)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (self self-0 b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (5 0)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 1))
    ((5 1) (0 3)) ((6 2) (5 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (2 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (neq self-0 self))
  (operation channel-test (displaced 7 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self-0 b (exp (gen) x-0)))))
  (label 2605)
  (parent 2474)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (self self-0 a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (5 0)) ((4 1) (0 0)) ((4 2) (3 1)) ((5 1) (0 3))
    ((6 2) (5 0)) ((7 1) (6 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (2 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 7 1) (trans 4 1) (trans 1 1) (trans 7 0) (trans 4 0)
    (trans 1 0) (neq self-0 self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 2606)
  (parent 2474)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (self self-0 a name) (pt pt-0 pt-1 pt-2 pval)
    (priv-stor priv-stor-0 locn) (l rndx) (alpha zeta expt)
    (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (5 0)) ((4 1) (0 0)) ((4 2) (3 1))
    ((5 1) (0 3)) ((6 3) (5 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (2 0) (5 0))
  (gen-st (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (neq self-0 self))
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 2607)
  (parent 2478)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (self self-0 a name) (pt pt-0 pt-1 pt-2 pval)
    (priv-stor priv-stor-0 locn) (l rndx) (alpha zeta expt)
    (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (5 0)) ((4 1) (0 0)) ((4 1) (6 0)) ((4 2) (3 1))
    ((5 1) (0 3)) ((6 3) (5 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (2 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (neq self-0 self))
  (operation channel-test (displaced 7 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self-0
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 2608)
  (parent 2478)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (self self-0 a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (5 0)) ((4 1) (0 0)) ((4 2) (3 1)) ((5 1) (0 3))
    ((6 3) (5 0)) ((7 1) (6 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-1))
  (precur (2 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 7 1)
    (trans 4 1) (trans 1 1) (trans 7 0) (trans 4 0) (trans 1 0)
    (neq self-0 self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1)))))
  (label 2609)
  (parent 2478)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) lb) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (3 0) (5 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a self))
  (operation nonce-test (displaced 7 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) lb) (one)))
      (send (cat (exp (gen) lb) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 2610)
  (parent 2484)
  (seen 2427)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (7 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a self))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2611)
  (parent 2484)
  (unrealized (7 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (lb l x rndx)
    (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (deflistener (cat (exp (gen) lb) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (6 0)) ((4 1) (3 1)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (2 0) (4 0) (6 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 5 1) (trans 1 1)
    (trans 5 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (displaced 7 3 resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) lb) (one)))
      (send (cat (exp (gen) lb) (one))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2612)
  (parent 2489)
  (seen 2429)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (self self-0 a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (6 0)) ((4 1) (3 1)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (2 0) (4 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 1 1) (trans 5 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 2613)
  (parent 2489)
  (unrealized (0 3) (6 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (6 0)) ((4 1) (3 1)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (2 0) (4 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 1 1) (trans 5 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 2614)
  (parent 2489)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (la l y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la la) (x x) (beta l)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha la) (zeta (one)))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (6 0)) ((4 1) (3 1)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb la l)
  (uniq-gen y x)
  (absent (y (one)) (y la) (y l) (x la) (x l))
  (precur (2 0) (4 0) (6 0))
  (gen-st (pv self l) (pv self-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 1 1) (trans 5 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (displaced 7 0 init 3) (exp (gen) x-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 la)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self-0 self
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 la)))
      (send
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 2615)
  (parent 2489)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (6 0)) ((4 1) (3 1)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (2 0) (4 0) (6 0))
  (gen-st (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 1 1) (trans 5 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (displaced 7 1 ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 2616)
  (parent 2489)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (self self-0 a b name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (6 0)) ((4 1) (3 1)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta))
  (precur (2 0) (4 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 1 1) (trans 5 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 2617)
  (parent 2489)
  (unrealized (0 3) (6 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (6 0)) ((4 1) (3 1)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (2 0) (4 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 1 1) (trans 5 0) (trans 1 0) (neq self-0 self))
  (operation nonce-test (displaced 7 5 ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 2618)
  (parent 2489)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (6 0)) ((4 1) (3 1)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (2 0) (4 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 7 1) (trans 5 1) (trans 1 1) (trans 7 0) (trans 5 0)
    (trans 1 0) (neq self-0 self))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 2619)
  (parent 2489)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (8 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1)))
  (label 2620)
  (parent 2490)
  (unrealized (0 3) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (8 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 2621)
  (parent 2491)
  (unrealized (0 3) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (8 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 2622)
  (parent 2492)
  (unrealized (0 3) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (8 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 2623)
  (parent 2493)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (8 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 2624)
  (parent 2494)
  (unrealized (0 3) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (8 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1)))
  (label 2625)
  (parent 2495)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 2626)
  (parent 2497)
  (unrealized (6 0) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (upsilon expt) (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (8 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 9 0 init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul upsilon x)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 2627)
  (parent 2497)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0) (neq a b))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2628)
  (parent 2497)
  (unrealized (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 9 1 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 2629)
  (parent 2497)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l lb rndx) (alpha zeta expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 3) (8 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y lb) (y alpha) (y zeta))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 7 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 2630)
  (parent 2497)
  (unrealized (6 0) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (2 0) (7 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 8 1) (trans 1 1)
    (trans 8 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq self-0 b))
  (operation nonce-test (displaced 2 9 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2631)
  (parent 2497)
  (seen 2449)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 9 1) (trans 2 1)
    (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2632)
  (parent 2497)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv a-0 la))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 2633)
  (parent 2499)
  (unrealized (6 0) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (upsilon expt) (la l x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) l) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (8 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na la l)
  (uniq-gen x)
  (absent (x la) (x l))
  (precur (3 0) (8 0))
  (gen-st (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 9 0 init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la l))
              (exp (gen) (mul upsilon x)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) l) la)) (send (cat (exp (gen) l) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-3 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 2634)
  (parent 2499)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (5 0)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0) (neq a b))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2635)
  (parent 2499)
  (unrealized (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 9 1 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2636)
  (parent 2499)
  (seen 2455)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l lb rndx) (alpha zeta expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 3) (8 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y lb) (y alpha) (y zeta))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 7 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 2637)
  (parent 2499)
  (unrealized (6 0) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 2) (2 0)) ((2 1) (0 1)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)) ((8 1) (6 0)) ((8 2) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (2 0) (7 0))
  (gen-st (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 8 1) (trans 1 1)
    (trans 8 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq self-0 b))
  (operation nonce-test (displaced 2 9 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2638)
  (parent 2499)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 9 1) (trans 2 1)
    (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2639)
  (parent 2499)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 8 1) (trans 5 1) (trans 8 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor (cat pt-0 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 2640)
  (parent 2503)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (6 0)) ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (5 0))
    ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 8 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l)) lb))
      (send (cat (exp (gen) (mul l l)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 2641)
  (parent 2504)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (6 0)) ((2 3) (8 0)) ((3 1) (2 0)) ((3 1) (5 0))
    ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0))
    ((8 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2642)
  (parent 2504)
  (unrealized (8 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (8 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 8 1) (trans 5 1) (trans 8 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv b l-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 2643)
  (parent 2511)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (2 0)) ((6 1) (8 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 1 1) (trans 6 0)
    (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0) (trans 4 0)
    (neq self self-0))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor-0 (cat pt-3 (pv self-0 l-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 2644)
  (parent 2515)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 2) (3 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (trans 4 1) (trans 4 0) (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 2645)
  (parent 2516)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (trans 2 1) (trans 2 0) (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 2646)
  (parent 2517)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta l) (a self) (b b) (x x) (upsilon upsilon) (na na)
        (nb nb) (priv-stor priv-stor) (pt pt-0) (self self)
        (priv-stor-0 priv-stor) (ignore ignore) (pt-0 pt) (pt-1 pt-0))))
  (origs (pt-1 (2 1)) (l (1 1)) (pt-0 (1 1)) (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0 l-0)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (x-0 beta)
    (x-0 l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq self b))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0 l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0 l-0))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2647)
  (parent 2518)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l l (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (x-0 beta)
    (x-0 l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq self b))
  (operation channel-test (displaced 9 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2648)
  (parent 2518)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (x-0 beta)
    (x-0 l-1))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 4 1) (trans 1 1)
    (trans 9 0) (trans 4 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))))
  (label 2649)
  (parent 2518)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq self b))
  (operation nonce-test (added-listener l-0) (mul l-0 l-0) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0)))
  (label 2650)
  (parent 2520)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((1 2) (7 0)) ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor (cat pt-0 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 2651)
  (parent 2521)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0 l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq self b))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0 l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0 l-0))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2652)
  (parent 2522)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq self b))
  (operation channel-test (displaced 9 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l l (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2653)
  (parent 2522)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-1))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 2654)
  (parent 2522)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l) (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((1 2) (2 0)) ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (x-0 beta)
    (x-0 l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq self b))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l) (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2655)
  (parent 2525)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) (rec l-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (x-0 beta)
    (x-0 l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq self b))
  (operation channel-test (displaced 9 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) (rec l-0))))
      (send (cat (exp (gen) x-0) (mul (rec x-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2656)
  (parent 2525)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (x-0 beta)
    (x-0 l-1))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 4 1) (trans 1 1)
    (trans 9 0) (trans 4 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))))
  (label 2657)
  (parent 2525)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq self b))
  (operation nonce-test (added-listener l-0) (rec l-0) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0)))
  (label 2658)
  (parent 2527)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((1 2) (2 0)) ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq self b))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l) (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2659)
  (parent 2529)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) (rec l-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq self b))
  (operation channel-test (displaced 9 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) (rec l-0))))
      (send (cat (exp (gen) y-0) (mul (rec y-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2660)
  (parent 2529)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-1))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 9 1) (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 2661)
  (parent 2529)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((7 1) (8 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 7 1) (trans 1 1) (trans 7 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq self self-0))
  (operation nonce-test (added-listener l-0) (mul (rec l-0) (rec l-0))
    (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0)))
  (label 2662)
  (parent 2530)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0)) ((3 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 3 1) (trans 3 0) (neq a self))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 2663)
  (parent 2533)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (3 0) (5 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a self))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2664)
  (parent 2534)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (5 0)) ((3 3) (6 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (0 3)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a self))
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 2665)
  (parent 2535)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor) (la l-0)
    (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((2 1) (7 0)) ((3 3) (6 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (0 3)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a self))
  (operation channel-test (displaced 8 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 2666)
  (parent 2535)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((8 1) (7 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 2 1) (trans 1 1) (trans 8 0) (trans 2 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq a self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 2667)
  (parent 2535)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (5 0)) ((3 3) (6 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (0 3)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0)
    (neq a self))
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 2668)
  (parent 2539)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((2 1) (7 0)) ((3 3) (6 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (0 3)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0)
    (neq a self))
  (operation channel-test (displaced 8 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 2669)
  (parent 2539)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)) ((7 3) (6 0)) ((8 1) (7 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-1))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv self l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 2 1) (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0)
    (trans 5 1) (trans 5 0) (neq a self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1)))))
  (label 2670)
  (parent 2539)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 9 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2671)
  (parent 2548)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 8 1) (trans 3 1) (trans 1 1)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2672)
  (parent 2548)
  (unrealized (9 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2673)
  (parent 2549)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 2674)
  (parent 2549)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 2675)
  (parent 2549)
  (unrealized (0 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) l-0 l-1)) (y l) (x l-0) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 2676)
  (parent 2549)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 3 1) (trans 1 1)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 2 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2677)
  (parent 2549)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((1 2) (9 0))
    ((2 1) (0 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 2678)
  (parent 2549)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (5 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2679)
  (parent 2549)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 10 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2680)
  (parent 2549)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 9 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 2681)
  (parent 2550)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2682)
  (parent 2550)
  (unrealized (9 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2683)
  (parent 2551)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 2684)
  (parent 2551)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (9 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 2685)
  (parent 2551)
  (unrealized (0 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 2686)
  (parent 2551)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((2 3) (0 3)) ((3 1) (2 0))
    ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 1) (7 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 3 1) (trans 1 1)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 2 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2687)
  (parent 2551)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 2688)
  (parent 2551)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (5 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2689)
  (parent 2551)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 10 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2690)
  (parent 2551)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 9 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2691)
  (parent 2552)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 8 1) (trans 3 1) (trans 1 1)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2692)
  (parent 2552)
  (unrealized (9 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2693)
  (parent 2553)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 2694)
  (parent 2553)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 2695)
  (parent 2553)
  (unrealized (0 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) l-0 l-1)) (y l) (x l-0) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 2696)
  (parent 2553)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 3 1) (trans 1 1)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 2 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2697)
  (parent 2553)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((1 2) (9 0))
    ((2 1) (0 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 2698)
  (parent 2553)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (7 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2699)
  (parent 2553)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 10 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2700)
  (parent 2553)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2701)
  (parent 2554)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 2702)
  (parent 2554)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w (rec l-1))) (gen)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1)))))
  (label 2703)
  (parent 2554)
  (unrealized (0 3) (7 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul (rec l) (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) la))
      (send (cat (exp (gen) (rec l)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul (rec l) (rec x))))
      (send (cat (exp (gen) x) (mul (rec l) (rec x))))))
  (label 2704)
  (parent 2554)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 1) (7 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 3 1) (trans 1 1)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 2 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2705)
  (parent 2554)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((1 2) (9 0))
    ((2 1) (0 0)) ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))))
  (label 2706)
  (parent 2554)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2707)
  (parent 2554)
  (unrealized (0 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 10 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2708)
  (parent 2554)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 9 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 2709)
  (parent 2555)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2710)
  (parent 2555)
  (unrealized (9 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2711)
  (parent 2556)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 2712)
  (parent 2556)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (9 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 2713)
  (parent 2556)
  (unrealized (0 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 2714)
  (parent 2556)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((2 3) (0 3)) ((3 1) (2 0))
    ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 1) (5 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 3 1) (trans 1 1)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 2 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2715)
  (parent 2556)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 2716)
  (parent 2556)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (7 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2717)
  (parent 2556)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 10 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2718)
  (parent 2556)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2719)
  (parent 2557)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 2720)
  (parent 2557)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w (rec l-1))) (gen)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1)))))
  (label 2721)
  (parent 2557)
  (unrealized (0 3) (7 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul (rec l) (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul (rec l) (rec x))))
      (send (cat (exp (gen) x) (mul (rec l) (rec x))))))
  (label 2722)
  (parent 2557)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 1) (5 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 3 1) (trans 1 1)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 2 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2723)
  (parent 2557)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((1 2) (9 0))
    ((2 1) (0 0)) ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))))
  (label 2724)
  (parent 2557)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2725)
  (parent 2557)
  (unrealized (0 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 10 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2726)
  (parent 2557)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)) ((3 1) (0 0))
    ((3 2) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2727)
  (parent 2558)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l-0) (beta l) (a self-0) (b b) (x x) (upsilon upsilon)
        (na na) (nb nb) (priv-stor priv-stor-0) (pt pt-3) (self self)
        (priv-stor-0 priv-stor) (ignore ignore) (pt-0 pt) (pt-1 pt-0))))
  (origs (l-0 (3 1)) (pt-3 (3 1)) (pt-1 (2 1)) (l (1 1)) (pt-0 (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0)
    (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 2728)
  (parent 2562)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (2 0))
    ((4 1) (2 0)) ((5 1) (0 0)) ((5 2) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 5 1) (trans 1 1)
    (trans 5 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq self-0 b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2729)
  (parent 2566)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0)) ((3 2) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 2730)
  (parent 2567)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta l-0) (a a) (b b) (x x) (upsilon upsilon) (na na)
        (nb nb) (priv-stor priv-stor) (pt pt) (self self)
        (priv-stor-0 priv-stor-0) (ignore ignore) (pt-0 pt-0)
        (pt-1 pt-1))))
  (origs (l-0 (1 1)) (pt-1 (1 1)) (pt-3 (3 1)) (l (2 1)) (pt (2 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0)
    (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 2731)
  (parent 2571)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 2) (0 1)) ((1 2) (5 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2732)
  (parent 2572)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 0)) ((3 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 self))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2733)
  (parent 2576)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (2 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 3) (5 0)) ((4 1) (0 0)) ((4 2) (3 1)) ((5 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 self))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2734)
  (parent 2577)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((1 1) (7 0))
    ((2 1) (0 1)) ((3 2) (2 0)) ((4 3) (6 0)) ((5 1) (0 0))
    ((5 2) (4 1)) ((6 1) (0 3)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (2 0) (6 0))
  (gen-st (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 1 1) (trans 5 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 self))
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 2735)
  (parent 2578)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (6 0)) ((5 1) (0 0)) ((5 1) (7 0))
    ((5 2) (4 1)) ((6 1) (0 3)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (2 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 1 1) (trans 5 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 self))
  (operation channel-test (displaced 8 5 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self-0 b (exp (gen) x-0)))))
  (label 2736)
  (parent 2578)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (self self-0 a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (6 0)) ((5 1) (0 0)) ((5 2) (4 1))
    ((6 1) (0 3)) ((7 2) (6 0)) ((8 1) (7 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (2 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 5 1) (trans 1 1) (trans 8 0) (trans 5 0)
    (trans 1 0) (trans 3 1) (trans 3 0) (neq self-0 self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))))
  (label 2737)
  (parent 2578)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 locn) (l rndx) (alpha zeta expt)
    (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((1 1) (7 0))
    ((2 1) (0 1)) ((3 2) (2 0)) ((4 3) (6 0)) ((5 1) (0 0))
    ((5 2) (4 1)) ((6 1) (0 3)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (2 0) (6 0))
  (gen-st (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 1 1) (trans 5 0) (trans 1 0) (trans 3 1) (trans 3 0)
    (neq self-0 self))
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 2738)
  (parent 2582)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 locn) (l rndx) (alpha zeta expt)
    (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (6 0)) ((5 1) (0 0)) ((5 1) (7 0))
    ((5 2) (4 1)) ((6 1) (0 3)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (2 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 1 1) (trans 5 0) (trans 1 0) (trans 3 1) (trans 3 0)
    (neq self-0 self))
  (operation channel-test (displaced 8 5 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self-0
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 2739)
  (parent 2582)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (self self-0 a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (6 0)) ((5 1) (0 0)) ((5 2) (4 1))
    ((6 1) (0 3)) ((7 3) (6 0)) ((8 1) (7 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-1))
  (precur (2 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 5 1) (trans 1 1) (trans 8 0) (trans 5 0) (trans 1 0)
    (trans 3 1) (trans 3 0) (neq self-0 self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1)))))
  (label 2740)
  (parent 2582)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 2) (6 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (3 0) (6 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (trans 7 1) (trans 4 1)
    (trans 7 0) (trans 4 0) (neq a self))
  (operation nonce-test (displaced 8 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) lb)) (send (cat (gen) lb)))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 2741)
  (parent 2588)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (8 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (6 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 4 1) (trans 7 0) (trans 4 0)
    (neq a self))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2742)
  (parent 2588)
  (unrealized (8 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (3 2)) ((1 1) (2 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 1) (0 0)) ((5 2) (4 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (4 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 5 1)
    (trans 1 1) (trans 5 0) (trans 1 0) (trans 2 1) (trans 2 0)
    (neq self-0 self))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2743)
  (parent 2592)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (deflistener (cat (gen) lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (7 0)) ((5 1) (4 1)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (2 0) (5 0) (7 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 6 1) (trans 1 1)
    (trans 6 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq self-0 self))
  (operation nonce-test (displaced 8 4 resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) lb)) (send (cat (gen) lb)))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2744)
  (parent 2593)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (self self-0 a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (7 0)) ((5 1) (4 1)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)) ((8 2) (7 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (2 0) (5 0) (7 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 1 1) (trans 6 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 self))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 2745)
  (parent 2593)
  (unrealized (0 3) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (7 0)) ((5 1) (4 1)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (2 0) (5 0) (7 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 1 1) (trans 6 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 self))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 2746)
  (parent 2593)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (la l y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la la) (x x) (beta l)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha la) (zeta (one)))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (7 0)) ((5 1) (4 1)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb la l)
  (uniq-gen y x)
  (absent (y (one)) (y la) (y l) (x la) (x l))
  (precur (2 0) (5 0) (7 0))
  (gen-st (pv self l) (pv self-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 1 1) (trans 6 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 self))
  (operation nonce-test (displaced 8 0 init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 la)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self-0 self
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 la)))
      (send
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 2747)
  (parent 2593)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((1 2) (7 0))
    ((2 1) (0 1)) ((3 2) (2 0)) ((4 3) (7 0)) ((5 1) (4 1))
    ((6 1) (0 0)) ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (2 0) (5 0) (7 0))
  (gen-st (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 1 1) (trans 6 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 self))
  (operation nonce-test (displaced 8 1 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 2748)
  (parent 2593)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (self self-0 a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (7 0)) ((5 1) (4 1)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)) ((8 3) (7 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta))
  (precur (2 0) (5 0) (7 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 1 1) (trans 6 0) (trans 1 0) (trans 3 1) (trans 3 0)
    (neq self-0 self))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 2749)
  (parent 2593)
  (unrealized (0 3) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (7 0)) ((5 1) (4 1)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (2 0) (5 0) (7 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 1 1) (trans 6 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 self))
  (operation nonce-test (displaced 8 6 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 2750)
  (parent 2593)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (7 0)) ((5 1) (4 1)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (2 0) (5 0) (7 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 6 1) (trans 1 1) (trans 8 0) (trans 6 0)
    (trans 1 0) (trans 3 1) (trans 3 0) (neq self-0 self))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 2751)
  (parent 2593)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 10 4 ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1))))
  (label 2752)
  (parent 2594)
  (unrealized (0 3) (9 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1)))
  (label 2753)
  (parent 2595)
  (unrealized (0 3) (9 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 4 1)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (8 0)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 2754)
  (parent 2596)
  (unrealized (0 3) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 4 1)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (8 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 2755)
  (parent 2597)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 2756)
  (parent 2598)
  (unrealized (0 3) (9 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 10 4 ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1))))
  (label 2757)
  (parent 2599)
  (unrealized (0 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 1) (0 0))
    ((6 2) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 6 1) (trans 1 1)
    (trans 6 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2758)
  (parent 2600)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 2759)
  (parent 2601)
  (unrealized (7 0) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (upsilon expt) (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (8 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 1) (6 0))
    ((8 1) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 9 0 init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul upsilon x)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 2760)
  (parent 2601)
  (unrealized (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 1) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2761)
  (parent 2601)
  (unrealized (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 1) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 9 1 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 2762)
  (parent 2601)
  (unrealized (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l lb rndx) (alpha zeta expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 3) (8 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y lb) (y alpha) (y zeta))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 2763)
  (parent 2601)
  (unrealized (7 0) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 1) (0 0)) ((8 2) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (2 0) (7 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 8 1) (trans 1 1)
    (trans 8 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 2 9 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2764)
  (parent 2601)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 9 1) (trans 2 1)
    (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2765)
  (parent 2601)
  (unrealized (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 2) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 2766)
  (parent 2602)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv a-0 la))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-5 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 2767)
  (parent 2603)
  (unrealized (7 0) (9 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (upsilon expt) (la l x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) l) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na la l)
  (uniq-gen x)
  (absent (x la) (x l))
  (precur (3 0) (9 0))
  (gen-st (pv a la) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la l))
              (exp (gen) (mul upsilon x)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) l) la)) (send (cat (exp (gen) l) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-4 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 2768)
  (parent 2603)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 1) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2769)
  (parent 2603)
  (seen 2986)
  (unrealized (9 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2770)
  (parent 2603)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l lb rndx) (alpha zeta expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y lb) (y alpha) (y zeta))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 4 1) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-5 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 2771)
  (parent 2603)
  (unrealized (7 0) (9 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 1) (0 0)) ((9 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (2 0) (8 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 9 1) (trans 1 1)
    (trans 9 0) (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0)
    (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 2 10 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0)))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2772)
  (parent 2603)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 10 1) (trans 2 1)
    (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 4 1) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2773)
  (parent 2603)
  (unrealized (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) lb) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (7 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0) (neq a self))
  (operation nonce-test (displaced 8 4 resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) lb) (one)))
      (send (cat (exp (gen) lb) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2774)
  (parent 2611)
  (seen 2534)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (7 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (0 3)) ((8 2) (7 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a self))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0)))))
  (label 2775)
  (parent 2611)
  (unrealized (0 3) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (7 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a self))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 2776)
  (parent 2611)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (la l y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la la) (x x) (beta l) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha la) (zeta (one)))
  (deflistener (cat (gen) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (7 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb la l)
  (uniq-gen y x)
  (absent (y (one)) (y la) (y l) (x la) (x l))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv a la) (pv self l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a self))
  (operation nonce-test (displaced 8 0 init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((recv (cat (gen) la)) (send (cat (gen) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-3 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 2777)
  (parent 2611)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (7 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a self))
  (operation nonce-test (displaced 8 1 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 2778)
  (parent 2611)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (7 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (0 3)) ((8 3) (7 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv a l-0) (pv self l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a self))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 2779)
  (parent 2611)
  (unrealized (0 3) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (0 3))
    ((7 1) (0 0)) ((7 1) (5 0)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (2 0) (4 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 7 1) (trans 1 1) (trans 7 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq self-0 self))
  (operation nonce-test (displaced 2 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2780)
  (parent 2611)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (7 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (0 3)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 2 1) (trans 1 1) (trans 8 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a self))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2781)
  (parent 2611)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (self self-0 b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 1) (7 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 1) (0 0))
    ((5 2) (4 0)) ((6 1) (0 3)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (2 0) (4 0) (6 0))
  (gen-st (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 1 1) (trans 5 0) (trans 1 0) (neq self-0 self))
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 2782)
  (parent 2613)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (self self-0 b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (6 0)) ((4 1) (3 1)) ((5 1) (0 0)) ((5 1) (7 0))
    ((5 2) (4 0)) ((6 1) (0 3)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (2 0) (4 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 1 1) (trans 5 0) (trans 1 0) (neq self-0 self))
  (operation channel-test (displaced 8 5 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self-0 b (exp (gen) x-0)))))
  (label 2783)
  (parent 2613)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (self self-0 a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (6 0)) ((4 1) (3 1)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)) ((7 2) (6 0)) ((8 1) (7 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (2 0) (4 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 5 1) (trans 1 1) (trans 8 0) (trans 5 0)
    (trans 1 0) (neq self-0 self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a l-1))) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))))
  (label 2784)
  (parent 2613)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (self self-0 a name) (pt pt-0 pt-1 pt-2 pval)
    (priv-stor priv-stor-0 locn) (l rndx) (alpha zeta expt)
    (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 1) (7 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 1) (0 0))
    ((5 2) (4 0)) ((6 1) (0 3)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (2 0) (4 0) (6 0))
  (gen-st (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 1 1) (trans 5 0) (trans 1 0) (neq self-0 self))
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 2785)
  (parent 2617)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (self self-0 a name) (pt pt-0 pt-1 pt-2 pval)
    (priv-stor priv-stor-0 locn) (l rndx) (alpha zeta expt)
    (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (6 0)) ((4 1) (3 1)) ((5 1) (0 0)) ((5 1) (7 0))
    ((5 2) (4 0)) ((6 1) (0 3)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (2 0) (4 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 1 1) (trans 5 0) (trans 1 0) (neq self-0 self))
  (operation channel-test (displaced 8 5 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self-0
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 2786)
  (parent 2617)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (self self-0 a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (6 0)) ((4 1) (3 1)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)) ((7 3) (6 0)) ((8 1) (7 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-1))
  (precur (2 0) (4 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 5 1) (trans 1 1) (trans 8 0) (trans 5 0) (trans 1 0)
    (neq self-0 self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1)))))
  (label 2787)
  (parent 2617)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 4 1) (trans 1 1)
    (trans 9 0) (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (displaced 2 10 ltx-gen 3) (exp (gen) l-1)
    (8 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2788)
  (parent 2620)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2789)
  (parent 2620)
  (unrealized (0 3) (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-1)
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 2790)
  (parent 2621)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2791)
  (parent 2621)
  (unrealized (0 3) (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 4 1) (trans 1 1)
    (trans 9 0) (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (displaced 2 10 ltx-gen 3) (exp (gen) l-1)
    (8 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2792)
  (parent 2622)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2793)
  (parent 2622)
  (unrealized (0 3) (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec l) (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec l) (rec w))) w))
    (exp (gen) (rec l)) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec l) (rec w))) w))
      (send (cat (exp (gen) (mul (rec l) (rec w))) w))))
  (label 2794)
  (parent 2623)
  (unrealized (0 3) (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-1)
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0)))
  (label 2795)
  (parent 2624)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2796)
  (parent 2624)
  (unrealized (0 3) (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec l) (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec l) (rec w))) w))
    (exp (gen) (rec l)) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec l) (rec w))) w))
      (send (cat (exp (gen) (mul (rec l) (rec w))) w))))
  (label 2797)
  (parent 2625)
  (unrealized (0 3) (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 1) (9 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0) (neq a b))
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2798)
  (parent 2626)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0) (neq a b))
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2799)
  (parent 2626)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (9 0)))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 10 1) (trans 2 1)
    (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 2800)
  (parent 2626)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (8 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 2801)
  (parent 2628)
  (seen 2562)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon alpha zeta expt)
    (l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 1) (9 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 3) (8 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 7 0) (neq a b))
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 2802)
  (parent 2630)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon expt) (l rndx)
    (alpha zeta expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 3) (8 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 7 0) (neq a b))
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec y) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 2803)
  (parent 2630)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (alpha zeta expt) (l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 3) (8 0)) ((10 1) (9 0)))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-1))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l) (pv b-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 2804)
  (parent 2630)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (5 0)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0) (neq a b))
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2805)
  (parent 2633)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 7 0) (neq a b))
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2806)
  (parent 2633)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (9 0)))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 10 1) (trans 2 1)
    (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 2807)
  (parent 2633)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (5 0)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (8 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 2808)
  (parent 2635)
  (seen 2571)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon expt) (l rndx)
    (alpha zeta expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 0)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 3) (8 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 7 0) (neq a b))
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec y) l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 2809)
  (parent 2637)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon alpha zeta expt)
    (l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 3) (8 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 7 0) (neq a b))
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 2810)
  (parent 2637)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (alpha zeta expt) (l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 3) (8 0)) ((10 1) (9 0)))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-1))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 2811)
  (parent 2637)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (trans 8 1) (trans 5 1)
    (trans 8 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 9 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l)) lb))
      (send (cat (exp (gen) (mul l l)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 2812)
  (parent 2640)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0) (9 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 8 1) (trans 5 1) (trans 8 0) (trans 5 0)
    (neq self b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2813)
  (parent 2640)
  (unrealized (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((1 2) (5 0))
    ((2 1) (4 0)) ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 2) (5 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 4 1) (trans 6 0) (trans 4 0)
    (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l)) lb))
      (send (cat (exp (gen) (mul l l)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 2814)
  (parent 2641)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (6 0)) ((2 3) (8 0)) ((3 1) (2 0)) ((3 1) (5 0))
    ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0))
    ((8 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (8 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0)
    (trans 5 0) (neq self b))
  (operation nonce-test (displaced 9 2 resp 4) (exp (gen) y-0) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l)) lb))
      (send (cat (exp (gen) (mul l l)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2815)
  (parent 2642)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (6 0)) ((2 3) (8 0)) ((3 1) (2 0)) ((3 1) (5 0))
    ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 2) (8 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (x-0 la)
    (x-0 beta))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-5 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2816)
  (parent 2642)
  (unrealized (0 3) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (6 0)) ((2 3) (8 0)) ((3 1) (2 0)) ((3 1) (5 0))
    ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0))
    ((8 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 2817)
  (parent 2642)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l la y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta la) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la la))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul la la)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) la) la))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (6 0)) ((2 3) (8 0)) ((3 1) (2 0)) ((3 1) (5 0))
    ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0))
    ((8 1) (0 3)))
  (uniq-orig na nb l la)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la la)) (y l) (x la))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 9 0 init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self la)))
      (recv
        (sig (body b (exp (gen) la) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la la)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) la la)) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la la)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul la la)) l))
      (send (cat (exp (gen) (mul la la)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) la) la)) (send (cat (exp (gen) la) la)))
    ((load priv-stor (cat pt-0 (pv self la)))
      (stor priv-stor (cat pt-4 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 2818)
  (parent 2642)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (6 0)) ((2 3) (8 0)) ((3 1) (2 0)) ((3 1) (5 0))
    ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0))
    ((8 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 9 1 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 2819)
  (parent 2642)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (6 0)) ((2 3) (8 0)) ((3 1) (2 0)) ((3 1) (5 0))
    ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 3) (8 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (y-0 lb)
    (y-0 alpha) (y-0 zeta))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-5 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 2820)
  (parent 2642)
  (unrealized (0 3) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((1 2) (5 0)) ((2 3) (7 0)) ((3 1) (2 1)) ((4 2) (3 0))
    ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (0 3)) ((8 1) (2 0))
    ((8 1) (4 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0))
  (precur (5 0) (7 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 1 1) (trans 8 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq self self-0))
  (operation nonce-test (displaced 3 9 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0)))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2821)
  (parent 2642)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (6 0)) ((2 3) (8 0)) ((3 1) (2 0)) ((3 1) (5 0))
    ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2822)
  (parent 2642)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (rec lb)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (8 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (trans 8 1) (trans 5 1)
    (trans 8 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 9 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (rec lb)) l))
      (send (cat (exp (gen) (rec lb)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb)))
  (label 2823)
  (parent 2643)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (8 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (6 0) (9 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 8 1) (trans 5 1) (trans 8 0) (trans 5 0)
    (neq self b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2824)
  (parent 2643)
  (unrealized (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta x))
  (deflistener (cat (exp (gen) (rec lb)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l lb))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (2 0)) ((6 1) (8 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 6 1)
    (trans 1 1) (trans 6 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq self self-0))
  (operation nonce-test (displaced 9 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (rec lb)) l))
      (send (cat (exp (gen) (rec lb)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb)))
  (label 2825)
  (parent 2644)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (2 0)) ((6 1) (8 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (5 0) (9 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 1 1) (trans 6 0)
    (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0) (trans 4 0)
    (neq self self-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2826)
  (parent 2644)
  (unrealized (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 2) (2 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 1 1) (trans 1 0)
    (trans 3 1) (trans 3 0) (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))) ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 2827)
  (parent 2645)
  (seen 2646)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 9 1) (trans 6 1) (trans 9 0) (trans 6 0)
    (neq self b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (8 0)
    (ch-msg priv-stor (cat pt-0 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 2828)
  (parent 2650)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((1 2) (7 0)) ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (2 0) (7 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq self b))
  (operation nonce-test (displaced 9 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l)) lb))
      (send (cat (exp (gen) (mul l l)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 2829)
  (parent 2651)
  (seen 2641)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((1 2) (7 0)) ((2 1) (0 1)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0) (9 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2830)
  (parent 2651)
  (unrealized (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (9 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 9 1) (trans 6 1) (trans 9 0) (trans 6 0)
    (neq self b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (8 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv b l-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 2831)
  (parent 2658)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((7 1) (9 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 7 1) (trans 1 1) (trans 7 0)
    (trans 1 0) (trans 9 1) (trans 5 1) (trans 9 0) (trans 5 0)
    (neq self self-0))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (8 0)
    (ch-msg priv-stor-0 (cat pt-3 (pv self-0 l-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 2832)
  (parent 2662)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0)) ((3 2) (0 3)))
  (uniq-orig na lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 3 1) (trans 3 0) (neq a self))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 2833)
  (parent 2663)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (4 0)) ((4 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a self))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2834)
  (parent 2664)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (4 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (0 0))
    ((7 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 7 1) (trans 2 1) (trans 1 1)
    (trans 7 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq self-0 b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2835)
  (parent 2671)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 8 1) (trans 3 1)
    (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2836)
  (parent 2672)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2837)
  (parent 2672)
  (unrealized (0 3) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 2838)
  (parent 2672)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) la) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self-0 b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) la l-0))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) la) l-0)) (send (cat (exp (gen) la) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 la)))
      (send
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 2839)
  (parent 2672)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 2840)
  (parent 2672)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 2841)
  (parent 2672)
  (unrealized (0 3) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-1) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (0 0)) ((7 2) (5 0)) ((8 1) (0 3)) ((9 1) (2 0))
    ((9 1) (4 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv self-1 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 7 1) (trans 1 1) (trans 9 0) (trans 7 0)
    (trans 1 0) (trans 6 1) (trans 4 1) (trans 6 0) (trans 4 0)
    (neq self-0 self-1))
  (operation nonce-test (displaced 3 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig (body self-1 (exp (gen) l) (pubk "sig" self-1))
          (privk "sig" self-1)))
      (send (cat na self-0 self-1 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 self-1 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-2 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 2842)
  (parent 2672)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation nonce-test (displaced 10 8 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 2843)
  (parent 2672)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 8 1) (trans 3 1) (trans 1 1) (trans 10 0)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-1 l-2)))
      (send
        (sig (body self-1 (exp (gen) l-2) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 2844)
  (parent 2672)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2845)
  (parent 2673)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2846)
  (parent 2673)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2847)
  (parent 2673)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 2848)
  (parent 2673)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2849)
  (parent 2674)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2850)
  (parent 2674)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2851)
  (parent 2674)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 2852)
  (parent 2674)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1)))
  (label 2853)
  (parent 2675)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2854)
  (parent 2677)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 3 1) (trans 1 1)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2855)
  (parent 2677)
  (unrealized (10 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (5 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 2856)
  (parent 2681)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2857)
  (parent 2682)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 2858)
  (parent 2682)
  (unrealized (0 3) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 2859)
  (parent 2682)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 2860)
  (parent 2682)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 2861)
  (parent 2682)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 2862)
  (parent 2682)
  (unrealized (0 3) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (8 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 1) (0 0))
    ((9 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation nonce-test (displaced 2 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2863)
  (parent 2682)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (8 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 1) (3 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self-0))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2864)
  (parent 2682)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 10 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2865)
  (parent 2682)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2866)
  (parent 2683)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2867)
  (parent 2683)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2868)
  (parent 2683)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 2869)
  (parent 2683)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2870)
  (parent 2684)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2871)
  (parent 2684)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2872)
  (parent 2684)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 2873)
  (parent 2684)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1)))
  (label 2874)
  (parent 2685)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2875)
  (parent 2688)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2876)
  (parent 2688)
  (unrealized (10 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (0 0))
    ((7 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 7 1) (trans 2 1) (trans 1 1)
    (trans 7 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq self-0 b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2877)
  (parent 2691)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 8 1) (trans 3 1)
    (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2878)
  (parent 2692)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2879)
  (parent 2692)
  (unrealized (0 3) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 2880)
  (parent 2692)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) la) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self-0 b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) la l-0))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) la) l)) (send (cat (exp (gen) la) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 la)))
      (send
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 2881)
  (parent 2692)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 2882)
  (parent 2692)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 2883)
  (parent 2692)
  (unrealized (0 3) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-1) (priv-stor priv-stor-1)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 3) (8 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (0 0)) ((7 2) (5 0)) ((8 1) (0 3)) ((9 1) (2 0))
    ((9 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv self-1 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 7 1) (trans 1 1) (trans 9 0) (trans 7 0)
    (trans 1 0) (trans 6 1) (trans 4 1) (trans 6 0) (trans 4 0)
    (neq self-0 self-1))
  (operation nonce-test (displaced 3 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig (body self-1 (exp (gen) l) (pubk "sig" self-1))
          (privk "sig" self-1)))
      (send (cat na self-0 self-1 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 self-1 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (stor priv-stor-1 (cat pt-2 "nil")) (send l-1))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 2884)
  (parent 2692)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation nonce-test (displaced 10 8 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 2885)
  (parent 2692)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 8 1) (trans 3 1) (trans 1 1) (trans 10 0)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-1 l-2)))
      (send
        (sig (body self-1 (exp (gen) l-2) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 2886)
  (parent 2692)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2887)
  (parent 2693)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2888)
  (parent 2693)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2889)
  (parent 2693)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 2890)
  (parent 2693)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2891)
  (parent 2694)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2892)
  (parent 2694)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2893)
  (parent 2694)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 2894)
  (parent 2694)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1)))
  (label 2895)
  (parent 2695)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2896)
  (parent 2697)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 3 1) (trans 1 1)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2897)
  (parent 2697)
  (unrealized (10 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2898)
  (parent 2701)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2899)
  (parent 2701)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
      (send (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2900)
  (parent 2701)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 2901)
  (parent 2701)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2902)
  (parent 2702)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2903)
  (parent 2702)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
      (send (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2904)
  (parent 2702)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 2905)
  (parent 2702)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-listener l-1) (rec l-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1)))
  (label 2906)
  (parent 2703)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (10 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (added-listener l-1) (mul (rec l-1) (rec l-1))
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1)))
  (label 2907)
  (parent 2707)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 1) (7 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 2908)
  (parent 2709)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2909)
  (parent 2710)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 2910)
  (parent 2710)
  (unrealized (0 3) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 2911)
  (parent 2710)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 2912)
  (parent 2710)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 2913)
  (parent 2710)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 2914)
  (parent 2710)
  (unrealized (0 3) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (8 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 1) (0 0))
    ((9 1) (5 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation nonce-test (displaced 2 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2915)
  (parent 2710)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (8 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 1) (3 0))
    ((9 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self-0))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2916)
  (parent 2710)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 10 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2917)
  (parent 2710)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2918)
  (parent 2711)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2919)
  (parent 2711)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2920)
  (parent 2711)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 2921)
  (parent 2711)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2922)
  (parent 2712)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2923)
  (parent 2712)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2924)
  (parent 2712)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 2925)
  (parent 2712)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1)))
  (label 2926)
  (parent 2713)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2927)
  (parent 2716)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2928)
  (parent 2716)
  (unrealized (10 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2929)
  (parent 2719)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2930)
  (parent 2719)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
      (send (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2931)
  (parent 2719)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 2932)
  (parent 2719)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2933)
  (parent 2720)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2934)
  (parent 2720)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
      (send (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2935)
  (parent 2720)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 2936)
  (parent 2720)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-listener l-1) (rec l-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1)))
  (label 2937)
  (parent 2721)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (10 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (added-listener l-1) (mul (rec l-1) (rec l-1))
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1)))
  (label 2938)
  (parent 2725)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1) (trans 6 0)
    (trans 4 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 2939)
  (parent 2728)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)) ((3 1) (0 3))
    ((4 1) (0 0)) ((4 2) (3 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2940)
  (parent 2729)
  (seen 2727)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1) (trans 6 0)
    (trans 4 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 2941)
  (parent 2731)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (0 3)) ((4 1) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2942)
  (parent 2732)
  (seen 2730)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 0)) ((3 2) (0 3)))
  (uniq-orig na lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 self))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2943)
  (parent 2733)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 0))
    ((3 2) (4 0)) ((4 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 self))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2944)
  (parent 2734)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 2) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (5 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1) (trans 3 1)
    (trans 6 0) (trans 3 0) (neq a self))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 2945)
  (parent 2741)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (8 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (3 0) (6 0) (8 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 4 1) (trans 7 0)
    (trans 4 0) (neq a self))
  (operation nonce-test (displaced 9 5 resp 4) (exp (gen) y-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) lb)) (send (cat (gen) lb)))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2946)
  (parent 2742)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (8 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 2) (8 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (3 0) (6 0) (8 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 4 1) (trans 7 0) (trans 4 0) (neq a self))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-5 (pv a-0 la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0)))))
  (label 2947)
  (parent 2742)
  (unrealized (0 3) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (8 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (6 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 4 1) (trans 7 0) (trans 4 0) (neq a self))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 2948)
  (parent 2742)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (la l y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la la) (x x) (beta l) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha la) (zeta (one)))
  (deflistener (cat (gen) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (8 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb la l)
  (uniq-gen y x)
  (absent (y (one)) (y la) (y l) (x la) (x l))
  (precur (3 0) (6 0) (8 0))
  (gen-st (pv a la) (pv self l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 4 1) (trans 7 0) (trans 4 0) (neq a self))
  (operation nonce-test (displaced 9 0 init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul la l)) (exp (gen) y))))))
    ((recv (cat (gen) la)) (send (cat (gen) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-4 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 2949)
  (parent 2742)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((1 2) (8 0))
    ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0))
    ((5 3) (8 0)) ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (6 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 4 1) (trans 7 0) (trans 4 0) (neq a self))
  (operation nonce-test (displaced 9 1 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 2950)
  (parent 2742)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (8 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 3) (8 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta))
  (precur (3 0) (6 0) (8 0))
  (gen-st (pv a l-0) (pv self l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (trans 7 1) (trans 4 1)
    (trans 7 0) (trans 4 0) (neq a self))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-5 (pv b lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 2951)
  (parent 2742)
  (unrealized (0 3) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (7 0)) ((5 1) (4 1)) ((6 2) (5 0))
    ((7 1) (0 3)) ((8 1) (0 0)) ((8 1) (6 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (2 0) (5 0) (7 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 1 1) (trans 8 0) (trans 1 0) (trans 6 1)
    (trans 3 1) (trans 6 0) (trans 3 0) (neq self-0 self))
  (operation nonce-test (displaced 2 9 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0)))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2952)
  (parent 2742)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (8 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0))
  (precur (3 0) (6 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 4 1) (trans 7 0) (trans 4 0)
    (neq a self))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2953)
  (parent 2742)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (0 0))
    ((4 2) (3 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (3 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 self))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2954)
  (parent 2743)
  (seen 2733)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (2 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 3) (6 0)) ((4 1) (3 1)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0) (6 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 5 1) (trans 1 1)
    (trans 5 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 self))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2955)
  (parent 2744)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((1 1) (8 0))
    ((2 1) (0 1)) ((3 2) (2 0)) ((4 3) (7 0)) ((5 1) (4 1))
    ((6 1) (0 0)) ((6 2) (5 0)) ((7 1) (0 3)) ((8 2) (7 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (2 0) (5 0) (7 0))
  (gen-st (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 1 1) (trans 6 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 self))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 2956)
  (parent 2745)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (7 0)) ((5 1) (4 1)) ((6 1) (0 0))
    ((6 1) (8 0)) ((6 2) (5 0)) ((7 1) (0 3)) ((8 2) (7 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (2 0) (5 0) (7 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 1 1) (trans 6 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 self))
  (operation channel-test (displaced 9 6 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self-0 b (exp (gen) x-0)))))
  (label 2957)
  (parent 2745)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (self self-0 a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (7 0)) ((5 1) (4 1)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)) ((8 2) (7 0)) ((9 1) (8 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (2 0) (5 0) (7 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 6 1) (trans 1 1) (trans 9 0) (trans 6 0)
    (trans 1 0) (trans 3 1) (trans 3 0) (neq self-0 self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))))
  (label 2958)
  (parent 2745)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 locn) (l rndx) (alpha zeta expt)
    (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((1 1) (8 0))
    ((2 1) (0 1)) ((3 2) (2 0)) ((4 3) (7 0)) ((5 1) (4 1))
    ((6 1) (0 0)) ((6 2) (5 0)) ((7 1) (0 3)) ((8 3) (7 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (2 0) (5 0) (7 0))
  (gen-st (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 1 1) (trans 6 0) (trans 1 0) (trans 3 1) (trans 3 0)
    (neq self-0 self))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 2959)
  (parent 2749)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 locn) (l rndx) (alpha zeta expt)
    (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (7 0)) ((5 1) (4 1)) ((6 1) (0 0))
    ((6 1) (8 0)) ((6 2) (5 0)) ((7 1) (0 3)) ((8 3) (7 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (2 0) (5 0) (7 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 1 1) (trans 6 0) (trans 1 0) (trans 3 1) (trans 3 0)
    (neq self-0 self))
  (operation channel-test (displaced 9 6 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self-0
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 2960)
  (parent 2749)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (self self-0 a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (7 0)) ((5 1) (4 1)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)) ((8 3) (7 0)) ((9 1) (8 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-1))
  (precur (2 0) (5 0) (7 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 6 1) (trans 1 1) (trans 9 0) (trans 6 0) (trans 1 0)
    (trans 3 1) (trans 3 0) (neq self-0 self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (recv (cat na self-0 self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1)))))
  (label 2961)
  (parent 2749)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 5 1) (trans 1 1)
    (trans 9 0) (trans 5 0) (trans 1 0) (trans 7 1) (trans 3 1)
    (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 2 10 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2962)
  (parent 2752)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2963)
  (parent 2752)
  (unrealized (0 3) (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 2964)
  (parent 2753)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2965)
  (parent 2753)
  (unrealized (0 3) (11 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 5 1) (trans 1 1)
    (trans 9 0) (trans 5 0) (trans 1 0) (trans 8 1) (trans 3 1)
    (trans 8 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 2 10 ltx-gen 3) (exp (gen) l-1)
    (8 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2966)
  (parent 2754)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 4 1)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2967)
  (parent 2754)
  (unrealized (0 3) (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec l) (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 4 1)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec l) (rec w))) w))
    (exp (gen) (rec l)) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec l) (rec w))) w))
      (send (cat (exp (gen) (mul (rec l) (rec w))) w))))
  (label 2968)
  (parent 2755)
  (unrealized (0 3) (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0)))
  (label 2969)
  (parent 2756)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 2970)
  (parent 2756)
  (unrealized (0 3) (11 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (deflistener (cat (exp (gen) (mul (rec l) (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec l) (rec w))) w))
    (exp (gen) (rec l)) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((recv (cat (exp (gen) (mul (rec l) (rec w))) w))
      (send (cat (exp (gen) (mul (rec l) (rec w))) w))))
  (label 2971)
  (parent 2757)
  (unrealized (0 3) (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (0 0)) ((5 2) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 5 1) (trans 1 1)
    (trans 5 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2972)
  (parent 2758)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 1) (9 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2973)
  (parent 2759)
  (unrealized (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2974)
  (parent 2759)
  (unrealized (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (9 0)))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 10 1) (trans 2 1)
    (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 2975)
  (parent 2759)
  (unrealized (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 9 1) (trans 4 1) (trans 9 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (8 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 2976)
  (parent 2761)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon alpha zeta expt)
    (l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 1) (9 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 1) (7 0)) ((9 3) (8 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 2977)
  (parent 2763)
  (unrealized (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon expt) (l rndx)
    (alpha zeta expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 1) (7 0)) ((9 3) (8 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) y) (mul (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec y) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 2978)
  (parent 2763)
  (unrealized (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (alpha zeta expt) (l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 3) (8 0)) ((10 1) (9 0)))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-1))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l) (pv b-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 2979)
  (parent 2763)
  (unrealized (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 1) (0 0)) ((7 2) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 7 1) (trans 1 1)
    (trans 7 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2980)
  (parent 2764)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (4 0)) ((1 1) (3 0)) ((1 2) (5 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 2) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 2) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 3 1) (trans 6 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 2981)
  (parent 2766)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 1) (10 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2982)
  (parent 2767)
  (unrealized (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2983)
  (parent 2767)
  (unrealized (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0))
    ((11 1) (10 0)))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv a-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 11 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 4 1) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-5 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv a-0 l-1)))))
  (label 2984)
  (parent 2767)
  (unrealized (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (upsilon expt) (la l x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) l) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (deflistener x)
  (precedes ((0 2) (10 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)))
  (uniq-orig na la l)
  (uniq-gen x)
  (absent (x la) (x l))
  (precur (3 0) (9 0))
  (gen-st (pv a la) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (added-listener x) (mul l (rec x)) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la l))
              (exp (gen) (mul upsilon x)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) l) la)) (send (cat (exp (gen) l) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-4 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))) ((recv x) (send x)))
  (label 2985)
  (parent 2768)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (displaced 10 4 ltx-disclose 3) l-0 (9 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2986)
  (parent 2769)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 2) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2987)
  (parent 2770)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon expt) (l rndx)
    (alpha zeta expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 1) (10 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 4 1) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec y) l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 2988)
  (parent 2771)
  (unrealized (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon alpha zeta expt)
    (l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 4 1) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 2989)
  (parent 2771)
  (unrealized (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (alpha zeta expt) (l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0))
    ((11 1) (10 0)))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 11 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0) (trans 4 0)
    (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-5 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv b-0 l-1)))))
  (label 2990)
  (parent 2771)
  (unrealized (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 1) (8 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (7 0))
    ((5 1) (4 1)) ((6 2) (5 0)) ((7 1) (0 3)) ((8 2) (7 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a self))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 2991)
  (parent 2775)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor) (la l-0)
    (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (7 0))
    ((5 1) (4 1)) ((6 2) (5 0)) ((7 1) (0 3)) ((8 2) (7 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 6 0) (neq a self))
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 2992)
  (parent 2775)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (7 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (0 3)) ((8 2) (7 0)) ((9 1) (8 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 6 0) (neq a self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 2993)
  (parent 2775)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 1) (8 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (7 0))
    ((5 1) (4 1)) ((6 2) (5 0)) ((7 1) (0 3)) ((8 3) (7 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a self))
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 2994)
  (parent 2779)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (l rndx)
    (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (7 0))
    ((5 1) (4 1)) ((6 2) (5 0)) ((7 1) (0 3)) ((8 3) (7 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (trans 6 1) (trans 6 0)
    (neq a self))
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 2995)
  (parent 2779)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (7 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (0 3)) ((8 3) (7 0)) ((9 1) (8 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-1))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv a l-0) (pv self l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0)
    (trans 6 1) (trans 6 0) (neq a self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1)))))
  (label 2996)
  (parent 2779)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (2 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 9 1)
    (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2997)
  (parent 2788)
  (seen 2671)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 4 1) (trans 1 1)
    (trans 9 0) (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2998)
  (parent 2788)
  (unrealized (10 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2999)
  (parent 2789)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3000)
  (parent 2789)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 3001)
  (parent 2789)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (10 0)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) l-0 l-1)) (y l) (x l-0) (x l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 3002)
  (parent 2789)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 4 1) (trans 1 1)
    (trans 10 0) (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (displaced 2 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3003)
  (parent 2789)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((1 2) (10 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 3004)
  (parent 2789)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (4 0))
    ((10 1) (6 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 2 1) (trans 1 1)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (displaced 5 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3005)
  (parent 2789)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3006)
  (parent 2789)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 10 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 3007)
  (parent 2790)
  (seen 2681)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3008)
  (parent 2790)
  (unrealized (10 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3009)
  (parent 2791)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3010)
  (parent 2791)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (4 2)) ((1 1) (10 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 3011)
  (parent 2791)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (10 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 3012)
  (parent 2791)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (2 0)) ((2 1) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 0))
    ((10 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0))
  (gen-st (pv b l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 4 1) (trans 1 1)
    (trans 10 0) (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (displaced 2 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3013)
  (parent 2791)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 3014)
  (parent 2791)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (4 0))
    ((10 1) (6 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 2 1) (trans 1 1)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (displaced 5 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3015)
  (parent 2791)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3016)
  (parent 2791)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (2 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 9 1)
    (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3017)
  (parent 2792)
  (seen 2691)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 4 1) (trans 1 1)
    (trans 9 0) (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3018)
  (parent 2792)
  (unrealized (10 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3019)
  (parent 2793)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3020)
  (parent 2793)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 3021)
  (parent 2793)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (10 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) l-0 l-1)) (y l) (x l-0) (x l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 3022)
  (parent 2793)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 4 1) (trans 1 1)
    (trans 10 0) (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (displaced 2 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3023)
  (parent 2793)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((1 2) (10 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 3024)
  (parent 2793)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (4 0))
    ((10 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 2 1) (trans 1 1)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (displaced 5 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3025)
  (parent 2793)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3026)
  (parent 2793)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3027)
  (parent 2794)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3028)
  (parent 2794)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w (rec l-1))) (gen)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1)))))
  (label 3029)
  (parent 2794)
  (unrealized (0 3) (8 0) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul (rec l) (rec x))))
  (precedes ((0 2) (10 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) la))
      (send (cat (exp (gen) (rec l)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul (rec l) (rec x))))
      (send (cat (exp (gen) x) (mul (rec l) (rec x))))))
  (label 3030)
  (parent 2794)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 0))
    ((10 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 4 1) (trans 1 1)
    (trans 10 0) (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (displaced 2 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3031)
  (parent 2794)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((1 2) (10 0))
    ((2 1) (0 0)) ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3))
    ((5 1) (4 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))))
  (label 3032)
  (parent 2794)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 2 1) (trans 1 1)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (displaced 5 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3033)
  (parent 2794)
  (unrealized (0 3) (7 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3034)
  (parent 2794)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 10 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 3035)
  (parent 2795)
  (seen 2709)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3036)
  (parent 2795)
  (unrealized (10 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3037)
  (parent 2796)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3038)
  (parent 2796)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (4 2)) ((1 1) (10 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 3039)
  (parent 2796)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (10 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 3040)
  (parent 2796)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (2 0)) ((2 1) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 0))
    ((10 1) (6 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0))
  (gen-st (pv b l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 4 1) (trans 1 1)
    (trans 10 0) (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (displaced 2 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3041)
  (parent 2796)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 3042)
  (parent 2796)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (4 0))
    ((10 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 2 1) (trans 1 1)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (displaced 5 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3043)
  (parent 2796)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3044)
  (parent 2796)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3045)
  (parent 2797)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3046)
  (parent 2797)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w (rec l-1))) (gen)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1)))))
  (label 3047)
  (parent 2797)
  (unrealized (0 3) (8 0) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul (rec l) (rec x))))
  (precedes ((0 2) (10 0)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul (rec l) (rec x))))
      (send (cat (exp (gen) x) (mul (rec l) (rec x))))))
  (label 3048)
  (parent 2797)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 0))
    ((10 1) (6 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 4 1) (trans 1 1)
    (trans 10 0) (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (displaced 2 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3049)
  (parent 2797)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((1 2) (10 0))
    ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3))
    ((5 1) (4 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))))
  (label 3050)
  (parent 2797)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 2 1) (trans 1 1)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (displaced 5 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3051)
  (parent 2797)
  (unrealized (0 3) (7 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3052)
  (parent 2797)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (5 0))
    ((7 2) (6 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 4 1) (trans 7 0) (trans 4 0)
    (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l)) lb))
      (send (cat (exp (gen) (mul l l)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 3053)
  (parent 2812)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (9 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (trans 8 1) (trans 5 1) (trans 8 0)
    (trans 5 0) (neq self b))
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l)) lb))
      (send (cat (exp (gen) (mul l l)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3054)
  (parent 2813)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (x-0 la)
    (x-0 beta))
  (precur (6 0) (9 0))
  (gen-st (pv b l) (pv self l-0) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (trans 8 1)
    (trans 5 1) (trans 8 0) (trans 5 0) (neq self b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-5 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3055)
  (parent 2813)
  (unrealized (0 3) (9 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0) (9 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (trans 8 1)
    (trans 5 1) (trans 8 0) (trans 5 0) (neq self b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3056)
  (parent 2813)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l la y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta la) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la la))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul la la)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul la la)))
  (deflistener la)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l la)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la la)) (y l) (x la))
  (precur (6 0) (9 0))
  (gen-st (pv b l) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (trans 8 1)
    (trans 5 1) (trans 8 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self la)))
      (recv
        (sig (body b (exp (gen) la) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la la)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) la la)) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la la)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul la la)) l))
      (send (cat (exp (gen) (mul la la)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul la la))) (send (cat (gen) (mul la la))))
    ((recv la) (send la))
    ((load priv-stor (cat pt-0 (pv self la)))
      (stor priv-stor (cat pt-4 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3057)
  (parent 2813)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0) (9 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (trans 8 1)
    (trans 5 1) (trans 8 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 3058)
  (parent 2813)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (y-0 lb)
    (y-0 alpha) (y-0 zeta))
  (precur (6 0) (9 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (trans 8 1) (trans 5 1)
    (trans 8 0) (trans 5 0) (neq self b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-5 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3059)
  (parent 2813)
  (unrealized (0 3) (9 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((2 3) (8 0)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 1) (2 0))
    ((9 1) (4 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0))
  (precur (5 0) (8 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 1 1) (trans 9 0) (trans 1 0) (trans 7 1)
    (trans 4 1) (trans 7 0) (trans 4 0) (neq self self-0))
  (operation nonce-test (displaced 3 10 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0)))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3060)
  (parent 2813)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0) (9 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 3 1) (trans 1 1) (trans 10 0) (trans 3 0)
    (trans 1 0) (trans 8 1) (trans 5 1) (trans 8 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3061)
  (parent 2813)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (3 0)) ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 5 1) (trans 3 1) (trans 5 0) (trans 3 0)
    (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 3062)
  (parent 2814)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((1 2) (5 0)) ((2 1) (4 0)) ((3 1) (7 0)) ((4 2) (3 0))
    ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (7 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1) (trans 6 0)
    (trans 4 0) (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l)) lb))
      (send (cat (exp (gen) (mul l l)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3063)
  (parent 2815)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 1) (9 0))
    ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (8 0)) ((3 1) (2 0))
    ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (0 3)) ((9 2) (8 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (x-0 beta)
    (x-0 l-0))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3064)
  (parent 2816)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (6 0)) ((2 3) (8 0)) ((3 1) (2 0)) ((3 1) (5 0))
    ((3 1) (9 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (0 3)) ((9 2) (8 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (x-0 beta)
    (x-0 l-0))
  (precur (6 0) (8 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation channel-test (displaced 10 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3065)
  (parent 2816)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (6 0)) ((2 3) (8 0)) ((3 1) (2 0)) ((3 1) (5 0))
    ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 2) (8 0)) ((10 1) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (x-0 beta)
    (x-0 l-1))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 3 1) (trans 1 1) (trans 10 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-5 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv a l-1)))))
  (label 3066)
  (parent 2816)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 1) (9 0))
    ((1 2) (0 1)) ((1 2) (6 0)) ((2 3) (8 0)) ((3 1) (2 0))
    ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (0 3)) ((9 3) (8 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (y-0 alpha)
    (y-0 zeta) (y-0 l-0))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 3067)
  (parent 2820)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (6 0)) ((2 3) (8 0)) ((3 1) (2 0)) ((3 1) (5 0))
    ((3 1) (9 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (0 3)) ((9 3) (8 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (y-0 alpha)
    (y-0 zeta) (y-0 l-0))
  (precur (6 0) (8 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self b))
  (operation channel-test (displaced 10 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 3068)
  (parent 2820)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (6 0)) ((2 3) (8 0)) ((3 1) (2 0)) ((3 1) (5 0))
    ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0))
    ((8 1) (0 3)) ((9 3) (8 0)) ((10 1) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (y-0 alpha)
    (y-0 zeta) (y-0 l-1))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 3 1) (trans 1 1) (trans 10 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-5 (pv b-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv b-0 l-1)))))
  (label 3069)
  (parent 2820)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (rec lb)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (7 0))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (5 0))
    ((7 2) (6 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 4 1) (trans 7 0) (trans 4 0)
    (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (rec lb)) l))
      (send (cat (exp (gen) (rec lb)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb)))
  (label 3070)
  (parent 2823)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (rec lb)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (8 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (9 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (trans 8 1) (trans 5 1) (trans 8 0)
    (trans 5 0) (neq self b))
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (rec lb)) l))
      (send (cat (exp (gen) (rec lb)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3071)
  (parent 2824)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (8 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (x-0 la)
    (x-0 beta))
  (precur (6 0) (9 0))
  (gen-st (pv b l-0) (pv self l) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (trans 8 1)
    (trans 5 1) (trans 8 0) (trans 5 0) (neq self b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-5 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3072)
  (parent 2824)
  (unrealized (0 3) (9 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (8 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0))
  (precur (6 0) (9 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (trans 8 1)
    (trans 5 1) (trans 8 0) (trans 5 0) (neq self b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3073)
  (parent 2824)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l la y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta la) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la la))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) la))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l la))
  (deflistener (cat (gen) (rec l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (8 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l la)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la la)) (y l) (x la))
  (precur (6 0) (9 0))
  (gen-st (pv b l) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (trans 8 1)
    (trans 5 1) (trans 8 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self la)))
      (recv
        (sig (body b (exp (gen) la) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la la)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) la la)) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la la)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) la))
      (send (cat (exp (gen) (rec l)) la)))
    ((load priv-stor (cat pt-0 (pv self la)))
      (stor priv-stor (cat pt-3 "nil")) (send la))
    ((recv (cat (gen) (rec l))) (send (cat (gen) (rec l))))
    ((recv l) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3074)
  (parent 2824)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (8 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0) (9 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (trans 8 1)
    (trans 5 1) (trans 8 0) (trans 5 0) (neq self b))
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (rec l))) (send (cat (gen) (rec l))))
    ((recv l) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 3075)
  (parent 2824)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (8 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (y-0 lb)
    (y-0 alpha) (y-0 zeta))
  (precur (6 0) (9 0))
  (gen-st (pv b l-0) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (trans 8 1) (trans 5 1)
    (trans 8 0) (trans 5 0) (neq self b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-5 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3076)
  (parent 2824)
  (unrealized (0 3) (9 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (8 0)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 1) (2 0))
    ((9 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0))
  (precur (5 0) (8 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 1 1) (trans 9 0) (trans 1 0) (trans 7 1)
    (trans 4 1) (trans 7 0) (trans 4 0) (neq self self-0))
  (operation nonce-test (displaced 3 10 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0)))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3077)
  (parent 2824)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (8 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0))
  (precur (6 0) (9 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 3 1) (trans 1 1) (trans 10 0) (trans 3 0)
    (trans 1 0) (trans 8 1) (trans 5 1) (trans 8 0) (trans 5 0)
    (neq self b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3078)
  (parent 2824)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) (rec lb)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 2) (2 0)) ((4 1) (2 0)) ((5 1) (7 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (4 0))
  (gen-st (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (trans 5 1) (trans 1 1) (trans 5 0)
    (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0)
    (neq self self-0))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec lb)) l))
      (send (cat (exp (gen) (rec lb)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb)))
  (label 3079)
  (parent 2825)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta (mul x w)))
  (deflistener (cat (exp (gen) (rec lb)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (2 0)) ((6 1) (8 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (9 0))
  (gen-st (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 6 1) (trans 1 1)
    (trans 6 0) (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0)
    (trans 4 0) (neq self self-0))
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (rec lb)) l))
      (send (cat (exp (gen) (rec lb)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3080)
  (parent 2826)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (self self-0 a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (2 0)) ((6 1) (8 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (x-0 la)
    (x-0 beta))
  (precur (5 0) (9 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 1 1) (trans 6 0) (trans 1 0) (trans 8 1)
    (trans 4 1) (trans 8 0) (trans 4 0) (neq self self-0))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-5 (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 3081)
  (parent 2826)
  (unrealized (0 3) (9 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (2 0)) ((6 1) (8 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0))
  (precur (5 0) (9 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 1 1) (trans 6 0) (trans 1 0) (trans 8 1)
    (trans 4 1) (trans 8 0) (trans 4 0) (neq self self-0))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3082)
  (parent 2826)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l la y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la la) (x x) (beta la)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la la))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l)) la))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l) (mul (rec l) (rec l))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (2 0)) ((6 1) (8 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l la)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la la)) (y l) (x la))
  (precur (5 0) (9 0))
  (gen-st (pv self la) (pv self-0 l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 1 1) (trans 6 0) (trans 1 0) (trans 8 1)
    (trans 4 1) (trans 8 0) (trans 4 0) (neq self self-0))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self la)))
      (recv
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self self-0
            (hash (exp (gen) (mul la la)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) la la)) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul la la)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l)) la))
      (send (cat (exp (gen) (rec l)) la)))
    ((load priv-stor (cat pt-0 (pv self la)))
      (stor priv-stor (cat pt-1 "nil")) (send la))
    ((recv (cat (exp (gen) l) (mul (rec l) (rec l))))
      (send (cat (exp (gen) l) (mul (rec l) (rec l)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l) (send l))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3083)
  (parent 2826)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) (rec l))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (2 0)) ((6 1) (8 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (5 0) (9 0))
  (gen-st (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 1 1) (trans 6 0) (trans 1 0) (trans 8 1)
    (trans 4 1) (trans 8 0) (trans 4 0) (neq self self-0))
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (mul (rec l) (rec l))))
      (send (cat (exp (gen) l) (mul (rec l) (rec l)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l) (send l))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 3084)
  (parent 2826)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (self self-0 a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (2 0)) ((6 1) (8 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (y-0 lb)
    (y-0 alpha) (y-0 zeta))
  (precur (5 0) (9 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 1 1) (trans 6 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq self self-0))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-5 (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3085)
  (parent 2826)
  (unrealized (0 3) (9 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (2 0)) ((6 1) (8 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0))
  (precur (5 0) (9 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 1 1) (trans 6 0) (trans 1 0) (trans 8 1)
    (trans 4 1) (trans 8 0) (trans 4 0) (neq self self-0))
  (operation nonce-test (displaced 10 6 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 3086)
  (parent 2826)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (2 0)) ((6 1) (8 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0))
  (precur (5 0) (9 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 6 1) (trans 1 1) (trans 10 0) (trans 6 0)
    (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0) (trans 4 0)
    (neq self self-0))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3087)
  (parent 2826)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (2 0) (7 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (trans 9 1) (trans 6 1)
    (trans 9 0) (trans 6 0) (neq self b))
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l)) lb))
      (send (cat (exp (gen) (mul l l)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 3088)
  (parent 2828)
  (seen 2812)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 9 1) (trans 6 1) (trans 9 0) (trans 6 0)
    (neq self b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3089)
  (parent 2828)
  (unrealized (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((1 2) (7 0)) ((2 1) (0 1)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (2 0) (7 0) (9 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq self b))
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l)) lb))
      (send (cat (exp (gen) (mul l l)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3090)
  (parent 2830)
  (seen 2815)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((1 2) (7 0)) ((2 1) (0 1)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (x-0 la)
    (x-0 beta))
  (precur (2 0) (7 0) (9 0))
  (gen-st (pv b l) (pv self l-0) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 8 0) (trans 6 0) (neq self b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-5 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3091)
  (parent 2830)
  (unrealized (0 3) (9 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((1 2) (7 0)) ((2 1) (0 1)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0) (9 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 8 0) (trans 6 0) (neq self b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3092)
  (parent 2830)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l la y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta la) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la la))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul la la)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) la) la))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((1 2) (7 0)) ((2 1) (0 1)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l la)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la la)) (y l) (x la))
  (precur (2 0) (7 0) (9 0))
  (gen-st (pv b l) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 8 0) (trans 6 0) (neq self b))
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self la)))
      (recv
        (sig (body b (exp (gen) la) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la la)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) la la)) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la la)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul la la)) l))
      (send (cat (exp (gen) (mul la la)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) la) la)) (send (cat (exp (gen) la) la)))
    ((load priv-stor (cat pt-0 (pv self la)))
      (stor priv-stor (cat pt-4 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3093)
  (parent 2830)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((1 2) (7 0)) ((2 1) (0 1)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0) (9 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 8 0) (trans 6 0) (neq self b))
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 3094)
  (parent 2830)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((1 2) (7 0)) ((2 1) (0 1)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (y-0 lb)
    (y-0 alpha) (y-0 zeta))
  (precur (2 0) (7 0) (9 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq self b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-5 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3095)
  (parent 2830)
  (unrealized (0 3) (9 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (2 0))
    ((1 2) (6 0)) ((2 1) (0 1)) ((3 3) (8 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3))
    ((9 1) (3 0)) ((9 1) (5 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (6 0) (8 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 1 1) (trans 9 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq self self-0))
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0)))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3096)
  (parent 2830)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((1 2) (7 0)) ((2 1) (0 1)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0) (9 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3097)
  (parent 2830)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (rec lb)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (9 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (2 0) (7 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (trans 9 1) (trans 6 1)
    (trans 9 0) (trans 6 0) (neq self b))
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (rec lb)) l))
      (send (cat (exp (gen) (rec lb)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb)))
  (label 3098)
  (parent 2831)
  (seen 2823)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (9 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 9 1) (trans 6 1) (trans 9 0) (trans 6 0)
    (neq self b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3099)
  (parent 2831)
  (unrealized (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta x))
  (deflistener (cat (exp (gen) (rec lb)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l lb))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((7 1) (9 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (2 0) (6 0))
  (gen-st (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 7 1)
    (trans 1 1) (trans 7 0) (trans 1 0) (trans 9 1) (trans 5 1)
    (trans 9 0) (trans 5 0) (neq self self-0))
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (rec lb)) l))
      (send (cat (exp (gen) (rec lb)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb)))
  (label 3100)
  (parent 2832)
  (seen 2825)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((7 1) (9 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (6 0) (10 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 7 1) (trans 1 1) (trans 7 0)
    (trans 1 0) (trans 9 1) (trans 5 1) (trans 9 0) (trans 5 0)
    (neq self self-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3101)
  (parent 2832)
  (unrealized (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0)) ((3 2) (0 3)))
  (uniq-orig na lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 3 1) (trans 3 0) (neq a self-0))
  (operation generalization separated self-0)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 3102)
  (parent 2833)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta lb) (a a) (b self-0) (x x) (upsilon y) (na na)
        (nb nb) (priv-stor priv-stor) (pt pt) (self self)
        (priv-stor-0 priv-stor-0) (ignore ignore) (pt-0 pt-0)
        (pt-1 pt-1))))
  (origs (pt-3 (3 1)) (lb (1 1)) (pt-1 (1 1)) (l (2 1)) (pt (2 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (0 3)) ((3 2) (4 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a self))
  (operation generalization deleted (4 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 3103)
  (parent 2834)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (3 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (0 0)) ((6 2) (4 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1)
    (trans 5 0) (trans 3 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3104)
  (parent 2835)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 1) (8 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (0 0)) ((7 2) (5 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 7 1) (trans 2 1)
    (trans 1 1) (trans 7 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq self-0 b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3105)
  (parent 2836)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((3 1) (10 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv b l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation channel-test (displaced 11 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3106)
  (parent 2837)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3107)
  (parent 2837)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (10 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation channel-test (displaced 11 8 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self-0 b-0 (exp (gen) x-0)))))
  (label 3108)
  (parent 2837)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1) (pv a l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 8 1) (trans 3 1) (trans 1 1) (trans 11 0)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a l-2)))))
  (label 3109)
  (parent 2837)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((3 1) (10 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv b l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation channel-test (displaced 11 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3110)
  (parent 2841)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3111)
  (parent 2841)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (10 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation channel-test (displaced 11 8 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3112)
  (parent 2841)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 3) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 11 0) (trans 8 0)
    (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0)
    (trans 5 0) (neq self-0 b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3113)
  (parent 2841)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 3114)
  (parent 2853)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3115)
  (parent 2853)
  (unrealized (11 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (4 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 8 1) (trans 2 1) (trans 1 1)
    (trans 8 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq self-0 b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3116)
  (parent 2854)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 9 1) (trans 3 1)
    (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 11 2 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3117)
  (parent 2855)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3118)
  (parent 2855)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3119)
  (parent 2855)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) la) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self-0 b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) la l-0))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) la) l-0)) (send (cat (exp (gen) la) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 la)))
      (send
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3120)
  (parent 2855)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3121)
  (parent 2855)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3122)
  (parent 2855)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-1) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)) ((9 1) (0 3))
    ((10 1) (2 0)) ((10 1) (4 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv self-1 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 8 1) (trans 1 1) (trans 10 0) (trans 8 0)
    (trans 1 0) (trans 6 1) (trans 4 1) (trans 6 0) (trans 4 0)
    (neq self-0 self-1))
  (operation nonce-test (displaced 3 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig (body self-1 (exp (gen) l) (pubk "sig" self-1))
          (privk "sig" self-1)))
      (send (cat na self-0 self-1 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 self-1 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3123)
  (parent 2855)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation nonce-test (displaced 11 9 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3124)
  (parent 2855)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 9 1) (trans 3 1) (trans 1 1) (trans 11 0)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-1 l-2)))
      (send
        (sig (body self-1 (exp (gen) l-2) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3125)
  (parent 2855)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((1 2) (5 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (4 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 3126)
  (parent 2856)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (5 0)) ((4 1) (8 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3127)
  (parent 2857)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((2 1) (10 0)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3128)
  (parent 2858)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((1 2) (7 0))
    ((2 1) (0 0)) ((2 1) (8 0)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3129)
  (parent 2858)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3130)
  (parent 2858)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 3131)
  (parent 2858)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((2 1) (10 0)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3132)
  (parent 2862)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((1 2) (7 0))
    ((2 1) (0 0)) ((2 1) (8 0)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3133)
  (parent 2862)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3134)
  (parent 2862)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 3) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3135)
  (parent 2862)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 3136)
  (parent 2874)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3137)
  (parent 2874)
  (unrealized (11 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 2) (0 1)) ((1 2) (8 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (5 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 3138)
  (parent 2875)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3139)
  (parent 2876)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3140)
  (parent 2876)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3141)
  (parent 2876)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3142)
  (parent 2876)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3143)
  (parent 2876)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3144)
  (parent 2876)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3))
    ((10 1) (0 0)) ((10 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (9 0))
  (gen-st (pv b l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 3 1) (trans 1 1) (trans 10 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation nonce-test (displaced 2 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3145)
  (parent 2876)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3))
    ((10 1) (3 0)) ((10 1) (5 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self-0))
  (operation nonce-test (displaced 4 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3146)
  (parent 2876)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3147)
  (parent 2876)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (0 0)) ((6 2) (4 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1)
    (trans 5 0) (trans 3 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3148)
  (parent 2877)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 1) (8 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (0 0)) ((7 2) (5 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 7 1) (trans 2 1)
    (trans 1 1) (trans 7 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq self-0 b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3149)
  (parent 2878)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((3 1) (10 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv b l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation channel-test (displaced 11 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3150)
  (parent 2879)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3151)
  (parent 2879)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (10 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation channel-test (displaced 11 8 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self-0 b-0 (exp (gen) x-0)))))
  (label 3152)
  (parent 2879)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1) (pv a l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 8 1) (trans 3 1) (trans 1 1) (trans 11 0)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a l-2)))))
  (label 3153)
  (parent 2879)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((3 1) (10 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv b l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation channel-test (displaced 11 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3154)
  (parent 2883)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3155)
  (parent 2883)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 1) (10 0))
    ((8 2) (6 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 8 1)
    (trans 3 1) (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation channel-test (displaced 11 8 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3156)
  (parent 2883)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)) ((10 3) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 8 1) (trans 3 1) (trans 1 1) (trans 11 0) (trans 8 0)
    (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0)
    (trans 5 0) (neq self-0 b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3157)
  (parent 2883)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 3158)
  (parent 2895)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3159)
  (parent 2895)
  (unrealized (11 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 8 1) (trans 2 1) (trans 1 1)
    (trans 8 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq self-0 b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3160)
  (parent 2896)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 9 1) (trans 3 1)
    (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 11 2 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3161)
  (parent 2897)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (8 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3162)
  (parent 2897)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3163)
  (parent 2897)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) la) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self-0 b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) la l-0))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) la) l)) (send (cat (exp (gen) la) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 la)))
      (send
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3164)
  (parent 2897)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3165)
  (parent 2897)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (8 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3166)
  (parent 2897)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-1) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)) ((9 1) (0 3))
    ((10 1) (2 0)) ((10 1) (6 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (7 0) (9 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv self-1 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 8 1) (trans 1 1) (trans 10 0) (trans 8 0)
    (trans 1 0) (trans 6 1) (trans 4 1) (trans 6 0) (trans 4 0)
    (neq self-0 self-1))
  (operation nonce-test (displaced 3 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig (body self-1 (exp (gen) l) (pubk "sig" self-1))
          (privk "sig" self-1)))
      (send (cat na self-0 self-1 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 self-1 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (stor priv-stor-1 (cat pt-2 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3167)
  (parent 2897)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation nonce-test (displaced 11 9 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3168)
  (parent 2897)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 9 1) (trans 3 1) (trans 1 1) (trans 11 0)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-1 l-2)))
      (send
        (sig (body self-1 (exp (gen) l-2) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3169)
  (parent 2897)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1)
    (trans 6 1) (trans 11 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 3170)
  (parent 2906)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 7 1)
    (trans 5 1) (trans 11 0) (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-1 (cat pt-6 (pv self-0 l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 3171)
  (parent 2907)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 2) (0 1)) ((1 2) (5 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 3172)
  (parent 2908)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 1) (7 0)) ((4 1) (8 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3173)
  (parent 2909)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((2 1) (10 0)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3174)
  (parent 2910)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((1 2) (7 0))
    ((2 1) (0 0)) ((2 1) (6 0)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3175)
  (parent 2910)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3176)
  (parent 2910)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 3177)
  (parent 2910)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((2 1) (10 0)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3178)
  (parent 2914)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((1 2) (7 0))
    ((2 1) (0 0)) ((2 1) (6 0)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3179)
  (parent 2914)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3180)
  (parent 2914)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 3) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3181)
  (parent 2914)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 3182)
  (parent 2926)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3183)
  (parent 2926)
  (unrealized (11 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 2) (0 1)) ((1 2) (8 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 1) (7 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 3184)
  (parent 2927)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3185)
  (parent 2928)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3186)
  (parent 2928)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3187)
  (parent 2928)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3188)
  (parent 2928)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3189)
  (parent 2928)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3190)
  (parent 2928)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3))
    ((10 1) (0 0)) ((10 1) (5 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (9 0))
  (gen-st (pv b l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 3 1) (trans 1 1) (trans 10 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation nonce-test (displaced 2 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3191)
  (parent 2928)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3))
    ((10 1) (3 0)) ((10 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq a self-0))
  (operation nonce-test (displaced 4 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3192)
  (parent 2928)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3193)
  (parent 2928)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1)
    (trans 6 1) (trans 11 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 3194)
  (parent 2937)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 7 1)
    (trans 5 1) (trans 11 0) (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-1 (cat pt-6 (pv self-0 l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 3195)
  (parent 2938)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1) (trans 5 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 3196)
  (parent 2939)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1) (trans 5 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 3197)
  (parent 2941)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 0)) ((3 2) (0 3)))
  (uniq-orig na lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 self-1))
  (operation generalization separated self-1)
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self-1 (exp (gen) lb) (pubk "sig" self-1))
          (privk "sig" self-1)))
      (send (cat na self-0 self-1 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3198)
  (parent 2943)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta lb) (a self-0) (b self-1) (x x) (upsilon y) (na na)
        (nb nb) (priv-stor priv-stor-0) (pt pt-3) (self self)
        (priv-stor-0 priv-stor) (ignore ignore) (pt-0 pt) (pt-1 pt-0))))
  (origs (l (3 1)) (pt-3 (3 1)) (lb (1 1)) (pt-0 (1 1)) (pt-1 (2 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 0))
    ((3 2) (0 3)) ((3 2) (4 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 self))
  (operation generalization deleted (4 1))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 3199)
  (parent 2944)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (4 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 5 1) (trans 3 1) (trans 5 0) (trans 3 0)
    (neq a self))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 3200)
  (parent 2945)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 2) (0 1)) ((4 3) (7 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (5 0) (7 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 3 1) (trans 6 0)
    (trans 3 0) (neq a self))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3201)
  (parent 2946)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((1 1) (9 0))
    ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0))
    ((5 3) (8 0)) ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (0 3))
    ((9 2) (8 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (6 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 4 1) (trans 7 0) (trans 4 0) (neq a self))
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 3202)
  (parent 2947)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor) (la l-0)
    (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((2 1) (9 0)) ((3 1) (0 1)) ((4 2) (3 0))
    ((5 3) (8 0)) ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (0 3))
    ((9 2) (8 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (6 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 2 1) (trans 1 1) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 4 1) (trans 7 0) (trans 4 0) (neq a self))
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 3203)
  (parent 2947)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (8 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 2) (8 0))
    ((10 1) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (3 0) (6 0) (8 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 7 1) (trans 4 1) (trans 7 0) (trans 4 0)
    (neq a self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-5 (pv a-0 l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv a-0 l-1)))))
  (label 3204)
  (parent 2947)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta (one)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((1 1) (9 0))
    ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0))
    ((5 3) (8 0)) ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (0 3))
    ((9 3) (8 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (3 0) (6 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (trans 7 1) (trans 4 1)
    (trans 7 0) (trans 4 0) (neq a self))
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 3205)
  (parent 2951)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((2 1) (9 0)) ((3 1) (0 1)) ((4 2) (3 0))
    ((5 3) (8 0)) ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (0 3))
    ((9 3) (8 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-0))
  (precur (3 0) (6 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 2 1)
    (trans 1 1) (trans 2 0) (trans 1 0) (trans 7 1) (trans 4 1)
    (trans 7 0) (trans 4 0) (neq a self))
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 3206)
  (parent 2951)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta (one)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (8 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (0 3)) ((9 3) (8 0))
    ((10 1) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y l) (y l-0) (y-0 alpha) (y-0 zeta)
    (y-0 l-1))
  (precur (3 0) (6 0) (8 0))
  (gen-st (pv a l-0) (pv self l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0)
    (trans 7 1) (trans 4 1) (trans 7 0) (trans 4 0) (neq a self))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-5 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv b l-1)))))
  (label 3207)
  (parent 2951)
  (unrealized (0 3) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (5 0))
    ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (3 0) (5 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 self))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3208)
  (parent 2955)
  (seen 2944)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (2 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 9 1)
    (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0) (trans 1 0)
    (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 10 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3209)
  (parent 2962)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 5 1) (trans 1 1)
    (trans 9 0) (trans 5 0) (trans 1 0) (trans 7 1) (trans 3 1)
    (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3210)
  (parent 2962)
  (unrealized (10 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0) (trans 4 0)
    (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3211)
  (parent 2963)
  (unrealized (0 3) (9 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3212)
  (parent 2963)
  (unrealized (0 3) (9 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 3213)
  (parent 2963)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (10 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0)) ((6 1) (8 0))
    ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) l-0 l-1)) (y l) (x l-0) (x l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 3214)
  (parent 2963)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (0 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 5 1) (trans 1 1)
    (trans 10 0) (trans 5 0) (trans 1 0) (trans 7 1) (trans 3 1)
    (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 2 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3215)
  (parent 2963)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 3216)
  (parent 2963)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (0 3)) ((6 1) (5 1))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0)) ((10 1) (5 0))
    ((10 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 2 1) (trans 1 1)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 4 1)
    (trans 7 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (displaced 6 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3217)
  (parent 2963)
  (unrealized (0 3) (8 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 4 1) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3218)
  (parent 2963)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (displaced 11 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l)))
  (label 3219)
  (parent 2964)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3220)
  (parent 2964)
  (unrealized (11 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3221)
  (parent 2965)
  (unrealized (0 3) (9 0) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3222)
  (parent 2965)
  (unrealized (0 3) (9 0) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 3223)
  (parent 2965)
  (seen 3568)
  (unrealized (0 3) (11 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (11 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 12 0 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-7 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 3224)
  (parent 2965)
  (unrealized (0 3) (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0)) ((6 1) (4 1))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 1) (0 0)) ((11 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 5 1) (trans 1 1)
    (trans 11 0) (trans 5 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 3 1) (trans 9 0) (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 2 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3225)
  (parent 2965)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 12 1 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 3226)
  (parent 2965)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 1))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 1) (5 0)) ((11 1) (7 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 2 1) (trans 1 1)
    (trans 11 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 4 1) (trans 9 0) (trans 7 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (displaced 6 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3227)
  (parent 2965)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 12 1) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 12 0) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3228)
  (parent 2965)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (2 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 9 1)
    (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0) (trans 1 0)
    (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 10 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3229)
  (parent 2966)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 5 1) (trans 1 1)
    (trans 9 0) (trans 5 0) (trans 1 0) (trans 8 1) (trans 3 1)
    (trans 8 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3230)
  (parent 2966)
  (unrealized (10 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 4 1) (trans 9 0) (trans 4 0)
    (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3231)
  (parent 2967)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 4 1)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3232)
  (parent 2967)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 4 1)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 3233)
  (parent 2967)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (10 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0)) ((6 1) (9 0))
    ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) l-0 l-1)) (y l) (x l-0) (x l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 4 1)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 3234)
  (parent 2967)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 5 1) (trans 1 1)
    (trans 10 0) (trans 5 0) (trans 1 0) (trans 8 1) (trans 3 1)
    (trans 8 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 2 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3235)
  (parent 2967)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 4 1)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 3236)
  (parent 2967)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (0 3)) ((6 1) (5 1))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (5 0))
    ((10 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 2 1) (trans 1 1)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (displaced 6 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3237)
  (parent 2967)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 4 1) (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3238)
  (parent 2967)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 4 1) (trans 9 0) (trans 4 0)
    (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3239)
  (parent 2968)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 4 1)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3240)
  (parent 2968)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 4 1)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w (rec l-1))) (gen)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1)))))
  (label 3241)
  (parent 2968)
  (unrealized (0 3) (8 0) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (deflistener (cat (exp (gen) (rec l)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul (rec l) (rec x))))
  (precedes ((0 2) (10 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 4 1)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((recv (cat (exp (gen) (rec l)) la))
      (send (cat (exp (gen) (rec l)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul (rec l) (rec x))))
      (send (cat (exp (gen) x) (mul (rec l) (rec x))))))
  (label 3242)
  (parent 2968)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (0 3)) ((5 1) (4 0)) ((6 1) (4 1))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 0))
    ((10 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 5 1) (trans 1 1)
    (trans 10 0) (trans 5 0) (trans 1 0) (trans 8 1) (trans 3 1)
    (trans 8 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 2 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3243)
  (parent 2968)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0))
    ((5 3) (0 3)) ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 4 1)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))))
  (label 3244)
  (parent 2968)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (5 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 2 1) (trans 1 1)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (displaced 6 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3245)
  (parent 2968)
  (unrealized (0 3) (7 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 4 1) (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3246)
  (parent 2968)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (displaced 11 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 3247)
  (parent 2969)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3248)
  (parent 2969)
  (unrealized (11 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3249)
  (parent 2970)
  (unrealized (0 3) (9 0) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3250)
  (parent 2970)
  (unrealized (0 3) (9 0) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 3251)
  (parent 2970)
  (seen 3621)
  (unrealized (0 3) (11 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (11 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 12 0 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 3252)
  (parent 2970)
  (unrealized (0 3) (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0)) ((6 1) (4 1))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 1) (0 0)) ((11 1) (7 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 5 1) (trans 1 1)
    (trans 11 0) (trans 5 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 3 1) (trans 9 0) (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 2 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3253)
  (parent 2970)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 12 1 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 3254)
  (parent 2970)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 1))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 1) (5 0)) ((11 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 2 1) (trans 1 1)
    (trans 11 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 4 1) (trans 9 0) (trans 7 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (displaced 6 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3255)
  (parent 2970)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 12 1) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 12 0) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3256)
  (parent 2970)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0) (trans 4 0)
    (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3257)
  (parent 2971)
  (unrealized (0 3) (9 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3258)
  (parent 2971)
  (unrealized (0 3) (9 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (gen) (rec l-1)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (contracted (l-2 l-1) (w (rec l-1))) (gen)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1)))))
  (label 3259)
  (parent 2971)
  (unrealized (0 3) (9 0) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (deflistener (cat (exp (gen) x) (mul (rec l) (rec x))))
  (precedes ((0 2) (10 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((recv (cat (exp (gen) x) (mul (rec l) (rec x))))
      (send (cat (exp (gen) x) (mul (rec l) (rec x))))))
  (label 3260)
  (parent 2971)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (0 3)) ((5 1) (4 0)) ((6 1) (4 1))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0)) ((10 1) (0 0))
    ((10 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 5 1) (trans 1 1)
    (trans 10 0) (trans 5 0) (trans 1 0) (trans 7 1) (trans 3 1)
    (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 2 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3261)
  (parent 2971)
  (unrealized (0 3) (8 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0))
    ((5 3) (0 3)) ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))))
  (label 3262)
  (parent 2971)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (5 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 2 1) (trans 1 1)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 4 1)
    (trans 7 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (displaced 6 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3263)
  (parent 2971)
  (unrealized (0 3) (8 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 11 1) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 11 0) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 4 1) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3264)
  (parent 2971)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (3 0)) ((3 1) (0 3))
    ((4 1) (0 0)) ((4 2) (3 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3265)
  (parent 2972)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 2) (0 1)) ((3 2) (6 0)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 2) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 3 1) (trans 8 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 3266)
  (parent 2976)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 1) (0 0))
    ((6 2) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 6 1) (trans 1 1)
    (trans 6 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3267)
  (parent 2980)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((1 2) (4 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1) (trans 5 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 3268)
  (parent 2981)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((3 2) (8 0)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 3269)
  (parent 2986)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (4 0)) ((1 1) (3 0)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 2) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 3 1) (trans 6 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 3270)
  (parent 2987)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (2 0) (10 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 9 1) (trans 4 1)
    (trans 1 1) (trans 9 0) (trans 4 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3271)
  (parent 2998)
  (seen 2836)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3272)
  (parent 2998)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3273)
  (parent 2998)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) la) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self-0 b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) la l-0))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) la) l-0)) (send (cat (exp (gen) la) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 la)))
      (send
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3274)
  (parent 2998)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3275)
  (parent 2998)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3276)
  (parent 2998)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-1) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)) ((9 1) (0 3))
    ((10 1) (3 0)) ((10 1) (5 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (2 0) (9 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv self-1 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 8 1) (trans 1 1) (trans 10 0) (trans 8 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 self-1))
  (operation nonce-test (displaced 4 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig (body self-1 (exp (gen) l) (pubk "sig" self-1))
          (privk "sig" self-1)))
      (send (cat na self-0 self-1 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 self-1 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-2 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3277)
  (parent 2998)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test (displaced 11 9 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3278)
  (parent 2998)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 9 1) (trans 4 1) (trans 1 1) (trans 11 0)
    (trans 9 0) (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-1 l-2)))
      (send
        (sig (body self-1 (exp (gen) l-2) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3279)
  (parent 2998)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3280)
  (parent 2999)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 1) (11 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3281)
  (parent 2999)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3282)
  (parent 2999)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 12 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3283)
  (parent 2999)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3284)
  (parent 3000)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 1) (11 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3285)
  (parent 3000)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3286)
  (parent 3000)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 12 1) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 12 0) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 3287)
  (parent 3000)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1)
    (trans 7 1) (trans 11 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1)))
  (label 3288)
  (parent 3001)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (2 0) (9 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (trans 10 1) (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3289)
  (parent 3003)
  (seen 2854)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 4 1) (trans 1 1)
    (trans 10 0) (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3290)
  (parent 3003)
  (unrealized (11 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 11 4 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3291)
  (parent 3008)
  (seen 2857)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3292)
  (parent 3008)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3293)
  (parent 3008)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3294)
  (parent 3008)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3295)
  (parent 3008)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3296)
  (parent 3008)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (2 0)) ((1 2) (7 0)) ((2 1) (0 1))
    ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3))
    ((10 1) (0 0)) ((10 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0))
  (gen-st (pv b l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test (displaced 2 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3297)
  (parent 3008)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (9 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3))
    ((10 1) (4 0)) ((10 1) (6 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a self-0))
  (operation nonce-test (displaced 5 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3298)
  (parent 3008)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3299)
  (parent 3008)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3300)
  (parent 3009)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3301)
  (parent 3009)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3302)
  (parent 3009)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 12 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3303)
  (parent 3009)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3304)
  (parent 3010)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3305)
  (parent 3010)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3306)
  (parent 3010)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 12 1) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 12 0) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 3307)
  (parent 3010)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1)
    (trans 7 1) (trans 11 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1)))
  (label 3308)
  (parent 3011)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 11 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 3309)
  (parent 3014)
  (seen 2875)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3310)
  (parent 3014)
  (unrealized (11 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (2 0) (10 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 9 1) (trans 4 1)
    (trans 1 1) (trans 9 0) (trans 4 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3311)
  (parent 3018)
  (seen 2878)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (2 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3312)
  (parent 3018)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3313)
  (parent 3018)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) la) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self-0 b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) la l-0))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) la) l)) (send (cat (exp (gen) la) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 la)))
      (send
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3314)
  (parent 3018)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3315)
  (parent 3018)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (2 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3316)
  (parent 3018)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-1) (priv-stor priv-stor-1)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)) ((9 1) (0 3))
    ((10 1) (3 0)) ((10 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (2 0) (9 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv self-1 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 8 1) (trans 1 1) (trans 10 0) (trans 8 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 self-1))
  (operation nonce-test (displaced 4 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig (body self-1 (exp (gen) l) (pubk "sig" self-1))
          (privk "sig" self-1)))
      (send (cat na self-0 self-1 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 self-1 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (stor priv-stor-1 (cat pt-2 "nil")) (send l-1))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3317)
  (parent 3018)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test (displaced 11 9 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3318)
  (parent 3018)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 9 1) (trans 4 1) (trans 1 1) (trans 11 0)
    (trans 9 0) (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-1 l-2)))
      (send
        (sig (body self-1 (exp (gen) l-2) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3319)
  (parent 3018)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3320)
  (parent 3019)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 1) (11 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3321)
  (parent 3019)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3322)
  (parent 3019)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 12 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3323)
  (parent 3019)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3324)
  (parent 3020)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 1) (11 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3325)
  (parent 3020)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3326)
  (parent 3020)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 12 1) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 12 0) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 3327)
  (parent 3020)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1)
    (trans 7 1) (trans 11 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1)))
  (label 3328)
  (parent 3021)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (2 0) (9 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (trans 10 1) (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3329)
  (parent 3023)
  (seen 2896)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 4 1) (trans 1 1)
    (trans 10 0) (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3330)
  (parent 3023)
  (unrealized (11 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3))
    ((5 1) (4 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3331)
  (parent 3027)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 1) (11 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3))
    ((5 1) (4 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3332)
  (parent 3027)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
      (send (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3333)
  (parent 3027)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 12 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3334)
  (parent 3027)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3))
    ((5 1) (4 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3335)
  (parent 3028)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 1) (11 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3))
    ((5 1) (4 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3336)
  (parent 3028)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
      (send (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3337)
  (parent 3028)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 12 1) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 12 0) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 3338)
  (parent 3028)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-listener l-1) (rec l-1) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1)))
  (label 3339)
  (parent 3029)
  (unrealized (0 3) (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (11 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 2 1) (trans 1 1)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (added-listener l-1) (mul (rec l-1) (rec l-1))
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1)))
  (label 3340)
  (parent 3033)
  (unrealized (0 3) (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 11 4 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3341)
  (parent 3036)
  (seen 2909)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3342)
  (parent 3036)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3343)
  (parent 3036)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3344)
  (parent 3036)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3345)
  (parent 3036)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3346)
  (parent 3036)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (2 0)) ((1 2) (7 0)) ((2 1) (0 1))
    ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3))
    ((10 1) (0 0)) ((10 1) (6 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0))
  (gen-st (pv b l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test (displaced 2 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3347)
  (parent 3036)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (9 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3))
    ((10 1) (4 0)) ((10 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a self-0))
  (operation nonce-test (displaced 5 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3348)
  (parent 3036)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 11 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3349)
  (parent 3036)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3350)
  (parent 3037)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3351)
  (parent 3037)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3352)
  (parent 3037)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 12 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3353)
  (parent 3037)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3354)
  (parent 3038)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3355)
  (parent 3038)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3356)
  (parent 3038)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 12 1) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 12 0) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 3357)
  (parent 3038)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1)
    (trans 7 1) (trans 11 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1)))
  (label 3358)
  (parent 3039)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 11 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 3359)
  (parent 3042)
  (seen 2927)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3360)
  (parent 3042)
  (unrealized (11 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3))
    ((5 1) (4 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3361)
  (parent 3045)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 1) (11 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3))
    ((5 1) (4 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3362)
  (parent 3045)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
      (send (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3363)
  (parent 3045)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 12 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3364)
  (parent 3045)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3))
    ((5 1) (4 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3365)
  (parent 3046)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 1) (11 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3))
    ((5 1) (4 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3366)
  (parent 3046)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
      (send (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3367)
  (parent 3046)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 12 1) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 12 0) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 3368)
  (parent 3046)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-listener l-1) (rec l-1) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1)))
  (label 3369)
  (parent 3047)
  (unrealized (0 3) (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (11 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 2 1) (trans 1 1)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (added-listener l-1) (mul (rec l-1) (rec l-1))
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1)))
  (label 3370)
  (parent 3051)
  (unrealized (0 3) (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (3 0))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 2) (5 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 3 1) (trans 6 0) (trans 3 0)
    (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 3371)
  (parent 3053)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((2 1) (4 0)) ((3 1) (8 0)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (8 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 4 1) (trans 7 0)
    (trans 4 0) (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l)) lb))
      (send (cat (exp (gen) (mul l l)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3372)
  (parent 3054)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((1 2) (0 1)) ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (5 0))
    ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (x-0 beta)
    (x-0 l-0))
  (precur (6 0) (9 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (trans 8 1)
    (trans 5 1) (trans 8 0) (trans 5 0) (neq self b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3373)
  (parent 3055)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((3 1) (10 0))
    ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (x-0 beta)
    (x-0 l-0))
  (precur (6 0) (9 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (trans 8 1)
    (trans 5 1) (trans 8 0) (trans 5 0) (neq self b))
  (operation channel-test (displaced 11 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3374)
  (parent 3055)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (x-0 beta)
    (x-0 l-1))
  (precur (6 0) (9 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 3 1) (trans 1 1) (trans 11 0) (trans 3 0)
    (trans 1 0) (trans 8 1) (trans 5 1) (trans 8 0) (trans 5 0)
    (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-5 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv a l-1)))))
  (label 3375)
  (parent 3055)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((1 2) (0 1)) ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (5 0))
    ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (y-0 alpha)
    (y-0 zeta) (y-0 l-0))
  (precur (6 0) (9 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (trans 8 1) (trans 5 1)
    (trans 8 0) (trans 5 0) (neq self b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 3376)
  (parent 3059)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((3 1) (10 0))
    ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (y-0 alpha)
    (y-0 zeta) (y-0 l-0))
  (precur (6 0) (9 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (trans 8 1) (trans 5 1)
    (trans 8 0) (trans 5 0) (neq self b))
  (operation channel-test (displaced 11 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 3377)
  (parent 3059)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 3) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (y-0 alpha)
    (y-0 zeta) (y-0 l-1))
  (precur (6 0) (9 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 3 1) (trans 1 1) (trans 11 0) (trans 3 0) (trans 1 0)
    (trans 8 1) (trans 5 1) (trans 8 0) (trans 5 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-5 (pv b-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv b-0 l-1)))))
  (label 3378)
  (parent 3059)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((1 2) (3 0))
    ((2 1) (0 3)) ((3 1) (0 3)) ((4 2) (3 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (3 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 4 1) (trans 4 0) (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 3379)
  (parent 3062)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (3 0)) ((3 2) (6 0)) ((4 1) (6 0))
    ((5 2) (4 0)) ((6 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (4 0) (6 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1) (trans 5 0)
    (trans 3 0) (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3380)
  (parent 3063)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 2) (5 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 6 1) (trans 3 1) (trans 6 0) (trans 3 0)
    (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb)))
  (label 3381)
  (parent 3070)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (rec lb)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 1) (7 0)) ((3 1) (8 0)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (8 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 4 1) (trans 7 0)
    (trans 4 0) (neq self b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (rec lb)) l))
      (send (cat (exp (gen) (rec lb)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3382)
  (parent 3071)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (10 0))
    ((1 2) (0 1)) ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (8 0))
    ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (x-0 beta)
    (x-0 l-0))
  (precur (6 0) (9 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (trans 8 1)
    (trans 5 1) (trans 8 0) (trans 5 0) (neq self b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (rec l))) (send (cat (gen) (rec l))))
    ((recv l) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3383)
  (parent 3072)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (8 0)) ((3 1) (10 0))
    ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (x-0 beta)
    (x-0 l-0))
  (precur (6 0) (9 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 3 1) (trans 1 1) (trans 3 0) (trans 1 0) (trans 8 1)
    (trans 5 1) (trans 8 0) (trans 5 0) (neq self b))
  (operation channel-test (displaced 11 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3384)
  (parent 3072)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (8 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (x-0 beta)
    (x-0 l-1))
  (precur (6 0) (9 0))
  (gen-st (pv b l-0) (pv self l) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 3 1) (trans 1 1) (trans 11 0) (trans 3 0)
    (trans 1 0) (trans 8 1) (trans 5 1) (trans 8 0) (trans 5 0)
    (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-5 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv a l-1)))))
  (label 3385)
  (parent 3072)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (10 0))
    ((1 2) (0 1)) ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (8 0))
    ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (y-0 alpha)
    (y-0 zeta) (y-0 l-0))
  (precur (6 0) (9 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (trans 8 1) (trans 5 1)
    (trans 8 0) (trans 5 0) (neq self b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (rec l))) (send (cat (gen) (rec l))))
    ((recv l) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 3386)
  (parent 3076)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (8 0)) ((3 1) (10 0))
    ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (y-0 alpha)
    (y-0 zeta) (y-0 l-0))
  (precur (6 0) (9 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 3 1)
    (trans 1 1) (trans 3 0) (trans 1 0) (trans 8 1) (trans 5 1)
    (trans 8 0) (trans 5 0) (neq self b))
  (operation channel-test (displaced 11 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 3387)
  (parent 3076)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (8 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 3)) ((10 3) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (y-0 alpha)
    (y-0 zeta) (y-0 l-1))
  (precur (6 0) (9 0))
  (gen-st (pv b l-0) (pv self l) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 3 1) (trans 1 1) (trans 11 0) (trans 3 0) (trans 1 0)
    (trans 8 1) (trans 5 1) (trans 8 0) (trans 5 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-5 (pv b-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv b-0 l-1)))))
  (label 3388)
  (parent 3076)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3))
    ((3 1) (0 3)) ((4 1) (6 0)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 2) (5 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (3 0))
  (gen-st (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 1 1) (trans 4 0)
    (trans 1 0) (trans 6 1) (trans 2 1) (trans 6 0) (trans 2 0)
    (neq self self-0))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb)))
  (label 3389)
  (parent 3079)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) (rec lb)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1))
    ((2 1) (8 0)) ((3 2) (2 0)) ((4 1) (2 0)) ((5 1) (7 0))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (4 0) (8 0))
  (gen-st (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 5 1) (trans 1 1)
    (trans 5 0) (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0)
    (trans 3 0) (neq self self-0))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec lb)) l))
      (send (cat (exp (gen) (rec lb)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3390)
  (parent 3080)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) (rec l))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 1) (10 0))
    ((1 2) (0 1)) ((2 3) (9 0)) ((3 1) (2 1)) ((4 2) (3 0))
    ((5 1) (3 0)) ((6 1) (2 0)) ((6 1) (8 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (x-0 beta)
    (x-0 l-0))
  (precur (5 0) (9 0))
  (gen-st (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 1 1) (trans 6 0) (trans 1 0) (trans 8 1)
    (trans 4 1) (trans 8 0) (trans 4 0) (neq self self-0))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (mul (rec l) (rec l))))
      (send (cat (exp (gen) l) (mul (rec l) (rec l)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l) (send l))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 3391)
  (parent 3081)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (2 0)) ((6 1) (8 0)) ((6 1) (10 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (x-0 beta)
    (x-0 l-0))
  (precur (5 0) (9 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 1 1) (trans 6 0) (trans 1 0) (trans 8 1)
    (trans 4 1) (trans 8 0) (trans 4 0) (neq self self-0))
  (operation channel-test (displaced 11 6 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self-0 b (exp (gen) x-0)))))
  (label 3392)
  (parent 3081)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (self self-0 a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (2 0)) ((6 1) (8 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (x-0 beta)
    (x-0 l-1))
  (precur (5 0) (9 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 6 1) (trans 1 1) (trans 11 0) (trans 6 0)
    (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0) (trans 4 0)
    (neq self self-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-5 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv a l-1)))))
  (label 3393)
  (parent 3081)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 locn) (l rndx) (alpha zeta expt)
    (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) (rec l))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 1) (10 0))
    ((1 2) (0 1)) ((2 3) (9 0)) ((3 1) (2 1)) ((4 2) (3 0))
    ((5 1) (3 0)) ((6 1) (2 0)) ((6 1) (8 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (y-0 alpha)
    (y-0 zeta) (y-0 l-0))
  (precur (5 0) (9 0))
  (gen-st (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 1 1) (trans 6 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq self self-0))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (mul (rec l) (rec l))))
      (send (cat (exp (gen) l) (mul (rec l) (rec l)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l) (send l))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 3394)
  (parent 3085)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 locn) (l rndx) (alpha zeta expt)
    (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (2 0)) ((6 1) (8 0)) ((6 1) (10 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (y-0 alpha)
    (y-0 zeta) (y-0 l-0))
  (precur (5 0) (9 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 1 1) (trans 6 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq self self-0))
  (operation channel-test (displaced 11 6 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self-0
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 3395)
  (parent 3085)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (self self-0 a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (2 0)) ((6 1) (8 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (y-0 alpha)
    (y-0 zeta) (y-0 l-1))
  (precur (5 0) (9 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 6 1) (trans 1 1) (trans 11 0) (trans 6 0) (trans 1 0)
    (trans 8 1) (trans 4 1) (trans 8 0) (trans 4 0) (neq self self-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-5 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv b l-1)))))
  (label 3396)
  (parent 3085)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (trans 9 1) (trans 6 1) (trans 9 0)
    (trans 6 0) (neq self b))
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l)) lb))
      (send (cat (exp (gen) (mul l l)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3397)
  (parent 3089)
  (seen 3054)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (x-0 la)
    (x-0 beta))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (trans 9 1)
    (trans 6 1) (trans 9 0) (trans 6 0) (neq self b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-5 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3398)
  (parent 3089)
  (unrealized (0 3) (10 0) (11 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (trans 9 1)
    (trans 6 1) (trans 9 0) (trans 6 0) (neq self b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3399)
  (parent 3089)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l la y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta la) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la la))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul la la)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul la la)))
  (deflistener la)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l la)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la la)) (y l) (x la))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (trans 9 1)
    (trans 6 1) (trans 9 0) (trans 6 0) (neq self b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self la)))
      (recv
        (sig (body b (exp (gen) la) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la la)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) la la)) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la la)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul la la)) l))
      (send (cat (exp (gen) (mul la la)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul la la))) (send (cat (gen) (mul la la))))
    ((recv la) (send la))
    ((load priv-stor (cat pt-0 (pv self la)))
      (stor priv-stor (cat pt-4 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3400)
  (parent 3089)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (trans 9 1)
    (trans 6 1) (trans 9 0) (trans 6 0) (neq self b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-1)
    (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 3401)
  (parent 3089)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (y-0 lb)
    (y-0 alpha) (y-0 zeta))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (trans 9 1) (trans 6 1)
    (trans 9 0) (trans 6 0) (neq self b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-5 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3402)
  (parent 3089)
  (unrealized (0 3) (10 0) (11 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 3))
    ((10 1) (3 0)) ((10 1) (5 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (6 0) (9 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 1 1) (trans 10 0) (trans 1 0) (trans 8 1)
    (trans 5 1) (trans 8 0) (trans 5 0) (neq self self-0))
  (operation nonce-test (displaced 4 11 ltx-gen 3) (exp (gen) l-1)
    (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0)))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3403)
  (parent 3089)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 1 1) (trans 11 0) (trans 4 0)
    (trans 1 0) (trans 9 1) (trans 6 1) (trans 9 0) (trans 6 0)
    (neq self b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3404)
  (parent 3089)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((1 2) (2 0)) ((1 2) (7 0)) ((2 1) (0 1)) ((3 3) (9 0))
    ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (x-0 beta)
    (x-0 l-0))
  (precur (2 0) (7 0) (9 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 8 0) (trans 6 0) (neq self b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3405)
  (parent 3091)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((1 2) (7 0)) ((2 1) (0 1)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (x-0 beta)
    (x-0 l-0))
  (precur (2 0) (7 0) (9 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 8 0) (trans 6 0) (neq self b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3406)
  (parent 3091)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((1 2) (7 0)) ((2 1) (0 1)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 2) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (x-0 beta)
    (x-0 l-1))
  (precur (2 0) (7 0) (9 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 1 1) (trans 11 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-5 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv a l-1)))))
  (label 3407)
  (parent 3091)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((1 2) (2 0)) ((1 2) (7 0)) ((2 1) (0 1)) ((3 3) (9 0))
    ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (y-0 alpha)
    (y-0 zeta) (y-0 l-0))
  (precur (2 0) (7 0) (9 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq self b))
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 3408)
  (parent 3095)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((1 2) (7 0)) ((2 1) (0 1)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((4 1) (10 0)) ((5 1) (3 1)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (y-0 alpha)
    (y-0 zeta) (y-0 l-0))
  (precur (2 0) (7 0) (9 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq self b))
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 3409)
  (parent 3095)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((1 2) (7 0)) ((2 1) (0 1)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)) ((10 3) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (y-0 alpha)
    (y-0 zeta) (y-0 l-1))
  (precur (2 0) (7 0) (9 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 11 1)
    (trans 4 1) (trans 1 1) (trans 11 0) (trans 4 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-5 (pv b-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv b-0 l-1)))))
  (label 3410)
  (parent 3095)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (rec lb)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (9 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (trans 9 1) (trans 6 1) (trans 9 0)
    (trans 6 0) (neq self b))
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (rec lb)) l))
      (send (cat (exp (gen) (rec lb)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3411)
  (parent 3099)
  (seen 3071)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (9 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (x-0 la)
    (x-0 beta))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (trans 9 1)
    (trans 6 1) (trans 9 0) (trans 6 0) (neq self b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-5 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3412)
  (parent 3099)
  (unrealized (0 3) (10 0) (11 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (9 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (trans 9 1)
    (trans 6 1) (trans 9 0) (trans 6 0) (neq self b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3413)
  (parent 3099)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l la y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta la) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la la))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) la))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l la))
  (deflistener (cat (gen) (rec l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (9 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l la)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la la)) (y l) (x la))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l) (pv self la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (trans 9 1)
    (trans 6 1) (trans 9 0) (trans 6 0) (neq self b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self la)))
      (recv
        (sig (body b (exp (gen) la) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la la)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) la la)) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la la)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) la))
      (send (cat (exp (gen) (rec l)) la)))
    ((load priv-stor (cat pt-0 (pv self la)))
      (stor priv-stor (cat pt-3 "nil")) (send la))
    ((recv (cat (gen) (rec l))) (send (cat (gen) (rec l))))
    ((recv l) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3414)
  (parent 3099)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (9 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (trans 9 1)
    (trans 6 1) (trans 9 0) (trans 6 0) (neq self b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-1)
    (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (rec l))) (send (cat (gen) (rec l))))
    ((recv l) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 3415)
  (parent 3099)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (9 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (y-0 lb)
    (y-0 alpha) (y-0 zeta))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (trans 9 1) (trans 6 1)
    (trans 9 0) (trans 6 0) (neq self b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-5 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3416)
  (parent 3099)
  (unrealized (0 3) (10 0) (11 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (9 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 3))
    ((10 1) (3 0)) ((10 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (6 0) (9 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 1 1) (trans 10 0) (trans 1 0) (trans 8 1)
    (trans 5 1) (trans 8 0) (trans 5 0) (neq self self-0))
  (operation nonce-test (displaced 4 11 ltx-gen 3) (exp (gen) l-1)
    (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0)))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3417)
  (parent 3099)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (9 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 1 1) (trans 11 0) (trans 4 0)
    (trans 1 0) (trans 9 1) (trans 6 1) (trans 9 0) (trans 6 0)
    (neq self b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3418)
  (parent 3099)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta (mul x w)))
  (deflistener (cat (exp (gen) (rec lb)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((7 1) (9 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (2 0) (6 0) (10 0))
  (gen-st (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 7 1) (trans 1 1)
    (trans 7 0) (trans 1 0) (trans 9 1) (trans 5 1) (trans 9 0)
    (trans 5 0) (neq self self-0))
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (rec lb)) l))
      (send (cat (exp (gen) (rec lb)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3419)
  (parent 3101)
  (seen 3080)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (self self-0 a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 la rndx) (beta expt)
    (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((7 1) (9 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (x-0 la)
    (x-0 beta))
  (precur (2 0) (6 0) (10 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 7 1) (trans 1 1) (trans 7 0) (trans 1 0) (trans 9 1)
    (trans 5 1) (trans 9 0) (trans 5 0) (neq self self-0))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-5 (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 3420)
  (parent 3101)
  (unrealized (0 3) (10 0) (11 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((7 1) (9 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (6 0) (10 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 7 1) (trans 1 1) (trans 7 0) (trans 1 0) (trans 9 1)
    (trans 5 1) (trans 9 0) (trans 5 0) (neq self self-0))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3421)
  (parent 3101)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l la y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la la) (x x) (beta la)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la la))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l)) la))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l) (mul (rec l) (rec l))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((7 1) (9 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l la)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la la)) (y l) (x la))
  (precur (2 0) (6 0) (10 0))
  (gen-st (pv self la) (pv self-0 l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 7 1) (trans 1 1) (trans 7 0) (trans 1 0) (trans 9 1)
    (trans 5 1) (trans 9 0) (trans 5 0) (neq self self-0))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self la)))
      (recv
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self self-0
            (hash (exp (gen) (mul la la)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) la la)) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul la la)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l)) la))
      (send (cat (exp (gen) (rec l)) la)))
    ((load priv-stor (cat pt-0 (pv self la)))
      (stor priv-stor (cat pt-1 "nil")) (send la))
    ((recv (cat (exp (gen) l) (mul (rec l) (rec l))))
      (send (cat (exp (gen) l) (mul (rec l) (rec l)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l) (send l))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3422)
  (parent 3101)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) (rec l))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((7 1) (9 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (6 0) (10 0))
  (gen-st (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 7 1) (trans 1 1) (trans 7 0) (trans 1 0) (trans 9 1)
    (trans 5 1) (trans 9 0) (trans 5 0) (neq self self-0))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-1)
    (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (mul (rec l) (rec l))))
      (send (cat (exp (gen) l) (mul (rec l) (rec l)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l) (send l))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 3423)
  (parent 3101)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (self self-0 a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb rndx)
    (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((7 1) (9 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (y-0 lb)
    (y-0 alpha) (y-0 zeta))
  (precur (2 0) (6 0) (10 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 7 1)
    (trans 1 1) (trans 7 0) (trans 1 0) (trans 9 1) (trans 5 1)
    (trans 9 0) (trans 5 0) (neq self self-0))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-5 (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3424)
  (parent 3101)
  (unrealized (0 3) (10 0) (11 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l x y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((7 1) (9 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (6 0) (10 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 7 1) (trans 1 1) (trans 7 0) (trans 1 0) (trans 9 1)
    (trans 5 1) (trans 9 0) (trans 5 0) (neq self self-0))
  (operation nonce-test (displaced 11 7 ltx-gen 3) (exp (gen) l-1)
    (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec x) y (rec l-0))))))
  (label 3425)
  (parent 3101)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((7 1) (9 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (6 0) (10 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 7 1) (trans 1 1) (trans 11 0) (trans 7 0)
    (trans 1 0) (trans 9 1) (trans 5 1) (trans 9 0) (trans 5 0)
    (neq self self-0))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3426)
  (parent 3101)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 1) (4 0)) ((3 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a self))
  (operation generalization weakened ((3 2) (4 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 3427)
  (parent 3103)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (0 0)) ((5 2) (3 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3428)
  (parent 3104)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (3 0))
    ((3 2) (7 0)) ((4 1) (7 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((6 2) (4 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3429)
  (parent 3105)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (5 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 5 1) (trans 9 0) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 3430)
  (parent 3114)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3431)
  (parent 3115)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3432)
  (parent 3115)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3433)
  (parent 3115)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) la) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 12 0 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) la) l-0)) (send (cat (exp (gen) la) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) la)) (send (cat (gen) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-7 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3434)
  (parent 3115)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 12 1 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3435)
  (parent 3115)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3436)
  (parent 3115)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)) ((11 1) (0 0)) ((11 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 3 1) (trans 1 1) (trans 11 0) (trans 3 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 5 1) (trans 9 0)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 2 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3437)
  (parent 3115)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)) ((11 1) (3 0)) ((11 1) (5 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (10 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 5 1) (trans 9 0)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (displaced 4 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3438)
  (parent 3115)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3439)
  (parent 3115)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (3 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 7 1) (trans 2 1) (trans 1 1)
    (trans 7 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1)
    (trans 5 0) (trans 3 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3440)
  (parent 3116)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 1) (9 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (9 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 8 1) (trans 2 1)
    (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq self-0 b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3441)
  (parent 3117)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((3 1) (11 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation channel-test (displaced 12 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3442)
  (parent 3118)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 1) (11 0)) ((1 2) (0 1))
    ((2 3) (10 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3443)
  (parent 3118)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation channel-test (displaced 12 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self-0 b-0 (exp (gen) x-0)))))
  (label 3444)
  (parent 3118)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1) (pv a l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 9 1) (trans 3 1) (trans 1 1) (trans 12 0)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a l-2)))))
  (label 3445)
  (parent 3118)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((3 1) (11 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation channel-test (displaced 12 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3446)
  (parent 3122)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 1) (11 0)) ((1 2) (0 1))
    ((2 3) (10 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3447)
  (parent 3122)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation channel-test (displaced 12 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3448)
  (parent 3122)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 12 1)
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 12 0) (trans 9 0)
    (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0)
    (trans 5 0) (neq self-0 b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3449)
  (parent 3122)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 1) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l)))
  (label 3450)
  (parent 3126)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (5 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (4 0)) ((4 2) (7 0)) ((5 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3451)
  (parent 3127)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (9 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (5 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 5 1) (trans 9 0) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 3452)
  (parent 3136)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3453)
  (parent 3137)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3454)
  (parent 3137)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3455)
  (parent 3137)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 12 0 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3456)
  (parent 3137)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 12 1 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3457)
  (parent 3137)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3458)
  (parent 3137)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (9 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)) ((11 1) (0 0)) ((11 1) (7 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 3 1) (trans 1 1) (trans 11 0) (trans 3 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 5 1) (trans 9 0)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 2 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3459)
  (parent 3137)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (9 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)) ((11 1) (3 0)) ((11 1) (5 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (10 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 5 1) (trans 9 0)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (displaced 4 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3460)
  (parent 3137)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3461)
  (parent 3137)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (4 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 3462)
  (parent 3138)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (5 0)) ((4 1) (9 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3463)
  (parent 3139)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((2 1) (11 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3464)
  (parent 3140)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (11 0)) ((1 2) (0 1)) ((1 2) (9 0))
    ((2 1) (0 0)) ((2 1) (8 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3465)
  (parent 3140)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3466)
  (parent 3140)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 3467)
  (parent 3140)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((2 1) (11 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3468)
  (parent 3144)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (11 0)) ((1 2) (0 1)) ((1 2) (9 0))
    ((2 1) (0 0)) ((2 1) (8 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3469)
  (parent 3144)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3470)
  (parent 3144)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 12 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 12 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3471)
  (parent 3144)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0)) ((3 2) (0 3))
    ((4 2) (0 3)) ((5 1) (0 0)) ((5 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1)
    (trans 4 0) (trans 3 0) (neq self-0 b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3472)
  (parent 3148)
  (seen 3748)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (7 0)) ((4 1) (7 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((6 2) (4 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3473)
  (parent 3149)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (7 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 5 1) (trans 9 0) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 3474)
  (parent 3158)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3475)
  (parent 3159)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3476)
  (parent 3159)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3477)
  (parent 3159)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) la) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 12 0 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) la) l)) (send (cat (exp (gen) la) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) la)) (send (cat (gen) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-7 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3478)
  (parent 3159)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 12 1 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3479)
  (parent 3159)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3480)
  (parent 3159)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)) ((11 1) (0 0)) ((11 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 3 1) (trans 1 1) (trans 11 0) (trans 3 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 5 1) (trans 9 0)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 2 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3481)
  (parent 3159)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)) ((11 1) (3 0)) ((11 1) (7 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (10 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 5 1) (trans 9 0)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (displaced 4 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3482)
  (parent 3159)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3483)
  (parent 3159)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 7 1) (trans 2 1) (trans 1 1)
    (trans 7 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1)
    (trans 5 0) (trans 3 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3484)
  (parent 3160)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 1) (9 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (9 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 8 1) (trans 2 1)
    (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq self-0 b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3485)
  (parent 3161)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((3 1) (11 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation channel-test (displaced 12 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3486)
  (parent 3162)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 1) (11 0)) ((1 2) (0 1))
    ((2 3) (10 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3487)
  (parent 3162)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 b))
  (operation channel-test (displaced 12 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self-0 b-0 (exp (gen) x-0)))))
  (label 3488)
  (parent 3162)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (8 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1) (pv a l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 9 1) (trans 3 1) (trans 1 1) (trans 12 0)
    (trans 9 0) (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a l-2)))))
  (label 3489)
  (parent 3162)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((3 1) (11 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation channel-test (displaced 12 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3490)
  (parent 3166)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 1) (11 0)) ((1 2) (0 1))
    ((2 3) (10 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3491)
  (parent 3166)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 3 1) (trans 1 1) (trans 9 0) (trans 3 0) (trans 1 0)
    (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation channel-test (displaced 12 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3492)
  (parent 3166)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (8 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 12 1)
    (trans 9 1) (trans 3 1) (trans 1 1) (trans 12 0) (trans 9 0)
    (trans 3 0) (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0)
    (trans 5 0) (neq self-0 b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3493)
  (parent 3166)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 3494)
  (parent 3170)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1)
    (trans 6 1) (trans 11 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3495)
  (parent 3170)
  (unrealized (12 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 9 1)
    (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0) (trans 7 0)
    (trans 5 0) (neq a self-0))
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 3496)
  (parent 3171)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 7 1)
    (trans 5 1) (trans 11 0) (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3497)
  (parent 3171)
  (unrealized (12 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 2) (0 3)) ((5 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 4 1)
    (trans 5 0) (trans 4 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 3498)
  (parent 3172)
  (seen 3762)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (5 0)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 2) (7 0)) ((5 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3499)
  (parent 3173)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (9 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 1) (7 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 5 1) (trans 9 0) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 3500)
  (parent 3182)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3501)
  (parent 3183)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3502)
  (parent 3183)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3503)
  (parent 3183)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 12 0 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3504)
  (parent 3183)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 12 1 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3505)
  (parent 3183)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3506)
  (parent 3183)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (9 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)) ((11 1) (0 0)) ((11 1) (5 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 3 1) (trans 1 1) (trans 11 0) (trans 3 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 5 1) (trans 9 0)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 2 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3507)
  (parent 3183)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (9 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)) ((11 1) (3 0)) ((11 1) (7 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (10 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 5 1) (trans 9 0)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (displaced 4 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3508)
  (parent 3183)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3509)
  (parent 3183)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 3510)
  (parent 3184)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 1) (7 0)) ((4 1) (9 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3511)
  (parent 3185)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((2 1) (11 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3512)
  (parent 3186)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (11 0)) ((1 2) (0 1)) ((1 2) (9 0))
    ((2 1) (0 0)) ((2 1) (6 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3513)
  (parent 3186)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3514)
  (parent 3186)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 3515)
  (parent 3186)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((2 1) (11 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3516)
  (parent 3190)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (11 0)) ((1 2) (0 1)) ((1 2) (9 0))
    ((2 1) (0 0)) ((2 1) (6 0)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3517)
  (parent 3190)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3518)
  (parent 3190)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 12 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 12 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3519)
  (parent 3190)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 3520)
  (parent 3194)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1)
    (trans 6 1) (trans 11 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3521)
  (parent 3194)
  (unrealized (12 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 9 1)
    (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0) (trans 7 0)
    (trans 5 0) (neq a self-0))
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 3522)
  (parent 3195)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 9 1) (trans 2 1) (trans 1 1)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 7 1)
    (trans 5 1) (trans 11 0) (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3523)
  (parent 3195)
  (unrealized (12 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (0 3))
    ((4 2) (3 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 3524)
  (parent 3196)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (0 3)) ((4 2) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1) (trans 4 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 3525)
  (parent 3197)
  (seen 3842)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 0))
    ((3 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 self))
  (operation generalization weakened ((3 2) (4 0)))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 3526)
  (parent 3199)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((4 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 4 1) (trans 3 1) (trans 4 0) (trans 3 0)
    (neq a self))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 3527)
  (parent 3200)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1) (trans 5 0)
    (trans 3 0) (neq a self))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3528)
  (parent 3201)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (7 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 8 1)
    (trans 4 1) (trans 1 1) (trans 8 0) (trans 4 0) (trans 1 0)
    (trans 6 1) (trans 2 1) (trans 6 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3529)
  (parent 3209)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (2 0) (10 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 9 1) (trans 5 1)
    (trans 1 1) (trans 9 0) (trans 5 0) (trans 1 0) (trans 7 1)
    (trans 3 1) (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 11 4 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3530)
  (parent 3210)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0)
    (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0)
    (neq self-0 b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3531)
  (parent 3210)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0)
    (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0)
    (neq self-0 b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3532)
  (parent 3210)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) la) l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0)
    (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0)
    (neq self-0 b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self-0 b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) la l-0))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) la) l-0)) (send (cat (exp (gen) la) l-0)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 la)))
      (send
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3533)
  (parent 3210)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 2) (10 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((3 2) (8 0)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0)
    (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0)
    (neq self-0 b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-1 "nil")) (send l-1))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3534)
  (parent 3210)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0) (trans 1 0)
    (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3535)
  (parent 3210)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-1) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (7 0)) ((4 3) (9 0)) ((5 1) (4 1)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)) ((9 1) (0 3))
    ((10 1) (4 0)) ((10 1) (6 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (2 0) (9 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv self-1 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 8 1) (trans 1 1) (trans 10 0) (trans 8 0)
    (trans 1 0) (trans 6 1) (trans 3 1) (trans 6 0) (trans 3 0)
    (neq self-0 self-1))
  (operation nonce-test (displaced 5 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig (body self-1 (exp (gen) l) (pubk "sig" self-1))
          (privk "sig" self-1)))
      (send (cat na self-0 self-1 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 self-1 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (stor priv-stor-1 (cat pt-2 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3536)
  (parent 3210)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0)
    (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0)
    (neq self-0 b))
  (operation nonce-test (displaced 11 9 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3537)
  (parent 3210)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 9 1) (trans 5 1) (trans 1 1) (trans 11 0)
    (trans 9 0) (trans 5 0) (trans 1 0) (trans 7 1) (trans 3 1)
    (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-1 l-2)))
      (send
        (sig (body self-1 (exp (gen) l-2) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3538)
  (parent 3210)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0) (trans 4 0)
    (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3539)
  (parent 3211)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0) (trans 4 0)
    (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3540)
  (parent 3211)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((6 1) (11 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0) (trans 4 0)
    (neq a b))
  (operation channel-test (displaced 12 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3541)
  (parent 3211)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 12 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3542)
  (parent 3211)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3543)
  (parent 3212)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3544)
  (parent 3212)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((6 1) (11 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 12 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3545)
  (parent 3212)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 12 1) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 12 0) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 4 1) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 3546)
  (parent 3212)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1)
    (trans 4 1) (trans 11 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1)))
  (label 3547)
  (parent 3213)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (0 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (2 0) (9 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (trans 10 1) (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0)
    (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0)
    (neq self-0 b))
  (operation nonce-test (displaced 11 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3548)
  (parent 3215)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 5 1) (trans 1 1)
    (trans 10 0) (trans 5 0) (trans 1 0) (trans 7 1) (trans 3 1)
    (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3549)
  (parent 3215)
  (unrealized (11 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 2) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 3 1) (trans 9 0) (trans 7 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l)))
  (label 3550)
  (parent 3219)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0)
    (neq a b))
  (operation nonce-test (displaced 12 5 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3551)
  (parent 3220)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3552)
  (parent 3220)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3553)
  (parent 3220)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 12 0 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-7 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3554)
  (parent 3220)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 12 1 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3555)
  (parent 3220)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3556)
  (parent 3220)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 2) (8 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)) ((11 1) (0 0)) ((11 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 5 1) (trans 1 1) (trans 11 0) (trans 5 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 3 1) (trans 9 0)
    (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 2 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3557)
  (parent 3220)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (10 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)) ((11 1) (5 0)) ((11 1) (7 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 4 1) (trans 9 0)
    (trans 7 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (displaced 6 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3558)
  (parent 3220)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3559)
  (parent 3220)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((2 1) (12 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3560)
  (parent 3221)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (12 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3561)
  (parent 3221)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 13 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3562)
  (parent 3221)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 3) (11 0)) ((13 1) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 13 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 3563)
  (parent 3221)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((2 1) (12 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3564)
  (parent 3222)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (12 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3565)
  (parent 3222)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 13 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3566)
  (parent 3222)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 13 1) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 13 0) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 3567)
  (parent 3222)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 12 4 ltx-disclose 3) l-1 (11 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 3568)
  (parent 3223)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (deflistener x)
  (precedes ((0 2) (12 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (11 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-listener x) (mul l-0 (rec x)) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-7 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))) ((recv x) (send x)))
  (label 3569)
  (parent 3224)
  (unrealized (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (displaced 12 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 3570)
  (parent 3226)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3571)
  (parent 3226)
  (unrealized (12 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (7 0)) ((5 1) (3 1))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 8 1)
    (trans 4 1) (trans 1 1) (trans 8 0) (trans 4 0) (trans 1 0)
    (trans 7 1) (trans 2 1) (trans 7 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3572)
  (parent 3229)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (2 0) (10 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 9 1) (trans 5 1)
    (trans 1 1) (trans 9 0) (trans 5 0) (trans 1 0) (trans 8 1)
    (trans 3 1) (trans 8 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 11 4 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3573)
  (parent 3230)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (2 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0)
    (trans 1 0) (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0)
    (neq self-0 b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3574)
  (parent 3230)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0)
    (trans 1 0) (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0)
    (neq self-0 b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3575)
  (parent 3230)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (deflistener (cat (exp (gen) la) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0)
    (trans 1 0) (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0)
    (neq self-0 b))
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self-0 b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) la l-0))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((recv (cat (exp (gen) la) l)) (send (cat (exp (gen) la) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 la)))
      (send
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3576)
  (parent 3230)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 2) (10 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((3 2) (6 0)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (8 0)) ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((9 2) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0)
    (trans 1 0) (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0)
    (neq self-0 b))
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-1 "nil")) (send l-1))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3577)
  (parent 3230)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (2 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0) (trans 1 0)
    (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3578)
  (parent 3230)
  (unrealized (0 3) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-1) (priv-stor priv-stor-1)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (5 0)) ((4 3) (9 0)) ((5 1) (4 1)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)) ((9 1) (0 3))
    ((10 1) (4 0)) ((10 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (2 0) (9 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv self-1 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 8 1) (trans 1 1) (trans 10 0) (trans 8 0)
    (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0)
    (neq self-0 self-1))
  (operation nonce-test (displaced 5 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig (body self-1 (exp (gen) l) (pubk "sig" self-1))
          (privk "sig" self-1)))
      (send (cat na self-0 self-1 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 self-1 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (stor priv-stor-1 (cat pt-2 "nil")) (send l-1))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3579)
  (parent 3230)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0)
    (trans 1 0) (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0)
    (neq self-0 b))
  (operation nonce-test (displaced 11 9 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3580)
  (parent 3230)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 9 1) (trans 5 1) (trans 1 1) (trans 11 0)
    (trans 9 0) (trans 5 0) (trans 1 0) (trans 8 1) (trans 3 1)
    (trans 8 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-1 l-2)))
      (send
        (sig (body self-1 (exp (gen) l-2) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3581)
  (parent 3230)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 4 1) (trans 9 0) (trans 4 0)
    (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3582)
  (parent 3231)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 4 1) (trans 9 0) (trans 4 0)
    (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3583)
  (parent 3231)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (9 0)) ((6 1) (11 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 4 1) (trans 9 0) (trans 4 0)
    (neq a b))
  (operation channel-test (displaced 12 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3584)
  (parent 3231)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 12 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 4 1)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3585)
  (parent 3231)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 4 1)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3586)
  (parent 3232)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 4 1)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3587)
  (parent 3232)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (9 0)) ((6 1) (11 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 4 1)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 12 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3588)
  (parent 3232)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 12 1) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 12 0) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 4 1) (trans 9 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 3589)
  (parent 3232)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1)
    (trans 4 1) (trans 11 0) (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1)))
  (label 3590)
  (parent 3233)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (2 0) (9 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (trans 10 1) (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0)
    (trans 1 0) (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0)
    (neq self-0 b))
  (operation nonce-test (displaced 11 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3591)
  (parent 3235)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 5 1) (trans 1 1)
    (trans 10 0) (trans 5 0) (trans 1 0) (trans 8 1) (trans 3 1)
    (trans 8 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3592)
  (parent 3235)
  (unrealized (11 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0))
    ((5 3) (0 3)) ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 4 1) (trans 9 0) (trans 4 0)
    (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3593)
  (parent 3239)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0))
    ((5 3) (0 3)) ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 4 1) (trans 9 0) (trans 4 0)
    (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3594)
  (parent 3239)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (11 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 4 1) (trans 9 0) (trans 4 0)
    (neq a b))
  (operation channel-test (displaced 12 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
      (send (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3595)
  (parent 3239)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 12 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 4 1)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3596)
  (parent 3239)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0))
    ((5 3) (0 3)) ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 4 1)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3597)
  (parent 3240)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0))
    ((5 3) (0 3)) ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 4 1)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3598)
  (parent 3240)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (11 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 4 1)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 12 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
      (send (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3599)
  (parent 3240)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 12 1) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 12 0) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 4 1) (trans 9 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 3600)
  (parent 3240)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (11 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 4 1)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-listener l-1) (rec l-1) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1)))
  (label 3601)
  (parent 3241)
  (unrealized (0 3) (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (5 0)) ((10 1) (11 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 2 1) (trans 1 1)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (added-listener l-1) (mul (rec l-1) (rec l-1))
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1)))
  (label 3602)
  (parent 3245)
  (unrealized (0 3) (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 2) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 3 1) (trans 9 0) (trans 7 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 3603)
  (parent 3247)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0)
    (neq a b))
  (operation nonce-test (displaced 12 5 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3604)
  (parent 3248)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3605)
  (parent 3248)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3606)
  (parent 3248)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 12 0 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3607)
  (parent 3248)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 12 1 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3608)
  (parent 3248)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3609)
  (parent 3248)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 2) (8 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)) ((11 1) (0 0)) ((11 1) (7 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 5 1) (trans 1 1) (trans 11 0) (trans 5 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 3 1) (trans 9 0)
    (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 2 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3610)
  (parent 3248)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (10 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)) ((11 1) (5 0)) ((11 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 4 1) (trans 9 0)
    (trans 7 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (displaced 6 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3611)
  (parent 3248)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3612)
  (parent 3248)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((2 1) (12 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3613)
  (parent 3249)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (12 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3614)
  (parent 3249)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (10 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 13 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3615)
  (parent 3249)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 3) (11 0)) ((13 1) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 13 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 3616)
  (parent 3249)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((2 1) (12 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3617)
  (parent 3250)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (12 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3618)
  (parent 3250)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (10 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 13 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3619)
  (parent 3250)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 13 1) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 13 0) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 3620)
  (parent 3250)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 12 4 ltx-disclose 3) l-1 (11 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 3621)
  (parent 3251)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (deflistener x)
  (precedes ((0 2) (12 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (11 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-listener x) (mul l-0 (rec x)) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))) ((recv x) (send x)))
  (label 3622)
  (parent 3252)
  (unrealized (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (displaced 12 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 3623)
  (parent 3254)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3624)
  (parent 3254)
  (unrealized (12 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0))
    ((5 3) (0 3)) ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0) (trans 4 0)
    (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3625)
  (parent 3257)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0))
    ((5 3) (0 3)) ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0) (trans 4 0)
    (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3626)
  (parent 3257)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (11 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 4 1) (trans 8 0) (trans 4 0)
    (neq a b))
  (operation channel-test (displaced 12 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
      (send (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 3627)
  (parent 3257)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (trans 12 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3628)
  (parent 3257)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0))
    ((5 3) (0 3)) ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3629)
  (parent 3258)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0))
    ((5 3) (0 3)) ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3630)
  (parent 3258)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (11 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 12 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
      (send (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3631)
  (parent 3258)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 12 1) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 12 0) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 8 1) (trans 4 1) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 3632)
  (parent 3258)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (11 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-listener l-1) (rec l-1) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1)))
  (label 3633)
  (parent 3259)
  (unrealized (0 3) (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (5 0)) ((10 1) (11 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 2 1) (trans 1 1)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 4 1)
    (trans 7 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (added-listener l-1) (mul (rec l-1) (rec l-1))
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1)))
  (label 3634)
  (parent 3263)
  (unrealized (0 3) (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 0)) ((3 2) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3635)
  (parent 3265)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l-0) (beta l) (a self-0) (b b) (x x) (upsilon upsilon)
        (na na) (nb nb) (priv-stor priv-stor-0) (pt pt-3) (self self)
        (priv-stor-0 priv-stor) (ignore ignore) (pt-0 pt) (pt-1 pt-0))))
  (origs (l-0 (3 1)) (pt-3 (3 1)) (pt-1 (2 1)) (pt-0 (1 1)) (na (0 2))
    (l (1 1))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((3 2) (5 0)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 3636)
  (parent 3266)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (3 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (0 0)) ((5 2) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 5 1) (trans 1 1)
    (trans 5 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3637)
  (parent 3267)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((1 2) (0 3)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 2) (0 1)) ((4 2) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1) (trans 4 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 3638)
  (parent 3268)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((3 2) (7 0)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 3 1) (trans 6 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 3639)
  (parent 3269)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (3 0)) ((1 2) (6 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1) (trans 5 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 3640)
  (parent 3270)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((9 2) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (2 0) (10 0))
  (gen-st (pv b l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3641)
  (parent 3272)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 1) (11 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((9 2) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3642)
  (parent 3272)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 1) (11 0)) ((9 2) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation channel-test (displaced 12 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self-0 b-0 (exp (gen) x-0)))))
  (label 3643)
  (parent 3272)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1) (pv a l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 9 1) (trans 4 1) (trans 1 1) (trans 12 0)
    (trans 9 0) (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a l-2)))))
  (label 3644)
  (parent 3272)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((9 2) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (2 0) (10 0))
  (gen-st (pv b l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3645)
  (parent 3276)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 1) (11 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((9 2) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3646)
  (parent 3276)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 1) (11 0)) ((9 2) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation channel-test (displaced 12 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3647)
  (parent 3276)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 12 1)
    (trans 9 1) (trans 4 1) (trans 1 1) (trans 12 0) (trans 9 0)
    (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq self-0 b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3648)
  (parent 3276)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation nonce-test (displaced 12 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 3649)
  (parent 3288)
  (seen 3114)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1)
    (trans 7 1) (trans 11 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3650)
  (parent 3288)
  (unrealized (12 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 10 1) (trans 4 1)
    (trans 1 1) (trans 10 0) (trans 4 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3651)
  (parent 3290)
  (seen 3117)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3652)
  (parent 3290)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3653)
  (parent 3290)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) la) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test (displaced 12 0 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self-0 b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) la l-0))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) la) l-0)) (send (cat (exp (gen) la) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 la)))
      (send
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3654)
  (parent 3290)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test (displaced 12 1 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3655)
  (parent 3290)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3656)
  (parent 3290)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-1) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 2) (8 0))
    ((10 1) (0 3)) ((11 1) (3 0)) ((11 1) (5 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (2 0) (8 0) (10 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv self-1 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 9 1) (trans 1 1) (trans 11 0) (trans 9 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 self-1))
  (operation nonce-test (displaced 4 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig (body self-1 (exp (gen) l) (pubk "sig" self-1))
          (privk "sig" self-1)))
      (send (cat na self-0 self-1 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 self-1 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3657)
  (parent 3290)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test (displaced 12 10 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3658)
  (parent 3290)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 10 1) (trans 4 1) (trans 1 1) (trans 12 0)
    (trans 10 0) (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-1 l-2)))
      (send
        (sig (body self-1 (exp (gen) l-2) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3659)
  (parent 3290)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (10 0))
    ((5 1) (4 0)) ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3660)
  (parent 3292)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 1) (0 0)) ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (10 0))
    ((5 1) (4 0)) ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3661)
  (parent 3292)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3662)
  (parent 3292)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 3663)
  (parent 3292)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (10 0))
    ((5 1) (4 0)) ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3664)
  (parent 3296)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 1) (0 0)) ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (10 0))
    ((5 1) (4 0)) ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3665)
  (parent 3296)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3666)
  (parent 3296)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 12 1)
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 12 0) (trans 5 0)
    (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3667)
  (parent 3296)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation nonce-test (displaced 12 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 3668)
  (parent 3308)
  (seen 3136)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1)
    (trans 7 1) (trans 11 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3669)
  (parent 3308)
  (unrealized (12 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 12 4 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3670)
  (parent 3310)
  (seen 3139)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3671)
  (parent 3310)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3672)
  (parent 3310)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test (displaced 12 0 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3673)
  (parent 3310)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test (displaced 12 1 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3674)
  (parent 3310)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3675)
  (parent 3310)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (2 0)) ((1 2) (9 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 3)) ((11 1) (0 0)) ((11 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0) (10 0))
  (gen-st (pv b l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 1 1) (trans 11 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test (displaced 2 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3676)
  (parent 3310)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 3)) ((11 1) (4 0)) ((11 1) (6 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (10 0))
  (gen-st (pv a l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a self-0))
  (operation nonce-test (displaced 5 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3677)
  (parent 3310)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3678)
  (parent 3310)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((9 2) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (2 0) (10 0))
  (gen-st (pv b l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3679)
  (parent 3312)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 1) (11 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((9 2) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3680)
  (parent 3312)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 1) (11 0)) ((9 2) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (2 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation channel-test (displaced 12 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self-0 b-0 (exp (gen) x-0)))))
  (label 3681)
  (parent 3312)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (2 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1) (pv a l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 9 1) (trans 4 1) (trans 1 1) (trans 12 0)
    (trans 9 0) (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a l-2)))))
  (label 3682)
  (parent 3312)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((9 2) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (2 0) (10 0))
  (gen-st (pv b l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3683)
  (parent 3316)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 1) (11 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((9 2) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3684)
  (parent 3316)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 1) (11 0)) ((9 2) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (2 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation channel-test (displaced 12 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3685)
  (parent 3316)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (2 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 12 1)
    (trans 9 1) (trans 4 1) (trans 1 1) (trans 12 0) (trans 9 0)
    (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq self-0 b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3686)
  (parent 3316)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation nonce-test (displaced 12 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 3687)
  (parent 3328)
  (seen 3158)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1)
    (trans 7 1) (trans 11 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3688)
  (parent 3328)
  (unrealized (12 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 10 1) (trans 4 1)
    (trans 1 1) (trans 10 0) (trans 4 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3689)
  (parent 3330)
  (seen 3161)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3690)
  (parent 3330)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3691)
  (parent 3330)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) la) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test (displaced 12 0 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self-0 b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) la l-0))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) la) l)) (send (cat (exp (gen) la) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 la)))
      (send
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3692)
  (parent 3330)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test (displaced 12 1 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3693)
  (parent 3330)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3694)
  (parent 3330)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-1) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 2) (8 0))
    ((10 1) (0 3)) ((11 1) (3 0)) ((11 1) (7 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (2 0) (8 0) (10 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv self-1 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 9 1) (trans 1 1) (trans 11 0) (trans 9 0)
    (trans 1 0) (trans 7 1) (trans 5 1) (trans 7 0) (trans 5 0)
    (neq self-0 self-1))
  (operation nonce-test (displaced 4 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig (body self-1 (exp (gen) l) (pubk "sig" self-1))
          (privk "sig" self-1)))
      (send (cat na self-0 self-1 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 self-1 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (stor priv-stor-1 (cat pt-2 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3695)
  (parent 3330)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test (displaced 12 10 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3696)
  (parent 3330)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 10 1) (trans 4 1) (trans 1 1) (trans 12 0)
    (trans 10 0) (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-1 l-2)))
      (send
        (sig (body self-1 (exp (gen) l-2) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3697)
  (parent 3330)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 9 1)
    (trans 7 1) (trans 12 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (11 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 3698)
  (parent 3339)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 2 1) (trans 1 1)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 8 1)
    (trans 6 1) (trans 12 0) (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (11 0)
    (ch-msg priv-stor-1 (cat pt-6 (pv self-0 l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 3699)
  (parent 3340)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (10 0))
    ((5 1) (4 0)) ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3700)
  (parent 3342)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (10 0))
    ((5 1) (4 0)) ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3701)
  (parent 3342)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3702)
  (parent 3342)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 3703)
  (parent 3342)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (10 0))
    ((5 1) (4 0)) ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3704)
  (parent 3346)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (10 0))
    ((5 1) (4 0)) ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3705)
  (parent 3346)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3706)
  (parent 3346)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 12 1)
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 12 0) (trans 5 0)
    (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3707)
  (parent 3346)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation nonce-test (displaced 12 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 3708)
  (parent 3358)
  (seen 3182)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1)
    (trans 7 1) (trans 11 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3709)
  (parent 3358)
  (unrealized (12 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 12 4 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3710)
  (parent 3360)
  (seen 3185)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3711)
  (parent 3360)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3712)
  (parent 3360)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test (displaced 12 0 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3713)
  (parent 3360)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test (displaced 12 1 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3714)
  (parent 3360)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3715)
  (parent 3360)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (2 0)) ((1 2) (9 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 3)) ((11 1) (0 0)) ((11 1) (6 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0) (10 0))
  (gen-st (pv b l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 4 1) (trans 1 1) (trans 11 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation nonce-test (displaced 2 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3716)
  (parent 3360)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 3)) ((11 1) (4 0)) ((11 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (10 0))
  (gen-st (pv a l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 2 1) (trans 1 1) (trans 11 0) (trans 2 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq a self-0))
  (operation nonce-test (displaced 5 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3717)
  (parent 3360)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 12 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3718)
  (parent 3360)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 9 1)
    (trans 7 1) (trans 12 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (11 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 3719)
  (parent 3369)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 2 1) (trans 1 1)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 8 1)
    (trans 6 1) (trans 12 0) (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (11 0)
    (ch-msg priv-stor-1 (cat pt-6 (pv self-0 l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 3720)
  (parent 3370)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 2) (4 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (3 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 3721)
  (parent 3371)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 1) (3 0)) ((3 2) (7 0)) ((4 1) (7 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (4 0) (7 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 3 1) (trans 6 0)
    (trans 3 0) (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3722)
  (parent 3372)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 2) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 3 1) (trans 3 0) (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 3723)
  (parent 3379)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((1 2) (3 0)) ((2 1) (5 0)) ((3 1) (5 0)) ((4 2) (3 0))
    ((5 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (5 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3724)
  (parent 3380)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 5 1) (trans 3 1) (trans 5 0) (trans 3 0)
    (neq self b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)) ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb)))
  (label 3725)
  (parent 3381)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1))
    ((2 1) (6 0)) ((3 2) (7 0)) ((4 1) (7 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (4 0) (7 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 3 1) (trans 6 0)
    (trans 3 0) (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3726)
  (parent 3382)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3))
    ((3 1) (5 0)) ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (gen-st (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 1 1) (trans 3 0)
    (trans 1 0) (trans 5 1) (trans 2 1) (trans 5 0) (trans 2 0)
    (neq self self-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb)))
  (label 3727)
  (parent 3389)
  (seen 3725)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 2) (7 0)) ((3 1) (7 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (7 0))
  (gen-st (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (trans 6 1) (trans 2 1) (trans 6 0)
    (trans 2 0) (neq self self-0))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3728)
  (parent 3390)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((1 1) (11 0))
    ((1 2) (2 0)) ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (x-0 beta)
    (x-0 l-0))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (trans 9 1)
    (trans 6 1) (trans 9 0) (trans 6 0) (neq self b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a l-1))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3729)
  (parent 3398)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (x-0 beta)
    (x-0 l-0))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (trans 9 1)
    (trans 6 1) (trans 9 0) (trans 6 0) (neq self b))
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a l-1))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3730)
  (parent 3398)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (x-0 beta)
    (x-0 l-1))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 4 1) (trans 1 1) (trans 12 0) (trans 4 0)
    (trans 1 0) (trans 9 1) (trans 6 1) (trans 9 0) (trans 6 0)
    (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a l-1))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-5 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv a l-1)))))
  (label 3731)
  (parent 3398)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((1 1) (11 0))
    ((1 2) (2 0)) ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (y-0 alpha)
    (y-0 zeta) (y-0 l-0))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (trans 9 1) (trans 6 1)
    (trans 9 0) (trans 6 0) (neq self b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b-0 l-1))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 3732)
  (parent 3402)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (y-0 alpha)
    (y-0 zeta) (y-0 l-0))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (trans 9 1) (trans 6 1)
    (trans 9 0) (trans 6 0) (neq self b))
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b-0 l-1))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 3733)
  (parent 3402)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (y-0 alpha)
    (y-0 zeta) (y-0 l-1))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 12 1)
    (trans 4 1) (trans 1 1) (trans 12 0) (trans 4 0) (trans 1 0)
    (trans 9 1) (trans 6 1) (trans 9 0) (trans 6 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b-0 l-1))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-5 (pv b-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv b-0 l-1)))))
  (label 3734)
  (parent 3402)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (11 0))
    ((1 2) (2 0)) ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (9 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (x-0 beta)
    (x-0 l-0))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (trans 9 1)
    (trans 6 1) (trans 9 0) (trans 6 0) (neq self b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a l-1))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (rec l))) (send (cat (gen) (rec l))))
    ((recv l) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3735)
  (parent 3412)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (9 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (x-0 beta)
    (x-0 l-0))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 1 1) (trans 4 0) (trans 1 0) (trans 9 1)
    (trans 6 1) (trans 9 0) (trans 6 0) (neq self b))
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a l-1))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3736)
  (parent 3412)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (9 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (x-0 beta)
    (x-0 l-1))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 4 1) (trans 1 1) (trans 12 0) (trans 4 0)
    (trans 1 0) (trans 9 1) (trans 6 1) (trans 9 0) (trans 6 0)
    (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a l-1))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-5 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv a l-1)))))
  (label 3737)
  (parent 3412)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (11 0))
    ((1 2) (2 0)) ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0))
    ((4 1) (9 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (y-0 alpha)
    (y-0 zeta) (y-0 l-0))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (trans 9 1) (trans 6 1)
    (trans 9 0) (trans 6 0) (neq self b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b-0 l-1))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (gen) (rec l))) (send (cat (gen) (rec l))))
    ((recv l) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 3738)
  (parent 3416)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (alpha zeta expt) (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (9 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (y-0 alpha)
    (y-0 zeta) (y-0 l-0))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 1 1) (trans 4 0) (trans 1 0) (trans 9 1) (trans 6 1)
    (trans 9 0) (trans 6 0) (neq self b))
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b-0 l-1))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 3739)
  (parent 3416)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (9 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (y-0 alpha)
    (y-0 zeta) (y-0 l-1))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 12 1)
    (trans 4 1) (trans 1 1) (trans 12 0) (trans 4 0) (trans 1 0)
    (trans 9 1) (trans 6 1) (trans 9 0) (trans 6 0) (neq self b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b-0 l-1))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-5 (pv b-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv b-0 l-1)))))
  (label 3740)
  (parent 3416)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) (rec l))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (11 0))
    ((1 2) (2 0)) ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (3 0)) ((7 1) (9 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3))
    ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (x-0 beta)
    (x-0 l-0))
  (precur (2 0) (6 0) (10 0))
  (gen-st (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 7 1) (trans 1 1) (trans 7 0) (trans 1 0) (trans 9 1)
    (trans 5 1) (trans 9 0) (trans 5 0) (neq self self-0))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a l-1))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (mul (rec l) (rec l))))
      (send (cat (exp (gen) l) (mul (rec l) (rec l)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l) (send l))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 3741)
  (parent 3420)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l rndx) (beta expt) (x y x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((7 1) (9 0)) ((7 1) (11 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3))
    ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (x-0 beta)
    (x-0 l-0))
  (precur (2 0) (6 0) (10 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 7 1) (trans 1 1) (trans 7 0) (trans 1 0) (trans 9 1)
    (trans 5 1) (trans 9 0) (trans 5 0) (neq self self-0))
  (operation channel-test (displaced 12 7 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a l-1))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self-0 b (exp (gen) x-0)))))
  (label 3742)
  (parent 3420)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (self self-0 a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((7 1) (9 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0))
    ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (x-0 beta)
    (x-0 l-1))
  (precur (2 0) (6 0) (10 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 7 1) (trans 1 1) (trans 12 0) (trans 7 0)
    (trans 1 0) (trans 9 1) (trans 5 1) (trans 9 0) (trans 5 0)
    (neq self self-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a l-1))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-5 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv a l-1)))))
  (label 3743)
  (parent 3420)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 locn) (l rndx) (alpha zeta expt)
    (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) (rec l))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (11 0))
    ((1 2) (2 0)) ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (3 0)) ((7 1) (9 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3))
    ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y (one)) (y (mul (rec l) l-0 l-0)) (y l) (y-0 alpha)
    (y-0 zeta) (y-0 l-0))
  (precur (2 0) (6 0) (10 0))
  (gen-st (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 7 1)
    (trans 1 1) (trans 7 0) (trans 1 0) (trans 9 1) (trans 5 1)
    (trans 9 0) (trans 5 0) (neq self self-0))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b l-1))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-0 l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (mul (rec l) (rec l))))
      (send (cat (exp (gen) l) (mul (rec l) (rec l)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l) (send l))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 3744)
  (parent 3424)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 locn) (l rndx) (alpha zeta expt)
    (x y y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((7 1) (9 0)) ((7 1) (11 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3))
    ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (y-0 alpha)
    (y-0 zeta) (y-0 l-0))
  (precur (2 0) (6 0) (10 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 7 1)
    (trans 1 1) (trans 7 0) (trans 1 0) (trans 9 1) (trans 5 1)
    (trans 9 0) (trans 5 0) (neq self self-0))
  (operation channel-test (displaced 12 7 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b l-1))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self-0
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y-0))))))))
  (label 3745)
  (parent 3424)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (self self-0 a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((7 1) (9 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0))
    ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y (one)) (y (mul l l (rec l-0))) (y l-0) (y-0 alpha)
    (y-0 zeta) (y-0 l-1))
  (precur (2 0) (6 0) (10 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 12 1)
    (trans 7 1) (trans 1 1) (trans 12 0) (trans 7 0) (trans 1 0)
    (trans 9 1) (trans 5 1) (trans 9 0) (trans 5 0) (neq self self-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b l-1))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-5 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv b l-1)))))
  (label 3746)
  (parent 3424)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a self))
  (operation generalization weakened ((3 1) (4 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 3747)
  (parent 3427)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (0 3))
    ((4 1) (0 0)) ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0)
    (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3748)
  (parent 3428)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 1) (6 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((5 2) (3 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3749)
  (parent 3429)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (4 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 4 1) (trans 8 0) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 3750)
  (parent 3430)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (5 0)) ((4 1) (10 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 7 1) (trans 5 1) (trans 9 0) (trans 7 0) (trans 5 0)
    (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3751)
  (parent 3431)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((2 1) (12 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3752)
  (parent 3432)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 1) (12 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3753)
  (parent 3432)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (12 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3754)
  (parent 3432)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0))
    ((13 1) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 3755)
  (parent 3432)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((2 1) (12 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3756)
  (parent 3436)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 1) (12 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3757)
  (parent 3436)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (12 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3758)
  (parent 3436)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0))
    ((13 1) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 13 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 13 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1)
    (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 3759)
  (parent 3436)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (0 0)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3760)
  (parent 3440)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (3 0))
    ((3 2) (8 0)) ((4 1) (8 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (8 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 7 1) (trans 2 1)
    (trans 1 1) (trans 7 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3761)
  (parent 3441)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (0 3))
    ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l)))
  (label 3762)
  (parent 3450)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 1) (6 0)) ((4 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3763)
  (parent 3451)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (4 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 4 1) (trans 8 0) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 3764)
  (parent 3452)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (9 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (5 0)) ((4 1) (10 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 7 1) (trans 5 1) (trans 9 0) (trans 7 0) (trans 5 0)
    (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3765)
  (parent 3453)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((2 1) (12 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3766)
  (parent 3454)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 1) (12 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (8 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3767)
  (parent 3454)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (12 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3768)
  (parent 3454)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0))
    ((13 1) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 3769)
  (parent 3454)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((2 1) (12 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3770)
  (parent 3458)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 1) (12 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (8 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3771)
  (parent 3458)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (12 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3772)
  (parent 3458)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0))
    ((13 1) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 13 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 13 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1)
    (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 3773)
  (parent 3458)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 1) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 3774)
  (parent 3462)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (4 0)) ((4 2) (8 0)) ((5 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3775)
  (parent 3463)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 2) (6 0)) ((4 2) (6 0)) ((5 1) (0 0)) ((5 2) (6 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 3 1) (trans 4 0) (trans 3 0) (neq self-0 b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3776)
  (parent 3473)
  (seen 4028)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 4 1) (trans 8 0) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 3777)
  (parent 3474)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (7 0)) ((4 1) (10 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 7 1) (trans 5 1) (trans 9 0) (trans 7 0) (trans 5 0)
    (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3778)
  (parent 3475)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((2 1) (12 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3779)
  (parent 3476)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 1) (12 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3780)
  (parent 3476)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (12 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3781)
  (parent 3476)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0))
    ((13 1) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 3782)
  (parent 3476)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((2 1) (12 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3783)
  (parent 3480)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 1) (12 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3784)
  (parent 3480)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (12 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3785)
  (parent 3480)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0))
    ((13 1) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 13 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 13 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1)
    (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 3786)
  (parent 3480)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0)) ((3 2) (0 3))
    ((4 2) (0 3)) ((5 1) (0 3)) ((6 1) (0 0)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1)
    (trans 4 0) (trans 3 0) (neq self-0 b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3787)
  (parent 3484)
  (seen 4031)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (8 0)) ((4 1) (8 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (8 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 7 1) (trans 2 1)
    (trans 1 1) (trans 7 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3788)
  (parent 3485)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (10 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 7 1)
    (trans 5 1) (trans 10 0) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 3789)
  (parent 3494)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 11 1)
    (trans 8 1) (trans 6 1) (trans 11 0) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 13 3 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3790)
  (parent 3495)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3791)
  (parent 3495)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3792)
  (parent 3495)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (gen) (rec l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0) (12 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 13 0 init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) la))
      (send (cat (exp (gen) (rec l)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (gen) (rec l))) (send (cat (gen) (rec l))))
    ((recv l) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3793)
  (parent 3495)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 13 1 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3794)
  (parent 3495)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3795)
  (parent 3495)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (11 0))
    ((3 1) (2 0)) ((3 1) (10 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 3)) ((12 1) (0 0)) ((12 1) (7 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 3 1) (trans 1 1) (trans 12 0) (trans 3 0)
    (trans 1 0) (trans 10 1) (trans 7 1) (trans 5 1) (trans 10 0)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 2 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3796)
  (parent 3495)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (11 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 3)) ((12 1) (3 0)) ((12 1) (10 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (11 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 2 1) (trans 1 1) (trans 12 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 7 1) (trans 5 1) (trans 10 0)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (displaced 4 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3797)
  (parent 3495)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1)
    (trans 6 1) (trans 11 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3798)
  (parent 3495)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (10 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (trans 8 1) (trans 2 1) (trans 1 1)
    (trans 8 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 6 1)
    (trans 4 1) (trans 10 0) (trans 6 0) (trans 4 0) (neq a self-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 3799)
  (parent 3496)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 9 1) (trans 2 1)
    (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0) (trans 11 1)
    (trans 7 1) (trans 5 1) (trans 11 0) (trans 7 0) (trans 5 0)
    (neq a self-0))
  (operation nonce-test (displaced 13 3 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self-0 (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3800)
  (parent 3497)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self self-0 a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0)))))
  (label 3801)
  (parent 3497)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3802)
  (parent 3497)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l) (mul (rec l) (rec l))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (8 0) (12 0))
  (gen-st (pv a la) (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (displaced 13 0 init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self-0
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) la))
      (send (cat (exp (gen) (rec l)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-4 "nil")) (send la))
    ((recv (cat (exp (gen) l) (mul (rec l) (rec l))))
      (send (cat (exp (gen) l) (mul (rec l) (rec l)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3803)
  (parent 3497)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (displaced 13 1 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3804)
  (parent 3497)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0) (trans 7 0)
    (trans 5 0) (neq a self-0))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3805)
  (parent 3497)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-1) (b self-0)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-1) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-1) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 3) (11 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (2 0)) ((8 1) (10 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 1) (0 0))
    ((12 1) (6 0)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (7 0) (11 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv self-1 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 8 1) (trans 1 1) (trans 12 0) (trans 8 0)
    (trans 1 0) (trans 10 1) (trans 6 1) (trans 4 1) (trans 10 0)
    (trans 6 0) (trans 4 0) (neq self-1 self-0))
  (operation nonce-test (displaced 2 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-1 l-1)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self-1 self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-1 self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig
          (body self-1 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-1)) (privk "sig" self-1)))
      (recv (cat na self-1 self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-1 self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-1 l-1)))
      (stor priv-stor-1 (cat pt-2 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3806)
  (parent 3497)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (displaced 13 9 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3807)
  (parent 3497)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 9 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 7 1)
    (trans 5 1) (trans 11 0) (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-1 l-2)))
      (send
        (sig (body self-1 (exp (gen) l-2) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3808)
  (parent 3497)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 2) (6 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 4 1) (trans 5 0) (trans 4 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3809)
  (parent 3499)
  (seen 4034)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 4 1) (trans 8 0) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 3810)
  (parent 3500)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (9 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 1) (7 0)) ((4 1) (10 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 7 1) (trans 5 1) (trans 9 0) (trans 7 0) (trans 5 0)
    (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3811)
  (parent 3501)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((2 1) (12 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3812)
  (parent 3502)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 1) (12 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (6 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3813)
  (parent 3502)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (12 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3814)
  (parent 3502)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0))
    ((13 1) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 6 1) (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 3815)
  (parent 3502)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((2 1) (12 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3816)
  (parent 3506)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 1) (12 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (6 0)) ((3 3) (11 0)) ((4 1) (3 0))
    ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3817)
  (parent 3506)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (12 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3818)
  (parent 3506)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0))
    ((13 1) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 13 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 13 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1)
    (trans 10 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 3819)
  (parent 3506)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 2) (0 3)) ((5 2) (0 3)) ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 4 1)
    (trans 5 0) (trans 4 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 3820)
  (parent 3510)
  (seen 4037)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 2) (8 0)) ((5 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3821)
  (parent 3511)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 1) (10 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 7 1)
    (trans 5 1) (trans 10 0) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 3822)
  (parent 3520)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 11 1)
    (trans 8 1) (trans 6 1) (trans 11 0) (trans 8 0) (trans 6 0)
    (neq a b))
  (operation nonce-test (displaced 13 3 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3823)
  (parent 3521)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3824)
  (parent 3521)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3825)
  (parent 3521)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0) (12 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 13 0 init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l))) (send (cat (gen) (rec l))))
    ((recv l) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3826)
  (parent 3521)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (displaced 13 1 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3827)
  (parent 3521)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3828)
  (parent 3521)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (11 0))
    ((3 1) (2 0)) ((3 1) (10 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 3)) ((12 1) (0 0)) ((12 1) (5 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 3 1) (trans 1 1) (trans 12 0) (trans 3 0)
    (trans 1 0) (trans 10 1) (trans 7 1) (trans 5 1) (trans 10 0)
    (trans 7 0) (trans 5 0) (neq self-0 b))
  (operation nonce-test (displaced 2 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3829)
  (parent 3521)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (11 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 3)) ((12 1) (3 0)) ((12 1) (10 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (11 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 2 1) (trans 1 1) (trans 12 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 7 1) (trans 5 1) (trans 10 0)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (displaced 4 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3830)
  (parent 3521)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1)
    (trans 6 1) (trans 11 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3831)
  (parent 3521)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (10 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (trans 8 1) (trans 2 1) (trans 1 1)
    (trans 8 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 6 1)
    (trans 4 1) (trans 10 0) (trans 6 0) (trans 4 0) (neq a self-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 3832)
  (parent 3522)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 9 1) (trans 2 1)
    (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0) (trans 11 1)
    (trans 7 1) (trans 5 1) (trans 11 0) (trans 7 0) (trans 5 0)
    (neq a self-0))
  (operation nonce-test (displaced 13 3 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self-0 (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3833)
  (parent 3523)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self self-0 a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0)))))
  (label 3834)
  (parent 3523)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3835)
  (parent 3523)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) (rec l))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (8 0) (12 0))
  (gen-st (pv a la) (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (displaced 13 0 init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self-0
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-3 "nil")) (send la))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (mul (rec l) (rec l))))
      (send (cat (exp (gen) l) (mul (rec l) (rec l)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3836)
  (parent 3523)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (displaced 13 1 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3837)
  (parent 3523)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0) (trans 7 0)
    (trans 5 0) (neq a self-0))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3838)
  (parent 3523)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-1) (b self-0)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-1) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self-1) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 3) (11 0))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (2 0)) ((8 1) (10 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 1) (0 0))
    ((12 1) (4 0)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (7 0) (11 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv self-1 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 8 1) (trans 1 1) (trans 12 0) (trans 8 0)
    (trans 1 0) (trans 10 1) (trans 6 1) (trans 4 1) (trans 10 0)
    (trans 6 0) (trans 4 0) (neq self-1 self-0))
  (operation nonce-test (displaced 2 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-1 l-1)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self-1 self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-1 self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig
          (body self-1 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-1)) (privk "sig" self-1)))
      (recv (cat na self-1 self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-1 self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-1 l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3839)
  (parent 3523)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (displaced 13 9 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3840)
  (parent 3523)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 9 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 7 1)
    (trans 5 1) (trans 11 0) (trans 7 0) (trans 5 0) (neq a self-0))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-1 l-2)))
      (send
        (sig (body self-1 (exp (gen) l-2) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3841)
  (parent 3523)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0)) ((3 2) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 3842)
  (parent 3524)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l-0) (beta l) (a a) (b b) (x x) (upsilon upsilon) (na na)
        (nb nb) (priv-stor priv-stor) (pt pt) (self self)
        (priv-stor-0 priv-stor-0) (ignore ignore) (pt-0 pt-0)
        (pt-1 pt-1))))
  (origs (pt-3 (3 1)) (l (1 1)) (pt-1 (1 1)) (l-0 (2 1)) (pt (2 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (4 0)) ((3 1) (0 0))
    ((3 1) (4 0)) ((3 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 self))
  (operation generalization weakened ((0 2) (4 0)))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 3843)
  (parent 3526)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((4 2) (0 3)))
  (uniq-orig na lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 4 1) (trans 3 1) (trans 4 0) (trans 3 0)
    (neq a self))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 3844)
  (parent 3527)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 2) (0 1)) ((4 2) (5 0)) ((5 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1) (trans 4 0)
    (trans 3 0) (neq a self))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3845)
  (parent 3528)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (6 0)) ((3 1) (5 0))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 7 1) (trans 3 1) (trans 1 1)
    (trans 7 0) (trans 3 0) (trans 1 0) (trans 5 1) (trans 2 1)
    (trans 5 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3846)
  (parent 3529)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (7 0))
    ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 8 1) (trans 4 1)
    (trans 1 1) (trans 8 0) (trans 4 0) (trans 1 0) (trans 6 1)
    (trans 2 1) (trans 6 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3847)
  (parent 3530)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (2 0) (10 0))
  (gen-st (pv b l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0)
    (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0)
    (neq self-0 b))
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3848)
  (parent 3531)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (11 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((3 2) (8 0)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0)
    (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0)
    (neq self-0 b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-1 "nil")) (send l-1))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3849)
  (parent 3531)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0)
    (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0)
    (neq self-0 b))
  (operation channel-test (displaced 12 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self-0 b-0 (exp (gen) x-0)))))
  (label 3850)
  (parent 3531)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1) (pv a l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 9 1) (trans 5 1) (trans 1 1) (trans 12 0)
    (trans 9 0) (trans 5 0) (trans 1 0) (trans 7 1) (trans 3 1)
    (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a l-2)))))
  (label 3851)
  (parent 3531)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (2 0) (10 0))
  (gen-st (pv b l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0) (trans 1 0)
    (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3852)
  (parent 3535)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (11 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((3 2) (8 0)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (0 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0) (trans 1 0)
    (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-1 "nil")) (send l-1))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3853)
  (parent 3535)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0) (trans 1 0)
    (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation channel-test (displaced 12 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3854)
  (parent 3535)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 12 1)
    (trans 9 1) (trans 5 1) (trans 1 1) (trans 12 0) (trans 9 0)
    (trans 5 0) (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0)
    (trans 3 0) (neq self-0 b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3855)
  (parent 3535)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 4 1) (trans 11 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (displaced 12 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 3856)
  (parent 3547)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1)
    (trans 4 1) (trans 11 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3857)
  (parent 3547)
  (unrealized (12 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (7 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 9 1)
    (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0) (trans 1 0)
    (trans 6 1) (trans 2 1) (trans 6 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3858)
  (parent 3548)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 10 1) (trans 5 1)
    (trans 1 1) (trans 10 0) (trans 5 0) (trans 1 0) (trans 7 1)
    (trans 3 1) (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 12 4 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3859)
  (parent 3549)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0)
    (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0)
    (neq self-0 b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3860)
  (parent 3549)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0)
    (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0)
    (neq self-0 b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3861)
  (parent 3549)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) la) l-0))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0)
    (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0)
    (neq self-0 b))
  (operation nonce-test (displaced 12 0 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self-0 b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) la l-0))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) la) l-0)) (send (cat (exp (gen) la) l-0)))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 la)))
      (send
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3862)
  (parent 3549)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 2) (11 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((3 2) (8 0)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (8 0)) ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0)
    (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0)
    (neq self-0 b))
  (operation nonce-test (displaced 12 1 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-1 "nil")) (send l-1))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3863)
  (parent 3549)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0) (trans 1 0)
    (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3864)
  (parent 3549)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-1) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (7 0)) ((4 3) (10 0)) ((5 1) (4 1)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (7 0)) ((9 1) (0 0)) ((9 2) (8 0))
    ((10 1) (0 3)) ((11 1) (4 0)) ((11 1) (6 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (2 0) (8 0) (10 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv self-1 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 9 1) (trans 1 1) (trans 11 0) (trans 9 0)
    (trans 1 0) (trans 6 1) (trans 3 1) (trans 6 0) (trans 3 0)
    (neq self-0 self-1))
  (operation nonce-test (displaced 5 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig (body self-1 (exp (gen) l) (pubk "sig" self-1))
          (privk "sig" self-1)))
      (send (cat na self-0 self-1 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 self-1 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (stor priv-stor-1 (cat pt-2 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3865)
  (parent 3549)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0)
    (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0)
    (neq self-0 b))
  (operation nonce-test (displaced 12 10 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3866)
  (parent 3549)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 10 1) (trans 5 1) (trans 1 1) (trans 12 0)
    (trans 10 0) (trans 5 0) (trans 1 0) (trans 7 1) (trans 3 1)
    (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-1 l-2)))
      (send
        (sig (body self-1 (exp (gen) l-2) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3867)
  (parent 3549)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((1 2) (7 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 2) (0 1)) ((4 1) (6 0)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 3 1) (trans 8 0) (trans 6 0) (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l)))
  (label 3868)
  (parent 3550)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 2) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 7 1) (trans 3 1) (trans 9 0) (trans 7 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3869)
  (parent 3551)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((2 1) (12 0)) ((3 1) (0 1)) ((4 2) (3 0))
    ((5 3) (11 0)) ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3870)
  (parent 3552)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (12 0)) ((1 2) (9 0))
    ((2 1) (0 0)) ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0))
    ((5 3) (11 0)) ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3871)
  (parent 3552)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((6 1) (12 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 13 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3872)
  (parent 3552)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0))
    ((13 1) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 3873)
  (parent 3552)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((2 1) (12 0)) ((3 1) (0 1)) ((4 2) (3 0))
    ((5 3) (11 0)) ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3874)
  (parent 3556)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (12 0)) ((1 2) (9 0))
    ((2 1) (0 0)) ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0))
    ((5 3) (11 0)) ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3875)
  (parent 3556)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((6 1) (12 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 13 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3876)
  (parent 3556)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0))
    ((13 1) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 13 1)
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 13 0) (trans 6 0)
    (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1)
    (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 3877)
  (parent 3556)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (displaced 12 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 3878)
  (parent 3568)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3879)
  (parent 3568)
  (unrealized (12 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 2) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 3 1) (trans 9 0) (trans 7 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 3880)
  (parent 3570)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0)
    (neq a b))
  (operation nonce-test (displaced 13 5 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3881)
  (parent 3571)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3882)
  (parent 3571)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3883)
  (parent 3571)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 13 0 init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-7 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3884)
  (parent 3571)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 13 1 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3885)
  (parent 3571)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3))
    ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3886)
  (parent 3571)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 2) (10 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 1) (0 3)) ((12 1) (0 0)) ((12 1) (9 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (10 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 5 1) (trans 1 1) (trans 12 0) (trans 5 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 3 1) (trans 9 0)
    (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 2 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3887)
  (parent 3571)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 1) (0 3)) ((12 1) (5 0)) ((12 1) (7 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 2 1) (trans 1 1) (trans 12 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 4 1) (trans 9 0)
    (trans 7 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (displaced 6 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3888)
  (parent 3571)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3889)
  (parent 3571)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (4 0)) ((3 1) (6 0))
    ((4 1) (0 3)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 0))
    ((7 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 7 1) (trans 3 1) (trans 1 1)
    (trans 7 0) (trans 3 0) (trans 1 0) (trans 6 1) (trans 2 1)
    (trans 6 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3890)
  (parent 3572)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (7 0)) ((5 1) (3 1))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 8 1) (trans 4 1)
    (trans 1 1) (trans 8 0) (trans 4 0) (trans 1 0) (trans 7 1)
    (trans 2 1) (trans 7 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3891)
  (parent 3573)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((9 2) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (2 0) (10 0))
  (gen-st (pv b l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0)
    (trans 1 0) (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0)
    (neq self-0 b))
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3892)
  (parent 3574)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (11 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((3 2) (6 0)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (8 0)) ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((9 2) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0)
    (trans 1 0) (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0)
    (neq self-0 b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-1 "nil")) (send l-1))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3893)
  (parent 3574)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 1) (11 0)) ((9 2) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (2 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0)
    (trans 1 0) (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0)
    (neq self-0 b))
  (operation channel-test (displaced 12 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self-0 b-0 (exp (gen) x-0)))))
  (label 3894)
  (parent 3574)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (2 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1) (pv a l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 9 1) (trans 5 1) (trans 1 1) (trans 12 0)
    (trans 9 0) (trans 5 0) (trans 1 0) (trans 8 1) (trans 3 1)
    (trans 8 0) (trans 3 0) (neq self-0 b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a l-2)))))
  (label 3895)
  (parent 3574)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((9 2) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (2 0) (10 0))
  (gen-st (pv b l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0) (trans 1 0)
    (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0) (neq self-0 b))
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3896)
  (parent 3578)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (11 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((3 2) (6 0)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (8 0)) ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((9 2) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0) (trans 1 0)
    (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0) (neq self-0 b))
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-1 "nil")) (send l-1))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3897)
  (parent 3578)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 1) (11 0)) ((9 2) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (2 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 5 1) (trans 1 1) (trans 9 0) (trans 5 0) (trans 1 0)
    (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0) (neq self-0 b))
  (operation channel-test (displaced 12 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3898)
  (parent 3578)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (2 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 12 1)
    (trans 9 1) (trans 5 1) (trans 1 1) (trans 12 0) (trans 9 0)
    (trans 5 0) (trans 1 0) (trans 8 1) (trans 3 1) (trans 8 0)
    (trans 3 0) (neq self-0 b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3899)
  (parent 3578)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 4 1) (trans 11 0) (trans 9 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (displaced 12 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 3900)
  (parent 3590)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1)
    (trans 4 1) (trans 11 0) (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3901)
  (parent 3590)
  (unrealized (12 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (7 0)) ((5 1) (3 1))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 9 1)
    (trans 4 1) (trans 1 1) (trans 9 0) (trans 4 0) (trans 1 0)
    (trans 7 1) (trans 2 1) (trans 7 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3902)
  (parent 3591)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 10 1) (trans 5 1)
    (trans 1 1) (trans 10 0) (trans 5 0) (trans 1 0) (trans 8 1)
    (trans 3 1) (trans 8 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 12 4 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3903)
  (parent 3592)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0)
    (trans 1 0) (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0)
    (neq self-0 b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3904)
  (parent 3592)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0)
    (trans 1 0) (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0)
    (neq self-0 b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3905)
  (parent 3592)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (deflistener (cat (exp (gen) la) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 la))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0)
    (trans 1 0) (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0)
    (neq self-0 b))
  (operation nonce-test (displaced 12 0 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb self-0 b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) la l-0))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((recv (cat (exp (gen) la) l)) (send (cat (exp (gen) la) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 la)))
      (send
        (sig (body self-0 (exp (gen) la) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3906)
  (parent 3592)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 2) (11 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((3 2) (6 0)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (8 0)) ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0)
    (trans 1 0) (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0)
    (neq self-0 b))
  (operation nonce-test (displaced 12 1 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-1 "nil")) (send l-1))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3907)
  (parent 3592)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0) (trans 1 0)
    (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3908)
  (parent 3592)
  (unrealized (0 3) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-1) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (5 0)) ((4 3) (10 0)) ((5 1) (4 1)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 2) (8 0))
    ((10 1) (0 3)) ((11 1) (4 0)) ((11 1) (7 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (2 0) (8 0) (10 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv self-1 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 11 1) (trans 9 1) (trans 1 1) (trans 11 0) (trans 9 0)
    (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0)
    (neq self-0 self-1))
  (operation nonce-test (displaced 5 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig (body self-1 (exp (gen) l) (pubk "sig" self-1))
          (privk "sig" self-1)))
      (send (cat na self-0 self-1 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 self-1 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (stor priv-stor-1 (cat pt-2 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3909)
  (parent 3592)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0)
    (trans 1 0) (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0)
    (neq self-0 b))
  (operation nonce-test (displaced 12 10 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3910)
  (parent 3592)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (b self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 10 1) (trans 5 1) (trans 1 1) (trans 12 0)
    (trans 10 0) (trans 5 0) (trans 1 0) (trans 8 1) (trans 3 1)
    (trans 8 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-1 l-2)))
      (send
        (sig (body self-1 (exp (gen) l-2) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 3911)
  (parent 3592)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 9 1)
    (trans 4 1) (trans 12 0) (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (11 0)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 3912)
  (parent 3601)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 2 1) (trans 1 1)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 8 1)
    (trans 4 1) (trans 12 0) (trans 8 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (11 0)
    (ch-msg priv-stor-1 (cat pt-6 (pv self-0 l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 3913)
  (parent 3602)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (3 0)) ((1 2) (7 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((4 1) (8 0)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 3 1) (trans 8 0) (trans 6 0) (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 3914)
  (parent 3603)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 2) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 7 1) (trans 3 1) (trans 9 0) (trans 7 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3915)
  (parent 3604)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((2 1) (12 0)) ((3 1) (0 1)) ((4 2) (3 0))
    ((5 3) (11 0)) ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3916)
  (parent 3605)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (12 0)) ((1 2) (9 0))
    ((2 1) (0 0)) ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0))
    ((5 3) (11 0)) ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3917)
  (parent 3605)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((6 1) (12 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 13 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3918)
  (parent 3605)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0))
    ((13 1) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 3919)
  (parent 3605)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((2 1) (12 0)) ((3 1) (0 1)) ((4 2) (3 0))
    ((5 3) (11 0)) ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3920)
  (parent 3609)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (12 0)) ((1 2) (9 0))
    ((2 1) (0 0)) ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0))
    ((5 3) (11 0)) ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3921)
  (parent 3609)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((6 1) (12 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 13 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3922)
  (parent 3609)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0))
    ((13 1) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 13 1)
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 13 0) (trans 6 0)
    (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1)
    (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 3923)
  (parent 3609)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (displaced 12 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 3924)
  (parent 3621)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3925)
  (parent 3621)
  (unrealized (12 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 2) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 3 1) (trans 9 0) (trans 7 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 3926)
  (parent 3623)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0)
    (neq a b))
  (operation nonce-test (displaced 13 5 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3927)
  (parent 3624)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3928)
  (parent 3624)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3929)
  (parent 3624)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 13 0 init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3930)
  (parent 3624)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 13 1 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3931)
  (parent 3624)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3))
    ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3932)
  (parent 3624)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 2) (10 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 1) (0 3)) ((12 1) (0 0)) ((12 1) (7 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (10 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 5 1) (trans 1 1) (trans 12 0) (trans 5 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 3 1) (trans 9 0)
    (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 2 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3933)
  (parent 3624)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 1) (0 3)) ((12 1) (5 0)) ((12 1) (9 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 2 1) (trans 1 1) (trans 12 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 4 1) (trans 9 0)
    (trans 7 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (displaced 6 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3934)
  (parent 3624)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3935)
  (parent 3624)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 1) (10 0)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 8 1)
    (trans 4 1) (trans 12 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (11 0)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 3936)
  (parent 3633)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 2 1) (trans 1 1)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 7 1)
    (trans 4 1) (trans 12 0) (trans 7 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (11 0)
    (ch-msg priv-stor-1 (cat pt-6 (pv self-0 l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 3937)
  (parent 3634)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((3 2) (0 1))
    ((3 2) (4 0)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 2) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 6 1) (trans 3 1) (trans 6 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 3938)
  (parent 3636)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (0 0))
    ((4 2) (3 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 4 1) (trans 1 1)
    (trans 4 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3939)
  (parent 3637)
  (seen 3635)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((4 2) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1) (trans 4 0)
    (trans 3 0) (neq a b))
  (operation generalization weakened ((1 2) (0 3)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 3940)
  (parent 3638)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta l-0) (a a) (b b) (x x) (upsilon upsilon) (na na)
        (nb nb) (priv-stor priv-stor) (pt pt) (self self)
        (priv-stor-0 priv-stor-0) (ignore ignore) (pt-0 pt-0)
        (pt-1 pt-1))))
  (origs (l-0 (1 1)) (pt-1 (1 1)) (pt-4 (4 1)) (pt-3 (3 1)) (l (2 1))
    (pt (2 1)) (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((3 2) (6 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1) (trans 5 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 3941)
  (parent 3639)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (3 0)) ((1 2) (5 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 2) (0 1)) ((4 2) (0 3)) ((5 1) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1) (trans 4 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 3942)
  (parent 3640)
  (seen 3638)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 11 1)
    (trans 9 1) (trans 7 1) (trans 11 0) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test (displaced 13 4 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3943)
  (parent 3650)
  (seen 3431)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3944)
  (parent 3650)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3945)
  (parent 3650)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) la) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 13 0 init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) la) l-0)) (send (cat (exp (gen) la) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) la)) (send (cat (gen) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-7 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3946)
  (parent 3650)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 13 1 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3947)
  (parent 3650)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3948)
  (parent 3650)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (0 3)) ((12 1) (0 0)) ((12 1) (10 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 4 1) (trans 1 1) (trans 12 0) (trans 4 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (displaced 2 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3949)
  (parent 3650)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (0 3)) ((12 1) (4 0)) ((12 1) (6 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (11 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 2 1) (trans 1 1) (trans 12 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (displaced 5 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3950)
  (parent 3650)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1)
    (trans 7 1) (trans 11 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3951)
  (parent 3650)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (12 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3952)
  (parent 3652)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 1) (12 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3953)
  (parent 3652)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation channel-test (displaced 13 10 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self-0 b-0 (exp (gen) x-0)))))
  (label 3954)
  (parent 3652)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0))
    ((13 1) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1) (pv a l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 10 1) (trans 4 1) (trans 1 1) (trans 13 0)
    (trans 10 0) (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a l-2)))))
  (label 3955)
  (parent 3652)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (12 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3956)
  (parent 3656)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 1) (12 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3957)
  (parent 3656)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation channel-test (displaced 13 10 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3958)
  (parent 3656)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0))
    ((13 1) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 13 1)
    (trans 10 1) (trans 4 1) (trans 1 1) (trans 13 0) (trans 10 0)
    (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq self-0 b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3959)
  (parent 3656)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 11 1)
    (trans 9 1) (trans 7 1) (trans 11 0) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test (displaced 13 4 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3960)
  (parent 3669)
  (seen 3453)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3961)
  (parent 3669)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3962)
  (parent 3669)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 13 0 init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3963)
  (parent 3669)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 13 1 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3964)
  (parent 3669)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3965)
  (parent 3669)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (0 3)) ((12 1) (0 0)) ((12 1) (8 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 4 1) (trans 1 1) (trans 12 0) (trans 4 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (displaced 2 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3966)
  (parent 3669)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (10 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (0 3)) ((12 1) (4 0)) ((12 1) (6 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (11 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 2 1) (trans 1 1) (trans 12 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (displaced 5 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3967)
  (parent 3669)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1)
    (trans 7 1) (trans 11 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3968)
  (parent 3669)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((2 1) (12 0)) ((3 1) (0 1)) ((4 3) (11 0))
    ((5 1) (4 0)) ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 3969)
  (parent 3671)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (12 0)) ((1 2) (3 0)) ((1 2) (10 0))
    ((2 1) (0 0)) ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (11 0))
    ((5 1) (4 0)) ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3970)
  (parent 3671)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 13 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3971)
  (parent 3671)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)) ((12 2) (11 0))
    ((13 1) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 3972)
  (parent 3671)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((2 1) (12 0)) ((3 1) (0 1)) ((4 3) (11 0))
    ((5 1) (4 0)) ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3973)
  (parent 3675)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (12 0)) ((1 2) (3 0)) ((1 2) (10 0))
    ((2 1) (0 0)) ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (11 0))
    ((5 1) (4 0)) ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3974)
  (parent 3675)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 13 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3975)
  (parent 3675)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)) ((12 3) (11 0))
    ((13 1) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 13 1)
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 13 0) (trans 5 0)
    (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3976)
  (parent 3675)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 11 1)
    (trans 9 1) (trans 7 1) (trans 11 0) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test (displaced 13 4 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3977)
  (parent 3688)
  (seen 3475)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3978)
  (parent 3688)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 3979)
  (parent 3688)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) la) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 13 0 init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) la) l)) (send (cat (exp (gen) la) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) la)) (send (cat (gen) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-7 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 3980)
  (parent 3688)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 13 1 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 3981)
  (parent 3688)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 3982)
  (parent 3688)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (0 3)) ((12 1) (0 0)) ((12 1) (10 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 4 1) (trans 1 1) (trans 12 0) (trans 4 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (displaced 2 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3983)
  (parent 3688)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (0 3)) ((12 1) (4 0)) ((12 1) (8 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (11 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 2 1) (trans 1 1) (trans 12 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (displaced 5 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3984)
  (parent 3688)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1)
    (trans 7 1) (trans 11 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 3985)
  (parent 3688)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (12 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 3986)
  (parent 3690)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 1) (12 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 3987)
  (parent 3690)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0)
    (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0)
    (neq self-0 b))
  (operation channel-test (displaced 13 10 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self-0 b-0 (exp (gen) x-0)))))
  (label 3988)
  (parent 3690)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0))
    ((13 1) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1) (pv a l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 10 1) (trans 4 1) (trans 1 1) (trans 13 0)
    (trans 10 0) (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a l-2)))))
  (label 3989)
  (parent 3690)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (12 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3990)
  (parent 3694)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 1) (12 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3991)
  (parent 3694)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 4 1) (trans 1 1) (trans 10 0) (trans 4 0) (trans 1 0)
    (trans 8 1) (trans 6 1) (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation channel-test (displaced 13 10 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 3992)
  (parent 3694)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0))
    ((13 1) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 13 1)
    (trans 10 1) (trans 4 1) (trans 1 1) (trans 13 0) (trans 10 0)
    (trans 4 0) (trans 1 0) (trans 8 1) (trans 6 1) (trans 8 0)
    (trans 6 0) (neq self-0 b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 3993)
  (parent 3694)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 9 1) (trans 7 1) (trans 12 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation nonce-test (displaced 13 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 3994)
  (parent 3698)
  (seen 3494)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 9 1)
    (trans 7 1) (trans 12 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3995)
  (parent 3698)
  (unrealized (13 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (displaced 13 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 3996)
  (parent 3699)
  (seen 3496)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 2 1) (trans 1 1)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 8 1)
    (trans 6 1) (trans 12 0) (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 3997)
  (parent 3699)
  (unrealized (13 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 11 1)
    (trans 9 1) (trans 7 1) (trans 11 0) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test (displaced 13 4 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 3998)
  (parent 3709)
  (seen 3501)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 3999)
  (parent 3709)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 4000)
  (parent 3709)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 13 0 init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 4001)
  (parent 3709)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 13 1 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 4002)
  (parent 3709)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 4003)
  (parent 3709)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (0 3)) ((12 1) (0 0)) ((12 1) (6 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 4 1) (trans 1 1) (trans 12 0) (trans 4 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (displaced 2 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4004)
  (parent 3709)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (10 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (0 3)) ((12 1) (4 0)) ((12 1) (8 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (11 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 2 1) (trans 1 1) (trans 12 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 6 1) (trans 10 0)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (displaced 5 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4005)
  (parent 3709)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1)
    (trans 7 1) (trans 11 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4006)
  (parent 3709)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((2 1) (12 0)) ((3 1) (0 1)) ((4 3) (11 0))
    ((5 1) (4 0)) ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 4007)
  (parent 3711)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (12 0)) ((1 2) (3 0)) ((1 2) (10 0))
    ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (11 0))
    ((5 1) (4 0)) ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 4008)
  (parent 3711)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation channel-test (displaced 13 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 4009)
  (parent 3711)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)) ((12 2) (11 0))
    ((13 1) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 4010)
  (parent 3711)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((2 1) (12 0)) ((3 1) (0 1)) ((4 3) (11 0))
    ((5 1) (4 0)) ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4011)
  (parent 3715)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (12 0)) ((1 2) (3 0)) ((1 2) (10 0))
    ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (11 0))
    ((5 1) (4 0)) ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4012)
  (parent 3715)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 13 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4013)
  (parent 3715)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)) ((12 3) (11 0))
    ((13 1) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 13 1)
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 13 0) (trans 5 0)
    (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 4014)
  (parent 3715)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 9 1) (trans 7 1) (trans 12 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation nonce-test (displaced 13 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4015)
  (parent 3719)
  (seen 3520)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 9 1)
    (trans 7 1) (trans 12 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 4016)
  (parent 3719)
  (unrealized (13 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (displaced 13 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4017)
  (parent 3720)
  (seen 3522)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 2 1) (trans 1 1)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 8 1)
    (trans 6 1) (trans 12 0) (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 4018)
  (parent 3720)
  (unrealized (13 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (0 3)) ((4 2) (3 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 4 1) (trans 4 0) (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))) ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 4019)
  (parent 3721)
  (seen 3723)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 1) (6 0)) ((3 1) (6 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (6 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0) (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4020)
  (parent 3722)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((3 2) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 3 1) (trans 3 0) (neq self b))
  (operation generalization weakened ((2 1) (0 3)))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 4021)
  (parent 3723)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1))
    ((2 1) (4 0)) ((3 2) (4 0)) ((4 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4022)
  (parent 3724)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 2) (0 3)) ((4 2) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 4 1) (trans 3 1) (trans 4 0) (trans 3 0)
    (neq self b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb)))
  (label 4023)
  (parent 3725)
  (seen 3723)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1))
    ((2 1) (5 0)) ((3 2) (6 0)) ((4 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1) (trans 5 0)
    (trans 3 0) (neq self b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)) ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4024)
  (parent 3726)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 2) (6 0)) ((3 1) (5 0)) ((3 2) (6 0)) ((4 1) (6 0))
    ((5 2) (4 0)) ((6 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (trans 5 1) (trans 2 1) (trans 5 0)
    (trans 2 0) (neq self self-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4025)
  (parent 3728)
  (seen 4024)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 1) (4 0)) ((3 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a self))
  (operation generalization weakened ((0 2) (4 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4026)
  (parent 3747)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((3 2) (0 3)) ((4 1) (0 0))
    ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0)
    (neq self-0 b))
  (operation generalization weakened ((2 1) (0 3)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4027)
  (parent 3748)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (5 0)) ((4 1) (0 0)) ((4 2) (5 0)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4028)
  (parent 3749)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 4029)
  (parent 3750)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (4 0)) ((4 2) (9 0)) ((5 1) (9 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 4 1) (trans 8 0) (trans 6 0) (trans 4 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4030)
  (parent 3751)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 1) (0 0)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0)
    (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4031)
  (parent 3760)
  (seen 3748)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (7 0))
    ((3 1) (7 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (7 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4032)
  (parent 3761)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b))
  (operation generalization weakened ((3 1) (0 3)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l)))
  (label 4033)
  (parent 3762)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 2) (5 0)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4034)
  (parent 3763)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 1) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0)))
  (label 4035)
  (parent 3764)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (4 0)) ((4 2) (9 0)) ((5 1) (9 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 4 1) (trans 8 0) (trans 6 0) (trans 4 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4036)
  (parent 3765)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 2) (0 1)) ((1 2) (5 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (0 3)) ((4 2) (0 3)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 4037)
  (parent 3774)
  (seen 3762)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 1) (7 0)) ((4 1) (7 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4038)
  (parent 3775)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (5 0))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 4039)
  (parent 3777)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (6 0)) ((4 2) (9 0)) ((5 1) (9 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 4 1) (trans 8 0) (trans 6 0) (trans 4 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4040)
  (parent 3778)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 2) (7 0)) ((4 2) (7 0)) ((5 1) (7 0)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (7 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 3 1) (trans 4 0) (trans 3 0) (neq self-0 b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4041)
  (parent 3788)
  (seen 4263)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (9 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 6 1)
    (trans 4 1) (trans 9 0) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4042)
  (parent 3789)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (11 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (10 0)) ((4 1) (11 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (11 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 7 1) (trans 5 1) (trans 10 0) (trans 7 0) (trans 5 0)
    (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4043)
  (parent 3790)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((2 1) (13 0)) ((3 3) (12 0)) ((4 1) (3 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 4044)
  (parent 3791)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 1) (13 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (8 0)) ((3 3) (12 0)) ((4 1) (3 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 4045)
  (parent 3791)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((4 1) (13 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 14 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 4046)
  (parent 3791)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1)
    (trans 6 1) (trans 11 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 4047)
  (parent 3791)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((2 1) (13 0)) ((3 3) (12 0)) ((4 1) (3 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4048)
  (parent 3795)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 1) (13 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (8 0)) ((3 3) (12 0)) ((4 1) (3 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4049)
  (parent 3795)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((4 1) (13 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 14 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4050)
  (parent 3795)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 3) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 14 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 14 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1)
    (trans 11 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 4051)
  (parent 3795)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (trans 7 1) (trans 2 1) (trans 1 1)
    (trans 7 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 5 1)
    (trans 3 1) (trans 9 0) (trans 5 0) (trans 3 0) (neq a self-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4052)
  (parent 3799)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (11 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (11 0)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (10 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (11 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 8 1) (trans 2 1)
    (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 6 1) (trans 4 1) (trans 10 0) (trans 6 0) (trans 4 0)
    (neq a self-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4053)
  (parent 3800)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor) (la l-1)
    (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((2 1) (13 0)) ((3 3) (12 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 4054)
  (parent 3801)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 1) (13 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (7 0)) ((3 3) (12 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 4055)
  (parent 3801)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 1) (13 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation channel-test (displaced 14 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self-0 b (exp (gen) x-0)))))
  (label 4056)
  (parent 3801)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a self self-0 a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 9 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 7 1)
    (trans 5 1) (trans 11 0) (trans 7 0) (trans 5 0) (neq a self-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 4057)
  (parent 3801)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((2 1) (13 0)) ((3 3) (12 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0) (trans 7 0)
    (trans 5 0) (neq a self-0))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4058)
  (parent 3805)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 1) (13 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (7 0)) ((3 3) (12 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0) (trans 7 0)
    (trans 5 0) (neq a self-0))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4059)
  (parent 3805)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 1) (13 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0) (trans 7 0)
    (trans 5 0) (neq a self-0))
  (operation channel-test (displaced 14 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4060)
  (parent 3805)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1) (pv b l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 14 1)
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 14 0) (trans 9 0)
    (trans 2 0) (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1)
    (trans 11 0) (trans 7 0) (trans 5 0) (neq a self-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b l-2)))))
  (label 4061)
  (parent 3805)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 2) (0 3)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 4 1) (trans 7 0) (trans 5 0) (trans 4 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 4062)
  (parent 3810)
  (seen 4266)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 2) (9 0)) ((5 1) (9 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 4 1) (trans 8 0) (trans 6 0) (trans 4 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4063)
  (parent 3811)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 2) (7 0)) ((5 2) (7 0))
    ((6 1) (7 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 4 1) (trans 5 0) (trans 4 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4064)
  (parent 3821)
  (seen 4268)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (9 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 6 1)
    (trans 4 1) (trans 9 0) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4065)
  (parent 3822)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (11 0)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 1) (10 0)) ((4 1) (11 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 7 1) (trans 5 1) (trans 10 0) (trans 7 0) (trans 5 0)
    (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4066)
  (parent 3823)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((2 1) (13 0)) ((3 3) (12 0)) ((4 1) (3 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 4067)
  (parent 3824)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 1) (13 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (6 0)) ((3 3) (12 0)) ((4 1) (3 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 4068)
  (parent 3824)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((4 1) (13 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (displaced 14 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 4069)
  (parent 3824)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 4 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1)
    (trans 6 1) (trans 11 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 4070)
  (parent 3824)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((2 1) (13 0)) ((3 3) (12 0)) ((4 1) (3 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4071)
  (parent 3828)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 1) (13 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (6 0)) ((3 3) (12 0)) ((4 1) (3 0))
    ((4 1) (11 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4072)
  (parent 3828)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((4 1) (13 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 4 1)
    (trans 2 1) (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0) (trans 8 0)
    (trans 6 0) (neq a b))
  (operation channel-test (displaced 14 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4073)
  (parent 3828)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 3) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 14 1)
    (trans 4 1) (trans 2 1) (trans 1 1) (trans 14 0) (trans 4 0)
    (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1)
    (trans 11 0) (trans 8 0) (trans 6 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 4074)
  (parent 3828)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (trans 7 1) (trans 2 1) (trans 1 1)
    (trans 7 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 5 1)
    (trans 3 1) (trans 9 0) (trans 5 0) (trans 3 0) (neq a self-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4075)
  (parent 3832)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (11 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 1) (11 0)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (10 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (11 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 8 1) (trans 2 1)
    (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 6 1) (trans 4 1) (trans 10 0) (trans 6 0) (trans 4 0)
    (neq a self-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4076)
  (parent 3833)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor) (la l-1)
    (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((2 1) (13 0)) ((3 3) (12 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 4077)
  (parent 3834)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 1) (13 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (5 0)) ((3 3) (12 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 4078)
  (parent 3834)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 1) (13 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq a self-0))
  (operation channel-test (displaced 14 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self-0 b (exp (gen) x-0)))))
  (label 4079)
  (parent 3834)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a self self-0 a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 9 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 9 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 7 1)
    (trans 5 1) (trans 11 0) (trans 7 0) (trans 5 0) (neq a self-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 4080)
  (parent 3834)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((2 1) (13 0)) ((3 3) (12 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0) (trans 7 0)
    (trans 5 0) (neq a self-0))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4081)
  (parent 3838)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 1) (13 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (5 0)) ((3 3) (12 0)) ((4 1) (3 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (3 0)) ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0) (trans 7 0)
    (trans 5 0) (neq a self-0))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4082)
  (parent 3838)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 1) (13 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)) ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 9 1)
    (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0) (trans 7 0)
    (trans 5 0) (neq a self-0))
  (operation channel-test (displaced 14 9 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4083)
  (parent 3838)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1) (pv b l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 14 1)
    (trans 9 1) (trans 2 1) (trans 1 1) (trans 14 0) (trans 9 0)
    (trans 2 0) (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1)
    (trans 11 0) (trans 7 0) (trans 5 0) (neq a self-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b l-2)))))
  (label 4084)
  (parent 3838)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (4 0)) ((3 1) (0 0))
    ((3 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 self))
  (operation generalization weakened ((3 1) (4 0)))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4085)
  (parent 3843)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((4 2) (0 3)))
  (uniq-orig na lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 4 1) (trans 3 1) (trans 4 0) (trans 3 0)
    (neq a self-0))
  (operation generalization separated self-0)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 4086)
  (parent 3844)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta lb) (a a) (b self-0) (x x) (upsilon y) (na na)
        (nb nb) (priv-stor priv-stor) (pt pt) (self self)
        (priv-stor-0 priv-stor-0) (ignore ignore) (pt-0 pt-0)
        (pt-1 pt-1))))
  (origs (pt-4 (4 1)) (lb (1 1)) (pt-1 (1 1)) (pt-3 (3 1)) (l (2 1))
    (pt (2 1)) (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 2) (0 1)) ((4 2) (0 3)) ((4 2) (5 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1) (trans 4 0)
    (trans 3 0) (neq a self))
  (operation generalization deleted (5 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4087)
  (parent 3845)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (4 0))
    ((4 2) (0 3)) ((5 1) (0 3)) ((6 1) (0 0)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 6 1) (trans 3 1) (trans 1 1)
    (trans 6 0) (trans 3 0) (trans 1 0) (trans 4 1) (trans 2 1)
    (trans 4 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4088)
  (parent 3846)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (6 0))
    ((3 1) (5 0)) ((4 1) (8 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 7 1) (trans 3 1)
    (trans 1 1) (trans 7 0) (trans 3 0) (trans 1 0) (trans 5 1)
    (trans 2 1) (trans 5 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4089)
  (parent 3847)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 2) (0 1)) ((3 2) (8 0)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 7 1) (trans 3 1) (trans 10 0) (trans 7 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 4090)
  (parent 3856)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 11 1)
    (trans 8 1) (trans 4 1) (trans 11 0) (trans 8 0) (trans 4 0)
    (neq a b))
  (operation nonce-test (displaced 13 5 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4091)
  (parent 3857)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 4 1) (trans 11 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 4092)
  (parent 3857)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 4 1) (trans 11 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 4093)
  (parent 3857)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) la) l-0))
  (deflistener (cat (gen) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 4 1) (trans 11 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 13 0 init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) la) l-0)) (send (cat (exp (gen) la) l-0)))
    ((recv (cat (gen) la)) (send (cat (gen) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-7 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 4094)
  (parent 3857)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (12 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 4 1) (trans 11 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 13 1 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 4095)
  (parent 3857)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 4 1) (trans 11 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 4096)
  (parent 3857)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 3)) ((12 1) (0 0)) ((12 1) (10 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 5 1) (trans 1 1) (trans 12 0) (trans 5 0)
    (trans 1 0) (trans 10 1) (trans 7 1) (trans 3 1) (trans 10 0)
    (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 2 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4097)
  (parent 3857)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (11 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 3)) ((12 1) (5 0)) ((12 1) (7 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (11 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 2 1) (trans 1 1) (trans 12 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 7 1) (trans 4 1) (trans 10 0)
    (trans 7 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (displaced 6 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4098)
  (parent 3857)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1)
    (trans 4 1) (trans 11 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4099)
  (parent 3857)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (6 0)) ((3 1) (5 0))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0))
    ((8 1) (0 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 8 1) (trans 3 1) (trans 1 1)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 5 1) (trans 2 1)
    (trans 5 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4100)
  (parent 3858)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (7 0))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 9 1) (trans 4 1)
    (trans 1 1) (trans 9 0) (trans 4 0) (trans 1 0) (trans 6 1)
    (trans 2 1) (trans 6 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4101)
  (parent 3859)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (8 0)) ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0)
    (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0)
    (neq self-0 b))
  (operation channel-test (displaced 13 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 4102)
  (parent 3860)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (12 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((3 2) (8 0)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (8 0)) ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0)
    (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0)
    (neq self-0 b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-1 "nil")) (send l-1))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 4103)
  (parent 3860)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (0 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0)
    (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0)
    (neq self-0 b))
  (operation channel-test (displaced 13 10 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self-0 b-0 (exp (gen) x-0)))))
  (label 4104)
  (parent 3860)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0))
    ((13 1) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1) (pv a l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 10 1) (trans 5 1) (trans 1 1) (trans 13 0)
    (trans 10 0) (trans 5 0) (trans 1 0) (trans 7 1) (trans 3 1)
    (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a l-2)))))
  (label 4105)
  (parent 3860)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (8 0)) ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0) (trans 1 0)
    (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation channel-test (displaced 13 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4106)
  (parent 3864)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (12 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((3 2) (8 0)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (8 0)) ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0) (trans 1 0)
    (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-1 "nil")) (send l-1))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4107)
  (parent 3864)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (0 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0) (trans 1 0)
    (trans 7 1) (trans 3 1) (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation channel-test (displaced 13 10 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4108)
  (parent 3864)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0))
    ((13 1) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 13 1)
    (trans 10 1) (trans 5 1) (trans 1 1) (trans 13 0) (trans 10 0)
    (trans 5 0) (trans 1 0) (trans 7 1) (trans 3 1) (trans 7 0)
    (trans 3 0) (neq self-0 b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 4109)
  (parent 3864)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((1 2) (6 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((4 1) (5 0)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 3 1) (trans 7 0) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l)))
  (label 4110)
  (parent 3868)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (3 0)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 2) (0 1)) ((4 1) (6 0)) ((5 1) (9 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 3 1) (trans 8 0) (trans 6 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4111)
  (parent 3869)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 2) (0 1)) ((3 2) (10 0)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 3 1) (trans 9 0) (trans 7 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 4112)
  (parent 3878)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0)
    (neq a b))
  (operation nonce-test (displaced 13 5 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4113)
  (parent 3879)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 4114)
  (parent 3879)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 4115)
  (parent 3879)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 13 0 init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-7 "nil")) (send la))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 4116)
  (parent 3879)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (12 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 13 1 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 4117)
  (parent 3879)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3))
    ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 4118)
  (parent 3879)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (10 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 1) (0 3)) ((12 1) (0 0)) ((12 1) (9 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (10 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 5 1) (trans 1 1) (trans 12 0) (trans 5 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 3 1) (trans 9 0)
    (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 2 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4119)
  (parent 3879)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (10 0)) ((5 3) (11 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 1) (0 3)) ((12 1) (5 0)) ((12 1) (7 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 2 1) (trans 1 1) (trans 12 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 4 1) (trans 9 0)
    (trans 7 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (displaced 6 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4120)
  (parent 3879)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4121)
  (parent 3879)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (3 0)) ((1 2) (9 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 2) (0 1)) ((4 1) (6 0)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 3 1) (trans 8 0) (trans 6 0) (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 4122)
  (parent 3880)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 2) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (10 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 7 1) (trans 3 1) (trans 9 0) (trans 7 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4123)
  (parent 3881)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((2 1) (13 0)) ((3 1) (0 1)) ((4 2) (3 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 4124)
  (parent 3882)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (13 0)) ((1 2) (11 0))
    ((2 1) (0 0)) ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 4125)
  (parent 3882)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((6 1) (13 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 4126)
  (parent 3882)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3))
    ((13 2) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 4127)
  (parent 3882)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((2 1) (13 0)) ((3 1) (0 1)) ((4 2) (3 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4128)
  (parent 3886)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (13 0)) ((1 2) (11 0))
    ((2 1) (0 0)) ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4129)
  (parent 3886)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((6 1) (13 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4130)
  (parent 3886)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3))
    ((13 3) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 14 1)
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 14 0) (trans 6 0)
    (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1)
    (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 4131)
  (parent 3886)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (5 0)) ((4 1) (0 3))
    ((5 2) (4 0)) ((6 1) (0 0)) ((6 2) (4 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 6 1) (trans 3 1) (trans 1 1)
    (trans 6 0) (trans 3 0) (trans 1 0) (trans 5 1) (trans 2 1)
    (trans 5 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4132)
  (parent 3890)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 1) (6 0)) ((4 1) (8 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (0 0)) ((7 2) (5 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 7 1) (trans 3 1)
    (trans 1 1) (trans 7 0) (trans 3 0) (trans 1 0) (trans 6 1)
    (trans 2 1) (trans 6 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4133)
  (parent 3891)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 2) (0 1)) ((3 2) (6 0)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (8 0)) ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 3 1) (trans 10 0) (trans 8 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 4134)
  (parent 3900)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 11 1)
    (trans 9 1) (trans 4 1) (trans 11 0) (trans 9 0) (trans 4 0)
    (neq a b))
  (operation nonce-test (displaced 13 5 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4135)
  (parent 3901)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 4 1) (trans 11 0)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 4136)
  (parent 3901)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 4 1) (trans 11 0)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 4137)
  (parent 3901)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (deflistener (cat (exp (gen) la) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 4 1) (trans 11 0)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 13 0 init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((recv (cat (exp (gen) la) l)) (send (cat (exp (gen) la) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) la)) (send (cat (gen) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-7 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 4138)
  (parent 3901)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (12 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 4 1) (trans 11 0)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 13 1 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 4139)
  (parent 3901)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 4 1) (trans 11 0) (trans 9 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 4140)
  (parent 3901)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (0 3)) ((12 1) (0 0)) ((12 1) (10 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 5 1) (trans 1 1) (trans 12 0) (trans 5 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 3 1) (trans 10 0)
    (trans 8 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 2 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4141)
  (parent 3901)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (11 0))
    ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (0 3)) ((12 1) (5 0)) ((12 1) (8 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (11 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 2 1) (trans 1 1) (trans 12 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (displaced 6 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4142)
  (parent 3901)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1)
    (trans 4 1) (trans 11 0) (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4143)
  (parent 3901)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (4 0)) ((3 1) (6 0))
    ((4 1) (0 3)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 8 1) (trans 3 1) (trans 1 1)
    (trans 8 0) (trans 3 0) (trans 1 0) (trans 6 1) (trans 2 1)
    (trans 6 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4144)
  (parent 3902)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (7 0)) ((5 1) (3 1))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 9 1) (trans 4 1)
    (trans 1 1) (trans 9 0) (trans 4 0) (trans 1 0) (trans 7 1)
    (trans 2 1) (trans 7 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4145)
  (parent 3903)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0)
    (trans 1 0) (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0)
    (neq self-0 b))
  (operation channel-test (displaced 13 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 4146)
  (parent 3904)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (12 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((3 2) (6 0)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (8 0)) ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0)
    (trans 1 0) (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0)
    (neq self-0 b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-1 "nil")) (send l-1))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 4147)
  (parent 3904)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self self-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0)
    (trans 1 0) (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0)
    (neq self-0 b))
  (operation channel-test (displaced 13 10 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self-0 b-0 (exp (gen) x-0)))))
  (label 4148)
  (parent 3904)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 2) (11 0))
    ((13 1) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1) (pv a l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 10 1) (trans 5 1) (trans 1 1) (trans 13 0)
    (trans 10 0) (trans 5 0) (trans 1 0) (trans 8 1) (trans 3 1)
    (trans 8 0) (trans 3 0) (neq self-0 b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a l-2)))))
  (label 4149)
  (parent 3904)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0) (trans 1 0)
    (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0) (neq self-0 b))
  (operation channel-test (displaced 13 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec l-1)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4150)
  (parent 3908)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (12 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((3 2) (6 0)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (8 0)) ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0) (trans 1 0)
    (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0) (neq self-0 b))
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (stor priv-stor (cat pt-1 "nil")) (send l-1))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4151)
  (parent 3908)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (0 3))
    ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 5 1) (trans 1 1) (trans 10 0) (trans 5 0) (trans 1 0)
    (trans 8 1) (trans 3 1) (trans 8 0) (trans 3 0) (neq self-0 b))
  (operation channel-test (displaced 13 10 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4152)
  (parent 3908)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self self-0 a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)) ((12 3) (11 0))
    ((13 1) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 13 1)
    (trans 10 1) (trans 5 1) (trans 1 1) (trans 13 0) (trans 10 0)
    (trans 5 0) (trans 1 0) (trans 8 1) (trans 3 1) (trans 8 0)
    (trans 3 0) (neq self-0 b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 4153)
  (parent 3908)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 9 1) (trans 4 1) (trans 12 0) (trans 9 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (displaced 13 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4154)
  (parent 3912)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 9 1)
    (trans 4 1) (trans 12 0) (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 4155)
  (parent 3912)
  (unrealized (13 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0)
    (trans 8 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (displaced 13 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4156)
  (parent 3913)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 2 1) (trans 1 1)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 8 1)
    (trans 4 1) (trans 12 0) (trans 8 0) (trans 4 0) (neq a self-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 4157)
  (parent 3913)
  (unrealized (13 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (3 0)) ((1 2) (6 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (7 0)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 3 1) (trans 7 0) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4158)
  (parent 3914)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (3 0)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 2) (0 1)) ((4 1) (8 0)) ((5 1) (9 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 3 1) (trans 8 0) (trans 6 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4159)
  (parent 3915)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((3 2) (10 0)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 9 1) (trans 7 1) (trans 3 1) (trans 9 0) (trans 7 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 4160)
  (parent 3924)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0)
    (neq a b))
  (operation nonce-test (displaced 13 5 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4161)
  (parent 3925)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 4162)
  (parent 3925)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 4163)
  (parent 3925)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 13 0 init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 4164)
  (parent 3925)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (12 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 13 1 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 4165)
  (parent 3925)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3))
    ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 4166)
  (parent 3925)
  (unrealized (0 3) (12 0) (13 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (10 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 1) (0 3)) ((12 1) (0 0)) ((12 1) (7 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (10 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 5 1) (trans 1 1) (trans 12 0) (trans 5 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 3 1) (trans 9 0)
    (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 2 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4167)
  (parent 3925)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (10 0)) ((5 3) (11 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 1) (0 3)) ((12 1) (5 0)) ((12 1) (9 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 12 1) (trans 2 1) (trans 1 1) (trans 12 0) (trans 2 0)
    (trans 1 0) (trans 9 1) (trans 7 1) (trans 4 1) (trans 9 0)
    (trans 7 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (displaced 6 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4168)
  (parent 3925)
  (unrealized (0 3) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3))
    ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 13 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4169)
  (parent 3925)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (3 0)) ((1 2) (9 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((4 1) (8 0)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 3 1) (trans 8 0) (trans 6 0) (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 4170)
  (parent 3926)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 2) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (10 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 7 1) (trans 3 1) (trans 9 0) (trans 7 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4171)
  (parent 3927)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((2 1) (13 0)) ((3 1) (0 1)) ((4 2) (3 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 4172)
  (parent 3928)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (13 0)) ((1 2) (11 0))
    ((2 1) (0 0)) ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 4173)
  (parent 3928)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((6 1) (13 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 4174)
  (parent 3928)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3))
    ((13 2) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 4175)
  (parent 3928)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((2 1) (13 0)) ((3 1) (0 1)) ((4 2) (3 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4176)
  (parent 3932)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (13 0)) ((1 2) (11 0))
    ((2 1) (0 0)) ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4177)
  (parent 3932)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((6 1) (13 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4178)
  (parent 3932)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3))
    ((13 3) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 14 1)
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 14 0) (trans 6 0)
    (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1)
    (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 4179)
  (parent 3932)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 1) (10 0)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (displaced 13 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4180)
  (parent 3936)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 8 1)
    (trans 4 1) (trans 12 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 4181)
  (parent 3936)
  (unrealized (13 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 7 1) (trans 4 1) (trans 12 0)
    (trans 7 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (displaced 13 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4182)
  (parent 3937)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (trans 10 1) (trans 2 1) (trans 1 1)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 7 1)
    (trans 4 1) (trans 12 0) (trans 7 0) (trans 4 0) (neq a self-0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 4183)
  (parent 3937)
  (unrealized (13 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1) (trans 5 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 4184)
  (parent 3938)
  (seen 3940)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((3 2) (5 0)) ((4 2) (0 3)) ((5 1) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1) (trans 4 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 4185)
  (parent 3941)
  (seen 3940)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((2 1) (13 0)) ((3 1) (0 1)) ((4 3) (12 0))
    ((5 1) (4 0)) ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 4186)
  (parent 3944)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 1) (13 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0))
    ((5 1) (4 0)) ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 4187)
  (parent 3944)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((5 1) (13 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 14 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 4188)
  (parent 3944)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1)
    (trans 7 1) (trans 11 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 4189)
  (parent 3944)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((2 1) (13 0)) ((3 1) (0 1)) ((4 3) (12 0))
    ((5 1) (4 0)) ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4190)
  (parent 3948)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 1) (13 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0))
    ((5 1) (4 0)) ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4191)
  (parent 3948)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((5 1) (13 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation channel-test (displaced 14 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4192)
  (parent 3948)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 3) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 14 1)
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 14 0) (trans 5 0)
    (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1)
    (trans 11 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 4193)
  (parent 3948)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((2 1) (13 0)) ((3 1) (0 1)) ((4 3) (12 0))
    ((5 1) (4 0)) ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 4194)
  (parent 3961)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 1) (13 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (12 0))
    ((5 1) (4 0)) ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 4195)
  (parent 3961)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((5 1) (13 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 14 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 4196)
  (parent 3961)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1)
    (trans 7 1) (trans 11 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 4197)
  (parent 3961)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((2 1) (13 0)) ((3 1) (0 1)) ((4 3) (12 0))
    ((5 1) (4 0)) ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4198)
  (parent 3965)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 1) (13 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (12 0))
    ((5 1) (4 0)) ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4199)
  (parent 3965)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((5 1) (13 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation channel-test (displaced 14 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4200)
  (parent 3965)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 3) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 14 1)
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 14 0) (trans 5 0)
    (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1)
    (trans 11 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 4201)
  (parent 3965)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((2 1) (13 0)) ((3 1) (0 1)) ((4 3) (12 0))
    ((5 1) (4 0)) ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 4202)
  (parent 3978)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 1) (13 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0))
    ((5 1) (4 0)) ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 4203)
  (parent 3978)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((5 1) (13 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 14 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 4204)
  (parent 3978)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1)
    (trans 7 1) (trans 11 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 4205)
  (parent 3978)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((2 1) (13 0)) ((3 1) (0 1)) ((4 3) (12 0))
    ((5 1) (4 0)) ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4206)
  (parent 3982)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 1) (13 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0))
    ((5 1) (4 0)) ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4207)
  (parent 3982)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((5 1) (13 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation channel-test (displaced 14 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4208)
  (parent 3982)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 3) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 14 1)
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 14 0) (trans 5 0)
    (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1)
    (trans 11 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 4209)
  (parent 3982)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 12 1)
    (trans 9 1) (trans 7 1) (trans 12 0) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test (displaced 14 4 resp 4) (exp (gen) y-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4210)
  (parent 3995)
  (seen 3790)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 9 1) (trans 7 1) (trans 12 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 4211)
  (parent 3995)
  (unrealized (0 3) (13 0) (14 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 9 1) (trans 7 1) (trans 12 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 4212)
  (parent 3995)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (gen) (rec l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 9 1) (trans 7 1) (trans 12 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 14 0 init 3) (exp (gen) x-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) la))
      (send (cat (exp (gen) (rec l)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (gen) (rec l))) (send (cat (gen) (rec l))))
    ((recv l) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 4213)
  (parent 3995)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 9 1) (trans 7 1) (trans 12 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 14 1 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 4214)
  (parent 3995)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 9 1) (trans 7 1) (trans 12 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 4215)
  (parent 3995)
  (unrealized (0 3) (13 0) (14 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)) ((13 1) (0 0))
    ((13 1) (8 0)) ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0) (12 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 4 1) (trans 1 1) (trans 13 0) (trans 4 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (displaced 2 14 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4216)
  (parent 3995)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)) ((13 1) (4 0))
    ((13 1) (11 0)) ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 2 1) (trans 1 1) (trans 13 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (displaced 5 14 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4217)
  (parent 3995)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 9 1)
    (trans 7 1) (trans 12 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4218)
  (parent 3995)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 10 1) (trans 2 1)
    (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0) (trans 12 1)
    (trans 8 1) (trans 6 1) (trans 12 0) (trans 8 0) (trans 6 0)
    (neq a self-0))
  (operation nonce-test (displaced 14 4 resp 4) (exp (gen) y-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self-0 (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4219)
  (parent 3997)
  (seen 3800)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self self-0 a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0)))))
  (label 4220)
  (parent 3997)
  (unrealized (0 3) (13 0) (14 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 4221)
  (parent 3997)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l) (mul (rec l) (rec l))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a la) (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (displaced 14 0 init 3) (exp (gen) x-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self-0
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) la))
      (send (cat (exp (gen) (rec l)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-4 "nil")) (send la))
    ((recv (cat (exp (gen) l) (mul (rec l) (rec l))))
      (send (cat (exp (gen) l) (mul (rec l) (rec l)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 4222)
  (parent 3997)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (displaced 14 1 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 4223)
  (parent 3997)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0) (trans 8 0)
    (trans 6 0) (neq a self-0))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 4224)
  (parent 3997)
  (unrealized (0 3) (13 0) (14 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-1) (b self-0)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-1) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-1) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (11 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 1) (0 0)) ((13 1) (7 0)) ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (8 0) (12 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv self-1 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 9 1) (trans 1 1) (trans 13 0) (trans 9 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq self-1 self-0))
  (operation nonce-test (displaced 2 14 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-1 l-1)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self-1 self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-1 self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig
          (body self-1 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-1)) (privk "sig" self-1)))
      (recv (cat na self-1 self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-1 self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-1 l-1)))
      (stor priv-stor-1 (cat pt-2 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 4225)
  (parent 3997)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (displaced 14 10 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 4226)
  (parent 3997)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 10 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 8 1)
    (trans 6 1) (trans 12 0) (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-1 l-2)))
      (send
        (sig (body self-1 (exp (gen) l-2) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 4227)
  (parent 3997)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((2 1) (13 0)) ((3 1) (0 1)) ((4 3) (12 0))
    ((5 1) (4 0)) ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 4228)
  (parent 3999)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 1) (13 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (12 0))
    ((5 1) (4 0)) ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 4229)
  (parent 3999)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((5 1) (13 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 14 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 4230)
  (parent 3999)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1)
    (trans 7 1) (trans 11 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 4231)
  (parent 3999)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((2 1) (13 0)) ((3 1) (0 1)) ((4 3) (12 0))
    ((5 1) (4 0)) ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4232)
  (parent 4003)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 1) (13 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (12 0))
    ((5 1) (4 0)) ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4233)
  (parent 4003)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((5 1) (13 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 7 1) (trans 11 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation channel-test (displaced 14 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4234)
  (parent 4003)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 3) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 14 1)
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 14 0) (trans 5 0)
    (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1) (trans 7 1)
    (trans 11 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 4235)
  (parent 4003)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 12 1)
    (trans 9 1) (trans 7 1) (trans 12 0) (trans 9 0) (trans 7 0)
    (neq a b))
  (operation nonce-test (displaced 14 4 resp 4) (exp (gen) y-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4236)
  (parent 4016)
  (seen 3823)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 9 1) (trans 7 1) (trans 12 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 4237)
  (parent 4016)
  (unrealized (0 3) (13 0) (14 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 9 1) (trans 7 1) (trans 12 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 4238)
  (parent 4016)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 9 1) (trans 7 1) (trans 12 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 14 0 init 3) (exp (gen) x-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l))) (send (cat (gen) (rec l))))
    ((recv l) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 4239)
  (parent 4016)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 9 1) (trans 7 1) (trans 12 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (displaced 14 1 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 4240)
  (parent 4016)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 9 1) (trans 7 1) (trans 12 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 4241)
  (parent 4016)
  (unrealized (0 3) (13 0) (14 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)) ((13 1) (0 0))
    ((13 1) (6 0)) ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0) (12 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 4 1) (trans 1 1) (trans 13 0) (trans 4 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq self-0 b))
  (operation nonce-test (displaced 2 14 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4242)
  (parent 4016)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)) ((13 1) (4 0))
    ((13 1) (11 0)) ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 2 1) (trans 1 1) (trans 13 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 6 1) (trans 11 0)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (displaced 5 14 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4243)
  (parent 4016)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 9 1)
    (trans 7 1) (trans 12 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4244)
  (parent 4016)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 10 1) (trans 2 1)
    (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0) (trans 12 1)
    (trans 8 1) (trans 6 1) (trans 12 0) (trans 8 0) (trans 6 0)
    (neq a self-0))
  (operation nonce-test (displaced 14 4 resp 4) (exp (gen) y-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self-0 (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4245)
  (parent 4018)
  (seen 3833)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self self-0 a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0)))))
  (label 4246)
  (parent 4018)
  (unrealized (0 3) (13 0) (14 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 4247)
  (parent 4018)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) (rec l))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a la) (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (displaced 14 0 init 3) (exp (gen) x-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self-0
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-3 "nil")) (send la))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) (mul (rec l) (rec l))))
      (send (cat (exp (gen) l) (mul (rec l) (rec l)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 4248)
  (parent 4018)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (displaced 14 1 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 4249)
  (parent 4018)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0) (trans 8 0)
    (trans 6 0) (neq a self-0))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 4250)
  (parent 4018)
  (unrealized (0 3) (13 0) (14 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-1) (b self-0)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-1) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self-1) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (11 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 1) (0 0)) ((13 1) (5 0)) ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (8 0) (12 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv self-1 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 9 1) (trans 1 1) (trans 13 0) (trans 9 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 5 1) (trans 11 0)
    (trans 7 0) (trans 5 0) (neq self-1 self-0))
  (operation nonce-test (displaced 2 14 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-1 l-1)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self-1 self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-1 self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig
          (body self-1 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-1)) (privk "sig" self-1)))
      (recv (cat na self-1 self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-1 self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-1 l-1)))
      (stor priv-stor-1 (cat pt-1 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 4251)
  (parent 4018)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (displaced 14 10 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 4252)
  (parent 4018)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 10 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 8 1)
    (trans 6 1) (trans 12 0) (trans 8 0) (trans 6 0) (neq a self-0))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-1 l-2)))
      (send
        (sig (body self-1 (exp (gen) l-2) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 4253)
  (parent 4018)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 1) (5 0)) ((3 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0) (neq self b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))) ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4254)
  (parent 4020)
  (seen 4022)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((3 2) (0 3)))
  (uniq-orig na l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 3 1) (trans 3 0) (neq self b))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 4255)
  (parent 4021)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1))
    ((2 1) (0 3)) ((2 1) (4 0)) ((3 2) (0 3)) ((3 2) (4 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq self b))
  (operation generalization deleted (4 1))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4256)
  (parent 4022)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1))
    ((2 1) (4 0)) ((3 2) (5 0)) ((4 2) (5 0)) ((5 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1) (trans 4 0)
    (trans 3 0) (neq self b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4257)
  (parent 4024)
  (seen 4022)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a self))
  (operation generalization weakened ((2 1) (4 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4258)
  (parent 4026)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((3 2) (0 3)) ((4 1) (0 0))
    ((4 2) (0 3)))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0)
    (neq self-0 b))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4259)
  (parent 4027)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((2 1) (5 0)) ((3 2) (0 3)) ((3 2) (5 0)) ((4 1) (0 0))
    ((4 2) (0 3)) ((4 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 b))
  (operation generalization deleted (5 1))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4260)
  (parent 4028)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 3)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 4261)
  (parent 4029)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (8 0)) ((4 1) (8 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4262)
  (parent 4030)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 2) (6 0)) ((4 1) (6 0)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (4 0) (6 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4263)
  (parent 4032)
  (seen 4028)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((4 2) (0 3)))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l)))
  (label 4264)
  (parent 4033)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (0 3)) ((3 1) (5 0)) ((4 2) (0 3)) ((4 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization deleted (5 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4265)
  (parent 4034)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (0 3)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0)))
  (label 4266)
  (parent 4035)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 1) (8 0)) ((4 1) (8 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4267)
  (parent 4036)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((1 2) (5 0)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 2) (6 0)) ((5 1) (6 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4268)
  (parent 4038)
  (seen 4034)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0)) ((3 1) (4 0))
    ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 4269)
  (parent 4039)
  (seen 4415)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (5 0)) ((4 1) (8 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 7 0) (trans 5 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4270)
  (parent 4040)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 1) (8 0))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0))
    ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 5 1)
    (trans 8 0) (trans 5 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 4271)
  (parent 4042)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (9 0)) ((4 2) (10 0)) ((5 1) (10 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (10 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 6 1) (trans 4 1) (trans 9 0) (trans 6 0) (trans 4 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4272)
  (parent 4043)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (8 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 8 0) (trans 4 0) (neq a self-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-5 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 4273)
  (parent 4052)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 2) (10 0)) ((4 1) (10 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (10 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 7 1) (trans 2 1)
    (trans 1 1) (trans 7 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 5 1) (trans 3 1) (trans 9 0) (trans 5 0) (trans 3 0)
    (neq a self-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4274)
  (parent 4053)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 2) (8 0)) ((5 2) (8 0))
    ((6 1) (8 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 4 1) (trans 7 0) (trans 5 0) (trans 4 0)
    (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4275)
  (parent 4063)
  (seen 4420)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (8 0)) ((4 2) (0 3)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 5 1)
    (trans 4 1) (trans 8 0) (trans 5 0) (trans 4 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4276)
  (parent 4065)
  (seen 4422)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 1) (9 0)) ((4 2) (10 0)) ((5 1) (10 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 6 1) (trans 4 1) (trans 9 0) (trans 6 0) (trans 4 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4277)
  (parent 4066)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (0 3)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 3 1) (trans 8 0) (trans 4 0) (trans 3 0) (neq a self-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4278)
  (parent 4075)
  (seen 4424)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (3 0)) ((3 2) (10 0)) ((4 1) (10 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (10 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 7 1) (trans 2 1)
    (trans 1 1) (trans 7 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 5 1) (trans 3 1) (trans 9 0) (trans 5 0) (trans 3 0)
    (neq a self-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4279)
  (parent 4076)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (2 0)) ((2 1) (4 0)) ((2 2) (0 1)) ((3 1) (0 0))
    ((3 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 self))
  (operation generalization weakened ((2 2) (4 0)))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4280)
  (parent 4085)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 2) (0 1)) ((4 1) (5 0)) ((4 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1) (trans 4 0)
    (trans 3 0) (neq a self))
  (operation generalization weakened ((4 2) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4281)
  (parent 4087)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (4 0)) ((3 1) (0 3))
    ((4 1) (0 3)) ((5 1) (0 0)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 5 1) (trans 3 1) (trans 1 1)
    (trans 5 0) (trans 3 0) (trans 1 0) (trans 2 1) (trans 2 0)
    (neq self-0 b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4282)
  (parent 4088)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (4 0)) ((4 2) (7 0)) ((5 1) (7 0)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 6 1) (trans 3 1)
    (trans 1 1) (trans 6 0) (trans 3 0) (trans 1 0) (trans 4 1)
    (trans 2 1) (trans 4 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4283)
  (parent 4089)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((3 2) (0 1))
    ((3 2) (7 0)) ((4 1) (6 0)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 6 1)
    (trans 3 1) (trans 9 0) (trans 6 0) (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 4284)
  (parent 4090)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 2) (0 1)) ((3 2) (8 0)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 7 1) (trans 3 1) (trans 10 0) (trans 7 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4285)
  (parent 4091)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((2 1) (13 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 4 1) (trans 11 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 4286)
  (parent 4092)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (13 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 4 1) (trans 11 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 4287)
  (parent 4092)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((6 1) (13 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 4 1) (trans 11 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 4288)
  (parent 4092)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1)
    (trans 4 1) (trans 11 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 4289)
  (parent 4092)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((2 1) (13 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 4 1) (trans 11 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4290)
  (parent 4096)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (13 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 4 1) (trans 11 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4291)
  (parent 4096)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((6 1) (13 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 4 1) (trans 11 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4292)
  (parent 4096)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 3) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 14 1)
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 14 0) (trans 6 0)
    (trans 2 0) (trans 1 0) (trans 11 1) (trans 8 1) (trans 4 1)
    (trans 11 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 4293)
  (parent 4096)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (4 0))
    ((4 2) (0 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 1) (0 0))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 7 1) (trans 3 1) (trans 1 1)
    (trans 7 0) (trans 3 0) (trans 1 0) (trans 4 1) (trans 2 1)
    (trans 4 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4294)
  (parent 4100)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (6 0))
    ((3 1) (5 0)) ((4 1) (9 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (6 0)) ((8 1) (0 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (9 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 8 1) (trans 3 1)
    (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0) (trans 5 1)
    (trans 2 1) (trans 5 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4295)
  (parent 4101)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((1 2) (5 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((4 1) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 3 1)
    (trans 6 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 4296)
  (parent 4110)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 2) (0 1)) ((4 1) (5 0)) ((5 2) (8 0))
    ((6 1) (8 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 3 1) (trans 7 0) (trans 5 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4297)
  (parent 4111)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((3 2) (0 1))
    ((3 2) (9 0)) ((4 1) (6 0)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 3 1) (trans 8 0) (trans 6 0) (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 4298)
  (parent 4112)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 2) (0 1)) ((3 2) (10 0)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (10 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 7 1) (trans 3 1) (trans 9 0) (trans 7 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4299)
  (parent 4113)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((2 1) (13 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 4300)
  (parent 4114)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (13 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 4301)
  (parent 4114)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((6 1) (13 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 4302)
  (parent 4114)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3))
    ((13 2) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 4303)
  (parent 4114)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((2 1) (13 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4304)
  (parent 4118)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (13 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4305)
  (parent 4118)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((6 1) (13 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4306)
  (parent 4118)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3))
    ((13 3) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 14 1)
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 14 0) (trans 6 0)
    (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1)
    (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 4307)
  (parent 4118)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (3 0)) ((1 2) (8 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((4 1) (5 0)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 3 1) (trans 7 0) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 4308)
  (parent 4122)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (3 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 2) (0 1)) ((4 1) (6 0)) ((5 1) (10 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 3 1) (trans 8 0) (trans 6 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4309)
  (parent 4123)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (4 0)) ((4 2) (0 3))
    ((5 1) (0 0)) ((5 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 5 1) (trans 3 1) (trans 1 1)
    (trans 5 0) (trans 3 0) (trans 1 0) (trans 4 1) (trans 2 1)
    (trans 4 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4310)
  (parent 4132)
  (seen 4430)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (5 0))
    ((4 1) (7 0)) ((5 2) (4 0)) ((6 1) (0 0)) ((6 2) (4 0))
    ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 6 1) (trans 3 1)
    (trans 1 1) (trans 6 0) (trans 3 0) (trans 1 0) (trans 5 1)
    (trans 2 1) (trans 5 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4311)
  (parent 4133)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((3 2) (0 1))
    ((3 2) (5 0)) ((4 1) (7 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 7 1)
    (trans 3 1) (trans 9 0) (trans 7 0) (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 4312)
  (parent 4134)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 2) (0 1)) ((3 2) (6 0)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (8 0)) ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 8 1) (trans 3 1) (trans 10 0) (trans 8 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4313)
  (parent 4135)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((2 1) (13 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 4 1) (trans 11 0)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 4314)
  (parent 4136)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (13 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 4 1) (trans 11 0)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 4315)
  (parent 4136)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (9 0)) ((6 1) (13 0)) ((7 1) (5 1))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 9 1) (trans 4 1) (trans 11 0)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 4316)
  (parent 4136)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 2) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1)
    (trans 4 1) (trans 11 0) (trans 9 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 4317)
  (parent 4136)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((2 1) (13 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 4 1) (trans 11 0) (trans 9 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4318)
  (parent 4140)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (13 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 4 1) (trans 11 0) (trans 9 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4319)
  (parent 4140)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (9 0)) ((6 1) (13 0)) ((7 1) (5 1))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 9 1) (trans 4 1) (trans 11 0) (trans 9 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4320)
  (parent 4140)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 3) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 14 1)
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 14 0) (trans 6 0)
    (trans 2 0) (trans 1 0) (trans 11 1) (trans 9 1) (trans 4 1)
    (trans 11 0) (trans 9 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 4321)
  (parent 4140)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (5 0)) ((4 1) (0 3))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 7 1) (trans 3 1) (trans 1 1)
    (trans 7 0) (trans 3 0) (trans 1 0) (trans 5 1) (trans 2 1)
    (trans 5 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4322)
  (parent 4144)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 1) (6 0)) ((4 1) (9 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (9 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 8 1) (trans 3 1)
    (trans 1 1) (trans 8 0) (trans 3 0) (trans 1 0) (trans 6 1)
    (trans 2 1) (trans 6 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4323)
  (parent 4145)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 2) (0 1)) ((3 2) (6 0)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 8 1) (trans 3 1) (trans 11 0) (trans 8 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4324)
  (parent 4154)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 12 1)
    (trans 9 1) (trans 4 1) (trans 12 0) (trans 9 0) (trans 4 0)
    (neq a b))
  (operation nonce-test (displaced 14 5 resp 4) (exp (gen) y-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4325)
  (parent 4155)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 9 1) (trans 4 1) (trans 12 0)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 4326)
  (parent 4155)
  (unrealized (0 3) (13 0) (14 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 9 1) (trans 4 1) (trans 12 0)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 4327)
  (parent 4155)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (deflistener (cat (exp (gen) (rec l)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (gen) (rec l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 9 1) (trans 4 1) (trans 12 0)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 14 0 init 3) (exp (gen) x-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((recv (cat (exp (gen) (rec l)) la))
      (send (cat (exp (gen) (rec l)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (gen) (rec l))) (send (cat (gen) (rec l))))
    ((recv l) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 4328)
  (parent 4155)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (13 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0))
    ((5 3) (13 0)) ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 9 1) (trans 4 1) (trans 12 0)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 14 1 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 4329)
  (parent 4155)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 9 1) (trans 4 1) (trans 12 0) (trans 9 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 4330)
  (parent 4155)
  (unrealized (0 3) (13 0) (14 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (12 0)) ((5 1) (4 0)) ((5 1) (11 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)) ((13 1) (0 0))
    ((13 1) (8 0)) ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0) (12 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 5 1) (trans 1 1) (trans 13 0) (trans 5 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 3 1) (trans 11 0)
    (trans 8 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 2 14 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4331)
  (parent 4155)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (12 0))
    ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)) ((13 1) (5 0))
    ((13 1) (11 0)) ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 2 1) (trans 1 1) (trans 13 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 8 1) (trans 4 1) (trans 11 0)
    (trans 8 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (displaced 6 14 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4332)
  (parent 4155)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 9 1)
    (trans 4 1) (trans 12 0) (trans 9 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4333)
  (parent 4155)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((3 2) (5 0)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (4 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 9 1)
    (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 7 1) (trans 3 1) (trans 11 0) (trans 7 0)
    (trans 3 0) (neq a self-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4334)
  (parent 4156)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 10 1) (trans 2 1)
    (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0) (trans 12 1)
    (trans 8 1) (trans 4 1) (trans 12 0) (trans 8 0) (trans 4 0)
    (neq a self-0))
  (operation nonce-test (displaced 14 5 resp 4) (exp (gen) y-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self-0 (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4335)
  (parent 4157)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self self-0 a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0)
    (trans 8 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0)))))
  (label 4336)
  (parent 4157)
  (unrealized (0 3) (13 0) (14 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0)
    (trans 8 0) (trans 4 0) (neq a self-0))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 4337)
  (parent 4157)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (deflistener (cat (exp (gen) (rec l)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l) (mul (rec l) (rec l))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a la) (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0)
    (trans 8 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (displaced 14 0 init 3) (exp (gen) x-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self-0
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((recv (cat (exp (gen) (rec l)) la))
      (send (cat (exp (gen) (rec l)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-4 "nil")) (send la))
    ((recv (cat (exp (gen) l) (mul (rec l) (rec l))))
      (send (cat (exp (gen) l) (mul (rec l) (rec l)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 4338)
  (parent 4157)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (13 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 3) (13 0)) ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0)
    (trans 8 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (displaced 14 1 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 4339)
  (parent 4157)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0) (trans 8 0)
    (trans 4 0) (neq a self-0))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 4340)
  (parent 4157)
  (unrealized (0 3) (13 0) (14 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-1) (b self-0)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-1) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-1) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (5 0)) ((4 3) (12 0)) ((5 1) (4 1)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (4 0)) ((9 1) (11 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 1) (0 0)) ((13 1) (7 0)) ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (8 0) (12 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv self-1 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 9 1) (trans 1 1) (trans 13 0) (trans 9 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 3 1) (trans 11 0)
    (trans 7 0) (trans 3 0) (neq self-1 self-0))
  (operation nonce-test (displaced 2 14 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-1 l-1)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self-1 self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-1 self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig
          (body self-1 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-1)) (privk "sig" self-1)))
      (recv (cat na self-1 self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-1 self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-1 l-1)))
      (stor priv-stor-1 (cat pt-2 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 4341)
  (parent 4157)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0)
    (trans 8 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (displaced 14 10 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 4342)
  (parent 4157)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 10 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 8 1)
    (trans 4 1) (trans 12 0) (trans 8 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-1 l-2)))
      (send
        (sig (body self-1 (exp (gen) l-2) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 4343)
  (parent 4157)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (3 0)) ((1 2) (0 3)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (6 0)) ((5 2) (0 3)) ((6 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 5 1)
    (trans 3 1) (trans 6 0) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4344)
  (parent 4158)
  (seen 4436)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 2) (0 1)) ((4 1) (7 0)) ((5 2) (8 0))
    ((6 1) (8 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 3 1) (trans 7 0) (trans 5 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4345)
  (parent 4159)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((3 2) (0 1))
    ((3 2) (9 0)) ((4 1) (8 0)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 3 1) (trans 8 0) (trans 6 0) (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 4346)
  (parent 4160)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((3 2) (10 0)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (10 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 7 1) (trans 3 1) (trans 9 0) (trans 7 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4347)
  (parent 4161)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((2 1) (13 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 4348)
  (parent 4162)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (13 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 4349)
  (parent 4162)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((6 1) (13 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)) ((13 2) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 4350)
  (parent 4162)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3))
    ((13 2) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1)
    (trans 4 1) (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 4351)
  (parent 4162)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((2 1) (13 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y (one)) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4352)
  (parent 4166)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (13 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0))
    ((5 3) (12 0)) ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4353)
  (parent 4166)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((6 1) (13 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 1) (0 3)) ((13 3) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 10 1) (trans 8 1) (trans 4 1) (trans 10 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 14 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4354)
  (parent 4166)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3))
    ((13 3) (12 0)) ((14 1) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 14 1)
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 14 0) (trans 6 0)
    (trans 2 0) (trans 1 0) (trans 10 1) (trans 8 1) (trans 4 1)
    (trans 10 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 4355)
  (parent 4166)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (3 0)) ((1 2) (8 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (7 0)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 3 1) (trans 7 0) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 4356)
  (parent 4170)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (3 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 2) (0 1)) ((4 1) (8 0)) ((5 1) (10 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 3 1) (trans 8 0) (trans 6 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4357)
  (parent 4171)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((3 2) (8 0)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 7 1) (trans 3 1) (trans 11 0) (trans 7 0)
    (trans 3 0) (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4358)
  (parent 4180)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 12 1)
    (trans 8 1) (trans 4 1) (trans 12 0) (trans 8 0) (trans 4 0)
    (neq a b))
  (operation nonce-test (displaced 14 5 resp 4) (exp (gen) y-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4359)
  (parent 4181)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 4360)
  (parent 4181)
  (unrealized (0 3) (13 0) (14 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 4361)
  (parent 4181)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (deflistener (cat (gen) (rec l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 14 0 init 3) (exp (gen) x-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((recv (cat (gen) (rec l))) (send (cat (gen) (rec l))))
    ((recv l) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 4362)
  (parent 4181)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (13 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0))
    ((5 3) (13 0)) ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (displaced 14 1 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 4363)
  (parent 4181)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 4364)
  (parent 4181)
  (unrealized (0 3) (13 0) (14 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (12 0)) ((5 1) (4 0)) ((5 1) (11 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)) ((13 1) (0 0))
    ((13 1) (7 0)) ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0) (12 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 5 1) (trans 1 1) (trans 13 0) (trans 5 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 3 1) (trans 11 0)
    (trans 7 0) (trans 3 0) (neq self-0 b))
  (operation nonce-test (displaced 2 14 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4365)
  (parent 4181)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (12 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)) ((13 1) (5 0))
    ((13 1) (11 0)) ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 2 1) (trans 1 1) (trans 13 0) (trans 2 0)
    (trans 1 0) (trans 11 1) (trans 7 1) (trans 4 1) (trans 11 0)
    (trans 7 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (displaced 6 14 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4366)
  (parent 4181)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 8 1)
    (trans 4 1) (trans 12 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4367)
  (parent 4181)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((3 2) (7 0)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0)) ((9 1) (4 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (trans 9 1)
    (trans 2 1) (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0)
    (trans 11 1) (trans 6 1) (trans 3 1) (trans 11 0) (trans 6 0)
    (trans 3 0) (neq a self-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4368)
  (parent 4182)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 10 1) (trans 2 1)
    (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0) (trans 12 1)
    (trans 7 1) (trans 4 1) (trans 12 0) (trans 7 0) (trans 4 0)
    (neq a self-0))
  (operation nonce-test (displaced 14 5 resp 4) (exp (gen) y-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self-0 (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4369)
  (parent 4183)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self self-0 a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 la rndx) (beta expt) (x y x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 7 1) (trans 4 1) (trans 12 0)
    (trans 7 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0)))))
  (label 4370)
  (parent 4183)
  (unrealized (0 3) (13 0) (14 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 l-1 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 7 1) (trans 4 1) (trans 12 0)
    (trans 7 0) (trans 4 0) (neq a self-0))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (zeta (one)) (w (mul (rec x) y)) (x-1 x)
      (y-1 y) (w-0 (mul (rec x) y))) (gen) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 4371)
  (parent 4183)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l la l-0 y x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la la) (x x) (beta l-0)
    (upsilon (mul y (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l) (y y) (alpha (mul (rec l) la l-0))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) (rec l))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y (one)) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a la) (pv self l-0) (pv self-0 l))
  (facts (neq (exp (gen) (mul y (rec x))) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 7 1) (trans 4 1) (trans 12 0)
    (trans 7 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (displaced 14 0 init 3) (exp (gen) x-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y (rec x)))
          (enc na nb a self-0
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul la l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-4 "nil")) (send la))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((recv (cat (exp (gen) l) (mul (rec l) (rec l))))
      (send (cat (exp (gen) l) (mul (rec l) (rec l)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 4372)
  (parent 4183)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (13 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0))
    ((5 3) (13 0)) ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (8 0)) ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 7 1) (trans 4 1) (trans 12 0)
    (trans 7 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (displaced 14 1 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 4373)
  (parent 4183)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 lb rndx) (alpha zeta expt) (x y y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 7 1) (trans 4 1) (trans 12 0) (trans 7 0)
    (trans 4 0) (neq a self-0))
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y-0))))))))
  (label 4374)
  (parent 4183)
  (unrealized (0 3) (13 0) (14 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-1) (b self-0)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-1) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self-1) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-1)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (7 0)) ((4 3) (12 0)) ((5 1) (4 1)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (7 0)) ((9 1) (4 0)) ((9 1) (11 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3))
    ((13 1) (0 0)) ((13 1) (6 0)) ((13 2) (12 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (8 0) (12 0))
  (gen-st (pv self l) (pv self-0 l-0) (pv self-1 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 13 1) (trans 9 1) (trans 1 1) (trans 13 0) (trans 9 0)
    (trans 1 0) (trans 11 1) (trans 6 1) (trans 3 1) (trans 11 0)
    (trans 6 0) (trans 3 0) (neq self-1 self-0))
  (operation nonce-test (displaced 2 14 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-1 l-1)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self-1 self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb self-1 self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv
        (sig
          (body self-1 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-1)) (privk "sig" self-1)))
      (recv (cat na self-1 self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-1 self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-1 l-1)))
      (stor priv-stor-1 (cat pt-2 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-1 l-1)))
      (send
        (sig (body self-1 (exp (gen) l-1) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 4375)
  (parent 4183)
  (unrealized (0 3) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 7 1) (trans 4 1) (trans 12 0)
    (trans 7 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (displaced 14 10 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec x) y (rec l-1))))))
  (label 4376)
  (parent 4183)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 x y l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-1)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 14 1) (trans 10 1) (trans 2 1) (trans 1 1) (trans 14 0)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 7 1)
    (trans 4 1) (trans 12 0) (trans 7 0) (trans 4 0) (neq a self-0))
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (mul (rec x) y (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec x) y (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-1 l-2)))
      (send
        (sig (body self-1 (exp (gen) l-2) (pubk "sig" self-1))
          (privk "sig" self-1)))))
  (label 4377)
  (parent 4183)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((2 1) (14 0)) ((3 1) (0 1)) ((4 3) (13 0))
    ((5 1) (4 0)) ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 9 1) (trans 7 1) (trans 12 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 15 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 4378)
  (parent 4211)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 1) (14 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (13 0))
    ((5 1) (4 0)) ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 9 1) (trans 7 1) (trans 12 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 15 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 4379)
  (parent 4211)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((5 1) (14 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 9 1) (trans 7 1) (trans 12 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 15 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 4380)
  (parent 4211)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)) ((14 2) (13 0)) ((15 1) (14 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 15 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 15 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 9 1)
    (trans 7 1) (trans 12 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 4381)
  (parent 4211)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((2 1) (14 0)) ((3 1) (0 1)) ((4 3) (13 0))
    ((5 1) (4 0)) ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 9 1) (trans 7 1) (trans 12 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation channel-test (displaced 15 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4382)
  (parent 4215)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 1) (14 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (13 0))
    ((5 1) (4 0)) ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 9 1) (trans 7 1) (trans 12 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation channel-test (displaced 15 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4383)
  (parent 4215)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((5 1) (14 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 9 1) (trans 7 1) (trans 12 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation channel-test (displaced 15 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4384)
  (parent 4215)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)) ((14 3) (13 0)) ((15 1) (14 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 15 1)
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 15 0) (trans 5 0)
    (trans 2 0) (trans 1 0) (trans 12 1) (trans 9 1) (trans 7 1)
    (trans 12 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 4385)
  (parent 4215)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor) (la l-1)
    (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((2 1) (14 0)) ((3 1) (0 1)) ((4 3) (13 0))
    ((5 1) (4 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation channel-test (displaced 15 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 4386)
  (parent 4220)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 1) (14 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (13 0))
    ((5 1) (4 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation channel-test (displaced 15 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 4387)
  (parent 4220)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 1) (14 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation channel-test (displaced 15 10 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self-0 b (exp (gen) x-0)))))
  (label 4388)
  (parent 4220)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a self self-0 a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)) ((15 1) (14 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 15 1) (trans 10 1) (trans 2 1) (trans 1 1) (trans 15 0)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 8 1)
    (trans 6 1) (trans 12 0) (trans 8 0) (trans 6 0) (neq a self-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 4389)
  (parent 4220)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((2 1) (14 0)) ((3 1) (0 1)) ((4 3) (13 0))
    ((5 1) (4 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0) (trans 8 0)
    (trans 6 0) (neq a self-0))
  (operation channel-test (displaced 15 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4390)
  (parent 4224)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 1) (14 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (13 0))
    ((5 1) (4 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0) (trans 8 0)
    (trans 6 0) (neq a self-0))
  (operation channel-test (displaced 15 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4391)
  (parent 4224)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 1) (14 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0) (trans 8 0)
    (trans 6 0) (neq a self-0))
  (operation channel-test (displaced 15 10 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4392)
  (parent 4224)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)) ((15 1) (14 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1) (pv b l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 15 1)
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 15 0) (trans 10 0)
    (trans 2 0) (trans 1 0) (trans 12 1) (trans 8 1) (trans 6 1)
    (trans 12 0) (trans 8 0) (trans 6 0) (neq a self-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b l-2)))))
  (label 4393)
  (parent 4224)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((2 1) (14 0)) ((3 1) (0 1)) ((4 3) (13 0))
    ((5 1) (4 0)) ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 9 1) (trans 7 1) (trans 12 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 15 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 4394)
  (parent 4237)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 1) (14 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (13 0))
    ((5 1) (4 0)) ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 9 1) (trans 7 1) (trans 12 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 15 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 4395)
  (parent 4237)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((5 1) (14 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 9 1) (trans 7 1) (trans 12 0)
    (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (displaced 15 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 4396)
  (parent 4237)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)) ((14 2) (13 0)) ((15 1) (14 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 15 1) (trans 5 1) (trans 2 1) (trans 1 1) (trans 15 0)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 9 1)
    (trans 7 1) (trans 12 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 4397)
  (parent 4237)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((2 1) (14 0)) ((3 1) (0 1)) ((4 3) (13 0))
    ((5 1) (4 0)) ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 9 1) (trans 7 1) (trans 12 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation channel-test (displaced 15 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4398)
  (parent 4241)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 1) (14 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (13 0))
    ((5 1) (4 0)) ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 9 1) (trans 7 1) (trans 12 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation channel-test (displaced 15 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4399)
  (parent 4241)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((5 1) (14 0)) ((6 1) (4 1)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 5 1)
    (trans 2 1) (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 9 1) (trans 7 1) (trans 12 0) (trans 9 0)
    (trans 7 0) (neq a b))
  (operation channel-test (displaced 15 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4400)
  (parent 4241)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)) ((14 3) (13 0)) ((15 1) (14 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 15 1)
    (trans 5 1) (trans 2 1) (trans 1 1) (trans 15 0) (trans 5 0)
    (trans 2 0) (trans 1 0) (trans 12 1) (trans 9 1) (trans 7 1)
    (trans 12 0) (trans 9 0) (trans 7 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 4401)
  (parent 4241)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor) (la l-1)
    (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((2 1) (14 0)) ((3 1) (0 1)) ((4 3) (13 0))
    ((5 1) (4 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation channel-test (displaced 15 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 4402)
  (parent 4246)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 1) (14 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (13 0))
    ((5 1) (4 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation channel-test (displaced 15 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 4403)
  (parent 4246)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 1) (14 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0)
    (trans 8 0) (trans 6 0) (neq a self-0))
  (operation channel-test (displaced 15 10 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self-0 b (exp (gen) x-0)))))
  (label 4404)
  (parent 4246)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a self self-0 a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)) ((15 1) (14 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 15 1) (trans 10 1) (trans 2 1) (trans 1 1) (trans 15 0)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 8 1)
    (trans 6 1) (trans 12 0) (trans 8 0) (trans 6 0) (neq a self-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 4405)
  (parent 4246)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((2 1) (14 0)) ((3 1) (0 1)) ((4 3) (13 0))
    ((5 1) (4 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0) (trans 8 0)
    (trans 6 0) (neq a self-0))
  (operation channel-test (displaced 15 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4406)
  (parent 4250)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 1) (14 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (13 0))
    ((5 1) (4 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0) (trans 8 0)
    (trans 6 0) (neq a self-0))
  (operation channel-test (displaced 15 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4407)
  (parent 4250)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 1) (14 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 8 1) (trans 6 1) (trans 12 0) (trans 8 0)
    (trans 6 0) (neq a self-0))
  (operation channel-test (displaced 15 10 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4408)
  (parent 4250)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)) ((15 1) (14 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1) (pv b l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 15 1)
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 15 0) (trans 10 0)
    (trans 2 0) (trans 1 0) (trans 12 1) (trans 8 1) (trans 6 1)
    (trans 12 0) (trans 8 0) (trans 6 0) (neq a self-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b l-2)))))
  (label 4409)
  (parent 4250)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b-0)
    (priv-stor priv-stor) (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((3 2) (0 3)))
  (uniq-orig na l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 2 1) (trans 1 1) (trans 2 0)
    (trans 1 0) (trans 3 1) (trans 3 0) (neq self b-0))
  (operation generalization separated b-0)
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body b-0 (exp (gen) l) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na self b-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 4410)
  (parent 4255)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta l) (a self) (b b-0) (x x) (upsilon y) (na na)
        (nb nb) (priv-stor priv-stor) (pt pt-0) (self self)
        (priv-stor-0 priv-stor) (ignore ignore) (pt-0 pt) (pt-1 pt-0))))
  (origs (pt-3 (3 1)) (l (1 1)) (pt-0 (1 1)) (lb (2 1)) (pt-1 (2 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1))
    ((2 1) (4 0)) ((3 2) (0 3)) ((3 2) (4 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq self b))
  (operation generalization weakened ((2 1) (0 3)))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4411)
  (parent 4256)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a self))
  (operation generalization weakened ((1 2) (4 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4412)
  (parent 4258)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b-0)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((3 2) (0 3)) ((4 1) (0 0))
    ((4 2) (0 3)))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0)
    (neq self-0 b-0))
  (operation generalization separated b-0)
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv
        (sig (body b-0 (exp (gen) l) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na self-0 b-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4413)
  (parent 4259)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l-0) (beta l) (a self-0) (b b-0) (x x) (upsilon y) (na na)
        (nb nb) (priv-stor priv-stor-1) (pt pt-5) (self self)
        (priv-stor-0 priv-stor) (ignore ignore) (pt-0 pt) (pt-1 pt-0))))
  (origs (l-0 (4 1)) (pt-5 (4 1)) (pt-3 (3 1)) (lb (2 1)) (pt-1 (2 1))
    (l (1 1)) (pt-0 (1 1)) (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (0 3)) ((3 2) (5 0)) ((4 1) (0 0)) ((4 2) (0 3))
    ((4 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 b))
  (operation generalization weakened ((2 1) (0 3)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4414)
  (parent 4260)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 1) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 5 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0)))
  (label 4415)
  (parent 4261)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (7 0)) ((4 2) (7 0)) ((5 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (7 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4416)
  (parent 4262)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b-0) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((4 2) (0 3)))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b-0))
  (operation generalization separated b-0)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b-0 (exp (gen) l-0) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na a b-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l)))
  (label 4417)
  (parent 4264)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta l-0) (a a) (b b-0) (x x) (upsilon y) (na na) (nb nb)
        (priv-stor priv-stor) (pt pt) (self self)
        (priv-stor-0 priv-stor-0) (ignore ignore) (pt-0 pt-0)
        (pt-1 pt-1))))
  (origs (l-0 (1 1)) (pt-1 (1 1)) (pt-5 (4 1)) (lb (3 1)) (pt-3 (3 1))
    (l (2 1)) (pt (2 1)) (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 2) (0 3)) ((4 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((3 1) (0 3)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4418)
  (parent 4265)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (0 3)) ((4 2) (0 3)) ((5 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 4 1)
    (trans 5 0) (trans 4 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0)))
  (label 4419)
  (parent 4266)
  (seen 4495)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 1) (7 0)) ((4 2) (7 0)) ((5 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4420)
  (parent 4267)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (4 0)) ((4 2) (7 0)) ((5 1) (7 0)) ((6 2) (5 0))
    ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (7 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4421)
  (parent 4270)
  (seen 4496)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (7 0))
    ((4 2) (0 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 4 1)
    (trans 7 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 4422)
  (parent 4271)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 1) (8 0)) ((4 1) (9 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (9 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 5 1) (trans 8 0) (trans 5 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4423)
  (parent 4272)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 1) (7 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 3 1)
    (trans 7 0) (trans 3 0) (neq a self-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-5 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 4424)
  (parent 4273)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (9 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 4 1) (trans 8 0) (trans 4 0) (neq a self-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-5 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4425)
  (parent 4274)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 1) (8 0)) ((4 2) (9 0)) ((5 2) (9 0))
    ((6 1) (9 0)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 5 1) (trans 4 1) (trans 8 0) (trans 5 0) (trans 4 0)
    (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4426)
  (parent 4277)
  (seen 4500)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (3 0)) ((3 2) (9 0)) ((4 2) (9 0)) ((5 1) (9 0))
    ((6 1) (8 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 4 1) (trans 3 1) (trans 8 0) (trans 4 0) (trans 3 0)
    (neq a self-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4427)
  (parent 4279)
  (seen 4502)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (2 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (0 0))
    ((3 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 self))
  (operation generalization weakened ((2 1) (4 0)))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4428)
  (parent 4280)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 2) (0 1)) ((4 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1) (trans 4 0)
    (trans 3 0) (neq a self))
  (operation generalization weakened ((4 1) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4429)
  (parent 4281)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (0 0))
    ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 3 1) (trans 1 1)
    (trans 4 0) (trans 3 0) (trans 1 0) (trans 2 1) (trans 2 0)
    (neq self-0 b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4430)
  (parent 4282)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 1) (6 0)) ((4 1) (6 0)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 5 1) (trans 3 1)
    (trans 1 1) (trans 5 0) (trans 3 0) (trans 1 0) (trans 2 1)
    (trans 2 0) (neq self-0 b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4431)
  (parent 4283)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((3 2) (0 1))
    ((3 2) (6 0)) ((4 1) (5 0)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 5 1)
    (trans 3 1) (trans 8 0) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 4432)
  (parent 4284)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 2) (0 1)) ((3 2) (7 0)) ((4 1) (6 0)) ((5 1) (10 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 6 1) (trans 3 1) (trans 9 0) (trans 6 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4433)
  (parent 4285)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (4 0)) ((3 1) (0 3))
    ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (0 0)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 6 1) (trans 3 1) (trans 1 1)
    (trans 6 0) (trans 3 0) (trans 1 0) (trans 2 1) (trans 2 0)
    (neq self-0 b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4434)
  (parent 4294)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (4 0)) ((4 2) (8 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 1) (0 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (8 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 7 1) (trans 3 1)
    (trans 1 1) (trans 7 0) (trans 3 0) (trans 1 0) (trans 4 1)
    (trans 2 1) (trans 4 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4435)
  (parent 4295)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((1 2) (0 3)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (0 3)) ((5 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1)
    (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 4436)
  (parent 4296)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((1 2) (5 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 2) (0 1)) ((4 1) (7 0)) ((5 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 3 1) (trans 6 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4437)
  (parent 4297)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((3 2) (0 1))
    ((3 2) (8 0)) ((4 1) (5 0)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 3 1) (trans 7 0) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 4438)
  (parent 4298)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 2) (0 1)) ((3 2) (9 0)) ((4 1) (6 0)) ((5 1) (10 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 3 1) (trans 8 0) (trans 6 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4439)
  (parent 4299)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (3 0)) ((1 2) (7 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((4 1) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 3 1)
    (trans 6 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 4440)
  (parent 4308)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 2) (0 1)) ((4 1) (5 0)) ((5 2) (9 0))
    ((6 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 3 1) (trans 7 0) (trans 5 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4441)
  (parent 4309)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (4 0))
    ((4 2) (6 0)) ((5 1) (0 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 5 1) (trans 3 1)
    (trans 1 1) (trans 5 0) (trans 3 0) (trans 1 0) (trans 4 1)
    (trans 2 1) (trans 4 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4442)
  (parent 4311)
  (seen 4506)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((3 2) (0 1))
    ((4 1) (6 0)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 6 1)
    (trans 3 1) (trans 8 0) (trans 6 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 4443)
  (parent 4312)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 2) (0 1)) ((3 2) (5 0)) ((4 1) (7 0)) ((5 1) (10 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 7 1) (trans 3 1) (trans 9 0) (trans 7 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4444)
  (parent 4313)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (4 0)) ((4 2) (0 3))
    ((5 1) (0 3)) ((6 1) (0 0)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 6 1) (trans 3 1) (trans 1 1)
    (trans 6 0) (trans 3 0) (trans 1 0) (trans 4 1) (trans 2 1)
    (trans 4 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4445)
  (parent 4322)
  (seen 4509)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (5 0))
    ((4 1) (8 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (8 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 7 1) (trans 3 1)
    (trans 1 1) (trans 7 0) (trans 3 0) (trans 1 0) (trans 5 1)
    (trans 2 1) (trans 5 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4446)
  (parent 4323)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((3 2) (0 1))
    ((3 2) (5 0)) ((4 1) (10 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 7 1)
    (trans 3 1) (trans 10 0) (trans 7 0) (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4447)
  (parent 4324)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 2) (0 1)) ((3 2) (6 0)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 11 1)
    (trans 8 1) (trans 3 1) (trans 11 0) (trans 8 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4448)
  (parent 4325)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((2 1) (14 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0))
    ((5 3) (13 0)) ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 9 1) (trans 4 1) (trans 12 0)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 15 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 4449)
  (parent 4326)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (14 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0))
    ((5 3) (13 0)) ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 9 1) (trans 4 1) (trans 12 0)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 15 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 4450)
  (parent 4326)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((6 1) (14 0)) ((7 1) (5 1))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 9 1) (trans 4 1) (trans 12 0)
    (trans 9 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 15 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 4451)
  (parent 4326)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)) ((14 2) (13 0)) ((15 1) (14 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 15 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 15 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 9 1)
    (trans 4 1) (trans 12 0) (trans 9 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 4452)
  (parent 4326)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((2 1) (14 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0))
    ((5 3) (13 0)) ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 9 1) (trans 4 1) (trans 12 0) (trans 9 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 15 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4453)
  (parent 4330)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (14 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0))
    ((5 3) (13 0)) ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 9 1) (trans 4 1) (trans 12 0) (trans 9 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 15 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4454)
  (parent 4330)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((6 1) (14 0)) ((7 1) (5 1))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 9 1) (trans 4 1) (trans 12 0) (trans 9 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 15 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4455)
  (parent 4330)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)) ((14 3) (13 0)) ((15 1) (14 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 15 1)
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 15 0) (trans 6 0)
    (trans 2 0) (trans 1 0) (trans 12 1) (trans 9 1) (trans 4 1)
    (trans 12 0) (trans 9 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 4456)
  (parent 4330)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((3 2) (0 1))
    ((3 2) (4 0)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (10 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (trans 8 1) (trans 2 1) (trans 1 1)
    (trans 8 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 6 1)
    (trans 3 1) (trans 10 0) (trans 6 0) (trans 3 0) (neq a self-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4457)
  (parent 4334)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((3 2) (5 0)) ((4 3) (12 0)) ((5 1) (4 1))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (4 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 9 1) (trans 2 1)
    (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0) (trans 11 1)
    (trans 7 1) (trans 3 1) (trans 11 0) (trans 7 0) (trans 3 0)
    (neq a self-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self-0 (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4458)
  (parent 4335)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor) (la l-1)
    (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((2 1) (14 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 3) (13 0)) ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0)
    (trans 8 0) (trans 4 0) (neq a self-0))
  (operation channel-test (displaced 15 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 4459)
  (parent 4336)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (14 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 3) (13 0)) ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0)
    (trans 8 0) (trans 4 0) (neq a self-0))
  (operation channel-test (displaced 15 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 4460)
  (parent 4336)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 1) (14 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0)
    (trans 8 0) (trans 4 0) (neq a self-0))
  (operation channel-test (displaced 15 10 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self-0 b (exp (gen) x-0)))))
  (label 4461)
  (parent 4336)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a self self-0 a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)) ((15 1) (14 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 15 1) (trans 10 1) (trans 2 1) (trans 1 1) (trans 15 0)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 8 1)
    (trans 4 1) (trans 12 0) (trans 8 0) (trans 4 0) (neq a self-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 4462)
  (parent 4336)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((2 1) (14 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 3) (13 0)) ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0) (trans 8 0)
    (trans 4 0) (neq a self-0))
  (operation channel-test (displaced 15 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4463)
  (parent 4340)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (14 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0))
    ((5 3) (13 0)) ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0) (trans 8 0)
    (trans 4 0) (neq a self-0))
  (operation channel-test (displaced 15 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4464)
  (parent 4340)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 1) (14 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0) (trans 8 0)
    (trans 4 0) (neq a self-0))
  (operation channel-test (displaced 15 10 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4465)
  (parent 4340)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)) ((15 1) (14 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1) (pv b l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 15 1)
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 15 0) (trans 10 0)
    (trans 2 0) (trans 1 0) (trans 12 1) (trans 8 1) (trans 4 1)
    (trans 12 0) (trans 8 0) (trans 4 0) (neq a self-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b l-2)))))
  (label 4466)
  (parent 4340)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 2) (0 1)) ((4 1) (6 0)) ((5 2) (7 0))
    ((6 2) (7 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 5 1) (trans 3 1) (trans 6 0) (trans 5 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4467)
  (parent 4345)
  (seen 4512)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((3 2) (8 0)) ((4 1) (7 0)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 3 1) (trans 7 0) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 4468)
  (parent 4346)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((3 2) (9 0)) ((4 1) (8 0)) ((5 1) (10 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 3 1) (trans 8 0) (trans 6 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4469)
  (parent 4347)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (3 0)) ((1 2) (7 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (6 0)) ((5 2) (0 3)) ((6 2) (0 3))
    ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 5 1)
    (trans 3 1) (trans 6 0) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 4470)
  (parent 4356)
  (seen 4515)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 2) (0 1)) ((4 1) (7 0)) ((5 2) (9 0))
    ((6 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 3 1) (trans 7 0) (trans 5 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4471)
  (parent 4357)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((3 2) (0 1))
    ((3 2) (7 0)) ((4 1) (10 0)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 6 1)
    (trans 3 1) (trans 10 0) (trans 6 0) (trans 3 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4472)
  (parent 4358)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((3 2) (8 0)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 11 1)
    (trans 7 1) (trans 3 1) (trans 11 0) (trans 7 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4473)
  (parent 4359)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((2 1) (14 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0))
    ((5 3) (13 0)) ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 15 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 4474)
  (parent 4360)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (14 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0))
    ((5 3) (13 0)) ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 15 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 4475)
  (parent 4360)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((6 1) (14 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0)
    (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (displaced 15 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 4476)
  (parent 4360)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)) ((14 2) (13 0)) ((15 1) (14 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 15 1) (trans 6 1) (trans 2 1) (trans 1 1) (trans 15 0)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 8 1)
    (trans 4 1) (trans 12 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 4477)
  (parent 4360)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((2 1) (14 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0))
    ((5 3) (13 0)) ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 15 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4478)
  (parent 4364)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (14 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0))
    ((5 3) (13 0)) ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 15 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4479)
  (parent 4364)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((6 1) (14 0)) ((7 1) (5 1))
    ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)) ((11 1) (10 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 6 1)
    (trans 2 1) (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 8 1) (trans 4 1) (trans 12 0) (trans 8 0)
    (trans 4 0) (neq a b))
  (operation channel-test (displaced 15 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4480)
  (parent 4364)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta (one)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)) ((14 3) (13 0)) ((15 1) (14 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 15 1)
    (trans 6 1) (trans 2 1) (trans 1 1) (trans 15 0) (trans 6 0)
    (trans 2 0) (trans 1 0) (trans 12 1) (trans 8 1) (trans 4 1)
    (trans 12 0) (trans 8 0) (trans 4 0) (neq a b))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 4481)
  (parent 4364)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((3 2) (6 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (6 0)) ((8 1) (10 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (trans 8 1) (trans 2 1) (trans 1 1)
    (trans 8 0) (trans 2 0) (trans 1 0) (trans 10 1) (trans 5 1)
    (trans 3 1) (trans 10 0) (trans 5 0) (trans 3 0) (neq a self-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4482)
  (parent 4368)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((3 2) (7 0)) ((4 3) (12 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0)) ((9 1) (4 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (trans 9 1) (trans 2 1)
    (trans 1 1) (trans 9 0) (trans 2 0) (trans 1 0) (trans 11 1)
    (trans 6 1) (trans 3 1) (trans 11 0) (trans 6 0) (trans 3 0)
    (neq a self-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self-0 (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4483)
  (parent 4369)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor) (la l-1)
    (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((2 1) (14 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0))
    ((5 3) (13 0)) ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (8 0)) ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 7 1) (trans 4 1) (trans 12 0)
    (trans 7 0) (trans 4 0) (neq a self-0))
  (operation channel-test (displaced 15 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 4484)
  (parent 4370)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (14 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0))
    ((5 3) (13 0)) ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (8 0)) ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 7 1) (trans 4 1) (trans 12 0)
    (trans 7 0) (trans 4 0) (neq a self-0))
  (operation channel-test (displaced 15 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 4485)
  (parent 4370)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self self-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx) (beta expt)
    (x y x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a self-0) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 1) (14 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0)
    (trans 1 0) (trans 12 1) (trans 7 1) (trans 4 1) (trans 12 0)
    (trans 7 0) (trans 4 0) (neq a self-0))
  (operation channel-test (displaced 15 10 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self-0 b (exp (gen) x-0)))))
  (label 4486)
  (parent 4370)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a self self-0 a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (beta expt) (x y x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 2) (13 0)) ((15 1) (14 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen)) (neq (gen) (gen))
    (trans 15 1) (trans 10 1) (trans 2 1) (trans 1 1) (trans 15 0)
    (trans 10 0) (trans 2 0) (trans 1 0) (trans 12 1) (trans 7 1)
    (trans 4 1) (trans 12 0) (trans 7 0) (trans 4 0) (neq a self-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 4487)
  (parent 4370)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((2 1) (14 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0))
    ((5 3) (13 0)) ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (8 0)) ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-1) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 7 1) (trans 4 1) (trans 12 0) (trans 7 0)
    (trans 4 0) (neq a self-0))
  (operation channel-test (displaced 15 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4488)
  (parent 4374)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-1)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (14 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0))
    ((5 3) (13 0)) ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (8 0)) ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y (one)) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-1) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 7 1) (trans 4 1) (trans 12 0) (trans 7 0)
    (trans 4 0) (neq a self-0))
  (operation channel-test (displaced 15 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-1)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4489)
  (parent 4374)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 rndx)
    (alpha zeta expt) (x y y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 1) (14 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 10 1)
    (trans 2 1) (trans 1 1) (trans 10 0) (trans 2 0) (trans 1 0)
    (trans 12 1) (trans 7 1) (trans 4 1) (trans 12 0) (trans 7 0)
    (trans 4 0) (neq a self-0))
  (operation channel-test (displaced 15 10 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y-0))))))))
  (label 4490)
  (parent 4374)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self self-0 a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (l l-0 l-1 rndx) (alpha zeta expt) (x y y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta (one)))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)) ((14 3) (13 0)) ((15 1) (14 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y (one)) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta) (y-0 l-2))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1) (pv b l-2))
  (facts (neq (exp (gen) (mul (rec x) y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq (gen) (gen)) (trans 15 1)
    (trans 10 1) (trans 2 1) (trans 1 1) (trans 15 0) (trans 10 0)
    (trans 2 0) (trans 1 0) (trans 12 1) (trans 7 1) (trans 4 1)
    (trans 12 0) (trans 7 0) (trans 4 0) (neq a self-0))
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b l-2))) (14 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (gen)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) y))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b l-2)))))
  (label 4491)
  (parent 4374)
  (unrealized (0 3) (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1))
    ((2 1) (4 0)) ((3 1) (4 0)) ((3 2) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq self b))
  (operation generalization weakened ((3 2) (4 0)))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4492)
  (parent 4411)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0)) ((3 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a self))
  (operation generalization weakened ((1 1) (4 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4493)
  (parent 4412)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 1) (5 0)) ((3 2) (0 3)) ((4 1) (0 0)) ((4 2) (0 3))
    ((4 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 b))
  (operation generalization weakened ((3 2) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4494)
  (parent 4414)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (0 3))
    ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0)))
  (label 4495)
  (parent 4415)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 1) (6 0)) ((4 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 5 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4496)
  (parent 4416)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 1) (5 0)) ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((4 2) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4497)
  (parent 4418)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 2) (6 0)) ((5 2) (6 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 4 1) (trans 5 0) (trans 4 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4498)
  (parent 4420)
  (seen 4535)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (6 0))
    ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 4 1)
    (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 4499)
  (parent 4422)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (7 0)) ((4 2) (8 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 4 1) (trans 7 0) (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4500)
  (parent 4423)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0)) ((3 2) (0 3))
    ((4 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 3 1)
    (trans 6 0) (trans 3 0) (neq a self-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-5 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 4501)
  (parent 4424)
  (seen 4499)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (8 0)) ((4 1) (8 0)) ((5 1) (7 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (4 0) (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 3 1) (trans 7 0) (trans 3 0) (neq a self-0))
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-5 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4502)
  (parent 4425)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 0)) ((3 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 self))
  (operation generalization weakened ((1 1) (4 0)))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4503)
  (parent 4428)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((3 2) (5 0)) ((4 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1) (trans 4 0)
    (trans 3 0) (neq a self))
  (operation generalization weakened ((0 2) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4504)
  (parent 4429)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((4 1) (0 0)) ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 3 1) (trans 1 1)
    (trans 4 0) (trans 3 0) (trans 1 0) (trans 2 1) (trans 2 0)
    (neq self-0 b))
  (operation generalization weakened ((3 1) (0 3)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4505)
  (parent 4430)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (5 0))
    ((4 1) (0 0)) ((4 2) (5 0)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 3 1)
    (trans 1 1) (trans 4 0) (trans 3 0) (trans 1 0) (trans 2 1)
    (trans 2 0) (neq self-0 b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4506)
  (parent 4431)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((3 2) (0 1))
    ((3 2) (5 0)) ((4 1) (0 3)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 3 1)
    (trans 7 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 4507)
  (parent 4432)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 2) (0 1)) ((3 2) (6 0)) ((4 1) (5 0)) ((5 2) (9 0))
    ((6 1) (9 0)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 5 1) (trans 3 1) (trans 8 0) (trans 5 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4508)
  (parent 4433)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (0 3))
    ((5 1) (0 0)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 5 1) (trans 3 1) (trans 1 1)
    (trans 5 0) (trans 3 0) (trans 1 0) (trans 2 1) (trans 2 0)
    (neq self-0 b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4509)
  (parent 4434)
  (seen 4430)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 1) (7 0)) ((4 1) (7 0)) ((5 1) (4 0)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (7 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 6 1) (trans 3 1)
    (trans 1 1) (trans 6 0) (trans 3 0) (trans 1 0) (trans 2 1)
    (trans 2 0) (neq self-0 b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4510)
  (parent 4435)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((4 1) (0 3)) ((5 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1)
    (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization weakened ((1 2) (0 3)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 4511)
  (parent 4436)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 2) (0 1)) ((4 1) (6 0)) ((5 2) (6 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4512)
  (parent 4437)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((3 2) (0 1))
    ((3 2) (7 0)) ((4 1) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 3 1)
    (trans 6 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 4513)
  (parent 4438)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((3 2) (8 0)) ((4 1) (5 0)) ((5 2) (9 0))
    ((6 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 3 1) (trans 7 0) (trans 5 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4514)
  (parent 4439)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (3 0)) ((1 2) (6 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (0 3)) ((5 2) (0 3)) ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1)
    (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 4515)
  (parent 4440)
  (seen 4436)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 2) (0 1)) ((4 1) (8 0)) ((5 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 3 1) (trans 6 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4516)
  (parent 4441)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((3 2) (0 1))
    ((4 1) (5 0)) ((5 2) (0 3)) ((6 1) (0 3)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 3 1) (trans 7 0) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 4517)
  (parent 4443)
  (seen 4544)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 2) (0 1)) ((4 1) (6 0)) ((5 1) (9 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 6 1) (trans 3 1) (trans 8 0) (trans 6 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4518)
  (parent 4444)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (4 0))
    ((4 2) (7 0)) ((5 1) (7 0)) ((6 1) (0 0)) ((6 2) (5 0))
    ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (7 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 6 1) (trans 3 1)
    (trans 1 1) (trans 6 0) (trans 3 0) (trans 1 0) (trans 4 1)
    (trans 2 1) (trans 4 0) (trans 2 0) (neq self-0 b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4519)
  (parent 4446)
  (seen 4546)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((3 2) (0 1))
    ((4 1) (9 0)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 6 1)
    (trans 3 1) (trans 9 0) (trans 6 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4520)
  (parent 4447)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (11 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((3 2) (5 0)) ((4 1) (10 0)) ((5 1) (11 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (11 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 7 1) (trans 3 1) (trans 10 0) (trans 7 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4521)
  (parent 4448)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (9 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (trans 7 1) (trans 2 1) (trans 1 1)
    (trans 7 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 5 1)
    (trans 3 1) (trans 9 0) (trans 5 0) (trans 3 0) (neq a self-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4522)
  (parent 4457)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (11 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((3 2) (4 0)) ((4 1) (11 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (10 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (11 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 8 1) (trans 2 1)
    (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 6 1) (trans 3 1) (trans 10 0) (trans 6 0) (trans 3 0)
    (neq a self-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4523)
  (parent 4458)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((3 2) (7 0)) ((4 1) (6 0)) ((5 2) (0 3)) ((6 2) (0 3))
    ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 5 1)
    (trans 3 1) (trans 6 0) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 4524)
  (parent 4468)
  (seen 4549)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((3 2) (8 0)) ((4 1) (7 0)) ((5 2) (9 0))
    ((6 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 3 1) (trans 7 0) (trans 5 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4525)
  (parent 4469)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 2) (0 1)) ((4 1) (6 0)) ((5 2) (8 0))
    ((6 2) (8 0)) ((7 1) (8 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 5 1) (trans 3 1) (trans 6 0) (trans 5 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4526)
  (parent 4471)
  (seen 4551)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((3 2) (6 0)) ((4 1) (9 0)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 5 1)
    (trans 3 1) (trans 9 0) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4527)
  (parent 4472)
  (seen 4553)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (11 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((3 2) (7 0)) ((4 1) (10 0)) ((5 1) (11 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 6 1) (trans 3 1) (trans 10 0) (trans 6 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4528)
  (parent 4473)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((3 2) (5 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (trans 7 1) (trans 2 1) (trans 1 1)
    (trans 7 0) (trans 2 0) (trans 1 0) (trans 9 1) (trans 4 1)
    (trans 3 1) (trans 9 0) (trans 4 0) (trans 3 0) (neq a self-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4529)
  (parent 4482)
  (seen 4555)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (11 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((3 2) (6 0)) ((4 1) (11 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (6 0)) ((8 1) (10 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (11 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 8 1) (trans 2 1)
    (trans 1 1) (trans 8 0) (trans 2 0) (trans 1 0) (trans 10 1)
    (trans 5 1) (trans 3 1) (trans 10 0) (trans 5 0) (trans 3 0)
    (neq a self-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4530)
  (parent 4483)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1))
    ((2 1) (4 0)) ((3 2) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq self b))
  (operation generalization weakened ((3 1) (4 0)))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4531)
  (parent 4492)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0)) ((3 2) (0 3)))
  (uniq-orig na lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a self))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4532)
  (parent 4493)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (0 3)) ((4 1) (0 0)) ((4 2) (0 3)) ((4 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 b))
  (operation generalization weakened ((3 1) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4533)
  (parent 4494)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b))
  (operation generalization weakened ((3 1) (0 3)))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0)))
  (label 4534)
  (parent 4495)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 2) (5 0)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4535)
  (parent 4496)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((4 1) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4536)
  (parent 4497)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 2) (0 3)) ((5 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 4 1)
    (trans 5 0) (trans 4 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 4537)
  (parent 4499)
  (seen 4495)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (6 0)) ((4 2) (7 0)) ((5 1) (7 0)) ((6 2) (5 0))
    ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 4 1) (trans 6 0) (trans 4 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4538)
  (parent 4500)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (7 0)) ((4 1) (6 0)) ((4 2) (7 0)) ((5 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 3 1) (trans 6 0) (trans 3 0) (neq a self-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-5 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4539)
  (parent 4502)
  (seen 4538)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 0)) ((3 2) (0 3)))
  (uniq-orig na lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 self))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4540)
  (parent 4503)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((3 2) (5 0)) ((4 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1) (trans 4 0)
    (trans 3 0) (neq a self))
  (operation generalization weakened ((2 1) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4541)
  (parent 4504)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((4 1) (0 0)) ((4 2) (0 3)))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 3 1) (trans 1 1)
    (trans 4 0) (trans 3 0) (trans 1 0) (trans 2 1) (trans 2 0)
    (neq self-0 b))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4542)
  (parent 4505)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((3 1) (5 0)) ((4 1) (0 0)) ((4 2) (0 3)) ((4 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 3 1)
    (trans 1 1) (trans 4 0) (trans 3 0) (trans 1 0) (trans 2 1)
    (trans 2 0) (neq self-0 b))
  (operation generalization deleted (5 1))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4543)
  (parent 4506)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((3 2) (0 1))
    ((4 1) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 3 1)
    (trans 6 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 4544)
  (parent 4507)
  (seen 4511)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((3 2) (5 0)) ((4 1) (8 0)) ((5 1) (8 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 3 1) (trans 7 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4545)
  (parent 4508)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (6 0))
    ((4 1) (6 0)) ((5 1) (0 0)) ((5 2) (4 0)) ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (4 0) (6 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 5 1) (trans 3 1)
    (trans 1 1) (trans 5 0) (trans 3 0) (trans 1 0) (trans 2 1)
    (trans 2 0) (neq self-0 b))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4546)
  (parent 4510)
  (seen 4506)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((5 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1)
    (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization weakened ((4 1) (0 3)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 4547)
  (parent 4511)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (0 3)) ((1 2) (6 0))
    ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1)) ((4 1) (0 3))
    ((4 1) (6 0)) ((5 2) (0 3)) ((5 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization deleted (6 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4548)
  (parent 4512)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((3 2) (6 0)) ((4 1) (0 3)) ((5 2) (0 3)) ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1)
    (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 4549)
  (parent 4513)
  (seen 4511)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((3 2) (7 0)) ((4 1) (8 0)) ((5 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 3 1) (trans 6 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4550)
  (parent 4514)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 2) (0 1)) ((4 1) (7 0)) ((5 2) (7 0))
    ((6 1) (7 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4551)
  (parent 4516)
  (seen 4512)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((4 1) (5 0)) ((5 2) (8 0)) ((6 1) (8 0))
    ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 3 1) (trans 7 0) (trans 5 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4552)
  (parent 4518)
  (seen 4571)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((4 1) (8 0)) ((5 2) (0 3)) ((6 1) (0 3)) ((7 1) (6 0))
    ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 5 1)
    (trans 3 1) (trans 8 0) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4553)
  (parent 4520)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((4 1) (9 0)) ((5 1) (10 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (10 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 6 1) (trans 3 1) (trans 9 0) (trans 6 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4554)
  (parent 4521)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((4 2) (0 3)) ((5 1) (0 3)) ((6 1) (8 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (trans 6 1) (trans 2 1) (trans 1 1)
    (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1) (trans 4 1)
    (trans 3 1) (trans 8 0) (trans 4 0) (trans 3 0) (neq a self-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4555)
  (parent 4522)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (10 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (10 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 7 1) (trans 2 1)
    (trans 1 1) (trans 7 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 5 1) (trans 3 1) (trans 9 0) (trans 5 0) (trans 3 0)
    (neq a self-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4556)
  (parent 4523)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((3 2) (7 0)) ((4 1) (6 0)) ((5 2) (8 0))
    ((6 2) (8 0)) ((7 1) (8 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 5 1) (trans 3 1) (trans 6 0) (trans 5 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4557)
  (parent 4525)
  (seen 4574)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((3 2) (6 0)) ((4 1) (9 0)) ((5 2) (10 0))
    ((6 1) (10 0)) ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 5 1) (trans 3 1) (trans 9 0) (trans 5 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4558)
  (parent 4528)
  (seen 4576)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 2) (0 1)) ((3 2) (5 0)) ((4 2) (10 0)) ((5 1) (10 0))
    ((6 1) (5 0)) ((7 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (10 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 7 1) (trans 2 1)
    (trans 1 1) (trans 7 0) (trans 2 0) (trans 1 0) (trans 9 1)
    (trans 4 1) (trans 3 1) (trans 9 0) (trans 4 0) (trans 3 0)
    (neq a self-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4559)
  (parent 4530)
  (seen 4578)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1))
    ((3 2) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq self b))
  (operation generalization weakened ((2 1) (4 0)))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4560)
  (parent 4531)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0)) ((3 2) (0 3)))
  (uniq-orig na lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq a self-0))
  (operation generalization separated self-0)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4561)
  (parent 4532)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta lb) (a a) (b self-0) (x x) (upsilon (mul w y))
        (na na) (nb nb) (priv-stor priv-stor) (pt pt) (self self)
        (priv-stor-0 priv-stor-0) (ignore ignore) (pt-0 pt-0)
        (pt-1 pt-1))))
  (origs (pt-3 (3 1)) (lb (1 1)) (pt-1 (1 1)) (l (2 1)) (pt (2 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (0 3)) ((4 1) (0 0)) ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 b))
  (operation generalization weakened ((4 2) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4562)
  (parent 4533)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((4 2) (0 3)))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0)))
  (label 4563)
  (parent 4534)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (0 3)) ((3 1) (5 0)) ((4 2) (0 3)) ((4 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization deleted (5 1))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w))))
  (label 4564)
  (parent 4535)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((3 1) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4565)
  (parent 4536)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 2) (6 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 4 1) (trans 5 0) (trans 4 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4566)
  (parent 4538)
  (seen 4535)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 0)) ((3 2) (0 3)))
  (uniq-orig na lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 1 1)
    (trans 3 0) (trans 1 0) (trans 2 1) (trans 2 0) (neq self-0 self-1))
  (operation generalization separated self-1)
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self-1 (exp (gen) lb) (pubk "sig" self-1))
          (privk "sig" self-1)))
      (send (cat na self-0 self-1 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4567)
  (parent 4540)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta lb) (a self-0) (b self-1) (x x) (upsilon (mul w y))
        (na na) (nb nb) (priv-stor priv-stor-0) (pt pt-3) (self self)
        (priv-stor-0 priv-stor) (ignore ignore) (pt-0 pt) (pt-1 pt-0))))
  (origs (l (3 1)) (pt-3 (3 1)) (lb (1 1)) (pt-0 (1 1)) (pt-1 (2 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((3 2) (0 1)) ((4 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1) (trans 4 0)
    (trans 3 0) (neq a self))
  (operation generalization weakened ((3 2) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4568)
  (parent 4541)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b-0)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((4 1) (0 0)) ((4 2) (0 3)))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 3 1) (trans 1 1)
    (trans 4 0) (trans 3 0) (trans 1 0) (trans 2 1) (trans 2 0)
    (neq self-0 b-0))
  (operation generalization separated b-0)
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv
        (sig (body b-0 (exp (gen) l) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na self-0 b-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 4569)
  (parent 4542)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l-0) (beta l) (a self-0) (b b-0) (x x) (upsilon y) (na na)
        (nb nb) (priv-stor priv-stor-1) (pt pt-5) (self self)
        (priv-stor-0 priv-stor) (ignore ignore) (pt-0 pt) (pt-1 pt-0))))
  (origs (l-0 (4 1)) (pt-5 (4 1)) (pt-1 (2 1)) (lb (3 1)) (pt-2 (3 1))
    (pt-0 (1 1)) (na (0 2)) (l (1 1))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (5 0))
    ((4 1) (0 0)) ((4 2) (0 3)) ((4 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 3 1)
    (trans 1 1) (trans 4 0) (trans 3 0) (trans 1 0) (trans 2 1)
    (trans 2 0) (neq self-0 b))
  (operation generalization weakened ((3 1) (0 3)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4570)
  (parent 4543)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((4 1) (7 0)) ((5 1) (7 0)) ((6 2) (5 0))
    ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (7 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 3 1) (trans 6 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4571)
  (parent 4545)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((5 2) (0 3)))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1)
    (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 4572)
  (parent 4547)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (0 3)) ((1 2) (6 0))
    ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1)) ((4 1) (6 0))
    ((5 2) (0 3)) ((5 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization weakened ((4 1) (0 3)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4573)
  (parent 4548)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((3 2) (6 0)) ((4 1) (7 0)) ((5 2) (7 0))
    ((6 1) (7 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4574)
  (parent 4550)
  (seen 4586)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((4 1) (7 0)) ((5 2) (0 3)) ((6 1) (0 3)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 5 1)
    (trans 3 1) (trans 7 0) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4575)
  (parent 4553)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (8 0)) ((5 2) (9 0)) ((6 1) (9 0))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (9 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 5 1) (trans 3 1) (trans 8 0) (trans 5 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4576)
  (parent 4554)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((4 2) (0 3)) ((5 1) (7 0)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (trans 5 1) (trans 2 1) (trans 1 1)
    (trans 5 0) (trans 2 0) (trans 1 0) (trans 7 1) (trans 4 1)
    (trans 3 1) (trans 7 0) (trans 4 0) (trans 3 0) (neq a self-0))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4577)
  (parent 4555)
  (seen 4575)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 2) (0 1)) ((4 2) (9 0)) ((5 1) (9 0)) ((6 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 6 1) (trans 2 1)
    (trans 1 1) (trans 6 0) (trans 2 0) (trans 1 0) (trans 8 1)
    (trans 4 1) (trans 3 1) (trans 8 0) (trans 4 0) (trans 3 0)
    (neq a self-0))
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4578)
  (parent 4556)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((3 2) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq self b))
  (operation generalization weakened ((0 2) (4 0)))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4579)
  (parent 4560)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((3 2) (0 3))
    ((4 1) (0 0)) ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 b))
  (operation generalization weakened ((2 1) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4580)
  (parent 4562)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((4 2) (0 3)))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 3 1) (trans 2 1) (trans 1 1)
    (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1) (trans 4 0)
    (neq a b-0))
  (operation generalization separated b-0)
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) l) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na a b-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0)))
  (label 4581)
  (parent 4563)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l-0) (beta l) (a a) (b b-0) (x x) (upsilon y) (na na) (nb nb)
        (priv-stor priv-stor) (pt pt) (self self)
        (priv-stor-0 priv-stor-0) (ignore ignore) (pt-0 pt-0)
        (pt-1 pt-1))))
  (origs (pt-5 (4 1)) (lb (3 1)) (pt-3 (3 1)) (l (1 1)) (pt-1 (1 1))
    (l-0 (2 1)) (pt (2 1)) (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 2) (0 3)) ((4 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((3 1) (0 3)))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w))))
  (label 4582)
  (parent 4564)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 2) (0 1)) ((1 2) (5 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((2 1) (5 0)) ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((0 2) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4583)
  (parent 4565)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 2) (0 1)) ((4 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1) (trans 4 0)
    (trans 3 0) (neq a self))
  (operation generalization weakened ((3 1) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4584)
  (parent 4568)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (5 0))
    ((4 1) (0 0)) ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 3 1)
    (trans 1 1) (trans 4 0) (trans 3 0) (trans 1 0) (trans 2 1)
    (trans 2 0) (neq self-0 b))
  (operation generalization weakened ((4 2) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4585)
  (parent 4570)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (6 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4586)
  (parent 4571)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b-0) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((5 2) (0 3)))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1) (trans 3 1)
    (trans 5 0) (trans 3 0) (neq a b-0))
  (operation generalization separated b-0)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b-0 (exp (gen) l-0) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na a b-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 4587)
  (parent 4572)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta l-0) (a a) (b b-0) (x x) (upsilon y) (na na) (nb nb)
        (priv-stor priv-stor) (pt pt) (self self)
        (priv-stor-0 priv-stor-0) (ignore ignore) (pt-0 pt-0)
        (pt-1 pt-1))))
  (origs (l-0 (1 1)) (pt-1 (1 1)) (pt-6 (5 1)) (lb (4 1)) (pt-4 (4 1))
    (pt-3 (3 1)) (l (2 1)) (pt (2 1)) (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 2) (0 1)) ((4 1) (6 0)) ((5 2) (0 3))
    ((5 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization weakened ((1 2) (0 3)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4588)
  (parent 4573)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((4 1) (6 0)) ((5 2) (0 3)) ((6 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (trans 4 1) (trans 2 1) (trans 1 1)
    (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1) (trans 5 1)
    (trans 3 1) (trans 6 0) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 4589)
  (parent 4575)
  (seen 4511)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (7 0)) ((5 2) (8 0)) ((6 1) (8 0))
    ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 5 1) (trans 3 1) (trans 7 0) (trans 5 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4590)
  (parent 4576)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 2) (0 1)) ((4 2) (8 0)) ((5 1) (7 0)) ((5 2) (8 0))
    ((6 1) (8 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 5 1) (trans 2 1)
    (trans 1 1) (trans 5 0) (trans 2 0) (trans 1 0) (trans 7 1)
    (trans 4 1) (trans 3 1) (trans 7 0) (trans 4 0) (trans 3 0)
    (neq a self-0))
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4591)
  (parent 4578)
  (seen 4590)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((3 2) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq self b))
  (operation generalization weakened ((1 2) (4 0)))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4592)
  (parent 4579)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((1 2) (5 0)) ((3 2) (0 3))
    ((4 1) (0 0)) ((4 1) (5 0)) ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 b))
  (operation generalization weakened ((0 2) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4593)
  (parent 4580)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 1) (5 0)) ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((4 2) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w))))
  (label 4594)
  (parent 4582)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 2) (0 1)) ((1 2) (5 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((2 1) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4595)
  (parent 4583)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((4 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1) (trans 4 0)
    (trans 3 0) (neq a self))
  (operation generalization weakened ((1 1) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4596)
  (parent 4584)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((4 1) (0 0))
    ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 3 1)
    (trans 1 1) (trans 4 0) (trans 3 0) (trans 1 0) (trans 2 1)
    (trans 2 0) (neq self-0 b))
  (operation generalization weakened ((3 1) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4597)
  (parent 4585)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (0 3)) ((4 1) (6 0)) ((5 2) (0 3))
    ((5 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization deleted (6 1))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w))))
  (label 4598)
  (parent 4586)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 2) (0 1)) ((4 1) (6 0)) ((5 1) (6 0))
    ((5 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization weakened ((5 2) (6 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4599)
  (parent 4588)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (6 0)) ((5 2) (7 0)) ((6 2) (7 0))
    ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 6 1)
    (trans 5 1) (trans 3 1) (trans 6 0) (trans 5 0) (trans 3 0)
    (neq a b))
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 4600)
  (parent 4590)
  (seen 4586)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((3 2) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq self b))
  (operation generalization weakened ((1 1) (4 0)))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4601)
  (parent 4592)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((1 2) (5 0)) ((3 2) (0 3))
    ((4 1) (0 0)) ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 b))
  (operation generalization weakened ((4 1) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4602)
  (parent 4593)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((4 1) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w))))
  (label 4603)
  (parent 4594)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((1 2) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4604)
  (parent 4595)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((4 2) (0 3)))
  (uniq-orig na lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1) (trans 4 0)
    (trans 3 0) (neq a self))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4605)
  (parent 4596)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((4 1) (0 0))
    ((4 1) (5 0)) ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 3 1)
    (trans 1 1) (trans 4 0) (trans 3 0) (trans 1 0) (trans 2 1)
    (trans 2 0) (neq self-0 b))
  (operation generalization weakened ((0 2) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4606)
  (parent 4597)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (6 0)) ((5 2) (0 3)) ((5 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization weakened ((4 1) (0 3)))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w))))
  (label 4607)
  (parent 4598)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 2) (0 1)) ((4 1) (6 0)) ((5 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization weakened ((5 1) (6 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4608)
  (parent 4599)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((3 2) (0 3)))
  (uniq-orig na l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq self b))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4609)
  (parent 4601)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((3 2) (0 3))
    ((4 1) (0 0)) ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 b))
  (operation generalization weakened ((1 2) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4610)
  (parent 4602)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((3 1) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w))))
  (label 4611)
  (parent 4603)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((1 1) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4612)
  (parent 4604)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((4 2) (0 3)))
  (uniq-orig na lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 4 1) (trans 3 1) (trans 4 0)
    (trans 3 0) (neq a self-0))
  (operation generalization separated self-0)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4613)
  (parent 4605)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta lb) (a a) (b self-0) (x x) (upsilon (mul w y))
        (na na) (nb nb) (priv-stor priv-stor) (pt pt) (self self)
        (priv-stor-0 priv-stor-0) (ignore ignore) (pt-0 pt-0)
        (pt-1 pt-1))))
  (origs (pt-4 (4 1)) (lb (1 1)) (pt-1 (1 1)) (pt-3 (3 1)) (l (2 1))
    (pt (2 1)) (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((4 1) (0 0))
    ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 3 1)
    (trans 1 1) (trans 4 0) (trans 3 0) (trans 1 0) (trans 2 1)
    (trans 2 0) (neq self-0 b))
  (operation generalization weakened ((4 1) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4614)
  (parent 4606)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (6 0)) ((5 1) (6 0)) ((5 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization weakened ((5 2) (6 0)))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w))))
  (label 4615)
  (parent 4607)
  (seen 4616)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (6 0)) ((5 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization weakened ((1 2) (6 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4616)
  (parent 4608)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b-0)
    (priv-stor priv-stor) (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((3 2) (0 3)))
  (uniq-orig na l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 2 1) (trans 1 1)
    (trans 2 0) (trans 1 0) (trans 3 1) (trans 3 0) (neq self b-0))
  (operation generalization separated b-0)
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body b-0 (exp (gen) l) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na self b-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4617)
  (parent 4609)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta l) (a self) (b b-0) (x x) (upsilon (mul w y))
        (na na) (nb nb) (priv-stor priv-stor) (pt pt-0) (self self)
        (priv-stor-0 priv-stor) (ignore ignore) (pt-0 pt) (pt-1 pt-0))))
  (origs (pt-3 (3 1)) (l (1 1)) (pt-0 (1 1)) (lb (2 1)) (pt-1 (2 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((3 2) (0 3)) ((4 1) (0 0))
    ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 b))
  (operation generalization weakened ((1 1) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4618)
  (parent 4610)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 2) (0 1)) ((1 2) (5 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((2 1) (5 0)) ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((0 2) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w))))
  (label 4619)
  (parent 4611)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((4 2) (0 3)))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4620)
  (parent 4612)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (2 0)) ((2 1) (5 0)) ((2 2) (0 1)) ((4 1) (0 0))
    ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 3 1)
    (trans 1 1) (trans 4 0) (trans 3 0) (trans 1 0) (trans 2 1)
    (trans 2 0) (neq self-0 b))
  (operation generalization weakened ((2 2) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4621)
  (parent 4614)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((5 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization weakened ((4 1) (6 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4622)
  (parent 4616)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((3 2) (0 3)) ((4 1) (0 0))
    ((4 2) (0 3)))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 b))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4623)
  (parent 4618)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 2) (0 1)) ((1 2) (5 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((2 1) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w))))
  (label 4624)
  (parent 4619)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b-0) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((4 2) (0 3)))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b-0))
  (operation generalization separated b-0)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b-0 (exp (gen) l-0) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na a b-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4625)
  (parent 4620)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta l-0) (a a) (b b-0) (x x) (upsilon (mul w y)) (na na)
        (nb nb) (priv-stor priv-stor) (pt pt) (self self)
        (priv-stor-0 priv-stor-0) (ignore ignore) (pt-0 pt-0)
        (pt-1 pt-1))))
  (origs (l-0 (1 1)) (pt-1 (1 1)) (pt-5 (4 1)) (lb (3 1)) (pt-3 (3 1))
    (l (2 1)) (pt (2 1)) (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (2 0)) ((1 1) (5 0)) ((2 2) (0 1)) ((4 1) (0 0))
    ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 3 1)
    (trans 1 1) (trans 4 0) (trans 3 0) (trans 1 0) (trans 2 1)
    (trans 2 0) (neq self-0 b))
  (operation generalization weakened ((2 1) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4626)
  (parent 4621)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((3 2) (6 0)) ((5 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization weakened ((0 2) (6 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4627)
  (parent 4622)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b-0)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((3 2) (0 3)) ((4 1) (0 0))
    ((4 2) (0 3)))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 3 1)
    (trans 3 0) (neq self-0 b-0))
  (operation generalization separated b-0)
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv
        (sig (body b-0 (exp (gen) l) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na self-0 b-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4628)
  (parent 4623)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l-0) (beta l) (a self-0) (b b-0) (x x) (upsilon (mul w y))
        (na na) (nb nb) (priv-stor priv-stor-1) (pt pt-5) (self self)
        (priv-stor-0 priv-stor) (ignore ignore) (pt-0 pt) (pt-1 pt-0))))
  (origs (l-0 (4 1)) (pt-5 (4 1)) (pt-3 (3 1)) (lb (2 1)) (pt-1 (2 1))
    (l (1 1)) (pt-0 (1 1)) (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((1 2) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w))))
  (label 4629)
  (parent 4624)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((4 1) (0 0)) ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 3 1)
    (trans 1 1) (trans 4 0) (trans 3 0) (trans 1 0) (trans 2 1)
    (trans 2 0) (neq self-0 b))
  (operation generalization weakened ((1 1) (5 0)))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4630)
  (parent 4626)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((3 2) (6 0)) ((5 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization weakened ((2 1) (6 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4631)
  (parent 4627)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization weakened ((1 1) (5 0)))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w))))
  (label 4632)
  (parent 4629)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((4 1) (0 0)) ((4 2) (0 3)))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 3 1)
    (trans 1 1) (trans 4 0) (trans 3 0) (trans 1 0) (trans 2 1)
    (trans 2 0) (neq self-0 b))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4633)
  (parent 4630)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 1) (6 0))
    ((3 2) (0 1)) ((5 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization weakened ((3 2) (6 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4634)
  (parent 4631)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((4 2) (0 3)))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w))))
  (label 4635)
  (parent 4632)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b-0)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((4 1) (0 0)) ((4 2) (0 3)))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 3 1)
    (trans 1 1) (trans 4 0) (trans 3 0) (trans 1 0) (trans 2 1)
    (trans 2 0) (neq self-0 b-0))
  (operation generalization separated b-0)
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv
        (sig (body b-0 (exp (gen) l) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na self-0 b-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv (cat (exp (gen) y) w))))
  (label 4636)
  (parent 4633)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l-0) (beta l) (a self-0) (b b-0) (x x) (upsilon (mul w y))
        (na na) (nb nb) (priv-stor priv-stor-1) (pt pt-5) (self self)
        (priv-stor-0 priv-stor) (ignore ignore) (pt-0 pt) (pt-1 pt-0))))
  (origs (l-0 (4 1)) (pt-5 (4 1)) (pt-1 (2 1)) (lb (3 1)) (pt-2 (3 1))
    (pt-0 (1 1)) (na (0 2)) (l (1 1))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((5 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization weakened ((3 1) (6 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4637)
  (parent 4634)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((4 2) (0 3)))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 3 1) (trans 2 1)
    (trans 1 1) (trans 3 0) (trans 2 0) (trans 1 0) (trans 4 1)
    (trans 4 0) (neq a b-0))
  (operation generalization separated b-0)
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) l) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na a b-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w))))
  (label 4638)
  (parent 4635)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l-0) (beta l) (a a) (b b-0) (x x) (upsilon (mul w y)) (na na)
        (nb nb) (priv-stor priv-stor) (pt pt) (self self)
        (priv-stor-0 priv-stor-0) (ignore ignore) (pt-0 pt-0)
        (pt-1 pt-1))))
  (origs (pt-5 (4 1)) (lb (3 1)) (pt-3 (3 1)) (l (1 1)) (pt-1 (1 1))
    (l-0 (2 1)) (pt (2 1)) (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((5 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization weakened ((1 1) (6 0)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4639)
  (parent 4637)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((5 2) (0 3)))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq a b))
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4640)
  (parent 4639)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b-0) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((5 2) (0 3)))
  (uniq-orig na l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (trans 4 1) (trans 2 1)
    (trans 1 1) (trans 4 0) (trans 2 0) (trans 1 0) (trans 5 1)
    (trans 3 1) (trans 5 0) (trans 3 0) (neq a b-0))
  (operation generalization separated b-0)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b-0 (exp (gen) l-0) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na a b-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w))))
  (label 4641)
  (parent 4640)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta l-0) (a a) (b b-0) (x x) (upsilon (mul w y)) (na na)
        (nb nb) (priv-stor priv-stor) (pt pt) (self self)
        (priv-stor-0 priv-stor-0) (ignore ignore) (pt-0 pt-0)
        (pt-1 pt-1))))
  (origs (l-0 (1 1)) (pt-1 (1 1)) (pt-6 (5 1)) (lb (4 1)) (pt-4 (4 1))
    (pt-3 (3 1)) (l (2 1)) (pt (2 1)) (na (0 2))))

(comment "Nothing left to do")
